// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/jbcfile/BCType.java,v $
// $Revision: 1.1 $
// $Date: 1998/02/27 12:34:14 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        BCType.java
//
// HISTORY:
//            
//      Cyril Inard d'Argence, TOG - Nov 06,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.jbcfile;

import java.util.Vector;

/**
 * User friendly representation for a bytecode type.
 */

public final class BCType
{


  /** 
   * Type array level.
   */

  public int getArrayLevel()  { return _arrayLevel; }

  /** 
   * Type name (Java notation).
   */
  
  public String getName() { return _name; }

  /** 
   * Is this type a builtin type.
   */

  public boolean isBuiltin()  { return _isBuiltin; }

  /** 
   * Is this type a class.
   */

  public boolean isClass()    { return ! _isBuiltin; }

  /**
   * Describes the type.
   */

  public String toString()
  {
     String description = _name;

     int arrayLevel = _arrayLevel;

     while ( arrayLevel-- > 0 )
     {
        description +="[]";
     }
     
     return description;
     
  }

  // -- Package constructor --

  /**
   * Constructs a BCType object out of a bytecode somewhat cryptic type 
   * representation.
   * @param bcTypeName name of the type as it appears in the byte code.
   */

  BCType(String bcTypeName)
  {
     
     int arrayLevel = 0;

     while ( bcTypeName.startsWith("[") )
     {
        arrayLevel++;

        bcTypeName = bcTypeName.substring(1,bcTypeName.length());
     }
     
     _arrayLevel = arrayLevel;

     _isBuiltin = ! bcTypeName.startsWith("L");

     String name = null;
     
     if (_isBuiltin )
     {
        // It's a builtin type.
        // Just translate name.

        if      (bcTypeName.equals("V")) { name = "void"; }
        else if (bcTypeName.equals("Z")) { name = "boolean"; }
        else if (bcTypeName.equals("B")) { name = "byte"; }
        else if (bcTypeName.equals("C")) { name = "char"; }
        else if (bcTypeName.equals("S")) { name = "short"; }
        else if (bcTypeName.equals("I")) { name = "int"; }
        else if (bcTypeName.equals("J")) { name = "long"; }
        else if (bcTypeName.equals("F")) { name = "float"; }
        else if (bcTypeName.equals("D")) { name = "double"; }
     }
     else
     {
        // It's a class.
        // Remove leading L and trailing ; and replace separator / with .

        name = bcTypeName.substring(1,bcTypeName.length()-1).replace('/','.');        
     }

     _name = name;
  }

  // -- Package methods --

  /**
   * Makes an array of BCType out of a String made up of a concatenation
   * of bytecode type names.
   */

  static BCType[] getBCTypes(String bcTypes)
  {
     Vector types = new Vector();

     while ( bcTypes != null && bcTypes.length() > 0 )
     {
        int length  = 0; // current type name length
        
        // length of leading [
        
        while ( bcTypes.charAt(length) == '[' )
        { 
           length++;
        }

        if ( bcTypes.charAt(length) == 'L' )
        {
           // class, search length up to trailing ;
           length = bcTypes.indexOf(";");
        }
        else
        {
           // builtin type, only one char, length is ok
        }

        types.addElement(new BCType(bcTypes.substring(0,length+1)));

        // Get rid of the type just exctracted

        bcTypes = bcTypes.substring(length+1);
      }

      BCType[] tmp = new BCType[types.size()];
      types.copyInto(tmp);
      return tmp; 
  }

  /** Type array level. */     

  private final int _arrayLevel;
  
  /** Type name (java notation) */

  private final String _name;
  
  /** Is builtin type flag. */

  private final boolean _isBuiltin;

}

// ***********************************************************************
// History Log:
// $Log: BCType.java,v $
// Revision 1.1  1998/02/27 12:34:14  cinard
// Cleaned up the API for jbcfile components by introducing a user friendly
// representation for bytecode types (BCType class).
// Updated copyright notice.
//
// ***********************************************************************
