// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/jbcfile/ClassFile.java,v $
// $Revision: 1.9 $
// $Date: 1998/02/27 12:34:14 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        ClassFile.java
//
// HISTORY:
//      Christian Fabre,      
//      Cyril Inard d'Argence, TOG - Nov 06,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.jbcfile;

import java.io.*;
import java.lang.reflect.*;
import java.util.*;

/**
 * Enable to hold information retrieved from the bytecode file of a Java class.
 */

public class ClassFile
{
  /** Debug flag. */

  public static boolean debug = false;

  /**
   * Constructs the class file from the supplied data input stream.
   * @param ClassFile data input stream opened from a .class bytecode file.
   * @param scopedName expected scoped name of the class.
   * @exception ORG.opengroup.adl.jbcfile.ClassFileException thrown in case
   *            read byte code does not correspond to the expected class
   *            (expected scoped name differs from scoped name in
   *            byte code).
   */

  public ClassFile(DataInputStream ClassFile, String scopedName)
  throws ClassFileException
  {
    classFile = ClassFile;


    // let's remove this part
    // if ( scopedName.startsWith("." + File.separator) )
    // {
    //    scopedName = scopedName.substring(2);
    // }

    // check scoped name in byte code and supplied scoped name
    // are identical. If not raise an exception.

    if ( ! getScopedName().equals(scopedName) )
    {
       throw new ClassFileException
                 ("Bytecode is not that of the expected class.");
    }
  }

  /**
   * Scoped name of the class.
   */
  public String getScopedName()
  {
     if ( _scopedName == null )
     {
        getConstantPool();
        int index    = ((string)constantPool[getThisClass()]).index;
        _scopedName  =  constantPool[index].toString().replace('/','.');
     }
     return _scopedName;
  }

  /**
   * Returns the name of the superclass of the class whose bytecode is read.
   */
  public String getSuperClassName()
  {
     decode();
     String classStr = null;
     if ( 
              0 <= getSuperClass() 
          &&  getSuperClass() < getConstantPoolCount() 
          &&  constantPool[getSuperClass()] instanceof string 
        )
     {
        int index = ((string)constantPool[getSuperClass()]).index;
        classStr = constantPool[index].toString().replace('/','.');
     }
     return classStr;
  }

  /**
   * Returns the names of the interfaces of the class whose bytecode is read.
   */

  public String[] getInterfaceNames()
  {
     decode();
     String[] ret = new String[getInterfacesCount()];
     int[] ic = getInterfaces();
     for (int i = 0; i < ret.length; i++)
     {
        int index = ((string)constantPool[ic[i]]).index;
        ret[i] = constantPool[index].toString().replace('/','.');
     }
     return ret;
  }

  /**
   * Returns the name of the classes referenced in the read bytecode.
   */

  public String[] getRelatedClassNames()
  {
     decode();
     Vector related = new Vector();

     // super class

     {
        String className = getSuperClassName();
        if ( className != null )
        {
           related.addElement(className);
        }
     }

     // interfaces 

     String[] interfacesName = getInterfaceNames();
     for (int i = 0; i < interfacesName.length; i++)
     {
        related.addElement(interfacesName[i]);
     }

     // Fields

     fieldInfo[] fi = getFields();
     for (int i = 0; i < getFieldsCount(); i++)
     {
        String relatedClass = fi[i].getRelatedClass();
        if ( relatedClass != null )
        {
           related.addElement(relatedClass);
        }
     }

     // Methods

     methodInfo[] mi = getMethods();
     for (int i = 0; i < getMethodsCount(); i++)
     {
        String[] relatedClasses = mi[i].getRelatedClasses();
        for (int j = 0; j < relatedClasses.length; j++)
        {
           related.addElement(relatedClasses[j]);
        }
     }

     String[] ret = new String[related.size()];
     related.copyInto(ret);
     return ret;
  }

  /**  Data input stream for the bytecode file. */
  private DataInputStream classFile;

  /** scoped name of the class for which byte code is being read. */
  private String _scopedName = null;


  // In all the fields and method of this class, we use widers type
  // than in the spec book because we want to treat unsigned values
  // properly.

  private short get1byte()
  {
    short val = -1;

    try
    {
       val = (short)classFile.readUnsignedByte();
    }
    catch(IOException e)
    {
    }
    finally
    {
       if ( val == -1 )
          throw new Error("Reached EOF too early.");
    }
    
    return val;
  }

  private int get2bytes()
  {
    int val = -1;

    try
    {
       val = classFile.readUnsignedShort();
    }
    catch(IOException e)
    {
    }
    finally
    {
       if ( val == -1 )
          throw new Error("Reached EOF too early.");
    }
    
    return val;
  }

  private long get4bytes()
  {
    return (get2bytes()<<16) | get2bytes(); 
  }

  private long          magic              = -1;
  private int           minorVersion       = -1;
  private int           majorVersion       = -1;
  private int           constantPoolCount  = -1;
  private cpInfo        constantPool[]     = null;
  private int           accessFlags        = -1;
  private int           thisClass          = -1;
  private int           superClass         = -2;
  private int           interfacesCount    = -1;
  private int           interfaces[]       = null;
  private int           fieldsCount        = -1;
  private fieldInfo     fields[]           = null;
  private int           methodsCount       = -1;
  private methodInfo    methods[]          = null;
  private int           attributesCount    = -1;
  private attributeInfo attributes[]       = null;

  public long getMagic()
  {
     if ( magic == -1 )
     {
        magic = get4bytes();
     }

     if ( debug && magic != 0xCafeBabe )
        throw new Error("Magic number of class file is garbled.");

     return magic;
  }

  public int getMinorVersion()
  {
     if ( minorVersion == -1 )
     {
        getMagic();
        minorVersion = get2bytes();
     }

     return minorVersion;
  }

  public int getMajorVersion()
  {
     if ( majorVersion == -1 )
     {
        getMinorVersion();
        majorVersion = get2bytes();
     }

     return majorVersion;
  }

  public int getConstantPoolCount()
  {
     if ( constantPoolCount == -1 )
     {
        getMajorVersion();
        constantPoolCount = get2bytes();
     }

     return constantPoolCount;
  }

  public cpInfo[] getConstantPool()
  {
     if ( constantPool == null )
     {
        getConstantPoolCount();
        try
        {
           constantPool = cpInfo.newConstantPool(classFile,constantPoolCount);
        }
        catch(IOException e)
        {
           throw new Error(e.toString());
        }
     }
     
     return constantPool;
  }

  public int getAccessFlags()
  {
     if ( accessFlags == -1 )
     {
        getConstantPool();
        accessFlags = get2bytes();
     }

     return accessFlags;
  }

  public int getThisClass()
  {
     if ( thisClass == -1 )
     {
        getAccessFlags();
        thisClass = get2bytes();
     }

     return thisClass;
  }
  
  public int getSuperClass()
  {
     if ( superClass == -2 )
     {
        getThisClass();
        superClass = get2bytes();
     }

     return superClass;
  }

  public int getInterfacesCount()
  {
     if ( interfacesCount == -1 )
     {
        getSuperClass();
        interfacesCount = get2bytes();
     }

     return interfacesCount;
  }

  public int[] getInterfaces()
  {
     if ( interfaces == null )
     {
        getInterfacesCount();
        
        interfaces = new int[interfacesCount];
        
        for( int i = 0 ; i < interfacesCount ; i++ )
        {
           interfaces[i] = get2bytes();
        }
     }

     return interfaces;
  }

  public int getFieldsCount()
  {
     if ( fieldsCount == -1 )
     {
        getInterfaces();
        fieldsCount = get2bytes();
     }

     return fieldsCount;
  }

  public fieldInfo[] getFields()
  {
     if ( fields == null )
     {
        getFieldsCount();
        try
        {
           fields = fieldInfo.newFields(classFile,
                                        fieldsCount,
                                        constantPool);
        }
        catch(IOException e)
        {
           throw new Error(e.toString());
        }
     }
     return fields;
  }

  public int getMethodsCount()
  {
     if ( methodsCount == -1 )
     {
        getFields();
        methodsCount = get2bytes();
     }
     
     return methodsCount;
  } 

  public methodInfo[] getMethods()
  {
     if ( methods == null )
     {
        getMethodsCount();
        try
        {
           methods = methodInfo.newMethods(classFile,
                                           methodsCount,
                                           constantPool);
        }
        catch(IOException e)
        {
           throw new Error(e.toString());
        }
     }
     
     return methods;
  }

  public int getAttributesCount()
  {
     if ( attributesCount == -1 )
     {
        getMethods();
        attributesCount = get2bytes();
     }

     return attributesCount;
  }

  public attributeInfo[] getAttributes()
  {
     if ( attributes == null )
     {
        getAttributesCount();
        
        try
        {
           attributes = attributeInfo.newAttributes(classFile,
                                                    attributesCount,
                                                    constantPool);
        }
        catch(IOException e)
        {
           throw new Error(e.toString());
        }
        
        try
        {
           classFile.close();
        }
        catch(IOException e)
        {
           // Who cares?
        }
        finally
        {
           classFile = null ; // All is read, we can drop he file by now.
        }
     }
     
     return attributes;
  }
  
  public void decode()
  {
     // Should put there a non recursive version.
     if ( classFile != null ) getAttributes();
  }

  /**
   * Returns a description of the ClassFile object.
   */

  public String toString()
  {
     String  s = "";

     s += "\n";
     s += "minorVersion = " + getMinorVersion() + " ";
     s += "majorVersion : " + getMajorVersion() + "\n";
     s += "\n";

     cpInfo[] cpi = getConstantPool();

     s +=   Modifier.toString(getAccessFlags()) + " " + getScopedName();
     s.trim();

     if ( getSuperClassName() != null ) 
     {
        s += " extends " + getSuperClassName();
     }

     String[] interfaceNames = getInterfaceNames();
     for (int i = 0; i < interfaceNames.length; i++)
     {
        if ( i == 0 ) { s += " implements "; }
        else          { s += ", "; }
        s +=  interfaceNames[i];
       
     }
     s += "\n";
     s += "{\n";
     fieldInfo[] fi = getFields();
     for (int i = 0; i < getFieldsCount(); i++)
     {
        s += "   " +  fi[i] + "\n" ;
     }
     methodInfo[] mi = getMethods();
     for (int i = 0; i < getMethodsCount(); i++)
     {
        s += "   " + mi[i]  + "\n" ;
     }
     s += "}\n";

     return s;
  }
}

// ***********************************************************************
// History Log:
// $Log: ClassFile.java,v $
// Revision 1.9  1998/02/27 12:34:14  cinard
// Cleaned up the API for jbcfile components by introducing a user friendly
// representation for bytecode types (BCType class).
// Updated copyright notice.
//
// Revision 1.8  1998/02/23 14:19:59  cinard
// Improved byte code import algorithm.
//
// Revision 1.7  1998/02/17 10:57:09  mallet
// Merged v1.0 with development trunk
//
// Revision 1.6.2.1  1998/02/16 09:57:52  bareau
// Bug fix: class files being directly accessible from classpath are now
// correctly imported.
//
// Revision 1.6  1998/02/12 16:01:53  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/28 18:11:28  cinard
// Added a check when importing java class bytecode (qualified name of java class
// must be equals to the qualified name in the bytecode).
//
// Revision 1.4  1998/01/12 13:18:24  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.3  1997/11/26 15:49:09  cinard
// Improved Makefile. Changed package name.
//
// Revision 1.2  1997/11/26 14:17:56  cinard
// Distinguished class file check (where full class name is got) from
// class file import => optimization of the JSymbolLoader class.
//
// Revision 1.1  1997/11/12 12:14:38  cinard
// Tool classes  enabling to read java bytecode.
//
// ***********************************************************************
