// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/jbcfile/cpInfo.java,v $
// $Revision: 1.7 $
// $Date: 1998/02/27 12:34:16 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        cpInfo.java
//
// HISTORY:
//      Christian Fabre,      
//      Cyril Inard d'Argence, TOG - Nov 06,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.jbcfile;

import java.io.DataInputStream;

/**
 * Java bytecode's cpInfo.
 */

public class cpInfo implements RuntimeConstants
{
  public static cpInfo[] newConstantPool(DataInputStream classFile, 
					 int cpCount)
       throws java.io.IOException
  {
    cpInfo ret[] = new cpInfo[cpCount];

    if ( ClassFile.debug )
       System.out.println("newConstantPool(<file>,"+cpCount+")");

    for ( int i = 1 ; i < cpCount ; i++ )
    {
       int tag = classFile.readUnsignedByte();
       if ( ClassFile.debug ) 
       {
          System.out.println("cpInfo[" + i + "] = " + tag);
       }

       switch( tag )
       {
       case -1:
          throw new Error("Unexpected EOF.");

       case CONSTANT_STRING:
       case CONSTANT_CLASS:
          ret[i] = new string(classFile.readUnsignedShort());
          break;

       case CONSTANT_FIELD:
       case CONSTANT_FLOAT:
       case CONSTANT_INTEGER:
       case CONSTANT_INTERFACEMETHOD:
       case CONSTANT_METHOD:
       case CONSTANT_NAMEANDTYPE:
          classFile.skip(4);
          break;

       case CONSTANT_LONG:
       case CONSTANT_DOUBLE:
          classFile.skip(8);
          i++;
          /*  
           *  We have to increment i here as well, because "All
           *  eight-byte entries takes up two entries in the
           *  constant_pool table of the class file,...". See
           *  [TJVMS] page 98.  They reckon that this was a poor
           *  choice.
           */
          break;

       case CONSTANT_UTF8:
          int len = classFile.readUnsignedShort();
          byte val[] = new byte[len];
          
          int numberRead = 0;
          do
          {
             numberRead += classFile.read(val,numberRead,len - numberRead);  
          }
          while (numberRead < len);

          ret[i] = new utf8(val);
          break;
          
       default:
          throw new Error("Unexpected constant pool tag:"+tag);
       }
    }

    return ret;
  }
}

// ***********************************************************************
// History Log:
// $Log: cpInfo.java,v $
// Revision 1.7  1998/02/27 12:34:16  cinard
// Cleaned up the API for jbcfile components by introducing a user friendly
// representation for bytecode types (BCType class).
// Updated copyright notice.
//
// Revision 1.6  1998/02/12 16:01:55  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/29 08:59:11  cinard
// Removed file parameter from newCpInfo() method.
//
// Revision 1.4  1998/01/27 13:07:21  cinard
// The jbcfile package no longer make use of the sun.tools.java package.
// An interface RuntimeConstants defines the bytecode constants now.
//
// Revision 1.3  1998/01/12 13:18:26  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.2  1997/11/26 15:49:10  cinard
// Improved Makefile. Changed package name.
//
// Revision 1.1  1997/11/12 12:14:39  cinard
// Tool classes  enabling to read java bytecode.
//
// ***********************************************************************
