// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/jbcfile/fieldInfo.java,v $
// $Revision: 1.6 $
// $Date: 1998/02/27 12:34:16 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        fieldInfo.java
//
// HISTORY:
//      Christian Fabre,      
//      Cyril Inard d'Argence, TOG - Nov 06,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.jbcfile;

import java.io.DataInputStream;
import java.lang.reflect.*;

/**
 * Stores information from Java bytecode's fieldInfo.
 */

public class fieldInfo implements RuntimeConstants
{
  private static final boolean debug = false;

  protected fieldInfo
            (
               short access_flags,
               short name_index,
               short descriptor_index,
               cpInfo[] CP
            )
  {
     this.access_flags     = access_flags;
     this.name_index       = name_index;
     this.descriptor_index = descriptor_index;
     this.CP               = CP;
     this.attribute_info   = null;
  }

  public short getAccessFlag()
  {
     return access_flags;
  }

  public String getName()
  {
     return  CP[name_index].toString();
  }

  public BCType getType()
  {
     return new BCType(CP[descriptor_index].toString()); 
  }

  public boolean hasRelatedClass()
  {
     return getType().isClass();
  }

  public String getRelatedClass()
  {
     if ( getType().isClass() )
     {
        return getType().getName();
     }
     else
     {
        return null;
     }
  }

  public static fieldInfo[] newFields(DataInputStream classFile,
                                      int fieldCount,
                                      cpInfo CP[])
     throws java.io.IOException
  {

    fieldInfo[] ret = new fieldInfo[fieldCount];
    for ( int i = 0 ; i < fieldCount ; i++ )
    {
       ret[i] = new fieldInfo(classFile.readShort(), // access_flags
                              classFile.readShort(), // name_index
                              classFile.readShort(), // descriptor_index
                              CP
                              );
       
       int attributes_count = classFile.readShort();
       
       ret[i].attribute_info   = attributeInfo.newAttributes(classFile,
                                                             attributes_count,
                                                             CP);
    }
    
    return ret;
  }

  public String toString()
  {
    String s =   Modifier.toString(access_flags) + " " + 
                 getType() + " " +  getName();
    return s;
  }

  private short  access_flags;
  private short  name_index;
  private short  descriptor_index;
  private attributeInfo[] attribute_info;
  private cpInfo[] CP = null;

}

// ***********************************************************************
// History Log:
// $Log: fieldInfo.java,v $
// Revision 1.6  1998/02/27 12:34:16  cinard
// Cleaned up the API for jbcfile components by introducing a user friendly
// representation for bytecode types (BCType class).
// Updated copyright notice.
//
// Revision 1.5  1998/02/12 16:01:55  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.4  1998/01/27 13:07:21  cinard
// The jbcfile package no longer make use of the sun.tools.java package.
// An interface RuntimeConstants defines the bytecode constants now.
//
// Revision 1.3  1998/01/12 13:18:26  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.2  1997/11/26 15:49:10  cinard
// Improved Makefile. Changed package name.
//
// Revision 1.1  1997/11/12 12:14:39  cinard
// Tool classes  enabling to read java bytecode.
//
// ***********************************************************************
