// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/jbcfile/methodInfo.java,v $
// $Revision: 1.8 $
// $Date: 1998/02/27 12:34:16 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        methodInfo.java
//
// HISTORY:
//      Christian Fabre,      
//      Cyril Inard d'Argence, TOG - Nov 06,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.jbcfile;

import java.io.DataInputStream;
import java.lang.reflect.*;
import java.util.Vector;

/**
 * Stores information from Java bytecode's methodInfo.
 */

public class methodInfo implements RuntimeConstants
{
  private static final boolean debug = false;

  // Public constructor ============================================

  public methodInfo
         (
            int     access_flags,
            int     name_index,
            int     descriptor_index,
            cpInfo[]  CP
         )
  {
     this.access_flags     = access_flags;
     this.name_index       = name_index;
     this.descriptor_index = descriptor_index;
     this.CP               = CP;
     this.attribute_info   = null;
  }

  // Methods used to manipulate methodInfo =========================

  public int getAccessFlag()
  {
     return access_flags;
  }

  public String getName()
  {
     return CP[name_index].toString();
  }

  public BCType getReturnType()
  {
      String descriptor = CP[descriptor_index].toString();
      int idxBeg = descriptor.indexOf(")");
      return new BCType(descriptor.substring(idxBeg+1,descriptor.length()));
  }

  public BCType[] getParameters()
  {
      String descriptor = CP[descriptor_index].toString();
      int idxBeg = descriptor.indexOf("(");
      int idxEnd = descriptor.indexOf(")");
      return BCType.getBCTypes(descriptor.substring(idxBeg+1,idxEnd));
  }

  public String[] getExceptions()
  {
     String[] ret = null;
     if ( attribute_info != null && attribute_info.length > 0 )
     {
        attributeInfo exceptionAttribute = attribute_info[0];
        ret = new String[exceptionAttribute.number_of_exceptions];
        for ( int i = 0; i < exceptionAttribute.number_of_exceptions; i++ )
        {
           int index = exceptionAttribute.exception_index_table[i];
           int index2 = ((string)(CP[index])).index;
           ret[i] = CP[index2].toString().replace('/','.');        
        }
     }
     else
     {
        ret = new String[0]; 
     }
     return ret; 
  }


  public String[] getRelatedClasses()
  {
    Vector related = new Vector();

    BCType returnType  = getReturnType();
    if ( returnType.isClass() )
    {
       // It's a class
       related.addElement(returnType.getName());
    }

    BCType[] parameters = getParameters();
    for (int j = 0; j < parameters.length; j++)
    {
        if ( parameters[j].isClass() )
        {
           // It's a class
           related.addElement(parameters[j].getName());
        }
     }

    String[] exceptionNames = getExceptions();
    for ( int k = 0; k < exceptionNames.length; k++)
    {
      related.addElement(exceptionNames[k]); 
    }

    String[] ret = new String[related.size()];
    related.copyInto(ret);
    return ret;
  }

  // Byte code extracting methods ======================================

  public static methodInfo[] newMethods(DataInputStream classFile,
                                        int methodCount,
                                        cpInfo CP[])
     throws java.io.IOException
  {
     if ( ClassFile.debug )
        System.out.println("newMethods(<file>,"+methodCount
                           +", CP (len="+CP.length+"))");
     
     methodInfo[] ret = new methodInfo[methodCount];
     for ( int i = 0 ; i < methodCount ; i++ )
     {
        ret[i] = new methodInfo
                     (
                      classFile.readUnsignedShort(), // access_flags
                      classFile.readUnsignedShort(), // name_index
                      classFile.readUnsignedShort(), // descriptor_index
                      CP
                     );
	
        int attributes_count = classFile.readUnsignedShort();

        ret[i].attribute_info = attributeInfo.newAttributes
                                (classFile,attributes_count,CP);
     }

     return ret;
  }

  public String toString()
  {
     String s = Modifier.toString(access_flags) 
              + " " + getReturnType() 
              + " " + getName();

     BCType[] parameters = getParameters();
     
     s += "(";
     for (int i = 0; i < parameters.length; i++)
     {
        if ( i > 0 ) { s += ", "; }
        s += parameters[i];
     } 
     s += ")";

     String[] exc = getExceptions();

     for ( int j = 0; j < exc.length; j++)
     {
        if   ( j == 0 ) { s += " throws "; }
        else            { s += ", "; }
        s += exc[j];
     }

     return s;
  }

  private int access_flags;
  private int name_index;
  private int descriptor_index;
  private attributeInfo[]  attribute_info;
  private cpInfo[] CP = null;
}


// ***********************************************************************
// History Log:
// $Log: methodInfo.java,v $
// Revision 1.8  1998/02/27 12:34:16  cinard
// Cleaned up the API for jbcfile components by introducing a user friendly
// representation for bytecode types (BCType class).
// Updated copyright notice.
//
// Revision 1.7  1998/02/23 14:20:00  cinard
// Improved byte code import algorithm.
//
// Revision 1.6  1998/02/12 16:01:55  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/27 13:07:21  cinard
// The jbcfile package no longer make use of the sun.tools.java package.
// An interface RuntimeConstants defines the bytecode constants now.
//
// Revision 1.4  1998/01/12 13:18:26  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.3  1997/12/19 21:14:46  cinard
// Added extraction of thrown clause in bytecode.
//
// Revision 1.2  1997/11/26 15:49:10  cinard
// Improved Makefile. Changed package name.
//
// Revision 1.1  1997/11/12 12:14:39  cinard
// Tool classes  enabling to read java bytecode.
//
// ***********************************************************************
