package ORG.opengroup.adl.parsers.adlc;

import ORG.opengroup.adl.parsers.*;

public class ADLCGenericVisitor implements ADLCParserVisitor
{

  static final private boolean debug = false;

  /**
    * Creates a parenthood relation between 2 nodes.
    * @param p parent node
    * @param c child node
    * @param i "c is the ith child of p"
    */
  public static void parentChild(Node p, Node c, int i) {
    p.jjtAddChild(c, i);
    c.jjtSetParent(p);
  }

  /**
    * Creates a node of type ASTString from a string
    * @param str the input string
    * @return the  node ASTString
    */
  public static ASTString makeString(String str) {

    if (debug) System.out.println("\tmakeString: "+ str);
    ASTString name = new ASTString(ADLCParserTreeConstants.JJTSTRING);
    name.setName("\"" + str + "\"");
    return name;
  }

  /**
    * Creates a node of type ASTConstant from a string
    * @param str the input string: true or false
    * @return the ASTConstant node
    */
  public static ASTConstant makeConstant(String str) {

    if (debug) System.out.println("\tmakeConstant: "+ str);
    ASTConstant name = new ASTConstant(ADLCParserTreeConstants.JJTCONSTANT);
    name.setName(str);
    return name;
  }

  /**
    * Creates a node of type ASTConstant from a boolean string
    * @param str the input string: true or false
    * @return the ASTConstant node
    */
  public static ASTConstant makeBoolean(String str) {

    if (debug) System.out.println("\tmakeBoolean: "+ str);
    ASTConstant name = new ASTConstant(ADLCParserTreeConstants.JJTCONSTANT);
    name.setTypeName("boolean");
    name.setValue(str);
    return name;
  }

  /**
    * Creates a node of type ASTName from a string
    * @param str the input string
    * @return the ASTName node
    */
  public static ASTName makeName (String str) {

    if (debug) System.out.println("\tmakeIdExpr: "+ str);
    ASTName name = new ASTName(ADLCParserTreeConstants.JJTNAME);
    name.setName(str);
    return name;
  }

  /**
    * Creates a node of type ASTDeclarationSpecifiers from a string
    * @param str the input string
    * @return the ASTDeclarationSpecifiers node
    */
  public static ASTDeclarationSpecifiers makeBuiltinType(String str) {

    if (debug) System.out.println("\tmakeBuiltinType: "+ str);
    ASTDeclarationSpecifiers ret = new
      ASTDeclarationSpecifiers(ADLCParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    ASTBuiltinTypeSpecifier type = new
      ASTBuiltinTypeSpecifier(ADLCParserTreeConstants.JJTBUILTINTYPESPECIFIER);
    type.setName(str);
    parentChild(ret, type, 0);
    return ret;
  }

  /**
    * Creates a node of type ASTDeclarationSpecifiers from a string
    * @param str the input string
    * @return the ASTDeclarationSpecifiers node
    */
  public static ASTDeclarationSpecifiers makeType(String str) {

    if (debug) System.out.println("\tmakeType: "+ str);
    ASTDeclarationSpecifiers type = new
      ASTDeclarationSpecifiers(ADLCParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    ASTName name = new ASTName(ADLCParserTreeConstants.JJTNAME);
    name.setName(str);
    parentChild(type, name, 0);
    return type;
  }

  /**
    * Creates a node of type ASTPtrOperator from a string
    * @param str the input string
    * @return the  node ASTPtrOperator
    */
  public static ASTPtrOperator makePtrOperator(String id) {

    if (debug) System.out.println("\tmakePtrOperator: " + id);
    ASTPtrOperator type = new ASTPtrOperator(ADLCParserTreeConstants.JJTPTROPERATOR);

    try {
      if(id.equals("ReferenceTo"))
        type.setProperty("ptrkind", id);
      else if(id.equals("PointerTo"))
        type.setProperty("ptrkind",id);
      else if(id.equals("PointerToMember"))
        type.setProperty("ptrkind",id);
      else
        type.setProperty("ptrkind",null);
    }
    catch (PropertyAccessException e) { }

    return type;
  }


  public Object visit(SimpleNode node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTranslationUnit node, Object data) {

  return node.childrenAccept(this, data);
 }

  public Object visit(ASTIncludeFileDeclaration node, Object data) {

  return data;
 }


  public Object visit(ASTADL_TranslationUnit node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTIncludeFileList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_AnnotatedDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_InlineDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_BehaviorDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Prolog node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Epilog node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_BehaviorSpecification node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_BehaviorClassification node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_NormalBehavior node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_AbnormalBehavior node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_AssertionGroup node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Binding node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_NamedParamList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_NamedParam node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Statement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_IfStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Assertion node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Labels node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Tags node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_QuantifiedAssertion node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Quantifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_DomainList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Domain node, Object data) { 

    return  node.childrenAccept(this, data);
  }

  public Object visit(ASTADL_DomainExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_ImplExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_ImplOp node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_CallStateExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Normal node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Abnormal node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Unchanged node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_ArgumentList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTName node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTExternalDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTFunctionDefinition node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDeclarationSpecifiers node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTypeQualifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTStorageStructSpecifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTBuiltinTypeSpecifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTInitDeclaratorList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTInitDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTStructSpecifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTMemberDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTMemberDeclaratorList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTEnumSpecifier node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTEnumeratorList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTEnumerator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTPtrOperator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTCvQualifierSeq node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDirectDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDeclaratorSuffixes node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTFunctionDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTFunctionDirectDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTParameterList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTParameterDeclarationList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTParameterDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTInitializer node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTypeName node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTAbstractDeclarator node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTAbstractDeclaratorSuffix node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTLabeledStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTCaseLabelStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDefaultLabelStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTCompoundStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTIfStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTSwitchStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTWhileStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDoWhileStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTForStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTGotoStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTContinueStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTBreakStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTReturnStatement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTAssignmentExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTConditionalExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTConstantExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTLogicalOrExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTLogicalAndExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTInclusiveOrExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTExclusiveOrExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTAndExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTEqualityExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTRelationalExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTShiftExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTAdditiveExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTMultiplicativeExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTCastExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTPreIncrementExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTPreDecrementExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTUnaryOperatorExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTSizeOfExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTPostfixExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTArraySuffix node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTArgumentList node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTDotAccessSuffix node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTRefAccessSuffix node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTPostDeIncrement node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTString node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTADL_Return node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTParentheizedExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTUnaryPlusMinusConstant node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTConstant node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_AnnotatedDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_ImportDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_UseDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_Declaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_FieldDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetDeclaration node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_FactoryDefinition node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_TestDirective node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetDomain node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetLiteral node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetMember node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetConcatenationExpr node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_DatasetSingleton node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_FactoryCall node, Object data) { 

    return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTDD_ADLExpression node, Object data) { 

    return  node.childrenAccept(this, data);
  }

}
