// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ************************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/parsers/adlc/ADLTypeCheckHelper.java,v $
// $Revision: 1.2 $
// $Date: 1998/06/29 16:15:41 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL for C parser
// NAME:        ADLTypeCheckHelper.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group. - Dec 01, 1997: Created
//
// ************************************************************************

package ORG.opengroup.adl.parsers.adlc;

import java.util.Stack;
import java.util.Enumeration;
import java.util.Vector;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

/**
 * Helper class for type check visitor.
 */

public class ADLTypeCheckHelper
{
  /** 
   * Check types of children of a node are scalar.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreScalar
                      (SimpleNode parent)
  throws BadTypeException
  {
     return checkTypesAreIntegral
            (parent, 0, parent.jjtGetNumChildren() - 1);
  }

  /** 
   * Check types of children of a node are scalar.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @param first index of first child node to consider
   * @param last index of last child node to consider
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreScalar
                      (SimpleNode parent,int first,int last)
  throws BadTypeException
  {
     LType[] types = new LType[last - first + 1];

     for (int i = first; i <= last; i++)
     {
        TypedNode child = (TypedNode)parent.jjtGetChild(i);

        if ( ! CType.isScalar(child.getType()) )
        {
           throw new BadTypeException
                     (child,child.getTypeName() + " not a scalar");
        }
        else
        {
           types[i -first] = child.getType();
        }
     }
     
     try
     {
        return synthetizeTypes(types);
     }
     catch ( BadTypeException e )
     {
        throw new Error("Error in type synthesis : " + e.getMessage());
     }
  }

  /** 
   * Check types of children of a node are integral.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreIntegral
                      (SimpleNode parent)
  throws BadTypeException
  {
     return checkTypesAreIntegral
            (parent, 0, parent.jjtGetNumChildren() - 1);
  }

  /** 
   * Check types of children of a node are integral.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @param first index of first child node to consider
   * @param last index of last child node to consider
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreIntegral
                      (SimpleNode parent,int first,int last)
  throws BadTypeException
  {
     LType[] types = new LType[last - first + 1];

     for (int i = first; i <= last; i++)
     {
        TypedNode child = (TypedNode)parent.jjtGetChild(i);

        if ( ! CType.isIntegral(child.getType()) )
        {
           throw new BadTypeException
                     (child,child.getTypeName() + " not an integral");
        }
        else
        {
           types[i -first] = child.getType();
        }
     }
     
     try
     {
        return synthetizeTypes(types);
     }
     catch ( BadTypeException e )
     {
        throw new Error("Error in type synthesis : " + e.getMessage());
     }
  }

  /** 
   * Check types of children of a node are arithmetic.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreArithmetic
                      (SimpleNode parent)
  throws BadTypeException
  {
     return checkTypesAreArithmetic
            (parent, 0, parent.jjtGetNumChildren() - 1);
  }

  /** 
   * Check types of children of a node are arithmetic.
   * If this is true the method returns the synthetized type.
   * @param parent the parent node of the typed nodes in the AST for
   *               which the check is to be performed.
   * @param first index of first child node to consider
   * @param last index of last child node to consider
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType checkTypesAreArithmetic
                      (SimpleNode parent,int first,int last)
  throws BadTypeException
  {
     LType[] types = new LType[last - first + 1];

     for (int i = first; i <= last; i++)
     {
        TypedNode child = (TypedNode)parent.jjtGetChild(i);

        if ( ! CType.isArithmetic(child.getType()) )
        {
           throw new BadTypeException
                     (child,child.getTypeName() + " not an arithmetic");
        }
        else
        {
           types[i -first] = child.getType();
        }
     }
     
     try
     {
        return synthetizeTypes(types);
     }
     catch ( BadTypeException e )
     {
        throw new Error("Error in type synthesis : " + e.getMessage());
     }
  }


  /** 
   * Synthetize types, with types being a subset of an AST node's children.
   * Note children must be instance of TypedNode and must exist.
   * @see ADLTypeCheckHelper#synthetizeTypes(LType[])
   * @param parent the parent node of the typed nodes in the AST for
   *               which the synthesis is to be performed.
   * @return the synthetized types if types are compatible, null otherwise.
   */
  static public LType synthetizeTypes(SimpleNode parent)
  throws BadTypeException
  {
     return synthetizeTypes(parent, 0, parent.jjtGetNumChildren() - 1);
  }

  /** 
   * Synthetize types, with types being a subset of an AST node's children.
   * Note children must be instance of TypedNode and must exist.
   * @see ADLTypeCheckHelper#synthetizeTypes(LType[])
   * @param parent the parent node of the typed nodes in the AST for
   *               which the synthesis is to be performed.
   * @param first  index of the first child typed node to consider
   * @param last   index of the last child typed node to consider 
   * @return the synthetized types if types are compatible, null otherwise.
   */
  static public LType synthetizeTypes(SimpleNode parent, int first, int last)
  throws BadTypeException
  {
     LType ret = null;

     LType[] tmp = new LType[last - first + 1];

     for (int i = first; i <= last; i++)
     {
        SimpleNode child = (SimpleNode)parent.jjtGetChild(i);

        if ( child != null && child instanceof TypedNode )
        {
           tmp[i - first] = ((TypedNode)child).getType();
           if ( tmp[i - first] == null )
           {
              throw new BadTypeException(child,"No type for expression.");
           }
        }
        else
        {
           String msg =    ( child == null )
                         ? ( "provided node has no child with index " + i )
                         : ( "trying to get the type of a non typed node." ) ;
           throw new Error(msg);
        }
     }

     try
     {
        ret = synthetizeTypes(tmp);
     }
     catch ( BadTypeException e )
     {
        e.setErrorNode
        ((TypedNode)(parent.jjtGetChild(first + e.getErrorIndex())));
        throw e;
     }
     return ret;
  }

  /** 
   * Synthetize types.
   * @see ADLTypeCheckHelper#synthetizeTypes(LType[])
   * @param typedNodes array containing the typed nodes to synthetize.
   * @return the synthetized types if types are compatible, null otherwise.
   */

  static public LType synthetizeTypes(TypedNode[] typedNodes)
  throws BadTypeException
  {
     LType ret = null;

     LType[] tmp = new LType[typedNodes.length];

     for (int i = 0; i < typedNodes.length; i++ )
     {
        if ( typedNodes[i] != null )
        {
           tmp[i] = typedNodes[i].getType();
        }
        if ( tmp[i] == null )
        {
           throw new BadTypeException(typedNodes[i],"No type for expression.");
        }

     }

     try
     {
        ret = synthetizeTypes(tmp);
     }
     catch ( BadTypeException e )
     {
        e.setErrorNode
        (typedNodes[e.getErrorIndex()]);
        throw e;
     }
     return ret;
  }


  // -- Private methods --

  // TBD - improve the following

  /** 
   * Synthetize types (types are provided as an array of types).
   * That is return the mininum type among the list of provided types 
   * considering the Java language types partial ordering.
   * @param types  array of types to be considered for synthesis.
   * @return the synthetized type if types are compatible, null otherwise.
   */

  public static LType synthetizeTypes(LType[] types)
  throws BadTypeException
  {
     int erroneousTypeIndex = 0;

     LType synthetizedType = ( types.length > 0 ) ? types[0] : null;

     // loop on children

     for ( int i = 1; i < types.length; i++ )
     {
        LType candidateType = types[i];

        if ( candidateType != null )
        {
           if (  synthetizedType instanceof CBuiltinType )
           {
              synthetizedType =  synthetizedType.synthesisType(candidateType);
           }
           else if ( candidateType instanceof CBuiltinType )
           {
              synthetizedType = candidateType.synthesisType(synthetizedType);
           }
           else
           {
              synthetizedType = synthetizedType.synthesisType(candidateType);
           }
        }
        if ( synthetizedType == null )
        {
           // error in type synthesis
           erroneousTypeIndex = i;
           break;
        }
     }

     if ( ( types.length > 0 ) && ( synthetizedType == null ) ) 
     {
        String msg = "Cannot convert " 
                   + types[erroneousTypeIndex] + " to " + types[0];

        throw new BadTypeException(erroneousTypeIndex,msg);
     }

     return synthetizedType ;
  }

  /** Debug flag. */
  private static final boolean debug = false ;

}
