// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ************************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/parsers/adlc/TypedNode.java,v $
// $Revision: 1.1 $
// $Date: 1998/04/28 13:03:21 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL for C parser
// NAME:        TypedNode.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group. - Dec 01, 1997: Created
//
// ************************************************************************

package ORG.opengroup.adl.parsers.adlc;

import ORG.opengroup.adl.symboltable.LType;

/**
 * Class that typed nodes (expression nodes) shall inherit from.
 */

public class TypedNode extends SimpleNode
{

  /** Type of the current node */
  private LType lType;

  /** Name of the type of the current node */
  private String typeName = "<no type>";


  public TypedNode(int id) {
    super(id);
  }


  public TypedNode(ADLCParser p, int id) {
    super(p, id);
  }


  /** Set current node type name. */
  public void setTypeName(String typeName) {
    this.typeName = typeName;
  }


  /** Get current node type name */
  public String getTypeName() {
    return this.typeName;
  }


  /** Set symbol defining current node type. */
  public void  setType(LType lType) 
  {
     this.lType    = lType;
     if ( lType != null ) {
       this.typeName = lType.getName();
     }
  }


  /** Get symbol defining current node type. */
  public LType getType() {
    return lType;
  }


  /** 
   *  Get the current node type kind. Useful for languages that defines
   *  several type kinds (e.g. in Java: boolean, arithmetic and references).
   *  @return an integer indicating the kind of the current node type. 
   */
  public Integer getTypeKind() 
  { 
     Integer ret = null;
     if ( this.lType != null )
       ret = this.lType.getTypeKind();
     return ret;
  } 


  /**
   * Returns true if current node type is equal to supplied typed
   * expression, false otherwise.
   * @param t the typed node for which the equality is checked.
   */
  public boolean equals(LType t) {
    return this.getType().equals(t);
  }


  /**
   * Can a value of type t be assigned to value of current node type.
   * @param  t The typed node  for which the assignability is checked.
   * @return True if it can, false otherwise.
   */
  public boolean assignable(LType t) {
    return this.getType().assignable(t);
  }

}
