// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/parsers/adlcpp/ADLCPPGenericVisitor.java,v $
// $Revision: 1.11 $
// $Date: 1998/06/29 16:30:40 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL/C++ Parser (generic visitor)
// NAME:        ADLCPPGenericVisitor.java
//
// HISTORY:
//      W. Berrayana, The Open Group RI - Jun 24, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;

public class ADLCPPGenericVisitor implements ADLCPPParserVisitor
{

  static final private boolean debug = false;

  /**
    * Creates a parenthood relation between 2 nodes.
    * @param p parent node
    * @param c child node
    * @param i "c is the ith child of p"
    */
  public static void parentChild(Node p, Node c, int i) {
    p.jjtAddChild(c, i);
    c.jjtSetParent(p);
  }

  /**
    * Creates a node of type ASTString from a string
    * @param str the input string
    * @return the  node ASTString
    */
  public static ASTString makeString(String str) {

    if (debug) System.out.println("\tmakeString: "+ str);
    ASTString name = new ASTString(ADLCPPParserTreeConstants.JJTSTRING);
    name.setName(str);
    return name;
  }

  /**
    * Creates a node of type ASTConstant from a string
    * @param str the input string: true or false
    * @return the ASTConstant node
    */
  public static ASTConstant makeConstant(String str) {

    if (debug) System.out.println("\tmakeConstant: "+ str);
    ASTConstant name = new ASTConstant(ADLCPPParserTreeConstants.JJTCONSTANT);
    name.setCstVal(str);
    return name;
  }

  /**
    * Creates a node of type ASTConstant from a boolean string
    * @param str the input string: true or false
    * @return the ASTConstant node
    */
  public static ASTConstant makeBoolean(String str) {

    if (debug) System.out.println("\tmakeBoolean: "+ str);
    ASTConstant name = new ASTConstant(ADLCPPParserTreeConstants.JJTCONSTANT);
    name.setTypeName("boolean");
    name.setCstVal(str);
    return name;
  }

  /**
    * Creates a node of type ASTIdExpr from a string
    * @param str the input string
    * @return the ASTIdExpr node
    */
  public static ASTIdExpr makeIdExpr (String str) {

    if (debug) System.out.println("\tmakeIdExpr: "+ str);
    ASTIdExpr name = new ASTIdExpr(ADLCPPParserTreeConstants.JJTIDEXPR);
    name.setIdName(str);
    return name;
  }

  /**
    * Creates a node of type ASTBuiltinTypeSpec from a string
    * @param str the input string
    * @return the ASTBuiltinTypeSpec node
    */
  public static ASTBuiltinTypeSpec makeBuiltinType(String str) {

    if (debug) System.out.println("\tmakeBuiltinType: "+ str);
    ASTBuiltinTypeSpec type = new ASTBuiltinTypeSpec(ADLCPPParserTreeConstants.JJTBUILTINTYPESPEC);
    type.setType(str);
    return type;
  }

  /**
    * Creates a node of type ASTQualifiedType from a string
    * @param str the input string
    * @return the  node ASTQualifiedType
    */
  public static ASTQualifiedType makeQualifiedType(String str) {

    if (debug) System.out.println("\tmakeQualifiedType: "+ str);
    ASTQualifiedType type = new ASTQualifiedType(ADLCPPParserTreeConstants.JJTQUALIFIEDTYPE);
    type.setType(str);
    return type;
  }

  /**
    * Creates a node of type ASTQualifiedId from a string
    * @param str the input string
    * @return the  node ASTQualifiedId
    */
  public static ASTQualifiedId makeQualifiedId(String str) {

    if (debug) System.out.println("\tmakeQualifiedId: "+ str);
    ASTQualifiedId id = new ASTQualifiedId(ADLCPPParserTreeConstants.JJTQUALIFIEDID);
    id.setName(str);
    return id;
  }

  /**
    * Creates a node of type ASTPtrOperator from a string
    * @param str the input string
    * @return the  node ASTPtrOperator
    */
  public static ASTPtrOperator makePtrOperator(String str) {

    if (debug) System.out.println("\tmakePtrOperator: "+ str);
    ASTPtrOperator type = new ASTPtrOperator(ADLCPPParserTreeConstants.JJTPTROPERATOR);
    type.setPtrkind(str);
    return type;
  }


  public Object visit(SimpleNode node, Object data) throws ParseException { 
   return  node.childrenAccept(this, data);
  }


  public Object visit(ASTTranslationUnit node, Object data) throws ParseException {

  return node.childrenAccept(this, data);
 }

  public Object visit(ASTIncludeFileDeclaration node, Object data) throws ParseException {

  return data;
 }



  public Object visit(ASTIncludeFileList node, Object data) throws ParseException {

  return node.childrenAccept(this, data) ;
 }

  public Object visit(ASTClassMemberList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTFunctionDefinition node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTExceptionDeclaration node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTExceptionSpec node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }



  public Object visit(ASTDeclaration node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTLinkageSpecification node, Object data)
  throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDeclarationSpecifiers node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTQualifiedId node, Object data) throws ParseException {
  return node.childrenAccept(this, data);
  }

  public Object visit(ASTQualifiedType node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTFullyScopedType node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
 }

  public Object visit(ASTBuiltinTypeSpec node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTInitDeclaratorList node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTInitDeclarator node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTInitializer node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }



public Object visit(ASTEnumSpecifier node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

public Object visit(ASTEnumerator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
public Object visit(ASTEnumeratorList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }



  public Object visit(ASTClassSpecifier node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTBaseClause node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTAccessSpecifier node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTBaseSpecifier node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTMemberDeclaration node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTMemberDeclaratorList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTPtrOperator node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDtorDefinition node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTCtorDefinition node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTCtorDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTSuperclassInit node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTCtorInitializer node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }



  public Object visit(ASTDtorDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDirectDeclarator node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTAbstractDeclarator node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTAbstractDeclaratorSuffix node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDeclaratorSuffixes node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTFunctionDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTFunctionDirectDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTParameterDeclarationList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTParameterDeclaration node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTLabelNode node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTCaseExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTNewExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTNewTypeId node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTNewDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDirectNewDeclarator node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }



  public Object visit(ASTDeleteExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }




  public Object visit(ASTCompoundStatement node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTStatement node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

 public Object visit(ASTADLNamedParamList node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }


 public Object visit(ASTADLDomain node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTReturnStmt node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTThrowStmt node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTElseNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTIfNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTSwitchNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTWhileNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTDoWhileNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTForNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTGotoNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTContinueNode node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTBreakNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTryNode node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTCatchNode node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTAssignmentExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data); 
  }
  public Object visit(ASTCondExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
 }
  public Object visit(ASTConstantExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTBinaryLogicalExpr node, Object data) throws ParseException   {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTBinaryArithExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTPmExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTCastExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTypeName node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTypeNameList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTUnaryExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLCallStateExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTPostfixExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLImplExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTArraySuffix node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTDotAccessSuffix node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTRefAccessSuffix node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTPostDeIncrement node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTArgumentList node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTPrimaryExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTIdExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTThis node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTString node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTParenthExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTConstant node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLClass node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLBhvDecl node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLBhvSpec node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLBhvClassification node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLProlog node, Object data) throws ParseException   { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLEpilog node, Object data) throws ParseException   { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLAssGrp node, Object data) throws ParseException   { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLInhSemantics node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLBinding node, Object data) throws ParseException {
    return node.childrenAccept(this, data); 
  }
  public Object visit(ASTADLInlineDecl node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLSuperClass node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLIfStmt node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTADLTryStmt node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLDomainList node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLAssertion node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTReturn node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLThrownExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTNormalExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTAbnormalExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTUnchangedExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTADLQuantifiedAssertion node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLLabel node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTADLTags node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTDDADLnewExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDADLExpr node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTTDDUseDeclaration node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDClassDecl node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDFieldDeclaration node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTTDDDatasetDecl node, Object data) throws ParseException { 
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDFactoryDef node,Object data) throws ParseException{ 
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTDDDatasetExpr node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDDatasetSingleton node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }
  public Object visit(ASTTDDFactoryCall node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTDDTestDirective node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }


  public Object visit(ASTTDDDatasetDomain node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTDDDatasetLiteral node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

  public Object visit(ASTTDDDatasetMember node, Object data) throws ParseException {
    return node.childrenAccept(this, data);
  }

}
// ***********************************************************************
// History Log:
// $Log: ADLCPPGenericVisitor.java,v $
// Revision 1.11  1998/06/29 16:30:40  mallet
// Added header and footer.
// Added import of ORG.opengroup.adl.parsers.ParseException.
// Removed visit of ASTNLDAnnotation: this node is no longer created.
//
// ***********************************************************************
