// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTADLBhvDecl.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -October,97:Modified to Introduce Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;
import java.util.*;
import com.objectspace.jgl.UnaryPredicate;


public class ASTADLBhvDecl extends SimpleNode {

  /** Basic constructor used when parser is static */
  public ASTADLBhvDecl(int id) {
    super(id);
  }
  /** Basic parser used when parser non-static */
  public ASTADLBhvDecl(ADLCPPParser p, int id) {
    super(p, id);
  }


  public static Node jjtCreate(int id) {
      return new ASTADLBhvDecl(id);
  }


  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTADLBhvDecl(p, id);
  }

    
 /**
   * Copy source function parameters and exceptions into destination method 
   * <p><b>Algorithm:</b><ul>
   * <li> Get source function parameters.
   * <li> loop to add each of these parameters to destination function.
   * <li> Do the same thing for exceptions.
   * <li> return the destination function with parameters and exceptions added.
   * </ul>
   * @param source function from which parameters and exceptions are extracted.
   * @param destination method to which source function parameters and 
   * exceptions are copied.
   * @result destination function modified.
   */
  public static LCallable copyMethodParametersExceptions 
  ( LCallable source, LCallable destination) {

    LParameter _params[] = source.getParameters() ;
    // copy method parameters. LParameter _params [] = source.getParameters() ;
    for ( int i = 0 ; i < _params.length ; i++ ) {
      destination.addParameter( _params[i] ) ;
    }

    // copy method exceptions 
    LType _exceptions[] = source.getExceptions() ;
    for ( int i = 0 ; i < _exceptions.length ; i++ ) {
      destination.addException ( _exceptions[i] ) ;
    } 

    return destination ;
  }
    

  /**
   * Build and check parsed ADL method.
   *
   * <p><b>Algorithm:</b><ul>
   * <li> Get function name, modifier and updated type.
   * <li> Build an ADL method with that type, mofifier and name.
   * <li> Add parameters to ADL contsructed ADL method.
   * <li> Build an auxiliary test function having the same signature as the
   * ADL method using method <a href="#copyMethodParametersExceptions">
   * copyMethodParametersExceptions </a>
   * <li> get comparator of the auxiliary function and make sure that a method
   * having the same comparator (signature ) is defined in C++ tested class.
   * method search is made locally in the C++ tested class and its base classes
   * <li> Save the ADLmethod into the current scope ( ADL class ) and as a
   * class field (ResultFct).
   * </ul>
   * @param clazzCPP the C++ class whose methods are ADL checked.
   * @param name of C++ class under test.
   * @result the recovered ADL method.
   */
  public LCallable checkADLBhvFct (Scope clazzCPP ,String className ) 
   throws ParseException {

    // get fucntion name.
    String fctname = getADLBhvFctName( );
    // get function type and update it
    LType fcttype = getADLBhvFctReturnType();
    fcttype = UpdateBhvFctType( fcttype);
    // get function modifier.
    int modif = adlBhvModifier ;
    // build adl method, fix its return type and add parameters.
    ADLMethod adlfct = sbf.newADLMethod(fctname,modif);
    adlfct.setReturnType(fcttype);
    adlfct = (ADLMethod) addBhvFctParameters ( (LCallable) adlfct);
    //look for C++ method having that comaparator in local scope of C++ calss
    LCallable searched =(LCallable) stm.getMethod(
                             fctname,
			     adlfct.getOperationComparator(), 
			     null,
			     clazzCPP,
			     true );
    if (debug)
      System.out.println("debug ADLBhvDecl: recovered C++ method "+ searched );
    // Tested method should exist and be public.
    if(searched == null )
    {
      errorMessage("Error: Method "+fctname+" not found in class "+className); 
    }
    if ( !((CModifier) (searched.getModifier())).isPublic() )
    {
     errorMessage("Tested method "+fctname+" of class "+className+
		  "should be declared public");
    }
	
   
    // store ADL Function in STM current Scope = ADLClass Scope
    try {
      adlfct= (ADLMethod) stm.putSymbol(adlfct);
    } catch (SymbolAlreadyDefinedException e) {
      errorMessage("Error : ADL function already tested");
    }
    
    // save adl method in file ResultFct
    ResultFct = adlfct ;
    return adlfct;
  }
    
 /**
   * Check ADL Constructor.
   * <p> Check that a constructor having the same signature was defined in
   * related C++ class.
   * <p><b>Algorithm:</b><ul>
   * <li> Get the tested C++ class.
   * <li> get the comparator of constrcutor undertest ctor and make sure a 
   * constructor with the same comparator was defined in the tested class.
   * <li> Build an ADL method with the same constructor ctor signature using<a
   * href="#copyMethodParametersExceptions">copyMethodParametersExceptions</a>
   * <li> Save the constrcuted adl method in current  ADL class scope and as a
   * class field <a href="#ResultFct"> ResultFct </a>.
   * </ul>
   * @param ctor The parsed constructor.
   * @param clazzAdl current ADL class.
   * @return ADL method constrcuted.
   */
  public LCallable checkADLConstructor
	 ( LConstructor ctor, ADLClass clazzAdl )throws ParseException {

    if ( debug )
      System.out.println("Entering checkADLConstructor with ctor : "+ ctor );

    // set void return type for constrcutor.
    ctor.setReturnType ( CPPBuiltinType.voidT );
    // get C++ class.
    LClass tested  = clazzAdl.getLClass() ;
    // Build constructor comparator pedicate.
    UnaryPredicate cmp = ctor.getComparator() ;
    LCallable cppCtor =  stm.getConstructor(ctor.getName(), cmp, null,tested );
    if ( cppCtor == null )
      errorMessage("Error: can't find constructor in class "+tested.getName());
 
    // Since we can not save LConstrcutor in ADL class scope => build an ADL
    // method having the same signature.
    ADLMethod adlmthd = sbf.newADLMethod(ctor.getName() ,
					 ctor.getModifier().getModifierCode());
    adlmthd.setReturnType ( CPPBuiltinType.voidT );
    // copy constructor exceptions and parameters into ADL method
    adlmthd = (ADLMethod ) copyMethodParametersExceptions (ctor , adlmthd );

    // save constructor into symbol table.
    try {
      adlmthd = (ADLMethod ) stm.putSymbol ( adlmthd  , clazzAdl ) ;
    } catch ( SymbolAlreadyDefinedException e) {
      errorMessage("Error : Constrcutor already defined in ADL class ");
    }
    
    // Save recovered constrcutor into the field Resultfct. 
    ResultFct = adlmthd ;
    return adlmthd ;
  }



  /**
   * Adds paremeters to a function .
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> peek the upper stack node.
   * <li> If node is a FunctionDeclartor call its add parameters method
   * <li> Else, do the same if peeked node is a FunctionDirectDeclarator.
   * <li> do nothing otherwise.
   * </ul>
   * <p> Note: method called when top node stack is Function[Direct]Declarator.
   * @param fct method to which parameters are added.
   * @result method with parameters added.
   * 
   * @see ASTFunctionDirectDeclarator#addFctParameters
   * @see ASTFunctionDeclarator#addFctParameters
   */
  public LCallable addBhvFctParameters( LCallable fct) throws ParseException {
    Node declNode= (parser.jjtree).peekNode();
    if( declNode instanceof ASTFunctionDeclarator ) {
       fct= ( (ASTFunctionDeclarator)declNode).addFctParameters(fct);
    }
    else if (declNode instanceof ASTFunctionDirectDeclarator ){
       fct =( (ASTFunctionDirectDeclarator) declNode).addFctParameters(fct);
    }     
       
    return fct;
  }


  /**
   * Updates function return type.
   *
   * <p><b>Algorithm:</b><ul>
   * Peek upper node stack and see if it is FunctionDeclarator. Call its 
   * update function type method. Do nothing otherwise.
   * @param curType function type to be updated.
   * @result updated function type.
   * <p> Note: method called when top node stack is Function[Direct]Declarator.
   * Called when the Node Function(Direct)Declarator is on the stack top.
   */
  public LType UpdateBhvFctType( LType curType) {

    Node declNode= (parser.jjtree).peekNode();
      if( declNode instanceof ASTFunctionDeclarator) {
	return ( (ASTFunctionDeclarator)declNode).UpdateFctType( curType);
      }
      else {
     return curType;
      }
  }



 /**
   * Return function name.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li>If peeked node is FunctionDeclarator=> Call its get function name.
   * <li> Else if peeked node is a FunctionDirectDeclarator => Call its get
   * function name menthod.
   * <li> Else ( never occurs !) return null.
   * </ul>
   * <p> Called when Function[Direct]Declarator child is on the node stack top
   * @result recovered function scoped name.
   *
   * @see ScopedName
   */
  public String getADLBhvFctName( ) throws ParseException {
   ScopedName fctSN = null;
 
   Node declNode= (parser.jjtree).peekNode();
   if( declNode instanceof ASTFunctionDeclarator ) 
	 fctSN = ( (ASTFunctionDeclarator)declNode).getFctName();
   else if ( declNode instanceof ASTFunctionDirectDeclarator )
	 fctSN= ( (ASTFunctionDirectDeclarator) declNode).getFctName();
   
   
   if(fctSN.Scope != null)
     errorMessage("Error:ADL Function Name can not be Fully Scoped");

   return fctSN.Name;
  }

 /**
   * Retuns ADL method return type and updates function modifier.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> Pop up the Function[Direct]Declarator node at the stack tope.
   * <li> Try to peek next node. <ul>
   *  <li> When node exits and is a DeclarationSpecifiers => Call its method
   * getType to get adl method return type and its method getModifier to update
   * <a href="#adlBhvModifier"> adl method modifier </a> field .
   *  <li> Else, Assume adl behavior  return type is an integer.
   *  </ul>
   * <li> Push back the Function declarator poped node and return found type.
   * </ul>
   * <p> Note that method called when node stack top is a FunctionDeclarator or
   * a FunctionDirectDeclarator. DeclarationSpecifiers is the node below when
   * defined function with return type.
   * <p> Method is not relevant for constructor.
   *
   * @result function return type.
   */

 public LType getADLBhvFctReturnType( ) throws ParseException {
    Node topStackNode;
    Node declSpecNode;
    LType typefct;

    topStackNode = (parser.jjtree).popNode(); // Secure call!

    try {
      declSpecNode= (parser.jjtree).peekNode();
    } catch (EmptyStackException e) { // case when the stack is empty.
      return (LType) stm.getSymbol( "int"); // Assume default value is int
    }

    // get type and modifier if declSpec is a ASTDeclarationSpecifiers
    if (  declSpecNode instanceof ASTDeclarationSpecifiers)  {
	typefct =  ( (ASTDeclarationSpecifiers) declSpecNode).getType();
	adlBhvModifier =((ASTDeclarationSpecifiers)declSpecNode).getModifier();
    }
    else {
      // else return integer.
      typefct = CPPBuiltinType.intT ;
    }
    // push back poped node.
    (parser.jjtree).pushNode(topStackNode); // Put back Node on the stack 
    return typefct;
 }

      
  /** Get the  method/constructor/destructor under test. */
  public ADLMethod getADLMethod() {
    return (ADLMethod)ResultFct;
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }


  /** this field is the recovered LCallable method at parsing time */
  public LCallable ResultFct ;
  /* ADL method  Modifier updated while getting the function type.
   * not relevant in case of destructor */
  private int adlBhvModifier = 0x0000;

}
