// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTADLInhSemantics.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -November,97: Added Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import java.util.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.Scope;
import ORG.opengroup.adl.symboltable.c.*;
import com.objectspace.jgl.UnaryPredicate;

public class ASTADLInhSemantics extends SimpleNode {
  public ASTADLInhSemantics(int id) {
    super(id);
  }

  public ASTADLInhSemantics(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTADLInhSemantics(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTADLInhSemantics(p, id);
  }


 /**
   * Checks ADL inherited semantics declaration.
   *
   * <p><b>Algorithm:</b><ul>
   * <li> loop on super classes and see if <code> superName</code>  is the
   * name of one of them
   * <li> Make sure that <code>adlfct</code> is a method of that super class :
   * get <code>adlfct</code> comparator and search in C++ for a method with 
   * that comparator and name.
   * <li> Emit a warning inviting user to make sure that file 'superName.adl'
   * exist and contains a behavior test of <code>adlfct </code>.
   * </ul>
   * 
   * @param adlfct adl function under test.
   * @supers vector of current adl class super classes.
   * @superName super class name figuring before <code>semantics</a> keyword
   * @isConstructor boolean telling if the tested adl method is a constructor.
   */

  public void checkInhSemantics(LCallable adlfct, Vector supers,
				String superName , boolean isConstructor ) 
  throws ParseException {
    
    LClass theSuper=null;
    LClass curClass;

    // Check if parsed superName matches one of of adlfct super Classesa
    if( supers== null) 
      errorMessage("Error : Illegal use of inherited semantics");

    for(int i=0; i< supers.size(); i++) {
      curClass= (LClass) supers.elementAt(i);
      if( superName.equals(curClass.getName())) {
	theSuper = curClass;
        break;
      }
    }

    if (theSuper==null)
      errorMessage("Error: "+superName+" is not an ADL super class");

    // Now we should verify that adlfct is a method of theSuper Class.
    // => verify it exists a theSuper class method with the same signature  
    // get comparator and adlfct name.
    UnaryPredicate cmpPrd = adlfct.getOperationComparator();
    // when testing a constructor, the searched constructor has the name of the
    // super class.
    String myfctName ;
    if ( !isConstructor )
    {
      myfctName = adlfct.getName();
    } else {
      myfctName = superName ;
    }


    // get super class scope.
    Scope classScope = theSuper.getScope();
    // look for a similar method in super class scope.
    LCallable myCPPfct = stm.getMethod(myfctName, cmpPrd,null , classScope);

    if(myCPPfct== null ) {
      errorMessage("Error: method "+ myfctName+ " not found in C++ class "+
		   superName); 
    }

  }
    
	
 /** Saves inherited super class name as a node property*/ 
  public void setSuperClassName(String id)
  {
    try {
      setProperty("supername", id);
    } catch (PropertyAccessException e) {
    }
  }

  /** Returns super class name saved as a node property */
  public String getSuperClassName () {
    try {
      return (String) getProperty("supername") ;
    } catch (PropertyAccessException e) {
      return "";
    }
  }

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
