// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTADLInlineDecl.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -November,97:  Added Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import java.util.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.Scope;
import ORG.opengroup.adl.symboltable.c.*;
import com.objectspace.jgl.UnaryPredicate;


public class ASTADLInlineDecl extends SimpleNode {
  public ASTADLInlineDecl(int id) {
    super(id);
  }

  public ASTADLInlineDecl(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTADLInlineDecl(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTADLInlineDecl(p, id);
  }

  /**
   * Build an ADL inline method and add its into the symbol table.
   *
   * <p><b>Algorithm:</b><ul>
   * <li> Get the ADL inline name and type.
   * <li> Update the ADL inline type and get its modifier.
   * <li> Build an ADL inline method then add its parameters.
   * <li> Make sure tested C++ class has not an already defined method with 
   * the same name and comparator.
   * <li> Extend symbol table with the constructed adl inline method.
   * <li> open the inline method scope.
   * <li> Save the inline method using <a href="#setADLInline">setADLInline</a>
   *
   * @param classADL current adl class in which the inline method is defined
   */
   
  public void MakeUpADLInline( ADLClass classADL) throws ParseException {
    // get inline name and type
    String fctname = getADLInlineName();
    LType fcttype = getADLInlineType();
    // update the inline type.
    fcttype = UpdateADLInlineType(fcttype);
    // get modifier saved as field
    int modif = this.modifier;
    // build an inline with that name and modifier
    ADLInline myInlinefct = sbf.newADLInline( fctname,modif);
    // set the inline return type
    myInlinefct.setReturnType( fcttype);
    // add the inline parameters
    myInlinefct = (ADLInline) addADLInlineParameters ( myInlinefct); 
    // verify no similar function was defined with the same prototye
    // get first the related C++ class scope.
    Scope scopeCPP = (classADL.getLClass()).getScope();
    String className = (classADL.getLClass()).getName();
    // get a comparator for the constructed inline
    UnaryPredicate cmpPrd = myInlinefct.getComparator();
    // look for a method in C++ class with that signature and name
    LCallable fctTest = stm.getMethod(fctname,cmpPrd,null,scopeCPP);

    if( fctTest != null) {
      errorMessage("Error : ADL inline method can not have the same name and"+
		   "signature as a method of the tested C++ class "+className);
    }

    // Put the constructed inline function into the symbol table
    try {
      myInlinefct = (ADLInline) stm.putSymbol( myInlinefct);
    } catch (SymbolAlreadyDefinedException e) {
      errorMessage("Error: Duplicate inline declaration "+ fctname );
    }
    
    // NOW we Open The Inline Scope.
    stm.openScope( myInlinefct);
    // Store the contructed ADL inline as a node Property
    setADLInline( myInlinefct);
    
  }


  /** Saves the ADL inline method as a node property.   */
  private void setADLInline( ADLInline fct) {
    try {
      setProperty("inlineMethod", fct);
    } catch (PropertyAccessException e) {
    }
  }


  /* Returns ADL Inline method saved as a node property. */
  public ADLInline getADLInline() {

   try {
      return (ADLInline) getProperty("inlineMethod");
    } catch (PropertyAccessException e) {
    return null; }
  }



  /** Closes the ADL Inline method Scope */
  public void closeADLInlineScope() {

    stm.closeScope();

  }
    
   /**
   * Adds paremeters to a ADL inline method .
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> peek the upper stack node.
   * <li> If node is a FunctionDeclartor call its add parameters method
   * <li> Else, do the same if peeked node is a FunctionDirectDeclarator.
   * <li> do nothing otherwise.
   * </ul>
   * <p> Note: method called when top node stack is Function[Direct]Declarator.
   * @param fct method to which parameters are added.
   * @result method with parameters added.
   * 
   * @see ASTFunctionDirectDeclarator#addFctParameters
   * @see ASTFunctionDeclarator#addFctParameters
   */
  public LCallable addADLInlineParameters( LCallable fct)
   throws ParseException {

    Node declNode= (parser.jjtree).peekNode();
    if( declNode instanceof ASTFunctionDeclarator ) {
       fct = ( (ASTFunctionDeclarator)declNode).addFctParameters(fct);
    }
    else if (declNode instanceof ASTFunctionDirectDeclarator ){
       fct = ( (ASTFunctionDirectDeclarator) declNode).
	                           addFctParameters(fct);
    }     
    return fct;
  }


 /**
   * Updates ADL inline method type with pointer operators function declarator
   * child may have.
   *
   * <p> Called when Function[Direct]Declarator child is on the node stack top.
   * <p><b>Algorithm:</b><ul>
   * <li> peek node, check if it is FunctionDeclarator and return its 
   * UpdateType method return type..
   * <li> Else, return unmodified type.
   * </ul>
   * @param curType adl inline method type before update.
   * @result updated inline method type.
   */
  private LType UpdateADLInlineType( LType curType) {

    Node declNode= (parser.jjtree).peekNode();
      if( declNode instanceof ASTFunctionDeclarator) {
	return ( (ASTFunctionDeclarator)declNode).UpdateFctType( curType);
      }
      else {
	return curType;
      }
  }


 /**
   * Return ADL inline method  name.
   * 
   * <p> Called when Function[Direct]Declarator child is on the node stack top
   * <p><b>Algorithm:</b><ul>
   * <li>If peeked node is FunctionDeclarator=> Call its get function name.
   * <li> Else: FunctionDirectDeclarator => Call its get function name menthod.
   * <li> Once obtained the inline scoped name, make sure method defined with
   * no scope override.
   * <li> Retunr the 
   * </ul>
   * @result recovered adl inline method name.
   *
   */

  private String getADLInlineName() throws ParseException {
    ScopedName inlineScName;
    // when this method is called Inline(Direct)declarator is on the stack top
    Node topStackNode = (parser.jjtree).peekNode();
    if( topStackNode instanceof ASTFunctionDeclarator) {
      inlineScName = ( (ASTFunctionDeclarator) topStackNode).getFctName();
    }
    else { // case topStackNode is InlineDirectDeclarator
      inlineScName =( (ASTFunctionDirectDeclarator) topStackNode).getFctName();
    }

    if(inlineScName.Scope != null )
      errorMessage("Error: scope override not allowed in ADL inline method");

    return inlineScName.Name;
  }


  /** Return the type of the ADL inline method
   *
   * <p><b>Algorithm:</b><ul>
   * <li> Pop up the Function[Direct]Declarator node at the stack tope.
   * <li> Try to peek next node. <ul>
   *  <li> When node exits and is a DeclarationSpecifiers => Call its method
   * getType to get inline method return type and its method getModifier to 
   * update  the inline <a href="#modifier">  modifier </a> field 
   *  <li> Else, Assume adl inline method return type is an integer.
   *  </ul>
   * <li> Push back the Function declarator poped node and return found type.
   * </ul>
   */

  private LType getADLInlineType() throws ParseException {
    Node declSpecNode;
   LType typeinline;
   String localstr;
 
   // Need to get Type of DeclSpec node which is the node before the top.
   // Pop the Higher none. Peek the DeclSpecNode and push again the poped one
   Node  topStackNode = (parser.jjtree).popNode(); // Secure call!

    try {
      declSpecNode= (parser.jjtree).peekNode();
    }  catch (EmptyStackException e) { // Case the stack is empty.
      (parser.jjtree).pushNode(topStackNode);      
      return (LType) stm.getSymbol( "int"); // Assume default value is int
    }
    if(  declSpecNode instanceof ASTDeclarationSpecifiers)  {
	typeinline =  ( (ASTDeclarationSpecifiers) declSpecNode).getType();
	modifier = ( (ASTDeclarationSpecifiers) declSpecNode).getModifier();
    }
    else {
      typeinline = (LType) stm.getSymbol( "int");
    }

   (parser.jjtree).pushNode(topStackNode); // Put back Node on the stack 
   return typeinline;
 }

    
   
 

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

  /** inline modifier */
  private int modifier = 0x0000;
}
