// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTAbstractDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - November,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTAbstractDeclarator extends SimpleNode {
  public ASTAbstractDeclarator(int id) {
    super(id);
  }

  public ASTAbstractDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }

  /**
   * Returns type of Abstract declarator.
   * Note: this method is called when an abstract pointer to function, function
   * type or pointer to member function is parsed.
   * <br> This occurs when current abstract declarator is as in this context:->
   * "("<underline>AbstractDeclarator</underline>")"(AbstractDeclaratorSuffix)?
   * <p><b>Algorithm:</b><ul>
   * <li> If node has no children => return type is a CFunctionType.
   * <li> Else,<ul>
   *  <li> Make sure node has a PtrOperator child that must not be reference
   *  <li> Construct declarator type accordingly and call 1st child update
   *  parameter Type method.
   *  </ul>
   * </ul>
   * @param comparator abstract declarator function signature.
   * @result abstract declarator type.
   */
  public LType abstractDeclaratorAsFunction
             (LCallableEqualsComparator comparator) throws ParseException {

    LType pointedType = null;
    int childNum = jjtGetNumChildren();
    // return function type in case of no children : ( ) ( char *)
    if ( childNum==0)
      return sbf.newFunctionType( comparator, null) ;

    if ( (childNum !=2) || !( jjtGetChild(0) instanceof ASTPtrOperator) )
      errorMessage("Error:illegal abstarct parameter declaration");

    Node child0 = jjtGetChild(0);
    String kind =  ((ASTPtrOperator) child0).getPtrkind();

    if ( kind.equals("PointerTo") ) {

        pointedType = sbf.newPointerToFunction( comparator, 0x00);	
    }
    else if ( kind.equals("ReferenceTo")) {

      // Function return types may not be references
      errorMessage("Error : cannot declare references to functions");
    }
    else if ( kind.equals("PointerToMember") ) {
      // get the scope oeverride
      Scope ptrScope =  (((ASTPtrOperator)jjtGetChild(0)).getScopedName()).Scope;
      // pointer to member can apply for classes and structs
      if ( ptrScope instanceof LClass ) {

	pointedType = sbf.newPointerToFunctionMember
                                    ( comparator, (LClass) ptrScope );
      }
      else {
	errorMessage
	    ("Error : illegal pointer to function member declartion");
      }
    }

    // now call AbstractDeclarator child update ParamType method
    return ( ( ASTAbstractDeclarator) jjtGetChild(1)).updateParamType
                                                           ( pointedType);
  }



  /**
   * Updates abstract declarator type according to parsed expansion.
   *
   *Note 3 possible grammer expansion casescases of AbstractDeclarator : <ul>
   *<li>AbstractDeclarator->"("AbstractDeclarator")"(AbstractDeclaratorSuffix)*
   *<li> AbstractDeclartor -> PtrOpeartor AbstractDeclarator
   *<li> AbstractDeclarator -> ( [ (ConstantExpression) ] )+
   */
  public LType updateParamType( LType curType) throws ParseException {

    // get children number
    int nbchild =jjtGetNumChildren();

    // first case
    if ( ( nbchild > 0) &&(jjtGetChild(0) instanceof ASTAbstractDeclarator)){
      if ( nbchild == 1) 
	curType = ((ASTAbstractDeclarator) jjtGetChild(0)).
                                               updateParamType(curType); 
      else {
	Object suffix = ((ASTAbstractDeclaratorSuffix) jjtGetChild(1)).
                                        updateDeclarationType( curType );

	if ( suffix instanceof LType ) 
	  curType = (LType) suffix;
	else if ( suffix instanceof LCallableEqualsComparator )
	  curType = ((ASTAbstractDeclarator) jjtGetChild(0)).
	    abstractDeclaratorAsFunction( (LCallableEqualsComparator) suffix);
      }
    }
    // second child
    else if (( nbchild > 0) &&(jjtGetChild(0) instanceof ASTPtrOperator)) {
      String  tmpstr = ((ASTPtrOperator)jjtGetChild(0)).getPtrkind();

      if ( tmpstr.equals( "PointerTo")) {
	curType =  (LType) sbf.newPointer( curType, 0x00); 
      }
      else if( tmpstr.equals("ReferenceTo") ) {

	curType =  (LType) sbf.newReference( curType);
      }
      else if ( tmpstr.equals("PointerToMember") ) {
	
	Scope sc = (((ASTPtrOperator)jjtGetChild(0)).getScopedName()).Scope;
	curType = sbf.newPointerToFieldMember(curType ,(LConstructedType) sc); 
      }

    }
    else {
      // Third case
      int nbArrays = getNumberArrays();
      for( int i=0; i < nbArrays; i++ ) {
	curType = sbf.newArray( curType);
      }
    }

    return curType;

  }

  public void setNumberArrays( int nb) {
    numberArrays = nb;
  }


  public int getNumberArrays() {
    return numberArrays;
 
  }



  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

  //______________________ private__________________________________________

  /* int telling the number of arrays */
  private int numberArrays =0;

}
