// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTBaseSpecifier.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// **********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import java.util.Vector;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTBaseSpecifier extends SimpleNode {

  public ASTBaseSpecifier(int id) {
    super(id);
  }

  public ASTBaseSpecifier(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTBaseSpecifier(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTBaseSpecifier(p, id);
  }

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
  
    return visitor.visit(this, data);
  }

  /** Returns the name of the base class.*/
  public String getName()
  {
    try {
      return (String) getProperty("name");
    } catch (PropertyAccessException e) {
      return " ";
    }
  }


  /** Saves base class name as a node property. */
  public void setName(String id)
  {
    try {
      setProperty("name", id);
    } catch (PropertyAccessException e) {
    }
  }

 /** Looks for base class in symbol table and returns its reference to caller.
  * @result the found base class. 
  */ 
  public LClass getSuperLClass() throws ParseException {
    String storedName;

    try {
      storedName= (String)getProperty("name");
    } catch (PropertyAccessException e) {
    storedName= " "; 
    }
    // Superclass scope:If 'class A:B' => null | If 'class A:C::B'=>Scope(C::)
    Scope superScope = getSuperClassScope();
    LClass resclass =  stm.getClass(storedName,superScope);
    if (resclass==null) 
      errorMessage("Class "+ storedName+" Not found");
    
    return resclass;
  }


  /** Saces base class scope if not null */
  public void setSuperClassScope ( Scope curScope) {
    if(curScope != null) {
      try {
	setProperty("superScope", curScope);
      } catch (PropertyAccessException e) {
  
      }
    }
  }

  /** Retursn base class scope.*/
  public Scope getSuperClassScope() {
     try {
      return (Scope) getProperty("superScope");
    } catch (PropertyAccessException e) {
      return null;  // Default value is null when no Scope is stored!.
    }
  }
    

  public void setAccessSpec(int id)
  {
    Integer intObj = new Integer(id);
    try {
      setProperty("acessSpec", intObj);
    } catch (PropertyAccessException e) {
    }
  }

  /** looks for the object modifier indicating super class access specifier*/
  public String getAccessSpec()
  {
    String strResult;
    Integer intObj=null ;
    try {
      intObj = (Integer)getProperty("acessSpec");
    } catch (PropertyAccessException e) {
      return "public";
    }
   

    CModifier modif = new CModifier( intObj.intValue());
    if( modif.isProtected() )
      strResult = "protected";
    else if ( modif.isPrivate() )
      strResult = "private";
    else {
      strResult = "public";
    }
    return strResult;

  }


  public void setVirtualParent(String id)
  {
    try {
      setProperty("virtualParent", id);
    } catch (PropertyAccessException e) {
    }
  }

  /**  @return to true if the super class is virtual.*/
  public boolean isVirtualParent() {
    String str;
    boolean result;
    try {
      str= (String) getProperty("virtualParent");
      result = str.equals("true");
    } catch ( PropertyAccessException e) {
      result = false;
    }

    return result;
  }




}

