// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ************************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTClassSpecifier.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.c.*;


public class ASTClassSpecifier extends SimpleNode {

  LConstructedType storedclass;
  /** flag updated by parser to tell if the class is with member declaration 
   * used for unparsing to know if {} should be generated or not.
   */
  public boolean withMembers = false;

  public ASTClassSpecifier(int id) {
    super(id) ;
  }

  public ASTClassSpecifier(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTClassSpecifier(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTClassSpecifier(p, id);
  }

  public void setName(String id)
  {
    try {
      setProperty("name", id);
    } catch (PropertyAccessException e) {
    }
  }

  /** returns the parsed class/struct/enum name */
  public String getName() {
 
   try {
      return (String ) getProperty("name");
    } catch (PropertyAccessException e) {
      return " ";
    }
  }


  public LType getClassTypeName() {
    return (LType) storedclass;

  }


  /** called by the parser to store the class kind ( union, class, struct) */ 
 public void setClassKind(String id)
  {
    try {
      setProperty("classkind", id);
    } catch (PropertyAccessException e) {
    }
  }

  /** Return the class kind ( union, class, struct)  */ 
 public String getClassKind()
  {
    try {
      return (String ) getProperty("classkind");
    } catch (PropertyAccessException e) {
      return "class";
    }
  }



  /** Puts the recovered symbol into symbol table and store it in storedclass.
   * stored calss is checked to open a scope in case of member Declaration
   */
  public void MakeUpClass  ( String className, int nbfather) 
    throws  ParseException {

    String classkind;
    LClass clazz = null ;

    // Determine if the class kind is a struct/union or class
    try {
      classkind= (String)getProperty("classkind");
    } catch (PropertyAccessException e) {
    classkind= "class";
    }

    if(classkind.equals("class")  ) {
      clazz = sbf.newClass(className,0x0000);
    } 
    else if ( classkind.equals("struct") ) {
      clazz = sbf.newStruct(className);
    } 
    else
      return ;

      
    if ( nbfather > 0) {
      
      Node superNode= (parser.jjtree).peekNode();
      if( superNode instanceof ASTBaseSpecifier){
	LClass superLC = ( (ASTBaseSpecifier)superNode).getSuperLClass();
	clazz.addSuperClass(superLC);
      }
      else if( superNode instanceof ASTBaseClause) {
	clazz = ( (ASTBaseClause)superNode).addSuperClasses( clazz);
      }
    }
    try{
      clazz= ( LClass) stm.putSymbol(clazz);
    } catch (SymbolAlreadyDefinedException e) {
      errorMessage("Error: duplicate definition of class/struct "+ className);
    }
    storedclass = clazz;					         

  }

  /* This function is called by the parser when the constructed class has members */
  public void openClassScope( ) {
    stm.openScope( storedclass);
  }
  /* Called to close by parser to close the previously opened class scope */
  public void closeClassScope(){
    stm.closeScope();
  }
        

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
