// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTCtorDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -October,97:Modified to Introduce Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.* ;
import ORG.opengroup.adl.symboltable.c.* ;
import com.objectspace.jgl.UnaryPredicate;

public class ASTCtorDeclarator extends SimpleNode {
  public ASTCtorDeclarator(int id) {
    super(id);
  }

  public ASTCtorDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }


  /**
   * Extends symbol table with a new build constructor definition.
   * <p> Two cases : constructor name with scope override, constructor should 
   * have been already declared.
   * Otherwise declare the constructor.
   * <p><b>Algorithm:</b><ul>
   * <li> get simple constructor name and scope from parsed name(with possibly
   * scope override).
   * <li> Construct a constructor with that name and the parsed modifier.
   * <li> Constructor return type is void.
   * <li> Call <a href="#addCtorParams"> addCtorParams </a> to add parameters.
   * <li> If Constructor with no scope override :
   *  <ul>
   *  <li> Save Constructor into current scope of symbol table manager.
   *  <li> return constructed symbol.
   *  </ul>
   * <li> Else <=> constructor scope not null. :
   *  <ul>
   *  <li> Get predicate calling comparator of the recovered constructor.
   *  <li> Make sure a constructor satisfying that predicate and having the
   *  constror name is found in the specified scope.
   *  <li> return the found constructor.
   *  </ul>
   * </ul>
   * @param name the parsed constructor name. May include scope override.
   * @param mdf the constructor modifier.
   * @return the recovered constructor.
   */
  public LConstructor declareConstructor (String name , int mdf  ) 
    throws ParseException {

    String ctorName ;
    Scope ctorScope ;
    // Extract the constructor name.
    if ( name.indexOf("::") == -1 ) {
      // no scope override !
      ctorName = name ;
      ctorScope = null ;
    }
    else {
      // case of scope override.
      ctorName = name.substring (name.lastIndexOf("::") + 2, name.length() ) ;
      // research scope is the enclosing scope == LClass => save cast
      ctorScope=(Scope)stm.getSymbol(name.substring(0,name.lastIndexOf("::")));
    }
      
    LConstructor ctor = sbf.newConstructor ( ctorName ,mdf);  
    ctor.setReturnType ( (LType) stm.getSymbol("void") );
    ctor = addCtorParams ( ctor ) ;

    // handle both cases.
    if ( ctorScope == null ) {
      try {
	ctor = ( LConstructor) stm.putSymbol( ctor );
      } catch (SymbolAlreadyDefinedException e) {
        errorMessage("Error : Constructor"+ ctorName +" already defined ");
      }

      if ( debug ) {
	System.out.println("Matched direct constructor "+ ctor );
      }
      return ctor ;

    }
    else {
      // get the comparator predicate of the found constructor
      UnaryPredicate cmp  = ctor.getComparator();
      LCallable stmCtor = stm.getConstructor(ctorName,cmp,null, ctorScope);

      if ( stmCtor == null  || ! (stmCtor instanceof LConstructor) )
	  errorMessage("Error: Constructor "+ name+ " was not found "+
		       "in the specified scope");
            if ( debug ) {
	      System.out.println("Matched constructor with scope override "+
				 stmCtor);
      }
      // return the already defined constructor
      return (LConstructor )  stmCtor ;
    }
 }
 
 /**
   * Builds a constructor without saving it into the symbol table.
   * <p><b>Algorithm:</b><ul>
   * <li> get simple constructor name and scope from parsed name
   * <li> When scope declartor name check scope override allowed.
   * <li> Construct a constructor with that name and the parsed modifier.
   * <li> Constructor return type is void.
   * <li> Call <a href="#addCtorParams"> addCtorParams </a> to add parameters.
   * <li> Return the recovered constructor without putting it in symbol table.
   * </ul>
   * @param name the parsed constructor name. May include scope override.
   * @param mdf the constructor modifier.
   * @param allowScopedName tell if scope override is allowed in declartor name
   * @return the recovered constructor.
   */
  public LConstructor buildConstructor
    (String name ,int mdf, boolean allowScopedName   )
    throws ParseException {

    String ctorName = name ;
    Scope ctorScope ;
    // Extract the constructor name.
    if ( name.indexOf("::") == -1 ) {
      // no scope override !
      ctorName = name ;
      ctorScope = null ;
    }
    else if ( ! allowScopedName )
      errorMessage("Error: scopeoverride forbidden in constructor declarator");
    else {
      // case of scope override.
      ctorName = name.substring (name.lastIndexOf("::") + 2, name.length() ) ;
      // research scope is the enclosing scope == LClass => save cast
      ctorScope=(Scope)stm.getSymbol(name.substring(0,name.lastIndexOf("::")));
    }
      
    LConstructor ctor = sbf.newConstructor ( ctorName ,mdf);  
    ctor.setReturnType ( CPPBuiltinType.voidT );
    ctor = addCtorParams ( ctor ) ;

    return ctor ;
 }
 

  /**
   * Adds parameters to the constructor.
   *
   * <p><b>Algorithm:</b><ul>
   * <li>Return unmodified constructor when second child does not exist.
   * <li> Else, get parameter(s) calling  getFctParameter(s) in both cases of
   * second child ( ParameterDeclaration/List ) node.
   * <li> Add recovered parameters to constructor.
   * <li> return constructor.
   * </ul>
   * @param ctor the constructor to which parameters are added
   * @result constructor after adding parameters.
   */
  public LConstructor addCtorParams( LConstructor ctor) throws ParseException {

   LParameter param;
   LParameter paramList[];
   // If no ParameterDeclaration(List) node exist return unmodified
   if ( jjtGetNumChildren() < 2)
     return ctor ;
  
   if( jjtGetChild(1) instanceof ASTParameterDeclaration ) {
     // get constructor parameter.
     param = ( (ASTParameterDeclaration)jjtGetChild(1)).getFctParameter();
     // add recovered parameter as a constructor parameter.
     ctor.addParameter(param);
   } 
   else if (jjtGetChild(1) instanceof ASTParameterDeclarationList ) {
     // get array of parameters
     paramList= ( (ASTParameterDeclarationList)jjtGetChild(1)).
                                                     getFctParameters();
     for (int i=0;i<paramList.length;i++) {
     // Add each of the array members parameters to constructor.
      ctor.addParameter(paramList[i]);
     }  
   }
   return ctor ;
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }


}
