// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTDeclaration.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import java.util.Vector;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;


public class ASTDeclaration extends SimpleNode {

 


  public ASTDeclaration(int id) {
    super(id);
  }

  public ASTDeclaration(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTDeclaration(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTDeclaration(p, id);
  }


  /* Called by the parser to extend symbol table with Declarations
   */
  public void MakeUpDecl() throws ParseException {

    int nbchild = jjtGetNumChildren();
    LType decltype;
    int modif = getDeclModifier();

    if( debug) {
      System.out.println(" Declaration Called in line : "+ getLineNumber()+
			  "with Number of children : " + nbchild  );
      CModifier tosee = new CModifier(modif);
      System.out.println(" recovered declartion modifier :" + tosee);
    }

    decltype = getDeclType();
     
    if( nbchild >1) {
     
      if( jjtGetChild(1) instanceof ASTDeclarator) {

	( (ASTDeclarator) jjtGetChild(1)).MakeUpDeclarator(decltype, modif);
      }
      else if( jjtGetChild(1) instanceof ASTDirectDeclarator) {

	( (ASTDirectDeclarator) jjtGetChild(1)).MakeUpDirectDeclarator
	                                                   (decltype,modif);
      }
      else if( jjtGetChild(1) instanceof ASTInitDeclaratorList) {

	((ASTInitDeclaratorList) jjtGetChild(1)).MakeUpInitDecList(decltype,modif);
      }
      else if( jjtGetChild(1) instanceof ASTInitDeclarator) {

	( (ASTInitDeclarator) jjtGetChild(1)).MakeUpInitDecl(decltype,modif);
      }

    }
  }


  /* Called by ADLCPPParser.ADL_NamedParam to return an LExpression resExpr
   * When called, 2 children on the stack => DeclSpec and [Direct]Declarator
   * resExpr has the Type and Modifier from DeclSpec and Name from Declarator.
   */

  public LExpression getExpressionDecl() throws ParseException {

   LExpression resExpr=null;
   LType decltype = getDeclType();
   int modifier = getDeclModifier();
   if( jjtGetChild(1) instanceof ASTDeclarator)
     resExpr = ( (ASTDeclarator) jjtGetChild(1)).getNamedDeclarator(decltype,modifier);
   else if( jjtGetChild(1) instanceof ASTDirectDeclarator) {
     resExpr= ( (ASTDirectDeclarator) jjtGetChild(1)).getNamedDirectDeclarator(decltype, modifier);
   }

   return resExpr;

  }

   


  /** @return Declaration type from Declaration Specifier child.
   * How:
   * If Declaration Specifier node exits => child(0), return its getType
   * Else, Assume declarationtype is int. 
   */
  public LType getDeclType() throws ParseException {
   LType decltype;
   
    if( (jjtGetNumChildren() > 0) && ( jjtGetChild(0) instanceof ASTDeclarationSpecifiers) ) {

	decltype =  ( (ASTDeclarationSpecifiers) jjtGetChild(0)).getType();
    }
    else {

    decltype = (LType) stm.getSymbol( "int");
    }

    return decltype;
  }


  /* called by MakeUpDecl and return declaration specifier modifier
   * How :
   * If DeclarationSpecifeirs child exist => child(0)=> return its getModifier
   * Else, return 0.
   */
  public int getDeclModifier() {

    if( (jjtGetNumChildren() > 0) && (jjtGetChild(0) instanceof ASTDeclarationSpecifiers) )  {

	return   ( (ASTDeclarationSpecifiers) jjtGetChild(0)).getModifier();
    }
    else {

      return 0x0000;
    }
  }


/* AccessSpec stores if the declaration is public, private or proteceted within the class declaration. shouldn't we store if we are in the scope of a class ??! */
  public void setAccessSpec(String id)
  {
    try {
      setProperty("AccessSpec", id);
    } catch (PropertyAccessException e) {
    }
  }
  protected Vector getDumpKeys(int level)
  {
    Vector keys = new Vector();
    keys.addElement("AccessSpec");
    try {
      setProperty("AccessSpec", "private");
    } catch (PropertyAccessException e) {
    }

    return keys;
  }

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

  //__________________ Private features___________________________________

  /* boolean to display debugging message if necessary */
  private boolean debug = false ; 


}
