// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import java.util.Vector;
import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTDeclarator extends SimpleNode {
  public ASTDeclarator(int id) {
    super(id);
  }

  public ASTDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTDeclarator(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTDeclarator(p, id);
  }

 /**
   * Updates declarator type.
   *
   * <p><b>Algorithm:</b><ul>
   * <li> get pointer kind from 1st child PtrOperator.
   * <li> Update declarator type according to recovered pointer kind <ul>
   *  <li> If pointer => current declartor type is a pointer to its old value.
   *  <li> If reference=> current declartor type is reference to its old value
   *  <li> If pointer to member => get class whose declarator is pointing to
   *  members. Current declartor type becomes the pointer to field member of
   *  that class having as a basic type the old current declartror type.
   *  </ul>
   * <li> return updated type.
   * </ul>
   *
   * @param declaratorType the initial declartor type.
   * @param declartor type updated.
   */
  public LType updateDeclaratorType ( LType declaratorType ) {

    String tmpstr;
    // Modifying current declarator type according to pointer kind.
    if ( jjtGetChild(0) instanceof ASTPtrOperator ) {
      // get pointer kind
      tmpstr = ((ASTPtrOperator)jjtGetChild(0)).getPtrkind();
      if ( tmpstr == null ) return declaratorType ;
      // discuss pointer kind 3 possiblilities. 
      if ( tmpstr.equals ("PointerTo") ) {
	declaratorType =  (LType) sbf.newPointer(declaratorType, 0x00); 
      }
      else if( tmpstr.equals ("ReferenceTo") ) {
	if ( debug)
	  System.out.println("debug: Matched ReferenceTo");

	declaratorType =  (LType) sbf.newReference( declaratorType);
      } 
      else if ( tmpstr.equals("PointerToMember")) {
	// get the class we are pointing its members
	Scope clazz = (((ASTPtrOperator)jjtGetChild(0)).getScopedName()).Scope;
	// create a pointer to mebers of that class
	declaratorType =  (LType) sbf.newPointerToFieldMember
	                      ( declaratorType, (LConstructedType) clazz );    
      }
    }
      return declaratorType ;
    }


  /**
   * Build a parameter.
   *
   * <p><b>Algorithm:</b><ul>
   * <li> Update parameter type calling method <a href="#updateDeclaratorType">
   * updateDeclaratorType() </a>.
   * <li> Make a recursive call to the appropriate build parameter method 
   * with the obtained parameter type according to second child.
   * </ul>
   *
   * @param type the paraneter type
   * @param modif the parameter type.
   * @result the built parameter.
   */
  public LParameter BuildParamDecl( LType type, int modif) 
  throws ParseException {

    // update parameter type
    type = updateDeclaratorType(type ) ;
    // Recursive call to handle multiple pointer, tables and declarators 
    if (  jjtGetChild(1) instanceof ASTDeclarator ) { 
      return ( (ASTDeclarator)jjtGetChild(1)).BuildParamDecl(type,modif);
    }
    else   {
      // here we have certainly ASTDirectDeclarator
      return ((ASTDirectDeclarator)jjtGetChild(1)).BuildParamDecl(type,modif);
    }
  }

 /**
  * Declares a declartor.
  * <p><b>Algorithm:</b><ul>
  * <li> Update parameter type calling method <a href="#updateDeclaratorType">
  * updateDeclaratorType() </a>.
  * <li> Make a recursive call to the appropriate make up  method  with the
  *  type according to second child.
  * </ul>
  *
  * @param type the declaration type
  * @param modif the declaration modifier 
  */
 public void MakeUpDeclarator( LType type, int modif) throws ParseException
 {
  // update declartor type.
  type = updateDeclaratorType( type );
  // call  MakeUpDeclarator if the Second child is a Declarator
  if (  jjtGetChild(1) instanceof ASTDeclarator ) { 
    ( (ASTDeclarator)jjtGetChild(1)).MakeUpDeclarator( type,modif) ;
  }
  else {
    // call MakeUpDirectDeclarator if the Second child is DirectDeclarator 
    ((ASTDirectDeclarator)jjtGetChild(1)).MakeUpDirectDeclarator(type,modif) ;
  }
 }


 /**
  * Returns the declarator LExpression.
  * 
  * <p><b>Algorithm:</b><ul>
  * <li> Update parameter type calling method <a href="#updateDeclaratorType">
  * updateDeclaratorType() </a>.
  * <li> Make a recursive call to the appropriate 2nd child  method according 
  * its kind.
  * </ul>
  *
  * @param type the declaration type
  * @param modifier the declaration modifier
  * @result the expression found.
  */
  public LExpression getNamedDeclarator( LType type, int modif) 
   throws ParseException {

   // update type
   type = updateDeclaratorType( type );
   // call appropriate second child method 
   if (  jjtGetChild(1) instanceof ASTDeclarator ) { 
     return ((ASTDeclarator)jjtGetChild(1)).getNamedDeclarator(type,modif);
   }
   else   { // here we have certainly ASTDirectDeclarator
     return ( (ASTDirectDeclarator)jjtGetChild(1)).getNamedDirectDeclarator
                                                             (type, modif);
    }
  }

 /**
   * Returns the parameter when declartor is a pointer to function or a pointer
   * to function member parameter.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> get pointer kind from 1st child PtrOperator.
   * <li> Update declarator type according to recovered pointer kind <ul>
   *  <li> If pointer => parameter type is a pointer to comparator function.
   *  <li> If reference=> Error : function return type is never a reference
   *  <li> If pointer to member => get class/struct whose declarator is 
   *  pointing to members. parameter type becomes the pointer to function
   *  member of that class/struct.
   *  </ul>
   * <li> use resulting parameter type to call appropriate build parameter of
   * second child. result parameter is the result of that call.
   * </ul>
   * 
   * <p> Method used when a parameter is expected and that parameter has a
   * function type.<br>
   * Example parameter x of in "int foo ( int ( A:: x)( char *) )".
   *
   * @param comparator holds information about parameter function signature
   * @param modif parameter modifier.
   * @result found parameter.
   *
   * @see ORG.opengroup.adl.symboltable.LCallableEqualsComparator
   * @see ORG.opengroup.adl.symboltable.LPointerToFunctionMember
   * @see ORG.opengroup.adl.symboltable.LPointerToFunction
   */
  public LParameter parameterAsFunction (LCallableEqualsComparator comparator,
					 int modif )  throws ParseException {

    LParameter parameter = null ;
    LType paramType = null;
    int nbchild = jjtGetNumChildren();

    if ( nbchild != 2) 
      errorMessage("Error: illegal parameter declaration");

    if ( ! (jjtGetChild(0) instanceof ASTPtrOperator )) 
      return null;
    // get pointer kind.
    String kind =  ((ASTPtrOperator)jjtGetChild(0)).getPtrkind();
    // discuss all possible 3 cases
    if ( kind.equals("PointerTo") ) {
      paramType =  sbf.newPointerToFunction( comparator, 0x00 );
    }
    else if ( kind.equals("ReferenceTo")) {
      // Function return types may not be references
      errorMessage("Error : cannot declare references to functions; use"+
		   " pointer to function instead");
    }
    else if ( kind.equals("PointerToMember") ) {
      // get the scope oeverride
      Scope ptrScope =(((ASTPtrOperator)jjtGetChild(0)).getScopedName()).Scope;
      // pointer to member can apply for classes and structs
      if ( ptrScope instanceof LClass ) {
	// update type
	paramType = sbf.newPointerToFunctionMember( comparator,
						      (LClass) ptrScope );
      }

    }
    else {
       errorMessage
	 ("Error: illegal pointer to function member parameter declaration");
    }

    // call appropriate build param type of second child.
    Node child1 = jjtGetChild(1);
    // call second child build param method
    if ( child1 instanceof ASTDirectDeclarator) {
      parameter=((ASTDirectDeclarator)child1).BuildParamDecl(paramType,
								modif);
    }
    else if ( child1 instanceof ASTDeclarator ) {
	parameter =((ASTDeclarator)child1).BuildParamDecl(paramType,modif);
    }

    return parameter;
  }


 /**
   * Builds a declarator when it is is a pointer to function or a pointer to
   * function member declartor.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> get pointer kind from 1st child PtrOperator.
   * <li> Update declarator type according to recovered pointer kind <ul>
   *  <li> If pointer => declartor type is a pointer to comparator function.
   *  <li> If reference=> Error : function return type is never a reference
   *  <li> If pointer to member => get class/struct whose declarator is 
   *  pointing to members. declartor type becomes the pointer to function
   *  member of that class/struct.
   *  </ul>
   * <li> use resulting  type to call appropriate make up declartor method 
   * according to second child. 
   * </ul>
   * 
   * <p> Method used when a expected declartor has a function type.<br>
   * Example declartor x of in "int ( A:: x)( char *)".
   *
   * @param comparator holds information about parameter function signature
   * @param modif parameter modifier.
   * @result found parameter.
   *
   * @see ORG.opengroup.adl.symboltable.LCallableEqualsComparator
   * @see ORG.opengroup.adl.symboltable.LPointerToFunctionMember
   * @see ORG.opengroup.adl.symboltable.LPointerToFunction
   */
  public void declaratorAsFunction ( LCallableEqualsComparator comparator,
				     int modif )  throws ParseException {

    LType pointedType = null;
    int nbchild = jjtGetNumChildren();
    
    if ( nbchild != 2) 
      errorMessage("Error : illegal declaration");

    if (!( jjtGetChild(0) instanceof ASTPtrOperator ) ) return ;

    String kind =  ((ASTPtrOperator)jjtGetChild(0)).getPtrkind();
    // discuss all possible 3 cases
    if ( kind.equals("PointerTo") ) {
      pointedType = sbf.newPointerToFunction( comparator, 0x00 );	
    }
    else if ( kind.equals("ReferenceTo")) {
      // Function return types may not be references
      errorMessage("Error : cannot declare references to functions; use"+
		   " pointer to function instead");
    }
    else if ( kind.equals("PointerToMember") ) {
      // get the scope oeverride
      Scope ptrScope =(((ASTPtrOperator)jjtGetChild(0)).getScopedName()).Scope;
      // pointer to member can apply for classes and structs
      if ( ptrScope instanceof LClass ) {
	// type is a pointer to function member.
	pointedType= sbf.newPointerToFunctionMember(comparator,
						    (LClass)ptrScope);
      }
      else {
	errorMessage("Error : illegal pointer to function member declartion");
      }
    }

    Node child1 = jjtGetChild(1);
    // call second child appropriate method
    if ( child1 instanceof ASTDirectDeclarator) {
      ((ASTDirectDeclarator)child1).MakeUpDirectDeclarator(pointedType, modif);
    }
    else if ( child1 instanceof ASTDeclarator ) {
      ((ASTDeclarator)child1).MakeUpDeclarator( pointedType, modif);
    }

  }    

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
