// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTDeclaratorSuffixes.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTDeclaratorSuffixes extends SimpleNode {
  public ASTDeclaratorSuffixes(int id) {
    super(id);
  }

  public ASTDeclaratorSuffixes(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTDeclaratorSuffixes(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTDeclaratorSuffixes(p, id);
  }

  /** Saves the declarator suffix type qualifier as a node property */
  public void setTypeQualifier(int qualif)
  {
    
    try {
      setProperty("qualifier", new Integer(qualif));
    } catch (PropertyAccessException e) {
    }
  }

  /** Returns the declartor suffix type qualifier saved as a node property */
  public int getTypeQualifier() {
    try {
      return ( (Integer) getProperty("qualifier")).intValue();
    } catch (PropertyAccessException e) {
      return 0x000 ;
    }
  }
    


 /**
   * Constructs and returns a suffixed parameter.
   * <p><b>Algorithm:</b>
   * <code>ArraySize</code> field tells if suffix is an array suffix (>0) or
   * a function signature suffix. <ul>
   * <li> In case of multidimensional array size=> update type to be an array 
   * whose size is <code>ArraySize</code>. Construct a parameter with that type
   * <li> Else => case of function signature suffix. <ul>
   *  <li> Get the parameter type qualifier.
   *  <li> Create comparator whose return type is <code>curType</code>. 
   *  <li> Add parameters to that comparator according to node parameter child.
   *  <li> If exception spec child exist create atemporarely function to which
   *  exceptions are added. Add function exceptions to comparator.
   *  <li> Construct a CFunctionType from that comparator.
   *  <li> Constrcut parameter with that type, <code>modifier</code> and the
   *  obtained CFunctionType..
   *  </ul>
   * <li> Return constructed parameter.
   * </ul>
   * @param curType parameter type.
   * @param modifier parameter modifier.
   * @param paramName paraneter name.
   * @result suffixed parameter.
   */
  public LParameter getSuffixedParam( LType curType,String paramName,
				      int modifier) {
   
    int nbchild = jjtGetNumChildren();
    LParameter param = null;
    if( ArraySize > 0)  {
      // update type to be an LArray
      for( int i=0;i< ArraySize; i++ ) {
	curType = sbf.newArray(curType) ;
      }
      param = sbf.newParameter(paramName,modifier,curType);
    }
    else { 
      int nextChild = 0;
      // Constrcut a Comparator
      LCallableEqualsComparator comparator = new LCallableEqualsComparator() ;
      // get parameter type qualifier
      int paramTypeMdf = getTypeQualifier() ;
      // add parameters from 1st child if ParameterDeclaration.
      if ( (nbchild>0) && jjtGetChild(0) instanceof ASTParameterDeclaration ) {
	try {
	  comparator.addParameterType((((ASTParameterDeclaration)
			       jjtGetChild(0)).getFctParameter()).getType() );
	} catch ( ParseException e) { }
	nextChild = 1;
      }
      else if ((nbchild>0) &&
	       jjtGetChild(0) instanceof ASTParameterDeclarationList ) {
	try {
	  LParameter paramList[]=((ASTParameterDeclarationList)jjtGetChild(0)).
                                                          getFctParameters();
	  // add found parameters to function
	  for (int i=0;i<paramList.length;i++) {
	    // Add each of parameters to fucntion.
	    comparator.addParameterType(paramList[i].getType());
	  }
	} catch ( ParseException ee) {}
	  nextChild =1 ;
      }

      // now add exceptions if any
      if ( (nbchild > 0 ) && ( nextChild == nbchild -1 ) ) {
	// create a function used to get node exceptions
	if ( jjtGetChild( nextChild ) instanceof ASTExceptionSpec ) {
	  LFunction used = sbf.newFunction(paramName ,paramTypeMdf);
	  try {
	  // add exceptions to fucntion if current child is an ExceptionSpec.
	    used =(LFunction) ((ASTExceptionSpec )jjtGetChild( nextChild)).
	                                                addExceptions(used);
	    LType exceptions[] = used.getExceptions();
	    // add exception to comparator.
	    for (int i=0;i<exceptions.length;i++) {
	      // Add each of parameters to function.
	      comparator.addException(exceptions[i]);
	    }
	  } catch ( ParseException pe) { }
	  //garbage collect used.
	  used= null ;
	
      }

      // construct a CFunctionType with that comparator
      CFunctionType type = sbf.newFunctionType( comparator, null) ;
      // build parameter with that type
      param = sbf.newParameter(paramName,modifier,type);
    }

    }
    return param;
  }

 /**
   * Updates declaration type.
   * <p><b>Algorithm:</b></ul>
   * <li> Returns an LType in case of array suffix.
   * <li> Returns a comparator containing suffix signature when function suffix
   * </ul>
   * @param declaratorType parameter type.
   * @result can either be an LType or a comparator.
   */
  public Object updateDeclarationType(LType  declaratorType) throws ParseException {
    int childNum = jjtGetNumChildren();
    if(  ArraySize > 0 ) {
      for (int i=0; i< ArraySize; i++ ) {

        declaratorType =  sbf.newArray(declaratorType) ;
      }
     
    }
    else if ( childNum ==0 ) {
      return declaratorType ;
    }
    else  if ( jjtGetChild(0) instanceof ASTParameterDeclaration ) {
      LCallableEqualsComparator ec = new LCallableEqualsComparator();
      ec.setReturnType ( declaratorType );
      LParameter param = ( ( ASTParameterDeclaration) jjtGetChild(0)).getFctParameter( );
      ec.addParameterType( param.getType() );

      return ec;
    }
else if ( jjtGetChild(0) instanceof ASTParameterDeclarationList ) {
      LCallableEqualsComparator ec = new LCallableEqualsComparator();
      ec.setReturnType ( declaratorType );
      LParameter parameters[] = ( ( ASTParameterDeclarationList) jjtGetChild(0)).getFctParameters( );
      for ( int i =0; i < parameters.length; i ++ ) {
         ec.addParameterType( parameters[i].getType() );
      }
      return ec;
    }
    
    return declaratorType;
  }



 /**
   * Declares a suffixed variable.
   * <p><b>Algorithm:</b>
   * <code>ArraySize</code> field tells if suffix is an array suffix (>0) or
   * a function signature suffix. <ul>
   * <li> In case of multidimensional array size=> update type to be an array 
   * whose size is <code>ArraySize</code>. Construct a variable with that type
   * <li> Else => case of function signature suffix. <ul>
   *  <li> Get the suffix type qualifier.
   *  <li> Create comparator having <code>curType</code> as return type..
   *  <li> Add parameters to that comparator according to node parameter child.
   *  <li> If exception exits, constrcut a function to which exception is added
   *  , then add function execptions to comparator.
   *  <li> Construct a CFunctionType from that comparator.
   *  <li> Constrcut a variable using that type, <code>modif</code> and the 
   *  found CFunctionType..
   *  </ul>
   * </ul>
   * @param curType the variable type.
   * @param modif parameter modifier.
   * @param declName paraneter name.
   */

  public void MakeUpDeclSuffix (LType curType, String declName, int modif)
  throws ParseException {
 
    int nbchild = jjtGetNumChildren();
    if( ArraySize > 0 ) {
      for (int i=0; i< ArraySize; i++ ) {

	curType = sbf.newArray(curType) ;
      }
      LVariable myVar= sbf.newVariable(declName,modif, curType);
      try {
	myVar= (LVariable) stm.putSymbol(myVar);
      } catch (SymbolAlreadyDefinedException e) {
	errorMessage("Error : Illegal redefinition of variable");
      }	
    }
    else {
      int nextChild = 0;
      // get declarator type qualifier
      int paramTypeMdf = getTypeQualifier() ;
      LCallableEqualsComparator comparator = new  LCallableEqualsComparator();
      // set comapartor return type.
      comparator.setReturnType( curType) ;
      // add parameters from 1st child if ParameterDeclaration.
      if ( (nbchild>0) && jjtGetChild(0) instanceof ASTParameterDeclaration ) {
	
	comparator.addParameterType((((ASTParameterDeclaration)
				jjtGetChild(0)).getFctParameter()).getType());
	nextChild = 1;
      }
      else if ((nbchild>0) &&
	       jjtGetChild(0) instanceof ASTParameterDeclarationList ) {
	try {
	  LParameter paramList[]=((ASTParameterDeclarationList)jjtGetChild(0)).
                                                          getFctParameters(); 
	  for (int i=0;i<paramList.length;i++) {
	    // Add each of parameters to fucntion.
	    comparator.addParameterType(paramList[i].getType());
	  }
	} catch ( ParseException e) {}
	  nextChild =1 ;
      }

      // now add exceptions if any
      if ( (nbchild > 0 ) && ( nextChild == nbchild -1 ) ) {
	// construct a function
	LFunction used = sbf.newFunction( declName,paramTypeMdf  );
	// add exceptions if current child is an ExceptionSpec.
	if ( jjtGetChild( nextChild ) instanceof ASTExceptionSpec ) {
	  used = (LFunction) ((ASTExceptionSpec )jjtGetChild( nextChild)).
	                                            addExceptions (used) ;
	  // add function exceptions to comparator
	  LType exceptions[] = used.getExceptions();
	  // add exception to comparator.
	  for (int i=0;i<exceptions.length;i++) {
	    // Add each of parameters to function.
	    comparator.addException(exceptions[i]);
	  }
	  //garbage collect used.
	  used= null ;
	}
      }
     
      // construct a CFunctionType with that comparator
      CFunctionType type = sbf.newFunctionType( comparator, null) ;
      // Now craete a variable with recovered type.
      LVariable var= sbf.newVariable(declName,modif, type);
      CModifier Mdf = new CModifier(modif ) ;
      // only non extern variables are saved in symbol table
      if ( !Mdf.isExtern() ) {
	try {
	  var= (LVariable) stm.putSymbol(var);
	} catch (SymbolAlreadyDefinedException e) {
	  errorMessage("Error: duplicate declaration of variable "+declName);
	}
      }
    }
  }



  public void setArraySize(int  nb)
  {
    ArraySize = nb;

  }



  /** returns the array size. If null then the node is a function definition*/
  public int getArraySize() {
    return ArraySize;
  }






  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
  //____________________ Private fields______________________________________

  /* An integer telling the array size .e.g for int a[][], ArraySize = 2
   *  updated by the parser if DeclaratorSuffixes -> ( [ (ConstantExpr) ?] )*
   *. If ArraySize = 0  Declarator suffix is a function
   */
  private int ArraySize =0;

  /** debug flag */
  private boolean debug = false; 


}
