// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTDirectDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Updated
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTDirectDeclarator extends SimpleNode {
  private String localName="__NULL";
  public ASTDirectDeclarator(int id) {
    super(id);
  }

  public ASTDirectDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }


  public static Node jjtCreate(int id) {
      return new ASTDirectDeclarator(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTDirectDeclarator(p, id);
  }

  /**
   * Build a parameter.
   * <p><b>Algorithm:</b><ul>
   * <li> if parsed  DirectDeclarator->QualifiedId["("ParameterList")"]
   * <=> Case 1st child is QualifiedId : <ul>
   *  <li> If node has only one child => get its name and construct a variable
   *  with that name , type and modifier. returned parameter is that variable.
   *  <li> Else: node has 2 children and second child is DeclaratorSuffixes =>
   *  call child suffixed parameter method to get parameter.
   *  </ul>
   * <li> case first child is a Declarator <ul>
   *  <li> If only one child => call Build parameter of Declarator child.
   *  <li> Else => 2nd child is DeclaratorSuffixes : Call DeclaratorSuffixes
   * updateDeclarationType method : <ul>
   *   <li> If returned object is a Comparator ( case of function signature
   *   suffix => call 1st child Declarator parameter as function method with
   *   that comparator to get parameter.
   *   <li> Else( returned suffix object is LType)=>Call 1st child build param
   *   </ul>
   *  </ul>
   * </ul>   
   * @param curType cuurent parameter type.
   * @param modif current modifier type.
   * @result the recovered parameter. 
   *
   * @see ASTDeclaratorSuffixes#updateDeclarationType
   * @see ASTDeclarator#declaratorAsParameter         
   * @see ORG.opengroup.adl.symboltable.LCallableEqualsComparator
   */
   
  public LParameter BuildParamDecl( LType curType, int modif)
  throws ParseException {
    String IDname;
    int nbchild = jjtGetNumChildren();
    LParameter param=null;

    //1st case  : this ->QualifiedId(DeclaratorSuffixes)? Grammar expansion
    if ( jjtGetChild(0) instanceof ASTQualifiedId ) {

      IDname = ( (ASTQualifiedId)jjtGetChild(0)).getName(); // Store the name.
      if (nbchild == 1) {  
	// Means the parameter is a variable 
	param = sbf.newParameter(IDname,modif,curType);
	return param;
      }
      else if ( jjtGetChild(1) instanceof ASTDeclaratorSuffixes ) {
	// call child method.
	param= ( (ASTDeclaratorSuffixes)jjtGetChild(1)).
                                     getSuffixedParam(curType,IDname, modif);
      }	
    }
    else if ( jjtGetChild(0) instanceof ASTDeclarator ) {
      // if number children = 0 => parameter is a variable
      if ( nbchild == 0) {
	param = ( ( ASTDeclarator) jjtGetChild(0)).BuildParamDecl
                                                           ( curType, modif);
      }
      else if ( jjtGetChild(1) instanceof ASTDeclaratorSuffixes ) {
	Object suffix =( (ASTDeclaratorSuffixes )jjtGetChild(1)).
                                            updateDeclarationType(curType);   
	
	// 2 cases : suffix is an CFunctionType or not (LArray)
	if( suffix instanceof LCallableEqualsComparator ) {
	  // case DeclaratorSuffixes is a function signature suffix. 
	  param = (( ASTDeclarator )jjtGetChild(0)).parameterAsFunction 
                             ( (LCallableEqualsComparator) suffix, modif );
	}
	else {
	  // case suffix is multiple array suffix.
	  param = ((ASTDeclarator ) jjtGetChild(0)).BuildParamDecl
                                                      ((LType)suffix,modif);
	} 

      }
    }      
    return param ;
  }


  /**
   * Declares a DirectDeclarator.
   * <p><b>Algorithm:</b><ul>
   * <li> In case : DirectDeclarator -> QualifiedId (DeclaratorSuffixes)?<ul> 
   *  <li> If node has only QualifiedId child => Constrcut and declare a 
   *  variable or an type alias if modifier is typedef. 
   *  <li> Else> call DeclaratorSuffix child make up method.
   *  </ul>
   * <li>Case DirectDeclarator->"("DeclaratorDeclarator")(DeclaratorSuffixes)?
   *  <ul>
   *  <li> If number children =1 => call Declarator child make up method.
   *  <li> Else : Update type with DeclaratorSuffix 2nd child method <ul>
   *   <li> If recovered object is an comparator=> call 1st child Declarator
   *   declaratorAsFunction method.
   *   <li> Else => suffix is LType => decalare  a variable with that type 
   *   </ul>
   *  </ul>
   * </ul>
   * @param curType cuurent parameter type.
   * @param modif current modifier type.   
   *
   * @see ASTDeclaratorSuffixes#updateDeclarationType
   * @see ASTDeclarator#declaratorAsFunction
   * @see ORG.opengroup.adl.symboltable.LCallableEqualsComparator  
   */
  public void MakeUpDirectDeclarator  ( LType curType,int modif)
 throws ParseException {

    String IDname;
    int nbchild = jjtGetNumChildren();
    CModifier modifObj = new CModifier(modif);
    Node child = jjtGetChild(0);

    // first case DirectDeclarator -> QualifiedId (DeclaratorSuffixes)? 
    if ( child  instanceof ASTQualifiedId ) {
      IDname = ( (ASTQualifiedId)jjtGetChild(0)).getName(); 

      // Handle case the parameter is a Variable 
      if (nbchild == 1) {  

	// test if modifier is typedef. If yes IDname is an alias of curType
	if( modifObj.isTypedef() && !IDname.equals( curType.getName()) ) {
	  // don't make an alis if both names are equal.
	  // this occurs in C like struct definition
	  if ( !IDname.equals( curType.getName() ) )
	  {
	    try {
	      stm.putSymbol(sbf.newTypedef(IDname,curType));
	    } catch ( SymbolAlreadyDefinedException e) {
	      errorMessage("Error: can not define typedef "+IDname );
	    }
	  }
	}
	// extern variable are not saved in symbol table
	else if ( !modifObj.isExtern() ) {
	   LVariable myVar = sbf.newVariable(IDname,modif,curType); 
	  try {
	    // extend symbol table with the new variable.
	    myVar= (LVariable) stm.putSymbol(myVar);
	  } catch (SymbolAlreadyDefinedException e) {
	    errorMessage("Error: duplicate declaration of variable"+IDname);
	  }
	}
      }
      
      else if(jjtGetChild(1) instanceof ASTDeclaratorSuffixes ) {

	( (ASTDeclaratorSuffixes)jjtGetChild(1)).MakeUpDeclSuffix
                                                    (curType,IDname,modif);
      }
    }
    else if (  child instanceof ASTDeclarator ) {

      // case of "(" DeclaratorDeclarator  ")"  ( DeclaratorSuffixes )?
      if ( nbchild == 1 ) {

	( (ASTDeclarator) child).MakeUpDeclarator( curType, modif);
      }
      else {

	Object suffix =( (ASTDeclaratorSuffixes ) jjtGetChild(1)).
                                               updateDeclarationType(curType);   
	
	// 2 cases : suffix is an LType or LCallableEqualsComparator
	if( suffix instanceof LCallableEqualsComparator ) {

	( ( ASTDeclarator )child).declaratorAsFunction 
                                ( (LCallableEqualsComparator) suffix, modif );
	}
	else {
	  // case suffix is an LType
	  ( ( ASTDeclarator ) child ).MakeUpDeclarator((LType)suffix,modif);
	} 
      }
    }
    else if ( child instanceof ASTDirectDeclarator ) {
      errorMessage("Error : Illegal declaration ");
    }
  }



  /**
   * Returns the DirectDeclarator LExpression according to children
   * <p><b>Algorithm:</b><ul>
   * Very similar to <a href="#BuildParamDecl">BuildParamDecl</a> algorithm.
   * </ul>
   * @param curType ADL named parameter declaration specifier type.
   * @param modif ADL named parameter declaration specifier modifier.
   * @result ADL named parameter LExpression.
   */ 

  public LExpression getNamedDirectDeclarator  ( LType curType, int modif)
  throws ParseException {

    int nbchild = jjtGetNumChildren();
    LExpression adlparam = null;

    Node child0 = jjtGetChild(0);
    // first case DirectDeclarator -> QualifiedId (DeclaratorSuffixes)? 
    if ( child0 instanceof ASTQualifiedId ) {
      // Store the name.
      String IDname = ( (ASTQualifiedId)child0).getName();
 
      if (nbchild > 1) { 
	// update declarator type
	adlparam = ( (ASTDeclaratorSuffixes)jjtGetChild(1)).getSuffixedParam
	                                           ( curType , IDname, modif);
      } 
      else {
	// create a variable in case of unique child.
	adlparam  = sbf.newVariable(IDname,modif,curType);
      }
      
    }
    else if ( child0 instanceof ASTDeclarator ) {

      if ( nbchild ==0 )
	adlparam  =((ASTDeclarator)jjtGetChild(0)).getNamedDeclarator(curType,
								      modif);
      if ( nbchild > 1) {
	// update declarator type
	Object suffix = ((ASTDeclaratorSuffixes)jjtGetChild(1))
	                                 .updateDeclarationType(curType);
	if ( suffix instanceof LCallableEqualsComparator ) {
	  // case DeclaratorSuffixes is a function signature suffix. 
	  adlparam = (( ASTDeclarator )jjtGetChild(0)).parameterAsFunction 
                             ( (LCallableEqualsComparator) suffix, modif );
	}
	else {
	  // case suffix is multiple array suffix.
	  adlparam = ((ASTDeclarator ) jjtGetChild(0)).BuildParamDecl
                                                      ((LType)suffix,modif);
	} 
      }
    }
    else {
      errorMessage("Error: illegal declaration of ADL named parameter");
    }

    return adlparam; 
  }





  /** set the destructor name as a node property */
  public void setDestructorName ( String name) {
       try {
      setProperty("Destname", name);
    } catch (PropertyAccessException e) {
    }
  }


  /** called by the unparser to return the destructor name */
  public String getDestructorName(){
    try {
      return (String)getProperty("Destname");
    } catch (PropertyAccessException e) {
    return " "; }
  }

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

  /** debug flag */
  private boolean debug = true;

}
