// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTDtorDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - November,  1997: Added parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;

public class ASTDtorDeclarator extends SimpleNode {
  public ASTDtorDeclarator(int id) {
    super(id);
  }

  public ASTDtorDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }

  /** called by the parser to declare new destructor
   * @param className the enclosing className. null if none
   * @param nameDtor the parsed destructor name.
   * @param @snDtor the destructor scope override may be null if none
   * @param modifier : the destructor modifier
   */
  public CDestructor declareDestructor ( String className, String nameDtor, ScopedName snDtor, int modifier ) throws ParseException {

    // first case : destructor definition in the scope of a class    
    if ( className != null) {

      // make sure no scope override and that class name destructor name match
      if( snDtor != null || (! className.equals( nameDtor)) ) 
  errorMessage ("Error : Illegal destructor definition in class "+ className);

      // create a destructor 
      CDestructor myDtor = sbf.newDestructor( "~"+nameDtor, modifier);
      try {
	myDtor = (CDestructor)stm.putSymbol(myDtor);
      } catch (SymbolAlreadyDefinedException e) {
  errorMessage("Error : destructor already defined");
      }
      return myDtor;
    }
    else {

      if( (snDtor == null ) || (!snDtor.Name.endsWith(nameDtor+"::") ) )
  errorMessage ("Error : Illegal destructor definition");
      
      CDestructor myDtor = (CDestructor) stm.getMethod("~"+nameDtor,null,null, snDtor.Scope);

      if( myDtor == null)
  errorMessage ("Error : Illegal destructor definition");
      // open the scope of the Destructor 
      Scope formerScope = stm.getCurrentScope();
      stm.setCurrentScope( snDtor.Scope);
      stm.openScope( myDtor );
      // now restor former scope
      stm.setCurrentScope( formerScope );
      
      return myDtor;
      
    }
      

  }

  /** set the destructor name as a node property */
  public void setDestructorName ( String name) {
       try {
      setProperty("Destname", name);
    } catch (PropertyAccessException e) {
    }
  }


  /** called by the unparser to return the destructor name */
  public String getDestructorName(){
    try {
      return (String)getProperty("Destname");
    } catch (PropertyAccessException e) {
    return " "; }
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
