// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTExceptionSpec.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - Jan, 1998: Added Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.* ;

public class ASTExceptionSpec extends SimpleNode {
  public ASTExceptionSpec(int id) {
    super(id);
  }

  public ASTExceptionSpec(ADLCPPParser p, int id) {
    super(p, id);
  }

  /**
   * Adds parameters to function/constructor..
   *
   * <p><b> Algorithm:</b><ul>
   * <li> No exceptions added if node without children.
   * <li> If 1st child is a ParameterDeclaration get the parameter, add its
   * type as an exception.
   * <li> Else => 1st child is ParameterDeclarationList, get its array of
   * parameters and add their types as exceptions.
   * <li> return the modified ORG.opengroup.adl.symboltable.LCallable
   * </ul>
   * @param fct the function to which exceptions should be added.
   * @result function with added exceptions
   *
   * @see ORG.opengroup.adl.symboltable.LCallable
   */
  public LCallable addExceptions ( LCallable fct ) throws ParseException {

    int nbchild = jjtGetNumChildren() ;
    // No exception to add if no children
    if ( nbchild == 0)
      return fct ;

    Node child = jjtGetChild(0) ;
    if ( child instanceof ASTParameterDeclaration ) {

      LParameter prm =( ( ASTParameterDeclaration ) child ).getFctParameter();
      // parameter type is the exception to add
      fct.addException( prm.getType() ) ;
    }
    else if ( child instanceof ASTParameterDeclarationList ) {

      LParameter p[]=((ASTParameterDeclarationList )child).getFctParameters();
      // loop on parameters and add their types as exceptions.
      for (int i=0;i  < p.length; i++)  {
       fct.addException( p[i].getType() );
     }
    }

    return fct ;
  }
      

  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data)
    throws ParseException {
    return visitor.visit(this, data);
  }
}
