// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTFunctionDefinition.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -October,1997: Added Parsing methods
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import java.util.*;
import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import com.objectspace.jgl.UnaryPredicate;

public class ASTFunctionDefinition extends SimpleNode {
 
  /** parsed function return type */
 private LType FctType;
  /** recovered function */
 public LCallable storedFunction;
  
  /** Basic constructor used with static parser */
  public ASTFunctionDefinition(int id) {
    super(id);
  }

  /** basic constructor used when parser is not static */ 
  public ASTFunctionDefinition(ADLCPPParser p, int id) {
    super(p, id);
  }


  public static Node jjtCreate(int id) {
      return new ASTFunctionDefinition(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTFunctionDefinition(p, id);
  }




 /**
   * Retuns function return type and updates function modifier.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> Pop up the Function[Direct]Declarator node at the stack tope.
   * <li> Try to peek next node. <ul>
   *  <li> When node exits and is a DeclarationSpecifiers => Call its method
   * getType to get function return type and its method getModifier to update
   * <a href="#functionModifier"> function modifier </a> field .
   *  <li> Else, Assume function return type is an integer.
   *  </ul>
   * <li> Push back the Function declarator poped node and return found type.
   * </ul>
   * <p> Note that method called when node stack top is a FunctionDeclarator or
   * a FunctionDirectDeclarator. DeclarationSpecifiers is the node below when
   * defined function with return type.
   *
   * @result function return type.
   */

 public LType getFctReturnType( ) throws ParseException {
   Node topStackNode;
   Node declSpecNode;
   LType typefct;
 

   // Need to get the type of DeclSpec node which is the node before the top.
   // Pop the Higher none. Peek the DeclSpecNode and push again the poped one
    topStackNode = (parser.jjtree).popNode(); // Secure call!

    try {
      declSpecNode= (parser.jjtree).peekNode();
    } catch (EmptyStackException e) { // Case the stack is empty.
      (parser.jjtree).pushNode(topStackNode);      
      return CPPBuiltinType.intT ; // Assume default value is int
    }

    if(  declSpecNode instanceof ASTDeclarationSpecifiers)  {
      // get function type
      typefct = ( (ASTDeclarationSpecifiers) declSpecNode).getType();
      // get function modifier
      functionModifier=((ASTDeclarationSpecifiers )declSpecNode).getModifier();
    }
    else {
      typefct = CPPBuiltinType.intT ;
    }
    // push back poped node.
    (parser.jjtree).pushNode(topStackNode); // Put back Node on the stack
 
    return typefct;
 }

 /**
   * Updates function type with pointer operators of function declarator child.
   *
   * <p> Called when Function[Direct]Declarator child is on the node stack top.
   * <p><b>Algorithm:</b><ul>
   * <li> peek node, check if it is FunctionDeclarator and return its 
   * UpdateType method return type..
   * <li> Else, return unmodified type.
   * </ul>
   * @param inType function type before update.
   * @result updated function type.
   */
  public LType UpdateFctType( LType inType ) {

    Node declNode= (parser.jjtree).peekNode();
      if( declNode instanceof ASTFunctionDeclarator) {
	return ( (ASTFunctionDeclarator)declNode).UpdateFctType( inType);
      }
      else {
	return inType;
      }
  }


 /**
   * Return function name.
   * 
   * <p> Called when Function[Direct]Declarator child is on the node stack top
   * <p><b>Algorithm:</b><ul>
   * <li>If peeked node is FunctionDeclarator=> Call its get function name.
   * <li> Else if peeked node is a FunctionDirectDeclarator => Call its get
   * function name menthod.
   * <li> Else ( never occurs !) return null.
   * </ul>
   * @result recovered function scoped name.
   *
   * @see ScopedName
   */

  public ScopedName getFctName () throws ParseException {

   Node declNode= (parser.jjtree).peekNode();
   ScopedName result = null;

   if( declNode instanceof ASTFunctionDeclarator ) 
	 result = ( (ASTFunctionDeclarator)declNode).getFctName();
   else if ( declNode instanceof ASTFunctionDirectDeclarator )
	 result = ( (ASTFunctionDirectDeclarator) declNode).getFctName();
   
   else { // should never occur
     errorMessage("Error : Function Name not found");
   }
   return result ;
  }


 /**
   * Build and return the parsed function.
   *
   * <p> called when parsed function is not a class member declaration.
   * <p><b>Algorithm:</b><ul>
   * <li> get function scoped name (name and scope)
   * <li> Get function type and update it. 
   * <li> Constrcut a function with the recovered name and modifier.
   * <li> Set function return type and add function parameters.
   * <ul>
   *  <li> If function has no scope overriode => Save function in the symbol 
   *  table and make sure it was not already defined.
   *  <li> In case of scope override function should already have been declared
   *   <ul>
   *   <li>Get Function comparator.
   *   <li>Search function with that name and comparator in the override scope
   *   <li>Make sure recovered function was only declared and never defined.
   *   field.
   *   </ul>
   *  </ul>
   * <li> Save function in field <a href="#storedFunction"> storedFunction</a>
   * <li> return build method.
   * 
   * @result build parsed function.
   */

  public LCallable MakeUpFct( ) throws ParseException {
 
    // get function scoped name.
    ScopedName fctscopedname = getFctName();
    String fctname = fctscopedname.Name;
    // get function scope. Null if no scope override 
    Scope fctscope= fctscopedname.Scope;
    // get function type.
    LType fcttype = getFctReturnType();
    // update function type
    fcttype = UpdateFctType( fcttype); 
    // contsruct the function
    LFunction fct = sbf.newFunction(fctname,functionModifier);
    // Set function return type
    fct.setReturnType(fcttype);
    // Add parameters to the built LFunction
    fct = (LFunction) addFctParameters ( (LCallable)  fct); 

    if( debug) {
      System.out.println("MakeUpFct function "+fctname+" with added parameters"
			 + " is "+ ((fct!=null)?fct.getName():"null"));
    }

    // Test if the sequence is : int foo()  First case=> Normal behavior
    // Or int A::foo(): 2nd case=>get reference foo already declared in A class
    if( fctscope == null) { // First case!
      try {
	fct= (LFunction)stm.putSymbol(fct);
      } catch (SymbolAlreadyDefinedException e) {
	CModifier mdfobj = new CModifier(functionModifier);
	if (! mdfobj.isExtern() )
	  errorMessage("Error: duplicate definition of a Function");
      }
      storedFunction= fct;   
    }
    else { 
      // Scope override exist =>get reference of the already declared function
      // in the appropriate scope.
      // Get comparator of recovered function
      UnaryPredicate cmpPrd = fct.getOperationComparator();
      // check if a function with that comparator was defined
      LMethod mthd =(LMethod) stm.getMethod(fctname,cmpPrd,null,fctscope);

      if(mthd == null ) {
	errorMessage("Error: Method "+ fctname + " was not found "+
		     "with the same signature in the specified scope"); 
      }
      if( mthd.isDefined())
	errorMessage("Error : attempt to redefine : "+ fctname );
	storedFunction= mthd ;	
    } 
    LCallable result = storedFunction;
    return result;
 }

 /**
   * Build and return the parsed class member.
   *
   * <p> called when parsed function is a class method.
   * <p><b>Algorithm:</b><ul>
   * <li> get function scoped name (name and scope)
   * <li> Get function type and update it. 
   * <li> Update function member with current class access specifier.
   * <li> Constrcut a function with the recovered name and modifier.
   * <li> Set function return type and add function parameters.
   * <li> Make sure method has no scope override.
   * <li> Save function in the symbol table and make sure it was not  defined.
   * <li> Save method in <a href="#storedFunction"> storedFunction</a> field
   * <li> return build method.
   * 
   * @param className current class name.
   * @result build parsed function.
   */

  public LCallable MakeUpFctMember( String className ) throws ParseException {
    
    Scope storeScope;

    ScopedName fctscopedname = getFctName();
    Scope fctscope = fctscopedname.Scope;
 
    // No Scope Override allowed within a Class Scope 
    if(fctscope != null)
      errorMessage("Error: scope override not allowed in scope of a class"); 
    //get bfunction name
    String fctname = fctscopedname.Name;
    // get function return type.
    LType  fcttype= (LType) getFctReturnType();
    // update function return type.
    fcttype = UpdateFctType( fcttype);
    // update modifier with parser current access specifier.
    functionModifier |= parser.currentAccessSpecifier;
    // construct function with that name and modifier
    LMethod fct = sbf.newMethod(fctname,functionModifier );
    //set function return type
    fct.setReturnType(fcttype);
    // add parameters to function
    fct = (LMethod) addFctParameters ( (LCallable) fct );   
    // try to save function into symbol table
    try {
      fct= (LMethod) stm.putSymbol(fct);
    } catch (SymbolAlreadyDefinedException e) {
      errorMessage("Error: function already defined in class"+ className);
    }
    // save found method as a private field
    storedFunction = fct ;   
    // return recovered method
    LCallable result = storedFunction;
    return result;
  }


  /**
   * Adds paremeters to a function .
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> peek the upper stack node.
   * <li> If node is a FunctionDeclartor call its add parameters method
   * <li> Else, do the same if peeked node is a FunctionDirectDeclarator.
   * <li> do nothing otherwise.
   * </ul>
   * <p> Note: method called when top node stack is Function[Direct]Declarator.
   * @param fct method to which parameters are added.
   * @result method with parameters added.
   * 
   * @see ASTFunctionDirectDeclarator#addFctParameters
   * @see ASTFunctionDeclarator#addFctParameters
   */

  public LCallable addFctParameters( LCallable fct) throws ParseException {

    // get stack top node
    Node declNode= (parser.jjtree).peekNode();
    // call appropriate method according to child.
    if( declNode instanceof ASTFunctionDeclarator ) {
       fct = ( (ASTFunctionDeclarator)declNode).addFctParameters(fct);
    }
    else if (declNode instanceof ASTFunctionDirectDeclarator ){
       fct = ( (ASTFunctionDirectDeclarator) declNode).addFctParameters (fct);
    }     
       
    return fct;
  }



  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

  //_____________________ private features_________________________________



  /* Function Modifier updated while getting the function type */
  private int functionModifier = 0x0000;


}
