// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTFunctionDirectDeclarator.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -October,1997: Added Parsing methods
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;

public class ASTFunctionDirectDeclarator extends SimpleNode {
  public ASTFunctionDirectDeclarator(int id) {
    super(id);
  }

  public ASTFunctionDirectDeclarator(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTFunctionDirectDeclarator(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTFunctionDirectDeclarator(p, id);
  }


  /** saves the function initial value as a node property */
  public void setInitValueImage(String id)
  {
    try {
      setProperty("initVal", id);
    } catch (PropertyAccessException e) {
    }
  }

  /** Returne the function initial value if any */
  public String getInitValueImage() {
    try {
      return (String) getProperty("initVal") ;
    } catch (PropertyAccessException e) {
      return " ";
    }
  }

  /**
   * Set the function name.
   * <p><b> Algorithm:</b><ul>
   * Set name of the QualifiedId child.
   * </ul>
   * @param name the function name.
   */
  public void setFctName (String name) {
    if ( jjtGetChild(0) instanceof ASTQualifiedId) {
      ( (ASTQualifiedId)jjtGetChild(0)).setName(name); 
    }
  }




  /**
   * Returns the function name.
   * <p><b> Algorithm:</b><ul>
   * return ScopedName of first child QualifiedId.
   * </ul>
   * @result the function scoped name. 
   * @see ORG.opengroup.adl.parsers.adlcpp.ScopedName
   */
  public ScopedName getFctName () {

    if ( jjtGetChild(0) instanceof ASTQualifiedId) {
      return new ScopedName(( (ASTQualifiedId)jjtGetChild(0)).getName(),
			   ( (ASTQualifiedId)jjtGetChild(0)).getScope() ); 
    } else // never occurs
      return null;
  }

  /**
   * Adds parameters to function according to ParameterDeclaration(List) node.
   *
   * <p><b> Algorithm:</b><ul>
   * <li> Return unmodified function when second child does not exist.
   * <li> Else, get parameter(s) calling  getFctParameter(s) in both cases of
   * second child ( ParameterDeclaration/List ) node.
   * <li> Add recovered parameters to function.
   * <li> Add exceptions if next child is ExceptionSpec.
   * <li> return modified ORG.opengroup.adl.symboltable.LCallable.
   * </ul>
   * @param fct the function to which parameters are added
   * @result function after adding parameters.
   */ 
 public LCallable addFctParameters(LCallable fct) throws ParseException {
   LParameter fctparam;
   LParameter paramList[];
   int curchild = 1;
     
   int nbchild = jjtGetNumChildren();
   // If no ParameterDeclaration(List) node exist return unmodified
   if(nbchild <2) 
     return fct;
 
   if( jjtGetChild(1) instanceof ASTParameterDeclaration ) {
     // get function parameter.
     fctparam = ( (ASTParameterDeclaration)jjtGetChild(1)).getFctParameter();
     // add recovered parameter as a fct parameter.
     fct.addParameter(fctparam);
     // increment current child.
     curchild++ ;
   }
   else if (jjtGetChild(1) instanceof ASTParameterDeclarationList ) {
     // get array of parameters
     paramList= ( (ASTParameterDeclarationList)jjtGetChild(1)).
                                                     getFctParameters();
     for (int i=0;i<paramList.length;i++) {
       // Add each of the array members parameters to fucntion.
       fct.addParameter(paramList[i]);
     }
     curchild ++ ;
   }   

   if ( curchild > nbchild -1 )
     return fct ;
   // add exceptions if current child is an ExceptionSpec.
   if ( jjtGetChild( curchild) instanceof ASTExceptionSpec )
     fct =((ASTExceptionSpec )jjtGetChild( curchild )).addExceptions(fct) ;
     
   return fct;
 }



  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
