// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTIdExpr.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -November,97: Added Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;

public class ASTIdExpr extends SimpleNode implements Typed {

  /** Tells if id expression is an attribute of the tested C++ class */
  private boolean isTestedClassAttribute = false ;

  /** Flag telling if id expression is a dataset name */
  public boolean isDataset = false ;

  public ASTIdExpr(int id) {
    super(id);
  }

  public ASTIdExpr(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTIdExpr(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTIdExpr(p, id);
  }


  /**
    * Sets indicator that tells if parsed Id Expression is an attribute of
    * the tested class 
    */
  public void indicateIfClassAttribute ( boolean indicator ) 
  {
    isTestedClassAttribute = indicator ;
  }

  /** Tells if Id Expression symbol is an attribute of the tested class */
  public boolean isTestedClassField() 
  {
    return isTestedClassAttribute;
  }



  /** Return the Id expression name saved as a node property */
  public String getIdName()
  {
    try {
      return (String)getProperty("IDname");
    } catch (PropertyAccessException e) {
    return " "; }
  }

  /** Save Id expression type as a node property */
  public void setType (LType type)
  {
    try {
      setProperty("type", type );
    } catch (PropertyAccessException e) {
    }
  }

  /** Return Id expression type saved as a node property */
  public LType getType() {
    try {
      return (LType) getProperty("type");
    } catch (PropertyAccessException e) {
      return null;  // never occurs
    }
  }


  /** Constructs a ScopedName from node name and scope 
   * @see ScopedName
   */
  public ScopedName getScopedName () {
    Scope _scope = getScope();
    String _name = getIdName();
 
    return new ScopedName( _name, _scope);
  }


  /** saves Id expression name as a node property */
  public void setIdName(String id)
  {
    try {
      setProperty("IDname", id);
    } catch (PropertyAccessException e) {
    }
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
 

}
