// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTMemberDeclaration.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - October,  1997: Added parsing methods
//     
//
// ***********************************************************************


package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;




public class ASTMemberDeclaration extends SimpleNode {
  public ASTMemberDeclaration(int id) {
    super(id);
  }

  public ASTMemberDeclaration(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTMemberDeclaration(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTMemberDeclaration(p, id);
  }

  /**
   * Declares a class member.
   * Note: Valid only when class member is  DeclarationSpecifiers + Declarators
   * => Case of data members declarations.
   * <p><b>Algorithm:</b><ul>
   * <li> get member type and modifier.
   * <li> 2nd child one of Declarator, DirectDeclarator or MemberDeclaratorList
   * => Call appropriate 2nd child declaration method.
   * </ul>
   */
  public void MakeUpMemberDecl() throws  ParseException {
    // get node children
    int nbchild = jjtGetNumChildren();
    // get meber type.
    LType type = getType();
    // get modifier.
    int mdf = getMemberModifier();
 
    // test just for security
    if( nbchild >1) {
      // Case 2nd child is Declarator : call its make up method
      if( jjtGetChild(1) instanceof ASTDeclarator) {
	( (ASTDeclarator) jjtGetChild(1)).MakeUpDeclarator(type,mdf );
      }
      else if( jjtGetChild(1) instanceof ASTDirectDeclarator) {
	((ASTDirectDeclarator)jjtGetChild(1)).MakeUpDirectDeclarator(type,mdf);
      }
      else if( jjtGetChild(1) instanceof ASTMemberDeclaratorList) {
	( (ASTMemberDeclaratorList) jjtGetChild(1)).
	  checkMemberDeclaratorList(type, mdf);
      } 

    }
  }

  /**
   * Return the member declaration type.
   * <p><b>Algorithm:</b><ul>
   * Simply calls getType of 1st child DeclarationSpecifiers when exits and 
   * return integer if not.
   * </ul>
   */
  private LType getType() throws ParseException {
   LType type;
   String localstr;
   
   if(  jjtGetChild(0) instanceof ASTDeclarationSpecifiers)  {
     type =  ( (ASTDeclarationSpecifiers) jjtGetChild(0)).getType();
     return type;
   }
   else {
     // default type is int
     type = (LType) stm.getSymbol( "int");
     return type;
   }
  }

  /** Returns the current member modifier.
   * <p><b>Algorithm:</b><ul>
   * Simply calls getModifier of 1st child DeclarationSpecifiers when exits 
   * and return 0 if not.
   * </ul>
   */
  private int getMemberModifier() {
    int mdf = 0;
    if(  jjtGetChild(0) instanceof ASTDeclarationSpecifiers)  {
      mdf = ( (ASTDeclarationSpecifiers) jjtGetChild(0)).getModifier();
    }
    
    mdf |= parser.currentAccessSpecifier;
    return mdf;
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
