// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTParameterDeclaration.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG -November,97: Added Parsing methods
//     
//
// ***********************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.parsers.ParseException;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

public class ASTParameterDeclaration extends SimpleNode {
  public ASTParameterDeclaration(int id) {
    super(id);
  }

  public ASTParameterDeclaration(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTParameterDeclaration(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTParameterDeclaration(p, id);
  }

  /**
   * Return function/constructor parameter.
   * 
   * <p><b>Algorithm:</b><ul>
   * <li> get parameter type and modifier from DeclarationSpecifiers 1st child.
   * <li> 2nd child may be Declarator, DirectDeclarator or AbstractDeclarator
   *  <ul>
   *  <li> In 1st and 2nd case => call child build parameter methods.
   *  <li> If 2nd child is an abstract declarator,update parameter type with 
   *  update param type method. Build a parameter with that type and an empty
   *  string name.
   *  </ul>
   * <li> return the obtained parameter.
   * </ul>
   * <p> Note that node has at least 2 children.
   * @result obtained parameter.
   */

  public LParameter getFctParameter( ) throws ParseException {

    LParameter param = null;  
    LType type = ( (ASTDeclarationSpecifiers)jjtGetChild(0)).getType();
    int modif = ( (ASTDeclarationSpecifiers)jjtGetChild(0)).getModifier();

    // 2nd child can be Declarator, DirectDeclarator or AbstractDeclarator.
    if (  jjtGetChild(1) instanceof ASTDirectDeclarator) {
      param= ((ASTDirectDeclarator)jjtGetChild(1)).BuildParamDecl(type, modif);
    }
    else  if(  jjtGetChild(1) instanceof ASTDeclarator  )  {
      param= ( (ASTDeclarator)jjtGetChild(1)).BuildParamDecl(type, modif); 
    } 
    else if (jjtGetChild(1) instanceof ASTAbstractDeclarator  )  {
      // just update the type.
      type = ( ( ASTAbstractDeclarator)jjtGetChild(1)).updateParamType(type) ;
      // create an anonymous parameter.
      param = sbf.newParameter("",modif,type);
      
    }

    if ( debug ) {
      System.out.println("debug ParameterDeclaration:found parameter "+ param);
    }
    return param;
  }  


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }

}
