// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTTDDDatasetDecl.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - November,  1997: Added parsing Methods
//     
//
// ***********************************************************************
package ORG.opengroup.adl.parsers.adlcpp;

import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;

public class ASTTDDDatasetDecl extends SimpleNode {
  public ASTTDDDatasetDecl(int id) {
    super(id);
  }

  public ASTTDDDatasetDecl(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTTDDDatasetDecl(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTTDDDatasetDecl(p, id);
  }

  /**
   * Called by the parser to add the declared dataset into symbol table
   * Get type, modifier and name of @param toadd. Creates the TDDDatset to add
   * Stores the dataset and its type as a node property for type checking use.
   */
  public void addDatasetDecl(LExpression toadd) throws ParseException {

    if( toadd == null)
  errorMessage("Error : illegal definition of dataset");

    TDDDataset toaddDataset = sbf.newTDDDataset( toadd.getName(),0,(toadd.getModifier()).getModifierCode(),toadd.getType());

    try {
      toaddDataset = (TDDDataset) stm.putSymbol( toaddDataset);
    } catch ( SymbolAlreadyDefinedException e) {
      errorMessage("EROOR : symbol '"+ toadd.getName()+"' already defined");
    }
    // save the declared dataset
    setDataset( toaddDataset ) ;
    // save its type
    setDatasetType( toaddDataset.getType());
  }

  /* Stores the dataset Type as a Node property */
  private void setDatasetType (LType dtsetType) {
    try {
      setProperty("datasetType", dtsetType);
    } catch (PropertyAccessException e) {
    }
  }
  /* Stores the declared dataset as a Node property */
  private void setDataset (TDDDataset dtset) {
    try {
      setProperty("dataset", dtset);
    } catch (PropertyAccessException e) {
    }
  }

  /** 
   * returns the declared dataset stored as a Node property
   */
  public TDDDataset getDataset() {
    try {
      return (TDDDataset) getProperty("dataset");
    } catch (PropertyAccessException e) { // should never occur
      return null ;
    }
  }

  
  /** 
   * returns the dataset Type stored as a Node property
   */
  public LType getDatasetType() {
    LType dtsetType = null;
    try {
      dtsetType= (LType) getProperty("datasetType");
    } catch (PropertyAccessException e) { // should never occur
    }
    return dtsetType;
  }



  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
