// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   adlcppparser
// NAME:        ASTTDDFactoryDef.java
//
// HISTORY:
//      October,97 : Generated by JJTree
//      Wassel BERRAYANA, TOG - November,1997: Added Parsing methods
//     
//
// ************************************************************************

package ORG.opengroup.adl.parsers.adlcpp;

import java.util.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import ORG.opengroup.adl.parsers.*;

public class ASTTDDFactoryDef extends SimpleNode {
  public ASTTDDFactoryDef(int id) {
    super(id);
  }

  public ASTTDDFactoryDef(ADLCPPParser p, int id) {
    super(p, id);
  }

  public static Node jjtCreate(int id) {
      return new ASTTDDFactoryDef(id);
  }

  public static Node jjtCreate(ADLCPPParser p, int id) {
      return new ASTTDDFactoryDef(p, id);
  }


  /* Retruns the factory Name. 
   * How:
   * When called 2 nodes on the stack.
   * peekNode stack = instanceof FunctionDeclarator or FunctionDirectDeclarator
   * Get factory Name calling  method getFctName() of that node.
   */

 private ScopedName getFactoryName () throws ParseException {

   Node declNode= (parser.jjtree).peekNode();
   ScopedName result = null;
   if( declNode instanceof ASTFunctionDeclarator ) 
	 result = ( (ASTFunctionDeclarator)declNode).getFctName();
   else if ( declNode instanceof ASTFunctionDirectDeclarator )
	 result = ( (ASTFunctionDirectDeclarator) declNode).getFctName();
   
   else { // shoould never occur
     errorMessage("Error : Factroy name not found");
   }
   return result;
  }


  /* Called by makeUpTDDFactoryDecl. Returns the factory type 
   * How :
   * When called, TDDFactoryDecl (not crated yet)has 2 children on node stack
   * node in the stack top = Function[Direct]Declarator below DeclarationSpec
   * Pop then peek the node to get DeclarationSpec node=> tells factory type
   * Finally push again top stack node
   */
public LType getFactoryReturnType( ) throws ParseException {
   Node topStackNode;
   Node declSpecNode;
   LType typefactory;
   String localstr;

 
    topStackNode = (parser.jjtree).popNode(); // Secure call!

    try {
      declSpecNode= (parser.jjtree).peekNode();
    } catch (EmptyStackException e) { 
      (parser.jjtree).pushNode(topStackNode);      
      return (LType) stm.getSymbol( "int"); // Assume default value is int
    }

   
    if(  declSpecNode instanceof ASTDeclarationSpecifiers)  {
	typefactory =  ( (ASTDeclarationSpecifiers) declSpecNode).getType();
    }
    else {
      typefactory = (LType) stm.getSymbol( "int");
    }

   (parser.jjtree).pushNode(topStackNode); // Put back Node on the stack 
   return typefactory;
 }


  /* Looks for child FunctionDeclarator and call its UpdateType method
   * Called when the Node Function(Direct)Declarator is on the stack top.
   */
  public LType UpdateFactoryType( LType curType) {

    Node fctNode= (parser.jjtree).peekNode();
      if( fctNode instanceof ASTFunctionDeclarator) {
	return ( (ASTFunctionDeclarator) fctNode).UpdateFctType( curType);
      }
      else {
	return curType;
      }
  }




  /* Called by the parser to define a factroy and add it into symbol table.
   * How :
   * when method called, ASTTDDFactoryDef node not created yet
   * It has 2 children on stack : ASTDeclarationSpec and ASTFunctionDeclarator
   *   1- Call getFactoryname to the factory name.
   *   2- Construct a TDD factory with that name.
   *   3- Call getFactoryReturnType and UpdateFactoryType to get factory  Type
   *   4- Add factory parameters calling addFactoryParameters
   *   5- store the resulting factory into the symbol table
   *   6- Store factroy scope and type as node properties for type checking use
   *   7- open factory scope.
   */ 
  public TDDFactory makeUpTDDFactory( ) throws ParseException {
    LType factoryType;
    String factoryName;
    Scope factoryScope;
   
    ScopedName factoryscopedname = getFactoryName();
    factoryName = factoryscopedname.Name;
    factoryScope= factoryscopedname.Scope;

    if( factoryScope !=null) 
      errorMessage("Error : illegal factory definition");

    TDDFactory madeFactory = sbf.newTDDFactory(factoryName);
    factoryType = getFactoryReturnType();

    factoryType = UpdateFactoryType( factoryType);

    madeFactory.setReturnType( factoryType);

    madeFactory = (TDDFactory) addFactoryParameters ((LCallable) madeFactory);

    try {
	madeFactory = (TDDFactory) stm.putSymbol(madeFactory);
    } catch (SymbolAlreadyDefinedException e) {
      errorMessage("Error : factory '"+ factoryName+" already defined");
    }
    
    setFactoryType(factoryType );
    setFactoryScope(  madeFactory);
    stm.openScope( madeFactory);
    return madeFactory ;
  }


  /* Method called to store the recovered factory type as a node property
   */
 private void setFactoryType (LType factoryType) {
    try {
      setProperty("factoryType", factoryType);
    } catch (PropertyAccessException e) {
    }
  }

  /* Method called to store the recovered factory scope as a node property
   */
 private void setFactoryScope (Scope factoryScope) {
    try {
      setProperty("factoryScope", factoryScope);
    } catch (PropertyAccessException e) {
    }
  }

  /** Set the relinquish child scope if exits */
 public void setRelinquishScope (Scope scope) {
    try {
      setProperty("relinquishScope", scope);
    } catch (PropertyAccessException e) {
    }
  }

  /** return the relinquish child scope if exists */
  public Scope getRelinquishScope() {
    Scope scope = null;
    try {
      scope= (Scope) getProperty("relinquishScope");
    } catch (PropertyAccessException e) { 
    }
    return scope;
  }



  /* Called to update current scope while type checking the node
   * returns the factory scope stored as a Node property
   */
  public Scope getFactoryScope() {
    Scope factoryScope = null;
    try {
      factoryScope= (Scope) getProperty("factoryScope");
    } catch (PropertyAccessException e) { // should never occur
    }
    return factoryScope;
  }


  /* Called to Type check ASTTDDFactoryDef node
   * returns the factory return Type stored as a Node property
   */
  public LType getFactoryType() {
    LType factoryType = null;
    try {
      factoryType= (LType) getProperty("factoryType");
    } catch (PropertyAccessException e) { // should never occur
    }
    return factoryType;
  }




  /* This Method adds paremeters to the Lcallable @param curfct
   * It is called when ASTFunction[Direct]Declarator is the stack top
   */
  public LCallable addFactoryParameters(LCallable curfct) throws ParseException {
    Node declNode= (parser.jjtree).peekNode();
    if( declNode instanceof ASTFunctionDeclarator ) {
       curfct = ( (ASTFunctionDeclarator)declNode).addFctParameters(curfct);
    }
    else if (declNode instanceof ASTFunctionDirectDeclarator ){
 
       curfct = ( (ASTFunctionDirectDeclarator) declNode).addFctParameters(curfct);
    }     
       
    return curfct;
  }


  /** Accept the visitor. **/
  public Object jjtAccept(ADLCPPParserVisitor visitor, Object data) throws ParseException {
    return visitor.visit(this, data);
  }
}
