//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// Copyright (c) 1994-1996 Sun Microsystems Inc.
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/runtime/ADLDataSet.java,v $
// $Revision: 1.10 $
// $Date: 1998/08/06 14:09:37 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	ADL Java Runtime Environment
// NAME:	ADLDataSet.java
// 	
// HISTORY:
// 	Matt Evans, Sun Microsystems Inc. - Dec 30, 1996: Created
//      Lionel Mallet, The Open Group RI  - Oct  9, 1997: Renamed
//      See Log at end of file
// 		
// ***********************************************************************

package ORG.opengroup.adl.runtime;

/**	
 * The ADLDataSet class is an abstract representation of a TDD dataset.
 * It defines the operations and attributes of all TDD data generation
 * objects.
 *
 * @see ORG.opengroup.adl.runtime.ADLLiteralSet
 * @see ORG.opengroup.adl.runtime.ADLConcatenationSet
 * @see ORG.opengroup.adl.runtime.ADLFactorySet
 */
public abstract class ADLDataSet {

  /** the test from which the dataset is used */
  protected ADLTest testContext = null;

  /** the dataset label */
  private String label = "Anonymous";

  /**
   * Returns the label of the element.
   *
   * @return the string label
   */
  public String getLabel() { return label; }

  /**
   * Sets the label of the element.
   */
  public void setLabel(String label) {
    if ((label != null) && !label.equals("")) {
      this.label = label;
    }
  }

  /** the number of values in the dataset */
  protected int numValues = 0;

  /**
   * Returns the potential number of values 
   * that this dataset can yield.
   *
   * @return the number of values of this dataset
   */
  public int getNumValues() { return numValues; }

  /**
   * Provides the object at the requested index. 
   *
   * @param index the index of the requested value (i.e., ordinal number)
   * @return Object the Object value at the index
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   * @exception ADLRuntimeException runtime exception that should be
   *            caught in <code>ADLTest.runTestInstance()</code>.
   */
  abstract public Object provide(int index)
    throws ArrayIndexOutOfBoundsException, ADLRuntimeException;

  /**
   * Relinquishes the object at the requested index.

   * <p>Returning false indicates an abort of the test. Not all
   * ADLDataSet objects will need to do anything in relinquish.
   *
   * @param index the index of the requested value (i.e., ordinal number)
   * @return boolean status of the relinquish (probably always true)
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   * @exception ADLRuntimeException runtime exception that should be
   *            caught in <code>ADLTest.runTestInstance()</code>.
   */
  abstract public boolean relinquish(int index)
    throws ArrayIndexOutOfBoundsException, ADLRuntimeException;


  /**
   * Verifies that a given ordinal number is within the bounds of the 
   * TDD dataset: greater than 0 and lower than <code>numValues</code>.
   *
   * @param index the index of the requested value (i.e., ordinal number)
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   */
  protected void checkBounds(int index) throws ArrayIndexOutOfBoundsException {
    if (index < 0 || index >= numValues) {
      throw new ArrayIndexOutOfBoundsException(getClass().getName() + 
					       ": index out of range (" + 
					       index + "/" + numValues +
					       ")");
    }
  }
}

// ***********************************************************************
// History Log:
// $Log: ADLDataSet.java,v $
// Revision 1.10  1998/08/06 14:09:37  mallet
// Moved Java runtime source tree in /afs/gr/users/mallet/Adl/adl2/src/ORG/opengroup/adl/runtime.
//
// Revision 1.9  1998/02/27 10:08:29  mallet
// Updated copyright notice.
//
// Revision 1.8  1998/02/12 16:04:33  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.7  1998/02/11 18:16:30  mallet
// Added ADLRuntimeException in the throw clause of provide and
// relinquish: required for factory to have tdd_end_case and tdd_end_test
// act as expected.
//
// Revision 1.6  1998/01/08 14:40:42  mallet
// Updated and fixed javadoc comments.
//
// Revision 1.5  1997/12/23 09:11:47  mallet
// Fixed javadoc comments.
//
// Revision 1.4  1997/11/26 13:59:45  mallet
// Fixed runtime package name: now ORG.opengroup.adl.runtime.
//
// Revision 1.3  1997/10/29 13:33:15  mallet
// Changed ADLDataSet to become an abstract class rather than an
// interface: all classes that used to implement this interface are in
// fact related and share some code that has now moved into the abstract
// class.
//
// Revision 1.2  1997/10/10 15:34:26  mallet
// Removed checkBounds from interface: there can't be private methods in
// an interface.
//
// Revision 1.1  1997/10/10 13:56:49  mallet
// First implementation of TDD datasets for Java runtime.
//
//
// ***********************************************************************
