//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// Copyright (c) 1994-1996 Sun Microsystems Inc.
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/runtime/ADLLiteralSet.java,v $
// $Revision: 1.17 $
// $Date: 1998/08/06 14:09:39 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	ADL Java Runtime Environment
// NAME:	ADLLiteralSet.java
// 	
// HISTORY:
// 	Matt Evans, Sun Microsystems Inc. - Dec 30, 1996: Created
//      Lionel Mallet, The Open Group RI  - Oct  9, 1997: Renamed
//      See Log at end of file
// 		
// ***********************************************************************

package ORG.opengroup.adl.runtime;

/**	
 *  The ADLLiteralSet class represents a TDD dataset made of constant
 *  (literal or static) values.
 *  <p.It is generally used as is (i.e., by instantiating) for anonymous
 *  datasets, whereas it is subclassed for a named dataset.
 */
public class ADLLiteralSet extends ADLDataSet {

  /**
   * The list of values in the dataset as a list of objects.
   * Primitive type values are converted into their respective object
   * type.
   */
  protected Object[] values = null;

  /**
   * Basic constructor for unnamed datasets (i.e., subclasses): initializes
   * only test context.
   *
   * @param tc    the test case in which this dataset is used
   */
  public ADLLiteralSet (ADLTest tc) {
    testContext = tc;
  }

  /**
   * Basic constructor for named datasets (i.e., subclasses): initializes
   * test context and label.
   *
   * @param tc    the test case in which this dataset is used
   * @param label the name of the dataset
   */
  protected ADLLiteralSet (ADLTest tc, String label) {
    testContext = tc;
    setLabel(label);
  }

  /**
   * Boolean constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of boolean values
   */
  public ADLLiteralSet (ADLTest tc, boolean[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Boolean(values[x]);
    }
  }

  /**
   * Character constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of char values
   */
  public ADLLiteralSet (ADLTest tc, char[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Character(values[x]);
    }
  }

  /**
   * Byte constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of byte values
   */
  public ADLLiteralSet (ADLTest tc, byte[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Byte(values[x]);
    }
  }

  /**
   * Short integer constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of short values
   */
  public ADLLiteralSet (ADLTest tc, short[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Short(values[x]);
    }
  }

  /**
   * Integer constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of int values
   */
  public ADLLiteralSet (ADLTest tc, int[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Integer(values[x]);
    }
  }

  /**
   * Long integer constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of long values
   */
  public ADLLiteralSet (ADLTest tc, long[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Long(values[x]);
    }
  }

  /**
   * Float constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of float values
   */
  public ADLLiteralSet (ADLTest tc, float[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Float(values[x]);
    }
  }

  /**
   * Double constructor for anonymous datasets: intializes test context,
   * number of values and array of values.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of double values
   */
  public ADLLiteralSet (ADLTest tc, double[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = new Double(values[x]);
    }
  }

  /**
   * Object constructor for anonymous datasets: intializes test context,
   * number of values and array of values. This constructor is
   * used for String as well.
   *
   * @param tc     the test case in which this dataset is used
   * @param values the static list of Object values
   */
  public ADLLiteralSet (ADLTest tc, Object[] values) {
    testContext = tc;
    numValues = values.length;
    this.values = new Object[numValues];
    for (int x = 0; x < numValues; x++) {
      this.values[x] = values[x];
    }
  }

  /**
   * Provides a value from the literal dataset: the returned value is
   * just the nth value in the initial static list. We first verify that
   * the requested test data instance number is within bounds for this
   * dataset.
   *
   * @param index the test data instance number to return
   * @return the <em>index</em>'ed value in the initial list of values
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   */
  public Object provide(int index) throws
    ArrayIndexOutOfBoundsException {
    // check instance number
    checkBounds(index);

    return values[index];
  }


  /**
   * Relinquishes a value from the literal dataset: there is nothing
   * particular to do. Anyway we first verify that the requested
   * test data instance number is within bounds for this dataset.
   *
   * @param index the test data instance number to return
   * @return Always true
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   */
  public boolean relinquish(int index) throws
    ArrayIndexOutOfBoundsException {
    // check instance number
    checkBounds(index);

    return true;
  }

  /**
   * Add a new boolean value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(boolean value) throws ADLRuntimeException {
    addValue(new Boolean(value));
  }

  /**
   * Add a new char value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(char value) throws ADLRuntimeException {
    addValue(new Character(value));
  }

  /**
   * Add a new byte value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(byte value) throws ADLRuntimeException {
    addValue(new Byte(value));
  }

  /**
   * Add a new short value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(short value) throws ADLRuntimeException {
    addValue(new Short(value));
  }

  /**
   * Add a new int value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(int value) throws ADLRuntimeException {
    addValue(new Integer(value));
  }

  /**
   * Add a new long value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(long value) throws ADLRuntimeException {
    addValue(new Long(value));
  }

  /**
   * Add a new float value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(float value) throws ADLRuntimeException {
    addValue(new Float(value));
  }

  /**
   * Add a new double value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(double value) throws ADLRuntimeException {
    addValue(new Double(value));
  }

  /**
   * Add a new Object value to a literal dataset.
   *
   * @param value the added value
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addValue(Object value) throws ADLRuntimeException {
    if (values == null) {
      values = new Object[1];
      values[0] = value;
    }
    else {
      Object[] newValues = new Object[values.length + 1];

      // copy previous values
      for (int i = 0; i < values.length; i++) {
	newValues[i] = values[i];
      }
      
      // new value
      newValues[values.length] = value;
      
      // now swap arrays
      values = newValues;
    }
    
    // increment the number of values
    numValues += 1;
  }

  /**
   * Add a range of char values to a literal dataset.
   *
   * Note: both minValue and maxValue must be digits.
   *
   * @param minValue the minimum value of the range
   * @param maxValue the maximum value of the range
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset, or, minValue or
   *            maxValue are not digits.
   */
  public void addRangeValues(char minValue, char maxValue)
    throws ADLRuntimeException {
    
    if (!Character.isDigit(minValue) || !Character.isDigit(maxValue)) {
      throw new ADLRuntimeException(getClass().getName() + ": " +
				    "cannot range on non-digit " +
				    "characters - '" + minValue +
				    "' -> '" + maxValue + "'");
    } else {
      int radix = 10;
      int minDigit = Character.digit(minValue, radix);
      int maxDigit = Character.digit(maxValue, radix);
      if (minDigit < maxDigit) {
	for (int i = minDigit; i <= maxDigit; i++) {
	  addValue(new Character(Character.forDigit(i, radix)));
	}
      } else {
	for (int i = minDigit; i >= maxDigit; i--) {
	  addValue(new Character(Character.forDigit(i, radix)));
	}
      }
    }
  }

  /**
   * Add a range of byte values to a literal dataset.
   *
   * @param minValue the minimum value of the range
   * @param maxValue the maximum value of the range
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addRangeValues(byte minValue, byte maxValue) throws ADLRuntimeException {
    if (minValue < maxValue) {
      for (byte i = minValue; i <= maxValue; i++) {
	addValue(new Byte(i));
      }
    } else {
      for (byte i = minValue; i >= maxValue; i--) {
	addValue(new Byte(i));
      }
    }
  }

  /**
   * Add a range of short values to a literal dataset.
   *
   * @param minValue the minimum value of the range
   * @param maxValue the maximum value of the range
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addRangeValues(short minValue, short maxValue) throws ADLRuntimeException {
    if (minValue < maxValue) {
      for (short i = minValue; i <= maxValue; i++) {
	addValue(new Short(i));
      }
    } else {
      for (short i = minValue; i >= maxValue; i--) {
	addValue(new Short(i));
      }
    }
  }

  /**
   * Add a range of int values to a literal dataset.
   *
   * @param minValue the minimum value of the range
   * @param maxValue the maximum value of the range
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addRangeValues(int minValue, int maxValue) throws ADLRuntimeException {
    if (minValue < maxValue) {
      for (int i = minValue; i <= maxValue; i++) {
	addValue(new Integer(i));
      }
    } else {
      for (int i = minValue; i >= maxValue; i--) {
	addValue(new Integer(i));
      }
    }
  }

  /**
   * Add a range of long values to a literal dataset.
   *
   * @param minValue the minimum value of the range
   * @param maxValue the maximum value of the range
   * @exception ADLRuntimeException the type of the value does not match
   *            that of previous values in the dataset.
   */
  public void addRangeValues(long minValue, long maxValue) throws ADLRuntimeException {
    if (minValue < maxValue) {
      for (long i = minValue; i <= maxValue; i++) {
	addValue(new Long(i));
      }
    } else {
      for (long i = minValue; i >= maxValue; i--) {
	addValue(new Long(i));
      }
    }
  }

  /**
   * Checks that a test data instance number is within bounds for
   * this dataset: the superclass check does the correct job.
   *
   * @param index the test data instance number to check
   * @exception ArrayIndexOutOfBoundsException the index is out of
   *            the range of potential instances of the dataset.
   */
  protected void checkBounds(int index) throws
    ArrayIndexOutOfBoundsException {
    if (values == null) {
      throw new ArrayIndexOutOfBoundsException("ADLLiteralSet: no values");
    }
    
    super.checkBounds(index);
  }

  /**
   * Copies the set of values from another ADLLiteralSet.
   *
   * <p>Note: the dataset values are not cloned, just referenced.
   *
   * @param copySet the dataset from which we get the values
   */
  protected void setValues(ADLLiteralSet copySet) {
    values = copySet.values;
    numValues = values.length;
  }
}

// ***********************************************************************
// History Log:
// $Log: ADLLiteralSet.java,v $
// Revision 1.17  1998/08/06 14:09:39  mallet
// Moved Java runtime source tree in /afs/gr/users/mallet/Adl/adl2/src/ORG/opengroup/adl/runtime.
//
// Revision 1.16  1998/03/10 17:27:29  mallet
// Merged v1.1 with development trunk
//
// Revision 1.15.2.1  1998/03/06 11:10:13  bareau
// In addValue(Object), suppressed the check that the new added object is
// assignable with precedently added values: this does not work when these
// different values are not directly assignable but inherit from the same class.
//
// Revision 1.15  1998/03/02 12:41:04  bareau
// Bug fix: the basic constructor for anonymous datasets must be public.
//
// Revision 1.14  1998/03/02 11:25:29  mallet
// Added missing constructor for unnamed literal sets.
// Fixed bug in addValue(Object): numValues was not incremented, leaving
// the set with no values.
//
// Revision 1.13  1998/02/27 10:08:30  mallet
// Updated copyright notice.
//
// Revision 1.12  1998/02/26 09:54:59  mallet
// Added the addRangeValues methods for integral types (char -> long).
//
// Revision 1.11  1998/02/23 13:48:17  mallet
// Added a set of methods (addValue(*)) to be able to add a single value
// to a dataset: this can simplify the generated code.
//
// Revision 1.10  1998/02/12 16:04:34  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.9  1998/01/08 14:40:44  mallet
// Updated and fixed javadoc comments.
//
// Revision 1.8  1997/12/23 09:11:47  mallet
// Fixed javadoc comments.
//
// Revision 1.7  1997/11/26 13:59:46  mallet
// Fixed runtime package name: now ORG.opengroup.adl.runtime.
//
// Revision 1.6  1997/11/24 16:39:43  mallet
// Added byte constructor and javadoc comments.
//
// Revision 1.5  1997/11/14 09:26:25  mallet
// Added a new protected constructor similar to those of other dataset
// classes. This constructor is protected since it should only be called
// from subclasses.
// Added a new protected method to copy the list of values from another
// ADLLiteralSet. This simplifies a little bit the way the code is
// generated.
//
// Revision 1.4  1997/10/29 13:37:15  mallet
// ADLLiteralSet now extends ADLDataSet: removed shared code.
//
// Revision 1.3  1997/10/15 08:45:19  mallet
// Made class public.
// Added field testContext as ADLTest.
// Added ADLTest parameter to constructors.
// Made "out of bounds" message more detailed.
//
// Revision 1.2  1997/10/10 15:42:06  mallet
// Fixed small compilation bugs.
//
// Revision 1.1  1997/10/10 13:56:50  mallet
// First implementation of TDD datasets for Java runtime.
//
//
// ***********************************************************************
