//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/runtime/ADLObjectACO.java,v $
// $Revision: 1.3 $
// $Date: 1998/08/06 14:09:39 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	ADL Java Runtime Environment
// NAME:	ADLObjectACO.java
// 	
// HISTORY:
// 	Lionel Mallet, The Open Group RI - Sep 23, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

package ORG.opengroup.adl.runtime;

import java.util.Enumeration;
import java.util.Vector;

/**	
 * The ADLObjectACO class is an abstract root class for all generated ACO
 * classes.
 */
abstract public class ADLObjectACO implements ADLResultCodes {

  /** the current test context, providing access to TET reporting methods
   */
  protected ADLTest testContext = null;

  /** the actually tested object */
  protected Object realObject = null;

  /** verbose reporting mode indicator */
  protected boolean debugMode = false;

  /**
   * boolean defined to indicate a super class not to report to TET as if
   * it were the top level ACO object.
   *
   * <p>The pattern is to intialize a local variable in the ACO method
   * to hold the current value of finalJournalReporting and to be used
   * as a local test in the method to decide where to report.
   * <p>When the ACO inherits specs from its super class, it will reset
   * finalJournalReporting to false before calling the super class ACO
   * method and then use the local variable to restore it to its previous
   * value.
   */
  protected boolean finalJournalReporting = true;

  /** skip assertions indicator.
   * 
   * <p>It is used to propagate the information between super/sub classes
   * both ways.
   * <p>It cannot be initialized in constructor since one may use the same
   * ACO object repeatedly in a single test directive and call tdd_skip
   * in between.
   */
  protected boolean skipAssertions = false;

  /**
   * Basic constructor: set real object and test context, and initialize
   * <code>debugMode</code> according to TET variable
   * <code>ADL_RPT_DETAIL</code>
   *
   * @param obj the implementation object under test
   * @param tc  the test directive object driving this ACO object
   */
  protected ADLObjectACO(Object obj, ADLTest tc) {
    realObject = obj;
    testContext = tc;

    debugMode = testContext.debugMode;
  }

  /**
   * Returns an enumeration of <code>short</code> values representing a
   * range.
   *
   * @param min the lowest value in the range
   * @param max the greatest value in the range
   * @return the enumeration of short values from min to max included
   */
  static public Enumeration ADL_short_range(short min, short max) {
    int arraySize = max - min + 1;
    Vector values = new Vector(arraySize);
    for (short index = min; index < max + 1; index++) {
      values.setElementAt(new Short(index), index - min);
    }

    return values.elements();
  }

  /**
   * Returns an enumeration of <code>int</code> values representing a
   * range.
   *
   * @param min the lowest value in the range
   * @param max the greatest value in the range
   * @return the enumeration of short values from min to max included
   */
  static public Enumeration ADL_int_range(int min, int max) {
    int arraySize = max - min + 1;
    Vector values = new Vector(arraySize);
    for (int index = min; index < max + 1; index++) {
      values.setElementAt(new Integer(index), index - min);
    }

    return values.elements();
  }

  /**
   * Returns an enumeration of <code>long</code> values representing a
   * range.
   *
   * @param min the lowest value in the range
   * @param max the greatest value in the range
   * @return the enumeration of short values from min to max included
   */
  static public Enumeration ALD_long_range(long min, long max) {
    int arraySize = (int)(max - min + 1);
    Vector values = new Vector(arraySize);
    for (long index = min; index < max + 1; index++) {
      values.setElementAt(new Long(index), (int)(index - min));
    }

    return values.elements();
  }
}

// ***********************************************************************
// History Log:
// $Log: ADLObjectACO.java,v $
// Revision 1.3  1998/08/06 14:09:39  mallet
// Moved Java runtime source tree in /afs/gr/users/mallet/Adl/adl2/src/ORG/opengroup/adl/runtime.
//
// Revision 1.2  1998/06/17 09:52:32  mallet
// Renamed ObjectACO to ADLObjectACO.
//
// Revision 1.13  1998/05/28 14:59:38  mallet
// Removed initialization of debugMode indicator through getvar: it's now
// done in the ADLTest class.
//
// Revision 1.12  1998/05/12 14:22:46  shane
// Added code to hide certain output unless debugMode is enabled.
// Also added debugMode to the testContext.
//
// Revision 1.11  1998/02/27 10:08:33  mallet
// Updated copyright notice.
//
// Revision 1.10  1998/02/12 16:04:36  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.9  1998/01/21 16:33:48  mallet
// Moved the "range" methods from ADL.java here with ADL_ prefix: static
// methods in ADL.java were causing parsing problems due to the ADL()
// rule for TDD.
//
// Revision 1.8  1998/01/08 14:40:46  mallet
// Updated and fixed javadoc comments.
//
// Revision 1.7  1997/12/23 09:11:48  mallet
// Fixed javadoc comments.
//
// Revision 1.6  1997/11/26 13:59:48  mallet
// Fixed runtime package name: now ORG.opengroup.adl.runtime.
//
// Revision 1.5  1997/10/29 13:34:16  mallet
// Changed ObjectACO to implement ADLResultCodes: this allows ACO to
// directly access result codes with any prefix or whatever.
//
// Revision 1.4  1997/10/16 14:07:44  mallet
// Added a protected boolean attribute skipAssertions to make propagating
// this information easier between super/sub ACO classes: once it is set
// somewhere, all other super/sub classes share the same value and act
// accordingly.
//
// Revision 1.3  1997/10/13 11:45:28  mallet
// Fixed import line for ADLTest class.
//
// Revision 1.2  1997/10/08 12:39:50  mallet
// Interim version to be completed.
// Removed temporaryResult boolean: TET is doing what this was supposed
// to do.
// Added debugMode boolean for subexpression breakdown. It is initialized
// in constructor according to ADL_RPT_DETAIL environment variable.
//
// ***********************************************************************
