//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/runtime/ADLTestCase.java,v $
// $Revision: 1.13 $
// $Date: 1998/08/06 14:09:41 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	ADL Java Runtime Environment
// NAME:	ADLTestCase.java
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - Sep 11, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

package ORG.opengroup.adl.runtime;

import java.util.*;
import java.lang.*;

/**
 * The ADLTestCase class implements the liaison between the ADL
 * runtime and TET. From the JET perspective, this class is a test
 * case implementation. However the test case implementation is rather
 * delegated to a subclass of ADLTest. So this class just bridges TET
 * and ADLTest.
 *
 * @see ORG.opengroup.adl.runtime.ADLTest
 */
public class ADLTestCase extends TET.BaseTestCase implements TET.RunTestPurpose
{
  /** the test case implementation */
  protected ADLTest theTest = null;

  public boolean debugMode = false;

  /**
   * Main routine for ADLTest: this should be called by the main
   * of an instance of a subclass of ADLTest.
   *
   * @param	name	the name of the test as it is shown to the user
   * @param	args[]	the command line arguments
   * @param	test	an instance of the (abstract) ADLTest class
   */
  public void main(String name, String args[], ADLTest test) {
    theTest = test;
    TET.BaseTestCase.main(name, args, this);
  }

  /**
   * Returns the minimum IC number for the test.
   */
  public int min_ic() {
    if (theTest == null) {
      return 0;
    } else {
      return 1;
    };
  }

  /**
   * Returns the maximum IC number for the test.
   */
  public int max_ic() { 
    if (theTest == null) {
      return 0;
    } else {
      return theTest.getNumInstances();
    }
  }

  /**
   * Returns whether the referenced IC is valid.
   *
   * @param ic	the IC to check
   */
  public boolean valid_ic(int ic) {
    if ((ic < this.min_ic()) || (ic > this.max_ic()))
      return false;
    else
      return true;
  }

  /**
   * Executes a specific IC.
   *
   * @param ts	the current TET TestSession
   * @param ic	the IC to execute
   */
  public void run_ic(TET.TestSession ts, int ic) {
    if (!valid_ic(ic))
      return;

    ts.ic_begin(ic, 1);
    try {
      ts.exec_tp(this, ic, 1, theTest);
    } catch (TET.TetException e) { 
      theTest.infoline("TP execution raised TetException" + e);
    }
    ts.ic_end(ic, 1);
  }

  /**
   * Executes TP for an IC. Called by the IC manager, so we need this
   * even though there is only ever one TP per IC.
   *
   * @param 	ts	the current TET testsession
   * @param	ic	the IC to run
   * @param	tpid	the TP to run (should always be 1)
   * @param	info	an unused optional info parameter passed in by the
   * IC manager
   */
  public void tp_run(TET.TestSession ts, int ic, int tpid, Object info)
  {
    // the ic number needs to be reduced by one here since instance
    // number starts from 0 in ADL but TET doesn't like IC number 
    // starting from 0.
    theTest.runTestInstance(ts, ic - 1);
  }

}

// ***********************************************************************
// History Log:
// $Log: ADLTestCase.java,v $
// Revision 1.13  1998/08/06 14:09:41  mallet
// Moved Java runtime source tree in /afs/gr/users/mallet/Adl/adl2/src/ORG/opengroup/adl/runtime.
//
// Revision 1.12  1998/05/12 14:22:46  shane
// Added code to hide certain output unless debugMode is enabled.
// Also added debugMode to the testContext.
//
// Revision 1.11  1998/03/10 17:27:29  mallet
// Merged v1.1 with development trunk
//
// Revision 1.10.2.1  1998/03/04 15:53:39  shane
// Made changes for NT building and for compatibility with updated JETPack.
//
// Revision 1.10  1998/02/27 10:08:32  mallet
// Updated copyright notice.
//
// Revision 1.9  1998/02/12 16:04:36  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.8  1998/01/08 14:40:45  mallet
// Updated and fixed javadoc comments.
//
// Revision 1.7  1997/12/23 09:11:48  mallet
// Fixed javadoc comments.
//
// Revision 1.6  1997/11/27 08:47:01  adl2ext
// Fixed some bugs on new package names.
//
// Revision 1.5  1997/11/26 13:59:47  mallet
// Fixed runtime package name: now ORG.opengroup.adl.runtime.
//
// Revision 1.4  1997/10/15 08:58:19  mallet
// Fixed an off-by-one bug in tp_run: TET IC number starts from 1 whereas
// dataset instance number starts from 0 (using 0 as a min_ic seems to be
// causing strange problems within TET, hence the current workaround).
//
// Revision 1.3  1997/10/10 20:26:36  shane
// Added blank constructor.
// Added main that calls the BaseTestCase main.
// Added documentation for various methods.
// Fixed to compile cleanly.
//
// Revision 1.2  1997/10/08 12:35:42  mallet
// Interim version to be completed.
// Cosmetic changes to add standard header and footer.
//
// ***********************************************************************
