//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/runtime/ORBWrapper.java,v $ 
// $Revision: 1.4 $
// $Date: 1998/08/06 14:09:41 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   orbwrapper
// NAME:        ORBWrapper.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Jan 15,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.runtime;

import java.lang.reflect.*;
import java.util.Properties;
import java.applet.Applet;

/**
 * Abstract class supplying the interface for all CORBA ORBs ADL wrappers.
 * <p>ORB wrappers provide a subset of the methods of the ORB CORBA class
 * enabling to initialize the ORB and to get an initial object reference.
 * This subset is both for client side and server side, but note that
 * an ADL test suite may have to use only the client side methods, for
 * example if the CORBA server is on a distant machine on the network
 * and implemented without using Java but a different language.
 * <p><b>Why:</b> 
 * <p>CORBA test suites portability is increased by using a portable ORB
 * wrapper. The wrapped ORB is selected using the "org.opengroup.adl.orbclass"
 * system property which gives the fully qualified name of the real ORB wrapper.
 * <p>The reflection API is used to instanciate the real ORB wrapper. Hence,
 * it is not necessary to have all the real ORB wrapper compiled, only
 * that reference by the "org.opengroup.adl.orbclass" property shall be
 * compiled and available at runtime.
 * 
 */

public abstract class ORBWrapper
{
   /**
    * Create a new ORB instance for an application.  
    * @param args application strings from main() method.
    * @param props application-specific properties.
    * @return reference to the real ORB implementation.
    */
    public static ORBWrapper init(String args[], Properties props)
    {
      try 
      {
         String className = System.getProperty
                            ("adl.orbwrapperclass",
                             "SunORBWrapper");

         Class  wrapperClass = Class.forName(className);

         ORBWrapper realORBWrapper = (ORBWrapper)wrapperClass.newInstance();

         return realORBWrapper._init(args,props); 

      }
      catch ( ClassNotFoundException e ) 
      {
         throw new Error("ClassNotFoundException. " + e.getMessage());
      }
      catch ( InstantiationException e )
      {
         throw new Error("InstantiationException. " + e.getMessage());
      }
      catch ( IllegalAccessException e )
      {
         throw new Error("IllegalAccessException. " + e.getMessage());
      } 
   }

   /**
    * Create a new ORB instance for an applet.  
    * This method is not static so that it can be overrided in the real 
    * wrapper subclass.
    * @param app the applet.
    * @param props applet-specific properties.
    * @return reference to the real ORB implementation.
    */
   public static ORBWrapper init(Applet app, Properties props)
   {
      try 
      {
         String className          = System.getProperty
                                     ("org.opengroup.adl.orbwrapperclass",
                                      "SunORBWrapper");

         Class  wrapperClass       = Class.forName(className);

         ORBWrapper realORBWrapper = (ORBWrapper)wrapperClass.newInstance();

         return realORBWrapper._init(app,props); 
      }
      catch ( ClassNotFoundException e ) 
      {
         throw new Error("ClassNotFoundException.");
      }
      catch ( InstantiationException e )
      {
         throw new Error("InstantiationException.");
      }
      catch ( IllegalAccessException e )
      {
         throw new Error("IllegalAccessException.");
      } 
   }

   /**
    * Create a new ORB instance for an application.  
    * This method is not static so that it can be overrided in the real 
    * wrapper subclass.
    * @param args application strings from main() method.
    * @param props application-specific properties.
    * @return reference to the real ORB implementation.
    */
   abstract protected ORBWrapper _init(String args[], Properties props);

   /**
    * Create a new ORB instance for an applet.  
    * This method is not static so that it can be overrided in the real 
    * wrapper subclass.
    * @param app the applet.
    * @param props applet-specific properties.
    * @return reference to the real ORB implementation.
    */
   abstract protected ORBWrapper _init(Applet app, Properties props);

   /**
    * Convert a string to a CORBA object reference. 
    * @param str the string.
    * @return the object reference. 
    */
   public abstract Object string_to_object(String str);

   /**
    * Convert a COBRA object reference to a string. 
    * @param the object reference to stringify. 
    * @return str the string.
    */
   public abstract String object_to_string(Object obj);

   /**
    * All objects created by the user become ORB objects only after connect is
    * called. 
    * @param skeleton the object to connect (actual typecast to CORBA skeleton
    *                 type is let to the real ORBWrapper concrete subclass.
    */
   public abstract void connect(Object skeleton);

   /**
    * To stop an ORB object from receiving remote invocations disconnect needs
    * to be called.  
    * @param skeleton the object to disconnect (actual typecast to CORBA 
    *                 skeleton type is let to the real ORBWrapper concrete 
    *                 subclass.
    */
   public abstract void disconnect(Object skeleton);

   /**
    * Unit test.
    */
   public static void main(String[] args)
   {
      ORBWrapper adlORB = ORBWrapper.init(args,null);
   }
}

// ***********************************************************************
// History Log:
// $Log: ORBWrapper.java,v $
// Revision 1.4  1998/08/06 14:09:41  mallet
// Moved Java runtime source tree in /afs/gr/users/mallet/Adl/adl2/src/ORG/opengroup/adl/runtime.
//
// Revision 1.3  1998/02/27 10:08:33  mallet
// Updated copyright notice.
//
// Revision 1.2  1998/02/17 10:57:44  mallet
// Merged v1.0 with development trunk
//
// Revision 1.1.2.1  1998/02/16 20:21:44  cinard
// Moved ADLWrapper in java runtime package.
// Modified Makefile accordingly.
//
// ***********************************************************************
