// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/ADLCompilationUnitInfo.java,v $ 
// $Revision: 1.3 $
// $Date: 1998/07/28 09:28:54 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        ADLCompilationUnitInfo.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Mar 16,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;

/**
 * Information used when compiling an ADL Compilation Unit.
 */

public class  ADLCompilationUnitInfo
{

  /**
   * Constructs ADL Compilation unit context information.
   * This information shall be used to manipulate symbols
   * of a given compilation unit.
   * @param usedADLClasses  list of used ADL classes.
   * @param usedTDDClasses  list of used TDD classes.
   */

  public ADLCompilationUnitInfo(Vector usedADLClasses,
				Vector usedTDDClasses)
  {
     _usedADLClasses = usedADLClasses;
     _usedTDDClasses = usedTDDClasses;
  }
  
  /**
   * Used ADL Classes.
   */
  
  public Vector getUsedADLClasses()
  {
     return _usedADLClasses;
  }

  /**
   * Used TDD Classes.
   */
  
  public Vector getUsedTDDClasses()
  {
     return _usedTDDClasses;
  }

  /**
   * Get a description.
   */

  public String toString()
  {
     String s = "usedADLClasses = ";
     Enumeration e = _usedADLClasses.elements();
     while ( e.hasMoreElements() )
     {
        s += "[" + ((ADLClass)e.nextElement()).getName() + "]"; 
     }
     s += ", usedTDDClasses = ";
     e = _usedTDDClasses.elements();
     while ( e.hasMoreElements() )
     {
        s += "[" + ((TDDClass)e.nextElement()).getName() + "]"; 
     }
     return s;
  }

  // -- Private --

  /** Used ADL Classes. */
  private final Vector _usedADLClasses;

  /** Used TDD Classes. */
  private final Vector _usedTDDClasses;

}

// ***********************************************************************
// History Log:
// $Log: ADLCompilationUnitInfo.java,v $
// Revision 1.3  1998/07/28 09:28:54  mallet
// Added list of used ADL classes.
//
// Revision 1.2  1998/05/27 09:54:54  cinard
// Added a toString() method to facilitate debugging.
//
// Revision 1.1  1998/03/18 15:52:57  cinard
// Implemented support for symbol table context (mainly
// used to keep context information when parsing and
// typecheck are separated).
//
// ***********************************************************************
