// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/ADLFileChecker.java,v $
// $Revision: 1.3 $
// $Date: 1998/07/01 12:15:05 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   jbcfile
// NAME:        ADLFileChecker.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Nov 25,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.io.*;
import java.util.*;

/**
 * Checks for existence of TDD files. Useful when "use" clause are
 * encountered in TDD files.
 */

public class ADLFileChecker
{

  /** Constructs teh file checker. 
   * <p> Part of the construction is to extract the "adl.incpath" property
   *     from the environement.
   */

  public ADLFileChecker()
  {

     // Get incPath, for convenience the INCPATH env variable 
     // is split and its components are put into incPath vector.

     String tmp = System.getProperty("adl.incpath",".");
     if ( tmp == null ) { tmp = "."; }
     StringTokenizer st = new StringTokenizer(tmp, File.pathSeparator);
     while (st.hasMoreTokens())
     {
         incPath.addElement(st.nextToken());
     }

  }

  /**
   * Returns full path name of the source file defining an ADL class if
   * this file exists and can be reached from incpath.
   * @param name name of the adl class.
   * @return full path name if file is found, null otherwise.
   */ 

  public String getFileNameForADLClass(String name)
  {
     return getPath(name + ".adl");
  }

  /**
    * Returns full path name of the source file defining a TDD class if
    * this file exists iand can be reached from incpath.
    * @param name name of the tdd class.
    * @return full path name if file is found, null otherwise.
    */ 

  public String getFileNameForTDDClass(String name)
  {
     return getPath(name + ".tdd" );
  }

  // -- Private methods -- 

  /**
   * Get full path of specified file, in case it's found and readable
   * from incpath.
   * @param name name of the source file.
   * @return the path if file is found, null otherzise.
   */ 

  private String getPath(String name)
  {
     String path = null;
     Enumeration e = incPath.elements();
     while (e.hasMoreElements())
     {
         String includeDirectory = (String)e.nextElement();
         File file = new File(includeDirectory,name);
         if ( file.canRead() )
         {
            path = file.getPath();
            break;
         }
     }
     return path;
  }

  // -- Private attributes --

  /** Stores directories specified in "adl.incpath" property. */
  private Vector incPath = new Vector();

  /** Debug flag. */
  private static final boolean debug = false;

  // -- Unit testing --

  /** 
   * Just check supplied TDD file names can be accessed to.
   * @param args names of the TDD files for which check is to be performed.
   */
  
  public static final void main(String[] args)
  {
     ADLFileChecker fc = new ADLFileChecker();

     // Display incpath
     Enumeration e = fc.incPath.elements();
     while (e.hasMoreElements())
     {
         String path = (String)e.nextElement();
         System.out.println("incpath += " + path);
     }

     // Check if file supplied in args are accessible
     for (int i = 0; i < args.length; i++)
     {
        System.out.println
        (
           "File name for tdd class " + args[i] + " : " 
           + ( fc.getFileNameForTDDClass(args[i]) )
        );
     }
  }

}

// ***********************************************************************
// History Log:
// $Log: ADLFileChecker.java,v $
// Revision 1.3  1998/07/01 12:15:05  mallet
// Fixed bug in getFileNameFor[TDD|ADL]Class: a tdd or adl class name
// never includes a .tdd or .adl suffix.
//
// Revision 1.2  1998/05/27 09:49:13  cinard
// File name can now be supplied with their adl extensions or not.
//
// Revision 1.1  1998/03/19 15:35:05  cinard
// Simplified ADL file check mechanism.
//
// ***********************************************************************
