// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/ADLMethod.java,v $
// $Revision: 1.11 $
// $Date: 1998/03/25 14:50:26 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        ADLMethod.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.predicates.UnaryAnd;

/**
 * ADL method symbol.
 */

public class ADLMethod extends LCallable
{
  // -- Constructor --

  /**
   * Constructs an ADL method symbol. Used to specifiy
   * semantic for target language methods.
   */

  public ADLMethod()
  {
     this("",null);
  }

  /**
   * Constructs an ADL method symbol. Used to specifiy
   * semantic for target language methods.
   * @param name Name of the method.
   * @param modifier the modifier flag.
   */

  public ADLMethod(String name,LModifier modifier)
  {
     super(name,modifier,SymbolId.ADL_METHOD);
  }

  /**
   * Set the return type of the ADLMethod.
   * @param type Return type of the callable.
   */

  public void setReturnType(LType type)
  {
     super.setReturnType(type);
     // add an alias for return
     putSymbol(new LParameter("return",new LModifier(0x0000),type));
  }

  /**
   * Is adl method defining semantics for a constructor ?
   */
  public boolean isConstructorSemantics()
  {
     ADLClass adlClass = (ADLClass)getEnclosingConstruct();
     return getName().equals(adlClass.getName());
  }

  // -- Protected methods --

  /** Returns a header for the description of the symbol. */

  protected String header()
  {
     return "";
  }

}

// ***********************************************************************
// History Log:
// $Log: ADLMethod.java,v $
// Revision 1.11  1998/03/25 14:50:26  cinard
// Added isConstructor() method.
//
// Revision 1.10  1998/03/18 15:49:52  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.9  1998/02/27 12:48:21  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.8  1998/02/23 15:08:50  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.7  1998/02/12 16:03:02  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.6  1998/01/19 14:36:58  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.5  1998/01/12 13:19:00  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.4  1997/12/16 09:21:49  cinard
// Added "return" symbol in ADLMethod.
//
// Revision 1.3  1997/12/08 12:58:40  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.2  1997/11/26 17:12:53  cinard
// Changed package name.
//
// Revision 1.1  1997/11/19 15:46:54  cinard
// New symbol and comparator classes.
//
// ***********************************************************************
