// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LArrayImpl.java,v $
// $Revision: 1.4 $
// $Date: 1998/04/28 11:55:28 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LArrayImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Implementation for target language Array Symbol.
 */

public class LArrayImpl extends LTypeImpl
                        implements LArray
{

  // -- Protected constructors --

  /** 
   * Constructs me to be an array type.
   */

  public LArrayImpl()
  {
     super("",new LModifier(0x0000),SymbolId.ARRAY);
     this.elementType = null;
  }

  /** 
   * Constructs me to be an array type.
   * @param elememtType type of the elements in the array.
   */

  public LArrayImpl(LType elementType)
  {
     super("" , new LModifier(0x0000), SymbolId.ARRAY);
     this.elementType = elementType;
  }

  // -- Public methods --

  // -- Symbol interface --

  public String getName()
  {
     return ( elementType != null )
            ? ( elementType.getName() + "[]" )
            : "<no name>[]";
  }

  // -- Scope management --
 
  /**
   * Returns true if the symbol has access to a scope.
   * @return always false for an array.
   */
  public boolean hasScopeAccess()
  {
     return false;
  }

  /**
   * Should not be called.
   */
  public Scope   getScope()
  {
     throw new Error("LArrayImpl.getScope() should not be called.");
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.ARRAY_SELECT;
  }

  // -- Type interface implementation --

  /**
   * Check if a value of type t be assigned to a variable of this type.
   * @param t type for which the check is performed.
   * @return true if it can be assigned, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
        return    ( t instanceof LArray )
               && this.getElementType().assignable
                                        (((LArray)(t)).getElementType());
  }

  /**
   * Get the type of the elements in this array.
   * @return The element's type.
   */
  public LType getElementType()
  {
     return elementType;
  }
  
  /**
   * Set type of the array's elements.
   * @param elememtType type of the elements in the array.
   */

  public void setElementType(LType elementType)
  {
     this.elementType = elementType;
  }
  

  public int getArrayLevel()
  {

     if ((elementType == null) || ! (elementType instanceof LArray)) 
     {
        return 1;
     }
     else 
     {
        return 1 + ((LArray)elementType).getArrayLevel();
     }
  }

  /**
   * Returns a description of the symbol, indented.
   * @param indent Indentation string.
   * @return Description of the symbol.
   */

  public String toString(String indent)
  {
     return indent + getName();
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LArrayTypedef(name,this);
  }
	  

  // -- Private feature --

  /** Type of the elements stored in the array. */
  private LType elementType;


  /** Debug flag */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LArrayImpl.java,v $
// Revision 1.4  1998/04/28 11:55:28  cinard
// Element type can be null (useful when defining c or cpp types with
// complex nested definitions - e.g. typedef int ( * foo [10] ) int p )
// foo is an array to pointer, but base type is known only after the
// whole expressoin has been parsed).
//
// Revision 1.3  1998/03/18 15:49:53  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:48:23  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:50  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
