// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LAttribute.java,v $
// $Revision: 1.9 $
// $Date: 1998/03/18 15:49:53 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LAttribute.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;


/**
 * LAttribute symbol.
 */

public class LAttribute extends LExpression
{

  /**
   * Constructs me to be an attribute.
   * @param name name of the attribute.
   * @param modifier modifier flag for the attribute.
   * @param type type of the attribute.
   */

  public LAttribute(String name,LModifier modifier,LType type)
  {
     super(name,modifier,type,SymbolId.ATTRIBUTE);
     if (debug)
     {
        System.out.println(  "In LAttribute.LAttribute, modifier = "
                          +  modifier
                          );
     }
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.ATTRIBUTE_SELECT;
  }

  /**
   * Returns a header for the description of the expression.
   */
 
  public String header() { return "" ; }

  /** Debug flag */
  private static boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LAttribute.java,v $
// Revision 1.9  1998/03/18 15:49:53  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.8  1998/02/27 12:48:23  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.7  1998/02/23 15:08:51  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.6  1998/02/12 16:03:03  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/12 13:19:02  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.4  1997/11/26 17:12:55  cinard
// Changed package name.
//
// Revision 1.3  1997/11/12 13:43:11  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.2  1997/11/03 12:59:56  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:02  cinard
// Added language independant symbols to the symboltable package.
//
//
//
// ***********************************************************************
