// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LBuiltinTypeImpl.java,v $
// $Revision: 1.3 $
// $Date: 1998/03/18 15:49:53 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LBuiltinTypeImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

import java.util.Hashtable;
import java.util.Enumeration;

/**
 * Implementation for builtin type.
 */

abstract public class LBuiltinTypeImpl extends    LTypeImpl
                                       implements LBuiltinType
{
  /**
   * Constructs me to be a builtin type.
   * @param name name of the type.
   */

  protected LBuiltinTypeImpl(String name)
  {
     super(name,new LModifier(0x0000),SymbolId.BUILTIN_TYPE);
  }

  // -- Symbol interface implementation --

  /**
   * Check if symbol has direct or indirect access to a scope. That is never
   * true for a simple type.
   * @return Always false.
   */

  public boolean hasScopeAccess()
  {
     return false;
  }

  /**
   * Get the scope the symbol has access to.
   * @return Always throws an error.
   */

  public Scope   getScope()
  {
     throw new Error("LBuiltinTypeImpl.getScope() should not be called.");
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.BUILTIN_TYPE_SELECT;
  }

  // -- Specific methods --

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  static public boolean isValidBuiltinTypeName(String name)
  {
     return builtinTypes.containsKey(name);
  }

  /**
   * Returns a string description of the symbol, indented version.
   * @param indent Indentation.
   * @return Description of the symbol.
   */

  public String toString(String indent)
  {
     return getName();
  }

  // -- LType interface --

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LBuiltinTypedef(name,this);
  }

  // -- Protected methods --

  /**
   * Get a builtin type.
   * @param name name of the builtin type.
   * @return the found builtin type (null if no type is found)
   */

  public static LBuiltinType getBuiltinType(String name)
  {
     return (LBuiltinType) builtinTypes.get(name);
  }

  /**
   * Get an enumeration of all the builtin types.
   * @return enumeration of the builtin types.
   */

  protected static Enumeration getBuiltinTypes()
  {
     return builtinTypes.elements();
  }


  // -- Protected Fields --

  /** Container for the builtin types. */

  protected static final Hashtable builtinTypes = new Hashtable();

}

// ***********************************************************************
// History Log:
// $Log: LBuiltinTypeImpl.java,v $
// Revision 1.3  1998/03/18 15:49:53  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:48:24  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:52  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
