// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LCallableAssignComparator.java,v $
// $Revision: 1.11 $
// $Date: 1998/04/28 12:01:42 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LCallableAssignComparator.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;

/**
 * Predicate enabling to check that a callable is assignable
 * to a callable expression with a supplied signature.
 */ 

public class LCallableAssignComparator extends SymbolSelector
{

  /**
   * Constructs the operation selector.
   */ 
  public LCallableAssignComparator()
  {
     super(SymbolId.OPERATION_MASK);
  }

  /**
   * Constructs the operation selector.
   * @param symbolIdMask bit mask enabling to refine symbol selection.
   */ 
  public LCallableAssignComparator(long symbolIdMask)
  {
     super(SymbolId.OPERATION_MASK & symbolIdMask);
  }

  /**
   * Add a parameter type to the comparator.
   * @param type parameter's type.
   */
  
  public void addParameterType(LType type)
  {
     parameterTypes.addElement(type);
  }

  /**
   * Get all parameter's types.
   * @return parameter's types.
   */

  public Vector getParameterTypes()
  {
     return parameterTypes;
  }

  /**
   * Returns true if executed against an operation symbol whose signature
   * is compatible with that of the comparator signature, false otherwise.
   */

  public boolean execute(Object object)
  {
     boolean ok = super.execute(object) ;

     if ( ok )
     {
        LCallable lc = (LCallable) object;
        LParameter[] parameters = lc.getParameters();

        if  (    ( getParameterTypes().size() == parameters.length )
              || (  lc.lastParameterIsEllipsis()
                    && getParameterTypes().size() >= parameters.length )
            )
        {
           Vector parameterTypes = getParameterTypes();
           
           for (int i = 0; i < parameters.length; i++)
           {
              LType fnParamT = parameters[i].getType();
              LType clParamT = (LType)(parameterTypes.elementAt(i));
              if ( ! fnParamT.assignable(clParamT) )
              {
                 ok = false;
                 break;
              }
           }
        }
        else
        {
           ok = false;
        }
     }

     return ok;
  }

  /** 
   * Returns a string description of the comparator.
   */ 

  public String toString()
  {
     String s = super.toString();

     // Parameters

     s += "<";
     Enumeration e = parameterTypes.elements();
     if ( e.hasMoreElements() )
     {
        LType t = (LType)e.nextElement();
        s +=  ( ( t != null ) ? t.getName() : "<no type>" );
     }
     while ( e.hasMoreElements() )
     {
        LType t = (LType)e.nextElement();
        s += ", " + ( ( t != null ) ?  t.getName() : "<no type>" );
     }
     s += ">";

     return s;
  }

  // -- Private fields --

  /** Parameters of the callable. */
  Vector parameterTypes = new Vector();

}

// ***********************************************************************
// History Log:
// $Log: LCallableAssignComparator.java,v $
// Revision 1.11  1998/04/28 12:01:42  cinard
// Implemented support for ellipsis in parameters.
//
// Revision 1.10  1998/03/19 11:15:14  cinard
// Improved debug trace.
//
// Revision 1.9  1998/03/18 15:49:51  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.8  1998/02/27 12:48:26  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.7  1998/02/12 16:03:04  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.6  1998/01/19 14:36:59  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.5  1998/01/12 13:19:03  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.4  1997/12/08 12:58:41  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.3  1997/11/26 17:12:56  cinard
// Changed package name.
//
// Revision 1.2  1997/11/19 15:48:58  cinard
// Corrected some minor bugs.
//
// Revision 1.1  1997/11/12 13:43:13  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// ***********************************************************************
