// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LCallableEqualsComparator.java,v $
// $Revision: 1.14 $
// $Date: 1998/06/15 11:56:41 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LCallableEqualsComparator.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;
import com.objectspace.jgl.Array;
import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.adapters.ObjectArray;
import com.objectspace.jgl.algorithms.Comparing;
import com.objectspace.jgl.algorithms.Copying;



/**
 * Predicate enabling to check that a callable is equals
 * to a callable expression with a supplied signature.
 */ 

public class LCallableEqualsComparator extends SymbolSelector
{

  /**
   * Constructs the operation selector.
   */ 

  public LCallableEqualsComparator()
  {
     super(SymbolId.OPERATION_MASK);
  }

  /**
   * Constructs the operation selector.
   * @param symbolIdMask bit mask enabling to refine symbol selection.
   */ 

  public LCallableEqualsComparator(long symbolIdMask)
  {
     super(SymbolId.OPERATION_MASK & symbolIdMask);
  }

  /**
   * Set predicate return type.
   * @param type return type.
   */

  public void setReturnType(LType type)
  {
     returnType = type;
  }

  /**
   * Get predicate return type.
   * @return return type.
   */

  public LType getReturnType()
  {
     return returnType;
  }

  
  /**
   * Add a parameter type to the comparator.
   * @param type parameter's type.
   */
  
  public void addParameterType(LType type)
  {
     parameterTypes.add(type);
  }

  /**
   * Get all parameter's types.
   * @return parameter's types.
   */

  public Array getParameterTypes()
  {
     return parameterTypes;
  }

  /**
   * Indicate the operation has an ellipsis as its last parameter.
   */

  public void setLastParameterIsEllipsis()
  {
     _lastParameterIsEllipsis = true;
  }

  /**
   * Is last parameter an ellipsis?
   */

  public boolean lastParameterIsEllipsis()
  {
     return _lastParameterIsEllipsis;
  }

  /**
   * Add an exception to the comparator.
   * @param type exception.
   */

  public void addException(LType exc)
  {
     exceptions.add(exc);
  }

  /**
   * Get all exceptions.
   */

  public Array getExceptions()
  {
     Array _exceptions = new Array();
     Copying.copy(exceptions,_exceptions);
     return _exceptions;
  }

  /**
   * Is symbol t equals this symbol?
   */
  public boolean equals( Object t )
  {
     if ( (t == null) || ! (t instanceof LCallableEqualsComparator) )
     {
        return false ;
     }
     LCallableEqualsComparator lc = (LCallableEqualsComparator) t;
     Array       _parameterTypes = lc.getParameterTypes();
     HashSet     _exceptions = new HashSet();
     Copying.copy(new ObjectArray(lc.getExceptions()),_exceptions);
     
     return      getReturnType().equals(lc.getReturnType())
              && Comparing.equal(parameterTypes,_parameterTypes)
              && (lastParameterIsEllipsis() == lc.lastParameterIsEllipsis())
              && Comparing.equal(exceptions,_exceptions);

  }

  /**
   * Returns true if executed against an operation symbol whose signature
   * is equals with that of the comparator signature, false otherwise.
   */

  public boolean execute(Object object)
  {
     if ( ! super.execute(object) )
     {
        return false;
     }
     else 
     {
        LCallable lc = (LCallable) object;
        ObjectArray _parameters = new ObjectArray(lc.getParameters());
        Array       _parameterTypes = new Array();
        Enumeration e  = _parameters.elements();
        while ( e.hasMoreElements() )
        {
           _parameterTypes.add(((LParameter)e.nextElement()).getType());
        }
	HashSet     _exceptions = new HashSet();
	Copying.copy(new ObjectArray(lc.getExceptions()),_exceptions);

        return      getReturnType().equals(lc.getReturnType())
                 && Comparing.equal(parameterTypes,_parameterTypes)
                 && (lastParameterIsEllipsis() == lc.lastParameterIsEllipsis())
                 && Comparing.equal(exceptions,_exceptions);
     }
  }

  /** 
   * Returns a string describing me.
   */ 

  public String toString()
  {
     return   super.toString() + " "
            + returnType + " "
            + parameterTypes + " "
            + lastParameterIsEllipsis() + " " 
            + exceptions;
  }

  /**
   * Returns a string describing me.
   * @param modifier optional modifier that may be used to describes
   *                 the LCallableEqualsComparator object as a pointer to
   *                 a callable (function).
   */

  public String toString(String modifier)
  {
     // Parameters

     String sParameters = "(";
     Enumeration e = parameterTypes.elements();
     if ( e.hasMoreElements() )
     {
        sParameters +=  e.nextElement();
     }
     while ( e.hasMoreElements() )
     {
        sParameters += ", " + e.nextElement();
     }

     if ( lastParameterIsEllipsis() )
     {
        if ( ! sParameters.equals("(") ) { sParameters += ", "; }
        sParameters += "...";
     }

     sParameters += ")";

     String s =    returnType + " "
                 +  ( modifier != null ? modifier : " " )
                 + sParameters;
     return s;
  }

  // -- Protected fields --

  /** Return type of the callable. */
  protected LType returnType;

  /** Parameters of the callable. */
  protected Array parameterTypes = new Array();

  /** Exceptions */
  protected HashSet exceptions = new HashSet();

  /** Is last parameter an ellipsis? */
  private boolean _lastParameterIsEllipsis = false;

}

// ***********************************************************************
// History Log:
// $Log: LCallableEqualsComparator.java,v $
// Revision 1.14  1998/06/15 11:56:41  cinard
// Fixed bug : throw clause is not order sensitive anymore.
//
// Revision 1.13  1998/04/28 12:01:43  cinard
// Implemented support for ellipsis in parameters.
//
// Revision 1.12  1998/03/18 15:49:51  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.11  1998/02/27 12:48:26  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.10  1998/02/23 15:08:53  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.9  1998/02/12 16:03:05  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.8  1998/02/05 10:56:21  wassel
// Added Equals method.
//
// Revision 1.7  1998/01/19 14:37:00  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.6  1998/01/12 13:19:03  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.5  1998/01/06 14:56:56  cinard
// Changed exception type in method signatures to LType (it was LClass before,
// which is not suitable for C++).
//
// Revision 1.4  1998/01/05 13:38:43  cinard
// Modified the execute methods (algorithm simplified by using containers
// with equals methods).
//
// Revision 1.3  1997/12/19 21:16:35  cinard
// Added support for exceptions in throws clauses of methods.
// LCallable and LCallableEqualsComparator now contains a addException()
// method.
//
// Revision 1.2  1997/11/26 17:12:57  cinard
// Changed package name.
//
// Revision 1.1  1997/11/12 13:43:13  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// ***********************************************************************
