// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LClass.java,v $ 
// $Revision: 1.23 $ 
// $Date: 1998/06/29 12:30:04 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LClass.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;

import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.UnaryPredicate;


/**
 * Interface for target language Class symbol.
 */

public interface LClass extends LConstructedType
{

  /** 
   * Add a superclass.
   * @param superClass The super class to add.
   */

  public void addSuperClass(LClass superClass);

  /** 
   * Add implemented interfaces.
   * @param lInterface The interface to add.
   */

  public void addInterface(LClass lInterface);

  /** 
   * Add a friend class.
   * @param aFriend friend class to add.
   */

  public void addFriend(LClass aFriend);

  /**
   * Returns the superclasses of this class.
   * @return Array containing this class superclasses.
   */

  public LClass[] getSuperClasses();

  /**
   * Returns the number of superclasses of this class.
   * @return number of superclasses.
   */

  public int getSuperClassesCount();

  /**
   * Returns the interfaces this class implements.
   * @return Array containing this class interfaces
   */

  public LClass[] getInterfaces();

  /**
   * Check the class defined by this symbol is a subclass of the
   * class defined by the supplied symbol.
   * @param jType Type symbol.
   * @return True if it is, false otherwise.
   */

  public boolean subclassOf(LType t);

  /**
   * Returns names of all methods in this class or its superclasses.
   */

  public HashSet getMethodNames();

  /**
   * Get methods satisfying predicates in this class and all its 
   * superclasses.
   * @param name The name of the expected symbol.
   * @param predicate An arbitrary predicate object.
   * @return The methods.
   */

  public Vector getMethods(String name, UnaryPredicate predicate);

  /**
   * Get symbol's enclosing construct (e.g. a package)
   */

  public Symbol getEnclosingConstruct();


}

// ***********************************************************************
// History Log:
// $Log: LClass.java,v $
// Revision 1.23  1998/06/29 12:30:04  cinard
// Added method getEnclosingConstruct() to LClass interface and
// updated related implementations.
//
// Revision 1.22  1998/05/15 13:25:37  bareau
// Added support for NL unparsing (mainly added a method getNLName() in Scope).
//
// Revision 1.21  1998/03/18 15:48:01  cinard
// Removal of ambiguities on operation invocation.
//
// Revision 1.20  1998/02/27 12:48:26  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.19  1998/02/23 15:08:53  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.18  1998/02/12 16:03:05  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.17  1998/02/05 10:58:45  wassel
// Added synthesisType method.
//
// Revision 1.16  1998/01/23 16:49:35  cinard
// Improved symbol lookup for "this" & <className> symbols in both LClass and
// ADLClass symbols.
//
// Revision 1.15  1998/01/22 14:23:58  cinard
// Disabled ambiguity checks when retrieving a LCallable symbols and
// several symbols satisfies the request.
//
// Revision 1.14  1998/01/20 12:58:17  cinard
// Fixed a bug in subclassOf() method. False was returned for the class
// itself.
//
// Revision 1.13  1998/01/19 14:37:00  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.12  1998/01/12 13:19:04  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.11  1997/12/19 21:16:35  cinard
// Added support for exceptions in throws clauses of methods.
// LCallable and LCallableEqualsComparator now contains a addException()
// method.
//
// Revision 1.10  1997/12/08 12:58:41  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.9  1997/11/27 08:28:58  cinard
// Fixed some bugs in Makefile.
//
// Revision 1.8  1997/11/26 17:12:57  cinard
// Changed package name.
//
// Revision 1.7  1997/11/19 15:48:59  cinard
// Corrected some minor bugs.
//
// Revision 1.6  1997/11/14 09:58:53  cinard
// LModule scope now accepts duplicate symbols.
// SymbolTableMgr has a getADLClass() method.
//
// Revision 1.5  1997/11/12 13:43:14  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.4  1997/11/04 16:04:21  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.3  1997/11/04 09:04:28  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symbol_table.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.2  1997/11/03 12:59:57  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:03  cinard
// Added language independant symbols to the symboltable package.
//
// 
// ***********************************************************************
