// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LClassImpl.java,v $ 
// $Revision: 1.8 $ 
// $Date: 1998/06/29 12:30:05 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LClassImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 17,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;

import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.util.ConditionalEnumeration;

/**
 * Implementation for target language Class symbol.
 */

public class LClassImpl extends    LConstructedTypeImpl implements LClass
{

  /**
   * Constructs me to be a target language class.
   * @param name name of the target language class.
   * @param modifier modifier bit mask
   */
   
  public LClassImpl(String name,LModifier modifier)
  {
     super(name,modifier,SymbolId.CLASS);
  
     superClasses = new Vector();
     interfaces   = new Vector();
     friends      = new Vector();
  }

 /**
  * Constructs me to be a target language class.
  * @param name name of the target language class.
  * @param modifier modifier bit mask
  * @param symbolId symbol's identifier
  */
   
  public LClassImpl(String name,LModifier modifier,long symbolId)
  {
     super(name,modifier,symbolId);
  
     superClasses = new Vector();
     interfaces   = new Vector();
     friends      = new Vector();
  }

  // -- Public methods --

  /**
   * Get a symbol in the scope, extended version.
   * When using this version of getSymbol, supplementary parameters are
   * provided that shall help finding the object.
   * <pre>
   * The symbol shall satisfy some criteria defined by the provided 
   * predicate object. This predicate object help finding the symbol in 
   * case several symbols are likely to match the supplied name 
   * (e.g. overloaded methods).
   * </pre>
   * <pre>
   * Credentials shall be supplied for those cases where some symbols
   * hava non-public visibility. Retrieving them may require the
   * use of credentials.
   * </pre>
   * @param name The name of the expected symbol.
   * @param predicate An arbitrary predicate object.
   * @param credentials Credentials of the scope.
   * @return The found symbol.
   */

  public Symbol getSymbol
         (String name,UnaryPredicate predicate,Object credentials)
  {

     Symbol s = null;

     if ( debug )
     {
        System.out.println("debug: LClassImpl.getSymbol("
                          + name + ", "
                          + predicate + ", "
                          + credentials + ")");
     }

     // Check that the requested symbol is not the class itself
     // or keyword this.
     if ( name.equals(getName()) )
     {
        // It is, return it if verifies the predicate.
        if ( predicate != null && predicate.execute(this) )
        {
           return this;
        }
     }


     // A special processing is needed for methods to remove
     // ambiguous calls.

     if (    predicate instanceof LCallableAssignComparator
          && ((LCallableAssignComparator)predicate).canSelect(SymbolId.METHOD)
        )
     {

        // Special processing for methods
        Vector methods = getMethods(name,predicate);

        if ( debug )
        {
           System.out.println("debug: LClassImpl.getSymbol() ... "
                             + " number of methods = " + methods.size());
        }

        if ( methods.size() > 1 )
        {
           s = removeAmbiguityOnOperationInvocation(methods,predicate);
        }
        else if ( methods.size() == 1 )
        {
           s = (Symbol)methods.elementAt(0);
        }
        
     }
     else
     {
        // Search in class fields
        
        s = super.getSymbol(name,predicate,credentials);

        // Check superclasses in case no symbol was found.
        // And credentials not set to Boolean.TRUE 

        if ( s == null && ! ( Boolean.TRUE.equals(credentials) ) )
        {
           Vector suitables = new Vector();
           
           Enumeration e = superClasses.elements();
           while ( e.hasMoreElements() && suitables.size() < 2 )
           {
              LClass sc = (LClass)e.nextElement();
              s = sc.getSymbol(name,predicate,credentials);
              if ( s != null )
              {
                 suitables.addElement(s);
              }
           }
           
           // Check that no more than 1 symbols were found.
           // Note the check is deactivated for operations (LCallable),
           // this behaviour should be improved.
           
           if ( suitables.size() > 1 )
           {
              // If more than one symbol was found, throw an error.

              String sSuitables = "";
              Enumeration eSuitables = suitables.elements();
              while ( eSuitables.hasMoreElements() )
              {
                 sSuitables += eSuitables.nextElement();
              }
              
              throw new Error
              (   "AmbiguousRequestException"
                  + "Several symbols satisfies your request : " 
                  + sSuitables
              );
           }
        }     
     }

     if ( debug )
     {
        System.out.println("debug: LClassImpl.getSymbol("
                          + name + ", "
                          + predicate + ", "
                          + credentials + ")" + " return " + s);
     }

     // Return found symbol.
     return s;
 
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     if ( isA(SymbolId.STRUCT) )
     {
        return SymbolSelector.STRUCT_SELECT;
     }
     else 
     {
        return SymbolSelector.CLASS_SELECT;
     }
  }
  

  /** 
   * Add a superclass.
   * @param superClass The super class to add.
   */

  public void addSuperClass(LClass superClass)
  {
     superClasses.addElement(superClass);
  }

  /** 
   * Add implemented interfaces.
   * @param lInterface The interface to add.
   */

  public void addInterface(LClass lInterface)
  {
     interfaces.addElement(lInterface);
  }

  /** 
   * Add a friend class.
   * @param aFriend friend class to add.
   */

  public void addFriend(LClass aFriend)
  {
     friends.addElement(aFriend);
  }

  /**
   * Returns the superclasses of this class.
   * @return Array containing this class superclasses.
   */

  public LClass[] getSuperClasses()
  {
     LClass[] ret = new LClass[superClasses.size()];
     superClasses.copyInto(ret);
     return ret;
  }

  /**
   * Returns the number of superclasses of this class.
   * @return number of superclasses.
   */

  public int getSuperClassesCount()
  {
     return superClasses.size();
  }

  /**
   * Returns the interfaces this class implements.
   * @return Array containing this class interfaces
   */

  public LClass[] getInterfaces()
  {
     LClass[] ret = new LClass[interfaces.size()];
     interfaces.copyInto(ret);
     return ret;
  }

  /**
   * Returns synthesis type of the @param t and type for which method applies
   * checks if any of them is subclass of the other and returns the super class
   */

  public LType synthesisType(LType t)
  {
    LType resType;

    if ( ( t == null ) || !( t instanceof LClass) ) 
    {
      return null;
    }
    else if (  this.subclassOf( t) )
    {

      resType = t;

    } 
    else if ((( LClass) t).subclassOf( this ) ) 
    {
      resType = this;
    }
    else 
    {
       resType = null;
    }
    return resType;
  }
 
  /**
   * Check the class defined by this symbol is a subclass of the
   * class defined by the supplied symbol.
   * @param jType Type symbol.
   * @return True if it is, false otherwise.
   */

  public boolean subclassOf(LType t)
  {

     if ( this.equals(t) )
     {
        return true;
     }
     else
     {
        Enumeration e = superClasses.elements();
        while ( e.hasMoreElements() )
        {
           LClass sc = (LClass)e.nextElement();
           
           if ( sc.equals(t) || sc.subclassOf(t) )
           {
              return true;
           } 
        }

        Enumeration i = interfaces.elements();
        while ( i.hasMoreElements() )
        {
           LClass sc = (LClass)i.nextElement();
           
           if ( sc.equals(t) || sc.subclassOf(t) )
           {
              return true;
           } 
        }
     }
     
     return false;
  };

  // -- Other useful methods --

  /**
   * Returns names of all methods in this class or its superclasses.
   */

  public HashSet getMethodNames()
  {
     HashSet names = new HashSet();
     Enumeration e = symbols.elements();
     while ( e.hasMoreElements() )
     {
        Symbol s = (Symbol) e.nextElement();
        if ( s instanceof LMethod )
        {
           names.add(s.getName());
        }
     }

     // Inject symbols from superclasses
     for ( int i = 0; i <  superClasses.size() ; i++ )
     {
        names = names.union
                (((LClass)superClasses.elementAt(i)).getMethodNames());
     } 
     return names;
  }

  // -- Symbol interface implementation --

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LClassTypedef(name,this);
  }
	  
  // -- Protected methods --

  /** Returns a header for the description of the symbol. */

  protected String header()
  {

     // Superclasses

     String sSuperClasses = "";

     Enumeration e1 = superClasses.elements();
     if ( e1.hasMoreElements() )
     {
        LClass c = (LClass)e1.nextElement();
        sSuperClasses += c.getName();
     }
     while ( e1.hasMoreElements() )
     {
        LClass c = (LClass)e1.nextElement();
        sSuperClasses += ", " + c.getName();
     }

     if ( sSuperClasses.length() > 0 ) 
     {
        sSuperClasses = " inherits " + sSuperClasses;
     }

     // Interfaces

     String sInterfaces = "";
     
     Enumeration e = interfaces.elements();
     if ( e.hasMoreElements() )
     {
        LClass i = (LClass)e.nextElement();
        sInterfaces += i.getName();
     }
     while ( e.hasMoreElements() )
     {
        LClass i = (LClass)e.nextElement();
        sInterfaces += ", " + i.getName();
     }

     if ( sInterfaces.length() > 0 ) 
     {
        sInterfaces = " implements " + sInterfaces;
     }
   
     // Friend classes

     String sFriends = "";

     Enumeration e2 = friends.elements();
     if ( e2.hasMoreElements() )
     {
        LClass f = (LClass)e2.nextElement();
        sFriends += f.getName();
     }
     while ( e2.hasMoreElements() )
     {
        LClass f = (LClass)e2.nextElement();
        sFriends += ", " + f.getName();
     }

     if ( sFriends.length() > 0 ) 
     {
        sFriends = " friends " + sFriends;
     }

     String header = isA(SymbolId.STRUCT) 
                   ? "Struct "
                   : "Class ";

     // Header string for the class

     return     header + getName() 
              + " " + sSuperClasses 
              + " " + sInterfaces
              + " " + sFriends;
  }


  /**
   * Get methods satisfying predicates in this class and all its 
   * superclasses.
   * @param name The name of the expected symbol.
   * @param predicate An arbitrary predicate object.
   * @return The methods.
   */

  public Vector getMethods(String name, UnaryPredicate predicate)
  {
     Vector methods = new Vector();

     Enumeration selectedSymbols = symbols.values(name);
      
     // Apply predicate

     selectedSymbols = new ConditionalEnumeration(selectedSymbols,predicate);
     while ( selectedSymbols.hasMoreElements() )
     {
        methods.addElement(selectedSymbols.nextElement());
     }

     // Add interface methods

     Enumeration e = interfaces.elements();
     while ( e.hasMoreElements() )
     {
        LClass theInterface = (LClass)e.nextElement();
        Vector interfaceMethods = theInterface.getMethods(name,predicate);
        Enumeration ee = interfaceMethods.elements();
        while (ee.hasMoreElements() )
        {
           methods.addElement(ee.nextElement());
        }
     }

     // Add superclasses methods

     Enumeration f = superClasses.elements();
     while ( f.hasMoreElements() )
     {
        LClass sc = (LClass)f.nextElement();
        Vector superMethods = sc.getMethods(name,predicate);
        Enumeration ff = superMethods.elements();
        while ( ff.hasMoreElements() )
        {
           methods.addElement(ff.nextElement());
        }
     }

     return methods;

  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     Scope parent = getParent();
     while ( parent != null )
     {
        if (    ( parent instanceof Symbol )
             && (  ((Symbol)parent).isA(SymbolId.CONSTRUCTED_MASK) 
                || ((Symbol)parent).isA(SymbolId.MODULE_MASK) )
           )
        {
           return (Symbol)parent;
        }
        parent = parent.getParent();
     }
     return null;
  }

  // -- Private feature --
 
  /** Super classes. */
  private Vector superClasses;

  /** Interfaces */
  private Vector interfaces;

  /** Friend classes. */
  private Vector friends;

  /** Debug flag. */
  private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: LClassImpl.java,v $
// Revision 1.8  1998/06/29 12:30:05  cinard
// Added method getEnclosingConstruct() to LClass interface and
// updated related implementations.
//
// Revision 1.7  1998/05/15 13:25:37  bareau
// Added support for NL unparsing (mainly added a method getNLName() in Scope).
//
// Revision 1.6  1998/04/28 12:02:28  cinard
// Minor fixes in comments.
//
// Revision 1.5  1998/03/19 15:33:49  cinard
// Fixed a bug : comparator was not correct when class symbol was used as
// a structure.
//
// Revision 1.4  1998/03/18 15:48:01  cinard
// Removal of ambiguities on operation invocation.
//
// Revision 1.3  1998/03/02 15:32:27  cinard
// Replaced LStruct with LClass (struct symbols behave just like classes).
// Factory and find methods (newStruct() and getStruct() remains with
// their signatures updated).
//
// Revision 1.2  1998/02/27 12:48:27  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:53  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
