// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LClassTypedef.java,v $ 
// $Revision: 1.5 $ 
// $Date: 1998/06/29 12:30:05 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LClassTypedef.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 17,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;

import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.UnaryPredicate;

/**
 * Typedef for class.
 */

public class LClassTypedef extends    LConstructedTypedef
                           implements LClass
{

  /**
   * Constructs a typedef over the supplied type.
   * @param name name of the typedef.
   * @param type type for which the typedef is defined.
   */

  protected LClassTypedef(String name,LClass type)
  {
     super(name,type);
  }

  // -- LClass interface --

  /** 
   * Add a superclass.
   * @param superClass The super class to add.
   */

  public void addSuperClass(LClass superClass)
  {
     ((LClass)getRealType()).addSuperClass(superClass);
  }

  /** 
   * Add implemented interfaces.
   * @param lInterface The interface to add.
   */

  public void addInterface(LClass lInterface)
  {
     ((LClass)getRealType()).addInterface(lInterface);
  }


  /** 
   * Add a friend class.
   * @param aFriend friend class to add.
   */

  public void addFriend(LClass aFriend)
  {
     ((LClass)getRealType()).addFriend(aFriend);
  }

  /**
   * Returns the superclasses of this class.
   * @return Array containing this class superclasses.
   */

  public LClass[] getSuperClasses()
  {
     return ((LClass)getRealType()).getSuperClasses();
  }

  /**
   * Returns the number of superclasses of this class.
   * @return number of superclasses.
   */

  public int getSuperClassesCount()
  {
     return ((LClass)getRealType()).getSuperClassesCount();
  }


  /**
   * Returns the interfaces this class implements.
   * @return Array containing this class interfaces
   */

  public LClass[] getInterfaces()
  {
     return ((LClass)getRealType()).getInterfaces();
  }

  /**
   * Check the class defined by this symbol is a subclass of the
   * class defined by the supplied symbol.
   * @param jType Type symbol.
   * @return True if it is, false otherwise.
   */

  public boolean subclassOf(LType t)
  {
     return ((LClass)getRealType()).subclassOf(t);
  }

  /**
   * Returns names of all methods in this class or its superclasses.
   */

  public HashSet getMethodNames()
  {
     return ((LClass)getRealType()).getMethodNames();
  }

  /**
   * Get methods satisfying predicates in this class and all its 
   * superclasses.
   * @param name The name of the expected symbol.
   * @param predicate An arbitrary predicate object.
   * @return The methods.
   */

  public Vector getMethods(String name, UnaryPredicate predicate)
  {
     return ((LClass)getRealType()).getMethods(name,predicate);
  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     return ((LClass)getRealType()).getEnclosingConstruct();
  }


}

// ***********************************************************************
// History Log:
// $Log: LClassTypedef.java,v $
// Revision 1.5  1998/06/29 12:30:05  cinard
// Added method getEnclosingConstruct() to LClass interface and
// updated related implementations.
//
// Revision 1.4  1998/05/15 13:25:37  bareau
// Added support for NL unparsing (mainly added a method getNLName() in Scope).
//
// Revision 1.3  1998/03/18 15:48:01  cinard
// Removal of ambiguities on operation invocation.
//
// Revision 1.2  1998/02/27 12:48:27  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:53  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
