// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LCompilationUnit.java,v $
// $Revision: 1.12 $
// $Date: 1998/03/18 15:49:54 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LCompilationUnit.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Scope of a target language compilation unit.
 */

public class LCompilationUnit extends LModule
{

   /**
    * Constructs compilation unit.
    * @param parent parent scope.
    */

   public LCompilationUnit(Scope parent)
   {
      super("",parent, SymbolId.COMPILATION_UNIT);
   }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.COMPILATION_UNIT_SELECT;
  }

  /**
   * Returns a header for the description of the symbol.
   */

  protected String header()
  {
     return "Compilation unit";
  }

  /** Debug flag. */
  private final static boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: LCompilationUnit.java,v $
// Revision 1.12  1998/03/18 15:49:54  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.11  1998/02/27 12:48:28  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.10  1998/02/23 15:08:54  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.9  1998/02/12 16:03:06  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.8  1998/01/12 13:19:04  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.7  1997/12/08 12:58:42  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.6  1997/11/26 17:12:58  cinard
// Changed package name.
//
// Revision 1.5  1997/11/12 13:43:14  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.4  1997/11/04 16:04:21  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.3  1997/11/04 09:04:29  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symbol_table.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.2  1997/11/03 12:59:58  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:03  cinard
// Added language independant symbols to the symboltable package.
//
//
//
// ***********************************************************************
