// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LConstructedType.java,v $
// $Revision: 1.21 $
// $Date: 1998/05/27 09:44:18 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LConstructedType.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable;

/**
 *  Interface for constructed types.
 */

public interface LConstructedType extends LType, Scope, FileScope
{

  /**
   * Add an additional lookup scopes. Useful to add additional scopes where
   * symbols can be searched (e.g. ADL global prologue).
   * @param lookupScope additional lookup scope.
   */

  public void addLookupScope(Scope scope);
}

// ***********************************************************************
// History Log:
// $Log: LConstructedType.java,v $
// Revision 1.21  1998/05/27 09:44:18  cinard
// Added support for FileScope interface.
//
// Revision 1.20  1998/02/27 12:48:28  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.19  1998/02/23 15:08:54  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.18  1998/02/12 16:03:06  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.17  1998/02/10 12:54:20  cinard
// Updated and fixed comments.
//
// Revision 1.16  1998/01/23 16:53:21  cinard
// Suppresed hook for "this" and <class name>, these expressions are now
// defined in LClass and ADLClass symbols.
//
// Revision 1.15  1998/01/22 14:23:58  cinard
// Disabled ambiguity checks when retrieving a LCallable symbols and
// several symbols satisfies the request.
//
// Revision 1.14  1998/01/22 11:42:28  cinard
// Fixed a bug in symbols equality checks.
//
// Revision 1.13  1998/01/19 14:37:01  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.12  1998/01/12 13:19:05  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.11  1998/01/06 14:58:13  cinard
// Allow aliases.
//
// Revision 1.10  1997/12/19 21:16:36  cinard
// Added support for exceptions in throws clauses of methods.
// LCallable and LCallableEqualsComparator now contains a addException()
// method.
//
// Revision 1.9  1997/12/08 12:58:42  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.8  1997/11/26 17:12:58  cinard
// Changed package name.
//
// Revision 1.7  1997/11/14 09:58:54  cinard
// LModule scope now accepts duplicate symbols.
// SymbolTableMgr has a getADLClass() method.
//
// Revision 1.6  1997/11/13 08:19:07  cinard
// Improved controls when adding constructors.
//
// Revision 1.5  1997/11/12 13:43:15  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.4  1997/11/04 16:04:22  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.3  1997/11/04 09:04:29  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symbol_table.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.2  1997/11/03 12:59:58  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:04  cinard
// Added language independant symbols to the symboltable package.
//
//
//
// ***********************************************************************
