// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LConstructedTypeImpl.java,v $
// $Revision: 1.9 $
// $Date: 1998/07/02 13:42:49 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LConstructedTypeImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 17,  1998: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable;

import java.util.Vector;
import java.util.Enumeration;

import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.HashMap;
import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.util.ConditionalEnumeration;

/**
 *  Implementation for constructed type interface.
 */

abstract public class LConstructedTypeImpl extends    LTypeImpl
                                           implements LConstructedType
                                  
{

   // -- Public methods --

   // -- Implementation of Symbol interface --

   /** 
    * Tells if the type has got an access to a scope.
    * @return Always true.
    */

   public boolean hasScopeAccess()
   {
       return true;
   }

   /**
    * Get the scope to which this symbol has access.
    * @return The scope.
    */

   public Scope getScope()
   {
       return this;
   }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     Scope parent = getParent();
     while ( parent != null )
     {
        if ( parent instanceof LModule )
        {
           return (LModule)parent;
        }
        parent = parent.getParent();
     }
     return null;
  }

  /**
   * Returns true if the symbol is both declared and defined,
   * false otherwise. An expression declaration implies its
   * definition.
   */

  public boolean isDefined()
  {
     return isDefined;
  }

  /**
   * Indicates that the symbol is both declared and defined.
   */

  public void setDefined()
  {
     isDefined = true;
  }

  // -- Scope interface  --

  /**
   * Opens an anonymous scope into this scope.
   * Anonymous scopes are those scopes not directly related to symbols,
   * an example of such an anonymous scope the scope defined by a simple
   * block of code.
   * @return The new anonymous scope opened.
   */

  public Scope openScope()
  {
     Scope newBlock = new  LBlockScope(this);
     anonymousScopes.addElement(newBlock);
     return newBlock;
  }

  /**
   * Opens a scope associated with a symbol.
   * The provided symbol must owns its own scope (its hasScope() method
   * shall return true).
   * @param s The symbol for which the scope is to be opend.
   * @return The new scope opened.
   */

  public Scope openScope(Symbol s)
  {
     if ( debug) { 
        String str = "";
        Enumeration se = symbols.elements();
        while ( se.hasMoreElements() )
        {
           str += ((Symbol)se.nextElement()).getName() + " " ;
        }
        System.out.println("debug: LConstructedTypeImpl.openScope() - symbols ");
        System.out.println(str);
     }
     // Check the symbol exists in that scope
     if (  symbols.countValues(s) == 0  )
     { 
        throw new Error("Symbol " + s.getName() + " not in that scope.");
     }
     else if ( ! ( s instanceof Scope ) )
     {
        throw new Error("Symbol " + s.getName() + " has no scope.");
     }
     else
     {
        // State symbol is defined
        s.setDefined();

        // Set scope parent.
        ((Scope)s).setParent(this);

        return (Scope)s;
     }
   }

   /**
    * Set the parent scope.
    * @param p the parent scope.
    */

   public void setParent(Scope p) { parent = p; }

   /**
    * Get the parent scope.
    * @return The parent scope.
    */

   public Scope getParent() { return parent; }

   /**
    * Put a symbol in the scope.
    * @param symbol The symbol to put into the scope.
    */

   public void putSymbol(Symbol symbol)
   {
      // Set default modifier
      symbol.getModifier().set(defaultModifier);

      // If symbol is a constructor or a destructor just check its return 
      // type is null or void.
      if  (    symbol instanceof LMethod                  
            && (    symbol.getName().equals(this.getName())
                 || symbol.getName().equals("~" + this.getName())
               )
          )
      {
          LType returnType = ((LCallable)symbol).getReturnType();
          if ( returnType != null && ! returnType.isVoid() )
          {
             String type =   symbol.getName().startsWith("~") 
                           ? "Destructor"
		           : "Constructor";
             throw new Error
                       ( type + " with non void return type" );
          }
      }
      if ( debug ) { 
        System.out.println("debug: LConstructedTypeImpl.putSymbol() - name "
                           + "<" + symbol.getName() + ">"
                          );
      }
      symbols.add(symbol.getName(),symbol);
   } 

   /**
    * Get a symbol in the scope, extended version.
    * When using this version of getSymbol, supplementary parameters are
    * provided that shall help finding the object.
    * <pre>
    * The symbol shall satisfy some criteria defined by the provided 
    * predicate object. This predicate object help finding the symbol in 
    * case several symbols are likely to match the supplied name 
    * (e.g. overloaded methods).
    * </pre>
    * <pre>
    * Credentials shall be supplied for those cases where some symbols
    * hava non-public visibility. Retrieving them may require the
    * use of credentials.
    * </pre>
    * @param name The name of the expected symbol.
    * @param predicate An arbitrary predicate object.
    * @param credentials Credentials of the scope.
    * @return The found symbol.
    */

   public Symbol getSymbol
                 (String name,UnaryPredicate predicate,Object credentials)
   {
      
      if ( debug ) 
      {
         System.out.println
         ("debug: LConstructedTypeImpl.getSymbol(" 
          + name + ","
          + predicate
          + ")"
         ); 
      }
  
      Enumeration selectedSymbols = symbols.values(name);
      
      // Apply predicate

      if ( predicate != null )
      {
         selectedSymbols = new ConditionalEnumeration
	                       (selectedSymbols,predicate);
      }

      /**
       * Depending on the number of selected symbols, the strategy is as
       * follows :
       * 1) if no symbols were selected, return null,
       * 2) if exatly one symbol was selected, return it,
       * 3) if more than one symbol were selected throw an Error
       *    (AmbiguousRequestException)
       */

      Symbol s;

      if ( ! selectedSymbols.hasMoreElements() )
      { 
         s = null;
      }
      else
      {
         s = (Symbol)selectedSymbols.nextElement();
   
         if (    ! ( s instanceof LCallable ) 
              && selectedSymbols.hasMoreElements()
            )
         {
            throw new Error
                      (   "AmbiguousRequestException: "
                          + "Several symbols satisfies your request :" 
                          + s + " and " + selectedSymbols.nextElement()
                          + " in this = " + this
                      ); 
         }
      }

      // Search in included modules

      if ( s == null )
      {
         for ( int i = 0; i < _includedModules.size(); i++ )
         {
            s = ((Scope)_includedModules.elementAt(i)).getSymbol
                (name,predicate,credentials);
            if ( s != null ) { break; }
         }
      }

      // Search in additional lookup scopes

      if ( s == null )
      {
         for ( int i = 0; i < lookupScopes.size(); i++ )
         {
            s = ((Scope)lookupScopes.elementAt(i)).getSymbol
                (name,predicate,credentials);
            if ( s != null ) { break; }
         }
      }


      if ( debug ) 
      {
         System.out.println
         ("debug: LConstructedTypeImpl.getSymbol(" 
          + name + ","
          + predicate
          + ") return " + s
         ); 
      }

      return s;
   }


   /**
    * Get a symbol in the scope.
    * @param name The name of the expected symbol.
    * @return The found symbol.
    */

   public Symbol getSymbol(String name)
   {
      return getSymbol(name,null,null);
   }
  
  /**
   * Set default modifier (int bit mask).  All symbols that
   * will be put in the scope from now to the next setDefaultModifier()
   * call will have their modifier modified (bitwise and with default
   * modifier).
   */

  public void setDefaultModifier(int defaultModifier) 
  {
     this.defaultModifier = defaultModifier;
  }

  // -- File scope interface implementation --

  /**
   * Get the name of the underlying file.
   * TBD - improve and take different extension into account.
   */

  public String getFileName() 
  { 
     String extension;
     if ( this instanceof ADLClass )
     {
        extension = "adl";
     }
     else // this instanceof TDDClass
     {
        extension = "tdd";
     }
     return getName() + "." + extension;
  }


  /**
   * Add an included file scope.
   * @return The parent scope.
   */

  public void addIncludedFile(FileScope module) 
  { 
     _includedModules.addElement(module);
  }

  /**
   * Indicate parsing is completed for this file, and recursively
   * for all included files.
   */

  public void setParsingCompleted()
  {
     // for this file

     _isParsingCompleted = true;

     // for included files
     
     Enumeration e = symbols.elements();
     while ( e.hasMoreElements() )
     {
        Object s = e.nextElement();
        if ( s instanceof LModule )
        {
           LModule module = (LModule)s;
           if ( ! module.isParsingCompleted() )
           {
              module.setParsingCompleted();
           }
        }
     }

  }

  /**
   * Is parsing completed?
   */

  public boolean isParsingCompleted()
  {
     return _isParsingCompleted;
  }

  // -- Type specific methods --

  /**
   * Add an additional lookup scope. Useful to add additional scopes where
   * symbols can be searched (e.g. ADL global prologue).
   * @param lookupScope additional lookup scope.
   */

  public void addLookupScope(Scope scope)
  {
     lookupScopes.addElement(scope);
  }


  /**
    * Get the scope name in NL format (vector of strings).
    * Here, this is in fact the parent scope name + the current name.
    */
  public Vector getNLName() {

    Symbol parent = getEnclosingConstruct();
    Vector ret = new Vector();
    ret.addElement("#<EXTERN>");
    if ((parent != null) && (parent instanceof LPackage))
      ret.addElement(parent.getName());
    ret.addElement(getName());
    return ret;
  }

  /**
   * Returns a description of the symbol, indented.
   * @param indent indent string.
   */

  public String toString(String indent)
  {

     // Modifier

     String sModf = modifier.toString();
     if ( sModf.length() > 0 ) { sModf += " "; }

     String s =   indent + sModf + header() + "\n"
                + indent + "{"  + "\n" ;
     // Put included modules before other symbols
     Enumeration includedModules = _includedModules.elements();
     while ( includedModules.hasMoreElements() )
     {
       s += indent + "  include " 
	 + ((FileScope)(includedModules.nextElement())).getFileName() + "\n";
     }
     Enumeration se = symbols.elements();
     while ( se.hasMoreElements() )
     {
        s += ((Symbol)se.nextElement()).toString(indent+"  ");
     }
     Enumeration ae = anonymousScopes.elements();
     while ( ae.hasMoreElements() )
     {
        s += ((Scope)ae.nextElement()).toString(indent+"  ");
     }
     s += indent + "}\n";
     return s;
  }

  /**
   * Returns a description of the symbol, not indented.
   */

  public String toString()
  {
     return getName();
  }

  // -- Protected constructors --

  /**
   * Constructs the target language constructed type 
   * @param name symbol's name
   * @modifier symbol's modifier
   * @param symbolId symbol's identifier
   */

  protected LConstructedTypeImpl(String name,LModifier modifier,long symbolId) 
  {
     super(name,modifier,symbolId);
     this.parent  = null;
     // Hash map that allows duplicates
     this.symbols = new HashMap(true);
  }


  // -- Protected methods --

  /**
   * Get a header for the concrete subclass description (e.g. implements ...)
   * @return The header.
   */
  abstract protected String header();

  /**
   * Remove ambiguity on an operation invocation.
   * @param name The name of the expected symbol.
   * @param predicate Operation comparator.
   * @return The suitable operation if ambiguity has been removed,
   *         null otherwise.
   */

  protected LCallable removeAmbiguityOnOperationInvocation
                      (Vector operations,UnaryPredicate predicate)
  {

     LCallable ret = null;

     if ( debug )
     {
        String p = "";
        Enumeration e = operations.elements();
        while ( e.hasMoreElements() )
        {
           p += ((LCallable)(e.nextElement())).getSignature() + ", ";
        }
        p += predicate.toString();
        System.out.println
        ("debug:LConstructedTypeImpl.removeAmbiguityOnOperationInvocation("
        + p + ")");
     }

     int paramCount = 
         ((LCallable)operations.elementAt(0)).getParameters().length;

     // Find the most specific method for this method invocation.
     // Create an array for method's enclosing class, and for each parameters 
     // in method invocation, create an array containing method's parameter 
     // types.

     LType[][] types   = new LType[paramCount + 1][operations.size()];

     for (int i = 0; i < operations.size() ; i++)
     {
        LCallable operation = (LCallable)operations.elementAt(i);
        LParameter[] parameters = operation.getParameters();
        Symbol enclosingConstruct = operation.getEnclosingConstruct();
        if ( debug ) 
        {
           System.out.println
           ("debug: LConstructedTypeImpl.removeAmbiguityOnOperationInvocation()");
           System.out.println
           ("debug: ... enclosing class = " 
            + ( enclosingConstruct != null 
                ? enclosingConstruct.getClass() + "" + enclosingConstruct
                : null ));
        }
        if ( enclosingConstruct instanceof ADLClass )
        {
           // Underlying tested class is ok for ADLClass
           types[0][i] = ((ADLClass)enclosingConstruct).getLClass();
        }
        else if ( enclosingConstruct instanceof LClass )
        {
           // Class is taken into account for algorithm.
           types[0][i] = (LClass)enclosingConstruct;
        }
        else
        {
           // Harmless value null is ok.
           types[0][i] = null;
        }

        if ( debug ) 
        {
           System.out.println
           ("debug: LConstructedTypeImpl.removeAmbiguityOnOperationInvocation()");
           System.out.println
           ("debug: ... enclosing class = " 
            + ( ( types[0][i] != null ) ? types[0][i].getName() : null ));
        }
        for (int j = 0; j < paramCount; j++)
        {
           types[j+1][i] = parameters[j].getType();
        }
     }

     // For each sub-array get set of operations having the maximmaly
     // specific element

     HashSet[] sets = new HashSet[paramCount + 1];
     for ( int i = 0; i < paramCount + 1; i++ )
     {
        sets[i] = new HashSet();
        LType mostSpecific = null;
        for (int j = 0; j < operations.size(); j++)
        {
           if ( mostSpecific == null )
           {
              mostSpecific = types[i][j];
              sets[i].add(new Integer(j));
           }
           else if (    ! mostSpecific.equals(types[i][j]) 
                       && mostSpecific.assignable(types[i][j]) )
           {
              mostSpecific = types[i][j];
              sets[i].clear();
              sets[i].add(new Integer(j));
           }
           else if ( mostSpecific.equals(types[i][j]) )
           {
              sets[i].add(new Integer(j));
           }
        }
     }

     if ( debug )
     {
        System.out.println
        ("debug: LConstructedTypeImpl.removeAmbiguityOnOperationInvocation()");
        for ( int i = 0; i < paramCount + 1; i++ )
        {
           System.out.println("debug: ... set = " + sets[i]);
        }
     }

     // Check if a single operation fits to the former most specific 
     // operation found.
     // Search in class fields

     HashSet mostSpecificOperations = sets[0];
     for ( int i = 1; i < paramCount + 1; i++ )
     {
        mostSpecificOperations = mostSpecificOperations.intersection(sets[i]);
     }

     // If there is only one element in the set, ambiguity has been removed,
     // otherwise, returns null.
     if ( mostSpecificOperations.size() == 1 )
     {
        int index = ((Integer)mostSpecificOperations.begin().get()).intValue();
        ret = (LCallable)operations.elementAt(index);
     }

     if ( debug )
     {
        System.out.println
        (   "debug:LClassImpl.removeAmbiguityOnOperationInvocation() - return"
            + " " + ( ret != null ? ret.getSignature() : null )
        );
     }

     return ret;

  }

  // -- protected fields --

  /** Parent scope.  */
  protected Scope parent;

  /** Hashtable to store the type's symbols.  */
  protected HashMap symbols;

  // -- Private fields --

  /** Anonymous scopes contained in the callable. */
  private Vector    anonymousScopes = new Vector();

  /**
   *  Holds additional scopes to explore when looking for a symbol
   *  in this scope.
   */  
  private Vector lookupScopes = new Vector();
  
  /** Defined flag */
  private boolean isDefined = false;

  /** Default modifier */
  private int defaultModifier = 0x0000;
 
  /** Is parsing for this module completed? */
  private boolean _isParsingCompleted = false;

  /** Included modules */
  private Vector _includedModules = new Vector();
 
  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LConstructedTypeImpl.java,v $
// Revision 1.9  1998/07/02 13:42:49  bareau
// In getNLName, the parent's name is added only in the case of a package
// (not for compilation units for instance).
//
// Revision 1.8  1998/06/30 11:12:57  bareau
// Changed the default getNLName to fix the "extern" problem.
//
// Revision 1.7  1998/05/27 09:44:18  cinard
// Added support for FileScope interface.
//
// Revision 1.6  1998/05/15 13:25:38  bareau
// Added support for NL unparsing (mainly added a method getNLName() in Scope).
//
// Revision 1.5  1998/04/28 12:03:34  cinard
// Improved debugging messages.
//
// Revision 1.4  1998/03/18 15:48:01  cinard
// Removal of ambiguities on operation invocation.
//
// Revision 1.3  1998/03/10 17:26:36  mallet
// Merged v1.1 with development trunk
//
// Revision 1.2.2.1  1998/03/09 16:00:16  cinard
// Modified toString() method so that only the name of the symbol is
// returned.
//
// Revision 1.2  1998/02/27 12:48:28  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:54  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
