// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LConstructedTypedef.java,v $
// $Revision: 1.4 $
// $Date: 1998/05/27 09:44:19 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LConstructedTypedef.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 19,  1998: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;
import java.util.Vector;

/**
 *  Abstarct class for constructed types typedefs.
 */

abstract public class LConstructedTypedef extends    LTypedef
                                          implements LConstructedType
{

  /**
   * Constructs a typedef over the supplied type.
   * @param name name of the typedef.
   * @param type type for which the typedef is defined.
   */

  protected LConstructedTypedef(String name,LConstructedType type)
  {
     super(name,type);
  }

  // -- Scope interface  --

  /**
   * Opens an anonymous scope into this scope.
   * Anonymous scopes are those scopes not directly related to symbols,
   * an example of such an anonymous scope the scope defined by a simple
   * block of code.
   * @return The new anonymous scope opened.
   */

  public Scope openScope()
  {
     return ((LConstructedType)getRealType()).openScope();
  }

  /**
   * Opens a scope associated with a symbol.
   * The provided symbol must owns its own scope (its hasScope() method
   * shall return true).
   * @param s The symbol for which the scope is to be opend.
   * @return The new scope opened.
   */

  public Scope openScope(Symbol s)
  {
     return ((LConstructedType)getRealType()).openScope(s);
  }


  /**
   * Set the parent scope.
   * @param p the parent scope.
   */

  public void setParent(Scope p)
  {
     ((LConstructedType)getRealType()).setParent(p);
  }


  /**
   * Get the parent scope.
   * @return The parent scope.
   */

  public Scope getParent()
  {
     return ((LConstructedType)getRealType()).getParent();
  }


  /**
   * Put a symbol in the scope.
   * @param symbol The symbol to put into the scope.
   */

  public void putSymbol(Symbol symbol)
  {
     ((LConstructedType)getRealType()).putSymbol(symbol);
  }

  /**
   * Get a symbol in the scope, extended version.
   * When using this version of getSymbol, supplementary parameters are
   * provided that shall help finding the object.
   * <pre>
   * The symbol shall satisfy some criteria defined by the provided 
   * predicate object. This predicate object help finding the symbol in 
   * case several symbols are likely to match the supplied name 
   * (e.g. overloaded methods).
   * </pre>
   * <pre>
   * Credentials shall be supplied for those cases where some symbols
   * hava non-public visibility. Retrieving them may require the
   * use of credentials.
   * </pre>
   * @param name The name of the expected symbol.
   * @param predicate An arbitrary predicate object.
   * @param credentials Credentials of the scope.
   * @return The found symbol.
   */

  public Symbol getSymbol
                (String name,UnaryPredicate predicate,Object credentials)
  {
     return ((LConstructedType)getRealType()).getSymbol
                                              (name,predicate,credentials);
  }


  /**
   * Get a symbol in the scope.
   * @param name The name of the expected symbol.
   * @return The found symbol.
   */

  public Symbol getSymbol(String name)
  {
     return ((LConstructedType)getRealType()).getSymbol(name);
  }

  
  /**
   * Set default modifier (int bit mask).  All symbols that
   * will be put in the scope from now to the next setDefaultModifier()
   * call will have their modifier modified (bitwise and with default
   * modifier).
   */

  public void setDefaultModifier(int defaultModifier) 
  {
     ((LConstructedType)getRealType()).setDefaultModifier(defaultModifier);
  }


  /**
    * Get the scope name in NL format (vector of strings).
    * Here, this is in fact the parent scope name + the current name.
    */
  public Vector getNLName() {

    Scope parent = getParent();
    Vector ret =  (parent == null) ? new Vector() : parent.getNLName();
    ret.addElement(getName());
    return ret;
  }

   // -- File scope interface implementation --
 
   /**
    * Get the name of the underlying file.
    */
 
   public String getFileName() 
   { 
      return ((LConstructedType)getRealType()).getFileName();
   }
 
 
   /**
    * Add an included file scope.
    * @return The parent scope.
    */
 
   public void addIncludedFile(FileScope module) 
   { 
      ((LConstructedType)getRealType()).addIncludedFile(module);
   }
 
   /**
    * Indicate parsing is completed for this file, and recursively
    * for all included files.
    */
 
   public void setParsingCompleted()
   {
      ((LConstructedType)getRealType()).setParsingCompleted();
   }
 
   /**
    * Is parsing completed?
    */
 
   public boolean isParsingCompleted()
   {
      return  ((LConstructedType)getRealType()).isParsingCompleted();
   }


  // -- Constructed type specific methods --

  /**
   * Add an additional lookup scope. Useful to add additional scopes where
   * symbols can be searched (e.g. ADL global prologue).
   * @param lookupScope additional lookup scope.
   */

  public void addLookupScope(Scope scope)
  {
     ((LConstructedType)getRealType()).addLookupScope(scope);
  }

}

// ***********************************************************************
// History Log:
// $Log: LConstructedTypedef.java,v $
// Revision 1.4  1998/05/27 09:44:19  cinard
// Added support for FileScope interface.
//
// Revision 1.3  1998/05/15 13:25:38  bareau
// Added support for NL unparsing (mainly added a method getNLName() in Scope).
//
// Revision 1.2  1998/02/27 12:48:29  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:55  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
