// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LFunction.java,v $
// $Revision: 1.9 $
// $Date: 1998/06/16 12:23:40 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LFunction.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;


/**
 * target language function symbol.
 */

public class LFunction extends LCallable
{
  // -- Constructor --

  /**
   * Constructs a target language function.
   * @param name Name of the function.
   * @param modifier modifier flag.
   */

  public LFunction(String name,LModifier modifier)
  {
     this(name,modifier,false);
  }

  /**
   * Constructs a target language function.
   * @param name Name of the function.
   * @param modifier modifier flag.
   * @param isRuntime true if function belongs to runtime, false otherwise
   */

  public LFunction(String name,LModifier modifier,boolean isRuntime)
  {
     super(name,modifier,SymbolId.FUNCTION);
     _isRuntime = isRuntime;
  }
  
  // -- public methods --

  /**
   * Is this symbol a TDD test function? 
   * <P> Useful for c functions defined in tdd annotated declarations.
   */

  public boolean isTestFunction()
  {
     Scope parent = getParent();
     while ( parent != null )
     {
        if ( parent instanceof TDDClass )
        {
           return !_isRuntime;
        }
        parent = parent.getParent();
     }
     return false;
  }


  // -- Protected methods --

  /** Returns a header for the description of the expression. */

  protected String header()
  {
     return "";
  }

  // -- private attributes --
  
  /** Is current function symbol a runtime symbol? */
  private boolean _isRuntime;
}

// ***********************************************************************
// History Log:
// $Log: LFunction.java,v $
// Revision 1.9  1998/06/16 12:23:40  cinard
// Added isTestFunction() method.
//
// Revision 1.8  1998/02/27 12:48:30  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.7  1998/02/23 15:08:56  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.6  1998/02/12 16:03:08  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/12 13:19:06  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.4  1997/11/26 17:12:59  cinard
// Changed package name.
//
// Revision 1.3  1997/11/12 13:43:16  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.2  1997/11/03 12:59:59  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:04  cinard
// Added language independant symbols to the symboltable package.
//
//
//
// ***********************************************************************
