// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LMethod.java,v $
// $Revision: 1.13 $
// $Date: 1998/06/10 15:29:04 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LMethod.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;


/**
 * target language method symbol.
 */

public class LMethod extends LCallable
{
  // -- Protected constructor --

  /**
   * Constructs a target language method.
   */

  public LMethod()
  {
     this("",null);
  }

  /**
   * Constructs a target language method.
   * @param name Name of the method.
   * @param modifier the modifier flag.
   */

  public LMethod(String name,LModifier modifier)
  {
     super(name,modifier,SymbolId.METHOD);
  }

  // -- Public methods --

  public void setIsRuntimeMethod() {

    isRuntimeMethod = true;
  }

  public boolean isRuntimeMethod() {

    return isRuntimeMethod;
  }


  /**
   * Is this  symbol a TDD test method? 
   */

  public boolean isTestMethod()
  {
     Scope parent = getParent();
     while ( parent != null )
     {
        if ( parent instanceof TDDClass )
        {
           return !isRuntimeMethod;
        }
        parent = parent.getParent();
     }
     return false;
  }

  /**
   * Is this symbol a TDD test function? 
   * <P> Useful for c functions defined in tdd annotated declarations.
   */

  public boolean isTestFunction()
  {
     return isTestMethod();
  }


  // -- Protected methods --
 
  /** Returns a header for the description of the expression. */

  protected String header()
  {
     return "";
  }

  private boolean isRuntimeMethod = false;

}

// ***********************************************************************
// History Log:
// $Log: LMethod.java,v $
// Revision 1.13  1998/06/10 15:29:04  bareau
// Moved "isTestMethod" from LCallable to LMethod (and fixed it: runtime
// methods like infoline, tdd_assert... are _not_ test methods).
// Added an "isRuntimeMethod" flag in LMethod, and set it in symbol
// factories for runtime methods.
//
// Revision 1.12  1998/03/18 15:49:54  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.11  1998/02/27 12:48:30  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.10  1998/02/23 15:08:56  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.9  1998/02/12 16:03:09  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.8  1998/02/11 10:00:34  cinard
// Moved isTestMethod() method upper in the inheritance tree.
//
// Revision 1.7  1998/02/03 12:49:27  cinard
// Added isTestMethod() method.
//
// Revision 1.6  1998/01/12 13:19:06  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.5  1997/11/26 17:12:59  cinard
// Changed package name.
//
// Revision 1.4  1997/11/19 15:48:59  cinard
// Corrected some minor bugs.
//
// Revision 1.3  1997/11/12 13:43:17  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.2  1997/11/03 12:59:59  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:05  cinard
// Added language independant symbols to the symboltable package.
//
//
// ***********************************************************************
