// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LObjectArray.java,v $
// $Revision: 1.9 $
// $Date: 1998/06/18 10:27:02 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LObjectArray.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Dec 05,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Target language Array Symbol, where arrays are first class objects.
 */

public class LObjectArray extends LClassImpl
{

  // -- Constructors --

  /** 
   * Constructs an array type.
   * @param elememtType type of the elements in the array.
   */

  public LObjectArray(LType elementType)
  {
     super( ( elementType != null ) ? elementType.getName() + "[]" : "" ,
            new LModifier(0x0000),
            SymbolId.CLASS | SymbolId.OBJECT_ARRAY
          );

     // Perform some checks

     if ( elementType == null ) 
     {
        throw new Error
        ("Could not construct array type : no valid element type provided");
     }

     this.elementType = elementType;
  }

  /**
   * Redefine getFullName to compute full name based on element type
   * full name.
   */
  public String getFullName() 
  {
    return ( elementType != null )
      ? ( elementType.getFullName() + "[]" )
      : "<no name>[]";
  }

  /** Am i equals to other object? */
  public boolean equals(Object other)
  {
    return    ( other instanceof LObjectArray )
           && ( getElementType().equals
	                         (((LObjectArray)other).getElementType()));
  }
           
  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.OBJECT_ARRAY_SELECT;
  }

  // -- Public methods --

  /**
   * Get the type of the elements in this array.
   * @return The element's type.
   */

  public LType getElementType()
  {
     return elementType;
  }
  
  // -- Private feature --

  /** Type of the elements stored in the array. */
  private LType elementType;

  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LObjectArray.java,v $
// Revision 1.9  1998/06/18 10:27:02  mallet
// Added missing overriding method getFullName() which returns the full
// name based on the associated elementType full name.
//
// Revision 1.8  1998/06/11 08:38:21  cinard
// Fixed bug : equality in between java array symbols was not correctly
// handled (subtle differences exists in between java array and other
// java objects).
//
// Revision 1.7  1998/03/18 15:49:56  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.6  1998/03/02 15:32:28  cinard
// Replaced LStruct with LClass (struct symbols behave just like classes).
// Factory and find methods (newStruct() and getStruct() remains with
// their signatures updated).
//
// Revision 1.5  1998/02/27 12:48:32  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.4  1998/02/23 15:08:57  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.3  1998/02/12 16:03:10  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/12 13:19:09  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.1  1997/12/08 12:58:43  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// ***********************************************************************
