// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LPackage.java,v $
// $Revision: 1.10 $
// $Date: 1998/06/03 07:34:35 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LPackage.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 15,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;
import java.util.Vector;

/**
 * Package symbol.
 */

public class LPackage extends LModule
{
   /**
    * Constructs the package.
    * @param name name of the package
    * @param parent parent scope
    */

   public LPackage(String name,Scope parent) 
   {
      super(name,parent,SymbolId.PACKAGE);
   }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.PACKAGE_SELECT;
  }

  /**
   * Get the symbol's name in NL scope format
   * @return The symbol's name.
   */

  public Vector getNLName() {

    Scope parent = getParent();
    Vector ret =  (parent == null) ? new Vector() : parent.getNLName();
    if (name != null && ! name.equals("")) {
      int sep1 = 0, sep2 = name.indexOf('.');
      while(sep2 != -1) {
        ret.addElement(name.substring(sep1, sep2));
        sep1 = sep2+1;
        sep2 = name.indexOf(sep1,'.');
      }
      ret.addElement(name.substring(sep1));
    }
    return ret;
  }
  
   // -- Protected Methods --

   /** Returns a header for the description of the expression. */

   protected String header()
   {
      return "Package";
   }

}

// ***********************************************************************
// History Log:
// $Log: LPackage.java,v $
// Revision 1.10  1998/06/03 07:34:35  bareau
// Moved the method getNLName's behavior from LModule to LPackage.
//
// Revision 1.9  1998/03/18 15:49:56  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.8  1998/02/27 12:48:32  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.7  1998/02/23 15:08:58  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.6  1998/02/12 16:03:11  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/01/12 13:19:09  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.4  1997/11/26 17:13:01  cinard
// Changed package name.
//
// Revision 1.3  1997/11/12 13:43:19  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.2  1997/11/04 16:04:23  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.1  1997/11/03 13:01:34  cinard
// Added some classes and exceptions.
//
//
//
// ***********************************************************************
