// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LPointerImpl.java,v $
// $Revision: 1.5 $
// $Date: 1998/04/28 12:10:17 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        PointerImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 18,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;


/**
 *  LPointer interface implementation.
 */

public class LPointerImpl extends LTypeImpl
                          implements LPointer
{

  /**
   * Constructs me to be a pointer to a type.
   * @param pointed pointed type.
   * @param modifier for the pointer
   */
  
  public LPointerImpl(LType pointed,LModifier modifier)
  {
     super( "" ,
            modifier,
            SymbolId.POINTER
          );
     
     this.pointed = pointed;
  }

  /**
   * Constructs me to be a pointer to a type.
   */
  
  public LPointerImpl()
  {
     super("", new LModifier(0x0000), SymbolId.POINTER);
     pointed = null;
  }

  // -- Symbol interface implementation --

  /**
   * Symbol's name.
   */

  public String getName()
  {
     return ( pointed != null )
            ? ( pointed.getName() + "*" )
            : "<no name>*";
  }


  /**
   * Check if symbol has direct or indirect access to a scope. That is never
   * true for a pointer.
   * @return Always false.
   */

  public boolean hasScopeAccess()
  {
     return false;
  }
  
  /**
   * Get the scope the symbol has access to.
   * @return Always throws an error.
   */

  public Scope   getScope()
  {
     throw new Error("LPointerImpl.getScope() should not be called.");
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.POINTER_SELECT;
  }

  // -- LType specific methods --

  /**
   * Checks if type t is equal to this type.
   * @param t the type for which equality is checked.
   * @return true if it is equals, false otherwise.
   */

  public boolean equals(Object t)
  {     
     return    ( t instanceof LPointer )
            && ( pointed.equals(((LPointer)t).getPointedToType()) ); 
  }


  // -- Pointer type specific methods --

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
     return    ( t instanceof LPointer )
            && ( pointed.assignable(((LPointer)t).getPointedToType()) );
  }

  /**
   * Returns the pointed type.
   */

  public LType getPointedToType()
  {
     return pointed;
  } 

  /**
   * Set pointed to type.
   */

  public void setPointedToType(LType pointed)
  {
     this.pointed = pointed;
  }

  /**
   * Returns a string description of the type, indented version.
   * @param indent Indentation.
   * @return Description of the type.
   */

  public String toString(String indent)
  {
     return indent + getName();
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LPointerTypedef(name,this);
  }

  // -- Private fields --

  /** Pointed type.  */
  protected LType pointed;

  /** Debug flag. */
  protected static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LPointerImpl.java,v $
// Revision 1.5  1998/04/28 12:10:17  cinard
// Added a method setPointedToType().
// Allow null pointed types (setPointedToType() enable to define the
// pointed to type later ina case it was not supplied at construction time).
//
// Revision 1.4  1998/03/18 15:49:57  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.3  1998/03/10 17:26:37  mallet
// Merged v1.1 with development trunk
//
// Revision 1.2.2.1  1998/03/09 16:00:17  cinard
// Modified toString() method so that only the name of the symbol is
// returned.
//
// Revision 1.2  1998/02/27 12:48:33  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:58  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
