// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Revision: 1.4 $
// $Date: 1998/03/18 15:49:57 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LPointerToFieldMemberImpl.java
//
// HISTORY:
//      Wassel BERRAYANA, TOG - December 22,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Implementation for pointer to field member interface.
 */

public class LPointerToFieldMemberImpl extends    LTypeImpl
                                       implements LPointerToFieldMember 

{

  /**
   * Constructs a pointer to field member.
   * @param pointed pointed type.
   * @param clazz class containing the filed.
   */
  
  public LPointerToFieldMemberImpl(LType pointed , LConstructedType clazz )
  {

     super(   ((clazz   != null) ? clazz.getName()   + "::" : "")
            + ((pointed != null) ? pointed.getName() + "*"  : ""),
            new LModifier(0x0000),
            SymbolId.POINTER_TO_FIELD_MEMBER
          );


     // Perform some checks

     if (!( clazz instanceof LClass ))
     {
        throw new Error ( "Pointer to Member can be created only for classes"
                        + " and structs.");
     }
     
     if ( pointed == null ) 
     {
        throw new Error ( "Could not construct pointer to member type :"
                        + " no valid pointed type provided.");
     }

     this.encloseClass = clazz;      
     this.pointed = pointed;
  }

  // -- Symbol interface implementation --

  /**
   * @return the Type pointed to by LPointerToFieldMember for which 
   *         method applies 
   */

  public LType getPointedToType() 
  {
     return pointed;
  } 

  /** 
   * @return the class whose this LPointerToFieldMember is pointing to 
   *         its members
   */

  public LConstructedType getEnclosingClass ( ) 
  {
    return encloseClass;
  }

  /**
   * Has the symbol direct or indirect access to a scope. 
   * That is always true for a pointer.
   * @return Always true.
   */

  public boolean hasScopeAccess()
  {
     return true;
  }
  
  /**
   * Scope the symbol has access to.
   */

  public Scope  getScope()
  {
     return encloseClass.getScope();
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.POINTER_TO_FIELD_MEMBER_SELECT;
  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     return getEnclosingClass();
  }

  // -- LType specific methods --

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LPointerToFieldMemberTypedef(name,this);
  }
	  

  /**
   * Checks if type t is equal to this type.
   * @param t the type for which equality is checked.
   * @return true if it is equals, false otherwise.
   */

  public boolean equals(Object t)
  {     
     return    (t instanceof LPointerToFieldMember)
            && (pointed.equals(((LPointerToFieldMember)t).getPointedToType()))
            && (encloseClass.equals
                (((LPointerToFieldMember)t).getEnclosingClass ())); 
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
     return    ( t instanceof LPointerToFieldMember )
            && ( pointed.assignable
                 (((LPointerToFieldMember)t).getPointedToType()))
            && ( this.encloseClass instanceof LClass )
            && ( ((LClass)this.encloseClass).subclassOf
                 (((LPointerToFieldMember)t).getEnclosingClass( ))) ;
  }

  /**
   * Returns a string description of the type, indented version.
   * @param indent Indentation.
   * @return Description of the type.
   */

  public String toString(String indent)
  {
     return     indent + pointed.getName() + " " + encloseClass.getName() 
              + "::* " + "\n";
  }

  // -- Private fields --

  /** Pointed type. */

  private LType pointed;

  /** Enclosing Class */

  private LConstructedType encloseClass;

  /** Debug flag. */

  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LPointerToFieldMemberImpl.java,v $
// Revision 1.4  1998/03/18 15:49:57  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.3  1998/03/02 15:54:00  cinard
// Removed classes related to LStruct. Updated code accordingly.
//
// Revision 1.2  1998/02/27 12:48:34  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:08:59  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
