// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LPointerToFunctionMemberImpl.java,v $
// $Revision: 1.4 $
// $Date: 1998/03/18 15:49:57 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LPointerToFunctionImpl.java
//
// HISTORY:
//      Wassel BERRAYANA, TOG - Dec 22,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Implementation for LPointerToFunctionMember interface.
 */

public class LPointerToFunctionMemberImpl extends    LTypeImpl
                                          implements LPointerToFunctionMember
                                          
{

  /**
   * Constructs a pointer to a function member.
   * @param pointed pointed function prototype (LCallableEqualsComparator).
   * @param clazz enclozing class.
   */

  public LPointerToFunctionMemberImpl(LCallableEqualsComparator pointed,
                                      LConstructedType clazz )
  {
     super (( pointed != null ) ? pointed.toString("(*)") : "" ,
            new LModifier(0x0000),
            SymbolId.POINTER_TO_FUNCTION_MEMBER
           );
     
     // Perform some checks

     if (!( clazz instanceof LClass ))
     {
        throw new Error(  "Pointer to function member can be created only"
                        + " for classes and structs");
     }
     
     if ( pointed == null ) 
     {
        throw new Error(  "Could not construct pointer to function member :"
                        + " no valid pointed function provided");
     }

     this.encloseClass = clazz;     
     this.pointed = pointed;
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LPointerToFunctionMemberTypedef(name,this);
  }
	  

  /**
   * Has symbol direct or indirect access to a scope.
   * @return Always true.
   */

  public boolean hasScopeAccess()
  {
     return true;
  }
  
  /**
   * Scope the symbol has access to.
   */

  public Scope   getScope()
  {
     return encloseClass.getScope();
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.POINTER_TO_FUNCTION_MEMBER_SELECT;
  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     return getEnclosingClass();
  }

  /**
   * Checks if type t is equal to this type.
   * @param t the type for which equality is checked.
   * @return true if it is equals, false otherwise.
   */

  public boolean equals(LType t)
  {     
     return    ( t instanceof LPointerToFunctionMember )
            && ( pointed.equals
                 (((LPointerToFunctionMember)t).getPointedComparator()))
            && ( encloseClass.equals
                 (((LPointerToFunctionMember)t).getEnclosingClass())); 
  }

  /**
   * Get the pointed comparator.
   * @result the pointed comparator.
   */
  public LCallableEqualsComparator getPointedComparator() 
  {
    return pointed ;
  }


  /**
   * Checks if LPointerToFieldMethod can be assigned to variable of this type.
   * <p> This is true only when <code>t</code> is also a pointer to function
   * member having the same comparator and whose enclosing class is a super 
   * class of <code>this</code> class.
   * @param  t The callable for which the assignability is checked.
   * @return True if it can, false otherwise.
   */
  public boolean assignable(Symbol t) 
  {
     return    (t !=  null) 
            && (t instanceof LPointerToFunctionMember) 
            && this.pointed.equals
               (((LPointerToFunctionMember)t).getPointedComparator())
            && (this.encloseClass instanceof LClass )
            && ((LClass)this.encloseClass).subclassOf 
               (((LPointerToFunctionMember)t).getEnclosingClass());
  }
  
  /**
   * Returns a string description of the type, indented version.
   * @param indent Indentation.
   * @return Description of the type.
   */

  public String toString(String indent)
  {
     return   indent + encloseClass.getName()+"::"+ getName() + "\n";
  }


  /**
   * Type pointed to by the current LPointerToFunctionMember.
   */

  public LType getPointedToType() 
  {
     return pointed.getReturnType();
  }

  /**
   * Class whose this LPointerToFunctionMember is pointing to its members.
   */

  public LConstructedType getEnclosingClass ()
  {
     return encloseClass;
  }

  /**
   * Returns a string for the declaration of an expression of this
   * type. Useful for mixed type and variable name (e.g. pointer
   * to function). Default behaviour is to return "type name".
   * @param name expression's name.
   */

  public String mixNameAndType(String name)
  {
     return pointed.toString("( "+ encloseClass.getName()+":: *" + name + ")");
  }

  // -- Private fields --

  /** Enclosing class */

  private LConstructedType encloseClass;

  /** Pointed type or callable */

  private LCallableEqualsComparator pointed;

  /** Debug flag. */

  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LPointerToFunctionMemberImpl.java,v $
// Revision 1.4  1998/03/18 15:49:57  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.3  1998/03/02 15:54:00  cinard
// Removed classes related to LStruct. Updated code accordingly.
//
// Revision 1.2  1998/02/27 12:48:36  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:00  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
