// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LReferenceImpl.java,v $
// $Revision: 1.4 $
// $Date: 1998/04/28 12:15:11 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LReferenceImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 21,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Reference symbol.
 */

public class LReferenceImpl extends LTypeImpl implements LReference 
{

  /**
   * Constructs a reference type.
   * @param referenced referenced type.
   */

   public LReferenceImpl(LType referenced)
   {
      super( "" ,
             new LModifier(0x0000),
             SymbolId.REFERENCE );
      this.referenced = referenced;
   }

  /**
   * Constructs a reference type.
   */

  public LReferenceImpl()
  {
     super("", new LModifier(0x0000), SymbolId.REFERENCE );
     referenced = null;
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LReferenceTypedef(name,this);
  }
	  
  /**
   * Symbol's name.
   */

  public String getName()
  {
     return ( referenced != null )
            ? ( referenced.getName() + "&" )
            : "<no name>&";
  }

  /**
   * Check if symbol has direct or indirect access to a scope. That is never
   * true for a reference.
   * @return Always false.
   */

  public boolean hasScopeAccess()
  {
     return false;
  }

  /**
   * Get the scope the symbol has access to.
   * @return Always throws an error.
   */

  public Scope getScope()
  {
     throw new Error("LReferenceImpl.getScope() should not be called.");
  } 

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.REFERENCE_SELECT;
  }

  /**
   * Checks if type t is equal to this type.
   * @param t the symbol for which equality is checked.
   * @return true if it is equals, false otherwise.
   */

  public boolean equals(Object t)
  {     
     return    (t instanceof LReference)
            && (referenced.equals(((LReference)t).getReferenced())); 
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

   public boolean assignable(Symbol t) 
   {
      return      (t instanceof LReference)
               && (referenced.assignable(((LReference)(t)).getReferenced()));
   }

  /**
   * Get the referenced type.
   * @return the referenced type.
   */

  public LType getReferenced()
  {
     return referenced;
  }

  /**
   * Set referenced type.
   */

  public void setReferenced(LType referenced)
  {
    if ( referenced != null )
    {
       this.referenced = referenced;
    }
  }

  /**
   * Returns a string description of the callable, indented version.
   * @param indent Indentation.
   * @return Description of the callable.
   */

  public String toString(String indent)
  {
     return   indent + getName() + "\n";
  }
  
  // -- Private feature --

  /** Refenrenced type */

  protected LType referenced;

  /** Debug flag. */

  protected static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LReferenceImpl.java,v $
// Revision 1.4  1998/04/28 12:15:11  cinard
// Added a method setReferencedType().
// Allow null pointed types (setReferencedType() enable to define the
// referenced type later in case it was not supplied at construction time).
//
// Revision 1.3  1998/03/18 15:49:58  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:48:37  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:02  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
