// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LReferenceToFunctionImpl.java,v $
// $Revision: 1.3 $
// $Date: 1998/03/18 15:49:58 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LReferenceToFunctionImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 22,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 *  Reference to function symbol.
 */

public class LReferenceToFunctionImpl extends    LTypeImpl
                                      implements LReferenceToFunction
                                      
{

  /**
   * Constructs a reference to a function.
   * @param referenced referenced function prototype 
   *                   (LCallableEqualsComparator).
   */

  public LReferenceToFunctionImpl(LCallableEqualsComparator referenced)
  {
     super((referenced != null) ? referenced.toString("(&)") : "",
           new LModifier(0x0000) ,
           SymbolId.REFERENCE_TO_FUNCTION);
     
     // Perform some checks
     
     if ( referenced == null ) 
     {
        throw new Error
        (   "Could not construct reference to function :"
          + " no valid referenced function provided"
        );
     }
     
     this.referenced = referenced;
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LReferenceToFunctionTypedef(name,this);
  }
	  

  /**
   * Check if symbol has direct or indirect access to a scope. That is never
   * true for a pointer.
   * @return Always false.
   */

  public boolean hasScopeAccess()
  {
     return false;
  }
  
  /**
   * Get the scope the symbol has access to.
   * @return Always throws an error.
   */

  public Scope   getScope()
  {
     throw new Error
               ("LReferenceToFunctionImpl.getScope() should not be called.");
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.REFERENCE_TO_FUNCTION_SELECT;
  }

  // -- Specific methods --

  /**
   * Checks if a callable can be assigned to a variable of this type.
   * @param  t The callable for which the assignability is checked.
   * @return True if it can, false otherwise.
   */
  
  public boolean assignable(Symbol t) 
  {
     return     t instanceof LCallable
             && ((LCallableEqualsComparator)referenced).execute((LCallable)t);
  }
  
  /**
   * Returns a string description of the type, indented version.
   * @param indent Indentation.
   * @return Description of the type.
   */

  public String toString(String indent)
  {
     return   indent + getName() + "\n";
  }

  /**
   * Returns a string for the declaration of an expression of this
   * type. Useful for mixed type and variable name (e.g. pointer
   * to function). Default behaviour is to return "type name".
   * @param name expression's name.
   */

  public String mixNameAndType(String name)
  {
     return referenced.toString("(&" + name + ")");
  }

  /** 
   * Referenced function.
   */

  public LCallableEqualsComparator getReferencedFunction()
  {
     return referenced;
  }

  // -- Private fields --

  /** Referenced callable */

  private LCallableEqualsComparator referenced;

  /** Debug flag.*/

  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: LReferenceToFunctionImpl.java,v $
// Revision 1.3  1998/03/18 15:49:58  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:48:38  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:02  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
