// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LType.java,v $
// $Revision: 1.16 $
// $Date: 1998/02/27 12:48:40 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LType.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

/**
 * Interface for target language types. 
 */

public interface LType extends Symbol
{

  /**
   * Get the symbol's name.
   * @return The symbol's name.
   */
   
  public String getName();
 
  /**
   * Returns synthesis type of the @param t and type for which method applies.
   * <p><b>Algorithm:</b><ul>
   * <li> If type <code>t</code> is null return null.
   * <li> Else if <code>t</code> assignable to <code>this</code> return <code>
   * this</code> type.
   * Else if <code>this</code> assignable to <code>t</code> return <code>t
   * </code>
   * <li> Else return null.
   * </ul>
   * @return is the smallest type assignable to both types,
   *         is null is no common assignable type found.
   */ 

  public LType synthesisType(LType t);

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t);

  /**
   * Returns true if type is void. Default is to return false.
   */

  public boolean isVoid();

  /**
   * Returns a string for the declaration of an expression of this
   * type. Useful for mixed type and variable name (e.g. pointer
   * to function). Default behaviour is to return "type name".
   * @param name expression's name.
   */

  public String mixNameAndType(String name);

  /** 
   *  Get the type kind. Useful for languages that defines several
   *  type kinds (e.g. in Java boolean, arithmetic and references).
   *  @return an integer indicating the kind of the type. 
   */

  public Integer getTypeKind();


  /**
   * Real type. That is the 'ultimate' real (i.e. not typedef'ed type ) type 
   * for which this type is defined. Default is to return this. 
   */

  public LType getRealType();

  /**
   * Make up new typedef for this type.
   * @param name typedef name.
   */

  public LType newTypedef(String name);

}

// ***********************************************************************
// History Log:
// $Log: LType.java,v $
// Revision 1.16  1998/02/27 12:48:40  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.15  1998/02/25 16:34:12  bareau
// Re-introduced the method getName().
//
// Revision 1.14  1998/02/23 15:09:04  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.13  1998/02/12 16:03:15  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.12  1998/02/10 12:54:52  cinard
// newComparator() modified to exclude ADLClass & TDDClass.
//
// Revision 1.11  1998/01/27 13:24:47  wassel
// synthesisType implmenting default type synthesis behavior in LType based in calls to assignable method.
// Removed syntheisType from LArray.
// Some fields and methods made public or protected in LReference, LPointer and LPointerToFunction.
//
// Revision 1.10  1998/01/22 11:42:30  cinard
// Fixed a bug in symbols equality checks.
//
// Revision 1.9  1998/01/19 14:37:03  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.8  1998/01/12 13:19:11  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.7  1998/01/06 13:19:09  wassel
// Added methods synthesisType for type checking
//
// Revision 1.6  1997/12/08 12:58:45  cinard
// Added a new type check mechanism.
// Fixed some bugs.
//
// Revision 1.5  1997/11/26 17:13:03  cinard
// Changed package name.
//
// Revision 1.4  1997/11/12 13:43:21  cinard
// Corrected some bugs.
// Replaced LCallableSearcher class with LCallableAssignComparator and
// LCallableEqualsComparator classes.
// Added LModuleComparator and LTypeComparator classes (useful for
// Java names classification).
//
// Revision 1.3  1997/11/04 16:04:24  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.2  1997/11/03 13:00:01  cinard
// Improved scope management : duplicates in LClass scope allowed.
// Added PoinerToFunction class.
//
// Revision 1.1  1997/10/27 11:18:06  cinard
// Added language independant symbols to the symboltable package.
//
//
//
// ***********************************************************************
