// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LTypedef.java,v $
// $Revision: 1.4 $
// $Date: 1998/03/18 15:48:02 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LTypeDef.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 * Abstract class for further typedef symbol.
 */

abstract public class LTypedef implements LType
{
  /**
   * Constructs a typedef over the supplied type.
   * @param name name of the typedef.
   * @param type type for which the typedef is defined.
   */

  protected LTypedef(String name,LType type)
  {
     _name = name;
     _type = type;
  }

  // -- Object --

  /**
   * Is this equals to supplied o.
   */
  public boolean equals(Object o)
  {
     return getRealType().equals(o);
  }

  // -- Symbol interface implementation --

  /**
   * Get the symbol's name.
   * @return The symbol's name.
   */

  public String getName() 
  {
     return _name;
  }

  /**
   * Get the symbol's full name.
   */
 
  public String getFullName() { return getRealType().getFullName(); }


  /**
   * Set the symbol's name.
   * @parma n The symbol's name.
   */

  public void   setName(String n) 
  {
     // TBD - think about removing this method
     // _name = n;
  }

  /**
   * Check if symbol has direct or indirect access to a scope.
   * Direct access means the symbol owns a scope (e.g. package, class)
   * Indirect access means the symbol does not own a scope but is 
   * related to a scope (e.g. a reference to an object)
   * Simply speaking any symbol on which the scope operator (. or ::)
   * can be applied has access to a scope.
   * @return True if symbol has direct or indirect access to a scope,
   *         false otherwise.
   */

  public boolean hasScopeAccess() 
  {
     return getRealType().hasScopeAccess();
  }

  /**
   * Get the scope the symbol has access to.
   * @return The scope that can be accessed to.
   */

  public Scope   getScope()
  {
     return getRealType().getScope();
  }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return getRealType().getComparator();
  }

  /**
   * Returns true if the symbol is both declared and defined,
   * false otherwise.
   */

  public boolean isDefined() 
  {
     return getRealType().isDefined();
  }

  /**
   * Indicates that the symbol is both declared and defined.
   */

  public void setDefined()
  {
     getRealType().setDefined(); 
  } 

  /**
   * Get the symbol's modifier.
   * @return symbol's modifier.
   */

  public LModifier getModifier() 
  {
     return getRealType().getModifier();
  }

  /**
   * Set the symbol's modifier.
   * @param modifier symbol's modifier.
   */

  public void setModifier(LModifier modifier) 
  {
     getRealType().setModifier(modifier); 
  }

  /**
   * Does the symbol's identifier is equals to id? Note a bit mask
   * containing the symbol's id bit is ok.
   * @param symbolId a symbol identifier.
   */

  public boolean isA(long symbolId) 
  {
     return getRealType().isA(symbolId);
  }

  /** 
   * Symbol's identifier.
   */

  public long getSymbolId() 
  {
     return getRealType().getSymbolId();
  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...). Default implementation is to return null,
   * constructed types shall override this.
   */

  public Symbol getEnclosingConstruct()
  {
     return getRealType().getEnclosingConstruct();
  }

  /**
   * Returns a description of the symbol, indented.
   * @param indent Indentation string.
   * @return Description of the symbol.
   */

  public String toString(String indent) 
  {
     return   indent + "<typedef ... " + _name + "> "  
            + getRealType().toString(indent) + "\n"; 
  }

  /**
   * Returns a description of the symbol, not indented.
   * @return Description of the symbol.
   */

  public String toString() 
  {
     return getName();
  }

  // -- LType interface implementation --
  
  /**
   * Make up new typedef for this type.
   * @param name typedef name.
   */

  public LType newTypedef(String name)
  {
     return _type.newTypedef(name);
  }

  /**
   * Returns synthesis type of the param t and type for which method applies.
   * @return is the smallest type assignable to both types,
   *         is null is no common assignable type found.
   */ 

  public LType synthesisType(LType t) 
  {
     return getRealType().synthesisType(t);
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t)
  {
     return getRealType().assignable(t);
  }

  /**
   * Returns true if type is void. Default is to return false.
   */

  public boolean isVoid()
  {
     return getRealType().isVoid();
  }


  /**
   * Returns a string for the declaration of an expression of this
   * type. Useful for mixed type and variable name (e.g. pointer
   * to function). Default behaviour is to return "type name".
   * @param name expression's name.
   */

  public String mixNameAndType(String name)
  {
     return _name + " " + name; // getRealType().mixNameAndType(name);
  }


  /** 
   *  Get the type kind. Useful for languages that defines several
   *  type kinds (e.g. in Java boolean, arithmetic and references).
   *  @return an integer indicating the kind of the type. 
   */

  public Integer getTypeKind()
  {
     return getRealType().getTypeKind();
  }


  /**
   * Real type. That is the 'ultimate' real (i.e. not typedef'ed type ) type 
   * for which this type is defined. Default is to return this. 
   */

  public LType getRealType()
  {
     return _type.getRealType();
  }

  // -- Private field --
  
  /** Name of the typedef. */

  private final String _name;

  /** Type for which the typedef is defined. */

  private final LType _type;

}

// ***********************************************************************
// History Log:
// $Log: LTypedef.java,v $
// Revision 1.4  1998/03/18 15:48:02  cinard
// Removal of ambiguities on operation invocation.
//
// Revision 1.3  1998/03/10 17:26:38  mallet
// Merged v1.1 with development trunk
//
// Revision 1.2.2.1  1998/03/09 16:00:18  cinard
// Modified toString() method so that only the name of the symbol is
// returned.
//
// Revision 1.2  1998/02/27 12:48:41  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:05  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
