// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/LUnionImpl.java,v $
// $Revision: 1.3 $
// $Date: 1998/03/18 15:49:59 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        LUnionImpl.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import java.util.Hashtable;
import com.objectspace.jgl.UnaryPredicate;

/**
 * Implementation for Union symbol.
 */

public class LUnionImpl extends LConstructedTypeImpl implements LUnion 
{
  // -- constructors --
 
  /**
   * Constructs a target language union.
   * @param name name of the union.
   */

  public LUnionImpl(String name)
  {
     super(name,new LModifier(0x0000),SymbolId.UNION);
  }

  // -- Public methods --

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.UNION_SELECT;
  }
  
  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new LUnionTypedef(name,this);
  }
	  
  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
     return this.equals(t);
  }

  // -- Protected features --

  /**
   * Returns a header for the description of the symbols.
   */

  protected String header()
  {
     // Modifier

     String sModf = ( modifier != null ? getModifier().toString() : "");
     if ( sModf.length() > 0 ) { sModf += " "; }

     return   sModf + "struct " + getName(); 
  }

}

// ***********************************************************************
// History Log:
// $Log: LUnionImpl.java,v $
// Revision 1.3  1998/03/18 15:49:59  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:48:42  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:06  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
