// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/SymbolFactory.java,v $
// $Revision: 1.6 $
// $Date: 1998/06/16 12:25:11 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        SymbolFactory.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.HashMap;

/**
 * Abstract factory for language independant symbols
 */

abstract public class SymbolFactory 
{

  /**
   * Constructs and returns a TDD factory.
   * @param name Name of the factory.
   */

  public TDDFactory newTDDFactory(String name)
  {
     return new TDDFactory(name);
  }

  /**
   * Constructs and returns a class.
   * @param name name of the class.
   * @param modifier modifier for the class
   */
   
  public LClass newClass(String name,int modifier)
  {
     return new LClassImpl(name,newModifier(modifier));
  }

  /**
   * Constructs and returns a compilation unit.
   * @param parent parent scope.
   */

  public LCompilationUnit newCompilationUnit(Scope parent)
  {
     return new LCompilationUnit(parent);
  }

  /**
   * Constructs and returns a compilation unit.
   * @param parent parent scope.
   */

  public LCompilationUnit newCompilationUnit(String name, Scope parent)
  {
     LCompilationUnit cu = new LCompilationUnit(parent);
     cu.setName(name);
     return cu;
  }

  /**
   * Constructs and returns a package.
   * @param name name of the package.
   * @param parent parent scope.
   */

  public LPackage newPackage(String name,Scope parent)
  {
     return new LPackage(name,parent);
  }

  /**
   * Constructs and returns a target language constructor.
   * @param name Name of the constructor.
   * @param modifier modifier flag.
   */

  public LConstructor newConstructor(String name,int modifier)
  {
     return new LConstructor(name,newModifier(modifier));
  }
 
  /**
   * Constructs and returns a target language function.
   * @param name Name of the function.
   * @param modifier modifier flag.
   */

  public LFunction newFunction(String name,int modifier)
  {
     return new LFunction(name,newModifier(modifier));
  }

  /**
   * Constructs and returns a target language function that belongs to
   * adl runtime.
   * @param name Name of the function.
   * @param modifier modifier flag.
   */

  public LFunction newRuntimeFunction(String name,int modifier)
  {
     return new LFunction(name,newModifier(modifier),true);
  }

  /**
   * Constructs and returns a target language method.
   * @param name Name of the method.
   * @param modifier the modifier flag.
   */

  public LMethod newMethod(String name,int modifier)
  {
     return new LMethod(name,newModifier(modifier));
  }

  /**
   * Constructs and returns an ADL method.
   * @param name Name of the method.
   * @param modifier the modifier flag.
   */

  public ADLMethod newADLMethod(String name,int modifier)
  {
     return new ADLMethod(name,newModifier(modifier));
  }

  /**
   * Constructs and returns an ADL inline.
   * @param name Name of the inline.
   * @param modifier the modifier flag.
   */

  public ADLInline newADLInline(String name,int modifier)
  {
     return new ADLInline(name,newModifier(modifier));
  }

  /**
   * Constructs and returns a name space
   * @param name name of the name space.
   */

  public LNameSpace newNameSpace(String name) 
  {
     return  new LNameSpace(name,null) ;
  }

  /**
   * Constructs and returns a language independent parameter.
   * @param name name of the symbol.
   * @param modifier modifier flag.
   * @param type parameter's type.
   */

  public LParameter newParameter(String name,int modifier,LType type)
  {
     return new LParameter(name,newModifier(modifier),type);
  }


  /**
   * Constructs and returns a type : pointer to type.
   * @param referenced pointer type.
   */
  
  public LPointer newPointer(LType pointed,int modifier)
  {
     return new LPointerImpl(pointed,newModifier(modifier));
  }

  /**
   * Constructs and returns a type : pointer to type.
   */
  
  public LPointer newPointer()
  {
     return new LPointerImpl();
  }


  /**
   * Constructs and returns a type : pointer to function.
   * @param referenced function (LCallableEqualsComparator).
   */
  
  public LPointerToFunction newPointerToFunction
                            (LCallableEqualsComparator pointed,
                             int modifier)
  {
     LPointerToFunction ret =  new LPointerToFunctionImpl
                                   (pointed,newModifier(modifier));
     return ret;
  }

  /**
   * Constructs and returns a type : pointer to function member.
   * @param pointed function (LCallableEqualsComparator).
   * @param clazz the enclosing class whose function mebers are pointed to.
   */
  
  public LPointerToFunctionMember newPointerToFunctionMember
                   (LCallableEqualsComparator pointed, LConstructedType clazz)
  {
     LPointerToFunctionMember ret= new LPointerToFunctionMemberImpl
                                       (pointed,clazz);
     ret.setModifier(newModifier(0));
     return ret;
  }

  /**
   * Constructs and returns a type : pointer to field member.
   * @param pointed type.
   * @param clazz the enclosing class whose function mebers are pointed to.
   */
  
  public LPointerToFieldMember newPointerToFieldMember
                                      (LType pointed, LConstructedType clazz)
  {
     LPointerToFieldMember ret= new LPointerToFieldMemberImpl(pointed,clazz);
     ret.setModifier(newModifier(0));
     return ret;
  }

  /**
   * Constructs and returns a reference type.
   * @param referenced referenced type.
   */

  public LReference newReference(LType referenced)
  {
     return new LReferenceImpl(referenced);
  }

  /**
   * Constructs and returns a reference type.
   */

  public LReference newReference()
  {
     return new LReferenceImpl();
  }

  /**
   * Constructs and returns a type : reference to function.
   * @param referenced function (LCallableEqualsComparator).
   */

  public LReferenceToFunction newReferenceToFunction
                              (LCallableEqualsComparator referenced)
  {
     return  new LReferenceToFunctionImpl(referenced);
  }

  /**
   * Constructs and returns a target language structure.
   * @param name name of the structure.
   */

  public LClass newStruct(String name)
  {
     return new LClassImpl(name,newModifier(0x0000),SymbolId.STRUCT);
  }

  /**
   * Constructs and returns a target language union.
   * @param name name of the union.
   */

  public LUnion newUnion(String name)
  {
     return new LUnionImpl(name);
  }

  /**
   * Constructs and returns a target language variable.
   * @param name name of the variable.
   * @param modifier modifier flag (bit mask where Modifier.PUBLIC...
   *                     can be set)
   * @param type type of the variable.
   */

  public LVariable newVariable(String name, int modifier, LType type)
  {
     return new LVariable(name,newModifier(modifier),type);
  }

  /**
   * Constructs and returns a TDD dataset
   * @param name name of the dataset.
   * @param mySize dataset's size.
   * @param modifier modifier flag (bit mask where Modifier.PUBLIC...
   *                     can be set)
   * @param type type of the dataset.
   */

  public TDDDataset newTDDDataset
         (String name, int mySize,int modifier, LType type)
  {
     return new TDDDataset(name,mySize,newModifier(modifier),type);
  }

  /**
   * Constructs and returns an ADL binding.
   * @param name name of the binding.
   * @param modifier modifier flag (bit mask where Modifier.PUBLIC...
   *                     can be set)
   * @param type type of the variable.
   */

  public ADLBinding newADLBinding(String name, int modifier, LType type)
  {
     return new ADLBinding(name,newModifier(modifier),type);
  }

  /**
   * Constructs and returns a TDD class symbol.
   * @param name name of the TDD class.
   */

  public TDDClass newTDDClass(String name) 
  {
     return new TDDClass(name);
  }

  /**
   * Constructs and returns an attribute.
   * @param name name of the attribute.
   * @param modifier modifier flag for the attribute.
   * @param type type of the attribute.
   */

  public LAttribute newAttribute(String name,int modifier,LType type)
  {
     return new LAttribute(name,newModifier(modifier),type);
  }

  /** 
   * Constructs and returns an array type.
   * @param elememtType type of the elements in the array.
   */

  public LType newArray(LType elementType)
  {
     return new LArrayImpl(elementType);
  }

  /** 
   * Constructs and returns  an array type.
   * @param elememtType type of the elements in the array.
   * @param dimension array dimension.
   */

  public LType newArray(LType elementType, int dimension)
  {
     LType ret = null;
     if ( dimension > 0 )
     {
       ret = newArray(elementType);
       dimension--;
     }
     while ( dimension > 0 )
     {
        ret = newArray(ret);
        dimension--;
     }
     return ret;
  }

  /**
   * Constructs and returns the ADL class to supply semantics for the 
   * target language class provided.
   * @param lClass The target language class for which semantics is to be 
   *               defined.
   * @param superClassName name of the adl superclass.
   */

  public ADLClass newADLClass(LClass lClass,String superClassName) 
  {
     return  new ADLClass(lClass,superClassName);
  }

  /**
   * Constructs a new typedef.
   * @param name name of the typedef.
   * @param type type over which typedef is defined.
   */

  public LType newTypedef(String name,LType type)
  {
     return type.newTypedef(name);
  }

  /**
   * Constructs a new nld module.
   * @param name name of the nld file.
   */

  public LModule newNLDModule(String name)
  {
     return new LModule(name, null, SymbolId.NLD_MODULE);
  }

  /**
   * Creates a new LModifier with all the bits in modifier set.
   * @param modifier bit mask for the modifier.
   */

  abstract protected LModifier newModifier(int modifier);

  /** Debug flag. */

  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: SymbolFactory.java,v $
// Revision 1.6  1998/06/16 12:25:11  cinard
// Added newRuntimeFunction() method in factory.
// Updated newRuntime() so that former methpod is used.
//
// Revision 1.5  1998/06/05 13:51:54  bareau
// Added an NLDModule to deal with pure nld files.
//
// Revision 1.4  1998/05/11 09:37:50  cinard
// Overloaded newTranslationUnit() method.
//
// Revision 1.3  1998/04/28 12:17:46  cinard
// Modified newReference() and newPointer() methods.
//
// Revision 1.2  1998/03/25 14:49:56  cinard
// Updated ADLClass constructor and newADLClass() method (now adl
// super class name is required).
//
// Revision 1.1  1998/03/18 15:51:27  cinard
// Symbol factory in now separated from symbol table.
//
// ***********************************************************************
