// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/SymbolId.java,v $
// $Revision: 1.4 $
// $Date: 1998/06/05 13:51:54 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        SymbolId.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 16,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

/**
 * Symbol's identifier.
 */

public class SymbolId
{
  // -- Predefined bit masks for symbols families --

  public final static long MODULE_MASK           = 0x00000000FFL;
  public final static long EXPRESSION_MASK       = 0x000000FF00L;
  public final static long TYPE_MASK             = 0x00FFFF0000L;
  public final static long OPERATION_MASK        = 0xFF00000000L;

  // -- Predefined identifiers for symbols --

  // -- module --

  public final static long COMPILATION_UNIT             = 1L << 0;
  public final static long PACKAGE                      = 1L << 1;
  public final static long NAME_SPACE                   = 1L << 2;
  public final static long NLD_MODULE                   = 1L << 3;

  // -- expressions --

  public final static long TDD_DATASET                  = 1L << 8;
  public final static long ADL_BINDING                  = 1L << 9;
  public final static long ATTRIBUTE                    = 1L << 10;
  public final static long PARAMETER                    = 1L << 11;
  public final static long VARIABLE                     = 1L << 12;

  // -- types --

  public final static long BUILTIN_TYPE                 = 1L << 16;
  public final static long CLASS                        = 1L << 17;
  public final static long ARRAY                        = 1L << 18;
  public final static long OBJECT_ARRAY                 = 1L << 19;
  public final static long STRUCT                       = 1L << 20;
  public final static long POINTER                      = 1L << 21;
  public final static long POINTER_TO_FUNCTION          = 1L << 22;
  public final static long POINTER_TO_FUNCTION_MEMBER   = 1L << 23;
  public final static long POINTER_TO_FIELD_MEMBER      = 1L << 24;
  public final static long REFERENCE                    = 1L << 25;
  public final static long REFERENCE_TO_FIELD_MEMBER    = 1L << 26;
  public final static long REFERENCE_TO_FUNCTION        = 1L << 27;
  public final static long UNION                        = 1L << 28;
  public final static long ENUM                         = 1L << 29;
  public final static long FUNCTION_TYPE                = 1L << 30;

  // -- operations

  public final static long TDD_FACTORY                  = 1L << 32;
  public final static long ADL_INLINE                   = 1L << 33;
  public final static long ADL_METHOD                   = 1L << 34;
  public final static long FUNCTION                     = 1L << 35;
  public final static long METHOD                       = 1L << 36;
  public final static long CONSTRUCTOR                  = 1L << 37;
  public final static long DESTRUCTOR                   = 1L << 38;

  // -- other test specific symbols identifier

  public final static long ADL_CLASS                   = 1L << 40;
  public final static long TDD_CLASS                   = 1L << 41;

  // -- constructed structures mask --

  public final static long CONSTRUCTED_MASK = CLASS 
                                            | STRUCT
                                            | UNION
                                            | TDD_CLASS
                                            | ADL_CLASS;

  /**
   * String representation for a symbol identifier mask.
   */

  public final static String stringize(long symbolId)
  {
     String s = "";

     if ((symbolId & COMPILATION_UNIT)!= 0 ) s += "COMPILATION_UNIT ";
     if ((symbolId & PACKAGE)!= 0 )          s += "PACKAGE ";
     if ((symbolId & NAME_SPACE)!= 0 )       s += "NAME_SPACE ";

     // -- expressions --

     if ((symbolId & TDD_DATASET)!= 0 )   s += "TDD_DATASET ";
     if ((symbolId & ADL_BINDING)!= 0 )   s += "ADL_BINDING ";
     if ((symbolId & ATTRIBUTE)!= 0 )     s += "ATTRIBUTE ";
     if ((symbolId & PARAMETER)!= 0 )     s += "PARAMETER ";
     if ((symbolId & VARIABLE)!= 0 )      s += "VARIABLE ";

     // -- types --

     if ((symbolId & BUILTIN_TYPE)!= 0 )  s += "BUILTIN_TYPE ";
     if ((symbolId & CLASS)!= 0 )         s += "CLASS ";
     if ((symbolId & ARRAY)!= 0 )         s += "ARRAY ";
     if ((symbolId & OBJECT_ARRAY)!= 0 )  s += "OBJECT_ARRAY ";
     if ((symbolId & STRUCT)!= 0 )        s += "STRUCT ";
     if ((symbolId & POINTER)!= 0 )       s += "POINTER ";
     if ((symbolId & POINTER_TO_FUNCTION)!= 0 )    
                                     s += "POINTER_TO_FUNCTION ";
     if ((symbolId & POINTER_TO_FUNCTION_MEMBER)!= 0 ) 
                                     s += "POINTER_TO_FUNCTION_MEMBER ";
     if ((symbolId & POINTER_TO_FIELD_MEMBER)!= 0 )
                                     s += "POINTER_TO_FIELD_MEMBER ";
     if ((symbolId & REFERENCE)!= 0 )     s += "REFERENCE ";
     if ((symbolId & REFERENCE_TO_FIELD_MEMBER)!= 0 )  
                                     s += "REFERENCE_TO_FIELD_MEMBER ";
     if ((symbolId & REFERENCE_TO_FUNCTION)!= 0 )  
                                     s += "REFERENCE_TO_FUNCTION ";
     if ((symbolId & UNION)!= 0 )         s += "UNION ";
     if ((symbolId & ENUM)!= 0 )          s += "ENUM ";
     if ((symbolId & FUNCTION_TYPE)!= 0 ) s += "FUNCTION_TYPE ";

     // -- operations
     
     if ((symbolId & TDD_FACTORY)!= 0 )      s += "TDD_FACTORY ";
     if ((symbolId & ADL_INLINE)!= 0 )       s += "ADL_INLINE ";
     if ((symbolId & ADL_METHOD)!= 0 )       s += "ADL_METHOD ";
     if ((symbolId & FUNCTION)!= 0 )         s += "FUNCTION ";
     if ((symbolId & METHOD)!= 0 )           s += "METHOD ";
     if ((symbolId & CONSTRUCTOR)!= 0 )      s += "CONSTRUCTOR ";
     if ((symbolId & DESTRUCTOR)!= 0 )       s += "DESTRUCTOR ";

     // -- other test specific symbols identifier

     if ((symbolId & ADL_CLASS)!= 0 )        s += "ADL_CLASS ";
     if ((symbolId & TDD_CLASS)!= 0 )        s += "TDD_CLASS ";
     
     return s.trim();
  }

}

// ***********************************************************************
// History Log:
// $Log: SymbolId.java,v $
// Revision 1.4  1998/06/05 13:51:54  bareau
// Added an NLDModule to deal with pure nld files.
//
// Revision 1.3  1998/03/18 15:50:42  cinard
// stringize() method added.
//
// Revision 1.2  1998/02/27 12:48:44  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 15:09:08  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// ***********************************************************************
