// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/SymbolSelector.java,v $
// $Revision: 1.4 $
// $Date: 1998/06/16 14:16:39 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        SymbolSelector.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Mar 17,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;


/**
 * Predicate enabling to select symbols.
 */ 

public class SymbolSelector implements UnaryPredicate
{

  /**
   * Constructs the symbol selector.
   * @param symbolIdMask bit mask made up of selectable symbols identifiers.
   */ 

  public SymbolSelector(long symbolIdMask)
  {
     _symbolIdMask = symbolIdMask;
  }
  
  /**
   * Returns true if executed against an operation symbol whose signature
   * is equals with that of the comparator signature, false otherwise.
   */

  public boolean execute(Object object)
  {
     return    ( object instanceof Symbol ) 
            && ( ((Symbol)object).getSymbolId() & _symbolIdMask ) != 0 ;
  }

  /**
   * Can this comparator selects symbols with supplied identifier.
   */

  public boolean canSelect(long symbolIdMask)
  {
     return ( _symbolIdMask & symbolIdMask ) != 0;
  }

  /**
   * Set symbol selector mask.
   */

  public void setSymbolIdMask(long symbolIdMask)
  {
     _symbolIdMask = symbolIdMask;
  }

  /**
   * Get symbol selector mask.
   */
  public long getSymbolIdMask() { return _symbolIdMask; }


  /** 
   * Returns a string describing me.
   */ 

  public String toString()
  {
     return  "SymbolSelector <" + SymbolId.stringize(_symbolIdMask) + ">" ;
  }

  // -- Predefined symbol selectors --

   public final static SymbolSelector MODULE_SELECT 
        = new SymbolSelector(SymbolId.MODULE_MASK);

   public final static SymbolSelector EXPRESSION_SELECT 
        = new SymbolSelector(SymbolId.EXPRESSION_MASK);

   public final static SymbolSelector TYPE_SELECT 
        = new SymbolSelector(SymbolId.TYPE_MASK);

   public final static SymbolSelector OPERATION_SELECT 
        = new SymbolSelector(SymbolId.OPERATION_MASK);

   public final static SymbolSelector COMPILATION_UNIT_SELECT 
        = new SymbolSelector(SymbolId.COMPILATION_UNIT);

   public final static SymbolSelector PACKAGE_SELECT 
        = new SymbolSelector(SymbolId.PACKAGE);

   public final static SymbolSelector NAME_SPACE_SELECT 
        = new SymbolSelector(SymbolId.NAME_SPACE);

   public final static SymbolSelector TDD_DATASET_SELECT 
        = new SymbolSelector(SymbolId.TDD_DATASET);

   public final static SymbolSelector ADL_BINDING_SELECT 
        = new SymbolSelector(SymbolId.ADL_BINDING);

   public final static SymbolSelector ATTRIBUTE_SELECT 
        = new SymbolSelector(SymbolId.ATTRIBUTE);

   public final static SymbolSelector PARAMETER_SELECT 
        = new SymbolSelector(SymbolId.PARAMETER);

   public final static SymbolSelector VARIABLE_SELECT 
        = new SymbolSelector(SymbolId.VARIABLE);

   public final static SymbolSelector BUILTIN_TYPE_SELECT 
        = new SymbolSelector(SymbolId.BUILTIN_TYPE);

   public final static SymbolSelector CLASS_SELECT 
        = new SymbolSelector(SymbolId.CLASS);

   public final static SymbolSelector ARRAY_SELECT 
        = new SymbolSelector(SymbolId.ARRAY);

   public final static SymbolSelector OBJECT_ARRAY_SELECT 
        = new SymbolSelector(SymbolId.OBJECT_ARRAY | SymbolId.ARRAY);

   public final static SymbolSelector STRUCT_SELECT 
        = new SymbolSelector(SymbolId.STRUCT);

   public final static SymbolSelector POINTER_SELECT 
        = new SymbolSelector(SymbolId.POINTER);

   public final static SymbolSelector POINTER_TO_FUNCTION_SELECT 
        = new SymbolSelector(SymbolId.POINTER_TO_FUNCTION);

   public final static SymbolSelector POINTER_TO_FUNCTION_MEMBER_SELECT 
        = new SymbolSelector(SymbolId.POINTER_TO_FUNCTION_MEMBER);

   public final static SymbolSelector POINTER_TO_FIELD_MEMBER_SELECT 
        = new SymbolSelector(SymbolId.POINTER_TO_FIELD_MEMBER);

   public final static SymbolSelector REFERENCE_SELECT 
        = new SymbolSelector(SymbolId.REFERENCE);

   public final static SymbolSelector REFERENCE_TO_FIELD_MEMBER_SELECT 
        = new SymbolSelector(SymbolId.REFERENCE_TO_FIELD_MEMBER);

   public final static SymbolSelector REFERENCE_TO_FUNCTION_SELECT 
        = new SymbolSelector(SymbolId.REFERENCE_TO_FUNCTION);

   public final static SymbolSelector UNION_SELECT 
        = new SymbolSelector(SymbolId.UNION);

   public final static SymbolSelector ENUM_SELECT 
        = new SymbolSelector(SymbolId.ENUM);

   public final static SymbolSelector ADL_CLASS_SELECT 
        = new SymbolSelector(SymbolId.ADL_CLASS);

   public final static SymbolSelector TDD_CLASS_SELECT 
        = new SymbolSelector(SymbolId.TDD_CLASS);


  // -- Private fields --

  /** Symbol selector mask. */

  private long _symbolIdMask;

}

// ***********************************************************************
// History Log:
// $Log: SymbolSelector.java,v $
// Revision 1.4  1998/06/16 14:16:39  cinard
// Added getSymbolIdMask() method.
//
// Revision 1.3  1998/04/28 12:18:53  cinard
// Modified a constructor visibility.
//
// Revision 1.2  1998/03/19 16:42:56  cinard
// Fixed a bug in SymbolTablMgr.getMethod() function with CPP.
//
// Revision 1.1  1998/03/18 15:49:52  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// ***********************************************************************
