// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/TDDDataset.java,v $
// $Revision: 1.6 $
// $Date: 1998/03/18 15:49:59 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        TDDDataset.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable;

import com.objectspace.jgl.UnaryPredicate;

/**
 * TDDDataset symbol.
 */

public class TDDDataset extends LExpression
{

  /**
   * Constructs a TDD dataset.
   * @param name name of the variable.
   * @param mySize size of the dataset.
   * @param modifier modifier flag (bit mask where Modifier.PUBLIC...
   *                     can be set).
   * @param type type of the dataset.
   */

  public TDDDataset(String name,int mySize, LModifier modifier, LType type)
  {
     super(name,modifier,type,SymbolId.TDD_DATASET);
     this.mySize = mySize;
  }

  // -- LExpression virtual methods implementation --  

  /**
   * Returns a header for the description of the expression.
   */
 
  public String header() { return "dataset" ; }

  // -- Specific methods --

  /** 
   * Returns dataset size.
   */

  public int getSize() { return this.mySize; }

  /** 
   * Sets dataset size.
   * @param mySize dataset's size.
   */

  public void setSize(int mySize) { this.mySize = mySize; }

  /**
   * Get a comparator enabling to select a symbol with the same
   * runtime class than that of the symbol against which this method
   * is called.
   * @return the comparator.
   */

  public UnaryPredicate getComparator()
  {
     return SymbolSelector.TDD_DATASET_SELECT;
  }

  // -- Private fields --

  /** Dataset's size. */

  private int mySize = 0;

}

// ***********************************************************************
// History Log:
// $Log: TDDDataset.java,v $
// Revision 1.6  1998/03/18 15:49:59  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.5  1998/02/27 12:48:46  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
// Updated copyright notice.
//
// Revision 1.4  1998/02/23 15:09:09  cinard
// Modified symbols inheritance tree. Now LType and its subclasses are
// interfaces. New L...Impl classes provide implementation for these
// interfaces. New L...Typedef classes provide implementations for
// typedefs (C and C++).
//
// Typedefs are created using the SymbolTableManager.newTypedef()
// method.
//
// Revision 1.3  1998/02/12 16:03:22  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/12 13:19:15  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.1  1997/11/26 17:13:07  cinard
// Changed package name.
//
// ***********************************************************************
