// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CBuiltinType.java,v $
// $Revision: 1.26 $
// $Date: 1998/07/28 13:48:50 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CBuiltinType.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 29,  1997: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;


import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;


/**
 * C builtin type class.
 */

public class CBuiltinType extends LBuiltinTypeImpl
{

  /**
   * Constructs a C builtin type.
   * @param name name of the type.
   */

  private CBuiltinType(String name)
  {
     super(name);
  }

  /**
   * Get a builtin type.
   * @param name name of the builtin type.
   * @return the found builtin type (null if no type is found)
   */

  public static LBuiltinType getBuiltinType(String name)
  {
     int     bits  = 0x0000;           // Builtin types bits
     Integer oneBit;                   // Store a single bit
     String  token;
     StringTokenizer st = new StringTokenizer(name);
     while (st.hasMoreTokens())
     {
        token  = st.nextToken();
        oneBit = (Integer)builtinTypeConstants.get(token);
        if ( oneBit != null )
        {
           bits |= oneBit.intValue();
        }
        else
        {
           return null;
        }
     }

     // Accept valid builtin types
     String btName;
     if      ( bits == CHAR )
     { 
        btName = "char";
     }
     else if ( bits == ( SIGNED | CHAR )) 
     { 
        btName = "signed char";
     }
     else if (    ( bits == ( UNSIGNED | CHAR )))
     {
        btName = "unsigned char";
     }
     else if (    ( bits == (          SHORT       ))
               || ( bits == (          SHORT | INT ))
               || ( bits == ( SIGNED | SHORT       ))
               || ( bits == ( SIGNED | SHORT | INT ))
             )
     {
        btName = "short"; 
     }
     else if (    ( bits == ( UNSIGNED | SHORT       ))
               || ( bits == ( UNSIGNED | SHORT | INT ))
             )
     {
        btName = "unsigned short"; 
     }
     else if (     ( bits == (          INT ))
                || ( bits == ( SIGNED | INT ))
                || ( bits == ( SIGNED       ))
             )
     {
        btName = "int"; 
     }
     else if (     ( bits == (UNSIGNED | INT ))
                || ( bits == (UNSIGNED      ))
             )
     {
        btName = "unsigned int"; 
     }
     else if (    ( bits == (          LONG      ))
               || ( bits == (         LONG | INT ))
               || ( bits == (SIGNED | LONG       ))
               || ( bits == (SIGNED | LONG | INT ))
             )
     {
        btName = "long"; 
     }
     else if (    ( bits == (UNSIGNED | LONG      ))
               || ( bits == (UNSIGNED | LONG | INT))
             )
     {
        btName = "unsigned long"; 
     }
     else if (    ( bits ==   FLOAT))
     {
        btName = "float"; 
     }
     else if (    ( bits ==   VOID))
     {
        btName = "void"; 
     }
     else if (     bits ==  DOUBLE ) 
     {
        btName = "double"; 
     }
     else if ( bits == (LONG | DOUBLE))
     {
        btName = "long double" ;
     }
     else
     {
        return null;
     }

     return (LBuiltinType) builtinTypes.get( btName);
  }

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  static public boolean isValidBuiltinTypeName(String name)
  {

     return getBuiltinType(name) != null ;
  }

  /**
    * Returns true if this type is equals to type t.
    * Note that if t is null it is considered equals to voidT.
    * @param t symbol for which the test is performed.
    */

  public boolean equals(Object t)
  {
     if ( t == null )
     {
        return ( super.equals(voidT) );
     }
     else
     {
        return super.equals(t);
     }
  }


  /**
   * Returns synthesis type of the @param t and type for which method applies.
   * <p><b>Algorithm:</b><ul>
   * <li> If type <code>t</code> return null.
   * <li> When type <code>t</code> is an LPointer return it.
   * <li> If <code>t</code> is builtintype =>Apply C++/ANSI type compatibility
   * </ul>
   * @param t type to synthetise with this type.
   * @return is the smallest type assignable to both types,
   *         is null is no common assignable type found.
   */ 
  public LType synthesisType( LType t) {
  
    LType returnType;
    if( t== null  ) {

      return null;
    } 
    else if ( t instanceof LPointer ) {
      return t ;
    }
    else if ( t instanceof LReference ) {
      return this.synthesisType( ((LReference)t).getReferenced() ) ;
    }
    else if ( t instanceof LTypedef ) {
      return this.synthesisType( t.getRealType() ) ;
    }
    else if (! ( t instanceof CBuiltinType) ) {
      return null ;
    }
    else if( this.equals( t) ) {

      returnType = t;

    }
    else if ( this.equals( ldoubleT ) || t.equals(ldoubleT ) ) {

      returnType= ldoubleT ;
    }
    else if ( this.equals( doubleT)) {
 
	  returnType= doubleT;
    }
    else if ( this.equals( floatT) ) {
	
	if(t.equals(ushortT ) || t.equals( ucharT )|| t.equals(charT) 
	   || t.equals(shortT)) {
	  returnType = floatT;
	} else {
	  returnType = doubleT;
	}
    }
    else if( this.equals( intT) ) {
      if(  t.equals( ushortT) || t.equals( ucharT) || t.equals( shortT) 
	   || t.equals( charT) || ( t instanceof CEnum )  )
      {
       
	returnType = intT;

      } else if ( t.equals ( longT) || t.equals( uintT) ) {
	returnType = longT;
      } 
      else {
	returnType = doubleT;
      }
    }
    else if( this.equals( longT) ) {
      if( t.equals( uintT) || t.equals( ushortT) || t.equals( ucharT) || t.equals( shortT) || t.equals( charT)||t.equals ( longT) || t.equals( intT)  ) {
       
	returnType = longT;

      }
      else {
	returnType = doubleT;
      }
    }
   else if( this.equals( shortT) ) {
      if(  t.equals( ucharT) ||  t.equals( charT)   ) {
       
	returnType = shortT;

      }
      else if( t.equals( ushortT) ||  t.equals( intT) ) {
	returnType = intT;
      }
      else if(  t.equals( longT) || t.equals( floatT)  ) {
	returnType = t;
      }
      else {
	returnType = doubleT;
      }
   }
   else if( this.equals( charT) ) {
      if(  t.equals( shortT) ||  t.equals( intT) || t.equals( longT) ||  t.equals( floatT)  ) {
       
	returnType = t;

      }

      else {
	returnType = doubleT;
      }

   }
   else if( this.equals( uintT) ) {
       if( t.equals( intT) || t.equals( ushortT) || t.equals( ucharT) ) {
       
	returnType = uintT;

      } else if ( t.equals ( ulongT)  ) {
	returnType = ulongT;
      } 
      else {
	returnType = doubleT;
      }
   }     
    else if( this.equals(ulongT) ) {
      if( t.equals( uintT) || t.equals( ushortT) || t.equals( ucharT)   ) {
       
	returnType = ulongT;

      }
      else {
	
	returnType = doubleT;
      }
    }
    else if( this.equals(ushortT) ) {
      if( t.equals( intT) || t.equals( longT) || t.equals( floatT)   ) {
       
	returnType = t;
      }
      else if ( t.equals( ucharT) ) {
	
	returnType = ushortT;
      }
      else {
	
	returnType = doubleT;
      }
    }
    else if( this.equals ( ucharT) ) {
      if( t.equals ( charT) || t.equals( doubleT)) {
	returnType = doubleT;
      }
      else {
	returnType = t;
      }
    }
    else {
      returnType = null;
    }
   
   return returnType; 

  }
      
  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
     boolean isAssignable =     ( t instanceof LType )
                             && ( ! CBuiltinType.voidT.equals(t) )
                             && CType.isArithmetic((LType)t);
   
     if ( debug )
     {
        System.out.println("debug: CBuiltinType<" + this + ">.assignable("
                          + t + ")"
                          + " return " + isAssignable );
     }

     return isAssignable;
  }

  /**
   * Returns true if type is void.
   */

  public boolean isVoid()
  {
     return this.equals(voidT);
  }

  // -- Builtin types --

  public static final CBuiltinType voidT  = new CBuiltinType("void");

  public static final CBuiltinType charT  = new CBuiltinType("char");
  public static final CBuiltinType scharT = new CBuiltinType("signed char");
  public static final CBuiltinType ucharT = new CBuiltinType("unsigned char");
  public static final CBuiltinType shortT = new CBuiltinType("short");
  public static final CBuiltinType ushortT= new CBuiltinType("unsigned short");

  public static final CBuiltinType intT   = new CBuiltinType("int");
  public static final CBuiltinType uintT  = new CBuiltinType("unsigned int");

  public static final CBuiltinType longT  = new CBuiltinType("long");
  public static final CBuiltinType ulongT = new CBuiltinType("unsigned long");

  public static final CBuiltinType floatT   = new CBuiltinType("float");
  public static final CBuiltinType doubleT  = new CBuiltinType("double");
  public static final CBuiltinType ldoubleT = new CBuiltinType("long double"); 
      
  static
  {
     builtinTypes.put(voidT.getName(),    voidT);

     builtinTypes.put(charT.getName(),    charT);
     builtinTypes.put(scharT.getName(),   scharT);
     builtinTypes.put(ucharT.getName(),   ucharT);

     builtinTypes.put(shortT.getName(),   shortT);
     builtinTypes.put(ushortT.getName(),  ushortT);

     builtinTypes.put(intT.getName(),     intT);
     builtinTypes.put(uintT.getName(),    uintT);

     builtinTypes.put(longT.getName(),    longT);
     builtinTypes.put(ulongT.getName(),   ulongT);

     builtinTypes.put(floatT.getName(),   floatT);

     builtinTypes.put(doubleT.getName(),  doubleT);
     builtinTypes.put(ldoubleT.getName(), ldoubleT);
  }

  // To check for builtin type correctness

  private static final int SIGNED   = 0x0001;
  private static final int UNSIGNED = 0x0002;
  private static final int CHAR     = 0x0004;
  private static final int SHORT    = 0x0008;
  private static final int INT      = 0x0010;
  private static final int LONG     = 0x0020;
  private static final int FLOAT    = 0x0040;
  private static final int DOUBLE   = 0x0080;
  private static final int VOID     = 0x0100;

  private static Hashtable builtinTypeConstants = new Hashtable();

  static 
  {  
     builtinTypeConstants.put("signed",  new Integer(SIGNED));
     builtinTypeConstants.put("unsigned",new Integer(UNSIGNED));
     builtinTypeConstants.put("char",    new Integer(CHAR));
     builtinTypeConstants.put("short",   new Integer(SHORT));
     builtinTypeConstants.put("int",     new Integer(INT));
     builtinTypeConstants.put("long",    new Integer(LONG));
     builtinTypeConstants.put("float",   new Integer(FLOAT));
     builtinTypeConstants.put("double",  new Integer(DOUBLE));
     builtinTypeConstants.put("void",    new Integer(VOID));
  }

  private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: CBuiltinType.java,v $
// Revision 1.26  1998/07/28 13:48:50  mallet
// Fixed bug in synthesisType: added a case for 'typedef' to synthetize
// against real type instead of typedef.
//
// Revision 1.25  1998/04/28 12:22:13  cinard
// C++ builtin types are now defined in CPPBuiltinType.
// CBuiltinType is dedicated to C builitin types.
//
// Revision 1.24  1998/02/27 14:25:11  wassel
// Fixed bug : bool saved as an Integer(BOOL) ( instaed of INT ) in hashtable.
//
// Revision 1.23  1998/02/27 12:42:10  cinard
// Updated copyright notice.
//
// Revision 1.22  1998/02/25 16:35:50  wassel
// Fixed bug in isValidBuiltinTypeName method.
//
// Revision 1.21  1998/02/23 14:49:46  cinard
// Added support for typedefs.
//
// Revision 1.20  1998/02/12 16:03:25  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.19  1998/02/11 16:49:31  wassel
// Adding "bool" as valid value in isValidBuiltinTypeName method.
//
// Revision 1.18  1998/02/11 10:41:29  wassel
// Type boolT is not an alias of intT any more.
// Changed methods synthesisType and assignable to take into account type compatibility with type boolT.
//
// Revision 1.17  1998/02/05 11:05:16  wassel
// Modified assignable methods to fully implement C++ ANSI type standard conversion.
//
// Revision 1.16  1998/02/03 13:31:11  wassel
// Modified assignable mthod to support type compatibility with references.
//
// Revision 1.14  1998/01/30 12:34:03  wassel
// Added CFunctionType.java to handle int (x)( char * ).
// Added a factory in symboltable for CFnctionType.
//
// Revision 1.13  1998/01/22 11:42:30  cinard
// Fixed a bug in symbols equality checks.
//
// Revision 1.12  1998/01/15 14:18:18  wassel
// All builtintypes are now public.
//
// Revision 1.11  1998/01/14 12:08:57  wassel
// boolean type boolT is now public.
//
// Revision 1.10  1998/01/12 13:19:17  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.9  1998/01/06 13:19:09  wassel
// Added methods synthesisType for type checking
//
// Revision 1.8  1997/12/08 18:44:31  wassel
// fixed a small bug regarding void type
//
// Revision 1.7  1997/12/08 17:13:13  cinard
// Fixed some issues with C++ builtin types.
//
// Revision 1.6  1997/12/08 12:46:30  cinard
// Calls to the ORG.opengroup.adl.symboltable.SymbolTableMgr.newArray()
// have been modified. The function returns now a LType instead of a
// LArray.
//
// Revision 1.5  1997/11/26 17:13:09  cinard
// Changed package name.
//
// Revision 1.4  1997/11/04 16:04:26  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.3  1997/11/03 15:43:12  cinard
// Added CBuiltinType class.
//
// ***********************************************************************
