// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CFunctionTypeImpl.java,v $
// $Revision: 1.4 $
// $Date: 1998/04/28 12:23:58 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CFunctionTypeImpl.java
//
// HISTORY:
//      Wassel BERRAYANA, TOG - January,  1998: Created
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 *  Symbol function returning Type.
 */

public class CFunctionTypeImpl extends LTypeImpl
                               implements CFunctionType
{

  /**
   * Constructs a function returning type symbol.
   * <p> All relevant information concerning the function type signature is
   * hold by <code> comparator</code>.
   * @param comparator Symbol comparator prototype (LCallableEqualsComparator).
   * @param parentClass the enclosing class when exits.
   */

  public CFunctionTypeImpl(LCallableEqualsComparator comparator,
		                   LConstructedType parentClass )
  {
     super("" , new LModifier(0x0000), SymbolId.FUNCTION_TYPE);
     
     // Perform some checks
     
     if ( comparator == null ) 
     {
        throw new Error
        (    "Could not construct function type :"
             + " no valid comparator provided"
        );
     }
     
     this.comparator = comparator;
     this.parentClass = parentClass ;
  }

  /** Symbol's name. */
  
  public String getName()
  {
	 return comparator.toString("");
  }
  
  /**
   * Returns a string for the declaration of an expression of this
   * type. Useful for mixed type and variable name (e.g. pointer
   * to function). Default behaviour is to return "type name".
   * @param name expression's name.
   */

  public String mixNameAndType(String name)
  {
     return comparator.toString("(" + name + ")");
  }

  /**
   * Check if symbol has direct or indirect access to a scope. That is never
   * true for a function type.
   * @return Always false.
   */

  public boolean hasScopeAccess()
  {
     return false;
  }
  
  /**
   * Get the scope the symbol has access to.
   * @return Always throws an error.
   */

  public Scope   getScope()
  {
     throw new Error("CFunctionTypeImpl.getScope() should not be called.");
  }

  /**
   * Get symbol's enclosing construct (e.g. a class, a TDD class,
   * an ADL class...)
   */

  public Symbol getEnclosingConstruct()
  {
     return getEnclosingClass();
  }

  /**
   * Checks if type t is equal to this type.
   * @param t the type for which equality is checked.
   * @return true if it is equals, false otherwise.
   */

  public boolean equals(Object t)
  {     
     return       ( t instanceof CFunctionType )
               && ( comparator.equals
                  (((CFunctionType)t).getFunctionTypeComparator()) ); 
  }

  /**
   * Checks if a callable can be assigned to a variable of this type.
   * @param  t The callable for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
 
    if ( (t == null) || !( t instanceof CFunctionType ) )
      return false ;
    return comparator.equals(((CFunctionType)t).getFunctionTypeComparator());

  }
  
  /**
   * Returns a string description of the type, indented version.
   * @param indent Indentation.
   * @return Description of the type.
   */

  public String toString(String indent)
  {
     return   indent + getName() ;
  }


  /**
   * Returns the return type of function type.
   */

  public LType getFunctionTypeReturnType()
  {
     return comparator.getReturnType();
  }

  /**
   * Set the return type of the function.
   */

  public void setFunctionTypeReturnType(LType type)
  {
     comparator.setReturnType(type);
  }

  /**
   * Returns function type parent class.
   */
  public LConstructedType getEnclosingClass() {
    return this.parentClass ;
  }

  /** Creates a new typedef. */

  public LType newTypedef(String name)
  {
     return new CFunctionTypedef(name,this);
  }
	  
  /**
   * Returns the function type comparator.
   */

   public LCallableEqualsComparator getFunctionTypeComparator() {
     return comparator ;
   }

  // -- Private fields --

  /** Comparator of Function Type */
  protected LCallableEqualsComparator comparator;

  /** function type enclosing class */
  protected LConstructedType parentClass ; 
  /** Debug flag. */
  protected static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: CFunctionTypeImpl.java,v $
// Revision 1.4  1998/04/28 12:23:58  cinard
// Added setFunctionTypeReturnType() method.
// Improved description methods (toString() and the likes).
//
// Revision 1.3  1998/03/18 15:50:01  cinard
// New strategy for symbol selection (a single SymbolSelector class
// is now used). Symbol classes updated acordingly.
//
// Revision 1.2  1998/02/27 12:42:12  cinard
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 14:49:48  cinard
// Added support for typedefs.
//
// ***********************************************************************
