// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CModifier.java,v $
// $Revision: 1.10 $
// $Date: 1998/02/27 12:42:13 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CModifier.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 21,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 * C modifiers.
 */
public final class CModifier extends LModifier
{
  // -- Constructor(s) --

  /** 
   * Constructs a modifier from an int bit mask.
   * @param modifier the bit mask for the modifier .
   */
  
  public CModifier(int modifier)
  {
     super(modifier);
  }

  // -- Constants --

  /** Public modifier constant. */
  public static final int PUBLIC =	 0x0001;

  /** Private modifier constant. */
  public static final int PRIVATE =	 0x0002;

  /** Protected modifier constant. */
  public static final int PROTECTED =	 0x0004;

  /** Static modifier constant. */
  public static final int STATIC =	 0x0008;

  /** Constant modifier constant. */
  public static final int CONST =	 0x0010;

  /** volatile modifier volatile. */
  public static final int VOLATILE =	 0x0020;

  /** typedef modifier typedef. */
  public static final int TYPEDEF =	 0x0040;

  /** auto modifier  */
  public static final int AUTO =	 0x0080;

  /** register modifier  */
  public static final int REGISTER =	 0x0100;

  /** friend modifier  */
  public static final int FRIEND =	 0x0200;

  /** extern modifier  */
  public static final int EXTERN =	 0x0400;

  /** inline modifier  */
  public static final int INLINE =	 0x0800;

  /** virtual modifier  */
  public static final int VIRTUAL =	 0x1000;



  // -- Public methods --

  /**
   * Checks if the modifier flag indicates public.
   * @return	   true if it indicates public, false otherwise.
   */

  public boolean isPublic() 
  {
     return ((modifier & PUBLIC) != 0);
  }

  /**
   * Checks if the modifier flag indicates private.
   * @return	   true if it indicates private, false otherwise
   */

  public boolean isPrivate()
  {
     return ((modifier & PRIVATE) != 0);
  }

  /**
   * Checks if the modifier flag indicates protected.
   * @return	   true if it indicates protected, false otherwise
   */

  public boolean isProtected()
  {
     return ((modifier & PROTECTED) != 0);
  }

  /**
   * Checks if the modifier flag indicates static.
   * @return	   true if it indicates static, false otherwise
   */

  public boolean isStatic()
  {
     return ((modifier & STATIC) != 0);
  }

  /**
   * Checks if the modifier flag indicates constant.
   * @return	   true if it indicates constant, false otherwise
   */

  public boolean isConstant() 
  {
     return ((modifier & CONST) != 0);
  }

  /**
   * Checks if the modifier flag indicates volatile.
   * @return	   true if it indicates volatile, false otherwise
   */

  public boolean isVolatile()
  {
     return ((modifier & VOLATILE) != 0);
  }

  /**
   * Checks if the modifier flag indicates typedef.
   * @return	   true if it indicates typedef, false otherwise
   */

  public boolean isTypedef()
  {
     return ((modifier & TYPEDEF) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates inline.
   * @return	   true if it indicates inline, false otherwise
   */

  public boolean isInline()
  {
     return ((modifier & INLINE) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates virtual.
   * @return	   true if it indicates virtual, false otherwise
   */

  public boolean isVirtual()
  {
     return ((modifier & VIRTUAL) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates friend.
   * @return	   true if it indicates friend, false otherwise
   */

  public boolean isFriend()
  {
     return ((modifier & FRIEND) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates register.
   * @return	   true if it indicates register, false otherwise
   */

  public boolean isRegister()
  {
     return ((modifier & REGISTER) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates auto.
   * @return	   true if it indicates auto, false otherwise
   */

  public boolean isAuto()
  {
     return ((modifier & AUTO) != 0);
  }
  
  /**
   * Checks if the modifier flag indicates extern.
   * @return	   true if it indicates extern, false otherwise
   */

  public boolean isExtern()
  {
     return ((modifier & EXTERN) != 0);
  }
  
  /**
   * Returns a string describing the modifier flag.
   */

  public String toString() 
  {
     String modifierStr = "";
     
     if      (isPublic())    modifierStr = modifierStr + "public ";
     else if (isPrivate())   modifierStr = modifierStr + "private ";
     else if (isProtected()) modifierStr = modifierStr + "protected ";

     if (isStatic())         modifierStr = modifierStr + "static ";
     if (isConstant())       modifierStr = modifierStr + "const ";
     if (isVolatile())       modifierStr = modifierStr + "volatile ";
     if (isAuto())       modifierStr = modifierStr + "auto ";
     if (isExtern())       modifierStr = modifierStr + "extern ";
     if (isRegister())       modifierStr = modifierStr + "register ";
     if (isFriend())       modifierStr = modifierStr + "friend ";
     if (isVirtual())       modifierStr = modifierStr + "virtual ";
     if (isTypedef())       modifierStr = modifierStr + "typedef ";
     if (isInline())       modifierStr = modifierStr + "inline ";
     
     return modifierStr;
  }
  
}


// ***********************************************************************
// History Log:
// $Log: CModifier.java,v $
// Revision 1.10  1998/02/27 12:42:13  cinard
// Updated copyright notice.
//
// Revision 1.9  1998/02/12 16:03:27  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.8  1998/01/12 13:19:17  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.7  1998/01/06 13:26:11  wassel
// Added modifiers
//
// Revision 1.6  1997/12/08 15:14:15  wassel
// Added all the C++ modifiers
//
// Revision 1.5  1997/11/26 17:13:09  cinard
// Changed package name.
//
// Revision 1.4  1997/11/12 13:36:08  cinard
// Fixed some bugs (void type && assignability).
//
// Revision 1.3  1997/11/03 15:58:40  cinard
// Added CModifier class (modifier is a bit mask made up of PUBLIC, PRIVATE,
// STATIC...
//
//
// ***********************************************************************
