// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CPPBuiltinType.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 13:48:50 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CPPBuiltinType.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group - Apr 16,  1997: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;


import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;


/**
 * C++ builtin type class.
 */

public class CPPBuiltinType extends LBuiltinTypeImpl
{

  /**
   * Constructs a C++ builtin type.
   * @param name name of the type.
   */

  private CPPBuiltinType(String name)
  {
     super(name);
  }

  /**
   * Get a builtin type.
   * @param name name of the builtin type.
   * @return the found builtin type (null if no type is found)
   */

  public static LBuiltinType getBuiltinType(String name)
  {
     int     bits  = 0x0000;           // Builtin types bits
     Integer oneBit;                   // Store a single bit
     String  token;
     StringTokenizer st = new StringTokenizer(name);
     while (st.hasMoreTokens())
     {
        token  = st.nextToken();
        oneBit = (Integer)builtinTypeConstants.get(token);
        if ( oneBit != null )
        {
           bits |= oneBit.intValue();
        }
        else
        {
           return null;
        }
     }

     // Accept valid builtin types
     String btName;
     if      (    ( bits == (          CHAR ))
               || ( bits == ( SIGNED | CHAR ))
             ) 
     { 
        btName = "char";
     }
     else if (    ( bits == ( UNSIGNED | CHAR )))
     {
        btName = "unsigned char";
     }
     else if (    ( bits == (          SHORT       ))
               || ( bits == (          SHORT | INT ))
               || ( bits == ( SIGNED | SHORT       ))
               || ( bits == ( SIGNED | SHORT | INT ))
             )
     {
        btName = "short"; 
     }
     else if (    ( bits == ( UNSIGNED | SHORT       ))
               || ( bits == ( UNSIGNED | SHORT | INT ))
             )
     {
        btName = "unsigned short"; 
     }
     else if (     ( bits == (          INT ))
                || ( bits == ( SIGNED | INT ))
                || ( bits == ( SIGNED       ))
             )
     {
        btName = "int"; 
     }
     else if (     ( bits == (UNSIGNED | INT ))
                || ( bits == (UNSIGNED      ))
             )
     {
        btName = "unsigned int"; 
     }
     else if (    ( bits == (          LONG      ))
               || ( bits == (         LONG | INT ))
               || ( bits == (SIGNED | LONG       ))
               || ( bits == (SIGNED | LONG | INT ))
             )
     {
        btName = "long"; 
     }
     else if (    ( bits == (UNSIGNED | LONG      ))
               || ( bits == (UNSIGNED | LONG | INT))
             )
     {
        btName = "unsigned long"; 
     }
     else if (    ( bits ==   FLOAT))
     {
        btName = "float"; 
     }
     else if (    ( bits ==   VOID))
     {
        btName = "void"; 
     }
     else if ( bits ==  BOOL ) 
     {
       btName = "bool" ;
     }
     else if (     bits ==  DOUBLE ) 
     {
        btName = "double"; 
     }
     else if ( bits == (LONG | DOUBLE))
     {
        btName = "long double" ;
     }
     else
     {
        return null;
     }

     return (LBuiltinType) builtinTypes.get( btName);
  }

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  static public boolean isValidBuiltinTypeName(String name)
  {

     return getBuiltinType(name) != null ;
  }

  /**
    * Returns true if this type is equals to type t.
    * Note that if t is null it is considered equals to voidT.
    * @param t symbol for which the test is performed.
    */

  public boolean equals(Object t)
  {
     if ( t == null )
     {
        return ( super.equals(voidT) );
     }
     else
     {
        return super.equals(t);
     }
  }


  /**
   * Tells if type t can be converted into bool type. According to C++ ANSI :
   * @param t parameter type
   * @return is true if t is LPointer (to member), CPPBuiltinType or LUnion
   */

  static public boolean convertibleToBool (LType t)
  {
    return     ( t instanceof LPointer ) 
            || ( t instanceof LArray ) 
            || ( t instanceof LUnion )
            || ( t instanceof CPPBuiltinType ) ;
  }


  /**
   * Returns synthesis type of the @param t and type for which method applies.
   * <p><b>Algorithm:</b><ul>
   * <li> If type <code>t</code> return null.
   * <li> When type <code>t</code> is an LPointer return it.
   * <li> If <code>t</code> is builtintype =>Apply C++/ANSI type compatibility
   * </ul>
   * @param t type to synthetise with this type.
   * @return is the smallest type assignable to both types,
   *         is null is no common assignable type found.
   */ 
  public LType synthesisType( LType t) {
  
    LType returnType;
    if( t== null  ) {

      return null;
    } 
    else if ( t instanceof LPointer ) {
      return t ;
    }
    else if ( t instanceof LReference ) {
      return this.synthesisType( ((LReference)t).getReferenced() ) ;
    }
    else if ( t instanceof LTypedef ) {
      return this.synthesisType( t.getRealType() ) ;
    }
    else if (! ( t instanceof CPPBuiltinType) ) {
      return null ;
    }
    else if( this.equals( t) ) {

      returnType = t;

    }
    else if ( this.equals( ldoubleT ) || t.equals(ldoubleT ) ) {

      returnType= ldoubleT ;
    }
    else if ( this.equals( doubleT)) {
 
	  returnType= doubleT;
    }
    else if ( this.equals( floatT) ) {
	
	if(t.equals(ushortT ) || t.equals( ucharT )|| t.equals(charT) 
	   || t.equals(shortT)) {
	  returnType = floatT;
	} else {
	  returnType = doubleT;
	}
    }
    else if( this.equals( intT) ) {
      if(  t.equals( ushortT) || t.equals( ucharT) || t.equals( shortT) 
	   || t.equals( charT) || ( t instanceof CEnum ) ||  t.equals(boolT) )
      {
       
	returnType = intT;

      } else if ( t.equals ( longT) || t.equals( uintT) ) {
	returnType = longT;
      } 
      else {
	returnType = doubleT;
      }
    }
    else if ( this.equals(boolT) ) {
      if(  t.equals( ushortT) || t.equals( ucharT) || t.equals( shortT) 
	   || t.equals( charT) || ( t instanceof CEnum ) ||  t.equals(boolT) )
      {
       
	returnType = intT;

      } else if ( t.equals ( longT) || t.equals( uintT) ) {
	returnType = longT;
      } 
      else {
	returnType = doubleT;
      }
    } 
    else if( this.equals( longT) ) {
      if( t.equals( uintT) || t.equals( ushortT) || t.equals( ucharT) || t.equals( shortT) || t.equals( charT)||t.equals ( longT) || t.equals( intT)  ) {
       
	returnType = longT;

      }
      else {
	returnType = doubleT;
      }
    }
   else if( this.equals( shortT) ) {
      if(  t.equals( ucharT) ||  t.equals( charT)   ) {
       
	returnType = shortT;

      }
      else if( t.equals( ushortT) ||  t.equals( intT) ) {
	returnType = intT;
      }
      else if(  t.equals( longT) || t.equals( floatT)  ) {
	returnType = t;
      }
      else {
	returnType = doubleT;
      }
   }
   else if( this.equals( charT) ) {
      if(  t.equals( shortT) ||  t.equals( intT) || t.equals( longT) ||  t.equals( floatT)  ) {
       
	returnType = t;

      }

      else {
	returnType = doubleT;
      }

   }
   else if( this.equals( uintT) ) {
       if( t.equals( intT) || t.equals( ushortT) || t.equals( ucharT) ) {
       
	returnType = uintT;

      } else if ( t.equals ( ulongT)  ) {
	returnType = ulongT;
      } 
      else {
	returnType = doubleT;
      }
   }     
    else if( this.equals(ulongT) ) {
      if( t.equals( uintT) || t.equals( ushortT) || t.equals( ucharT)   ) {
       
	returnType = ulongT;

      }
      else {
	
	returnType = doubleT;
      }
    }
    else if( this.equals(ushortT) ) {
      if( t.equals( intT) || t.equals( longT) || t.equals( floatT)   ) {
       
	returnType = t;
      }
      else if ( t.equals( ucharT) ) {
	
	returnType = ushortT;
      }
      else {
	
	returnType = doubleT;
      }
    }
    else if( this.equals ( ucharT) ) {
      if( t.equals ( charT) || t.equals( doubleT)) {
	returnType = doubleT;
      }
      else {
	returnType = t;
      }
    }
    else {
      returnType = null;
    }
   
   return returnType; 

  }
      
  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
     if ( debug )
        System.out.println("   debug: CPPBuiltinType.assignable() - testing " 
                          + t + " assignable to " + this);
     boolean isAssignable = false;
     if ( t == null )
       return false ;
     if ( t instanceof LReference )
     {
       isAssignable = this.assignable( ((LReference) t).getReferenced() ) ;
     }
     else if ( ! ( t instanceof LBuiltinType ) )
     {
        isAssignable = false;
     }
     else if ( this.equals(t) )
     {
        isAssignable = true;
     }
     else if ( this.equals(shortT) )
     {
        isAssignable =    t.equals(charT) || t.equals(ucharT);
     }
     else if ( this.equals(intT)  )
     {
        isAssignable =    t.equals(charT)  || t.equals(ucharT)
                       || t.equals(shortT) || t.equals(ushortT)
	               || ( t instanceof CEnum) || t.equals(boolT);
     }
     else if ( this.equals(boolT ) ) {
       isAssignable =  ( t instanceof LPointer ) ||(t instanceof LArray) || 
                     ( t instanceof LUnion)||(t instanceof CPPBuiltinType)  ;
     }
     else if ( this.equals(uintT) ) {
        isAssignable =    t.equals(charT)  || t.equals(ucharT)
                       || t.equals(shortT) || t.equals(ushortT) ;
     }
     else if ( this.equals(longT) || this.equals(ulongT) )
     {
        isAssignable =    t.equals(charT)  || t.equals(ucharT)
                       || t.equals(shortT) || t.equals(ushortT)
                       || t.equals(intT)   || t.equals(uintT);
     }
     else if ( this.equals(floatT) || this.equals(ufloatT) )
     {
        isAssignable =    t.equals(charT)  || t.equals(ucharT)
                       || t.equals(shortT) || t.equals(ushortT)
                       || t.equals(intT)   || t.equals(uintT)
                       || t.equals(longT)  || t.equals(ulongT);
     }
     else if ( this.equals(doubleT) || this.equals(udoubleT) )
     {
        isAssignable =    t.equals(charT)  || t.equals(ucharT)
                       || t.equals(shortT) || t.equals(ushortT)
                       || t.equals(intT)   || t.equals(uintT)
                       || t.equals(longT)  || t.equals(ulongT)
                       || t.equals(floatT) || t.equals(ufloatT);
     }

     if ( debug )
        System.out.println( "   debug: CPPBuiltinType.assignable() - Result is " 
                          + isAssignable );

     return isAssignable;
  }

  /**
   * Returns true if type is void.
   */
  public boolean isVoid()
  {
     return this.equals(voidT);
  }

  // -- Builtin types --

   public static final CPPBuiltinType voidT    = new CPPBuiltinType("void");

   public static final CPPBuiltinType charT    = new CPPBuiltinType("char");
   public static final CPPBuiltinType ucharT   = new CPPBuiltinType
                                                  ("unsigned char");
   public static final CPPBuiltinType shortT   = new CPPBuiltinType("short");
   public static final CPPBuiltinType ushortT  = new CPPBuiltinType
                                                  ("unsigned short");

  public static final CPPBuiltinType intT     = new CPPBuiltinType("int");
  public static final CPPBuiltinType boolT    = new CPPBuiltinType("bool");
  public static final CPPBuiltinType uintT    = new CPPBuiltinType
                                                   ("unsigned int");

  public static final CPPBuiltinType longT    = new CPPBuiltinType("long");
  public static final CPPBuiltinType ulongT   = new CPPBuiltinType
                                                   ("unsigned long");

  public static final CPPBuiltinType floatT   = new CPPBuiltinType("float");
  public static final CPPBuiltinType ufloatT  = new CPPBuiltinType
                                                   ("unsigned float");
  public static final CPPBuiltinType doubleT  = new CPPBuiltinType("double");
  public static final CPPBuiltinType udoubleT = new CPPBuiltinType
                                                   ("unsigned double");
  public static final CPPBuiltinType ldoubleT = new CPPBuiltinType
                                                   ("long double"); 
   
  static
  {
     builtinTypes.put(voidT.getName(),    voidT);

     builtinTypes.put(charT.getName(),    charT);
     builtinTypes.put(ucharT.getName(),   ucharT);

     builtinTypes.put(shortT.getName(),   shortT);
     builtinTypes.put(ushortT.getName(),  ushortT);

     builtinTypes.put(boolT.getName(),    boolT);

     builtinTypes.put(intT.getName(),     intT);
     builtinTypes.put(uintT.getName(),    uintT);

     builtinTypes.put(longT.getName(),    longT);
     builtinTypes.put(ulongT.getName(),   ulongT);

     builtinTypes.put(floatT.getName(),   floatT);
     builtinTypes.put(ufloatT.getName(),  ufloatT);

     builtinTypes.put(doubleT.getName(),  doubleT);
     builtinTypes.put(udoubleT.getName(), udoubleT);
     builtinTypes.put(ldoubleT.getName(), ldoubleT);
  }

  // To check for builtin type correctness

  private static final int SIGNED   = 0x0001;
  private static final int UNSIGNED = 0x0002;
  private static final int CHAR     = 0x0004;
  private static final int SHORT    = 0x0008;
  private static final int INT      = 0x0010;
  private static final int LONG     = 0x0020;
  private static final int FLOAT    = 0x0040;
  private static final int DOUBLE   = 0x0080;
  private static final int VOID     = 0x0100;
  private static final int BOOL     = 0x0200;

  private static Hashtable builtinTypeConstants = new Hashtable();

  static 
  {  
     builtinTypeConstants.put("signed",  new Integer(SIGNED));
     builtinTypeConstants.put("unsigned",new Integer(UNSIGNED));
     builtinTypeConstants.put("char",    new Integer(CHAR));
     builtinTypeConstants.put("short",   new Integer(SHORT));
     builtinTypeConstants.put("int",     new Integer(INT));
     builtinTypeConstants.put("bool",     new Integer(BOOL));
     builtinTypeConstants.put("long",    new Integer(LONG));
     builtinTypeConstants.put("float",   new Integer(FLOAT));
     builtinTypeConstants.put("double",  new Integer(DOUBLE));
     builtinTypeConstants.put("void",  new Integer(VOID));
  }
  private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: CPPBuiltinType.java,v $
// Revision 1.2  1998/07/28 13:48:50  mallet
// Fixed bug in synthesisType: added a case for 'typedef' to synthetize
// against real type instead of typedef.
//
// Revision 1.1  1998/04/28 12:22:13  cinard
// C++ builtin types are now defined in CPPBuiltinType.
// CBuiltinType is dedicated to C builitin types.
//
// ***********************************************************************
