// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CPPSymbolFactory.java,v $
// $Revision: 1.2 $
// $Date: 1998/06/10 15:29:05 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CPPSymbolFactory.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 * Concrete symbol factory for C++ symbols.
 */

public class CPPSymbolFactory extends CStarSymbolFactory 
{

  /**
   * Construct and returns a class.
   * @param name name of the class.
   * @param modifier modifier for the class
   */
   
  public LClass newClass(String name,int modifier)
  {
     LClass lClass = super.newClass(name,modifier);
     LAttribute _this = newAttribute("this",CModifier.PRIVATE,lClass);
     lClass.putSymbol(_this);
     return lClass;
  }

  /**
   * Construct and returns a struct.
   * @param name name.
   * @param modifier modifier.
   */
   
  public LClass newStruct(String name)
  {
     LClass lStruct = super.newStruct(name);
     LAttribute _this = newAttribute("this",CModifier.PRIVATE,lStruct);
     lStruct.putSymbol(_this);
     return lStruct;
  }

 // -- Attributes common to all adl classes --

  private static LAttribute _adl_thrownException = null;
  private static LMethod    _ADL_short_range      = null;
  private static LMethod    _ADL_int_range        = null;
  private static LMethod    _ADL_long_range       = null;
 


  /**
    * Construct and returns the ADL class to supply semantics for the target 
    * language class provided.
    * <p> All ADL class builtin keywords are declared hereby as attributes. 
    * @param lClass The target language class for which semantics is to be 
    * defined.
    * @param adlSuperClassName name of the adl superclass.
    */

  public ADLClass newADLClass(LClass lClass,String adlSuperClassName) 
  {
     if ( _adl_thrownException == null )
     {
        _adl_thrownException = newAttribute
                               ("adl_thrownException",0x00,
				newPointer(CPPBuiltinType.voidT,0x00));
        _ADL_short_range     = newMethod("ADL_short_range",
                                         CModifier.PUBLIC | CModifier.STATIC);
        _ADL_short_range.setReturnType(CPPBuiltinType.shortT);
        _ADL_short_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.shortT));
        _ADL_short_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.shortT));
        _ADL_int_range     = newMethod("ADL_int_range",
                                       CModifier.PUBLIC | CModifier.STATIC);
        _ADL_int_range.setReturnType(CPPBuiltinType.intT);
        _ADL_int_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.intT));
        _ADL_int_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.intT));
        _ADL_long_range     = newMethod("ADL_long_range",
                                        CModifier.PUBLIC | CModifier.STATIC);
        _ADL_long_range.setReturnType(CPPBuiltinType.longT);
        _ADL_long_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.longT));
        _ADL_long_range.addParameter
                         (newParameter("_",0x00,CPPBuiltinType.longT));

     }
     ADLClass adlClass = new ADLClass(lClass,adlSuperClassName);
     adlClass.putSymbol(_adl_thrownException ) ;
     adlClass.putSymbol(_ADL_short_range);
     adlClass.putSymbol(_ADL_int_range);
     adlClass.putSymbol(_ADL_long_range);
     // define "this" keyword if "lClass" is not null
     if ( lClass != null )
     {
        LAttribute _this = newAttribute("this",0x00,lClass);
        adlClass.putSymbol(_this);
     }
     
     return  adlClass; 
  }

  // -- Methods common to all tdd classes --

  private static LMethod _tdd_tdd_result   = null;
  private static LMethod _tdd_tdd_assert   = null;
  private static LMethod _tdd_tdd_skip     = null;
  private static LMethod _tdd_tdd_end_case = null;
  private static LMethod _tdd_infoline     = null;

  // -- Attributes common to all tdd classes --

  private static LAttribute _ADL_PASS;
  private static LAttribute _ADL_FAIL;
  private static LAttribute _ADL_UNRESOLVED;
  private static LAttribute _ADL_NOTINUSE;
  private static LAttribute _ADL_UNSUPPORTED;
  private static LAttribute _ADL_UNTESTED;
  private static LAttribute _ADL_UNINITIATED;
  private static LAttribute _ADL_NORESULT;
  private static LAttribute _ADL_UNEVALUATED;
  private static LAttribute _ADL_AMBIGUOUS;
  private static LAttribute _ADL_UNDEFINED;

  /**
   * Construct and returns a TDD class symbol.
   * @param name name of the TDD class.
   */

  public TDDClass newTDDClass(String name) 
  {
     if ( _tdd_tdd_result== null )
     {
        _tdd_tdd_result = newMethod("tdd_result",CModifier.PUBLIC);
        _tdd_tdd_result.addParameter(newParameter("res",
						  0x00,CPPBuiltinType.intT));
        _tdd_tdd_result.addParameter(newParameter
				  ("msg",0x00,newPointer(CPPBuiltinType.charT,0x00)));
        _tdd_tdd_result.setReturnType(CPPBuiltinType.boolT);

        _tdd_tdd_assert = newMethod("tdd_assert",CModifier.PUBLIC);
        _tdd_tdd_assert.addParameter
                    (newParameter("assertText",
				  0x00,newPointer(CPPBuiltinType.charT,0x00)));
        _tdd_tdd_assert.addParameter
                    (newParameter("assertRes",0x00,CPPBuiltinType.boolT));
        _tdd_tdd_assert.setReturnType(CPPBuiltinType.boolT);

        _tdd_tdd_skip = newMethod("tdd_skip",CModifier.PUBLIC);
        _tdd_tdd_skip.setReturnType(CPPBuiltinType.boolT);

        _tdd_tdd_end_case = newMethod("tdd_end_case",CModifier.PUBLIC);
        _tdd_tdd_end_case.addParameter(newParameter
				  ("msg",0x00,newPointer(CPPBuiltinType.charT,0x00)));
        _tdd_tdd_end_case.setReturnType(CPPBuiltinType.boolT);
        _tdd_infoline = newMethod("infoline",CModifier.PUBLIC);
        _tdd_infoline.setReturnType(CPPBuiltinType.voidT) ;
        _tdd_infoline.addParameter(newParameter
                                   ("line",0x00,newPointer(CPPBuiltinType.charT,0x00)));

        _ADL_PASS       = newAttribute("ADL_PASS",0x00,CPPBuiltinType.intT);
        _ADL_FAIL       = newAttribute("ADL_FAIL",0x00,CPPBuiltinType.intT);
        _ADL_UNRESOLVED = newAttribute("ADL_UNRESOLVED",0x00,CPPBuiltinType.intT);
        _ADL_NOTINUSE   = newAttribute("ADL_NOTINUSE",0x00,CPPBuiltinType.intT);
        _ADL_UNSUPPORTED= newAttribute("ADL_UNSUPPORTED",0x00,CPPBuiltinType.intT);
        _ADL_UNTESTED   = newAttribute("ADL_UNTESTED",0x00,CPPBuiltinType.intT);
        _ADL_UNINITIATED= newAttribute("ADL_UNINITIATED",0x00,CPPBuiltinType.intT);
        _ADL_NORESULT   = newAttribute("ADL_NORESULT",0x00,CPPBuiltinType.intT);
        _ADL_UNEVALUATED= newAttribute("ADL_UNEVALUATED",0x00,CPPBuiltinType.intT);
        _ADL_AMBIGUOUS  = newAttribute("ADL_AMBIGUOUS",0x00,CPPBuiltinType.intT);
        _ADL_UNDEFINED  = newAttribute("ADL_UNDEFINED",0x00,CPPBuiltinType.intT);

     }

     TDDClass tddClass = super.newTDDClass(name);
     tddClass.putSymbol(_tdd_tdd_result);
     _tdd_tdd_result.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_infoline);
     _tdd_infoline.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_assert);
     _tdd_tdd_assert.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_skip);
     _tdd_tdd_skip.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_end_case);
     _tdd_tdd_end_case.setIsRuntimeMethod();

     tddClass.putSymbol(_ADL_PASS);
     tddClass.putSymbol(_ADL_FAIL);
     tddClass.putSymbol(_ADL_UNRESOLVED);
     tddClass.putSymbol(_ADL_NOTINUSE);
     tddClass.putSymbol(_ADL_UNSUPPORTED);
     tddClass.putSymbol(_ADL_UNTESTED);
     tddClass.putSymbol(_ADL_UNINITIATED);
     tddClass.putSymbol(_ADL_NORESULT);
     tddClass.putSymbol(_ADL_UNEVALUATED);
     tddClass.putSymbol(_ADL_AMBIGUOUS);
     tddClass.putSymbol(_ADL_UNDEFINED);

     return tddClass;
  }


  // TDD Factory builtin methods
  // Initialized when newTDDFactory()is called for the first time.
  
  static LMethod _tdd_end_case_1 = null;
  static LMethod _tdd_end_case_2 = null;
  static LMethod _tdd_skip       = null;
  static LMethod _tdd_end_test   = null;
  static LMethod _tdd_result     = null; 

  /**
   * Construct and returns a TDD factory.
   * @param name Name of the factory.
   */

  public TDDFactory newTDDFactory(String name)
  {
     if ( _tdd_end_case_1 == null )
     {
        // Initialize tdd factory builtin methods
        _tdd_end_case_1 = newMethod("tdd_end_case",CModifier.PUBLIC);
        _tdd_end_case_1.setReturnType(CPPBuiltinType.boolT);
        
        _tdd_end_case_2 = newMethod("tdd_end_case",CModifier.PUBLIC);
        _tdd_end_case_2.addParameter
        (newParameter("msg",0x00,newPointer(CPPBuiltinType.charT,0x00)));
        _tdd_end_case_2.setReturnType(CPPBuiltinType.boolT);
  
        _tdd_skip = newMethod("tdd_skip",CModifier.PUBLIC);
        _tdd_skip.setReturnType(CPPBuiltinType.boolT);
     
        _tdd_end_test = newMethod("tdd_end_test",CModifier.PUBLIC);
        _tdd_end_test.setReturnType(CPPBuiltinType.boolT);

        _tdd_result = newMethod("tdd_result",CModifier.PUBLIC);
        _tdd_result.addParameter(newParameter("res",0x00,CPPBuiltinType.intT));
        _tdd_result.addParameter(newParameter("msg",0x00,
					      newPointer(CPPBuiltinType.intT,0x00)));
        _tdd_result.setReturnType(CPPBuiltinType.boolT);

     }
     
     TDDFactory factory = super.newTDDFactory(name);
     factory.putSymbol(_tdd_end_case_1);
     factory.putSymbol(_tdd_end_case_2);
     factory.putSymbol(_tdd_skip);
     factory.putSymbol(_tdd_end_test);
     factory.putSymbol(_tdd_result);
     return factory;
  }

}

// ***********************************************************************
// History Log:
// $Log: CPPSymbolFactory.java,v $
// Revision 1.2  1998/06/10 15:29:05  bareau
// Moved "isTestMethod" from LCallable to LMethod (and fixed it: runtime
// methods like infoline, tdd_assert... are _not_ test methods).
// Added an "isRuntimeMethod" flag in LMethod, and set it in symbol
// factories for runtime methods.
//
// Revision 1.1  1998/04/28 12:26:43  cinard
// C++ symbols are now obtained using the CPPSymbolFactory.
// CSymbolFactory is dedicated to C symbols.
//
// ***********************************************************************
