// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CPPSymbolTableMgr.java,v $
// $Revision: 1.4 $
// $Date: 1998/07/28 13:46:13 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CPPSymbolTableMgr.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;


import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.predicates.UnaryOr;


import ORG.opengroup.adl.symboltable.*;

/**
 * Manages the symbol table and scopes within a given translation unit.
 */

public class CPPSymbolTableMgr extends SymbolTableMgr 
{

  // -- Public Constructors --

  /**
   * Constructs me.
   * The global scope is set to the provided one (e.g. a Translation 
   * Unit scope).
   */
  
  public CPPSymbolTableMgr()
  {
     super();

     // add NULL as an attribute of type void *
     LAttribute _NULL = _sbf.newAttribute
                        ("NULL",
                         0x00,
                         _sbf.newPointer( CPPBuiltinType.voidT,0x00));

     // initialize NULL in CPointer 
     CPointer._NULL = getSymbolFactory().newPointer(CPPBuiltinType.voidT,0x00);

     try 
     {
        putSymbol(_NULL );
     } 
     catch ( SymbolAlreadyDefinedException e ) { /* never thrown */ }

  };
  

  // -- public methods --

  /**
   * Reinitialize the symbol table. Should be used when a new translation
   * unit is parsed.
   */

  public void reInit()
  {
     super.reInit();

     /** Re-init current scope. */
     setCurrentScope(unitScope);

  }

  /**
   * Indicate the current adl translation unit.
   */

  public void setAdlTranslationUnit(FileScope adlTranslationUnit) {
    _adlTranslationUnit = adlTranslationUnit;
  }

  /**
   * Get the current adl translation unit.
   */

  public FileScope getAdlTranslationUnit() {
    return _adlTranslationUnit;
  }

  /**
   * Reinitialize the symbol table, and set symbol table context.
   * Should be used when a new translation unit is type checked.
   * @param stmContext context information obtained from a former parsing.
   */

  public void setContext(Object stmContext)
  {
     reInit();

     if ( stmContext instanceof CTranslationUnitInfo ) {
       _adlTranslationUnit = ((CTranslationUnitInfo)stmContext)
	               .getAdlTranslationUnit();
     }

     super.setContext(stmContext);
  }

  /**
   * Get opaque context information that can be used later to manipulates 
   * symbols for current translation unit.
   */

  public Object getContext()
  {
     return new CTranslationUnitInfo(_adlTranslationUnit,
				     getUsedADLClasses(),
				     getUsedTDDClasses());
  }


  /**
   * Put a symbol in the symbol table, extended version.
   * The symbol is put in the supplied scope or in current scope in case no 
   * scope was supplied.
   * In case the symbol is already in the scope (because of a earlier
   * declaration) the methods returns the already declared symbol, 
   * otherwise, it put the supplied symbol in the current scope and 
   * returns it.
   * @param symbol The symbol to put into the scope.
   * @param scopeOverride optionnal scope to take into account instead
   *                      of current scope.
   * @return the symbol that was actually found or put in the symbol
   *         table.
   * @exception ORG.opengroup.adl.symboltable.SymbolAlreadyDefinedException 
   *            in case symbol is already defined.
   */

  public Symbol putSymbol(Symbol symbol,Scope scopeOverride)
  throws SymbolAlreadyDefinedException
  {
     if ( debug )
     {
        System.out.println("debug: CPPSymbolTableMgr.putSymbol(" + symbol
                          + ", " + scopeOverride + ")");
     }

     if ( scopeOverride == null ) { scopeOverride = getCurrentScope(); }
  
     // Check if a symbol with the same declaration exists in the 
     // scope

     String         name       = symbol.getName();
     UnaryPredicate comparator = symbol.getComparator();

     Symbol existingSymbol = scopeOverride.getSymbol
                             (symbol.getName(),comparator,Boolean.TRUE);

     if ( existingSymbol != null )
     {
        // Symbol exists, check this symbol is not defined.
        if ( existingSymbol.isDefined() )
        {
           if ( debug )
           {
              System.out.println("debug: CPPSymbolTableMgr.putSymbol(" + symbol
                                 + ",scopeOverride)"
                                 + " exception - symbol already defined");
           }
           // It is defined, throw an exception.
           throw  new SymbolAlreadyDefinedException
                      (symbol + " already defined");
        }
        else
        {
           if ( debug )
           {
              System.out.println("debug: CPPSymbolTableMgr.putSymbol(" + symbol
                                 + ",scopeOverride)"
                                 + " return - symbol exists " + existingSymbol);
           }
  
           // Symbol is not yet defined, return the symbol already
           // in scope.
           return existingSymbol;
        }
     }
     else
     {
        //Test if symbol is LExpression and save related scope in LExpression
        if ( symbol instanceof LExpression ) 
        {
           ((LExpression)symbol).setRelated( scopeOverride ) ;
        }
        else if ( symbol instanceof LCallable )
        {
           if ( ((LCallable)symbol).getParent() == null )
           {
              ((LCallable)symbol).setParent(scopeOverride);
           }
        }
        // Symbol is not yet declared, put it in the scope and return it.
        scopeOverride.putSymbol(symbol);
        if ( debug )
        {
           System.out.println("debug: CPPSymbolTableMgr.putSymbol(" + symbol
                              + ",scopeOverride)"
                              + " return - new symbol " + symbol);
        }
  
        return symbol;
     }
     
  }


  /**
   * Get a symbol, extended version.
   * Should be called to search for names that were classified ambiguous.
   * The symbol table manager tries to classify the supplied name and to 
   * return an adhoc symbol for it.
   * <pre>
   * Scope override shall be supplied to override the default behaviour,
   * which is to search in the current scope. This enable incremental
   * search for complex expressions like foo.bar[].boo.baa().zip.
   * </pre>
   * <p> <b> Algorithm: </b> <p> <ul>
   * <li>set search scope to current scope in case no scope was supplied
   * <li>before searching through all scopes, check it's not a builtin type
   * <li>in case scope name starts with :: set search scope to parent scope
   * <li>split the scoped name -> scopedName [ :: | . ] name
   * <li>in case the name was not a simple identifier, recursively called
   *     getSymbol with the name made up of scoped name identifiers minus 
   *     the last
   * <li>... and update serch scope to the result of this call to getSymbol
   * <li>look for a symbol with its name equals to the last identifier
   *     of scoped name in the search scope and all its parents until
   *     a suitabble symbol is found
   * </ul>
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate     An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param symbolListener Reference to the symbol listener that shall
   *                       be called back later in case the symbol
   *                       is not found during the call (may be null).
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public Symbol getSymbol
                (
                   String scopedName, 
                   UnaryPredicate predicate,
                   SymbolListener symbolListener,
                   Scope scopeOverride,
                   boolean localSearch
                )
  {
     if ( debug )
     {
        System.out.println("debug: CPPSymbolTableMgr.getSymbol("
                          + scopedName + ", "
                          + predicate + ", "
                          + "symbolListener, "
                          + scopeOverride + ", " 
                          + localSearch + ")");
     }

     String name   = null; // Last component of the scoped name
     Symbol symbol = null;
     Symbol scopeSymbol = null; // Intermediary scope symbol for
                                // composed scoped names.
     Object sli = null;         // Symbol listener information

     // Set search scope to current scope in case no scope was supplied

     if ( scopeOverride == null )
     {
        scopeOverride = currentScope;
     }

     // Before searching through all scopes, check it's not a builtin type.

     symbol = CPPBuiltinType.getBuiltinType(scopedName);

     if ( symbol != null )
     {
        return symbol;
     }
     
     // Save symbol listener if needed

     if ( symbolListener != null )
     {
        sli = newSymbolListenerInfo
              (symbolListener, scopedName, currentScope, predicate);
     }
     
     // In case scope name starts with :: set search scope to parent 
     // scope.

     if ( scopedName.startsWith("::") )
     {
        scopeOverride = scopeOverride.getParent();
        scopedName = scopedName.substring(2,scopedName.length());
     }

     if ( scopedName.endsWith("::") )
     {
        scopedName = scopedName.substring(0,scopedName.length()-2);
        if ( predicate == null )
        {
           predicate = new UnaryOr
                           (
                            SymbolSelector.MODULE_SELECT,
                            SymbolSelector.TYPE_SELECT
                           );
        }
        
           
     }

     // Split the scoped name -> scopedName [ :: | . ] name 

     int index              = scopedName.lastIndexOf(".");
     int scopeOverrideIndex = scopedName.lastIndexOf("::");
     UnaryPredicate scopedNameComparator = null;
     
     if ( index == -1 && scopeOverrideIndex == -1 )
     {
        name       = scopedName;
        scopedName = null;
     }
     else
     {
        int maxIndex = Math.max(index,scopeOverrideIndex);
        if ( index == maxIndex )
        {   
           name       = scopedName.substring(index + 1);
           scopedName = scopedName.substring(0,index);
        }
        else // scopeOverrideIndex == maxIndex
        {
           name       = scopedName.substring(scopeOverrideIndex + 2);
           scopedName = scopedName.substring(0,scopeOverrideIndex);

           // scopedName is for a type or a name space
           scopedNameComparator = new UnaryOr
                                      (
                                         SymbolSelector.MODULE_SELECT,
                                         SymbolSelector.TYPE_SELECT
                                      );
        }
        
     }

     // In case the name was not a simple identifier
     // getSymbol (and therefore classify) for the
     // remaining scoped name.
  
     if ( scopedName != null )
     {
        scopeSymbol = getSymbol
                      (scopedName,
                       scopedNameComparator,
                       null,
                       scopeOverride,
                       localSearch);

        // Now search will be local!
        localSearch = true;

        if ( scopeSymbol != null && scopeSymbol.hasScopeAccess() )
        {
           // New search scope to resolve the symbol.
           scopeOverride =  scopeSymbol.getScope();
        }
        else
        {
           scopeOverride = null;
        }
     }

     // have we checked in adl compilaiton unit scope
     boolean searchedIntoAdlTranslationUnit = false;

     // search in this scope and propagate search in parent scopes
     while (symbol == null && scopeOverride != null )
     {
        symbol        = scopeOverride.getSymbol(name,predicate,null);

        // check symbol is ok

        if ( symbol != null && ! ( canGetSymbol(symbol,localSearch) ) )
        {
           symbol = null;
        }

	// if needed indicate current adl translation unit has
	// been checked
	if ( scopeOverride == _adlTranslationUnit )
	{
	   searchedIntoAdlTranslationUnit = true;
	}

        // search in parent scopes only in case localSearch flag is not set

        if ( localSearch ) 
        {
           break;
        }
        else
        {
           scopeOverride = scopeOverride.getParent();
        }
     }
       
     // Check adl translation unit if needed

     if (      ( symbol == null )
	  && ! searchedIntoAdlTranslationUnit 
	  && ! localSearch 
	  && ! ( _adlTranslationUnit == null )
	)
     {
       symbol = _adlTranslationUnit.getSymbol(name,predicate,null);
     }

     // Check if symbol was not found.

     if ( symbol == null && symbolListener != null )
     {
        // In case symbol was not found store symbol listener information so
        // that it can be called back after further successful research.
        
        storeSymbolListenerInfo(sli);
     }

     if ( debug )
     {
        System.out.println("debug: CPPSymbolTableMgr.getSymbol("
                          + scopedName + ", "
                          + predicate + ", "
                          + "symbolListener, "
                          + scopeOverride + ", " 
                          + localSearch + ")" + " return " + symbol);
     }
     return symbol;

  }

  /**
   * Tells if a string is a constructor name.
   *
   * <p> We assume the string is a constructor name when  <ul>: 
   * <li> It has no scope override and current scope name is a class/struct of
   * that name.
   * <li> Or it has scope override and enclosing scope name is the string name.
   * </ul>
   * @param ctorName the constructor name to test.
   * @result boolean telling if ctorName is the name of a constructor or not.
   */

  public boolean isConstructor ( String ctorName ) {
    
    if (ctorName == null )
      return false ;

    // Case ctorName has no scope override 
    if ( ctorName.indexOf("::") == -1 ) {
      Scope workScope = getCurrentScope() ;
      return  (workScope !=null) && (workScope instanceof LConstructedType)
	&& ctorName.equals (((LType)workScope).getName() ) ;  
    }

    // case of scope override.
    // extract pure destructor name.
    String endName = ctorName.substring (ctorName.lastIndexOf("::") + 2,
					    ctorName.length() ) ;
    String beginName = ctorName.substring (0,ctorName.lastIndexOf("::") );
    // look for enclosoing symbol
    Symbol enclose = getSymbol( beginName ) ;
    // return true when name constructor is name of enclosing class.
    return beginName.endsWith ( endName ) && ( enclose != null )  &&
      ( (enclose instanceof LClass) ) ;
    
  }

  /**
   * Get a builtin type symbol.
   * @param name   The name of the builtin type.
   * @return The found symbol.
   * @exception ORG.opengroup.adl.symboltable.BadSymbolNameException
   *            in case supplied name is not that of a builtin type.
   */

  public LType getBuiltinType(String name)
  throws BadSymbolNameException
  {
     LType t = CPPBuiltinType.getBuiltinType(name);
     if ( t == null ) 
     {
        throw new BadSymbolNameException("Bad builtin type name : " + name);
     }
     return t;
  }

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  public boolean isValidBuiltinTypeName(String name)
  {
     return CPPBuiltinType.isValidBuiltinTypeName(name);
  }

  /** Return a factory for symbols. */

  public SymbolFactory getSymbolFactory() 
  {
     return _sbf;
  }

  // -- private methods --

  /** 
   * Can symbol be retrieved ? This is true if search is not
   * local, or search is local and symbol is not private.
   * @param symbol symbol to check.
   * @param local search flag.
   */

  private boolean canGetSymbol(Symbol symbol, boolean localSearch)
  {
     if ( localSearch )
     {
        LModifier modif = symbol.getModifier();
        if ( modif instanceof CModifier )
        {
           return ! ((CModifier)modif).isPrivate();
        }
     }
     return true;
  }

  // -- Private fields --

  /** Symbol factory */
  private static final CPPSymbolFactory _sbf = new CPPSymbolFactory();

  /** curent adl translation unit */
  private FileScope _adlTranslationUnit ;

  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: CPPSymbolTableMgr.java,v $
// Revision 1.4  1998/07/28 13:46:13  mallet
// Added list of used ADL classes to parameters of *UnitInfo constructor.
//
// Revision 1.3  1998/05/27 09:57:00  cinard
// Made some clean up in code and improved debug messages.
//
// Revision 1.2  1998/05/12 08:01:00  cinard
// Replaced reference to CBuiltinType by a reference to CPPBuiltinType.
//
// Revision 1.1  1998/04/28 12:32:21  cinard
// Manages C++ symbols (former CSymbolTableMgr).
//
// ***********************************************************************
