// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CPointer.java,v $
// $Revision: 1.9 $
// $Date: 1998/04/28 12:27:57 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CPointer.java
//
// HISTORY:
//      Wassel BERRAYANA, TOG - Jan,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 *  CPointer symbol.
 */

public class CPointer extends LPointerImpl
{

  /**
   * Constructs me to be a pointer to a type.
   * @param pointed pointed type.
   */
  
  public CPointer(LType pointed,LModifier modifier)
  {
     super ( pointed, modifier ) ;
  }

 
  /**
   * Overrides assignable of LPointer.
   * Can a value of type t be assigned to a variable of this type.
   * <p><b>Algorithm:</b><ul>
   * return true only if : <ul><br>
   * <li> If <code>t</code> is a CBuiltinType<=> "builtintypes can be converted
   * to pointers".
   * <li> If <code>this</code> is void * and <code>t</code> is LPointer <=> 
   * pointers can be converted to void *.
   * <li> <code>this</code> points a class <code>t</code> is a pointer to a 
   * subclass of that class.
   * <li> Array of T can be converted into Pointer of T.
   * </ul>
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t) 
  {
    if ( t== null )
      return false ;
    else if ( t instanceof LBuiltinType ) {
      return false ;
    }
    else if ( t.equals ( _NULL )) {
      return true ;
    }
    else if ( this.equals ( _NULL ) && ( t instanceof LPointer ) ) {
     // any pointer is convertible to _NULL
     return true ;
    }
    else if (t instanceof LPointer && (this.pointed instanceof LClass)  ) {
      if (((LPointer )t).getPointedToType() instanceof LClass   
	  && (( LClass )((LPointer )t).getPointedToType()).
	                                       subclassOf( this.pointed )) {
	return true ;
      }
      else
	return false ;
    }
    else if ( t instanceof LArray ) {
      return this.pointed.equals ( ((LArray)t).getElementType() ) ;
    }
    else if ( t instanceof LPointerToFunction ) {
      return this.pointed.assignable(((LPointerToFunction) t).
				     getPointedToType()) ;
    }
    else 
      return ( t instanceof LPointer )
            && ( pointed.assignable(((LPointer)t).getPointedToType()  ) ) ;             
  }

  /** NULL pointer defined as (void *)0. */
  public static LType _NULL ;
      
}

