// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CStarSymbolFactory.java,v $
// $Revision: 1.1 $
// $Date: 1998/04/28 12:29:30 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CStarSymbolFactory.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 * Symbol factory for C like language symbols.
 */

public class CStarSymbolFactory extends SymbolFactory 
{

  /** 
   * Constructs an expression.
   */

  public LExpression newExpression(long symbolId,int modifier)
  {
     LExpression ret = null;

     if ( symbolId == SymbolId.PARAMETER )
     {
        ret = newParameter("",modifier,null);
     }
     else if ( symbolId == SymbolId.VARIABLE )
     {
        ret = newVariable("",modifier,null);
     }
     else if ( symbolId == SymbolId.ATTRIBUTE )
     {
        ret = newAttribute("",modifier,null);
     }
     else if ( symbolId == SymbolId.ADL_BINDING )
     {
        ret = newADLBinding("",modifier,null);
     }
     else if ( symbolId == SymbolId.TDD_DATASET )
     {
        ret = newTDDDataset("",0,modifier,null);
     }
     else
     {
        throw new Error("Unknown expression id.");
     }
     return ret;
  }
  
 /**
  * Constructs and returns a target enumerator.
  * @param name name of the enumeration.
  */

  public CEnum newEnum(String name)
  {
     return new CEnumImpl(name);
  }


  /**
   * Constructs a C++ destructor.
   * Constructs and returns a C++ destructor.
   * @param name Name of the destructor.
   */

  public CDestructor newDestructor(String name,int modifier)
  {
     return new CDestructor(name,newModifier(modifier));
  }

 

  /**
   * Constructs and returns a function type.
   * @param function type comparator (LCallableEqualsComparator).
   */
  
  public CFunctionType newFunctionType (LCallableEqualsComparator comparator,
					LConstructedType parent )
  {
     CFunctionType ret =  new CFunctionTypeImpl(comparator, parent );
     ret.setModifier(newModifier(0));
     return ret;
  }


  /**
   * Constructs and returns a reference type.
   * @param referenced referenced type.
   */

  public LReference newReference(LType referenced)
  {
     return new CReference(referenced);
  }
 

  /**
   * Constructs and returns a type : pointer to type.
   * @param referenced pointer type.
   */
  
  public LPointer newPointer(LType pointed,int modifier)
  {
     return new CPointer(pointed,newModifier(modifier));
  }


  /**
   * Constructs and returns a type : pointer to function.
   * @param referenced function (LCallableEqualsComparator).
   * @param modifier modifier.
   */
  
  public LPointerToFunction newPointerToFunction
                            (LCallableEqualsComparator pointed,
                             int modifier)
  {
     LPointerToFunction ret =  new CPointerToFunction
                                   (pointed,newModifier(modifier));
     return ret;
  }

  /**
   * Creates a new LModifier with all the bits in modifier set.
   * @param modifier the modifier.
   */

  public LModifier newModifier(int modifier)
  {
     return new CModifier(modifier);
  }

  // -- Private fields --

  /** Debug flag. */

  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: CStarSymbolFactory.java,v $
// Revision 1.1  1998/04/28 12:29:30  cinard
// Abstract superclass for c and c++ symbol factories.
//
// ***********************************************************************
