// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CSymbolTableMgr.java,v $
// $Revision: 1.42 $
// $Date: 1998/07/28 13:45:30 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CSymbolTableMgr.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;


import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.predicates.UnaryOr;


import ORG.opengroup.adl.symboltable.*;


/**
 * Manages the symbol table and scopes within a given compilation unit.
 */

public class CSymbolTableMgr extends SymbolTableMgr 
{

  // -- Public Constructors --

  /**
   * Constructs me.
   * The global scope is set to the provided one (e.g. a Compilation 
   * Unit scope).
   */
  
  public CSymbolTableMgr()
  {
     super();

     // initialize NULL in CPointer 
     CPointer._NULL = getSymbolFactory().newPointer(CBuiltinType.voidT,0x00);

     try 
     {
        // put adl runtime symbols in symbol table
        Symbol[] runtimeSymbols = _sbf.newRuntime();
        Scope unitScope = getUnitScope();
        for ( int i = 0; i < runtimeSymbols.length; i++ )
        {
           putSymbol(runtimeSymbols[i],unitScope);
        }
     }
     catch ( SymbolAlreadyDefinedException e ) { /* never occurs */ }
     
  }

  // -- Public Methods --

  /** Get symbol for ADL_short_range function. */
  public LFunction getADLShortRange() 
  { 
    return _sbf.getADLShortRange(); 
  }

  /** Get symbol for ADL_int_range function. */
  public LFunction getADLIntRange() 
  { 
    return _sbf.getADLIntRange(); 
  }
  
  /** Get symbol for ADL_long_range function. */
  public LFunction getADLLongRange() 
  { 
    return _sbf.getADLLongRange(); 
  }

  /**
   * Reinitialize the symbol table. Should be used when a new compilation
   * unit is parsed.
   */

  public void reInit()
  {
     super.reInit();

     /** Re-init current scope. */
     setCurrentScope(unitScope);

  }

  /**
   * Indicate the current adl translation unit.
   */

  public void setAdlTranslationUnit(FileScope adlTranslationUnit) {
    _adlTranslationUnit = adlTranslationUnit;
  }

  /**
   * Get the current adl translation unit.
   */

  public FileScope getAdlTranslationUnit() {
    return _adlTranslationUnit;
  }

  /**
   * Reinitialize the symbol table, and set symbol table context.
   * Should be used when a new translation unit is type checked.
   * @param stmContext context information obtained from a former parsing.
   */

  public void setContext(Object stmContext)
  {
     reInit();

     if ( stmContext instanceof CTranslationUnitInfo ) {
       _adlTranslationUnit = ((CTranslationUnitInfo)stmContext)
	               .getAdlTranslationUnit();
     }

     super.setContext(stmContext);
  }

  /**
   * Get opaque context information that can be used later to manipulates 
   * symbols for current translation unit.
   */

  public Object getContext()
  {
     return new CTranslationUnitInfo(_adlTranslationUnit,
				     getUsedADLClasses(),
				     getUsedTDDClasses());
  }

  
  /**
   * Put a symbol in the symbol table, extended version.
   * The symbol is put in the supplied scope or in current scope in case no 
   * scope was supplied.
   * In case the symbol is already in the scope (because of a earlier
   * declaration) the methods returns the already declared symbol, 
   * otherwise, it put the supplied symbol in the current scope and 
   * returns it.
   * @param symbol The symbol to put into the scope.
   * @param scopeOverride optionnal scope to take into account instead
   *                      of current scope.
   * @return the symbol that was actually found or put in the symbol
   *         table.
   * @exception ORG.opengroup.adl.symboltable.SymbolAlreadyDefinedException 
   *            in case symbol is already defined.
   */

  public Symbol putSymbol(Symbol symbol,Scope scopeOverride)
  throws SymbolAlreadyDefinedException
  {
     if ( debug )
     {
        System.out.println("debug: CSymbolTableMgr.putSymbol(" + symbol
                          + ", " + scopeOverride + ")");
     }

     if ( scopeOverride == null ) { scopeOverride = getCurrentScope(); }

     // Check if a symbol with the same declaration exists in the 
     // scope

     String         name           = symbol.getName();
     Symbol         existingSymbol = null;

     // Special check for functions and variables (except adl functions)

     if (   
             symbol.isA(SymbolId.EXPRESSION_MASK) 
          || symbol.isA(SymbolId.OPERATION_MASK & ( ~ SymbolId.ADL_METHOD ) )
        )
     {
        // Check an expression or function with same and different 
        // declaration does not exists.
        Symbol feature = getFeature(name,
                                    scopeOverride,
                                    symbol.isA(SymbolId.EXPRESSION_MASK));

        if ( feature != null )
        {
           // Further checks are necessary for the found symbol.
           if ( feature.getSymbolId() != symbol.getSymbolId() ) 
           {
              // In case it exists and symbols are of different kind 
              // throw exception

              throw new SymbolAlreadyDefinedException
                        (name + " redeclared as different kind of symbol");
           }
           else if ( feature.isA(SymbolId.FUNCTION) )
           {

              // existing and new symbols are both functions, check if 
              // declaration are for the same function

              LCallable formerDeclaration = (LCallable)feature;
              LCallable newDeclaration    = (LCallable)symbol;

              // function declarations shall satisfy the following conditions :
              // (1) only one declaration has a parameter type list, and the
              // other has none
              // (2) both declaration have parameter type lists (and the type
              // of each parameter in each declaration are compatibles)

              if ( formerDeclaration.getParameters().length == 0 
                   || newDeclaration.getParameters().length == 0 
                 )
              {
                 // only one declaration has a parameter type list, 
                 // and the other has none : verify return types are 
                 // compatibles

                 if ( formerDeclaration.getReturnType().equals
                      (newDeclaration.getReturnType())
                    )
                 {
                    // refine former declaration in case parameters
                    // were not supplied
                    if ( formerDeclaration.getParameters().length == 0 )
                    {
                       LParameter[] parameters=newDeclaration.getParameters();
                       for ( int i = 0; i < parameters.length; i++ )
                       {
                          formerDeclaration.addParameter(parameters[i]);
                       }
                    }
                    
                 }
                 else
                 {
                    throw new SymbolAlreadyDefinedException
                              ("conflicting types for " + name);
                 }

                    
              }
              else if ( ! formerDeclaration.getComparator().equals
                          (newDeclaration.getComparator()) )
              {
                 // error
                 throw new SymbolAlreadyDefinedException
                           ("conflicting types for " + name);
              }
           }
        }

        existingSymbol = feature;
     }
     else
     {
        UnaryPredicate comparator = symbol.getComparator();
        existingSymbol = scopeOverride.getSymbol
                         (symbol.getName(),comparator,Boolean.TRUE);
     }

     if ( existingSymbol != null )
     {
        // Symbol exists, check this symbol is not defined.
        if (    existingSymbol.isDefined() 
             && ! existingSymbol.isA(SymbolId.FUNCTION) )
        {
           if ( debug )
           {
              System.out.println("debug: CSymbolTableMgr.putSymbol(" + symbol
                                 + ",scopeOverride)"
                                 + " exception - symbol already defined");
           }
           // It is defined, throw an exception.
           throw  new SymbolAlreadyDefinedException
                      (symbol.getName() + " already defined");
        }
        else
        {
           if ( debug )
           {
              System.out.println("debug: CSymbolTableMgr.putSymbol(" + symbol
                                 + ",scopeOverride)"
                                 + " return - symbol exists " + existingSymbol);
           }
  
           // Symbol is not yet defined, return the symbol already
           // in scope.
           return existingSymbol;
        }
     }
     else
     {
        //Test if symbol is LExpression and save related scope in LExpression
        if ( symbol instanceof LExpression ) 
        {
           ((LExpression)symbol).setRelated( scopeOverride ) ;
        }
        else if ( symbol instanceof LCallable )
        {
           if ( ((LCallable)symbol).getParent() == null )
           {
              ((LCallable)symbol).setParent(scopeOverride);
           }
        }

        // Symbol is not yet declared, put it in the scope and return it.
        scopeOverride.putSymbol(symbol);

        if ( debug )
        {
           System.out.println("debug: CSymbolTableMgr.putSymbol(" + symbol
                              + ",scopeOverride)"
                              + " return - new symbol " + symbol);
        }
  
        return symbol;
     }
     
  }

  /**
   * Get a feature, that is either an expression or a function.
   *
   * @param name           name of the expected feature.
   * @param scopeOverride  scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return found feature.
   */

  public Symbol getFeature
                (
                   String name, 
                   Scope scopeOverride,
                   boolean localSearch
                )
  {
     if ( debug )
     {
        System.out.println("debug: CSymbolTableMgr.getFeature("
                          + name + ", "
                          + scopeOverride + ", " 
                          + localSearch + ")");
     }

     Symbol feature = null;

     SymbolSelector selector = new SymbolSelector( SymbolId.EXPRESSION_MASK
                                                 | SymbolId.OPERATION_MASK
                                                 );
     feature = getSymbol
               (name,
                selector,
                null,
                scopeOverride,
                localSearch);

     if ( debug )
     {
        System.out.println("debug: CSymbolTableMgr.getFeature("
                          + name + ", "
                          + scopeOverride + ", " 
                          + localSearch + ")" + " return " + feature);
     }

     return feature;

  }

  /**
   * Get a symbol, extended version.
   * Should be called to search for names that were classified ambiguous.
   * The symbol table manager tries to classify the supplied name and to 
   * return an adhoc symbol for it.
   * <pre>
   * Scope override shall be supplied to override the default behaviour,
   * which is to search in the current scope. This enable incremental
   * search for complex expressions like foo.bar[].boo.baa().zip.
   * </pre>
   * <p> <b> Algorithm: </b> <p> <ul>
   * <li>set search scope to current scope in case no scope was supplied
   * <li>before searching through all scopes, check it's not a builtin type
   * <li>in case scope name starts with :: set search scope to parent scope
   * <li>split the scoped name -> scopedName [ :: | . ] name
   * <li>in case the name was not a simple identifier, recursively called
   *     getSymbol with the name made up of scoped name identifiers minus 
   *     the last
   * <li>... and update serch scope to the result of this call to getSymbol
   * <li>look for a symbol with its name equals to the last identifier
   *     of scoped name in the search scope and all its parents until
   *     a suitabble symbol is found
   * </ul>
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate     An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param symbolListener Reference to the symbol listener that shall
   *                       be called back later in case the symbol
   *                       is not found during the call (may be null).
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public Symbol getSymbol
                (
                   String scopedName, 
                   UnaryPredicate predicate,
                   SymbolListener symbolListener,
                   Scope scopeOverride,
                   boolean localSearch
                )
  {
     if ( debug )
     {
        System.out.println("debug: CSymbolTableMgr.getSymbol("
                          + scopedName + ", "
                          + predicate + ", "
                          + "symbolListener, "
                          + scopeOverride + ", " 
                          + localSearch + ")");
     }

     String name   = null; // Last component of the scoped name
     Symbol symbol = null;
     Symbol scopeSymbol = null; // Intermediary scope symbol for
                                // composed scoped names.
 
     // Set search scope to current scope in case no scope was supplied

     if ( scopeOverride == null )
     {
        scopeOverride = currentScope;
     }

     // Before searching through all scopes, check it's not a builtin type.

     symbol = CBuiltinType.getBuiltinType(scopedName);

     if ( symbol != null )
     {
        return symbol;
     }
     
     // Split the scoped name -> scopedName . name 

     int index = scopedName.lastIndexOf(".");

     UnaryPredicate scopedNameComparator = null;
     
     if ( index == -1 ) 
     {
        name       = scopedName;
        scopedName = null;
     }
     else
     {
        name       = scopedName.substring(index + 1);
        scopedName = scopedName.substring(0,index);
     }

     // In case the name was not a simple identifier
     // getSymbol (and therefore classify) for the
     // remaining scoped name.
  
     if ( scopedName != null )
     {
        scopeSymbol = getSymbol
                      (scopedName,
                       scopedNameComparator,
                       null,
                       scopeOverride,
                       localSearch);

        // Now search will be local!
        localSearch = true;

        if ( scopeSymbol != null && scopeSymbol.hasScopeAccess() )
        {
           // New search scope to resolve the symbol.
           scopeOverride =  scopeSymbol.getScope();
        }
        else
        {
           scopeOverride = null;
        }
     }

     // have we checked in adl compilaiton unit scope
     boolean searchedIntoAdlTranslationUnit = false;

     // search in this scope and propagate search in parent scopes
     while (symbol == null && scopeOverride != null )
     {
        symbol = scopeOverride.getSymbol(name,predicate,null);

        // check symbol is ok

        if ( symbol != null && ! ( canGetSymbol(symbol,localSearch) ) )
        {
           symbol = null;
        }
	
	// if needed indicate current adl translation unit has
	// been checked
	if ( scopeOverride == _adlTranslationUnit )
	{
	   searchedIntoAdlTranslationUnit = true;
	}
	 
        // search in parent scopes only in case localSearch flag is not set

        if ( localSearch ) 
        {
           break;
        }
        else
        {
           scopeOverride = scopeOverride.getParent();
        }
     }

     // Check adl translation unit if needed

     if (      ( symbol == null )
	  && ! searchedIntoAdlTranslationUnit 
	  && ! localSearch 
	  && ! ( _adlTranslationUnit == null )
	)
     {
       symbol = _adlTranslationUnit.getSymbol(name,predicate,null);
     }
       
     if ( debug )
     {
        System.out.println("debug: CSymbolTableMgr.getSymbol("
                          + scopedName + ", "
                          + predicate + ", "
                          + "symbolListener, "
                          + scopeOverride + ", " 
                          + localSearch + ")" + " return " + symbol);
     }

     return symbol;

  }

  /**
   * Get a builtin type symbol.
   * @param name   The name of the builtin type.
   * @return The found symbol.
   * @exception ORG.opengroup.adl.symboltable.BadSymbolNameException
   *            in case supplied name is not that of a builtin type.
   */

  public LType getBuiltinType(String name)
  throws BadSymbolNameException
  {
     LType t = CBuiltinType.getBuiltinType(name);
     if ( t == null ) 
     {
        throw new BadSymbolNameException("Bad builtin type name : " + name);
     }
     return t;
  }

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  public boolean isValidBuiltinTypeName(String name)
  {
     return CBuiltinType.isValidBuiltinTypeName(name);
  }

  /** Return a factory for symbols. */

  public SymbolFactory getSymbolFactory() 
  {
     return _sbf;
  }

  // -- private methods --

  /** 
   * Can symbol be retrieved ? This is true if search is not
   * local, or search is local and symbol is not private.
   * @param symbol symbol to check.
   * @param local search flag.
   */

  private boolean canGetSymbol(Symbol symbol, boolean localSearch)
  {
     if ( localSearch )
     {
        LModifier modif = symbol.getModifier();
        if ( modif instanceof CModifier )
        {
           return ! ((CModifier)modif).isPrivate();
        }
     }
     return true;
  }

  /** Symbol factory. */
  CSymbolFactory _sbf = new CSymbolFactory();

  /** curent adl translation unit */
  private FileScope _adlTranslationUnit ;

  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: CSymbolTableMgr.java,v $
// Revision 1.42  1998/07/28 13:45:30  mallet
// Added list of used ADL classes to parameters of *UnitInfo constructor.
//
// Revision 1.41  1998/06/23 13:22:45  cinard
// Added getter for runtime functions ADL_short_range(), ADL_int_range() and
// ADL_long_range().
//
// Revision 1.40  1998/05/27 09:57:00  cinard
// Made some clean up in code and improved debug messages.
//
// Revision 1.39  1998/05/11 11:51:01  cinard
// Added methods to reinitialize the symbol table manger so that it can
// be used for several compilation units in a single adl compilation
// session.
//
// Revision 1.38  1998/04/28 12:31:42  cinard
// Symbol table for C symbols. C++ symbols are now managed using the
// CPPSymbolTableMgr.
//
// Revision 1.37  1998/03/19 16:42:57  cinard
// Fixed a bug in SymbolTablMgr.getMethod() function with CPP.
//
// Revision 1.36  1998/03/19 15:35:50  cinard
// Improved debug trace for putSymbol() and getSymbol() methods.
//
// Revision 1.35  1998/03/18 15:51:28  cinard
// Symbol factory in now separated from symbol table.
//
// Revision 1.34  1998/03/02 15:54:02  cinard
// Removed classes related to LStruct. Updated code accordingly.
//
// Revision 1.33  1998/03/02 15:32:29  cinard
// Replaced LStruct with LClass (struct symbols behave just like classes).
// Factory and find methods (newStruct() and getStruct() remains with
// their signatures updated).
//
// Revision 1.32  1998/02/27 12:40:04  cinard
// Made "this" a private attribut of CClass so that it is not visible from
// outside the class definition.
// Updated copyright notice.
//
// Revision 1.31  1998/02/25 11:05:43  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
//
// Revision 1.30  1998/02/23 14:49:50  cinard
// Added support for typedefs.
//
// Revision 1.29  1998/02/19 10:35:04  wassel
// Added runtime method infoline as a builtin method of tdd classes.
//
// Revision 1.28  1998/02/18 15:21:19  wassel
// Fixed bug : when adding tdd runtime methods tdd_result, tdd_assert, tdd_skip and tdd_end_case in newTDDClass, their parent scope should also be set so that the call to their isTestMethod() returns true.
//
// Revision 1.27  1998/02/12 16:03:29  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.26  1998/02/11 17:04:26  wassel
// Symbol NULL is created and added in the CSymbolTableMgr constructor.
//
// Revision 1.25  1998/02/11 16:48:06  wassel
// added symbol NULL having the type void *
//
// Revision 1.24  1998/02/10 13:10:20  cinard
// Reinitialize method added so that the symbol table can be reinitialized
// (useful when new compilation unit are parsed).
//
// Revision 1.23  1998/02/09 15:25:36  wassel
// Fixed runtime TDD predefined symbols.
//
// Revision 1.22  1998/02/06 15:23:47  wassel
// Modified putSymbol method to save definition scope if added symbol is an LExpression.
//
// Revision 1.21  1998/02/05 11:07:37  wassel
// Added AD/TDD Runtime methods.
// Added factory newFunctionType.
// Added other factories to override SymbolTableMgr target language independant factories.
//
// Revision 1.20  1998/01/30 12:34:02  wassel
// Added CFunctionType.java to handle int (x)( char * ).
// Added a factory in symboltable for CFnctionType.
//
// Revision 1.19  1998/01/27 13:29:39  wassel
// Factories construct CPointer, CPointerToFunction, CReference instead of LPointer, LPointeToFunction LReference resp.
//
// Revision 1.18  1998/01/19 14:37:08  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.17  1998/01/13 09:00:50  cinard
// Fixed javadoc comments.
//
// Revision 1.16  1998/01/13 08:41:53  wassel
// Added method isConstructor( String )
//
// Revision 1.15  1998/01/12 13:19:18  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.14  1998/01/06 13:32:06  wassel
// Added CDestructor symbol
//
// Revision 1.13  1997/12/08 17:13:14  cinard
// Fixed some issues with C++ builtin types.
//
// Revision 1.12  1997/12/08 15:16:54  wassel
// Method newModifier made public to be usabale by Wassel
//
// Revision 1.11  1997/11/26 17:13:10  cinard
// Changed package name.
//
// Revision 1.10  1997/11/19 15:44:20  cinard
// Added getADLMethod() and newADLMethod() methods.
//
// Revision 1.9  1997/11/13 13:27:42  cinard
// Added a scope override parameter to putSymbol method.
// Both Symbol putSymbol(Symbol s, Scope scopeOverride) and
// Symbol putSymbol(Symbol s) can be used now.
//
// Revision 1.8  1997/11/13 09:43:32  cinard
// Improved Builtin type management. Notably symboltable.c.CSymbolTableMgr.
// getBuiltinType() handles all forms of builtin types names.
//
// Revision 1.7  1997/11/12 13:36:08  cinard
// Fixed some bugs (void type && assignability).
//
// Revision 1.6  1997/11/04 09:04:32  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symbol_table.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.5  1997/11/03 15:43:12  cinard
// Added CBuiltinType class.
//
// Revision 1.4  1997/11/03 12:56:11  cinard
// Made CSymbolTableMgr inherits from abstract SymbolTableMgr.
//
// Revision 1.3  1997/10/27 11:10:20  cinard
// Replaced the C and C++ symbols hierarchy of symboltable.c package by the
// language independant symbols hierarchy in symboltable packege.
// Added the factory methods (newVariable(), newClass(), newMethod()...)
// to CSymbolTableMgr class.
// Updated the unit testing CTest.java.
//
// Revision 1.2  1997/10/22 14:58:35  cinard
// Improved C and CPP symbols and symbol table manager. Notably :
// o added CPP, ADL and TDD useful symbols
// o added getMethod(), getExpression() and getType() methods in addition
//   to getSymbols() method in the CSymbolTableMgr class. This enables
//   to get symbols which have already been classified as methods,
//   expressions or types by the parser more efficiently.
// o added a static method isValidBuiltinTypeName() in the LBuiltinType
//   class so that the parser may check names syntax more efficiently
//   without requiring a call to the getSymbol method.
// o improved builtin type management (assignability checking...)
//
// Revision 1.1  1997/10/13 15:08:25  cinard
// Symbol and symbol management classes for the ADL/C parser.
//
// ***********************************************************************
