// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CTest.java,v $
// $Revision: 1.23 $
// $Date: 1998/06/08 13:42:18 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CTest.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 * Unit testing of this package.
 */

class CTest 
{

  public final static void main(String[] args) 
  throws SymbolAlreadyDefinedException
  {
     CTest ctest = new CTest();
     
     ctest.test1();
     ctest.test2();
     ctest.test3();
     ctest.test4();
     ctest.test5(); 
     ctest.test6();
     ctest.test7();
     ctest.test8();
     ctest.test9();
     ctest.test10();
     ctest.test11();
     ctest.test12();
     ctest.test13();
     ctest.test14();
  }
  

  private static void test1()
  throws SymbolAlreadyDefinedException 
  {
     System.out.println("");
     System.out.println("-- Testing symbol searching --");
     System.out.println("");
      
     CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
     CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

     // Declares a pointer type
     LPointer ptrIntT = sbf.newPointer((LType)stm.getSymbol("int"),0x00);
     stm.putSymbol(ptrIntT);
     
     // Declares a typedefed type
     stm.putSymbol(sbf.newTypedef("ptrIntTypedef",ptrIntT));
     
     // Declares a variable of simple type
     
     LVariable aValueV = sbf.newVariable
                         ("aValueV",0,(LType)stm.getSymbol("int") );
     stm.putSymbol(aValueV);

     // -- ASSERTION BEGIN --

     System.out.println
     (
         ( ptrIntT.assignable((LType)stm.getSymbol("ptrIntTypedef")) ) +
         ": testing assignability of ptrIntT to ptrIntTypedefT"
     );

      // -- ASSERTION END --

      LPointer pt1 = sbf.newPointer(CPPBuiltinType.intT,0x00);
      LPointer pt2 = sbf.newPointer(CPPBuiltinType.intT,0x00);
      System.out.println
      (pt1.equals(pt2) + ": testing int* == int*");


      // Defines a constructed type 

      LClass ComplexT = sbf.newStruct("ComplexT");
      stm.putSymbol(ComplexT);
      stm.openScope(ComplexT);
      LAttribute xS = sbf.newAttribute
                      ("x",
                       CModifier.PUBLIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(xS);
      LAttribute yS = sbf.newAttribute
                      ("y",
                       CModifier.PUBLIC & CModifier.STATIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(yS);

      // -- ASSERTION BEGIN --

      System.out.println
      (
         ( ( stm.getSymbol("ComplexT.x")) == xS ) +
         ": testing reachability of ComplexT.x in block"
      );
      System.out.println
      (
         ( ( stm.getSymbol("x")) == xS ) +
         ": testing reachability of x in block"
      );
      System.out.println
      (
         ( xS.assignable((LType)stm.getSymbol("int")) ) +
         ": testing assignability of intT to  ComplexT.x"
      );
      System.out.println
      (
         ( ! xS.assignable((LType)stm.getSymbol("long")) ) +
         ": testing unassignability of long to ComplexT.x"
      );


      // -- ASSERTION END --

      // Close current scope

      stm.closeScope();

      // -- ASSERTION BEGIN --

      System.out.println
      (
         ( ( stm.getSymbol("ComplexT")) == ComplexT ) +
         ": testing reachability of ComplexT out of block"
      );
      System.out.println
      (
         ( ( stm.getSymbol("ComplexT.x")) == xS ) +
         ": testing reachability of ComplexT.x out of block"
      );
      System.out.println
      (
         ( ( stm.getSymbol("x")) == null ) +
         ": testing unreachability of x out of block"
      );

      // -- ASSERTION END --

      // Defines a new constructed type

      Symbol PairT = sbf.newStruct("PairT");
      stm.putSymbol(PairT);
      stm.openScope(PairT);
      Symbol firstF = sbf.newAttribute
                          ("first",0,(LType)stm.getSymbol("ComplexT") );
      stm.putSymbol(firstF);
      Symbol secondF = sbf.newAttribute
                          ("second",0,(LType)stm.getSymbol("ComplexT") );
      stm.putSymbol(secondF);

      // -- ASSERTIONS BEGIN --

      System.out.println
      (
         ( ( stm.getSymbol("PairT.first.x")) == xS ) +
         ": testing reachability of PairT.first.x in block"
      );
      System.out.println
      (
         ( ( stm.getSymbol("first.x")) == xS ) +
         ": testing reachability of first.x in block"
      );

      // -- ASSERTIONS END --

      stm.closeScope();

      // -- ASSERTIONS BEGIN --

      System.out.println
      (
         ( ( stm.getSymbol("PairT.first.x")) == xS ) +
         ": testing reachability of PairT.first.x out of block"
      );
      System.out.println
      (
         ( ( stm.getSymbol("first.x")) == null ) +
         ": testing unreachability of first.x out of block"
      );

      // -- ASSERTIONS END --

      // Defines a function

      LFunction bar = sbf.newFunction("bar",0);
      stm.putSymbol(bar);
      bar.setReturnType((LType)stm.getSymbol("int"));
      bar.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      bar.addParameter(sbf.newParameter("q",0,(LType)stm.getSymbol("int")));

      // Opens function scope

      stm.openScope(bar);

      stm.putSymbol(sbf.newVariable("complexV",0,ComplexT ));

      // Open a block scope inside function body

      stm.openScope();
      stm.putSymbol(sbf.newVariable("complex1V",0,ComplexT ));
      stm.putSymbol(sbf.newVariable("complex2V",0,ComplexT ));
      stm.closeScope();

      stm.closeScope();

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }


  private static void test2() 
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing function searcher --");
      System.out.println("");

      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Defines a function

      LFunction foo = sbf.newFunction("foo",0);
      stm.putSymbol(foo);
      foo.setReturnType((LType)stm.getSymbol("int"));
      foo.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      foo.addParameter
      (sbf.newParameter("q",0,sbf.newPointer((LType)stm.getSymbol("int"),0x00)));

      // Search for foo function

      LCallableAssignComparator fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("int"));
      fs.addParameterType(sbf.newPointer((LType)stm.getSymbol("int"),0x00));

      Symbol s1 = stm.getSymbol("foo",fs,null,null);

      System.out.println
      (
         ( s1 == foo ) +
         ": testing reachability of foo() - good searcher"
      );

      // Search for foo function

      fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("long"));
      fs.addParameterType(sbf.newPointer((LType)stm.getSymbol("int"),0x00));

      s1 = stm.getSymbol("foo",fs,null,null);

      System.out.println
      (
         ( s1 == null ) +
         ": testing unreachability of foo() - bad searcher"
      );

      // Defines a function

      LFunction bar = sbf.newFunction("bar",0);
      stm.putSymbol(bar);
      bar.setReturnType((LType)stm.getSymbol("int"));
      bar.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      bar.addParameter(sbf.newParameter("q",0,(LType)stm.getSymbol("int")));


      // Search for bar function

      fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("int"));
      fs.addParameterType((LType)stm.getSymbol("int"));

      Symbol s2 = stm.getSymbol("bar",fs,null,null);

      System.out.println
      (
         ( s2 == bar ) +
         ": testing reachability of bar() - good searcher"
      );

      // Search for bar function

      fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("int"));
      fs.addParameterType((LType)stm.getSymbol("long"));

      Symbol s3 = stm.getSymbol("bar",fs,null,null);

      System.out.println
      (
         ( s3 == null ) +
         ": testing unreachability of bar() - bad searcher"
      );

      // Defines "w" function

      LFunction w = sbf.newFunction("w",0);
      stm.putSymbol(w);
      w.setReturnType(CPPBuiltinType.boolT);
      w.addParameter
      (sbf.newParameter("p",0,sbf.newPointer(CPPBuiltinType.intT,0x00)));

      // Search for "w" function

      LCallableEqualsComparator wc = new LCallableEqualsComparator();
      wc.setReturnType(CPPBuiltinType.boolT);
      wc.addParameterType(sbf.newPointer(CPPBuiltinType.intT,0x00));

      Symbol _w = stm.getSymbol("w",wc,null,null);

      System.out.println
      (
         ( _w == w ) +
         ": testing reachability of w() - good searcher"
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }

  private static void test3() 
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing assignability --");
      System.out.println("");


      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Get ref on builtin types
      LType intT  = (LType)stm.getSymbol("int");
      LType longT = (LType)stm.getSymbol("long");

      // Typedef
      LType intTypedef = sbf.newTypedef("intTypedef",intT);
      stm.putSymbol(intTypedef);

      // Declares a pointer type
      LPointer ptrIntT = sbf.newPointer((LType)stm.getSymbol("int"),0x00);
      stm.putSymbol(ptrIntT);

      // Declares an array type
      LArray intArrayT = (LArray)sbf.newArray((LType)stm.getSymbol("int"));
      stm.putSymbol(intArrayT);

      // Declares a typedefed type
      stm.putSymbol(sbf.newTypedef("ptrIntTypedef",ptrIntT));
       
      // Declares a variable of simple type

      LVariable aValueV = sbf.newVariable("aValueV",0,intTypedef);
      stm.putSymbol(aValueV);

      LVariable gogo = sbf.newVariable("GOGO",0,intTypedef);
      stm.putSymbol(gogo);

      // -- ASSERTION BEGIN --

      System.out.println
      (
         ( intT.assignable(intT) ) +
         ": testing assignability of int to int"
      );

      System.out.println
      (
         ( intTypedef.assignable(intT) ) +
         ": testing assignability of int to typedef int"
      );

      System.out.println
      (
         ( intT.assignable(intTypedef) ) +
         ": testing assignability of typedef int to int"
      );

      System.out.println
      (
         ( longT.assignable(intT) ) +
         ": testing assignability of int to long"
      );

      System.out.println
      (
         ( ! ptrIntT.assignable(longT) ) +
         ": testing non assignability of long to int*"
      );

      System.out.println
      (
         ( ptrIntT.assignable(ptrIntT) ) +
         ": testing assignability of int* to int*"
      );

      System.out.println
      (
         ((LType)stm.getSymbol("ptrIntTypedef")).assignable(ptrIntT) +
         ": testing assignability of int* to typedefined int*"
      );

      System.out.println
      (
       ( intArrayT.assignable(sbf.newArray((LType)stm.getSymbol("int"))) ) +
       ": testing assignability of int[] to int[]"
      );

      // -- ASSERTION END --

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);

   }

  // Test callbacks

  private void test4()
  throws SymbolAlreadyDefinedException 
  {

      System.out.println("");
      System.out.println("-- Testing callback mechanism --");
      System.out.println("");

      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Prepare foo function searcher

      LCallableAssignComparator fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("int"));
      fs.addParameterType((LType)stm.getSymbol("int"));

      // Try to get foo function symbol (provide this as a FunctionGetter
      // object). The call should fail.

      Symbol s1 = stm.getSymbol("foo",fs,new MethodSymbolListener(),null);

      System.out.println
      (
         ( s1 == null ) +
         ": testing unreachability of foo() - not defined"
      );

      // Defines the function

      LFunction foo = sbf.newFunction("foo",0);
      stm.putSymbol(foo);
      foo.setReturnType((LType)stm.getSymbol("int"));
      foo.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      foo.addParameter(sbf.newParameter("q",0,(LType)stm.getSymbol("int")));

      // Search for foo function

      Symbol s2 = stm.getSymbol("foo",fs,null,null);

      System.out.println
      (
         ( s2 == foo ) +
         ": testing reachability of foo() - issued"
      );

      // Ask symbol table manager to issue call backs.

      stm.finishParsing();

      // Check foo was retured

      System.out.println
      (
         ( gotAfterCallback == foo ) +
         ": testing callback was issued"
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);

   }

   class MethodSymbolListener implements SymbolListener
   {
      public void symbolFound(Symbol s)
      {
         gotAfterCallback = s;
      }
   }

  private static void test5()
  throws SymbolAlreadyDefinedException  
  {
     System.out.println("");
     System.out.println("-- Testing CPP types --");
     System.out.println("");

     CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
     CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();
      
     // Creates some classes

      LClass c1 = sbf.newClass("C1",CModifier.PUBLIC);
      LClass c2 = sbf.newClass("C2",CModifier.PUBLIC);

      LClass c3 = sbf.newClass("C3",CModifier.PUBLIC);
      c3.addSuperClass(c1);
      c3.addSuperClass(c2);

      LClass c4 = sbf.newClass("C4",CModifier.PUBLIC);
      c4.addFriend(c3);

      LAttribute att1 = sbf.newAttribute("attribute1",0,c1);
      c4.putSymbol(att1);
      LAttribute att2 = sbf.newAttribute("attribute2",0,c1);
      c4.putSymbol(att2);

      // Defines a method.

      LMethod m1 = sbf.newMethod("m1",0);
      m1.setReturnType(c1);
      m1.addParameter(sbf.newParameter("p",0,c1));
      m1.addParameter(sbf.newParameter("q",0,c1));
      c4.putSymbol(m1);
      

      // Display them

      System.out.println(c1);
      System.out.println(c2);
      System.out.println(c3);
      System.out.println(c4);

      // Creates some variables

      // Creates some adl classes

      ADLClass a1 = sbf.newADLClass(c1,"");
      ADLClass a2 = sbf.newADLClass(c2,"");
      a2.addSuperClass(a1);

      // Display them

      System.out.println(a1);
      System.out.println(a2);

      // Creates a TDD class

      TDDClass t1 = sbf.newTDDClass("T1");

      // Display them

      System.out.println(t1);

   }

  private static void test6()
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing CPP symbol searching --");
      System.out.println("");
      
      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // -- ASSERTION END --


      // Defines a class type 

      LClass ComplexT = sbf.newClass("ComplexT",CModifier.PUBLIC);
      stm.putSymbol(ComplexT);
      stm.openScope(ComplexT);

      // Add an attribute

      LAttribute foo1 = sbf.newAttribute("foo",0,(LType)stm.getSymbol("int") );
      stm.putSymbol(foo1);

      // Add a method

      LMethod foo2 = sbf.newMethod("foo",0);
      stm.putSymbol(foo2);
      foo2.setReturnType((LType)stm.getSymbol("int"));
      foo2.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));

      // Overload this method

      LMethod foo3 = sbf.newMethod("foo",0);
      stm.putSymbol(foo3);
      foo3.setReturnType((LType)stm.getSymbol("int"));
      foo3.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      foo3.addParameter(sbf.newParameter("q",0,(LType)stm.getSymbol("int")));


      // -- ASSERTION BEGIN --

      System.out.println
      (
         ( stm.getSymbol("this") != null ) +
         ": tesing reachability of this in class definition"
      );

      // Close current scope

      stm.closeScope();

      System.out.println
      (
         ( stm.getSymbol("Complex.this") == null ) +
         ": tesing unreachability of this outside class definition"
      );

      // Find foo1
      Symbol s1;
      try 
      {
         s1 = stm.getExpression("ComplexT.foo",null);
      }
      catch ( Exception e )
      {
         System.out.println(e);
         s1 = null;
      }
 
      System.out.println
      (
         ( s1 == foo1 ) +
         ": testing reachability of foo attribute "
      );


      // Defines a predicate to find foo2

      LCallableAssignComparator p2 = new LCallableAssignComparator();
      p2.addParameterType((LType)stm.getSymbol("int"));

      Symbol s2 = stm.getSymbol("ComplexT.foo",p2,null,null);
 
      System.out.println
      (
         ( s2 == foo2 ) +
         ": testing reachability of int foo(int) "
      );

      // Defines a predicate to find foo3

      LCallableAssignComparator p3 = new LCallableAssignComparator();
      p3.addParameterType((LType)stm.getSymbol("int"));
      p3.addParameterType((LType)stm.getSymbol("int"));

      Symbol s3 = stm.getSymbol("ComplexT.foo",p3,null,null);

      System.out.println
      (
         ( s3 == foo3 ) +
         ": testing reachability of int foo(int,int)"
      );


      // -- ASSERTION END --

      stm.closeScope();

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

  private static void test7()
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing pointer to function --");
      System.out.println("");
      
      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Add a method

      LFunction foo = sbf.newFunction("foo",0);
      foo.setReturnType((LType)stm.getSymbol("int"));
      foo.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      stm.putSymbol(foo);

      // Add a method

      LFunction bar = sbf.newFunction("bar",0);
      bar.setReturnType((LType)stm.getSymbol("int"));
      bar.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      bar.addParameter(sbf.newParameter("q",0,(LType)stm.getSymbol("int")));
      stm.putSymbol(bar);

      // Defines a predicate to create pointer to functions with foo prototype

      LCallableEqualsComparator p = new LCallableEqualsComparator();
      p.setReturnType((LType)stm.getSymbol("int"));
      p.addParameterType((LType)stm.getSymbol("int"));

      // Defines a pointer to function (foo like).

      LPointerToFunction ptFn = sbf.newPointerToFunction(p,0x00);
      stm.putSymbol(ptFn);

      // Defines a variable of type ptFn
      LVariable v = sbf.newVariable("blou",0,ptFn);
      stm.putSymbol(v);

      // -- Assertions --

      System.out.println
      (
         ( ptFn.assignable(foo) ) +
         ": testing assignibility of foo() to pointer to function"
      );

      System.out.println
      (
         ( ! ptFn.assignable(bar) ) +
         ": testing non assignibility of bar() to pointer to function"
      );

      System.out.println
      (
         ( v.assignable(foo) ) +
         ": testing assignibility of foo() to v"
      );


      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

  private static void test8()
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing CPP namespace --");
      System.out.println("");
      
      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Defines namespaces

      LNameSpace ns1 = sbf.newNameSpace("NS1");
      stm.putSymbol(ns1);
      stm.openScope(ns1);

      LNameSpace ns2 = sbf.newNameSpace("NS2");
      stm.putSymbol(ns2);
      stm.openScope(ns2);

      // Defines a class type 

      LClass ComplexT = sbf.newClass("ComplexT",CModifier.PUBLIC);
      stm.putSymbol(ComplexT);
      stm.openScope(ComplexT);

      // Add an attribute

      LAttribute foo1 = sbf.newAttribute("foo",0,(LType)stm.getSymbol("int") );
      stm.putSymbol(foo1);

      // Add a method

      LMethod foo2 = sbf.newMethod("foo",0);
      stm.putSymbol(foo2);
      foo2.setReturnType((LType)stm.getSymbol("int"));
      foo2.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));

      stm.closeScope(); // Class
      stm.closeScope(); // Namspace 2
      stm.closeScope(); // Namspace 1

      Symbol s1 = stm.getSymbol("NS1::");
      Symbol s2 = stm.getSymbol("NS1::NS2::");
      Symbol s3 = stm.getSymbol("NS1::NS2::ComplexT");
      Symbol s4 = stm.getExpression("NS1::NS2::ComplexT.foo",null);
      
      System.out.println
      (
         ( s1 == ns1 ) +
         ": testing reachability of NS1::"
      );

      System.out.println
      (
         ( s2 == ns2 ) +
         ": testing reachability of NS1::NS2::"
      );

      System.out.println
      (
         ( s3 == ComplexT ) +
         ": testing reachability of NS1::NS2::ComplexT"
      );

      System.out.println
      (
         ( s4 == foo1 ) +
         ": testing reachability of NS1::NS2::ComplexT.foo"
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

  private static void test9()
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing isInScope()  --");
      System.out.println("");
      
      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      // Defines scope classes
      Class lCallableClass = LCallable.class;
      Class lClassClass    = LClass.class;

      // Defines a namespace

      LNameSpace ns = sbf.newNameSpace("NS");
      stm.putSymbol(ns);
      stm.openScope(ns);

      // Defines a class type 

      LClass ComplexT = sbf.newClass("ComplexT",CModifier.PUBLIC);
      stm.putSymbol(ComplexT);
      stm.openScope(ComplexT);

      System.out.println
      (
         ( stm.isInScope( lClassClass) ) +
         ": in class scope"
      );

      System.out.println
      (
         ( ! stm.isInScope( lCallableClass ) ) +
         ": not in methode scope"
      );

      // Add an attribute

      LAttribute foo1 = sbf.newAttribute("foo",0,(LType)stm.getSymbol("int") );
      stm.putSymbol(foo1);

      // Add a method

      LMethod foo2 = sbf.newMethod("foo",0);
      stm.putSymbol(foo2);
      foo2.setReturnType((LType)stm.getSymbol("int"));
      foo2.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      stm.openScope(foo2);

      System.out.println
      (
         ( stm.isInScope( lClassClass) ) +
         ": in class scope"
      );

      System.out.println
      (
         ( stm.isInScope( lCallableClass ) ) +
         ": in methode scope"
      );
      stm.closeScope(); // Method
      stm.closeScope(); // Class
      stm.closeScope(); // Namspace

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

  private static void test10() 
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing function searcher --");
      System.out.println("");

      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      LClass a = sbf.newClass("A",0);
      stm.putSymbol(a);
      stm.openScope(a);

      // Defines foo1

      LFunction foo1 = sbf.newFunction("foo",0);
      stm.putSymbol(foo1);
      foo1.setReturnType((LType)stm.getSymbol("int"));
      foo1.addParameter(sbf.newParameter("c",0,(LType)stm.getSymbol("char")));

      // Defines foo2

      LFunction foo2 = sbf.newFunction("foo",0);
      stm.putSymbol(foo2);
      foo2.setReturnType((LType)stm.getSymbol("char"));
      foo2.addParameter(sbf.newParameter("c",0,(LType)stm.getSymbol("char")));

      // Searcher for foo1

      LCallableEqualsComparator lc1 = new LCallableEqualsComparator();
      lc1.setReturnType((LType)stm.getSymbol("int"));
      lc1.addParameterType((LType)stm.getSymbol("char"));

      // Searcher for foo2 

      LCallableEqualsComparator lc2 = new LCallableEqualsComparator();
      lc2.setReturnType((LType)stm.getSymbol("char"));
      lc2.addParameterType((LType)stm.getSymbol("char"));

      Symbol s1 = stm.getSymbol("foo",lc1,null,null);
      Symbol s2 = stm.getSymbol("foo",lc2,null,null);

      System.out.println
      (
         ( s1 == foo1 ) +
         ": testing reachability of int foo(char) - good searcher"
      );
      
      System.out.println
      (
         ( s2 == foo2 ) +
         ": testing reachability of char foo(char) - good searcher"
      );

      System.out.println
      (
         ( ! foo1.equals(foo2) ) +
         ": testing foo1 != foo2 "
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }

  private static void test11() 
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing builtin types --");
      System.out.println("");

      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();

      try 
      {
         Symbol t1 = stm.getBuiltinType("int");
         Symbol t2 = stm.getBuiltinType("int signed");
         System.out.println(( t1.equals(t2) ) + ": testing int == signed int");
      } catch ( BadSymbolNameException e ) { }

      try 
      {
         Symbol t3 = stm.getBuiltinType("unsigned int unsigned");
         Symbol t4 = stm.getBuiltinType("unsigned");
         
         System.out.println
         (( t3.equals(t4) ) + ": testing unsigned int unsigned == unsigned");
      } catch ( BadSymbolNameException e ) { }


      try { Symbol t5 = stm.getBuiltinType("int long"); }
      catch ( BadSymbolNameException e )
      { System.out.println( true + ": " + e.getMessage() ); }

      try { Symbol t6 = stm.getBuiltinType("unsigned signed"); }
      catch ( BadSymbolNameException e )
      { System.out.println( true + ": " + e.getMessage() ); }

      try { Symbol t7 = stm.getBuiltinType("unsigned long signed"); }
      catch ( BadSymbolNameException e )
      { System.out.println( true + ": " + e.getMessage() ); } 

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }

   private static void test12()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing types --");
      System.out.println("");

      CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
      CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();
      
      // Creates some classes

      LClass c1 = sbf.newClass("C1",CModifier.PUBLIC);
      stm.putSymbol(c1);
      stm.openScope(c1);

      // Defines a method.

      LMethod m1 = sbf.newMethod("m1",0x0000);
      m1.setReturnType(c1);
      m1.addParameter(sbf.newParameter("p",0x0000,c1));
      m1.addParameter(sbf.newParameter("q",0x0000,c1));
      stm.putSymbol(m1);

      stm.closeScope();

      LClass c2 = sbf.newClass("C2",CModifier.PUBLIC);
      stm.putSymbol(c2);
      stm.openScope(c2);
      c2.addSuperClass(c1);

      LAttribute f1 = sbf.newAttribute("field1",CModifier.PUBLIC,c1);
      stm.putSymbol(f1);
      LAttribute f2 = sbf.newAttribute("field2",CModifier.PRIVATE,c1);
      stm.putSymbol(f2);

      // Overload the method.

      LMethod m2 = sbf.newMethod("m1",0x0000);
      m2.setReturnType(c1);
      m2.addParameter(sbf.newParameter("p",0x0000,c1));
      m2.addParameter(sbf.newParameter("q",0x0000,c1));
      stm.putSymbol(m2);
      
      stm.closeScope(); // C2 scope

      // Creates some variables

      LVariable v1 = sbf.newVariable("v1",0x0000,c2);

      stm.putSymbol(v1);

      // Creates some adl classes

      ADLClass a1 = sbf.newADLClass(c1,"");
      stm.putSymbol(a1);
      ADLClass a2 = sbf.newADLClass(c2,"");
      stm.putSymbol(a2);
      a2.addSuperClass(a1);
      stm.openScope(a2);

      // Add semantics

      ADLMethod am1 = sbf.newADLMethod("m1",0x0000);
      am1.setReturnType(c1);
      am1.addParameter(sbf.newParameter("p",0x0000,c1));
      am1.addParameter(sbf.newParameter("q",0x0000,c1));
      stm.putSymbol(am1);    
  
      stm.closeScope(); // a2

      // Creates a TDD class

      TDDClass t1 = sbf.newTDDClass("T1");

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);

   }

  private static void test13()
  throws SymbolAlreadyDefinedException 
  {
     System.out.println("");
     System.out.println("-- Testing local searching --");
     System.out.println("");
      
     CPPSymbolTableMgr stm = new CPPSymbolTableMgr();
     CPPSymbolFactory  sbf = (CPPSymbolFactory)stm.getSymbolFactory();


      // Defines a constructed type 

      LClass _S = sbf.newStruct("S");
      stm.putSymbol(_S);
      stm.openScope(_S);
      LAttribute _x = sbf.newAttribute
                      ("x",
                       CModifier.PUBLIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(_x);
      LAttribute _y = sbf.newAttribute
                      ("y",
                       CModifier.PUBLIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(_y);
      LClass _T = sbf.newStruct("T");
      stm.putSymbol(_T);
      stm.openScope(_T);
      LAttribute _z = sbf.newAttribute
                      ("z",
                       CModifier.PUBLIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(_z);
      LAttribute _t = sbf.newAttribute
                      ("t",
                       CModifier.PUBLIC,
                       (LType)stm.getSymbol("int")
                      );
      stm.putSymbol(_t);
      stm.closeScope();
      stm.closeScope();

      Symbol v = sbf.newVariable("v",0x00,_T);
      stm.putSymbol(v);
      Symbol w = sbf.newVariable("w",0x00,_S);
      stm.putSymbol(w);
      // -- ASSERTION BEGIN --

      Scope s = (Scope)stm.getType("S::T",null);
      System.out.println
      (
         ( ( stm.getExpression("x",s)) == _x ) +
         ": testing misuse of stm (wrong reachability S::T.x)"
      );
      System.out.println
      (
         ( ( stm.getExpression("T.x",null,true)) == null ) +
         ": testing unreachability T.x"
      );
      System.out.println
      (
         ( ( stm.getExpression("S.x",null,true)) == _x ) +
         ": testing reachability S.x"
      );


      System.out.println
      (
         ( ( stm.getExpression("v.x",null,true)) == null ) +
         ": testing unreachability v.x"
      );
      System.out.println
      (
         ( ( stm.getExpression("w.x",null,true)) == _x ) +
         ": testing reachability w.x"
      );

      // -- ASSERTION END --

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

  private static void test14() 
  throws SymbolAlreadyDefinedException 
  {
      System.out.println("");
      System.out.println("-- Testing ellipsis --");
      System.out.println("");

      CSymbolTableMgr stm = new CSymbolTableMgr();
      CSymbolFactory  sbf = (CSymbolFactory)stm.getSymbolFactory();

      // Defines a function

      LFunction foo = sbf.newFunction("foo",0);
      stm.putSymbol(foo);
      foo.setReturnType((LType)stm.getSymbol("int"));
      foo.addParameter(sbf.newParameter("p",0,(LType)stm.getSymbol("int")));
      foo.addParameter
      (sbf.newParameter("q",0,sbf.newPointer((LType)stm.getSymbol("int"),0x00)));
      foo.setLastParameterIsEllipsis();

      // Search for foo function

      LCallableAssignComparator fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("int"));
      fs.addParameterType(sbf.newPointer((LType)stm.getSymbol("int"),0x00));
      fs.addParameterType(CBuiltinType.intT);

      Symbol s1 = stm.getSymbol("foo",fs,null,null);

      System.out.println
      (
         ( s1 == foo ) +
         ": testing reachability of foo() - good searcher"
      );

      // Search for foo function

      fs = new LCallableAssignComparator();
      fs.addParameterType((LType)stm.getSymbol("long"));

      s1 = stm.getSymbol("foo",fs,null,null);

      System.out.println
      (
         ( s1 == null ) +
         ": testing unreachability of foo() - bad searcher"
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }

   // -- Private --
   
   private Symbol gotAfterCallback;
   private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: CTest.java,v $
// Revision 1.23  1998/06/08 13:42:18  cinard
// Replaced  calls to obsolete method LClass.setSuperClass() with
// calls to LClass.addSuperClass().
//
// Revision 1.22  1998/04/28 12:33:07  cinard
// Updated tests for C symbol table manager.
//
// Revision 1.21  1998/03/25 14:48:36  cinard
// Updated newADLClass() method (now super class name is required).
//
// Revision 1.20  1998/03/18 15:51:29  cinard
// Symbol factory in now separated from symbol table.
//
// Revision 1.19  1998/03/02 15:32:29  cinard
// Replaced LStruct with LClass (struct symbols behave just like classes).
// Factory and find methods (newStruct() and getStruct() remains with
// their signatures updated).
//
// Revision 1.18  1998/02/27 12:42:14  cinard
// Updated copyright notice.
//
// Revision 1.17  1998/02/25 11:05:44  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
//
// Revision 1.16  1998/02/23 14:49:50  cinard
// Added support for typedefs.
//
// Revision 1.15  1998/02/12 16:03:29  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.14  1998/01/22 11:42:31  cinard
// Fixed a bug in symbols equality checks.
//
// Revision 1.13  1998/01/12 13:19:18  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.12  1997/12/08 12:46:30  cinard
// Calls to the ORG.opengroup.adl.symboltable.SymbolTableMgr.newArray()
// have been modified. The function returns now a LType instead of a
// LArray.
//
// Revision 1.11  1997/11/26 17:13:10  cinard
// Changed package name.
//
// Revision 1.10  1997/11/19 15:45:01  cinard
// Added tests for method overloading and ADLMethod management.
//
// Revision 1.9  1997/11/14 09:58:52  cinard
// LModule scope now accepts duplicate symbols.
// SymbolTableMgr has a getADLClass() method.
//
// Revision 1.8  1997/11/13 09:43:32  cinard
// Improved Builtin type management. Notably symboltable.c.CSymbolTableMgr.
// getBuiltinType() handles all forms of builtin types names.
//
// Revision 1.7  1997/11/12 13:36:09  cinard
// Fixed some bugs (void type && assignability).
//
// Revision 1.6  1997/11/04 16:04:27  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.5  1997/11/04 09:04:32  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symbol_table.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.4  1997/11/03 12:56:11  cinard
// Made CSymbolTableMgr inherits from abstract SymbolTableMgr.
//
// Revision 1.3  1997/10/27 11:10:20  cinard
// Replaced the C and C++ symbols hierarchy of symboltable.c package by the
// language independant symbols hierarchy in symboltable packege.
// Added the factory methods (newVariable(), newClass(), newMethod()...)
// to CSymbolTableMgr class.
// Updated the unit testing CTest.java.
//
// Revision 1.2  1997/10/22 14:58:35  cinard
// Improved C and CPP symbols and symbol table manager. Notably :
// o added CPP, ADL and TDD useful symbols
// o added getMethod(), getExpression() and getType() methods in addition
//   to getSymbols() method in the CSymbolTableMgr class. This enables
//   to get symbols which have already been classified as methods,
//   expressions or types by the parser more efficiently.
// o added a static method isValidBuiltinTypeName() in the LBuiltinType
//   class so that the parser may check names syntax more efficiently
//   without requiring a call to the getSymbol method.
// o improved builtin type management (assignability checking...)
//
// Revision 1.1  1997/10/13 15:08:25  cinard
// Symbol and symbol management classes for the ADL/C parser.
//
// ***********************************************************************
