// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/c/CType.java,v $
// $Revision: 1.4 $
// $Date: 1998/04/28 12:30:41 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.c
// NAME:        CType.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 29,  1997: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable.c;

import ORG.opengroup.adl.symboltable.*;

/**
 * Helper methods to manipulate C types.
 */

public class CType
{


  /** Is type a char? */
  static public boolean isChar(LType t)
  {
     return      CBuiltinType.charT.equals(t);
  }

  /** Is type a signed integer? */
  static public boolean isSignedInteger(LType t)
  {
     return      CBuiltinType.charT.equals(t)
              || CBuiltinType.shortT.equals(t)
              || CBuiltinType.intT.equals(t)
              || CBuiltinType.longT.equals(t);
  }

  /** Is type an unsigned integer? */
  static public boolean isUnsignedInteger(LType t)
  {
     return      CBuiltinType.ucharT.equals(t)
              || CBuiltinType.ushortT.equals(t)
              || CBuiltinType.uintT.equals(t)
              || CBuiltinType.ulongT.equals(t);
  }

  /** Is type a floating type? */
  static public boolean isFloat(LType t)
  {
     return      CBuiltinType.floatT.equals(t)
              || CBuiltinType.doubleT.equals(t)
              || CBuiltinType.ldoubleT.equals(t);
  }

  /** Is type a basic type? */
  static public boolean isBasic(LType t)
  {
     return    isChar(t) 
            || isSignedInteger(t) 
            || isUnsignedInteger(t) 
            || isFloat(t);
  }

  /** Is type an integral type? */
  static public boolean isIntegral(LType t)
  {
     return    isChar(t) 
            || isSignedInteger(t)
            || isUnsignedInteger(t) 
            || t instanceof CEnum;
  }

  /** Is type arithmetic? */
  static public boolean isArithmetic(LType t)
  {
     return isIntegral(t) || isFloat(t);
  }

  /** Is type a pointer? */
  static public boolean isPointer(LType t)
  {
     return     ( t instanceof LPointer )
             || ( t instanceof LPointerToFunction ) ;
  }

  /** Is type scalar? */
  static public boolean isScalar(LType t)
  {
     return isArithmetic(t) || isPointer(t);
  }

  /** Is expression with supplied type a function designator? */
  static public boolean isFunctionDesignator(LType t)
  {
     return t instanceof CFunctionType;
  }

  /** Is type a bitfield? */
  static public boolean isBitfield(LType t)
  {
     // TBC - implement method!
     return false;
  }

  /** Is type a pointer to an object or an incoplete type? */
  static public boolean isPointerToObject(LType t)
  {
     return t instanceof LPointer;
  }

  /** Is type a pointer to a qualified or unqualified version of void. */
  static public boolean isVoidPointer(LType t)
  {
     if ( t instanceof LPointer ) 
     {
        LType pointed = ((LPointer)t).getPointedToType();
        return ( pointed != null ) && pointed.isVoid();
     }
     else 
     {
        return false;
     }
  }

  /** Is null pointer constant. */
  static public boolean isNullPointerConstant(LType t)
  {
     return CPointer._NULL.equals(t);
  }

  private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: CType.java,v $
// Revision 1.4  1998/04/28 12:30:41  cinard
// Implements methods usefull to manage C types (e.g. isArithmetic(),
// isScalar()...).
//
// ***********************************************************************
