// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JADLCompilationUnitInfo.java,v $ 
// $Revision: 1.3 $
// $Date: 1998/07/28 12:28:18 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JADLCompilationUnitInfo.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;

import java.util.Vector;
import com.objectspace.jgl.HashSet;

/**
 * Information used when compiling an ADL Compilation Unit.
 */

public class  JADLCompilationUnitInfo extends ADLCompilationUnitInfo
{

  /**
   * Constructs ADL Compilation unit context information.
   * This information shall be used to manipulate symbols
   * of a given compilation unit.
   * @param importOnDemands list of import on demand.
   * @param singleImports   list of single import.
   * @param usedADLClasses  list of used ADL classes.
   * @param usedTDDClasses  list of used TDD classes.
   */

  public JADLCompilationUnitInfo(Vector importOnDemands,
                                 Vector singleImports,
                                 Vector usedADLClasses,
                                 Vector usedTDDClasses,
                                 HashSet classesFromUnamedPackage)
  {
     super(usedADLClasses, usedTDDClasses);
     _importOnDemands = importOnDemands;
     _singleImports = singleImports;
     _classesFromUnamedPackage = classesFromUnamedPackage;
  }
  
  /**
   * Get import on demands.
   */

  public Vector getImportOnDemands()
  {
     return _importOnDemands;
  }

  /**
   * Get single imports.
   */

  public Vector getSingleImports()
  {
     return _singleImports;
  }

  /** 
   * Get classes from unnamed package.
   */
  public HashSet getClassesFromUnamedPackage()
  {
     return _classesFromUnamedPackage;
  }

  // -- Private --

  /** Import on demands. */
  private final Vector _importOnDemands;

  /** Single imports. */
  private final Vector _singleImports;

  /** Classes from unamed package. */
  private final HashSet _classesFromUnamedPackage;

}

// ***********************************************************************
// History Log:
// $Log: JADLCompilationUnitInfo.java,v $
// Revision 1.3  1998/07/28 12:28:18  mallet
// Added list of used ADL classes in constructor parameters.
//
// Revision 1.2  1998/03/19 09:17:56  cinard
// Added unamed packages to stm context.
//
// Revision 1.1  1998/03/18 15:52:58  cinard
// Implemented support for symbol table context (mainly
// used to keep context information when parsing and
// typecheck are separated).
//
// ***********************************************************************
