// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JArray.java,v $
// $Revision: 1.3 $
// $Date: 1998/04/28 14:02:48 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        JArray.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 19,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;

/**
 *  Java Array Symbol.
 */

public class JArray extends LObjectArray
{

  // -- Constructors --

  /** 
   * Constructs an array type.
   * @param elememtType type of the elements in the array.
   */

  public JArray(LType elementType)
  {
     super(elementType);
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t)
  {
     if ( ! ( t instanceof LType ) ) { return false; }
     
     LType other = (LType)t;
     
     if ( this.equals(other) || other.equals(JTypes._null) ) { return true; }

     if (    JTypeKind.REFERENCE_TYPE.equals(other.getTypeKind())
          && other instanceof LObjectArray )
     {
        LType btFirst  = this;
        LType btSecond = other;
        
        while (    ( btFirst  instanceof LObjectArray )
                && ( btSecond instanceof LObjectArray ))
        {
           btFirst  = ((LObjectArray)btFirst).getElementType();
           btSecond = ((LObjectArray)btSecond).getElementType();
        }
              
        if ( btFirst  instanceof LClass
             && btSecond instanceof LClass) {
          return ((LClass)btSecond).subclassOf((LClass)btFirst);
        }
        else if (btFirst instanceof LBuiltinType &&
                 btSecond instanceof LBuiltinType) {
          return btFirst.equals(btSecond);
        }
          
     }
     return false;
  }

  /** 
   *  Get the type kind. Useful for languages that defines several
   *  type kinds (e.g. in Java boolean, arithmetic and references).
   *  @return an integer indicating the kind of the type. 
   */

  public Integer getTypeKind() 
  {
     return JTypeKind.REFERENCE_TYPE;
  }
}

// ***********************************************************************
// History Log:
// $Log: JArray.java,v $
// Revision 1.3  1998/04/28 14:02:48  bareau
// Bug fix: assignable was incorrect when comparing arrays of builtin-types.
//
// Revision 1.2  1998/02/27 12:38:43  cinard
// Updated copyright notice.
//
// Revision 1.1  1998/02/23 17:50:25  cinard
// Classe implementing Java versions of the symbol interfaces.
//
// ***********************************************************************
