// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JBuiltinType.java,v $
// $Revision: 1.12 $
// $Date: 1998/02/27 12:38:43 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JBuiltinType.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 29,  1997: Created
//      See Log at end of file
//
// ***********************************************************************


package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;


import java.util.Hashtable;
import java.util.Enumeration;


/**
 * Java builtin type class.
 */

public class JBuiltinType extends LBuiltinTypeImpl
{
  /** Debug flag. */
  private static final boolean debug = false;

  /**
   * Constructs a Java builtin type.
   * @param name name of the type.
   */

  JBuiltinType(String name)
  {
     super(name);
  }

  /**
   * Get a builtin type.
   * @param name name of the builtin type.
   * @return the found builtin type (null if no type is found)
   */

  public static LBuiltinType getBuiltinType(String name)
  {
     return (LBuiltinType) builtinTypes.get(name);
  }

  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  static public boolean isValidBuiltinTypeName(String name)
  {
     return builtinTypes.containsKey(name);
  }

 /**
  * Returns true if this type is equals to type t.
  * Note that if t is null it is considered equals to voidT.
  * @param t symbol for which the test is performed.
  */

  public boolean equals(Object t)
  {
     if ( t == null )
     {
        return ( this == JTypes.voidT );
     }
     else
     {
        return super.equals(t);
     }
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t)
  {
     // Other type.
     if ( ! ( t instanceof LType ) ) { return false; }

     LType other = (LType)t;
     
     if      ( other == null )                               { return false; }
     else if ( JTypeKind.REFERENCE_TYPE.equals(other.getTypeKind()) )  
                                                             { return false; }
     else if ( this.equals(other) )                          { return true;  }
     else if ( this.equals(JBuiltinType.shortT)  )
     {
        return    other.equals(JBuiltinType.byteT);
     }
     else if ( this.equals(JBuiltinType.intT)  )
     {
        return    other.equals(JBuiltinType.byteT)
               || other.equals(JBuiltinType.charT) 
               || other.equals(JBuiltinType.shortT);
     }
     else if ( this.equals(JBuiltinType.longT) )
     {
        return    other.equals(JBuiltinType.byteT)
               || other.equals(JBuiltinType.charT) 
               || other.equals(JBuiltinType.shortT)
               || other.equals(JBuiltinType.intT) ;
     }
     else if ( this.equals(JBuiltinType.floatT) )
     {
        return    other.equals(JBuiltinType.byteT)   
               || other.equals(JBuiltinType.charT) 
               || other.equals(JBuiltinType.shortT)
               || other.equals(JBuiltinType.intT) 
               || other.equals(JBuiltinType.longT);
     }
     else if ( this.equals(JBuiltinType.doubleT)  )
     {
        return    other.equals(JBuiltinType.byteT)   
               || other.equals(JBuiltinType.charT) 
               || other.equals(JBuiltinType.shortT)
               || other.equals(JBuiltinType.intT) 
               || other.equals(JBuiltinType.longT)
               || other.equals(JBuiltinType.floatT);
     }
     return false;
  }

  /**
   * Returns true if type is void.
   */

  public boolean isVoid()
  {
     return this.equals(JTypes.voidT);
  }

  /** 
   *  Get the type kind. Useful for languages that defines several
   *  type kinds (e.g. in Java boolean, arithmetic and references).
   *  @return an integer indicating the kind of the type. 
   */

  public Integer getTypeKind() 
  {
     if ( this == JTypes.booleanT ) 
     {
        // boolean kind
        return JTypeKind.BOOLEAN_TYPE;
     }
     else if ( this != JTypes.voidT && this != (LType)JTypes._null )
     {
        // arithmetic kind
        return JTypeKind.ARITHMETIC_TYPE;
     }
     else if ( this == JTypes.voidT )
     {
        // void kind
        return JTypeKind.VOID_TYPE;
     }
     throw new Error("Unknown builtin type " + this);
  } 

  public static final JBuiltinType voidT    = new JBuiltinType("void");
  public static final JBuiltinType booleanT = new JBuiltinType("boolean");
  public static final JBuiltinType byteT    = new JBuiltinType("byte");
  public static final JBuiltinType charT    = new JBuiltinType("char");
  public static final JBuiltinType shortT   = new JBuiltinType("short");
  public static final JBuiltinType intT     = new JBuiltinType("int");
  public static final JBuiltinType longT    = new JBuiltinType("long");
  public static final JBuiltinType floatT   = new JBuiltinType("float");
  public static final JBuiltinType doubleT  = new JBuiltinType("double");

  /** Static initializer. Fill in the builtin types hashtable. */
  static
  {
     builtinTypes.put(voidT.getName(),    voidT);
     builtinTypes.put(booleanT.getName(), booleanT);
     builtinTypes.put(byteT.getName(),    byteT);
     builtinTypes.put(charT.getName(),    charT);
     builtinTypes.put(shortT.getName(),   shortT);
     builtinTypes.put(intT.getName(),     intT);
     builtinTypes.put(longT.getName(),    longT);
     builtinTypes.put(floatT.getName(),   floatT);
     builtinTypes.put(doubleT.getName(),  doubleT);
  }

}

// ***********************************************************************
// History Log:
// $Log: JBuiltinType.java,v $
// Revision 1.12  1998/02/27 12:38:43  cinard
// Updated copyright notice.
//
// Revision 1.11  1998/02/23 14:29:22  cinard
// True Proxy design pattern implemented for JClassProxy.
// Improved type compatibility checks (complex type checking mechanism
// has been replaced by assignable() methods in subclasses of LType).
// JTypeKind.java now defines type kind constants that were formerly
// in JTypeChecker.java (update your client code!).
//
// Revision 1.10  1998/02/12 16:03:31  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.9  1998/01/22 11:42:31  cinard
// Fixed a bug in symbols equality checks.
//
// Revision 1.8  1998/01/12 13:19:20  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.7  1997/12/16 09:18:45  cinard
// Improved behavior to take void type into account.
//
// Revision 1.6  1997/12/08 12:53:58  cinard
// Added type check mechanism.
// Fixed some byte code import bugs.
//
// Revision 1.5  1997/11/26 17:13:11  cinard
// Changed package name.
//
// Revision 1.4  1997/11/12 13:37:04  cinard
// *** empty log message ***
//
// Revision 1.3  1997/11/04 16:04:27  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.2  1997/11/04 09:04:34  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symboltable.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.1  1997/11/03 12:53:34  cinard
// Replaced Java symbols hierarchy with languag independent symbols hierarchy.
// Made JSymbolTableManager inherits from abstract SymbolTableMgr.
//
// ***********************************************************************
