// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JClass.java,v $ 
// $Revision: 1.7 $ 
// $Date: 1998/03/18 16:04:57 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable
// NAME:        JClass.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Feb 17,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;

import java.util.Vector;
import java.util.Enumeration;

import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.UnaryPredicate;

/**
 * Java language class symbol.
 */

public final class JClass extends LClassImpl
{

  /**
   * Constructs a Java class.
   * @param fullName fully qualified name of the class.
   * @param modifier modifier bit mask
   */
   
  public JClass(String fullName,LModifier modifier)
  {
     super(fullName.substring(fullName.lastIndexOf(".")+1),
           modifier,SymbolId.CLASS);  

     _fullName = fullName;
  }

  /**
   * Constructs a Java class.
   * @param fullName fully qualified name of the class.
   * @param modifier modifier bit mask
   * @param symbolId symbol's identifier
   */
   
  public JClass(String fullName,LModifier modifier,long symbolId)
  {
     super(fullName.substring(fullName.lastIndexOf(".")+1),
           modifier,
           symbolId | SymbolId.CLASS);

     _fullName = fullName;
  }

  /**
   * Is this equals to supplied o.
   */

  public boolean equals(Object o)
  {
     if ( o instanceof JClassProxy )
     {
        return o.equals(this);
     }
     else
     {
        return super.equals(o);
     }
  }

  /**
   * Can a value of type t be assigned to a variable of this type.
   * @param  t The type for which the assignability is checked.
   * @return True if it can, false otherwise.
   */

  public boolean assignable(Symbol t)
  {
     if ( ! ( t instanceof LType ) ) { return false; }
     
     LType other = (LType)t;
     
     if ( JTypeKind.REFERENCE_TYPE.equals(other.getTypeKind()) )
     {
        if      ( this.equals(JTypes._null) )    { return false;  }
        else if ( this.equals(other) )           { return true;   }
        else if ( this.equals(JTypes._Object) )  { return true;   }
        else if ( other.equals(JTypes._null) )   { return true;   }
        else if ( ! this.equals(JTypes._Cloneable) )
        {
           return    ( other instanceof LClass )
                  && ((LClass)other).subclassOf(this) ;
        }
        else if ( this.equals(JTypes._Cloneable) )
        { 
           return    (    ( other instanceof LClass )
                       && ((LClass)other).subclassOf(this) )
                  || ( other instanceof LArray );
        }
     }
     return false;
  }

  /** 
   *  Get the type kind. Useful for languages that defines several
   *  type kinds (e.g. in Java boolean, arithmetic and references).
   *  @return an integer indicating the kind of the type. 
   */

  public Integer getTypeKind() 
  {
     return JTypeKind.REFERENCE_TYPE;
  }

  /**
   *  Return the class fully qualified name.
   */

  public String getFullName() 
  {
     return _fullName;
  }

  /** Fully qualified name for the class */

  private final String _fullName;

}

// ***********************************************************************
// History Log:
// $Log: JClass.java,v $
// Revision 1.7  1998/03/18 16:04:57  cinard
// Added fullName() method.
//
// Revision 1.6  1998/03/10 17:26:39  mallet
// Merged v1.1 with development trunk
//
// Revision 1.5.2.1  1998/03/06 19:26:50  cinard
// Added full class name.
//
// Revision 1.5  1998/02/27 12:38:43  cinard
// Updated copyright notice.
//
// Revision 1.4  1998/02/23 17:50:26  cinard
// Classe implementing Java versions of the symbol interfaces.
//
// ***********************************************************************
