// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JModifier.java,v $
// $Revision: 1.6 $
// $Date: 1998/02/27 12:38:44 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JModifier.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Nov 05,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;

/**
 *  Java modifiers.
 */

public final class JModifier extends LModifier
{

 /** Public modifier constant. */
 public static final int PUBLIC =	 0x0001;

 /** Private modifier constant. */
 public static final int PRIVATE =	 0x0002;

 /** Protected modifier constant. */
 public static final int PROTECTED =	 0x0004;

 /** Static modifier constant. */
 public static final int STATIC =	 0x0008;

 /** Final modifier constant. */
 public static final int FINAL =        0x0010;

 /** Synchronized modifier constant. */
 public static final int SYNCHRONIZED = 0x0020;

 /** Volatile modifier constant. */
 public static final int VOLATILE =     0x0040;

 /** Transient modifier constant. */
 public static final int TRANSIENT =    0x0080;

 /** Native modifier constant. */
 public static final int NATIVE =       0x0100;

 /** Interface modifier constant. */
 public static final int INTERFACE =    0x0200;

 /** Abstract modifier constant. */
 public static final int ABSTRACT =     0x0400;

 /** 
  * Constructs a modifier from an int bit mask.
  */
 public JModifier(int bitMask)
 {
    super(bitMask);
 }

 /**
  * Checks if the modifier flag indicates public.
  * @return	   true if it indicates public, false otherwise.
  */

 public boolean isPublic() 
 {
   return ((modifier & PUBLIC) != 0);
 }

 /**
  * Checks if the modifier flag indicates private.
  * @return	   true if it indicates private, false otherwise
  */

 public boolean isPrivate()
 {
   return ((modifier & PRIVATE) != 0);
 }

 /**
  * Checks if the modifier flag indicates protected.
  * @return	   true if it indicates protected, false otherwise
  */

 public boolean isProtected()
 {
   return ((modifier & PROTECTED) != 0);
 }

 /**
  * Checks if the modifier flag indicates static.
  * @return	   true if it indicates static, false otherwise
  */

 public boolean isStatic()
 {
    return ((modifier & STATIC) != 0);
 }

 /**
  * Returns true if the modifier flag indicates final, false otherwise.
  */
 public boolean isFinal()
 {
    return ((modifier & FINAL) != 0);
 }

 /**
  * Returns true if the modifier flag indicates synchronized, false otherwise.
  */
 public boolean isSynchronized()
 {
    return ((modifier & SYNCHRONIZED) != 0);
 }

 /**
  * Returns true if the modifier flag indicates volatile, false otherwise.
  */
 public boolean isVolatile()
 {
    return ((modifier & VOLATILE) != 0);
 }

 /**
  * Returns true if the modifier flag indicates transient, false otherwise.
  */
 public boolean isTransient()
 {
    return ((modifier & TRANSIENT) != 0);
 }

 /**
  * Returns true if the modifier flag indicates native, false otherwise.
  */
 public boolean isNative()
 {
    return ((modifier & NATIVE) != 0);
 }

 /**
  * Returns true if the modifier flag indicates interface, false otherwise.
  */
 public boolean isInterface()
 {
    return ((modifier & INTERFACE) != 0);
 }

 /**
  * Returns true if the modifier flag indicates abstract, false otherwise.
  */
 public boolean isAbstract()
 {
    return ((modifier & ABSTRACT) != 0);
 }

 /**
  * Returns a string describing the modifier flag.
  * @param modifier      modifier flag.
  */

 public String toString() 
 {

   String modifierStr = "";

   if (isStatic())         modifierStr += "static ";

   if      (isPublic())    modifierStr += "public ";
   else if (isPrivate())   modifierStr += "private ";
   else if (isProtected()) modifierStr += "protected ";

   if (isFinal())          modifierStr += "final ";
   if (isSynchronized())   modifierStr += "synchronized ";
   if (isVolatile())       modifierStr += "volatile ";
   if (isTransient())      modifierStr += "transient ";
   if (isNative())         modifierStr += "native ";
   if (isInterface())      modifierStr += "interface ";
   if (isAbstract())       modifierStr += "abstract ";

   return modifierStr;
 }

   /** Debug flag */
   private static boolean debug = false;
}


// ***********************************************************************
// History Log:
// $Log: JModifier.java,v $
// Revision 1.6  1998/02/27 12:38:44  cinard
// Updated copyright notice.
//
// Revision 1.5  1998/02/12 16:03:33  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.4  1998/01/12 13:19:21  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.3  1998/01/06 14:57:37  bareau
// Bug fix: added the "static" modifier in toString()
//
// Revision 1.2  1997/11/26 17:13:12  cinard
// Changed package name.
//
// Revision 1.1  1997/11/12 13:38:28  cinard
// Added classes to enable importing bytecode to populate the
// symbol table.
//
// ***********************************************************************
