// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JSymbolFactory.java,v $
// $Revision: 1.3 $
// $Date: 1998/06/10 15:29:06 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JSymbolFactory.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import ORG.opengroup.adl.symboltable.*;

/**
 * Concrete symbol factory for Java symbols.
 */

public class JSymbolFactory extends SymbolFactory
{


  /**
   * Construct and returns a class.
   * @param fullName full name of the class.
   * @param modifier modifier for the class
   */
   
  public LClass newClass(String fullName,int modifier)
  {
     LClass lClass = new JClass(fullName,new JModifier(modifier));
     LAttribute _this = newAttribute("this",JModifier.PRIVATE,lClass);
     lClass.putSymbol(_this);
     return lClass;
  }

  // -- Attributes common to all adl classes --

  private static LAttribute _adl_thrownException = null;
  private static LMethod    _ADL_short_range      = null;
  private static LMethod    _ADL_int_range        = null;
  private static LMethod    _ADL_long_range       = null;
 
  /**
    * Construct and returns the ADL class to supply semantics for the target 
    * language class provided.
    * @param lClass The target language class for which semantics is to be 
    * defined.
    * @param superClassName name of the adl super class.
    */

  public ADLClass newADLClass(LClass lClass,String superClassName) 
  {
     if ( _adl_thrownException == null )
     {
        _adl_thrownException = newAttribute
                               ("adl_thrownException",0x00,JTypes._Throwable);
        _ADL_short_range     = newMethod("ADL_short_range",
                                         JModifier.PUBLIC | JModifier.STATIC);
        _ADL_short_range.setReturnType(JTypes._Enumeration);
        _ADL_short_range.addParameter
                         (newParameter("_",0x00,JTypes.shortT));
        _ADL_short_range.addParameter
                         (newParameter("_",0x00,JTypes.shortT));
        _ADL_int_range     = newMethod("ADL_int_range",
                                       JModifier.PUBLIC | JModifier.STATIC);
        _ADL_int_range.setReturnType(JTypes._Enumeration);
        _ADL_int_range.addParameter
                         (newParameter("_",0x00,JTypes.intT));
        _ADL_int_range.addParameter
                         (newParameter("_",0x00,JTypes.intT));
        _ADL_long_range     = newMethod("ADL_long_range",
                                        JModifier.PUBLIC | JModifier.STATIC);
        _ADL_long_range.setReturnType(JTypes._Enumeration);
        _ADL_long_range.addParameter
                         (newParameter("_",0x00,JTypes.longT));
        _ADL_long_range.addParameter
                         (newParameter("_",0x00,JTypes.longT));

     }
     ADLClass adlClass = new ADLClass(lClass,superClassName);
     adlClass.putSymbol(_adl_thrownException);
     LAttribute _this = newAttribute("this",0x00,lClass);
     adlClass.putSymbol(_this);
     adlClass.putSymbol(_ADL_short_range);
     adlClass.putSymbol(_ADL_int_range);
     adlClass.putSymbol(_ADL_long_range);
     
     return  adlClass; 
  }

  // -- Methods common to all tdd classes --
  private static LMethod _tdd_tdd_result   = null;
  private static LMethod _tdd_tdd_assert   = null;
  private static LMethod _tdd_tdd_skip     = null;
  private static LMethod _tdd_tdd_end_case = null;
  private static LMethod _tdd_infoline     = null;

  // -- Attributes common to all tdd classes --
  private static LAttribute _ADL_PASS;
  private static LAttribute _ADL_FAIL;
  private static LAttribute _ADL_UNRESOLVED;
  private static LAttribute _ADL_NOTINUSE;
  private static LAttribute _ADL_UNSUPPORTED;
  private static LAttribute _ADL_UNTESTED;
  private static LAttribute _ADL_UNINITIATED;
  private static LAttribute _ADL_NORESULT;
  private static LAttribute _ADL_UNEVALUATED;
  private static LAttribute _ADL_AMBIGUOUS;
  private static LAttribute _ADL_UNDEFINED;

  /**
   * Construct and returns a TDD class symbol.
   * @param name name of the TDD class.
   */

  public TDDClass newTDDClass(String name) 
  {
     if ( _tdd_tdd_result== null )
     {
        _tdd_tdd_result = newMethod("tdd_result",JModifier.PUBLIC);
        _tdd_tdd_result.addParameter(newParameter("res",0x00,JTypes.intT));
        _tdd_tdd_result.addParameter(newParameter("msg",0x00,JTypes._String));
        _tdd_tdd_result.setReturnType(JTypes.booleanT);

        _tdd_tdd_assert = newMethod("tdd_assert",JModifier.PUBLIC);
        _tdd_tdd_assert.addParameter
                    (newParameter("assertText",0x00,JTypes._String));
        _tdd_tdd_assert.addParameter
                    (newParameter("assertRes",0x00,JTypes.booleanT));
        _tdd_tdd_assert.setReturnType(JTypes.booleanT);

        _tdd_tdd_skip = newMethod("tdd_skip",JModifier.PUBLIC);
        _tdd_tdd_skip.setReturnType(JTypes.booleanT);

        _tdd_tdd_end_case = newMethod("tdd_end_case",JModifier.PUBLIC);
        _tdd_tdd_end_case.addParameter(newParameter("msg",0x00,JTypes._String));
        _tdd_tdd_end_case.setReturnType(JTypes.booleanT);

        _tdd_infoline = newMethod("infoline",JModifier.PUBLIC);
        _tdd_infoline.addParameter(newParameter("info",0x00,JTypes._String));
        _tdd_infoline.setReturnType(JTypes.voidT);

        _ADL_PASS        = newAttribute("ADL_PASS",0x00,JTypes.intT);
        _ADL_FAIL        = newAttribute("ADL_FAIL",0x00,JTypes.intT);
        _ADL_UNRESOLVED  = newAttribute("ADL_UNRESOLVED",0x00,JTypes.intT);
        _ADL_NOTINUSE    = newAttribute("ADL_NOTINUSE",0x00,JTypes.intT);
        _ADL_UNSUPPORTED = newAttribute("ADL_UNSUPPORTED",0x00,JTypes.intT);
        _ADL_UNTESTED    = newAttribute("ADL_UNTESTED",0x00,JTypes.intT);
        _ADL_UNINITIATED = newAttribute("ADL_UNINITIATED",0x00,JTypes.intT);
        _ADL_NORESULT    = newAttribute("ADL_NORESULT",0x00,JTypes.intT);
        _ADL_UNEVALUATED = newAttribute("ADL_UNEVALUATED",0x00,JTypes.intT);
        _ADL_AMBIGUOUS   = newAttribute("ADL_AMBIGUOUS",0x00,JTypes.intT);
        _ADL_UNDEFINED   = newAttribute("ADL_UNDEFINED",0x00,JTypes.intT);

     }

     TDDClass tddClass = super.newTDDClass(name);
     tddClass.putSymbol(_tdd_tdd_result);
     _tdd_tdd_result.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_assert);
     _tdd_tdd_assert.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_skip);
     _tdd_tdd_skip.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_tdd_end_case);
     _tdd_tdd_end_case.setIsRuntimeMethod();
     tddClass.putSymbol(_tdd_infoline);
     _tdd_infoline.setIsRuntimeMethod();

     tddClass.putSymbol(_ADL_PASS);
     tddClass.putSymbol(_ADL_FAIL);
     tddClass.putSymbol(_ADL_UNRESOLVED);
     tddClass.putSymbol(_ADL_NOTINUSE);
     tddClass.putSymbol(_ADL_UNSUPPORTED);
     tddClass.putSymbol(_ADL_UNTESTED);
     tddClass.putSymbol(_ADL_UNINITIATED);
     tddClass.putSymbol(_ADL_NORESULT);
     tddClass.putSymbol(_ADL_UNEVALUATED);
     tddClass.putSymbol(_ADL_AMBIGUOUS);
     tddClass.putSymbol(_ADL_UNDEFINED);

     return tddClass;
  }


  // TDD Factory builtin methods
  // Initialized when newTDDFactory()is called for the first time.
  
  static LMethod _tdd_end_case_1 = null;
  static LMethod _tdd_end_case_2 = null;
  static LMethod _tdd_skip       = null;
  static LMethod _tdd_end_test   = null;
  static LMethod _tdd_result     = null; 

  /**
   * Construct and returns a TDD factory.
   * @param name Name of the factory.
   */

  public TDDFactory newTDDFactory(String name)
  {
     if ( _tdd_end_case_1 == null )
     {
        // Initialize tdd factory builtin methods
        _tdd_end_case_1 = newMethod("tdd_end_case",JModifier.PUBLIC);
        _tdd_end_case_1.setReturnType(JTypes.booleanT);
        _tdd_end_case_1.addException(JTypes._Throwable);
        
        _tdd_end_case_2 = newMethod("tdd_end_case",JModifier.PUBLIC);
        _tdd_end_case_2.addParameter
        (newParameter("msg",0x00,JTypes._String));
        _tdd_end_case_2.setReturnType(JTypes.booleanT);
        _tdd_end_case_2.addException(JTypes._Throwable);
  
        _tdd_skip = newMethod("tdd_skip",JModifier.PUBLIC);
        _tdd_skip.setReturnType(JTypes.booleanT);
        _tdd_skip.addException(JTypes._Throwable);
     
        _tdd_end_test = newMethod("tdd_end_test",JModifier.PUBLIC);
        _tdd_end_test.setReturnType(JTypes.booleanT);
        _tdd_end_test.addException(JTypes._Throwable);

        _tdd_result = newMethod("tdd_result",JModifier.PUBLIC);
        _tdd_result.addParameter(newParameter("code",0x00,JTypes.intT));
        _tdd_result.addParameter(newParameter("msg",0x00,JTypes._String));
        _tdd_result.setReturnType(JTypes.booleanT);
        _tdd_result.addException(JTypes._Throwable);

     }
     
     TDDFactory factory = super.newTDDFactory(name);
     factory.putSymbol(_tdd_end_case_1);
     factory.putSymbol(_tdd_end_case_2);
     factory.putSymbol(_tdd_skip);
     factory.putSymbol(_tdd_end_test);
     factory.putSymbol(_tdd_result);
     return factory;
  }

  /** 
   * Construct and returns  an array type.
   * @param elememtType type of the elements in the array.
   */

  public LType newArray(LType elementType)
  {
     LObjectArray lObjectArray = new JArray(elementType);
     lObjectArray.addSuperClass(JTypes._Cloneable);
     lObjectArray.putSymbol
     (newAttribute("length",JModifier.PUBLIC,JTypes.intT));
     return lObjectArray;
  }


  /**
   * Creates a new LModifier with all the bits in modifier set.
   * @param modifier the modifier.
   */

  public LModifier newModifier(int modifier)
  {
     return new JModifier(modifier);
  }


  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: JSymbolFactory.java,v $
// Revision 1.3  1998/06/10 15:29:06  bareau
// Moved "isTestMethod" from LCallable to LMethod (and fixed it: runtime
// methods like infoline, tdd_assert... are _not_ test methods).
// Added an "isRuntimeMethod" flag in LMethod, and set it in symbol
// factories for runtime methods.
//
// Revision 1.2  1998/03/25 14:47:18  cinard
// Updated newADLClass() method (superclass name shall be supplied now).
//
// Revision 1.1  1998/03/18 15:51:30  cinard
// Symbol factory in now separated from symbol table.
//
// ***********************************************************************
