// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JSymbolTableMgr.java,v $
// $Revision: 1.33 $
// $Date: 1998/07/28 13:45:29 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JSymbolTableMgr.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Date;

import com.objectspace.jgl.UnaryPredicate;
import com.objectspace.jgl.predicates.InstanceOf;
import com.objectspace.jgl.HashSet;
import com.objectspace.jgl.algorithms.Copying;
import com.objectspace.jgl.adapters.ObjectArray;
import com.objectspace.jgl.predicates.UnaryAnd;

import ORG.opengroup.adl.symboltable.*;

/**
 * Manages the symbol table and scopes within a given compilation unit.
 */

public final class JSymbolTableMgr extends SymbolTableMgr
{

  /**
   * Constructs the java symbols table manager.
   */
  
  public JSymbolTableMgr()
  {
     super();

     // Perform some initialization

     jSymbolLoader = new JSymbolLoader(this);

     // add java.lang in import on demand

     try
     {
        addImportOnDemand("java.lang");
     }
     catch (PackageNotAccessibleException e)
     {
        throw new Error("Caught : " + e.toString());
     }

     // Import useful classes.
     JTypes._Object           = getClass("java.lang.Object");
     JTypes._Cloneable        = getClass("java.lang.Cloneable");
     JTypes._Throwable        = getClass("java.lang.Throwable");
     JTypes._RuntimeException = getClass("java.lang.RuntimeException");
     JTypes._String           = getClass("java.lang.String");
     JTypes._Enumeration      = getClass("java.util.Enumeration");
     JTypes._null             = _sbf.newClass("null",0x0000);

  }

  // -- Public Methods --

  /**
   * Reinitialize the symbol table. Should be used when a new compilation
   * unit is parsed.
   */

  public void reInit()
  {
     super.reInit();

     /** Method listeners. */
     methodListenerInfos = new Vector();

     /** Classes from unnamed package. */
     classesFromUnamedPackage = new HashSet();

     /** Re-init current scope. */
     currentScope = unitScope;

     try
     {
        importOnDemands = new Vector();
        singleImports   = new Vector();
        addImportOnDemand("java.lang");
     }
     catch (PackageNotAccessibleException e)
     {
        throw new Error("Caught : " + e.toString());
     }
  }

  /**
   * Reinitialize the symbol table, and set symbol table context.
   * Should be used when a new compilation unit is type checked.
   * @param stmContext context information obtained from a former parsing.
   */

  public void setContext(Object stmContext)
  {
     reInit();

     super.setContext(stmContext);

     if ( stmContext instanceof JADLCompilationUnitInfo )
     {
        JADLCompilationUnitInfo adlInfo = (JADLCompilationUnitInfo)stmContext;
        importOnDemands = adlInfo.getImportOnDemands();
        singleImports = adlInfo.getSingleImports();
        classesFromUnamedPackage = adlInfo.getClassesFromUnamedPackage();
        
     }
  }

  /**
   * Get opaque context information that can be used later to manipulates 
   * symbols for current compilation unit.
   */

  public Object getContext()
  {
     return new JADLCompilationUnitInfo(importOnDemands,
                                        singleImports,
					getUsedADLClasses(),
                                        getUsedTDDClasses(),
                                        classesFromUnamedPackage);
  }

  /**
   * Import a package for import on demand queries.
   * @param name Package scoped name.
   * @exception ORG.opengroup.adl.symboltable.j.PackageNotAccessibleException
   *            in case package cannot be reached
   */

  public void addImportOnDemand(String name)
  throws PackageNotAccessibleException
  {
     if ( debug )
     {
        System.out.println
        ("debug: JSymbolTableMgr.addImportOnDemand(" + name + ")");
     }
          
     LPackage pack = jSymbolLoader.importPackage(name);
     if (pack != null)
     {
        importOnDemands.addElement(pack);
     }
     else
     {
        throw new PackageNotAccessibleException
                  ("package " + name + " not found.");
     }
     if (debug) 
     {
        System.out.println
        ("debug: JSymbolTableMgr.addImportOnDemand(" + name + ")"
        + " package = " + ( pack != null ? pack.getName() : null ) 
        + ". Return.");
     }
   }

  /**
   * Add a single import clause for a class.
   * @param name class scoped name.
   * @exception ORG.opengroup.adl.symboltable.j.ClassNotAccessibleException
   *            if class is not accessible. 
   */

  public void addSingleImport(String name)
  throws ClassNotAccessibleException
  {
     // Class is ready to be used.
     if ( jSymbolLoader.canImportClass(name) )
     {
        singleImports.addElement(name);
     }
     else
     { 
        throw new ClassNotAccessibleException
                  ("Class " + name + " not found.");
     }
  }

  /**
   * Put a symbol in the symbol table, extended version.
   * The symbol is put in the supplied scope or in current scope in case no 
   * scope was supplied.
   * In case we are in the scope of a symbol with the same name, and for
   * which there is no overriding possible, the SymbolAlreadyDefinedException
   * is throwed.
   * @param  symbol        The symbol to put into the scope.
   * @param  scopeOverride optionnal scope to take into account instead
   *                       of current scope.
   * @return               the symbol that was actually found or put in 
   *                       the symbol table.
   * @exception ORG.opengroup.adl.symboltable.SymbolAlreadyDefinedException 
   *                       in case symbol is already defined.
   */

  public Symbol putSymbol(Symbol symbol,Scope scopeOverride)
  throws SymbolAlreadyDefinedException
  {
    if ( debug ) System.out.println
		 ("debug : JSymbolTableMgr.putSymbol(" 
                  + symbol + "," + scopeOverride + ")");
     if ( scopeOverride == null ) { scopeOverride = getCurrentScope(); }
  
     // Check if a symbol with the same declaration exists in the 
     // scope

     String         name       = symbol.getName();

     UnaryPredicate comparator = null;
     Symbol existingSymbol     = null;
     if ( symbol instanceof LExpression )
     {
        // For local variable, parameters, binding there should be no 
        // expression with the same name in scope.
        comparator = SymbolSelector.EXPRESSION_SELECT;
        Scope searchScope = scopeOverride;
        while ( existingSymbol == null && searchScope != null )
        {
           existingSymbol = searchScope.getSymbol
                            (symbol.getName(),comparator,Boolean.TRUE);
           searchScope = searchScope.getParent();
        }
     }
     else
     {
        comparator     = symbol.getComparator();
        existingSymbol = scopeOverride.getSymbol
                         (symbol.getName(),comparator,Boolean.TRUE);
     }


     if ( existingSymbol != null )
     {
        // It is defined, throw an exception.
        throw  new SymbolAlreadyDefinedException
                   (symbol.toString() + " already defined");
     }
     else
     {
        // Symbol is not yet declared, put it in the scope and return it.

        //Test if symbol is LExpression and save related scope in LExpression
        if ( symbol instanceof LExpression ) 
        {
           ((LExpression)symbol).setRelated(scopeOverride);
        }

        scopeOverride.putSymbol(symbol);
	if ( debug ) System.out.println
		     ("debug : JSymbolTableMgr.putSymbol(" 
                      + symbol + "," + scopeOverride + ") return " + symbol);
        
        return symbol;
     }
  }


  /**
   * Get a symbol, extended version.
   * Should be called to search for names that were classified ambiguous.
   * The symbol table manager tries to classify the supplied name and to 
   * return an adhoc symbol for it.
   * <pre>
   * Scope override shall be supplied to override the default behaviour,
   * which is to search in the current scope. This enable incremental
   * search for complex expressions like foo.bar[].boo.baa().zip.
   * </pre>
   * <p> <b> Algorithm: </b> <ul>
   * <li> described in the ADL Translator Design Document.
   * </ul>
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate     An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param symbolListener Reference to the symbol listener that shall
   *                       be called back later in case the symbol
   *                       is not found during the call (may be null).
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public Symbol getSymbol
                (
                   String scopedName, 
                   UnaryPredicate predicate,
                   SymbolListener symbolListener,
                   Scope scopeOverride,
                   boolean localSearch
                )
  {
     throw new Error("JSymbolTableMgr.getSymbol() should not be called!");
  }

  /**
   * Get a package symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public LPackage getPackage(String scopedName,
                             Scope scopeOverride,
                             boolean localSearch)
  {
     // TBD - localSearch flag and scopeOverride are ignored.

     if (debug) 
     { 
        System.out.println
        ("debug: JSymbolTableMgr.getPackage(" + scopedName + ")");
     }

     // First check if this package has not been loaded in symbol table.

     LPackage pack = (LPackage)unitScope.getSymbol
                               (scopedName,
                                SymbolSelector.PACKAGE_SELECT,
                                null);

     // If it was not found, check if a package with that name exist
     // and can be accessed.

     if ( pack == null )
     {
        pack = jSymbolLoader.getPackage(scopedName);
     }

     if (debug) 
     { 
        System.out.println
        ("debug: JSymbolTableMgr.getPackage(" + scopedName + ")"
         + " return " + pack);
     }

     return pack;

  }

  /**
   * Get a class symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public LClass getClass(String name, Scope scopeOverride, boolean localSearch)
  {
     // TBD - localSearch flag is ignored and scopeOverride too.

     if (debug) 
     { 
        System.out.println("debug: JSymbolTableMgr.getClass(" + name + ")");
     }

     LClass theClass = null;

     QualifiedName qualifiedName = new QualifiedName(name);

     // Processing depends on name being fully qualified or not.
     
     if ( ! qualifiedName.isSimple() )
     {
        // Fully qualified name. First locate package.

        String packName  = qualifiedName.getPrefix();
        String className = qualifiedName.getSuffix();

        // First get package then get class
        LModule pack = getPackage(packName,null,false);

        if ( pack != null )
        {
           theClass = (LClass)pack.getSymbol
                              (className,
                               SymbolSelector.CLASS_SELECT,
                               null);
        }

        // Try to import symbol if needed
        if ( theClass == null )
        {
           // import it
           theClass = jSymbolLoader.getClass(name);
        }
     }
     else
     {
        // Simple name. Search in unnamed package, single imports
        // and import on demands.

        theClass = (LClass)unitScope.getSymbol
                                     (name,
                                      SymbolSelector.CLASS_SELECT,
                                      null);

        // Try to import symbol if needed

        if ( theClass == null )
        {
           // import it
           theClass = jSymbolLoader.getClass(name);
        }

        // Update set of classes from unamed package.
        if (    ( theClass != null )
             && ( theClass.getFullName().indexOf(".") == -1 ) )        
        {
           classesFromUnamedPackage.add(name);
        }

        if ( theClass == null ) 
        {
           // Search Single type import

           for (int i = 0; i < singleImports.size(); i++)
           {
              if (    singleImports.elementAt(i).equals(name)
                   || ((String)singleImports.elementAt(i)).endsWith("." + name)
                 )
              {
                 String scopedName = (String)singleImports.elementAt(i);

                 QualifiedName qName = new QualifiedName(scopedName);
                 
                 // First get package then get class
                 if ( ! qName.isSimple() )
                 {
                    LModule pack = getPackage(qName.getPrefix(), null, false);
                    if ( pack != null )
                    {
                       theClass = (LClass)pack.getSymbol
                                  (name,
                                   SymbolSelector.CLASS_SELECT,
                                   null);
                    }
                 }
                 else
                 {
                    theClass = (LClass)unitScope.getSymbol
                               (name,
                                SymbolSelector.CLASS_SELECT,
                                null);
                 }
                 if ( theClass == null )
                 {
                    // import it
                    theClass = jSymbolLoader.getClass(scopedName);
                 }
              }
              if ( theClass != null ) { break; }
           }
        }
        
        // Check it is not in a type import on demand (already loaded)

        if (theClass == null)
        {
           for (int i = 0; i < importOnDemands.size(); i++)
           {
              LPackage pack = (LPackage)(importOnDemands.elementAt(i));
              theClass = (LClass)pack.getSymbol
                                      (name,
                                       SymbolSelector.CLASS_SELECT,
                                       null);

              if ( theClass != null ) { break; }
           }
        }
   
        // Check it is not in a type import on demand (not loaded)

        if (theClass == null)
        {
           for (int i = 0; i < importOnDemands.size(); i++)
           {
              LPackage pack      = (LPackage)(importOnDemands.elementAt(i));
              String   className = pack.getName() + "." + name;
              theClass = jSymbolLoader.getClass(className);
              
              if ( theClass != null ) { break; }
           }
        }
     }

     if (debug) 
     { 
        System.out.println("debug: JSymbolTableMgr.getClass(" + name + ")"
                           + " return " 
                           + theClass);
     }

     // Return symbol
     return theClass;
  }

  /**
   * Get a type symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

   public LType getType(String scopedName,
                        Scope scopeOverride,
                        boolean localSearch)
   {

      if ( debug ) 
      {
         System.out.println("debug: JSymbolTableMgr.getType("
                            + scopedName + ", " + localSearch + ")");
      }

      // TBD - Ignored scopeOverride and localSearch.

      // First search in builtin types.
      LType ret =  JBuiltinType.getBuiltinType(scopedName);

      // If necessary search for a class.
      if ( ret == null )
      {
         ret = getClass(scopedName);
      }

      if ( debug ) 
      {
         System.out.println("debug: JSymbolTableMgr.getType("
                            + scopedName + ", " + localSearch + ")"
                            + " return " + ret);
      }
      
      return ret;
   }

  /**
   * Get an ADL class symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @return The found symbol.
   */

   public ADLClass getADLClass(String scopedName,Scope scopeOverride)
   {
      if ( debug ) 
      {
         System.out.println("debug: JSymbolTableMgr.getADLClass("
                            +scopedName+")");
      }
      ADLClass ret = 
             (ADLClass) getUnitScope().getSymbol
                                       (scopedName,
                                        SymbolSelector.ADL_CLASS_SELECT,
                                        null);
      if ( debug ) 
      {
         System.out.println("debug: JSymbolTableMgr.getADLClass("
                            +scopedName+")" + " return " + ret);
      }

      return ret;
   }

  /**
   * Get a TDD class symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @return The found symbol.
   */

  public TDDClass getTDDClass(String scopedName)
  {
     if ( debug ) 
     {
        System.out.println
        ("debug: JSymbolTableMgr.getTDDClass(" +scopedName+")");
     }

     TDDClass ret = 
              (TDDClass)getUnitScope().getSymbol
                                       (scopedName,
                                        SymbolSelector.TDD_CLASS_SELECT,
                                        null);
     if ( debug ) 
     {
        System.out.println
        ("debug: JSymbolTableMgr.getTDDClass(" +scopedName+")"
        + " return "  + ret);
     }
     
     return ret;

  }

  /**
   * Get an expression symbol (local variable, field, parameter).
   * Should be called to search for names that were classified as expressions.
   * @param scopedName   The scoped name of the expected symbol.
   * @param scopeOverride  Scope in which the research shall take place,
   *                       if null is supplied current scope is assumed.
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public LExpression getExpression(String scopedName,
                                   Scope scopeOverride,
                                   boolean localSearch)
  {  
     LExpression expression = null;

     if ( debug )
     {
        System.out.println("debug: JSymbolTableMgr.getExpression("
                           + scopedName + ", "
                           + "scopeOverride, "
                           + localSearch + ")");
     }

     // Set search scope to current scope in case no scope was supplied

     if ( scopeOverride == null ) { scopeOverride = currentScope; }

     QualifiedName qualifiedName = new QualifiedName(scopedName);

     if ( qualifiedName.isSimple() )
     {
        // Simple name - browse scope
        
        if ( ! ( scopeOverride instanceof LModule ) )
        {
           while ( ( expression == null ) && ( scopeOverride != null ) )
           {
              
              expression = (LExpression)scopeOverride.getSymbol
                           (scopedName,
                            SymbolSelector.EXPRESSION_SELECT,
                            null);
           
              if (      expression != null 
                   && ! canGetSymbol(expression,localSearch) )
              {
                 expression = null;
              }

              // In case it's a local search - exit loop.
              if ( localSearch )
              {
                 break;
              }

              // If leaving (J,ADL,TDD)class scope - exit loop.
              if (    scopeOverride instanceof LClass
                   || scopeOverride instanceof ADLClass
                   || scopeOverride instanceof TDDClass )
              {
                 break;
              }
              
              scopeOverride = scopeOverride.getParent();
           }  
        }
     }
     else
     {
        // Fully qualified name
        // Prefix of qualified name may be another expression or a class.
        String prefixName = qualifiedName.getPrefix();

        // First try to get an expression.
        LExpression parentExpr = getExpression
                                 (prefixName,scopeOverride,localSearch);

        if ( parentExpr != null )
        {
           // local search in parent expression scope
           if ( parentExpr.hasScopeAccess() )
           {
              Scope parentScope = parentExpr.getScope();
              expression = (LExpression)parentScope.getSymbol
                           (qualifiedName.getSuffix(),
                            SymbolSelector.EXPRESSION_SELECT,
                            null);
           }
        }

        if ( expression != null && ! ( canGetSymbol(expression,true) ) )
        {
           expression = null;
        }
        
        // If necessary, try to get another class.

        LClass  parentClass = getClass(prefixName);

        if ( parentClass != null )
        {
           // local search in parent class scope
           expression = (LExpression)parentClass.getSymbol
                        (qualifiedName.getSuffix(),
                         SymbolSelector.EXPRESSION_SELECT,
                         null);
        }

        if ( expression != null && ! ( canGetSymbol(expression,true) ) )
        {
           expression = null;
        }

     }

     if ( debug )
     {
        System.out.println("debug: JSymbolTableMgr.getExpression("
                           + scopedName + ", "
                           + "scopeOverride, "
                           + localSearch + ")" + " return " + expression);
     }

     return expression;

  }

  /**
   * Get a function or method.
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate     An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param symbolListener Reference to the symbol listener that shall
   *                       be called back later in case the symbol
   *                       is not found during the call (may be null).
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public LCallable getMethod
                   (
                    String scopedName,
                    UnaryPredicate predicate,
                    SymbolListener symbolListener,
                    Scope scopeOverride,
                    boolean localSearch
                   )
  {
     LCallable method = null;

     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getMethod("
                          + scopedName + ", " + predicate + ", "
                          + symbolListener + ", "
                          + scopeOverride + ", " 
                          + localSearch + ")");
     }

     // Update predicate to select only java mehods:
     // add a condition to the operation predicate supplied : symbol
     // shall verifies the LMethodComparator

     if ( predicate instanceof SymbolSelector )
     {
       SymbolSelector selector = (SymbolSelector)predicate;
       long id = selector.getSymbolIdMask();
       id &= ( ~ SymbolId.ADL_METHOD);
       ((SymbolSelector)predicate).setSymbolIdMask(id);
     }

     // Set search scope to current scope in case no scope was supplied

     if ( scopeOverride == null ) { scopeOverride = currentScope; }

     QualifiedName qualifiedName = new QualifiedName(scopedName);

     if ( qualifiedName.isSimple()  )
     {
        // Simple name - browse scope

        if ( ! ( scopeOverride instanceof LModule ) )
        {
           while ( ( method == null ) && ( scopeOverride != null ) )
           {
              method = (LCallable)scopeOverride.getSymbol
                                  (scopedName,predicate,null);
              if ( method != null && ! canGetSymbol(method,localSearch) )
              {
                 method = null;
              }

              // In case it's a local search - exit loop.
              if ( localSearch )
              {
                 break;
              }

              // If leaving (J,ADL,TDD)class scope - exit loop.
              if (    scopeOverride instanceof LClass
                   || scopeOverride instanceof ADLClass
                   || scopeOverride instanceof TDDClass )
              {
                 break;
              }

              scopeOverride = scopeOverride.getParent();
           }  
        }
     }
     else
     {
        // Fully qualified name
        // Prefix of qualified name may be another expression or a class.
        String prefixName = qualifiedName.getPrefix();

        // First try to get an expression.
        LExpression parentExpr = getExpression
                                 (prefixName,scopeOverride,localSearch);

        if ( parentExpr != null )
        {
           // local search in parent expression scope
           if ( parentExpr.hasScopeAccess() )
           {
              Scope parentScope = parentExpr.getScope();
              method = (LCallable)parentScope.getSymbol
                                              (qualifiedName.getSuffix(),
                                               predicate,
                                               null);
           }
        }

        if ( method != null && ! ( canGetSymbol(method,true) ) )
        {
           method = null;
        }
        
        // If necessary, try to get another class.

        LClass  parentClass = getClass(prefixName);

        if ( parentClass != null )
        {
           // local search in parent class scope
           method = (LCallable)parentClass.getSymbol(qualifiedName.getSuffix(),
                                                     predicate,
                                                     null);
        }

        if ( method != null && ! ( canGetSymbol(method,true) ) )
        {
           method = null;
        }

     }

     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getMethod("
                          + scopedName + ", " + predicate + ", "
                          + symbolListener + ", "
                          + scopeOverride + ", " 
                          + localSearch + ")" + " return " + method);
     }

     return method;
  }

  /**
   * Get an ADL method symbol. 
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate    An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param localSearch  should be true in case the search has to
   *                     take place in current scope only.
   * @return The found symbol.
   */

  public LCallable getADLMethod(String scopedName,
                                UnaryPredicate predicate,
                                Scope scopeOverride,
                                boolean localSearch)
  {
     ADLMethod adlMethod = null;

     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getADLMethod("
                          + scopedName + ", " + predicate + ", "
                          + ", scopeOverride, " 
                          + localSearch + ")");
     }

     // Locate ADL Class.

     Scope current = ( scopeOverride != null ) 
                   ? scopeOverride
                   : getCurrentScope();

     while (  current != null && ! (current instanceof ADLClass) )
     {
        current = current.getParent();
     }

     // If ADL Class is found, search adl method.

     if ( current != null )
     {
        ADLClass adlClass = (ADLClass)current;

        // add a condition to the operation predicate supplied : symbol
        // shall verifies the ADLMethodComparator

        if ( predicate instanceof SymbolSelector )
        {
           ((SymbolSelector)predicate).setSymbolIdMask(SymbolId.ADL_METHOD);
           adlMethod = (ADLMethod)adlClass.getSymbol
                                  (scopedName, predicate, null);
        }
     }

     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getADLMethod("
                          + scopedName + ", " + predicate + ", "
                          + ", scopeOverride, " 
                          + localSearch + ")" + " return " + adlMethod);
     }

     
     return adlMethod;
  }

  /**
   * Get a TDD factory symbol.
   * @param scopedName   The scoped name of the expected symbol.
   * @param predicate    An arbitrary predicate object defining some
   *                     criteria that the searched symbol shall
   *                     satisfy (may be null).
   * @param localSearch    should be true in case the search has to
   *                       take place in current scope only.
   * @return The found symbol.
   */

  public TDDFactory getTDDFactory
                    (
                     String scopedName,
                     UnaryPredicate predicate,
                     Scope scopeOverride,
                     boolean localSearch 
                    )
  {
     TDDFactory tddFactory = null;

     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getTDDFactory("
                          + scopedName + ", " + predicate + ", "
                          + ", scopeOverride, " 
                          + localSearch + ")");
     }


     // Locate TDD Class.

     Scope current = ( scopeOverride != null ) 
                   ? scopeOverride
                   : getCurrentScope();

     while (  current != null && ! (current instanceof TDDClass) )
     {
        current = current.getParent();
     }

     // If TDD Class is found, search tdd factory

     if ( current instanceof TDDClass )
     {
        TDDClass tddClass = (TDDClass)current;

        // add a condition to the operation predicate supplied : symbol
        // shall verifies the TDDFactoryComparator

        if ( predicate instanceof SymbolSelector )
        {
           ((SymbolSelector)predicate).setSymbolIdMask(SymbolId.TDD_FACTORY);
           tddFactory = (TDDFactory)tddClass.getSymbol
                                    (scopedName, predicate, null);
        }
     }
     
     if ( debug ) 
     {
        System.out.println("debug: JSymbolTableMgr.getTDDFactory("
                          + scopedName + ", " + predicate + ", "
                          + ", scopeOverride, " 
                          + localSearch + ")" + " return " + tddFactory);
     }

     return tddFactory;

  }

  
  // -- Check methods --
 
  /**
   * Checks if a builtin type name is valid or not.
   * @param name The name to check.
   * @return true if it is valid, false otherwise.
   */

  public boolean isValidBuiltinTypeName(String name)
  {
     return JBuiltinType.isValidBuiltinTypeName(name);
  }

  /**
   * Get a builtin type symbol.
   * @param name   The name of the builtin type.
   * @return The found symbol.
   * @exception ORG.opengroup.adl.symboltable.BadSymbolNameException
   *            in case supplied name is not that of a builtin type.
   */

  public LType getBuiltinType(String name)
  throws BadSymbolNameException
  {
     LType type = JBuiltinType.getBuiltinType(name);

     if ( type != null )
     {
        return type;
     }
     else
     {
        throw new BadSymbolNameException
                  ("Bad builtin type name : " + name);
      
     }
  }

  /** Return a factory for symbols. */

  public SymbolFactory getSymbolFactory() 
  {
     return _sbf;
  }


  // -- Private --

  /** 
   * Can symbol be retrieved ? This is true if search is not
   * local, or search is local and symbol is not private.
   * @param symbol symbol to check.
   * @param local search flag.
   */
  private boolean canGetSymbol(Symbol symbol, boolean localSearch)
  {
     if ( localSearch )
     {
        LModifier modif = symbol.getModifier();
        if ( modif instanceof JModifier )
        {
           return ! ((JModifier)modif).isPrivate();
        }
     }
     return true;
  }



  /**
   * Get names of classes from unamed package.
   */
  public String[] getClassesFromUnamedPackage()
  {
     String[] ret = new String[classesFromUnamedPackage.size()];
     Copying.copy(classesFromUnamedPackage,(new ObjectArray(ret)).begin());
     return ret;
  }


  // -- Private fields --

  private Vector methodListenerInfos = new Vector();

  /** Symbol loader. */
  private JSymbolLoader jSymbolLoader;

  /** Import on demands. */
  Vector importOnDemands = new Vector();

  /** Single imports. */
  Vector singleImports = new Vector();

  /** Classes from unnamed package. */
  HashSet classesFromUnamedPackage = new HashSet();

  /** Symbol factory */
  private static final JSymbolFactory _sbf = new JSymbolFactory();

  /** Debug flag. */
  private static final boolean debug = false;

}

// ***********************************************************************
// History Log:
// $Log: JSymbolTableMgr.java,v $
// Revision 1.33  1998/07/28 13:45:29  mallet
// Added list of used ADL classes to parameters of *UnitInfo constructor.
//
// Revision 1.32  1998/06/16 14:17:59  cinard
// Fixed bug in getMethod() : only adl method are discarded, other
// LCallable symbols are now returned.
//
// Revision 1.31  1998/06/15 11:53:57  cinard
// Improved getMethod() so that only LMethod symbols can be returned.
//
// Revision 1.30  1998/03/19 09:17:57  cinard
// Added unamed packages to stm context.
//
// Revision 1.29  1998/03/18 16:12:11  cinard
// Implemented support for symbol table context (mainly
// used to keep context information when parsing and
// typecheck are separated).
// Modified getSymbol() and likes methods (the complex
// and generic algorithm that was used is replaced
// by a much simpler one - each get...() method
// doing its part of the job).
//
// Revision 1.28  1998/03/10 17:26:40  mallet
// Merged v1.1 with development trunk
//
// Revision 1.27.2.4  1998/03/06 19:27:59  cinard
// Fixed a bug in classesFromUnamedPackage update mechanism.
//
// Revision 1.27.2.3  1998/03/06 16:21:39  cinard
// Fixed bug in getClassesFromUnamedPackage() : classes cached from
// earlier compilation are also taken into account.
//
// Revision 1.27.2.2  1998/03/06 11:29:13  cinard
// Added getClassesFromUnamedPackage() method.
//
// Revision 1.27.2.1  1998/03/05 14:35:20  cinard
// Added builtin tdd symbol for method void infoline(String).
//
// Revision 1.27  1998/02/27 12:36:56  cinard
// Made "this" a private attribut of JClass so that it is not visible from
// outside the class definition.
// Updated copyright notice.
//
// Revision 1.26  1998/02/25 11:05:44  cinard
// Introduced a new flag (localSearch) in getSymbol() and likes methods. A local
// search is restricted to the current (or supplied) scope, that is if the
// searched symbol is not found in current or supplied scope, the research
// does not continue in the parent scopes.
//
// Revision 1.25  1998/02/23 14:29:24  cinard
// True Proxy design pattern implemented for JClassProxy.
// Improved type compatibility checks (complex type checking mechanism
// has been replaced by assignable() methods in subclasses of LType).
// JTypeKind.java now defines type kind constants that were formerly
// in JTypeChecker.java (update your client code!).
//
// Revision 1.24  1998/02/12 16:03:34  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.23  1998/02/10 12:52:29  cinard
// Reinitialize method added so that the symbol table can be reinitialized
// (useful when new compilation unit are parsed).
//
// Revision 1.22  1998/01/22 15:05:12  cinard
// Fixed a bug in ADL_long_range, ADL_short_range and ADL_int_range signatures.
//
// Revision 1.21  1998/01/22 11:37:20  cinard
// Added symbols static public Enumeration ADL_long_range(long, long),
// ADL_short_range(short, short) and ADL_int_range(int, int) in ADLClass
// symbol.
//
// Revision 1.20  1998/01/20 16:09:49  cinard
// Fixed bug on fully qualified class name resolution.
//
// Revision 1.19  1998/01/19 14:37:09  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.18  1998/01/13 09:00:49  cinard
// Fixed javadoc comments.
//
// Revision 1.17  1998/01/12 13:19:22  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.16  1998/01/07 19:15:44  cinard
// Added _RuntimeException attribute to JTypes.
//
// Revision 1.15  1998/01/06 10:01:59  cinard
// Fixed problem with this expression in java class and adl class symbols.
//
// Revision 1.14  1998/01/06 09:01:39  cinard
// Added new symbol (adl_thrownException iof type Throwable) iin ADL classes.
//
// Revision 1.13  1998/01/06 08:47:21  cinard
// Fixed ADL constants names (FAIL -> ADL_FAIL...)
//
// Revision 1.12  1997/12/22 16:14:32  cinard
// Added support for ADL/TDD predefined runtime methods and constants.
//
// Revision 1.11  1997/12/19 21:15:20  cinard
// Added support for exctracting exception (methods throw clauses) from
// the bytecode.
//
// Revision 1.10  1997/12/17 13:10:35  cinard
// Modified behavior of put symbol method to prevent several expressions
// (variable, parameter, binding...) with same name and scopes with non
// null intersection to be defined.
//
// Revision 1.9  1997/12/08 12:53:59  cinard
// Added type check mechanism.
// Fixed some byte code import bugs.
//
// Revision 1.8  1997/11/27 16:10:19  cinard
// Improve error reporting for class and package import mechanism.
//
// Revision 1.7  1997/11/27 09:04:09  cinard
// Debug flag off!!
//
// Revision 1.6  1997/11/26 17:13:13  cinard
// Changed package name.
//
// Revision 1.5  1997/11/19 15:42:14  cinard
// Improved import on demand and single type import behavior.
//
// Revision 1.4  1997/11/13 09:43:33  cinard
// Improved Builtin type management. Notably symboltable.c.CSymbolTableMgr.
// getBuiltinType() handles all forms of builtin types names.
//
// Revision 1.3  1997/11/12 13:37:04  cinard
// *** empty log message ***
//
// Revision 1.2  1997/11/04 09:04:34  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symboltable.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.1  1997/11/03 12:53:38  cinard
// Replaced Java symbols hierarchy with languag independent symbols hierarchy.
// Made JSymbolTableManager inherits from abstract SymbolTableMgr.
//
// ***********************************************************************
