// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/JavaTest.java,v $
// $Revision: 1.19 $
// $Date: 1998/06/08 13:42:19 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        JavaTest.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Oct 09,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

import com.objectspace.jgl.*;
import com.objectspace.jgl.adapters.*;
import com.objectspace.jgl.algorithms.*;

import ORG.opengroup.adl.symboltable.*;

/**
 * Unit testing of the symboltable.java package.
 */
 
class JavaTest 
{

   public final static void main(String[] args)
   throws SymbolAlreadyDefinedException
   {
      JavaTest jtest = new JavaTest();
      jtest.test1();
      jtest.test2();
      jtest.test3();
      jtest.test4();
 //      jtest.test5(); 
      jtest.test6();
      jtest.test7(); 
      jtest.test8();
      jtest.test9();
      jtest.test10();
   }

   private static void test1()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing types --");
      System.out.println("");

      JSymbolTableMgr stm = new JSymbolTableMgr();
      SymbolFactory   sbf = stm.getSymbolFactory();
      
      // Creates some interface

      LClass i1 = sbf.newClass("I1",JModifier.PUBLIC);
      stm.putSymbol(i1);

      LClass i2 = sbf.newClass("I2",JModifier.PUBLIC);
      i2.addInterface(i1);
      stm.putSymbol(i2);

      LClass i3 = sbf.newClass("I3",JModifier.PUBLIC);
      i3.addInterface(i1);
      i3.addInterface(i2);
      stm.putSymbol(i3);

      // Creates some classes

      LClass c1 = sbf.newClass("C1",JModifier.PUBLIC);
      stm.putSymbol(c1);

      LClass c2 = sbf.newClass("C2",JModifier.PUBLIC);
      c2.addSuperClass(c1);
      c2.addInterface(i1);
      c2.addInterface(i2);
      c2.addInterface(i3);
      stm.putSymbol(c2);
      stm.openScope(c2);


      LAttribute f1 = sbf.newAttribute("field1",JModifier.PUBLIC,c1);
      stm.putSymbol(f1);
      LAttribute f2 = sbf.newAttribute("field2",JModifier.PRIVATE,c1);
      stm.putSymbol(f2);

      // Defines a method.

      LMethod m1 = sbf.newMethod("m1",0x0000);
      m1.setReturnType(c1);
      m1.addParameter(sbf.newParameter("p",0x0000,c1));
      m1.addParameter(sbf.newParameter("q",0x0000,c1));
      stm.putSymbol(m1);
      

      // Open method scope
      stm.openScope(m1);

      // Creates some variables

      LVariable v1 = sbf.newVariable("v1",0x0000,c2);
      stm.putSymbol(v1);

      // Close method scope.
      stm.closeScope();

      // Close class scope.
      stm.closeScope();

      // Creates some adl classes

      ADLClass a1 = sbf.newADLClass(c1,"");
      stm.putSymbol(a1);
      ADLClass a2 = sbf.newADLClass(c2,"");
      a2.addSuperClass(a1);
      stm.putSymbol(a2);
      stm.openScope(a2);

      // Defines an adl method.

      ADLMethod am1 = sbf.newADLMethod("m1",0x0000);
      am1.setReturnType(c1);
      am1.addParameter(sbf.newParameter("p",0x0000,c1));
      am1.addParameter(sbf.newParameter("q",0x0000,c1));
      stm.putSymbol(am1);
      stm.openScope(am1);
      stm.openScope();

      Symbol b1 = sbf.newParameter("p",0x0000,c1);
      try {
         stm.putSymbol(b1); 
         System.out.println("false : symbol p put!");
      }
      catch ( SymbolAlreadyDefinedException e ) 
      { System.out.println("true : symbol p already defined."); }
      stm.closeScope();
      stm.closeScope();
      stm.closeScope();

      // Creates a TDD class

      TDDClass t1 = sbf.newTDDClass("T1");

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);

   }


   private void test2()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing symbol searching --");
      System.out.println("");
      
      JSymbolTableMgr stm = new JSymbolTableMgr();
      SymbolFactory   sbf = stm.getSymbolFactory();

      // Defines a constructed type 

      LClass complexT = sbf.newClass("Complex",JModifier.PUBLIC);
      stm.putSymbol(complexT);
      stm.openScope(complexT);
      LAttribute xS = sbf.newAttribute
                      ("x",0x0000,(LType)stm.getType("int",null));
      stm.putSymbol(xS);
      LAttribute yS = sbf.newAttribute
                      ("y",0x0000,(LType)stm.getType("int",null));
      stm.putSymbol(yS);

      // -- ASSERTION BEGIN --

      System.out.println
      (
         ( ( stm.getExpression("Complex.x",null)) == xS ) +
         ": tesing reachability of Complex.x in block"
      );
      System.out.println
      (
         ( ( stm.getExpression("x",null)) == xS ) +
         ": tesing reachability of x in block"
      );
      System.out.println
      (
         ( xS.assignable((LType)stm.getType("int",null)) ) +
         ": tesing assignability of intT to  complexT.x"
      );
      System.out.println
      (
         ( ! xS.assignable((LType)stm.getType("long",null)) ) +
         ": tesing unassignability of long to complexT.x"
      );
      System.out.println
      (
         ( stm.getExpression("this",null) != null ) +
         ": tesing reachability of this in class definition"
      );


      // == ASSERTION END ==================================================

      // Close current scope

      stm.closeScope();

      // == ASSERTION BEGIN ==================================================

      System.out.println
      (
         ( ( stm.getType("Complex",null)) == complexT ) +
         ": tesing reachability of Complex out of block"
      );
      System.out.println
      (
         ( ( stm.getExpression("Complex.x",null)) == xS ) +
         ": tesing reachability of Complex.x out of block"
      );
      System.out.println
      (
         ( ( stm.getExpression("x",null)) == null ) +
         ": tesing unreachability of x out of block"
      );
      System.out.println
      (
         ( stm.getExpression("Complex.this",null) == null ) +
         ": tesing unreachability of this outside class definition"
      );

      // == ASSERTION END  ==================================================

      // Defines a new constructed type

      Symbol pairT = sbf.newClass("Pair",JModifier.PUBLIC);
      stm.putSymbol(pairT);
      stm.openScope(pairT);
      Symbol firstF = sbf.newAttribute
                      ("first",0x0000,(LType)stm.getType("Complex",null));
      stm.putSymbol(firstF);
      Symbol secondF = sbf.newAttribute
                       ("second",0x0000,(LType)stm.getType("Complex",null));
      stm.putSymbol(secondF);

      // == ASSERTIONS BEGIN ================================================

      System.out.println
      (
         ( ( stm.getExpression("Pair.first.x",null)) == xS ) +
         ": tesing reachability of Pair.first.x in block"
      );
      System.out.println
      (
         ( ( stm.getExpression("first.x",null)) == xS ) +
         ": tesing reachability of first.x in block"
      );

      // == ASSERTIONS END =================================================

      stm.closeScope();

      // == ASSERTIONS BEGIN ================================================

      System.out.println
      (
         ( ( stm.getExpression("Pair.first.x",null)) == xS ) +
         ": tesing reachability of Pair.first.x out of block"
      );
      System.out.println
      (
         ( ( stm.getExpression("first.x",null)) == null ) +
         ": tesing unreachability of first.x out of block"
      );

      // == ASSERTIONS END =================================================

      // Defines a function

      LMethod bar = sbf.newMethod("bar",0x0000);
      stm.putSymbol(bar);
      bar.setReturnType((LType)stm.getType("int",null));
      bar.addParameter(sbf.newParameter("p",0x0000,(LType)stm.getType("int",null)));
      bar.addParameter(sbf.newParameter("q",0x0000,(LType)stm.getType("int",null)));

      // Opens function scope

      stm.openScope(bar);

      stm.putSymbol(sbf.newVariable("complexV",0x0000,complexT));

      // Open a block scope inside function body

      stm.openScope();
      stm.putSymbol(sbf.newVariable("complex1V",0x0000,complexT));
      stm.putSymbol(sbf.newVariable("complex2V",0x0000,complexT));
      stm.closeScope();

      stm.closeScope();

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
   }

   private static void test3()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing method searcher --");
      System.out.println("");

      JSymbolTableMgr stm = new JSymbolTableMgr();
      SymbolFactory   sbf = stm.getSymbolFactory();

      LClass complexT = sbf.newClass("Complex",JModifier.PUBLIC);
      stm.putSymbol(complexT);
      stm.openScope(complexT);

      // Defines a constructor.

      LMethod cstr = sbf.newMethod("Complex",0x0000);
      cstr.addParameter(sbf.newParameter("p",0x0000,stm.getType("int",null)));
      stm.putSymbol(cstr);

      // Defines a function

      LMethod bar = sbf.newMethod("bar",0x0000);
      stm.putSymbol(bar);
      bar.setReturnType((LType)stm.getType("int",null));
      bar.addParameter(sbf.newParameter("p",0x0000,(LType)stm.getType("int",null)));
      bar.addParameter(sbf.newParameter("q",0x0000,(LType)stm.getType("int",null)));

      // Search for constructor

      LCallableAssignComparator cs = new LCallableAssignComparator();
      cs.addParameterType((LType)stm.getType("int",null));

      Symbol cstr1 =  stm.getConstructor("Complex",cs,null,null);

      System.out.println
      (
         ( cstr1 == cstr ) +
         ": tesing reachability of Complex constructorr"
      );

      // Search for bar function

      LCallableAssignComparator ms = new LCallableAssignComparator();
      ms.addParameterType((LType)stm.getType("int",null));
      ms.addParameterType((LType)stm.getType("int",null));

      Symbol s2 = stm.getMethod("bar",ms,null,null);

      System.out.println
      (
         ( s2 == bar ) +
         ": tesing reachability of bar() - good searcher"
      );

      // Search for bar function

      ms = new LCallableAssignComparator();
      ms.addParameterType((LType)stm.getType("int",null));
      ms.addParameterType((LType)stm.getType("long",null));

      Symbol s3 = stm.getMethod("bar",ms,null,null);

      System.out.println
      (
         ( s3 == null ) +
         ": tesing unreachability of bar() - bad searcher"
      );

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }


   private static void test4()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing assignability --");
      System.out.println("");


      JSymbolTableMgr stm = new JSymbolTableMgr();
      SymbolFactory   sbf = stm.getSymbolFactory();

      // Get ref on builtin types
      LType intT  = (LType)stm.getType("int",null);
      LType longT = (LType)stm.getType("long",null);

      // Declares a variable of simple type

      LVariable aValueV = sbf.newVariable
                              ("aValueV",0x0000,(LType)stm.getType("int",null));
      stm.putSymbol(aValueV);

      // == ASSERTION BEGIN ================================================

      System.out.println
      (
         ( intT.assignable(intT) ) +
         ": tesing assignability of int to int"
      );

      System.out.println
      (
         ( longT.assignable(intT) ) +
         ": tesing assignability of int to long"
      );

      System.out.println
      (
         ( ! intT.assignable(longT) ) +
         ": tesing non assignability of long to int"
      );

      // == ASSERTION END ==================================================

      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);

   }


//    private void test5()
//    throws SymbolAlreadyDefinedException
//    {

//       System.out.println("");
//       System.out.println("-- Testing callback mechanism --");
//       System.out.println("");

//       JSymbolTableMgr stm = new JSymbolTableMgr();
//       SymbolFactory   sbf = stm.getSymbolFactory();

//       // Prepare foo function searcher

//       LCallableAssignComparator ms = new LCallableAssignComparator();
//       ms.setReturnType((LBuiltinType)stm.getType("int",null));
//       ms.addParameterType((LBuiltinType)stm.getType("int",null));
//       ms.addParameterType((LBuiltinType)stm.getType("int",null));

//       // Try to get foo function symbol (provide this as a FunctionGetter
//       // object). The call should fail.

//       Symbol s1 = stm.getMethod("A.foo",ms,new MethodSymbolListener(),null);

//       System.out.println
//       (
//          ( s1 == null ) +
//          ": tesing unreachability of foo() - not defined"
//       );

//       // Define a class
//       LClass  a   = sbf.newClass("A",0x0000);
//       stm.putSymbol(a);
//       stm.openScope(a);

//       // Define a function
//       LMethod foo = sbf.newMethod("foo",0x0000);
//       stm.putSymbol(foo);
//       foo.setReturnType((LBuiltinType)stm.getType("int",null));
//       foo.addParameter(sbf.newParameter("p",0x0000,(LBuiltinType)stm.getType("int",null)));
//       foo.addParameter(sbf.newParameter("q",0x0000,(LBuiltinType)stm.getType("int",null)));

//       stm.closeScope();

//       // Search for foo function

//       Symbol s2 = stm.getMethod("A.foo",ms,null,null);

//       System.out.println
//       (
//          ( s2 == foo ) +
//          ": tesing reachability of foo() - issued"
//       );

//       // Ask symbol table manager to issue call backs.

//       stm.finishParsing();

//       // Check foo was retured

//       System.out.println
//       (
//          ( gotAfterCallback == foo ) +
//          ": tesing callback was issued"
//       );

//       System.out.println("");
//       System.out.println("-- Dumping the symbol table --");
//       System.out.println("");

//       System.out.println(stm);

//    }

  private void test6()
  throws SymbolAlreadyDefinedException
  {
     
     System.out.println("");
     System.out.println("-- Testing bytecode import --");
     System.out.println("");

     JSymbolTableMgr stm = new JSymbolTableMgr();
     SymbolFactory   sbf = stm.getSymbolFactory();
     
     // Getting Object class

     LClass objectC = (LClass)stm.getType("Object",null);

     // Display Object
     System.out.println( ( objectC != null )
                       + " testing reachability of Object.");

     LClass stringC = (LClass)stm.getType("java.lang.String",null);

     // Display Object
     System.out.println( ( stringC != null )
                       + " testing reachability of java.lang.String.");

     System.out.println("");
     System.out.println("-- Dumping the symbol table --");
     System.out.println("");

     System.out.println(stm);
  }

  private void test7()
  throws SymbolAlreadyDefinedException
  {
     
     System.out.println("");
     System.out.println("-- Testing bytecode import --");
     System.out.println("");

     JSymbolTableMgr stm = new JSymbolTableMgr();
     SymbolFactory   sbf = stm.getSymbolFactory();
     
     // Import on demand package
     try { stm.addSingleImport("bank_example.bankAccount"); }
     catch (ClassNotAccessibleException e) 
     {
        throw new Error(e.getMessage());
     }

     // Getting Object class

     LClass lClass = (LClass)stm.getType("bankAccount",null);

      System.out.println
      (
         ( lClass != null ) +
         ": tesing reachability of bankAccount"
      );

      // Getting a non imported object
      LClass c2 = (LClass)stm.getType("bankAux",stm.getUnitScope());
      System.out.println
      (  ( c2 == null ) +
         ": tesing unreachability of bankAux - before single import"
      );

      // Issue single type import
      try
      {
         stm.addSingleImport("bank_example.bankAux");
      } catch (ClassNotAccessibleException e) { }

      // Getting a non imported object
      LClass c3 = (LClass)stm.getType("bankAux",null);
      System.out.println
      (  ( c3 != null ) +
         ": tesing reachability of bankAux - after single import"
      );
      
      // Force import of bankAux
      stm.getExpression("bankAux.foo",null);

     System.out.println("");
     System.out.println("-- Dumping the symbol table --");
     System.out.println("");

     System.out.println(stm);
  }

  private void test8()
  throws SymbolAlreadyDefinedException
  {
     
     System.out.println("");
     System.out.println("-- Testing bytecode import (8)  --");
     System.out.println("");

     JSymbolTableMgr stm = new JSymbolTableMgr();
     SymbolFactory   sbf = stm.getSymbolFactory();
     
     // Import on demand package
     try { stm.addImportOnDemand("bank_example"); }
     catch ( PackageNotAccessibleException e )
     {
        throw new Error(e.getMessage());
     }

     // Getting bankAccount

     LClass lClass = (LClass)stm.getType("bankAccount",null);

      System.out.println
      (
         ( lClass != null ) +
         ": tesing reachability of bankAccount"
      );

      // Getting bankAux

      LClass c2 = (LClass)stm.getType("bankAux",null);
      System.out.println
      (  ( c2 != null ) +
         ": tesing reachability of bankAux"
      );

      // Get class
      LClass c3 = (LClass)stm.getType("Class",null);
      System.out.println
      (  ( c3 != null ) +
         ": tesing reachability of Class"
      );

      // Get class
      LClass c4 = (LClass)stm.getType("java.lang.Class",null);
      System.out.println
      (  ( c4 != null ) +
         ": tesing reachability of java.lang.Class"
      );

     System.out.println("");
     System.out.println("-- Dumping the symbol table --");
     System.out.println("");

     System.out.println(stm);
  }

  private void test9()
  throws SymbolAlreadyDefinedException
  {
     
     System.out.println("");
     System.out.println("-- Testing type compatibility (9)  --");
     System.out.println("");

     JSymbolTableMgr stm = new JSymbolTableMgr();
     SymbolFactory   sbf = stm.getSymbolFactory();


     LType booleanT = stm.getType("boolean",null); 
     LType doubleT  = stm.getType("double",null);
     LType floatT   = stm.getType("float",null);
     LType longT    = stm.getType("long",  null);
     LType intT     = stm.getType("int",  null);
     LType charT    = stm.getType("char",null);
     LType shortT   = stm.getType("short",null);
     LType byteT    = stm.getType("byte",null);

     // testing boolean assignability

     System.out.println((      booleanT.assignable(booleanT)
                          && ! booleanT.assignable(charT)
                          && ! booleanT.assignable(byteT)
                          && ! booleanT.assignable(shortT)
                          && ! booleanT.assignable(intT)
                          && ! booleanT.assignable(longT)
                          && ! booleanT.assignable(floatT)
                          && ! booleanT.assignable(doubleT) 
                        ) + " : boolean test");

     // testing char assignability

     System.out.println((    ! charT.assignable(booleanT)
                          &&   charT.assignable(charT)
                          && ! charT.assignable(byteT)
                          && ! charT.assignable(shortT)
                          && ! charT.assignable(intT)
                          && ! charT.assignable(longT)
                          && ! charT.assignable(floatT)
                          && ! charT.assignable(doubleT) 
                        ) + " : char test");

     // testing byte assignability

     System.out.println((    ! byteT.assignable(booleanT)
                          && ! byteT.assignable(charT)
                          &&   byteT.assignable(byteT)
                          && ! byteT.assignable(shortT)
                          && ! byteT.assignable(intT)
                          && ! byteT.assignable(longT)
                          && ! byteT.assignable(floatT)
                          && ! byteT.assignable(doubleT) 
                        ) + " : byte test");

     // testing short assignability

     System.out.println((    ! shortT.assignable(booleanT)
                          && ! shortT.assignable(charT)
                          &&   shortT.assignable(byteT)
                          &&   shortT.assignable(shortT)
                          && ! shortT.assignable(intT)
                          && ! shortT.assignable(longT)
                          && ! shortT.assignable(floatT)
                          && ! shortT.assignable(doubleT) 
                        ) + " : short test");

     // testing int assignability

     System.out.println((    ! intT.assignable(booleanT)
                          &&   intT.assignable(charT)
                          &&   intT.assignable(byteT)
                          &&   intT.assignable(shortT)
                          &&   intT.assignable(intT)
                          && ! intT.assignable(longT)
                          && ! intT.assignable(floatT)
                          && ! intT.assignable(doubleT) 
                        ) + " : int test");

     // testing long assignability

     System.out.println((    ! longT.assignable(booleanT)
                          &&   longT.assignable(charT)
                          &&   longT.assignable(byteT)
                          &&   longT.assignable(shortT)
                          &&   longT.assignable(intT)
                          &&   longT.assignable(longT)
                          && ! longT.assignable(floatT)
                          && ! longT.assignable(doubleT)
                        ) + " : long test");

     // testing float assignability

     System.out.println((    ! floatT.assignable(booleanT)
                          &&   floatT.assignable(charT)
                          &&   floatT.assignable(byteT)
                          &&   floatT.assignable(shortT)
                          &&   floatT.assignable(intT)
                          &&   floatT.assignable(longT)
                          &&   floatT.assignable(floatT)
                             && ! floatT.assignable(doubleT) 
                        ) + " : float test");

     // testing double assignability

     System.out.println((    ! doubleT.assignable(booleanT)
                          &&   doubleT.assignable(charT)
                          &&   doubleT.assignable(byteT)
                          &&   doubleT.assignable(shortT)
                          &&   doubleT.assignable(intT)
                          &&   doubleT.assignable(longT)
                          &&   doubleT.assignable(floatT)
                          &&   doubleT.assignable(doubleT)
                        ) + " : double test"); 

     // Creating an array of arithmetic types

     LType[] types = new LType[5];
     types[0] = intT;
     types[1] = floatT;
     types[2] = intT;
     types[3] = shortT;
     types[4] = charT;

     // testing reference assignability

     LClass _Object = JTypes._Object;
     LClass _Cloneable = JTypes._Cloneable;
     LClass _J = (LClass)sbf.newClass("J",0x0000);
            _J.addSuperClass(_Object);
     LClass _I = (LClass)sbf.newClass("I",0x0000);
            _I.addSuperClass(_J);
     LClass _S = (LClass)sbf.newClass("S",0x0000);
            _S.addSuperClass(_Object);
     LClass _C = (LClass)sbf.newClass("C",0x0000);
            _C.addSuperClass(_S);
            _C.addInterface(_I);
     LObjectArray _A   = (LObjectArray)sbf.newArray(stm.getType("int",null));
     LObjectArray _A_S = (LObjectArray)sbf.newArray(_S);
     LObjectArray _A_C = (LObjectArray)sbf.newArray(_C);
     LClass _null = JTypes._null;


     System.out.println( 
                        (     _Object.assignable(_Object)
                           && _Object.assignable(_Cloneable)
                           && _Object.assignable(_J)
                           && _Object.assignable(_I)
                           && _Object.assignable(_C)
                           && _Object.assignable(_S)
                           && _Object.assignable(_A)
                           && _Object.assignable(_A_S)
                           && _Object.assignable(_A_C)
                           && _Object.assignable(_null)
                        ) + " : testing Object"
                       );

     System.out.println( 
                        (     ! _Cloneable.assignable(_Object)
                           &&   _Cloneable.assignable(_Cloneable)
                           && ! _Cloneable.assignable(_J)
                           && ! _Cloneable.assignable(_I)
                           && ! _Cloneable.assignable(_C)
                           && ! _Cloneable.assignable(_S)
                           &&   _Cloneable.assignable(_A)
                           &&   _Cloneable.assignable(_A_S)
                           &&   _Cloneable.assignable(_A_C)
                           &&   _Cloneable.assignable(_null)
                        ) + " : testing Cloneable"
                       );

     System.out.println( 
                         (    ( ! _J.assignable(_Object)    ) 
                           && ( ! _J.assignable(_Cloneable) ) 
                           && (   _J.assignable(_J)           ) 
                           && (   _J.assignable(_I)           ) 
                           && (   _J.assignable(_C)           ) 
                           && ( ! _J.assignable(_S)         ) 
                           && ( ! _J.assignable(_A)         ) 
                           && ( ! _J.assignable(_A_S)       ) 
                           && ( ! _J.assignable(_A_C)       ) 
                           && (   _J.assignable(_null)        ) 
                         )
                           + " : testing J"
                       );

     System.out.println( 
                          (    ( ! _I.assignable(_Object) ) 
                           &&  ( ! _I.assignable(_Cloneable) )
                           &&  ( ! _I.assignable(_J) ) 
                           &&  (   _I.assignable(_I) ) 
                           &&  (   _I.assignable(_C) ) 
                           &&  ( ! _I.assignable(_S) ) 
                           &&  ( ! _I.assignable(_A) ) 
                           &&  ( ! _I.assignable(_A_S) ) 
                           &&  ( ! _I.assignable(_A_C) ) 
                           &&  (   _I.assignable(_null) )
                          )
                           + " : testing I"
                       );

     System.out.println( 
                        (     ! _C.assignable(_Object)
                           && ! _C.assignable(_Cloneable)
                           && ! _C.assignable(_J)
                           && ! _C.assignable(_I)
                           &&   _C.assignable(_C)
                           && ! _C.assignable(_S)
                           && ! _C.assignable(_A)
                           && ! _C.assignable(_A_S)
                           && ! _C.assignable(_A_C)
                           &&   _C.assignable(_null)
                        ) + " : testing C"
                       );

     System.out.println( 
                        (     ! _S.assignable(_Object)
                           && ! _S.assignable(_Cloneable)
                           && ! _S.assignable(_J)
                           && ! _S.assignable(_I)
                           &&   _S.assignable(_C)
                           &&   _S.assignable(_S)
                           && ! _S.assignable(_A)
                           && ! _S.assignable(_A_S)
                           && ! _S.assignable(_A_C)
                           &&   _S.assignable(_null)
                        ) + " : testing S"
                       );

     System.out.println( 
                        (      ! _A.assignable(_Object)    
                           &&  ! _A.assignable(_Cloneable) 
                           &&  ! _A.assignable(_J)         
                           &&  ! _A.assignable(_I)              
                           &&  ! _A.assignable(_C)         
                           &&  ! _A.assignable(_S)         
                           &&    _A.assignable(_A)         
                           &&  ! _A.assignable(_A_S)       
                           &&  ! _A.assignable(_A_C)       
                           &&    _A.assignable(_null)      
                        ) + " : testing A"
                       );

     System.out.println( 
                        (     ! _A_S.assignable(_Object)
                           && ! _A_S.assignable(_Cloneable)
                           && ! _A_S.assignable(_J)
                           && ! _A_S.assignable(_I)
                           && ! _A_S.assignable(_C)
                           && ! _A_S.assignable(_S)
                           && ! _A_S.assignable(_A)
                           &&   _A_S.assignable(_A_S)
                           &&   _A_S.assignable(_A_C)
                           &&   _A_S.assignable(_null)
                        ) + " : testing A[S]"
                       );
     System.out.println( 
                        (     ! _A_C.assignable(_Object)
                           && ! _A_C.assignable(_Cloneable)
                           && ! _A_C.assignable(_J)
                           && ! _A_C.assignable(_I)
                           && ! _A_C.assignable(_C)
                           && ! _A_C.assignable(_S)
                           && ! _A_C.assignable(_A)
                           && ! _A_C.assignable(_A_S)
                           &&   _A_C.assignable(_A_C)
                           &&   _A_C.assignable(_null)
                        ) + " : testing A[C]"
                       );
     System.out.println( 
                        (     ! _null.assignable(_Object)
                           && ! _null.assignable(_Cloneable)
                           && ! _null.assignable(_J)
                           && ! _null.assignable(_I)
                           && ! _null.assignable(_C)
                           && ! _null.assignable(_S)
                           && ! _null.assignable(_A)
                           && ! _null.assignable(_A_S)
                           && ! _null.assignable(_A_C)
                           && ! _null.assignable(_null)
                        ) + " : testing null"
                       );

     
     // Testing inter type kind incompatibility
     System.out.println( 
                        (     ! longT.assignable(_Object)
                           && ! longT.assignable(_Cloneable)
                           && ! longT.assignable(_J)
                           && ! longT.assignable(_I)
                           && ! longT.assignable(_C)
                           && ! longT.assignable(_S)
                           && ! longT.assignable(_A)
                           && ! longT.assignable(_A_S)
                           && ! longT.assignable(_A_C)
                           && ! longT.assignable(_null)
                        ) + " : testing inter type kind incompatibility (1)"
                       );
     System.out.println( 
                        (     ! _Object.assignable(longT)
                           && ! _Cloneable.assignable(longT)
                           && ! _J.assignable(longT)
                           && ! _I.assignable(longT)
                           && ! _C.assignable(longT)
                           && ! _S.assignable(longT)
                           && ! _A.assignable(longT)
                           && ! _A_S.assignable(longT)
                           && ! _A_C.assignable(longT)
                           && ! _null.assignable(longT)
                        ) + " : testing inter type kind incompatibility (2)"
                       );

     System.out.println("");
     System.out.println("-- Dumping the symbol table --");
     System.out.println("");

     // System.out.println(stm);
  }

   private static void test10()
   throws SymbolAlreadyDefinedException
   {
      System.out.println("");
      System.out.println("-- Testing ambiguous method invocation --");
      System.out.println("");

      JSymbolTableMgr stm = new JSymbolTableMgr();
      SymbolFactory   sbf = stm.getSymbolFactory();

      LClass _Foo = sbf.newClass("Foo",JModifier.PUBLIC);
      stm.putSymbol(_Foo);
      stm.openScope(_Foo);

      // Defines a method

      LMethod bar1 = sbf.newMethod("bar",JModifier.PUBLIC);
      stm.putSymbol(bar1);
      bar1.setReturnType(JTypes.intT);
      bar1.addParameter(sbf.newParameter("p",0x00,JTypes._Object));
      bar1.addParameter(sbf.newParameter("q",0x00,JTypes._Object));
      stm.openScope(bar1);
      stm.closeScope();
      
      // Defines a method

      LMethod bar2 = sbf.newMethod("bar",JModifier.PUBLIC);
      stm.putSymbol(bar2);
      bar2.setReturnType(JTypes.intT);
      bar2.addParameter(sbf.newParameter("p",0x00,JTypes._Object));
      bar2.addParameter(sbf.newParameter("q",0x00,JTypes._String));
      stm.openScope(bar2);
      stm.closeScope();

      // Define a method

      LMethod foo1 = sbf.newMethod("foo",JModifier.PUBLIC);
      stm.putSymbol(foo1);
      foo1.setReturnType(JTypes.intT);
      foo1.addParameter(sbf.newParameter("p",0x00,JTypes._String));
      foo1.addParameter(sbf.newParameter("q",0x00,JTypes._Object));
      stm.openScope(foo1);
      stm.closeScope();

      // Defines a method

      LMethod foo2 = sbf.newMethod("foo",JModifier.PUBLIC);
      stm.putSymbol(foo2);
      foo2.setReturnType(JTypes.intT);
      foo2.addParameter(sbf.newParameter("p",0x00,JTypes._Object));
      foo2.addParameter(sbf.newParameter("q",0x00,JTypes._String));
      stm.openScope(foo2);
      stm.closeScope();

      // Defines a method
      LMethod test = sbf.newMethod("test",JModifier.PUBLIC);
      stm.putSymbol(test);
      test.setReturnType(JTypes.intT);
      stm.openScope(test);
      
      // Search method for for bar(String,String) invocation

      LCallableAssignComparator cas1 = new LCallableAssignComparator
                                           (SymbolId.METHOD);
      cas1.addParameterType(JTypes._String);
      cas1.addParameterType(JTypes._String);

      System.out.println
      (
         ( stm.getMethod("bar",cas1,null,null) == bar2 ) +
         ": tesing right bar() version is found"
      );

      // Search method for for foo(String,String) invocation

      System.out.println
      (
       ( stm.getMethod("foo",cas1,null,null) == null ) +
       ": tesing bar(String,String) is ambiguous"
      );

      stm.closeScope();
      stm.closeScope();

      // Defines a subclass
      LClass _Bar= sbf.newClass("Bar",JModifier.PUBLIC);
      _Bar.addSuperClass(_Foo);
      stm.putSymbol(_Bar);
      stm.openScope(_Bar);

      LMethod foo3 = sbf.newMethod("foo",JModifier.PUBLIC);
      stm.putSymbol(foo3);
      foo3.setReturnType(JTypes.intT);
      foo3.addParameter(sbf.newParameter("p",0x00,JTypes._String));
      foo3.addParameter(sbf.newParameter("q",0x00,JTypes._Object));
      stm.openScope(foo3);
      stm.closeScope();

      // Defines a method

      LMethod bar4 = sbf.newMethod("bar",JModifier.PUBLIC);
      stm.putSymbol(bar4);
      bar4.setReturnType(JTypes.intT);
      bar4.addParameter(sbf.newParameter("p",0x00,JTypes._Object));
      bar4.addParameter(sbf.newParameter("q",0x00,JTypes._Object));
      stm.openScope(bar4);
      stm.closeScope();

      // Defines a method

      LMethod test2 = sbf.newMethod("test",JModifier.PUBLIC);
      stm.putSymbol(test2);
      test2.setReturnType(JTypes.intT);
      stm.openScope(test2);

      // Search method for foo(String,Object) invocation

      LCallableAssignComparator cas2 = new LCallableAssignComparator
                                           (SymbolId.METHOD);
      cas2.addParameterType(JTypes._String);
      cas2.addParameterType(JTypes._Object);

      System.out.println
      (
         ( stm.getMethod("foo",cas2,null,null) == foo3 ) +
         ": tesing right foo() version is found"
      );

      // Search method for for foo(String,String) invocation

      LCallableAssignComparator cas3 = new LCallableAssignComparator
                                           (SymbolId.METHOD);
      cas3.addParameterType(JTypes._Object);
      cas3.addParameterType(JTypes._String);

      System.out.println
      (
       ( stm.getMethod("bar",cas3,null,null) == null ) +
       ": tesing bar(String,Object) is ambiguous"
      );



      System.out.println("");
      System.out.println("-- Dumping the symbol table --");
      System.out.println("");

      System.out.println(stm);
      
   }


//    class MethodSymbolListener implements SymbolListener
//    {
//       public void symbolFound(Symbol s)
//       {
//          gotAfterCallback = s;
//       }
//    }

   // == Private =============================================================
   
//    private Symbol gotAfterCallback;
   private static final boolean debug = false;
}

// ***********************************************************************
// History Log:
// $Log: JavaTest.java,v $
// Revision 1.19  1998/06/08 13:42:19  cinard
// Replaced  calls to obsolete method LClass.setSuperClass() with
// calls to LClass.addSuperClass().
//
// Revision 1.18  1998/03/25 14:47:19  cinard
// Updated newADLClass() method (superclass name shall be supplied now).
//
// Revision 1.17  1998/03/18 15:51:30  cinard
// Symbol factory in now separated from symbol table.
//
// Revision 1.16  1998/02/27 12:38:46  cinard
// Updated copyright notice.
//
// Revision 1.15  1998/02/23 14:29:24  cinard
// True Proxy design pattern implemented for JClassProxy.
// Improved type compatibility checks (complex type checking mechanism
// has been replaced by assignable() methods in subclasses of LType).
// JTypeKind.java now defines type kind constants that were formerly
// in JTypeChecker.java (update your client code!).
//
// Revision 1.14  1998/02/12 16:03:35  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.13  1998/01/19 14:37:09  cinard
// Changed COM.objectspace.jgl to com.objectspace.jgl (update from JGL 3.0.0
// to 3.1.0).
//
// Revision 1.12  1998/01/12 13:19:23  cinard
// Updated and fixed javadoc comments.
//
// Revision 1.11  1997/12/17 13:10:36  cinard
// Modified behavior of put symbol method to prevent several expressions
// (variable, parameter, binding...) with same name and scopes with non
// null intersection to be defined.
//
// Revision 1.10  1997/12/08 12:54:00  cinard
// Added type check mechanism.
// Fixed some byte code import bugs.
//
// Revision 1.9  1997/11/27 16:10:19  cinard
// Improve error reporting for class and package import mechanism.
//
// Revision 1.8  1997/11/26 17:13:13  cinard
// Changed package name.
//
// Revision 1.7  1997/11/19 15:43:14  cinard
// Added bytecode import tests for JSymbolLoader and JSymbolTablMgr classes.
//
// Revision 1.6  1997/11/12 13:37:04  cinard
// *** empty log message ***
//
// Revision 1.5  1997/11/04 16:04:28  cinard
// Made some modification for C and C++ parsers.
//
// Revision 1.4  1997/11/04 09:04:35  cinard
// Scope.getSymbol() does not throw AmbiguousRequestException anymore.
// symboltable.java package name replaced with symboltable.j (problems
// with javadoc). Note that symboltable/CVS/* should be updated manually
// (replacing java with j where necessary).
//
// Revision 1.3  1997/11/03 12:53:39  cinard
// Replaced Java symbols hierarchy with languag independent symbols hierarchy.
// Made JSymbolTableManager inherits from abstract SymbolTableMgr.
//
// Revision 1.2  1997/10/27 11:18:11  cinard
// Added language independant symbols to the symboltable package.
//
// Revision 1.1  1997/10/13 15:10:50  cinard
// Symbol and symbol management classes for the ADL/Java parser.
//
// ***********************************************************************
