// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/symboltable/j/QualifiedName.java,v $ 
// $Revision: 1.1 $
// $Date: 1998/03/18 16:15:40 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   symboltable.j
// NAME:        QualifiedName.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Mar 16,  1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.symboltable.j;

/**
 * Qualified name.
 * <P>
 * A qualified name is made up of identifiers separated by dots.
 * Identifiers minus last one make the qualified name's prefix.
 * Last identifier makes the qualified name suffix.
 * E.g. in qualified name "java.lang.Object",  prefix is "java.lang"
 * while suffix is "Object".
 * </P>
 */

public class  QualifiedName
{

  /**
   * Constructs a qualified name from a string representation of a qualified
   * name.
   */

  public QualifiedName(String qualifiedName)
  {
     int index = qualifiedName.lastIndexOf(".");
     _prefix = ( index > 0 ) ? qualifiedName.substring(0,index)   
                             : null;
     _suffix = ( index > 0 ) ? qualifiedName.substring(index + 1) 
                             : qualifiedName;
  }

  /**
   * Is qualified a simple name (made up of a simple identifier)?
   */
  
  public boolean isSimple()
  {
     return ( _prefix == null );
  }

  /**
   * Get string representation of the qualified name prefix (that is qualified
   * name minus last identifier).
   * @return String representation of the qualified name prefix if qualified 
   *         name is not simple null otherwise.
   */

  public String getPrefix()
  {
     return _prefix;
  }

  /**
   * Get qualified name suffix (that is the simple name made up by the
   * last identifier of the qualified name).
   */
  
  public String getSuffix()
  {
     return _suffix;
  }

  /**
   * Made up a string out of the qualified name.
   */

  public String toString()
  {
     return   isSimple()
            ? _suffix
            : _prefix + "." + _suffix;
  }

  // -- Private --

  private final String _suffix;
  private final String _prefix;

}

// ***********************************************************************
// History Log:
// $Log: QualifiedName.java,v $
// Revision 1.1  1998/03/18 16:15:40  cinard
// Tool class enabling to handle Java qualified names.
//
// ***********************************************************************
