// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1997-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions to use, copy, modify and distribute this software are
// governed by the terms and conditions set forth in the file COPYRIGHT,
// located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/ACOauxiliary.java,v $
// $Revision: 1.3 $
// $Date: 1998/07/28 16:07:47 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        ACOauxiliary.java
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 17, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Auxiliary class for the construction of generic parts of code in ACOs.
 * @see ADLCTransformVisitor
 * @author Cyrille Bareau
 */

class ACOauxiliary {

  /* Flag for debug mode */
  private final static boolean debug = false;

  /* handles */

  /** Handle for the name of the adl module */
  private String moduleName;

  /** Handle for the current fct return type */
  private String fctReturnType;

  /** Handle for the current fct type default value*/
  private LType fctTypeDefaultValue;

  /** Handle for the current fct declarator */
  private ASTFunctionDeclarator fctDeclTor;

  /** Handle for the current fct return type */
  private ASTDeclarationSpecifiers fctDeclSpec;

  /** Name of the current fct */
  private String fctName;

  /** Handle for the parameters of the current fct */
  private LParameter[] fctParams;

  /** Handle for the "normal" expression */
  private Node normalSpec;

  /** Handle for the "abnormal" expression */
  private Node abnormalSpec;

  /** Statements for the initialization of the current function */
  private Vector ACO_Initialization;

  /** Statements for the call to the real object fct,
    * catch of exceptions, etc */
  private Vector ACO_CallBlock;

  /** Statements for the finalization of the current fct
    * (e.g. re-throw of exceptions) */
  private Vector ACO_Final;

  /** Statements for the initialization of the assertion code */
  private Vector ACO_AssertionInit;

  /** The ADL transform visitor that uses the current ACO class */
  private ADLCTransformVisitor associatedVisitor;


  private boolean firstFct;
  private String excName;
  private ASTStatement setCompoundStmt, stmt1, stmt2, stmt3, stmt4;


  /**
    * Single constructor
    * @param visitor The ADL transform visitor that uses the current ACO class
    */
  ACOauxiliary(ADLCTransformVisitor visitor) {

    associatedVisitor = visitor;
    firstFct = true;
  }

  void initializeFct() {

    if (debug) System.out.println("ACOauxiliary: initializeFct ...");

    ACO_Initialization = new Vector();
    ACO_CallBlock = new Vector();
    ACO_Final = new Vector();
    ACO_AssertionInit = new Vector();

    fctReturnType = null;
    fctTypeDefaultValue = null;
    fctDeclTor = null;
    fctName = null;
    fctParams = null;
    normalSpec = null;
    abnormalSpec = null;
  }


  void setNormalSpec(Node s) {
    normalSpec = s;
  }

  Node getNormalSpec() {
    return normalSpec;
  }

  void setAbnormalSpec(Node s) {
    abnormalSpec = s;
  }

  Node getAbnormalSpec() {
    return abnormalSpec;
  }

  void setFctReturnType(String type) {
    fctReturnType = type;
  }

  String getFctReturnType() {
    return fctReturnType;
  }

  void setFctTypeDefaultValue(LType type) {
    fctTypeDefaultValue = type;
  }

  void setFctDeclTor(ASTFunctionDeclarator type) {
    fctDeclTor = type;
  }

  ASTFunctionDeclarator getFctDeclTor() {
    return fctDeclTor;
  }

  void setFctDeclSpec(ASTDeclarationSpecifiers d) {
    fctDeclSpec = d;
  }

  ASTDeclarationSpecifiers getFctDeclSpec() {
    return fctDeclSpec;
  }

  void setFctName(String name) {
    fctName = name;
  }

  String getFctName() {
    return fctName;
  }

  void setFctParams(LParameter[] params) {
    fctParams = params;
  }

  LParameter[] getFctParams() {
    return fctParams;
  }

  void setModuleName(String name) {
    moduleName = name;
  }
  
  String getModuleName() {
    return moduleName;
  }


  /**
   * Generates the generic initialization code for each
   * function of an ACO class.
<PRE>
|  bool doReport = finalJournalReporting && debugMode;
|  bool abnormal = false;
|  bool normal = true;
|  bool tddRes;
</PRE>
   * and in the case of a constructor:
<PRE>
|  skipAssertions |= testContext->skipAssertions;
</PRE>
   * else:
<PRE>
|  skipAssertions |= testContext->skipAssertions || realObject == NULL;
|  bank *bankObject = (bank *)realObject;
</PRE>
   * @param inherits flag when the tested function has inherited semantics.
   * @return The handle on the generated statements (CompoundStatement ASTs)
   */
  Vector makeACO_Initialization() {

    if (debug) System.out.println("\nACOauxiliary: makeACO_Initialization ...");

    // statements that are common to every function

    // <fctReturnType> __ADL_return = <default value>;
    if (debug) System.out.println("Function return type = " + fctReturnType);
    if (! fctReturnType.equals("void")) {
      Node declTor = getFctDeclTor();
      ASTDeclarator ret = new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);
      Node tmp1 = declTor, tmp2 = ret;
      while(tmp1.jjtGetChild(0) instanceof ASTPtrOperator) {
        ADLCGenericVisitor.parentChild(tmp2, declTor.jjtGetChild(0), 0);
        ADLCGenericVisitor.parentChild(tmp2,
                                       new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR),
                                       1);
        tmp2 = tmp2.jjtGetChild(1);
        tmp1 = tmp1.jjtGetChild(1);
      }
      ADLCGenericVisitor.parentChild(tmp2,
                  new DirectDeclaratorBuilder("__ADL_return").generatedAST(),
                  0);
      ACO_Initialization.addElement
        ( new DeclarationBuilder
          ( getFctDeclSpec(),
            new InitDeclaratorListBuilder
             ( new InitDeclaratorBuilder
               ( ret, 
                 CTypeAux.getDefaultValue(fctTypeDefaultValue, true)))
            .generatedAST())
          .generatedAST());
    }

    // int doReport = finalJournalReporting;
    ACO_Initialization.addElement(new DeclarationBuilder
                                  ( ADLCGenericVisitor.makeBuiltinType("int"),
                                    new InitDeclaratorBuilder
                                    ( new DirectDeclaratorBuilder("doReport").generatedAST(),
                                      ADLCGenericVisitor.makeName("ADL_finalJournalReporting"))
                                    .generatedAST())
                                  .generatedAST());

    // int abnormal = 0;
    ACO_Initialization.addElement(new DeclarationBuilder
                                  ( ADLCGenericVisitor.makeBuiltinType("int"),
                                    new InitDeclaratorBuilder
                                    ( new DirectDeclaratorBuilder("abnormal").generatedAST(),
                                      ADLCGenericVisitor.makeConstant("0"))
                                    .generatedAST())
                                  .generatedAST()
                                  );
    // int normal = 1;
    ACO_Initialization.addElement(new DeclarationBuilder
                                  ( ADLCGenericVisitor.makeBuiltinType("int"),
                                    new InitDeclaratorBuilder
                                    ( new DirectDeclaratorBuilder("normal").generatedAST(),
                                      ADLCGenericVisitor.makeConstant("1"))
                                    .generatedAST())
                                  .generatedAST());
    // int tddRes = 0;
    ACO_Initialization.addElement(new DeclarationBuilder
                                  ( ADLCGenericVisitor.makeBuiltinType("int"),
                                    new InitDeclaratorBuilder
                                    ( new DirectDeclaratorBuilder("tddRes").generatedAST(),
                                      ADLCGenericVisitor.makeConstant("0"))
                                    .generatedAST())
                                  .generatedAST());

    // int skipAssertions = ADL_skipAssertions
    ACO_Initialization.addElement(new DeclarationBuilder
                                  ( ADLCGenericVisitor.makeBuiltinType("int"),
                                    new InitDeclaratorBuilder
                                    ( new DirectDeclaratorBuilder("skipAssertions").generatedAST(),
                                      ADLCGenericVisitor.makeConstant("ADL_skipAssertions"))
                                    .generatedAST())
                                  .generatedAST());

    if (debug) System.out.println("ACOauxiliary: Successfull ACO_Initialisation\n");
    return ACO_Initialization;
  }


  /**
   * Generates the generic code for the call to the real object function.
   * For instance for function "bankAccount open_account(long amount, long &acctNum)
   * throw (negativeAmountExc, bankFullExc)" of class bank:
   <PRE>
   |  if (doReport) {
   |     ADL_setblock();
   |     ADL_infoline("*** Checking deposit(int, int) ***");
   |  }
   |  if (!skipAssertions) {
   |     __ADL_returned = deposit(acct, amt);	 
   |
   |     // evaluation of the abnormal clause
   |     abnormal = (__ADL_returned < 0);
   |
   |     // evaluation of the normal clause
   |     normal = (__ADL_returned >= 0);
   |  }
   </PRE>
   * @return The handle on the generated statements
   */
  Vector makeACO_CallBlock() {

    if (debug) System.out.println("ACOauxiliary: makeACO_CallBlock ...");

    //    if (doReport) {
    //      ADL_setblock();
    //      ADL_infoline("*** Checking deposit(int, int) ***");
    //    }

    ASTIfStatement ifDoReport = new
      ASTIfStatement(ADLCParserTreeConstants.JJTIFSTATEMENT);
    ADLCGenericVisitor.parentChild(ifDoReport,
                                   ADLCGenericVisitor.makeName("doReport"),
                                   0);

    if (firstFct) {
      setCompoundStmt = new StatementBuilder
        ( new PostfixExprBuilder
          ( "ADL_setblock",
            new ArgumentListBuilder().generatedAST()))
        .generatedAST();
    }

    ASTString str = ADLCGenericVisitor.makeString("*** Checking " + fctName + " ***");

    ASTStatement infolineStmt = new StatementBuilder
      ( new PostfixExprBuilder
        ( "ADL_infoline",
          new ArgumentListBuilder(str).generatedAST()))
      .generatedAST();

    ASTCompoundStatement thenBlk = new 
      ASTCompoundStatement(ADLCParserTreeConstants.JJTCOMPOUNDSTATEMENT);

    ADLCGenericVisitor.parentChild(thenBlk, setCompoundStmt, 0);
    ADLCGenericVisitor.parentChild(thenBlk, infolineStmt, 1);
    
    ADLCGenericVisitor.parentChild(ifDoReport, thenBlk, 1);
    ACO_CallBlock.addElement(ifDoReport);


    // if (! skipAssertions)
    ASTIfStatement ifNotSkip = new
      ASTIfStatement(ADLCParserTreeConstants.JJTIFSTATEMENT);
    ASTUnaryOperatorExpression notSkip = 
      new ASTUnaryOperatorExpression(ADLCParserTreeConstants.JJTUNARYOPERATOREXPRESSION);
    try { notSkip.setProperty("operator", "!"); }
    catch(PropertyAccessException e) {}
    ADLCGenericVisitor.parentChild(notSkip,
                                   ADLCGenericVisitor.makeName("skipAssertions"),
                                   0);
    ADLCGenericVisitor.parentChild(ifNotSkip, notSkip, 0);


    ASTCompoundStatement ifNotSkipBlock = new
      ASTCompoundStatement(ADLCParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCGenericVisitor.parentChild(ifNotSkip, ifNotSkipBlock, 1);
    int ifNotSkipBlockIndex = 0;

    //  [ __ADL_return = ] <fctName>(<fctParams.name>); }

    ArgumentListBuilder callArgs = new ArgumentListBuilder();
    for (int i = 0; i < fctParams.length; i++) { // LParameter[] fctParam
      callArgs.addArgument(ADLCGenericVisitor.makeName(fctParams[i].getName()));
    }

    ASTStatement callStmt;
    ASTPostfixExpression callExpr = new PostfixExprBuilder
      (fctName,
       callArgs.generatedAST())
      .generatedAST();

    if ( ! fctReturnType.equals("void")) {
      callStmt = new StatementBuilder
        ( new AssignmentBuilder
          ( ADLCGenericVisitor.makeName("__ADL_return"),
            "=",
            callExpr))
        .generatedAST();
    }
    else {
      callStmt = new StatementBuilder(callExpr, true).generatedAST();
    }

    ADLCGenericVisitor.parentChild(ifNotSkipBlock, callStmt, ifNotSkipBlockIndex++);

    // abnormal = <abnormalSpec>
    // normal = <normalSpec>

    boolean bothSpecified = false;

    // visit (ab)normalSpec if already defined

    if (abnormalSpec != null) {
      abnormalSpec = (Node)abnormalSpec.jjtAccept(associatedVisitor, null);
    }
    if (normalSpec != null) {
      normalSpec = (Node)normalSpec.jjtAccept(associatedVisitor, null);
    }

    if (normalSpec == null) {
      if (abnormalSpec == null) {
        // default: abnormalSpec = 0
	abnormalSpec = ADLCGenericVisitor.makeConstant("0");
	normalSpec = ADLCGenericVisitor.makeConstant("1");
      }
      else {
        // default: normalSpec = !abnormal
        ASTUnaryOperatorExpression notabnormal = 
          new ASTUnaryOperatorExpression(ADLCParserTreeConstants.JJTUNARYOPERATOREXPRESSION);
        try { notabnormal.setProperty("operator", "!"); }
        catch(PropertyAccessException e) {}
        ADLCGenericVisitor.parentChild(notabnormal,
                                       ADLCGenericVisitor.makeName("abnormal"),
                                       0);
        normalSpec = notabnormal;
      }
    }
    else { // normalSpec != null
      if (abnormalSpec == null) {

        // abnormalSpec = !normalSpec
	ASTUnaryOperatorExpression notnormal = 
	  new ASTUnaryOperatorExpression(ADLCParserTreeConstants.JJTUNARYOPERATOREXPRESSION);
	try { notnormal.setProperty("operator", "!"); }
        catch(PropertyAccessException e) {}
	ADLCGenericVisitor.parentChild(notnormal, ADLCGenericVisitor.makeName("normal"), 0);
        abnormalSpec = notnormal;
      }
      else // normal and abnormal both user-defined
        bothSpecified = true;
    }

    ADLCGenericVisitor.parentChild
      ( ifNotSkipBlock,
	new StatementBuilder
	( new AssignmentBuilder
	  ("abnormal", "=", abnormalSpec))
        .generatedAST(),
	ifNotSkipBlockIndex++);

    ADLCGenericVisitor.parentChild
      ( ifNotSkipBlock,
	new StatementBuilder
	( new AssignmentBuilder
	  ("normal", "=", normalSpec))
        .generatedAST(),
	ifNotSkipBlockIndex++);
    
    
    // if (normal && abnormal) ADL_result(ADL_AMBIGUOUS);
    
    if (bothSpecified) {
      // if normal and/or abnormal have not been user-defined, they have been
      // given default opposite values, hence the condition (normal && 
      // abnormalSpec) is false and the code is not to be generated.
      
      ASTIfStatement ifNormAbn = new
        ASTIfStatement(ADLCParserTreeConstants.JJTIFSTATEMENT);
      ASTLogicalAndExpression nANDabn = 
        new ASTLogicalAndExpression(ADLCParserTreeConstants.JJTLOGICALANDEXPRESSION);
      ADLCGenericVisitor.parentChild
	( nANDabn,
	  ADLCGenericVisitor.makeName("normal"),
	  0);
      ADLCGenericVisitor.parentChild
	( nANDabn,
	  ADLCGenericVisitor.makeName("abnormal"),
	  1);

      ADLCGenericVisitor.parentChild(ifNormAbn, nANDabn, 0);

      ADLCGenericVisitor.parentChild
	( ifNormAbn,
	  new StatementBuilder
	  ( new PostfixExprBuilder
	    ( "ADL_result",
              new ArgumentListBuilder
              ( ADLCGenericVisitor.makeName("ADL_AMBIGUOUS"))
              .generatedAST()))
          .generatedAST(),
          1);

      ADLCGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn, ifNotSkipBlockIndex++);

      // if (!(normal || abnormalSpec)) ADL_result(ADL_UNDEFINED);

      // if normal and/or abnormal have not been user-defined, they have been
      // given default opposite values, hence the condition !(normal || 
      // abnormalSpec) is false and the code is not to be generated.

      ASTIfStatement ifNormAbn2 = new
        ASTIfStatement(ADLCParserTreeConstants.JJTIFSTATEMENT);

      ASTLogicalOrExpression nORabn = 
        new ASTLogicalOrExpression(ADLCParserTreeConstants.JJTLOGICALOREXPRESSION);
      ADLCGenericVisitor.parentChild
	( nORabn,
	  ADLCGenericVisitor.makeName("normal"),
	  0);
      ADLCGenericVisitor.parentChild
	( nORabn,
	  ADLCGenericVisitor.makeName("abnormal"),
	  1);
      
      ASTUnaryOperatorExpression notnORabn = 
	new ASTUnaryOperatorExpression(ADLCParserTreeConstants.JJTUNARYOPERATOREXPRESSION);
      try { notnORabn.setProperty("operator", "!"); }
      catch(PropertyAccessException e) {}

      ASTParentheizedExpression parenth = 
        new ASTParentheizedExpression(ADLCParserTreeConstants.JJTPARENTHEIZEDEXPRESSION);

      ADLCGenericVisitor.parentChild(notnORabn, parenth, 0);
      ADLCGenericVisitor.parentChild(parenth, nORabn, 0);

      ADLCGenericVisitor.parentChild(ifNormAbn2,notnORabn, 0);

      ADLCGenericVisitor.parentChild
	( ifNormAbn2,
	  new StatementBuilder
	  ( new PostfixExprBuilder
	    ( "ADL_result",
              new ArgumentListBuilder
              ( ADLCGenericVisitor.makeName("ADL_UNDEFINED"))
              .generatedAST()))
          .generatedAST(),
	  1);

      ADLCGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn2, ifNotSkipBlockIndex++);
    }


    ACO_CallBlock.addElement(ifNotSkip);
    
    return ACO_CallBlock;
  }


  /**
   * Generates the generic final code for each function of an ACO class
   * (i.e. "return __ADL_return;" if the function has a non-void return type).
   * @return The handle on the generated statements
   */
  Vector makeACO_Final() {

    if (debug) System.out.println("ACOauxiliary: makeACO_Final ...");

    ACO_Final = new Vector();

    //    [ return __ADL_return ; ]

    if ( ! fctReturnType.equals("void")) {
      ASTReturnStatement finalReturn = new
        ASTReturnStatement(ADLCParserTreeConstants.JJTRETURNSTATEMENT);
      ADLCGenericVisitor.parentChild
	(finalReturn, ADLCGenericVisitor.makeName("__ADL_return"), 0);
      ACO_Final.addElement(finalReturn);
    }
    firstFct = false;
    return ACO_Final;
  }


}

// ***********************************************************************
// History Log:
// $Log: ACOauxiliary.java,v $
// Revision 1.3  1998/07/28 16:07:47  mallet
// Replaced makeIdExpression calls with makeName.
//
// Revision 1.2  1998/06/30 16:05:00  mallet
// Removed function getFctTypeDefaultValue to use getDefaultValue in
// CTypeAux instead.
//
// Revision 1.1  1998/06/23 13:39:43  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:09  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
