// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/ArgumentListBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/06/30 08:36:07 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        ArgumentListBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C list of arguments
  * @author Cyrille Bareau
  */

public class ArgumentListBuilder {

  /** Root of the generated tree */
  private ASTArgumentList root;

  /** Expression root when ArgumentList contains more than one argument */
  private ASTExpression exprRoot;

  /** number of childrens */
  private int index;


  /**
    * Creates an empty list of arguments
    */
  public ArgumentListBuilder () {

    root = new
      ASTArgumentList(ADLCParserTreeConstants.JJTARGUMENTLIST);
  }


  /**
    * Creates a list of 1 argument
    * @param arg The initial argument
    */
  public ArgumentListBuilder (Node arg) {

    this();
    ADLCGenericVisitor.parentChild(root, arg, 0);
    index = 1;
  }


  /**
    * Adds an argument to the argument list
    * @param arg The new argument
    * @return this (so that it is possible to chain-adding
    * new arguments)
    */
  public ArgumentListBuilder addArgument (Node arg) {

    if (index >= 1) {
      if (exprRoot == null) {
	exprRoot = 
	  new ASTExpression(ADLCParserTreeConstants.JJTEXPRESSION);
	ADLCGenericVisitor.parentChild(exprRoot, root.jjtGetChild(0), 0);
	ADLCGenericVisitor.parentChild(root, exprRoot, 0);
      }
      ADLCGenericVisitor.parentChild(exprRoot, arg, index);
    } else {
      ADLCGenericVisitor.parentChild(root, arg, index);
    }

    index += 1;
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTArgumentList generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ArgumentListBuilder.java,v $
// Revision 1.2  1998/06/30 08:36:07  mallet
// Fixed bug when argument list contains more than one argument: there
// should be an intermediate ASTExpression node holding the arguments.
//
// Revision 1.1  1998/06/23 13:39:44  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:10  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
