// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/AssignmentBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:19:26 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        AssignmentBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.parsers.PropertyAccessException;

/**
  * Creates a tree for a C assignment.
  * @author Cyrille Bareau
  */

public class AssignmentBuilder {

  /** Root of the generated tree */
  private ASTAssignmentExpression root;


  /**
    * Creates a C assignment expression
    * @param lhs The name of the variable to assign
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (String lhs, String op, Node rhs) {

    this(ADLCGenericVisitor.makeName(lhs), op, rhs);
  }


  /**
    * Creates a C assignment expression
    * @param lhs The variable to assign
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (ASTName lhs, String op, Node rhs) {

    root = new
      ASTAssignmentExpression(ADLCParserTreeConstants.JJTASSIGNMENTEXPRESSION);
    try { root.setProperty("operator", op); }
    catch(PropertyAccessException e) {}
    ADLCGenericVisitor.parentChild(root, lhs, 0);
    ADLCGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a C assignment expression
    * @param lhs The variable to assign (case of an array element)
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (SimpleNode lhs, String op, Node rhs) {

    root = new
      ASTAssignmentExpression(ADLCParserTreeConstants.JJTASSIGNMENTEXPRESSION);
    try { root.setProperty("operator", op); }
    catch(PropertyAccessException e) {}
    ADLCGenericVisitor.parentChild(root, lhs, 0);
    ADLCGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTAssignmentExpression generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: AssignmentBuilder.java,v $
// Revision 1.2  1998/07/28 16:19:26  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.1  1998/06/23 13:39:44  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:10  bareau
// First version of the ADL/C translator (ADL part only).
//
// ***********************************************************************
