// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/CTDDTransformVisitor.java,v $
// $Revision: 1.4 $
// $Date: 1998/10/14 08:17:28 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        CTDDTransformVisitor
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 5, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.PropertyAccessException;
import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.symboltable.c.*;
import java.util.Vector;
import java.util.Enumeration;

/**
  * Transformation visitor.  Must change a TDD AST into Java ASTs.
  * <P> For an overview of the algorithms used by this transformation,
  * see the ADL2.0 Translation System Design Specification, Chapter 5
  * (Code Generation), paragraph 5.2.2.
  * @see ADLCGenericVisitor
  * @see ADLCParserVisitor
  * @author Cyrille Bareau
 */

public class CTDDTransformVisitor extends CGeneric {

  /** Public handle for vector of generated Java ASTs that
    * correspond to datasets */
  public Vector datasets;

  /** Public handle for vector of generated Java ASTs that
    * correspond to factories */
  public Vector factories;

  /** Public handle for vector of generated Java ASTs that
    * correspond to directives */
  public Vector directives;

  /** Public handle for the generated Java AST that
    * corresponds to the interface for the TDD fields */
  public ASTTranslationUnit constantsClass;

  /** Handle for the generated Java AST that corresponds
    * to the source tddclass (contains only test functions) */
  private ASTTranslationUnit TDDClass;

  /** Handle for the "include" declarations of source tdd class */
  private ASTIncludeFileList includeList;

  /** Handle for the "include" declarations of generated CST class */
  private ASTIncludeFileList includeListCST;

  /** Handle for the "include" declarations of generated TDD class */
  private ASTIncludeFileList includeListTDD;

  /** Handle for the declaration of the TDD local fields
    * (will be used to construct constantsClass)
    * @see #constantsClass */
  private Vector constants;

  /** Name of the TDD class (i.e. name of the source file prefix) */
  private static String unitName; // tddClassName

  /** Names of the ACOs to generate. */
  private Vector acoNames = null;

  /** Names of the TDD classes to generate. */
  private Vector tddClassNames = null;

  /** Names of the TDD constant classes to generate. */
  private Vector cstClassNames = null;

  /** Names of the TDD datasets to generates. */
  private Vector tddDatasetNames = null;

  /** Names of the TDD factories to generates. */
  private Vector tddFactoryNames = null;

  /** Names of the TDD test directives to generates. */
  private Vector tddTestDirectiveNames = null;


  static final private boolean debug = false;


  /** 
    * Entry point. Initializes needed handles and launches the visit.
    * @param topNode the root node of the tree to visit
    * @return the root node of the transformed tree
    */
  public Node translate(Node topNode) {

    init();
    return (Node)topNode.jjtAccept(this, null);
  }


  /**
    * Initialization of handles.
    */
  private void init() {

    datasets = new Vector();
    directives = new Vector();
    factories = new Vector();
    constants = new Vector();
    constantsClass = null;

    /** Names of the ACOs to generate. */
    acoNames = new Vector();

    /** Names of the TDD classes to generate. */
    tddClassNames = new Vector();

    /** Names of the TDD constant classes to generate. */
    cstClassNames = new Vector();

    /** Names of the TDD datasets to generate. */
    tddDatasetNames = new Vector();

    /** Names of the TDD factories to generate. */
    tddFactoryNames = new Vector();

    /** Names of the TDD test directives to generate. */
    tddTestDirectiveNames = new Vector();

  }


  /** Names of the ACOs to generate. */
  public Vector getACONames() { return acoNames; }

  /** Names of the TDD classes to generate. */
  public Vector getTDDClassNames() { return tddClassNames; }

  /** Names of the TDD constant classes to generate. */
  public Vector getTDDConstantNames() { return cstClassNames; }
  
  /** Names of the TDD datasets to generates. */
  public Vector getTDDDatasetNames() { return tddDatasetNames; }

  /** Names of the TDD factories to generates. */
  public Vector getTDDFactoryNames() { return tddFactoryNames; }

  /** Names of the TDD test directives to generates. */
  public Vector getTDDTestDirectiveNames() { return tddTestDirectiveNames; }



  /**
    * Visit pattern for a node of type ASTTranslationUnit.
    * <P><B>Algorithm</B><UL>
    * <LI>Stores the ASTIncludeFileDeclaration children nodes in includeDecls;
    * <LI>Visits the ASTTDD_ClassDeclaration child node.
    * </UL>
    * @param node the ASTADL_CompilationUnit node to visit
    * @param data unused
    * @return the transformed ASTADL_CompilationUnit node
    */
  public Object visit(ASTADL_TranslationUnit node, Object data) {

    if (debug)
      System.out.println("visiting ASTTranslationUnit");

    // save former stm context
    Object savedStmContext = SimpleNode.stm.getContext();

    // set stm context for this ast
    try {
      Object stmContext = node.getProperty("stmContext");
      SimpleNode.stm.setContext(stmContext);
    }
    catch ( PropertyAccessException e ) { }

    Node child;
    TDDClass = new
      ASTTranslationUnit(ADLCParserTreeConstants.JJTTRANSLATIONUNIT);

    includeList = (ASTIncludeFileList)node.jjtGetChild(0);
    try {
      includeListCST = (ASTIncludeFileList)includeList.clone();
    } catch (CloneNotSupportedException e) {
      includeListCST = includeList;
    }
    includeListTDD = new
      ASTIncludeFileList(ADLCParserTreeConstants.JJTINCLUDEFILELIST);
    addInclude(includeListTDD, "ADLTest.h");

    // visit class declaration node, i.e., the last one
    // doing so we skip the TDDUseDeclaration node
    node.jjtGetChild(node.jjtGetNumChildren() - 1).jjtAccept(this, data);

    TDDClass.setName(unitName + "TDD");
    try { TDDClass.setProperty("fileExtension", "c"); }
    catch(PropertyAccessException e) {}

    // restore former stm context
    SimpleNode.stm.setContext(savedStmContext);

    return TDDClass;
  }

  /**
    * Visit pattern for a node of type ASTTDDClassDecl.
    * <P><B>Algorithm</B><OL>
    * <LI>Creates an ASTClassDeclaration node for the class
    * "&#60;tddclass name&#62;TDD". This class has an implementation child
    * for interface "&#60;tddclass name&#62;_CST".
    * <LI>The input node children that correspond to method declarations
    * are added to the created node while the others are just visited
    * (they don't return anything but create the public handles datasets,
    * factories and directives).
    * <LI>Calls makeConstantsInterface to create the interface
    * "&#60;tddclass name&#62;_CST".
    * <LI>Returns the created node.
    * </OL>
    * @param node the ASTTDD_ClassDeclaration node to visit
    * @param data unused
    * @return the created node (ASTClassDeclaration)
    */
  public Object visit(ASTTDD_AnnotatedDeclaration node, Object data) {

    if (debug)
      System.out.println("visiting ASTTDD_ClassDeclaration");

    unitName = node.getName();

    addInclude(includeListTDD, unitName + "_CST.h");

    // update tdd classes names list
    tddClassNames.addElement(unitName + "TDD");

    // update constant classes names list
    cstClassNames.addElement(unitName + "_CST");


    parentChild(TDDClass, includeListTDD, 0);

    makeConstantsClass();

    int numChildren = node.jjtGetNumChildren();

    // jump over all TDD_UseDeclaration and Name until the
    // first TDD_Declaration
    int firstDeclarationChild = 0;
    while (! (node.jjtGetChild(firstDeclarationChild)
	   instanceof ASTTDD_Declaration)) {
      firstDeclarationChild++;
    }

    for (int i = firstDeclarationChild; i < numChildren; i++) {
      node.jjtGetChild(i) // the TDD_Declaration child
	.jjtGetChild(0) // the only child of the TDD_Declaration
	.jjtAccept(this, data);
    }

    return null;
  }

  /**
    * Visit pattern for a node of type ASTTDDDatasetDecl.
    * <BR>Creates a CDatasetBuilder with as parameters the current node
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current dataset definition, and adds this field in the "datasets" handle.
    * @see CDatasetBuilder
    * @param node the ASTTDDDatasetDecl node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDD_DatasetDeclaration node, Object data) {

    if (debug)
      System.out.println("visiting ASTTDDDatasetDeclaration");

    CDatasetBuilder jdb =
      new CDatasetBuilder(node, unitName, includeList);

    // update tdd dataset name list
    tddDatasetNames.addElement(jdb.getClassName());

    datasets.addElement(jdb.compilUnit);
    return null;
  }


  /**
    * Visit pattern for a node of type ASTTDDFactoryDef.
    * <BR>Creates a CFactoryBuilder with as parameters the current node
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current factory definition, and adds this field in the "factories" handle.
    * @see CFactoryBuilder
    * @param node the ASTTDDFactoryDef node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDD_FactoryDefinition node, Object data) {

    if (debug)
      System.out.println("visiting ASTTDD_FactoryDeclaration");

    CFactoryBuilder jfb =
      new CFactoryBuilder(node, unitName, includeList);

    // update tdd factory name list
    tddFactoryNames.addElement(jfb.getClassName());    

    factories.addElement(jfb.compilUnit);

    return null;
  }


  /**
    * Visit pattern for a node of type ASTTDDTestDirective.
    * <BR>Creates a CDirectiveBuilder with as parameters the current node,
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current directive definition, and adds this field in the "directives" handle.
    * @see CDirectiveBuilder
    * @param node the ASTTDDTestDirective node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDD_TestDirective node, Object data) {

    if (debug)
      System.out.println("visiting ASTTDD_TestDirective");

    CDirectiveBuilder jdb = new CDirectiveBuilder(node, unitName,
						  includeList);

    // update tdd test directive names list
    tddTestDirectiveNames.addElement(jdb.getClassName());

    directives.addElement(jdb.compilUnit);

    // update list of ACO names
    Enumeration e = jdb.getACONames().elements();
    while ( e.hasMoreElements() ) {
      String acoName = (String)e.nextElement();
      if ( ! acoNames.contains(acoName) ) {
	acoNames.addElement(acoName);
      }
    }

    return null;
  }
  

  /**
    * Visit pattern for a node of type ASTTDDFieldDeclaration.
    * <BR>Creates Java field declaration that corresponds to the TDD field
    * declaration, and adds it to the "constants" handle.
    * @see #makeConstantsClass
    * @param node the ASTTDDFieldDeclaration node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDD_FieldDeclaration node, Object data) {

    if (debug)
      System.out.println("visiting ASTTDD_FieldDeclaration");

    ASTExternalDeclaration externalDecl;
    ASTDeclaration def;
    ASTDeclarationSpecifiers defSpec = null;
    int nbChildren = constantsClass.jjtGetNumChildren();

    for (int i = 1; i < node.jjtGetNumChildren(); i += 2) {
      externalDecl = new
        ASTExternalDeclaration(ADLCParserTreeConstants.JJTEXTERNALDECLARATION);
      try {
        defSpec = (ASTDeclarationSpecifiers)
          ((ASTDeclarationSpecifiers)node.jjtGetChild(0)).clone();
      } catch (CloneNotSupportedException e) {}

      // Fix up the name of the declared field
      Node decl = node.jjtGetChild(i);
      while (!(decl.jjtGetChild(0) instanceof ASTName)) {
	while (!(decl.jjtGetChild(0) instanceof ASTDirectDeclarator))
	  decl = decl.jjtGetChild(1);
	decl = decl.jjtGetChild(0);
      }
      ASTName declName = (ASTName) decl.jjtGetChild(0);
      declName.setName(unitName + "_" + declName.getName());

      def = new DeclarationBuilder
        ( CTypeAux.makeStatic(defSpec),
          new InitDeclaratorListBuilder
          ( new InitDeclaratorBuilder
            ( node.jjtGetChild(i),
              (Node)node.jjtGetChild(i+1).jjtAccept(this, data)))
          .generatedAST())
        .generatedAST();

      parentChild(externalDecl, def, 0);
      parentChild(constantsClass, externalDecl, nbChildren++);
    }
    return null;
  }
  
  
  /**
    * Visit pattern for a node of type Name.
    * <P><B>Algorithm</B><UL>
    * <li>Gets defining tdd class name
    * <LI><B>IF</B> the name is not a dataset name, then it is a
    *    constant:<ul>
    * <li>Sets name to
    *    <samp>&lt;defining tdd class name>_CST.&lt;constant name></samp>
    * </UL></UL>
    * @see CBuilder#makeQualifiedName(String, String, String)
    * @param node the Name node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTName node, Object data) {
    if (debug)
      System.out.println("visiting Name : " + node.getName());

    String nameKind = null;
    String TDDClassName = null;
    try {
      TDDClassName = (String)node.getProperty("enclosingConstructName");
      nameKind = (String)node.getProperty("astNameKind");
    }
    catch(PropertyAccessException e) { }

    if (nameKind != null && nameKind.equals("LAttribute")) {  // Field
      if (TDDClassName != null && !TDDClassName.equals("")) { // TDD Field
	// ANSI C constant expressions are a real nightmare, just
	// issue a warning
	warningMessage(node.getName()
		       + " may not be valid in a constant expression!",
		       node.getLineNumber());
 	addInclude(includeListCST, TDDClassName + "_CST.h");
 	node.setName(TDDClassName + "_" + node.getName());
      }
    }

    return node;
  }


  /**
    * Visit pattern for a node of type ASTFunctionDefinition.
    * <BR>Returns the node unchanged except for last child (ASTCompoundStatement)
    * which is visited by an RemoveADLVisitor to transform its casual
    * ADL/ADL_new expressions.
    * @see ORG.opengroup.adl.translators.c.RemoveADLVisitor
    * @param node the ASTFunctionDefinition node to visit
    * @param data unused
    * @return the transformed node
    */
  public Object visit(ASTFunctionDefinition node, Object data) {

    if (debug)
      System.out.println("visiting ASTFunctionDefinition");

    int bodyIndex = node.jjtGetNumChildren()-1;

    RemoveADLVisitor v = new RemoveADLVisitor();
    node.jjtGetChild(bodyIndex).jjtAccept(v, null);
    Vector inclACO = v.getACONames();
    for (Enumeration e = inclACO.elements(); e.hasMoreElements(); ) {
      addInclude(includeListTDD, (String)e.nextElement() + ".c");
    }

    parentChild(TDDClass, node, TDDClass.jjtGetNumChildren());

    Enumeration e = v.getACONames().elements();
    while ( e.hasMoreElements() )
    {
       String acoName = (String)e.nextElement();
       if ( ! acoNames.contains(acoName) )
       {
          acoNames.addElement(acoName);
       }
    }

    return null;
  }


  /**
    * Creates the AST for interface "&#60;tddclass name&#62;_CST", that
    * gathers the tdd field declarations stored in the "constants" handle.
    * This interface will be implemented by all created classes.
    * @see #visit(ORG.opengroup.parsers.adljava.ASTTDD_FieldDeclaration, Object)
    */
  private void makeConstantsClass() {

    if (debug)
      System.out.println("makeConstantsInterface");

    constantsClass = new
      ASTTranslationUnit(ADLCParserTreeConstants.JJTTRANSLATIONUNIT);
    constantsClass.setName(unitName + "_CST");
    try { constantsClass.setProperty("fileExtension", "h"); }
    catch(PropertyAccessException e) {}

    parentChild(constantsClass, includeListCST, 0);
  }


  public void addInclude(ASTIncludeFileList incList, String inclFile) {

    if (debug)
      System.out.println("addInclude: " + inclFile);

    ASTIncludeFileDeclaration incl = new
      ASTIncludeFileDeclaration(ADLCParserTreeConstants.JJTINCLUDEFILEDECLARATION);
    incl.setName("\"" + inclFile + "\"");
    parentChild(incList, incl, incList.jjtGetNumChildren());
  }


  /**
    * Print an error message.
    * @param msg the error message
    * @param lineNumber the line number of the error in the source file
    */
  static public void errorMessage(String msg, int lineNumber) {

    if (lineNumber == 0) {
      System.err.println(unitName + ": Error: " + msg);
    }
    else {
      System.err.println(unitName + ".tdd:" + lineNumber + ": Error: "
                         + msg);
    }
    throw new Error();
  }


  /**
    * Print a warning message.
    * @param msg the warning message
    * @param lineNumber the line number of the error in the source file
    */
  static public void warningMessage(String msg, int lineNumber) {

    if (lineNumber == 0) {
      System.err.println(unitName + ": Warning: " + msg);
    }
    else {
      System.err.println(unitName + ".tdd:" + lineNumber + ": Warning: "
			 + msg);
    }
  }

}

// ***********************************************************************
// History Log:
// $Log: CTDDTransformVisitor.java,v $
// Revision 1.4  1998/10/14 08:17:28  mallet
// Bugs fixed: pr/155, pr/156: removed useless index of anonymous test
// directives.
//
// Revision 1.3  1998/08/05 16:42:08  mallet
// Fixed returned type for all declared objects from, e.g.,
// FunctionDefinition to ExternalDeclaration.
// Made all generated functions and constants static.
//
// Revision 1.2  1998/07/30 08:39:37  mallet
// Made CTDDTransformVisitor inherit from CGeneric instead of
// ADLCGenericVisitor: required because of the visit of the initializer
// part of a TDD_FieldDeclaration.
// Changed name of constant file from <unit>_CST.c to <unit>_CST.h.
// Improved visit method for TDD_AnnotatedDeclaration: skip all
// TDD_ImportDeclaration and TDD_UseDeclaration nodes and loop on
// TDD_Declaration nodes.
// Fixed visit method for TDD_FieldDeclaration: fix up the declared field
// name, visit the initializer part of the declaration and return an
// ExternalDeclaration.
// Replaced IdExpression with Name.
// Fixed visit method for Name: when the visited name is that of a TDD
// field, emit a warning since it may not be valid in a constant
// expression (we've decided not to implement the ANSI C standard
// constant expression verification).
// Cosmetic changes to methods errorMessage() and warningMessage().
//
// Revision 1.1  1998/07/03 10:07:34  bareau
// Current (incomplete) version of the C/TDD translator.
//
//
// ***********************************************************************
