// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/CompoundStatementBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/06/29 11:49:00 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        CompoundStatementBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C Compound statement
  * @author Cyrille Bareau
  */

public class CompoundStatementBuilder {

  /** Root of the generated tree */
  private ASTCompoundStatement root;


  /**
    * Creates a C compound statement that is a statement
    * @param expr The expression node (AST)
    */
  public CompoundStatementBuilder (Node expr) {

    root = new
      ASTCompoundStatement(ADLCParserTreeConstants.JJTCOMPOUNDSTATEMENT);

    ADLCGenericVisitor.parentChild(root,
                                   new StatementBuilder(expr, true)
                                   .generatedAST(),
                                   0);
  }


  /**
    * Creates a C compound statement that is a declaration
    * @param expr The declaration node (AST)
    */
  public CompoundStatementBuilder (ASTDeclaration expr) {

    root = new
      ASTCompoundStatement(ADLCParserTreeConstants.JJTCOMPOUNDSTATEMENT);

    ADLCGenericVisitor.parentChild(root, expr, 0);

  }


  /**
    * @return the root of the generated tree
    */
  public ASTCompoundStatement generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: CompoundStatementBuilder.java,v $
// Revision 1.2  1998/06/29 11:49:00  bareau
// Enhanced some builders.
//
// Revision 1.1  1998/06/23 13:39:45  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:11  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
