// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/DeclarationBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/06/29 12:52:19 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        DeclarationBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C declaration
  * (Declaration).
  * @author Cyrille Bareau
  */

public class DeclarationBuilder {

  /** Root of the generated tree */
  private ASTDeclaration root;

  /**
    * Creates a declaration with 2 children.
    * @param lhs First child is declaration specifiers.
    * @param rhs Second child can be Init declarator,
    * declarator or direct declarator.
    */
  public DeclarationBuilder (ASTDeclarationSpecifiers lhs, Node rhs) {

    root = new
      ASTDeclaration(ADLCParserTreeConstants.JJTDECLARATION);
    ADLCGenericVisitor.parentChild(root, lhs, 0);
    ADLCGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a non-initialized declaration with 2 children.
    * @param lhs The child of declaration specifiers.
    * @param rhs The second child of the declaration.
    */
  public DeclarationBuilder (Node lhs, Node rhs) {

    root = new
      ASTDeclaration(ADLCParserTreeConstants.JJTDECLARATION);
    ASTDeclarationSpecifiers declSpec = 
      new ASTDeclarationSpecifiers(ADLCParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCGenericVisitor.parentChild(declSpec, lhs, 0);
    ADLCGenericVisitor.parentChild(root, declSpec, 0);
    ADLCGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a non-initialized declaration with 2 children.
    * @param type The qualified type, child of declaration specifiers.
    * @param rhs The second child of the declaration.
    */
  public DeclarationBuilder (String type, Node rhs) {

    root = new
      ASTDeclaration(ADLCParserTreeConstants.JJTDECLARATION);

    ADLCGenericVisitor.parentChild(root,
                                   ADLCGenericVisitor.makeBuiltinType(type),
                                   0);
    ADLCGenericVisitor.parentChild(root, rhs, 1);
  }

  /**
    * @return the root of the generated tree
    */
  public ASTDeclaration generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DeclarationBuilder.java,v $
// Revision 1.2  1998/06/29 12:52:19  bareau
// Enhanced declaration builders.
//
// Revision 1.1  1998/06/23 13:39:45  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:11  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
