// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/DeclaratorBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/07/28 16:16:50 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        DeclaratorBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C declarator
  * (Declarator).
  * @author Cyrille Bareau
  */

public class DeclaratorBuilder {

  /** Root of the generated tree */
  private ASTDeclarator root;

  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, ASTDirectDeclarator dirDecl) {

    root = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ASTDeclarator decl = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ptr, 0);
    ADLCGenericVisitor.parentChild(root, decl, 1);
    ADLCGenericVisitor.parentChild(decl, dirDecl, 0);
  }


  /**
    * Creates a declarator.
    * @param ptr1 The 1st pointer of the declarator.
    * @param ptr1 The snd pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr1, ASTPtrOperator ptr2,
                            ASTDirectDeclarator decl) {

    this(ptr1, new DeclaratorBuilder(ptr2, decl).generatedAST());
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, ASTDeclarator decl) {

    root = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ptr, 0);
    ADLCGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, String declId) {

    root = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);
    ASTDeclarator decl = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ptr, 0);
    ADLCGenericVisitor.parentChild(root, decl, 1);
    ADLCGenericVisitor.parentChild(decl, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(declId),
                                   0);
  }


  public DeclaratorBuilder (String decl) {

    root = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root,
                                   new DirectDeclaratorBuilder(decl).generatedAST(),
                                   0);
  }


  public DeclaratorBuilder (Node node) {

    root = new
      ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, node, 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DeclaratorBuilder.java,v $
// Revision 1.4  1998/07/28 16:16:50  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.3  1998/07/03 10:07:34  bareau
// Current (incomplete) version of the C/TDD translator.
//
// Revision 1.2  1998/06/29 12:52:19  bareau
// Enhanced declaration builders.
//
// Revision 1.1  1998/06/23 13:39:45  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:12  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
