// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/DirectDeclaratorBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:16:51 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        DirectDeclaratorBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C directDeclarator
  * (DirectDeclarator).
  * @author Cyrille Bareau
  */

public class DirectDeclaratorBuilder {

  /** Root of the generated tree */
  private ASTDirectDeclarator root;


  /**
    * Creates a direct declarator.
    * @param first First child of the direct declarator.
    * @param second Second child of the direct declarator.
    */
  public DirectDeclaratorBuilder (Node first, Node second) {

    root = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, first, 0);
    ADLCGenericVisitor.parentChild(root, second, 1);
  }


  /**
    * Creates a direct declarator.
    * @param id The qualified id, child of the direct declarator.
    * @param declSuff The declaration suffixes.
    */
  public DirectDeclaratorBuilder (String id, Node declSuff) {

    root = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(id), 0);
    ADLCGenericVisitor.parentChild(root, declSuff, 1);
  }


  /**
    * Creates a direct declarator.
    * @param id String to build the qualified id, child of the direct declarator.
    */
  public DirectDeclaratorBuilder (String id) {

    root = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(id), 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDirectDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DirectDeclaratorBuilder.java,v $
// Revision 1.2  1998/07/28 16:16:51  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.1  1998/06/23 13:39:46  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:12  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
