// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/DotAccessSuffixBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:16:51 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        DotAccessSuffixBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C dot access suffix:
  *    "."  Name  [ ArgumentList ]
  * @author Cyrille Bareau
  */

public class DotAccessSuffixBuilder {

  /** Root of the generated tree */
  private ASTDotAccessSuffix root;


  /**
    * Creates a dot access suffix composed of 
    * "." and an expression
    * @param expr The expression
    */
  public DotAccessSuffixBuilder (Node expr) {

    root = new
      ASTDotAccessSuffix(ADLCParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCGenericVisitor.parentChild(root, expr, 0);
  }


  /**
    * Creates a dot access suffix composed of 
    * "." and an Id Expression
    * @param expr An id expression
    */
  public DotAccessSuffixBuilder (ASTName expr) {

    root = new
      ASTDotAccessSuffix(ADLCParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCGenericVisitor.parentChild(root, expr, 0);
  }

  /**
    * Creates a dot access suffix composed of 
    * "." and an Id Expression
    * @param expr The name of the id expression 
    */
  public DotAccessSuffixBuilder (String expr) {

    root = new
      ASTDotAccessSuffix(ADLCParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(expr), 0);
  }

  /**
    * Creates a dot access suffix composed of 
    * ".", an Name and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public DotAccessSuffixBuilder (Node expr, Node arg) {

    root = new
      ASTDotAccessSuffix(ADLCParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCGenericVisitor.parentChild(root, expr, 0);
    ADLCGenericVisitor.parentChild(root, arg, 1);
  }

  /**
    * Creates a dot access suffix composed of 
    * ".", an Name and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public DotAccessSuffixBuilder (String expr, Node arg) {

    root = new
      ASTDotAccessSuffix(ADLCParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(expr), 0);
    ADLCGenericVisitor.parentChild(root, arg, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDotAccessSuffix generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DotAccessSuffixBuilder.java,v $
// Revision 1.2  1998/07/28 16:16:51  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.1  1998/06/23 13:39:46  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:12  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
