// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/FunctionDeclaratorBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:19:08 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        FunctionDeclaratorBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 25, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C Function declarator
  * (Declarator).
  * @author Cyrille Bareau
  */

public class FunctionDeclaratorBuilder {

  /** Root of the generated tree */
  private ASTFunctionDeclarator root;


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public FunctionDeclaratorBuilder (ASTPtrOperator ptr, String declId) {

    root = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDeclarator decl = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDirectDeclarator directDecl = new
      ASTFunctionDirectDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, ptr, 0);
    ADLCGenericVisitor.parentChild(root, decl, 1);
    ADLCGenericVisitor.parentChild(decl, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(declId),
                                   0);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public FunctionDeclaratorBuilder (String declId) {

    root = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDirectDeclarator directDecl = new
      ASTFunctionDirectDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDIRECTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(declId),
                                   0);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public FunctionDeclaratorBuilder (ASTPtrOperator ptr, String declId,
                                    ASTParameterDeclarationList params) {

    root = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDeclarator decl = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDirectDeclarator directDecl = new
      ASTFunctionDirectDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDIRECTDECLARATOR);
    ASTParameterList plist = new
      ASTParameterList(ADLCParserTreeConstants.JJTPARAMETERLIST);

    ADLCGenericVisitor.parentChild(root, ptr, 0);
    ADLCGenericVisitor.parentChild(root, decl, 1);
    ADLCGenericVisitor.parentChild(decl, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(declId),
                                   0);
    ADLCGenericVisitor.parentChild(directDecl, plist, 1);
    ADLCGenericVisitor.parentChild(plist, params, 0);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public FunctionDeclaratorBuilder (String declId, ASTParameterDeclarationList params) {

    root = new
      ASTFunctionDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDECLARATOR);
    ASTFunctionDirectDeclarator directDecl = new
      ASTFunctionDirectDeclarator(ADLCParserTreeConstants.JJTFUNCTIONDIRECTDECLARATOR);
    directDecl.setName(declId);

    ASTParameterList plist = new
      ASTParameterList(ADLCParserTreeConstants.JJTPARAMETERLIST);

    ADLCGenericVisitor.parentChild(root, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(declId),
                                   0);
    ADLCGenericVisitor.parentChild(directDecl, plist, 1);
    ADLCGenericVisitor.parentChild(plist, params, 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTFunctionDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: FunctionDeclaratorBuilder.java,v $
// Revision 1.2  1998/07/28 16:19:08  mallet
// Replaced calls to makeIdExpression by makeName.
// Fixed bug in constructors: the parameter list child was not attached
// to its parent at the right position.
//
// Revision 1.1  1998/07/03 10:07:35  bareau
// Current (incomplete) version of the C/TDD translator.
//
//
// ***********************************************************************
