// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/InitDeclaratorBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:16:52 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        InitDeclaratorBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C init declarator
  * (InitDeclarator).
  * @author Cyrille Bareau
  */

public class InitDeclaratorBuilder {

  /** Root of the generated tree */
  private ASTInitDeclarator root;


  /**
    * Creates an initialized declarator.
    * @param  decl The first child is declarator or direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(Node decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);

    ADLCGenericVisitor.parentChild(root, decl, 0);
    ADLCGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The first child is declarator or direct declarator.
    * @param init The initializer -) an Id expression.
    */
  public InitDeclaratorBuilder(Node decl, String init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);

    ADLCGenericVisitor.parentChild(root, decl, 0);
    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(init), 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The declarator (with pointer operator).
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(ASTDeclarator decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);

    ADLCGenericVisitor.parentChild(root, decl, 0);
    ADLCGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The declarator (with pointer operator).
    * @param init The initializer -) an Id expression.
    */
  public InitDeclaratorBuilder(ASTDeclarator decl, String init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);

    ADLCGenericVisitor.parentChild(root, decl, 0);
    ADLCGenericVisitor.parentChild(root, ADLCGenericVisitor.makeName(init), 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(ASTDirectDeclarator decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);
    ASTDeclarator declTor =
      new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, declTor, 0);
    ADLCGenericVisitor.parentChild(declTor, decl, 0);
    ADLCGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The direct declarator.
    * @param init The initializer -) an id expression.
    */
  public InitDeclaratorBuilder(ASTDirectDeclarator decl, String init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);
    ASTDeclarator declTor =
      new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, declTor, 0);
    ADLCGenericVisitor.parentChild(declTor, decl, 0);
    ADLCGenericVisitor.parentChild(root,
                                   ADLCGenericVisitor.makeName(init),
                                   1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The name of the direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(String decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);
    ASTDeclarator declTor =
      new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, declTor, 0);
    ADLCGenericVisitor.parentChild(declTor, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(decl),
                                   0);
    ADLCGenericVisitor.parentChild(root, init, 1);
  }

  /**
    * Creates an initialized declarator.
    * @param  decl The name of the direct declarator.
    * @param init The initializer -) an id expression.
    */
  public InitDeclaratorBuilder(String decl, String init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);
    ASTDeclarator declTor =
      new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, declTor, 0);
    ADLCGenericVisitor.parentChild(declTor, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(decl),
                                   0);
    ADLCGenericVisitor.parentChild(root,
                                   ADLCGenericVisitor.makeName(init),
                                   1);
  }

  /**
    * Creates an initialized declarator.
    * @param  decl The direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(Node init) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);

    ADLCGenericVisitor.parentChild(root, init, 0);
  }

  /**
    * Creates an initialized declarator.
    * @param  decl The name of the direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(String decl) {

    root = new
      ASTInitDeclarator(ADLCParserTreeConstants.JJTINITDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCParserTreeConstants.JJTDIRECTDECLARATOR);
    ASTDeclarator declTor =
      new ASTDeclarator(ADLCParserTreeConstants.JJTDECLARATOR);

    ADLCGenericVisitor.parentChild(root, declTor, 0);
    ADLCGenericVisitor.parentChild(declTor, directDecl, 0);
    ADLCGenericVisitor.parentChild(directDecl,
                                   ADLCGenericVisitor.makeName(decl),
                                   0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTInitDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: InitDeclaratorBuilder.java,v $
// Revision 1.2  1998/07/28 16:16:52  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.1  1998/06/23 13:39:47  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:12  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
