// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/InitDeclaratorListBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/06/23 13:39:47 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        InitDeclaratorListBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C list of initializers
  * @author Cyrille Bareau
  */

public class InitDeclaratorListBuilder {

  /** Root of the generated tree */
  private ASTInitDeclaratorList root;

  /** index of initializers */
  private int index;


  /**
    * Creates an empty list of initializers
    */
  public InitDeclaratorListBuilder () {

    root = new
      ASTInitDeclaratorList(ADLCParserTreeConstants.JJTINITDECLARATORLIST);
  }


  /**
    * Creates a list of 1 initializer
    * @param init The initial initializer
    */
  public InitDeclaratorListBuilder (ASTInitDeclarator init) {

    this();
    ADLCGenericVisitor.parentChild(root, init, 0);
    index = 1;
  }


  /**
    * Creates a list of 1 initializer
    * @param init The initial initializer builder
    */
  public InitDeclaratorListBuilder (InitDeclaratorBuilder init) {

    this();
    ADLCGenericVisitor.parentChild(root, init.generatedAST(), 0);
    index = 1;
  }


  /**
    * Adds a initializer to the initializer declration list 
    * @param arg The new initializer declaration
    * @return this (so that it is possible to chain-adding
    * new initializers)
    */
  public InitDeclaratorListBuilder addInitializer (ASTInitDeclarator init) {

    ADLCGenericVisitor.parentChild(root, init, index++);
    return this;
  }


  /**
    * Adds a initializer to the initializer declaration list
    * @param arg The new initializer declaration builder
    * @return this (so that it is possible to chain-adding
    * new initializers)
    */
  public InitDeclaratorListBuilder addInitializer (InitDeclaratorBuilder init) {

    ADLCGenericVisitor.parentChild(root, init.generatedAST(), index++);
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTInitDeclaratorList generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: InitDeclaratorListBuilder.java,v $
// Revision 1.1  1998/06/23 13:39:47  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:13  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
