// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/NotBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/06/23 13:39:47 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        NotBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.parsers.PropertyAccessException;

/**
  * Creates a tree for a C "!" expression
  * (UnaryExpression).
  * @author Cyrille Bareau
  */

public class NotBuilder {

  /** Root of the generated tree */
  private ASTUnaryOperatorExpression root;


  /**
    * Creates a <I>not</I> of an expression.
    * @param node The input expression
    * @param parenth Flag to precise if the input expression
    * must be parenthetized (default yes)
    */
  public NotBuilder (Node node, boolean parenth) {

    root = new
      ASTUnaryOperatorExpression(ADLCParserTreeConstants.JJTUNARYOPERATOREXPRESSION);
    try { root.setProperty("operator", "!"); }
    catch(PropertyAccessException e) {}

    if (parenth) {
      ASTParentheizedExpression parenthnode = new
        ASTParentheizedExpression(ADLCParserTreeConstants.JJTPARENTHEIZEDEXPRESSION);

      ADLCGenericVisitor.parentChild(root, parenthnode, 0);
      ADLCGenericVisitor.parentChild(parenthnode, node, 0);
    }
    else {
      ADLCGenericVisitor.parentChild(root, node, 0);
    }
  }


  /**
    * Creates a <I>not</I> of an expression (with parentheses).
    * @param node The input expression
    */
  public NotBuilder (Node node) {

    this(node, true);
  }
      

  /**
    * @return the root of the generated tree
    */
  public ASTUnaryOperatorExpression generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: NotBuilder.java,v $
// Revision 1.1  1998/06/23 13:39:47  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:13  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************




