// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/ParameterDeclarationBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/06/23 13:39:48 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        ParameterDeclarationBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C parameter declaration
  * @author Cyrille Bareau
  */

public class ParameterDeclarationBuilder {

  /** Root of the generated tree */
  private ASTParameterDeclaration root;


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTBuiltinTypeSpecifier type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCGenericVisitor.parentChild(root, declSpec, 0);
    ADLCGenericVisitor.parentChild(declSpec, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTName type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCGenericVisitor.parentChild(root, declSpec, 0);
    ADLCGenericVisitor.parentChild(declSpec, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter direct declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDirectDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter abstract declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTAbstractDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
  }



  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
    ADLCGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter direct declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDirectDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
    ADLCGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter abstract declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTAbstractDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCGenericVisitor.parentChild(root, type, 0);
    ADLCGenericVisitor.parentChild(root, param, 1);
    ADLCGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTParameterDeclaration generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ParameterDeclarationBuilder.java,v $
// Revision 1.1  1998/06/23 13:39:48  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:14  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
