// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/ParameterDeclarationListBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/06/23 13:39:48 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        ParameterDeclarationListBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C list of parameters
  * @author Cyrille Bareau
  */

public class ParameterDeclarationListBuilder {

  /** Root of the generated tree */
  private ASTParameterDeclarationList root;

  /** index of parameters */
  private int index;


  /**
    * Creates an empty list of parameters
    */
  public ParameterDeclarationListBuilder () {

    root = new
      ASTParameterDeclarationList(ADLCParserTreeConstants.JJTPARAMETERDECLARATIONLIST);
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial parameter
    */
  public ParameterDeclarationListBuilder (ASTParameterDeclaration param) {

    this();
    ADLCGenericVisitor.parentChild(root, param, 0);
    index = 1;
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial parameter builder
    */
  public ParameterDeclarationListBuilder (ParameterDeclarationBuilder param) {

    this();
    ADLCGenericVisitor.parentChild(root, param.generatedAST(), 0);
    index = 1;
  }


  /**
    * Adds a parameter to the parameter declration list 
    * @param arg The new parameter declaration
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public ParameterDeclarationListBuilder addParameter (ASTParameterDeclaration param) {

    ADLCGenericVisitor.parentChild(root, param, index++);
    return this;
  }


  /**
    * Adds a parameter to the parameter declaration list
    * @param arg The new parameter declaration builder
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public ParameterDeclarationListBuilder addParameter (ParameterDeclarationBuilder param) {

    ADLCGenericVisitor.parentChild(root, param.generatedAST(), index++);
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTParameterDeclarationList generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ParameterDeclarationListBuilder.java,v $
// Revision 1.1  1998/06/23 13:39:48  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:14  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
