// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/PostfixExprBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:16:52 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        PostfixExprBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;

/**
  * Creates a tree for a C postfix expression
  * @author Cyrille Bareau
  */

public class PostfixExprBuilder {

  /** Root of the generated tree */
  private ASTPostfixExpression root;

  /** index of suffixes */
  private int index;


  /**
    * Creates an <U>incomplete</U> postfix expression composed of a
    * prefix only.
    * It is <U>necessary</U> to add a suffix afterwards.
    * @see #addSuffix
    * @param prefix The prefix of the postfix expression
    */
  public PostfixExprBuilder (Node prefix) {

    root = new
      ASTPostfixExpression(ADLCParserTreeConstants.JJTPOSTFIXEXPRESSION);

    ADLCGenericVisitor.parentChild(root, prefix, 0);
    index = 1;
  }


  /**
    * Adds a suffix to the primary expression
    * @param suffix The new suffix
    * @return this (so that it is possible to chain-adding
    * new suffixes)
    */
  public PostfixExprBuilder addSuffix (Node suffix) {

    ADLCGenericVisitor.parentChild(root, suffix, index++);
    return this;
  }


  /**
    * Creates a postfix expression composed of 
    * an expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" Name [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (Node first, Node second) {

    root = new
      ASTPostfixExpression(ADLCParserTreeConstants.JJTPOSTFIXEXPRESSION);

    ADLCGenericVisitor.parentChild(root, first, 0);
    ADLCGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * an id expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" Name [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (ASTName first, Node second) {

    root = new
      ASTPostfixExpression(ADLCParserTreeConstants.JJTPOSTFIXEXPRESSION);

    ADLCGenericVisitor.parentChild(root, first, 0);
    ADLCGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * a id expression build form a string
    * and a Reference Access Builder : "->" Name [ArgumentList]
    * @param first A string to build the first child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (String first, Node second) {

    root = new
      ASTPostfixExpression(ADLCParserTreeConstants.JJTPOSTFIXEXPRESSION);

    ADLCGenericVisitor.parentChild(root,
                                   ADLCGenericVisitor.makeName(first),
                                   0);
    ADLCGenericVisitor.parentChild(root, second, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTPostfixExpression generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: PostfixExprBuilder.java,v $
// Revision 1.2  1998/07/28 16:16:52  mallet
// Replaced calls to makeIdExpression by makeName.
//
// Revision 1.1  1998/06/23 13:39:48  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:14  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
