// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/RemoveADLVisitor.java,v $
// $Revision: 1.2 $
// $Date: 1998/07/28 16:22:45 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.c
// NAME:        RemoveADLVisitor.java
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 12, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.symboltable.*;
import com.objectspace.jgl.UnaryPredicate;
import java.util.*;

/**
 * Transformation visitor.  Must change an ADL expression.
 * <BR>
 * Used in directives and test methods.
 * @see CDirectiveBuilder
 * @author Cyrille Bareau
 */

public class RemoveADLVisitor extends CGeneric {

  /** Names of the effectively used ACO modules. */
  private Vector acoNames = new Vector();

  static private boolean debug = false;
  
  /**
   * Returns the list of used ACO names.
   */
  public Vector getACONames() {
    return acoNames;
  }


  /**
    * Visit pattern for a node of type ASTTDD_ADLExpression.
    * <P><B>Algorithm</B><UL>
    * <li>get the PostfixExpression child
    * <li>get the Name child of the postfix expression
    * <li>try to get the <samp>acoName</samp> property of the name child
    *     to determine the ADL module which defines an annotated function
    * <li>add it to the list of used ADL modules
    * <li>change the name of the name child to:
    *     <samp>"ADL_&lt;ADL module name>ACO_&lt;function name>"</samp>
    * <li>return the postfix expression child
    * </ul>
    *
    * @param node the ASTTDD_ADLExpression to visit.
    * @param data unused.
    * @return the PostfixExpression child.
    */
  public Object visit(ASTTDD_ADLExpression node, Object data)  {

    if (debug)
      System.out.println("visiting ASTTDD_ADLExpression.");

    // the grammar says:
    // TDD_ADLExpression ::= "ADL" "(" Name ArgumentList ")"
    // and creates a PostfixExpression to hold the Name and ArgumentList
    Node postfix = node.jjtGetChild(0); // PostfixExpression
    SimpleNode astName = (SimpleNode)postfix.jjtGetChild(0); // Name
    
    try {
      String acoName = (String)astName.getProperty("acoName") + "ACO";
      // we do have an acoName property set, change the name of the node
      // update aco names
      if ( ! acoNames.contains(acoName) ) {
	acoNames.addElement(acoName);
      }
      astName.setName("ADL_" + acoName + "_" + astName.getName());
    } catch (PropertyAccessException e) { }

    return postfix;
  }

}

// ***********************************************************************
// History Log:
// $Log: RemoveADLVisitor.java,v $
// Revision 1.2  1998/07/28 16:22:45  mallet
// Reimplemented the RemoveADLVisitor according to the modifications in
// the syntax of TDD_ADLExpression and the type check visitor (property
// "acoName" added): when there is an acoName property set in the Name
// grand-child of the TDD_ADLExpression, just change the name of this
// node to "ADL_<acoName>ACO_<name>".
//
// Revision 1.1  1998/07/03 10:07:36  bareau
// Current (incomplete) version of the C/TDD translator.
//
// Revision 1.3  1998/03/10 17:26:45  mallet
// Merged v1.1 with development trunk
//
// Revision 1.2.2.2  1998/03/09 10:28:38  bareau
// Cleaned up the code for error handling (wrote a static method
// "errorMessage" in CTDDTransformVisitor).
//
// Revision 1.2.2.1  1998/03/03 10:59:56  bareau
// Set the debug flag to false.
// Bug fix in CDatasetExpressionVisitor: ASTTDDDatasetSingleton nodes were
// sent to the unparser.
//
// Revision 1.2  1998/02/27 13:07:36  bareau
// Updated copyright notice.
//
// Revision 1.1  1998/02/26 17:50:01  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// ***********************************************************************
