// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/c/StatementBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/06/23 13:39:49 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.c
// NAME:        StatementBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Jun 15, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.c;

import ORG.opengroup.adl.parsers.adlc.*;
import ORG.opengroup.adl.parsers.PropertyAccessException;

/**
  * Creates a tree for a C statement (when children are expressions)
  * @author Cyrille Bareau
  */

public class StatementBuilder {

  /** Root of the generated tree */
  private ASTStatement root;


  /**
    * Creates a C statement having a node expression as child
    * @param expr The postfix expression (AST)
    */
  public StatementBuilder (Node expr, boolean endsWithSemiColon) {

    root = new
      ASTStatement(ADLCParserTreeConstants.JJTSTATEMENT);
    try { root.setProperty("endsWithSemiColon",
                           new Boolean(endsWithSemiColon)); }
    catch(PropertyAccessException e) {}

    ADLCGenericVisitor.parentChild(root, expr, 0);
  }


  /**
    * Creates a C statement that is a postfix expression builder
    * (function call for instance)
    * @param expr The postfix expression (AST)
    */
  public StatementBuilder (PostfixExprBuilder expr) {

    root = new
      ASTStatement(ADLCParserTreeConstants.JJTSTATEMENT);
    try { root.setProperty("endsWithSemiColon", Boolean.TRUE); }
    catch(PropertyAccessException e) {}

    ADLCGenericVisitor.parentChild(root, expr.generatedAST(), 0);
  }


  /**
    * Creates a C statement that is an assignment builder
    * @param assig The assignment builder
    */
  public StatementBuilder (AssignmentBuilder assigB) {

    root = new
      ASTStatement(ADLCParserTreeConstants.JJTSTATEMENT);
    try { root.setProperty("endsWithSemiColon", Boolean.TRUE); }
    catch(PropertyAccessException e) {}

    ADLCGenericVisitor.parentChild(root, assigB.generatedAST(), 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTStatement generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: StatementBuilder.java,v $
// Revision 1.1  1998/06/23 13:39:49  bareau
// First version of the ADL/C translator (ADL part only).
//
// Revision 1.1  1998/06/23 13:27:15  bareau
// First version of the ADL/C translator (ADL part only).
//
//
// ***********************************************************************
