// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1997-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions to use, copy, modify and distribute this software are
// governed by the terms and conditions set forth in the file COPYRIGHT,
// located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/ACOauxiliary.java,v $
// $Revision: 1.15 $
// $Date: 1998/10/15 18:10:46 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        ACOauxiliary.java
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 9, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlcpp.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;
import java.util.Vector;
import java.util.Enumeration;

/**
 * Auxiliary class for the construction of generic parts of code in ACOs.
 * @see ADLCPPTransformVisitor
 * @author Rami KHEMAKHEM
 */

class ACOauxiliary {

  /* Flag for debug mode */
  private final static boolean debug = false;

  /* handles */

  /** Handle for the name of the adl class */
  private String adlClassName;

  /** Handle for the current fct return type */
  private String fctReturnType;

  /** Handle for the current fct type default value*/
  private LType fctTypeDefaultValue;

  /** Handle for the current fct return type (with pointer) */
  private Node fctPointedType;

  /** Name of the current fct */
  private String fctName;

  /** Handle for the parameters of the current fct */
  private LParameter[] fctParams;

  /** Handle for the list of thrown exceptions of the current fct */
  private LType[] fctThrowsList;

  /** The current fct calls uses the "SuperClass::semantics" feature */
  private String SuperSemantics;

  /** The current fct calls uses the "SuperClass::semantics" feature */
  private boolean InheritedSemantics;

  /** The current tested fct is a constructor */
  private boolean isConstructor;

  /** Handle for the "normal" expression */
  private Node normalSpec;

  /** Handle for the "abnormal" expression */
  private Node abnormalSpec;

  /** Handle for ADl super Classes names */
  private Vector superClasses;

  /** Statements for the initialization of the current function */
  private Vector ACO_Initialization;

  /** Statements for the call to the real object fct,
    * catch of exceptions, etc */
  private Vector ACO_CallBlock;

  /** Statements for the finalization of the current fct
    * (e.g. re-throw of exceptions) */
  private Vector ACO_Final;

  /** Statements for the initialization of the assertion code */
  private Vector ACO_AssertionInit;

  /** The ADL transform visitor that uses the current ACO class */
  private ADLCPPTransformVisitor associatedVisitor;


  private boolean firstFct;
  private String excName;
  private ASTStatement setCompoundStmt, stmt1, stmt2, stmt3, stmt4;


  /**
    * Single constructor
    * @param visitor The ADL transform visitor that uses the current ACO class
    */
  ACOauxiliary(ADLCPPTransformVisitor visitor) {

    associatedVisitor = visitor;
    firstFct = true;
    superClasses = new Vector();
  }

  void initializeFct() {

    if (debug) System.out.println("ACOauxiliary: initializeFct ...");

    ACO_Initialization = new Vector();
    ACO_CallBlock = new Vector();
    ACO_Final = new Vector();
    ACO_AssertionInit = new Vector();

    SuperSemantics = null;
    InheritedSemantics = false;
    isConstructor = false;
    fctReturnType = null;
    fctTypeDefaultValue = null;
    fctPointedType = null;
    fctName = null;
    fctParams = null;
    fctThrowsList = null;
    normalSpec = null;
    abnormalSpec = null;
  }

  void setSuperClasses(String name) {
    superClasses.addElement(name);
  }

  String getSuperClasses() {
    String str = "";
    for (Enumeration e=superClasses.elements();
	 e.hasMoreElements();) {
      if (str.equals(""))
	str = str + (String)e.nextElement();
      else
	str = str + "::" + (String)e.nextElement();
    }
    return str;
  }

  void setSuperSemantics(String s) {
    SuperSemantics = s;
  }

  String getSuperSemantics() {
    return SuperSemantics;
  }

  void setNormalSpec(Node s) {
    normalSpec = s;
  }

  Node getNormalSpec() {
    return normalSpec;
  }

  void setAbnormalSpec(Node s) {
    abnormalSpec = s;
  }

  Node getAbnormalSpec() {
    return abnormalSpec;
  }

  void setIsConstructor(boolean s) {
    isConstructor = s;
    if (isConstructor) {
      fctReturnType = adlClassName;
    }
  }

  boolean getIsConstructor() {
    return isConstructor;
  }

  void setFctReturnType(String type) {
    fctReturnType = type;
  }

  String getFctReturnType() {
    return fctReturnType;
  }

  void setFctTypeDefaultValue(LType type) {
    fctTypeDefaultValue = type;
  }

  void setFctPointedType(Node type) {
    fctPointedType = type;
  }

  Node getFctPointedType() {
    return fctPointedType;
  }

  void setFctName(String name) {
    fctName = name;
  }

  String getFctName() {
    return fctName;
  }

  void setFctParams(LParameter[] params) {
    fctParams = params;
  }

  LParameter[] getFctParams() {
    return fctParams;
  }

  void setFctThrowsList(LType[] throwslist) {
    fctThrowsList = throwslist;
  }

  void setAdlClassName(String name) {
    adlClassName = name;
  }
  
  String getAdlClassName() {
    return adlClassName;
  }

  String findExc(LType exc) {
    
    String numExc = "";
    for (int j=0; (!(fctThrowsList == null)&&(j< fctThrowsList.length)); j++) {
      if (fctThrowsList[j].equals(exc))
	numExc = "wasThrown_Exc" + ( new Integer(j)).toString();
    }
    if ((fctThrowsList == null) || (numExc == ""))
      numExc = exc.getName();	

    return numExc;
  }

  /**
    * Generates the generic constructor of the ACO class.
    * For instance for class "bank":
    <PRE>
    |  bankACO(bank *b, ADLTest *tc) : ADLObjectACO(b, tc) {}
    </PRE>
    * @return The CtorDefinition AST
    */
  ASTCtorDefinition makeACO_Constructor() {

    if (debug) System.out.println("ACOauxiliary: makeACO_Constructor ...");

    // Created node is function definition without function type: constructor
    ASTCtorDefinition createdNode = new
      ASTCtorDefinition(ADLCPPParserTreeConstants.JJTCTORDEFINITION);

    // The lonely child, function direct declarator, is the declarator name
    ASTCtorDeclarator FctDecltor = new
      ASTCtorDeclarator(ADLCPPParserTreeConstants.JJTCTORDECLARATOR);

    ASTQualifiedId FctName = new
      ASTQualifiedId(ADLCPPParserTreeConstants.JJTQUALIFIEDID);
    FctName.setName(adlClassName + "ACO");


    // second child of CtorDeclarator: parameter declaration list

    // First childs of parameter declarations
    ASTDeclarationSpecifiers declSpec1 = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    ASTDeclarationSpecifiers declSpec2 = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild
      ( declSpec1,
	ADLCPPGenericVisitor.makeQualifiedType(adlClassName),
	0);
    ADLCPPGenericVisitor.parentChild
      ( declSpec2, 
	ADLCPPGenericVisitor.makeQualifiedType("ADLTest"),
	0);

    // Construction of the parameter declaration list with 2 children
    ASTParameterDeclarationList pdl = new ParameterDeclarationListBuilder
      ( new ParameterDeclarationBuilder
	( declSpec1,
	  new DeclaratorBuilder("PointerTo", "b").generatedAST()
	  ).generatedAST()
	).addParameter( new ParameterDeclarationBuilder
			( declSpec2,
			  new DeclaratorBuilder("PointerTo", "tc").generatedAST()
			  ).generatedAST()
			).generatedAST();

    ADLCPPGenericVisitor.parentChild(FctDecltor, FctName, 0);
    ADLCPPGenericVisitor.parentChild(FctDecltor, pdl, 1);

    ADLCPPGenericVisitor.parentChild(createdNode, FctDecltor, 0);

    // bankACO(bank *b, ADLTest *tc) : ADLObjectACO(b, tc) {}
    ASTCtorInitializer ctorInit = new
      ASTCtorInitializer(ADLCPPParserTreeConstants.JJTCTORINITIALIZER);

    ASTSuperclassInit superInit = new
      ASTSuperclassInit(ADLCPPParserTreeConstants.JJTSUPERCLASSINIT);
    ASTQualifiedId superName = new
      ASTQualifiedId(ADLCPPParserTreeConstants.JJTQUALIFIEDID);

    if (getSuperClasses().equals(""))
      superName.setName("ADLObjectACO");
    else 
      superName.setName(getSuperClasses());

    ASTArgumentList args = new ArgumentListBuilder
      ( ADLCPPGenericVisitor.makeIdExpr("b")
	).addArgument(ADLCPPGenericVisitor.makeIdExpr("tc")).generatedAST();
    ADLCPPGenericVisitor.parentChild(superInit, superName, 0);
    ADLCPPGenericVisitor.parentChild(superInit, args, 1);
    ADLCPPGenericVisitor.parentChild(ctorInit, superInit, 0);
    
    ADLCPPGenericVisitor.parentChild(createdNode, ctorInit, 1);
    ASTCompoundStatement ctorBlk = new
      ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCPPGenericVisitor.parentChild(createdNode, ctorBlk, 2);
    return createdNode;
  }


  /**
   * Generates the generic initialization code for each
   * function of an ACO class.
<PRE>
|  bool doReport = finalJournalReporting && debugMode;
|  bool abnormal = false;
|  bool normal = true;
|  bool tddRes;
|  bool wasThrownException = false;
|  void *adl_thrownException = NULL;
</PRE>
   * and in the case of a constructor:
<PRE>
|  skipAssertions |= testContext->skipAssertions;
</PRE>
   * else:
<PRE>
|  skipAssertions |= testContext->skipAssertions || realObject == NULL;
|  bank *bankObject = (bank *)realObject;
</PRE>
   * @param inherits flag when the tested function has inherited semantics.
   * @return The handle on the generated statements (CompoundStatement ASTs)
   */
  Vector makeACO_Initialization(boolean inherits) {

    if (debug) System.out.println("\nACOauxiliary: makeACO_Initialization ...");

    InheritedSemantics = inherits;

    // statements that are common to every function

    // bool doReport = finalJournalReporting && debugMode;
    ASTBinaryLogicalExpr doReportInit = new
      ASTBinaryLogicalExpr(ADLCPPParserTreeConstants.JJTBINARYLOGICALEXPR);
      doReportInit.addOp("&&");

      ADLCPPGenericVisitor.parentChild(doReportInit, ADLCPPGenericVisitor.makeIdExpr("finalJournalReporting"), 0);
      ADLCPPGenericVisitor.parentChild(doReportInit, ADLCPPGenericVisitor.makeIdExpr("debugMode"), 1);
    ACO_Initialization.addElement( new DeclarationBuilder
				   ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
				     new InitDeclaratorBuilder
				     ( new DirectDeclaratorBuilder("doReport").generatedAST(),
				       doReportInit).generatedAST()
				     ).generatedAST());
    // bool abnormal = false;
    ACO_Initialization.addElement( new DeclarationBuilder
				   ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
				     new InitDeclaratorBuilder
				     ( new DirectDeclaratorBuilder("abnormal").generatedAST(),
				     ADLCPPGenericVisitor.makeBoolean("false")
				       ).generatedAST()
                                     ).generatedAST()
				   );
    // bool normal = true;
    ACO_Initialization.addElement( new DeclarationBuilder
				   ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
				     new InitDeclaratorBuilder
				     ( new DirectDeclaratorBuilder("normal").generatedAST(),
				     ADLCPPGenericVisitor.makeBoolean("true")
				       ).generatedAST()
                                     ).generatedAST());
    // bool tddRes = false;
    ACO_Initialization.addElement( new DeclarationBuilder
				   ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
				     new InitDeclaratorBuilder
				     ( new DirectDeclaratorBuilder("tddRes").generatedAST(),
				     ADLCPPGenericVisitor.makeBoolean("false")
				       ).generatedAST()
				     ).generatedAST());
    // bool wasThrownException = false;
    ACO_Initialization.addElement( new DeclarationBuilder
				   ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
				     new InitDeclaratorBuilder
				     ( new DirectDeclaratorBuilder("wasThrownException").generatedAST(),
				     ADLCPPGenericVisitor.makeBoolean("false")
				       ).generatedAST()
				     ).generatedAST());
    // void *adl_thrownException = NULL;
    ACO_Initialization.addElement
      ( new DeclarationBuilder
	( ADLCPPGenericVisitor.makeBuiltinType("void"),
	  new InitDeclaratorBuilder
	  ( new DeclaratorBuilder
	    ( "PointerTo","adl_thrownException").generatedAST(),
	    "NULL").generatedAST()
	  ).generatedAST());
    
    // bool wasThrown_<Exci> = false;
    for (int i=0; (!(fctThrowsList == null)&&(i< fctThrowsList.length)); i++) {
      ACO_Initialization.addElement
	( new DeclarationBuilder
	  ( ADLCPPGenericVisitor.makeBuiltinType("bool"),
	    new InitDeclaratorBuilder
	    ( new DirectDeclaratorBuilder
//Modif	      ( "wasThrown_"+fctThrowsList[i].getName()
	      ( "wasThrown_Exc"+i
		).generatedAST(),
	      ADLCPPGenericVisitor.makeBoolean("false")
	      ).generatedAST()
	    ).generatedAST());
    }

    if (isConstructor) {
      // skipAssertions |= testContext->skipAssertions;
      ACO_Initialization.addElement( new StatementBuilder
				     ( new AssignmentBuilder
				       ( "skipAssertions", "|=",
					 new PostfixExprBuilder
					 ( "testContext",
					   new RefAccessSuffixBuilder("skipAssertions").generatedAST()
					   ).generatedAST()
					 ).generatedAST()
				       ).generatedAST());
    }
    else {
      // skipAssertions |= testContext->skipAssertions || (realObject == NULL);
      ASTBinaryLogicalExpr expr = new
	ASTBinaryLogicalExpr(ADLCPPParserTreeConstants.JJTBINARYLOGICALEXPR);
      expr.addOp("||");

      ASTParenthExpr exp2 = new
	ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);

      ASTBinaryLogicalExpr exp2_1 = new
	ASTBinaryLogicalExpr(ADLCPPParserTreeConstants.JJTBINARYLOGICALEXPR);
      exp2_1.addOp("==");

      ADLCPPGenericVisitor.parentChild(exp2_1, ADLCPPGenericVisitor.makeIdExpr("realObject"), 0);
      ADLCPPGenericVisitor.parentChild(exp2_1, ADLCPPGenericVisitor.makeIdExpr("NULL"), 1);
      ADLCPPGenericVisitor.parentChild(exp2, exp2_1, 0);

      ADLCPPGenericVisitor.parentChild(expr,
					 new PostfixExprBuilder
					 ( "testContext",
					   new RefAccessSuffixBuilder("skipAssertions").generatedAST()
					   ).generatedAST(),
					 0);
      ADLCPPGenericVisitor.parentChild(expr, exp2, 1);

      ACO_Initialization.addElement( new StatementBuilder
				     ( new AssignmentBuilder
				       ( "skipAssertions",
					 "|=", 
					 expr).generatedAST()
				       ).generatedAST());
      

      // <adlClassName> *<adlClassName>Object = (<adlClassName> *)realObject;

      /* Build a TypeName node with 2 children:
       * DeclarationSpecifiers (have a QualifiedType as child)
       * & AbstractDeclarator (have PtrOperator & AbstractDeclarator as children) 
       */
      ASTDeclarationSpecifiers declSpec = new
	ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
      ADLCPPGenericVisitor.parentChild(declSpec,
					 ADLCPPGenericVisitor.makeQualifiedType(adlClassName),
					 0);
      ASTAbstractDeclarator abstract1 = new
	ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
      ASTAbstractDeclarator abstract2 = new
	ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
      ADLCPPGenericVisitor.parentChild(abstract1,
					 ADLCPPGenericVisitor.makePtrOperator("PointerTo"), 0);
      ADLCPPGenericVisitor.parentChild(abstract1, abstract2, 1);
      ASTTypeName typeName = new
	ASTTypeName(ADLCPPParserTreeConstants.JJTTYPENAME);
      ADLCPPGenericVisitor.parentChild(typeName, declSpec, 0);
      ADLCPPGenericVisitor.parentChild(typeName, abstract1, 1);

      /* Build a CastExpr with 2 children:
       * TypeName for (bank *) and IdExpr for realObject
       */
      ASTCastExpr cast = new
	ASTCastExpr(ADLCPPParserTreeConstants.JJTCASTEXPR);
      ADLCPPGenericVisitor.parentChild(cast,typeName, 0);
      ADLCPPGenericVisitor.parentChild(cast,
					 ADLCPPGenericVisitor.makeIdExpr("realObject"),
					 1);

      /*R Build a direct declarator with a child
       * Qualified Id for <adlClassName>Object
       *
      ASTDirectDeclarator decl1 = new
	ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);
      ADLCPPGenericVisitor.parentChild(decl1,
					 ADLCPPGenericVisitor.makeQualifiedId(adlClassName+"Object"),
					 0);
*/
      /* Build the declaration node with 2 children
       * Declaration specifiers for <adlClassName>
       * and Init Declarator for the remainder
       */
      ACO_Initialization.addElement( new DeclarationBuilder
				     ( ADLCPPGenericVisitor.makeQualifiedType(adlClassName),
				       new InitDeclaratorBuilder
				       ( new DeclaratorBuilder
				        ( "PointerTo", 
					  adlClassName+"Object"
					  ).generatedAST(),
					 cast
					 ).generatedAST()
				       ).generatedAST()
				     );
    }
    if (debug) System.out.println("ACOauxiliary: Successfull ACO_Initialisation\n");
    return ACO_Initialization;
  }


  /**
   * Generates the generic code for the call to the real object function.
   * For instance for function "bankAccount open_account(long amount, long &acctNum)
   * throw (negativeAmountExc, bankFullExc)" of class bank:
   <PRE>
   |  if (doReport) {
   |     testContext->setblock();
   |     testContext->infoline("*** Checking bank::open_account ***");
   |  }
   |  bankAccount *__ADL_return = NULL;
   |  if (!skipAssertions) {
   |     try {
   |        __ADL_return = bankObject->open_account(amount, acctNum);
   |     }
   |     catch (negativeAmountExc& open_accountExc) {
   |       wasThrownException = true;
   |       thrownException = &open_accountExc;
   |       wasThrown_negativeAmountExc = true;
   |     }
   |     catch (bankFullExc& open_accountExc) {
   |        wasThrownException = true;
   |        thrownException = &open_accountExc;
   |        wasThrown_bankFullExc = true;
   |     }
   |     catch (...) {
   |        testContext->infoline("Unexpected exception caught when calling bank::open_account");
   |     testContext->result(ADL_UNRESOLVED);
   |   
   |     // now set skipAssertions to not evaluate remaining code
   |     // and inform potential subclass callers to not evaluate
   |     // their assertions as well.
   |     skipAssertions = true;
   |	
   |     // set skipAssertions in testContext as well for remaining
   |     // test instructions to be skipped.
   |     testContext->skipAssertions = true;
	 
   |     // evaluation of the abnormal clause
   |     abnormal = (wasThrown_negativeAmountExc || wasThrown_bankFullExc);
   |
   |     // evaluation of the normal clause
   |     normal = !abnormal;
   |  }
   </PRE>
   * @return The handle on the generated statements
   */
  Vector makeACO_CallBlock() {

    if (debug) System.out.println("ACOauxiliary: makeACO_CallBlock ...");

    //    if (doReport) { // when nested through inheritance, don't report
    //      testContext->setblock();
    //      testContext->infoline("*** Checking bank::close_account ***");
    //    }

    ASTIfNode ifDoReport = new
      ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);
    ADLCPPGenericVisitor.parentChild(ifDoReport, ADLCPPGenericVisitor.makeIdExpr("doReport"), 0);

    if (firstFct) {
      setCompoundStmt = new StatementBuilder
        ( new PostfixExprBuilder
          ( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	    new RefAccessSuffixBuilder( ADLCPPGenericVisitor.makeIdExpr("setblock"),
				      new ArgumentListBuilder().generatedAST()
				      ).generatedAST()
	    ).generatedAST()
	  ).generatedAST();
    }

    ASTString str;
    if (isConstructor)
      str = ADLCPPGenericVisitor.makeString("*** Checking "+ adlClassName+ "::"+ adlClassName+ " ***");
    else {
      str = ADLCPPGenericVisitor.makeString("*** Checking "+ adlClassName+ "::"+ fctName+ " ***");
    }
    ASTPostfixExpr infolineStmt = new
      PostfixExprBuilder( ADLCPPGenericVisitor.makeQualifiedId("testContext"),
			  new RefAccessSuffixBuilder
			  ( ADLCPPGenericVisitor.makeIdExpr("infoline"),
			    new ArgumentListBuilder(str).generatedAST()
			    ).generatedAST()
			  ).generatedAST();

    ASTCompoundStatement thenBlk = new 
      ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);

    ADLCPPGenericVisitor.parentChild(thenBlk, setCompoundStmt, 0);
    ADLCPPGenericVisitor.parentChild( thenBlk,
					new StatementBuilder(infolineStmt).generatedAST(),
					1);
    
    ADLCPPGenericVisitor.parentChild(ifDoReport, thenBlk, 1);
    ACO_CallBlock.addElement(ifDoReport);


    // <fctReturnType> __ADL_return = <default value>;
    if (debug) System.out.println("Function return type = " + fctReturnType);
    if (! fctReturnType.equals("void")) 
      if (getFctPointedType().jjtGetChild(1) instanceof ASTFunctionDirectDeclarator)  {
	ACO_CallBlock.addElement( new DeclarationBuilder
                                  ( (ASTDeclarationSpecifiers)getFctPointedType().jjtGetChild(0),
                                    new InitDeclaratorBuilder
				    ("__ADL_return",
				     CPPTypeAux.getDefaultValue(fctTypeDefaultValue, true)
				      ).generatedAST()
                                ).generatedAST());
      } 
      else { // Case of declarator with pointer operator
        Node child = getFctPointedType().jjtGetChild(1), tmp, tmp1, tmp2;
        tmp1 = new ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);
        tmp = tmp1;
        while (child.jjtGetChild(0) instanceof ASTPtrOperator) {
          ADLCPPGenericVisitor.parentChild(tmp1, child.jjtGetChild(0), 0);
          child = child.jjtGetChild(1);
          if (child.jjtGetChild(0) instanceof ASTPtrOperator) {
            tmp2 = new ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);
            ADLCPPGenericVisitor.parentChild(tmp1, tmp2, 1);
            tmp1 = tmp2;
          }
        }
        ADLCPPGenericVisitor.parentChild
          (tmp1,
           new DirectDeclaratorBuilder("__ADL_return").generatedAST(),
           1);
	ACO_CallBlock.addElement( new DeclarationBuilder
                                  ( (ASTDeclarationSpecifiers)getFctPointedType().jjtGetChild(0),
                                    new InitDeclaratorBuilder
				    (tmp,
				     CPPTypeAux.getDefaultValue(fctTypeDefaultValue, true)
				      ).generatedAST()
				    ).generatedAST());
      }
    
	
    // if (! skipAssertions)
    ASTIfNode ifNotSkip = new
      ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);
    ASTUnaryExpr notSkip = 
      new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
    notSkip.setOp("!");
    ADLCPPGenericVisitor.parentChild(notSkip, ADLCPPGenericVisitor.makeIdExpr("skipAssertions"), 0);
    ADLCPPGenericVisitor.parentChild(ifNotSkip, notSkip, 0);


    ASTCompoundStatement ifNotSkipBlock = new
      ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCPPGenericVisitor.parentChild(ifNotSkip, ifNotSkipBlock, 1);
    int ifNotSkipBlockIndex = 0;

    if (InheritedSemantics) {  // finalJournalReporting = false;
      ADLCPPGenericVisitor.parentChild
	( ifNotSkipBlock,
	  new StatementBuilder
	  ( new AssignmentBuilder
	    ( "finalJournalReporting", "=", ADLCPPGenericVisitor.makeBoolean("false")).generatedAST()
	    ).generatedAST(),
	  ifNotSkipBlockIndex++);
    }

    // try { [ __ADL_return = ] <adlClassName>Object-><fctName>(<fctParams.name>); }
    // or, if constructor:
    // try { __ADL_return = new <fctName>(<fctParams.name>); }

    ASTTryNode tryStmt = new
      ASTTryNode(ADLCPPParserTreeConstants.JJTTRYNODE);
    ADLCPPGenericVisitor.parentChild(ifNotSkipBlock, tryStmt, ifNotSkipBlockIndex++);

    ASTCompoundStatement tryCompoundStatement = new ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCPPGenericVisitor.parentChild(tryStmt, tryCompoundStatement, 0);

    ArgumentListBuilder callArgs = new ArgumentListBuilder();
    for (int i = 0; i < fctParams.length; i++) { // LParameter[] fctParam
      callArgs.addArgument(ADLCPPGenericVisitor.makeIdExpr(fctParams[i].getName()));
    }

    if (isConstructor) { 
/*
      ASTNewExpr newObj = new
        ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
      ASTDeclarationSpecifiers declObj = new
        ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
      ADLCPPGenericVisitor.parentChild
	( declObj,
	  ADLCPPGenericVisitor.makeQualifiedType(getAdlClassName()),
	  0);
      ADLCPPGenericVisitor.parentChild(newObj, declObj, 0);
      ADLCPPGenericVisitor.parentChild(newObj, callArgs.generatedAST(), 1);
*/
      ADLCPPGenericVisitor.parentChild
	( tryCompoundStatement,
	  new StatementBuilder
	  ( new AssignmentBuilder
	    ( ADLCPPGenericVisitor.makeIdExpr("__ADL_return"),
	      "=",
	      new NewExprBuilder(getAdlClassName(), callArgs.generatedAST()).generatedAST()
	      ).generatedAST()
	    ).generatedAST(),
	  0);
    }
    else {
      ASTIdExpr caller = null;
      Node callFct = null;
      if (InheritedSemantics) {
	caller = ADLCPPGenericVisitor.makeIdExpr
	  (getSuperSemantics()+ "ACO::" + fctName);
	callFct = new PrimaryExpressionBuilder
	  ( caller, callArgs.generatedAST()).generatedAST();
      }
      else {
      	caller = ADLCPPGenericVisitor.makeIdExpr(adlClassName+"Object");
	callFct = new PostfixExprBuilder
	  ( caller,
	    new RefAccessSuffixBuilder
	    ( ADLCPPGenericVisitor.makeIdExpr(fctName),
	      callArgs.generatedAST() ).generatedAST()
	    ).generatedAST();
      }
      if (fctReturnType.equals("void")) {
        ADLCPPGenericVisitor.parentChild
	  ( tryCompoundStatement,
	    new StatementBuilder(callFct).generatedAST(),
	    0);
      }
      else { // fctReturnType != void
	ADLCPPGenericVisitor.parentChild
	  ( tryCompoundStatement,
	    new StatementBuilder
	    ( new AssignmentBuilder
	      ( ADLCPPGenericVisitor.makeIdExpr("__ADL_return"),
		"=", callFct).generatedAST()
	      ).generatedAST(),
	    0);
      }
    }

    // catch block

    int insertChild = 1;
    excName = fctName + "Exc";

    for (int i=0; (!(fctThrowsList == null)&&(i< fctThrowsList.length)); i++) {
      ASTCatchNode catchBlock = new ASTCatchNode(ADLCPPParserTreeConstants.JJTCATCHNODE);
      ADLCPPGenericVisitor.parentChild(tryStmt, catchBlock, insertChild++);
      
      // catch (<ExcName> & <fctName>Exc) {
      
      ASTExceptionDeclaration excDecl = new ASTExceptionDeclaration(ADLCPPParserTreeConstants.JJTEXCEPTIONDECLARATION);
      ASTQualifiedType excType = ADLCPPGenericVisitor.makeQualifiedType(fctThrowsList[i].getFullName());
      ASTDeclarationSpecifiers declSpec = new ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
      ADLCPPGenericVisitor.parentChild(declSpec, excType, 0);
      
      ADLCPPGenericVisitor.parentChild(excDecl, declSpec, 0);
      ADLCPPGenericVisitor.parentChild
	( excDecl, new DeclaratorBuilder
	  ( "ReferenceTo", new DirectDeclaratorBuilder(excName).generatedAST()).generatedAST(), 1);
      ADLCPPGenericVisitor.parentChild(catchBlock, excDecl, 0);
      
      // wasThrownException = true;
      // adl_thrownException = &close_accountExc;
      // wasThrown_<ExcName> = true;
      
      ASTCompoundStatement catchBody = new ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
      ADLCPPGenericVisitor.parentChild(catchBlock, catchBody, 1);
      // wasThrownException = true;
      ADLCPPGenericVisitor.parentChild(catchBody,
					 new StatementBuilder
					 ( new AssignmentBuilder
					   ( "wasThrownException",
					     "=",
					     ADLCPPGenericVisitor.makeBoolean("true")
					     ).generatedAST()
					   ).generatedAST(),
					 0);
      
      // adl_thrownException = &close_accountExc;

      ASTUnaryExpr excExpr = new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
      excExpr.setOp("&");
      ADLCPPGenericVisitor.parentChild(excExpr, ADLCPPGenericVisitor.makeIdExpr(excName), 0);
      ADLCPPGenericVisitor.parentChild
	( catchBody,
	  new StatementBuilder
	  ( new AssignmentBuilder
	    ( "adl_thrownException", 
	      "=",
	      //	      ADLCPPGenericVisitor.makeIdExpr(excName)
	      excExpr
	      ).generatedAST()
	    ).generatedAST(),
	  1);
      // wasThrown_<ExcName> = true;
      ADLCPPGenericVisitor.parentChild
	( catchBody,
	  new StatementBuilder
	  ( new AssignmentBuilder
//Modif	    ( "wasThrown_" + fctThrowsList[i].getName(),
	    ( "wasThrown_Exc" + i,
	      "=",
	      ADLCPPGenericVisitor.makeBoolean("true")
	      ).generatedAST()
	    ).generatedAST(),
	  2);
      // thrown_<ExcName> = &close_accountExc;
      // NOT YET VALID
      ASTUnaryExpr excExpr1 = new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
      excExpr1.setOp("&");
      ADLCPPGenericVisitor.parentChild(excExpr1, ADLCPPGenericVisitor.makeIdExpr(excName), 0);
      //      ADLCPPGenericVisitor.parentChild
      //	( catchBody,
      //	  new StatementBuilder
      //	  ( new AssignmentBuilder
      //	    ( "thrown_"+ fctThrowsList[i].getName(),
      //	      "=",
      //	      excExpr1
      //	      ).generatedAST()
      //	    ).generatedAST(),
      //	  3);
      
      ADLCPPGenericVisitor.parentChild(catchBlock, catchBody, 1);    
    }    
    
    //    catch (...) {
    //      testContext->infoline("Unexpected exception caught when calling <fctName>");
    //      testContext->result(ADL_UNRESOLVED);
    //      skipAssertions = true;
    //      testContext->skipAssertions = true;
    //    }

    ASTCatchNode catchFinalBlk = new ASTCatchNode(ADLCPPParserTreeConstants.JJTCATCHNODE);
    ASTExceptionDeclaration finalExc = new ASTExceptionDeclaration(ADLCPPParserTreeConstants.JJTEXCEPTIONDECLARATION);
    ASTCompoundStatement catchFinalBody = new ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);

    //testContext->infoline("Unexpected exception caught when calling <fctName>");

    stmt1 = new StatementBuilder
      ( new PostfixExprBuilder
	( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	  new RefAccessSuffixBuilder
	  ( "infoline",
	    new ArgumentListBuilder
	    ( ADLCPPGenericVisitor.makeString("Unexpected exception caught when calling "+ adlClassName+ "::"+ fctName)
	      ).generatedAST()
	    ).generatedAST()
	  ).generatedAST()
	).generatedAST();
    
    // testContext->result(ADL_UNRESOLVED);
    stmt2 = new StatementBuilder
      ( new PostfixExprBuilder
	( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	  new RefAccessSuffixBuilder
	  ( "result",
	    new ArgumentListBuilder
	    (ADLCPPGenericVisitor.makeIdExpr("ADL_UNRESOLVED")
	      ).generatedAST()
	    ).generatedAST()
	  ).generatedAST()
	).generatedAST();
    
    // skipAssertions = true;
    stmt3 = new StatementBuilder
      ( new AssignmentBuilder
	( "skipAssertions", 
	  "=", 
	  ADLCPPGenericVisitor.makeBoolean("true")
	  ).generatedAST()
	).generatedAST();
    
    // testContext->skipAssertions = true;
    stmt4 = new StatementBuilder
      ( new AssignmentBuilder
	( new PostfixExprBuilder
	  ( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	    new RefAccessSuffixBuilder("skipAssertions").generatedAST()
	    ).generatedAST(),
	  "=", 
	  ADLCPPGenericVisitor.makeBoolean("true")
	  ).generatedAST()
	).generatedAST();
  
    ADLCPPGenericVisitor.parentChild(catchFinalBody, stmt1, 0);
    ADLCPPGenericVisitor.parentChild(catchFinalBody, stmt2, 1);
    ADLCPPGenericVisitor.parentChild(catchFinalBody, stmt3, 2);
    ADLCPPGenericVisitor.parentChild(catchFinalBody, stmt4, 3);

    ADLCPPGenericVisitor.parentChild(catchFinalBlk, finalExc, 0);
    ADLCPPGenericVisitor.parentChild(catchFinalBlk, catchFinalBody, 1);
    ADLCPPGenericVisitor.parentChild(tryStmt, catchFinalBlk, insertChild++);

    // finalJournalReporting = doReport; // if SuperClass::semantics

    if (InheritedSemantics) {
      ADLCPPGenericVisitor.parentChild(ifNotSkipBlock,
                                  new StatementBuilder
                                  ( new AssignmentBuilder
                                    ( "finalJournalReporting", "=",
                                      ADLCPPGenericVisitor.makeIdExpr("doReport")).generatedAST()
				    ).generatedAST(),
                                  ifNotSkipBlockIndex++);
    }

    // abnormal = <abnormalSpec>
    // normal = <normalSpec>

    boolean bothSpecified = false;

    // visit (ab)normalSpec if already defined

    try {
      if (abnormalSpec != null) {
	abnormalSpec = (Node)abnormalSpec.jjtAccept(associatedVisitor, null);
      }
      if (normalSpec != null) {
	normalSpec = (Node)normalSpec.jjtAccept(associatedVisitor, null);
      }
    }
    catch (ParseException e) {}
    if (normalSpec == null) {
      //TBR: abnormal = (wasThrown_negativeAmountExc || wasThrown_bankFullExc);
      if (abnormalSpec == null) {
        // default: abnormalSpec = wasThrownException
	abnormalSpec = ADLCPPGenericVisitor.makeIdExpr("wasThrownException");
      }
      
      // default: normalSpec = !abnormal
      ASTUnaryExpr notabnormal = 
	new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
      notabnormal.setOp("!");
      ADLCPPGenericVisitor.parentChild(notabnormal, ADLCPPGenericVisitor.makeIdExpr("abnormal"), 0);
      normalSpec = notabnormal;
    }
    else { // normalSpec != null
      if (abnormalSpec == null) {

        // abnormalSpec = !normalSpec
	ASTUnaryExpr notnormal = 
	  new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
	notnormal.setOp("!");
	ADLCPPGenericVisitor.parentChild(notnormal, ADLCPPGenericVisitor.makeIdExpr("normal"), 0);
        abnormalSpec = notnormal;
      }
      else // normal and abnormal both user-defined
        bothSpecified = true;
    }

    ADLCPPGenericVisitor.parentChild
      ( ifNotSkipBlock,
	new StatementBuilder
	( new AssignmentBuilder
	  ("abnormal", "=", abnormalSpec).generatedAST()
	  ).generatedAST(),
	ifNotSkipBlockIndex++);

    ADLCPPGenericVisitor.parentChild
      ( ifNotSkipBlock,
	new StatementBuilder
	( new AssignmentBuilder
	  ("normal", "=", normalSpec).generatedAST()
	  ).generatedAST(),
	ifNotSkipBlockIndex++);
    
    
    // if (normal && abnormal) testContext->result(ADL_AMBIGUOUS);
    
    if (bothSpecified) {
      // if normal and/or abnormal have not been user-defined, they have been
      // given default opposite values, hence the condition (normal && 
      // abnormalSpec) is false and the code is not to be generated.
      
      ASTIfNode ifNormAbn = new
        ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);
      ASTBinaryLogicalExpr nANDabn = 
        new ASTBinaryLogicalExpr(ADLCPPParserTreeConstants.JJTBINARYLOGICALEXPR);
      nANDabn.addOp("&&");
      ADLCPPGenericVisitor.parentChild
	( nANDabn,
	  ADLCPPGenericVisitor.makeIdExpr("normal"),
	  0);
      ADLCPPGenericVisitor.parentChild
	( nANDabn,
	  ADLCPPGenericVisitor.makeIdExpr("abnormal"),
	  1);

      ADLCPPGenericVisitor.parentChild(ifNormAbn, nANDabn, 0);

      ADLCPPGenericVisitor.parentChild
	( ifNormAbn,
	  new StatementBuilder
	  ( new PostfixExprBuilder
	    ( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	      new RefAccessSuffixBuilder
	      ( "result",
		new ArgumentListBuilder
		( ADLCPPGenericVisitor.makeIdExpr("ADL_AMBIGUOUS")
		  ).generatedAST()
		).generatedAST()
	      ).generatedAST()
	    ).generatedAST(),
	  1);

      ADLCPPGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn, ifNotSkipBlockIndex++);

      // if (!(normal || abnormalSpec)) testContext->result(ADL_UNDEFINED);

      // if normal and/or abnormal have not been user-defined, they have been
      // given default opposite values, hence the condition !(normal || 
      // abnormalSpec) is false and the code is not to be generated.

      ASTIfNode ifNormAbn2 = new
        ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);

      ASTBinaryLogicalExpr nORabn = 
        new ASTBinaryLogicalExpr(ADLCPPParserTreeConstants.JJTBINARYLOGICALEXPR);
      nANDabn.addOp("||");
      ADLCPPGenericVisitor.parentChild
	( nORabn,
	  ADLCPPGenericVisitor.makeIdExpr("normal"),
	  0);
      ADLCPPGenericVisitor.parentChild
	( nORabn,
	  ADLCPPGenericVisitor.makeIdExpr("abnormal"),
	  1);
      
      ASTUnaryExpr notnORabn = 
	new ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
      notnORabn.setOp("!");
      ADLCPPGenericVisitor.parentChild(notnORabn, nORabn, 0);

      ADLCPPGenericVisitor.parentChild(ifNormAbn2,notnORabn, 0);

      ADLCPPGenericVisitor.parentChild
	( ifNormAbn2,
	  new StatementBuilder
	  ( new PostfixExprBuilder
	    ( ADLCPPGenericVisitor.makeIdExpr("testContext"),
	      new RefAccessSuffixBuilder
	      ( "result",
		new ArgumentListBuilder
		( ADLCPPGenericVisitor.makeIdExpr("ADL_UNDEFINED")
		  ).generatedAST()
		).generatedAST()
	      ).generatedAST()
	    ).generatedAST(),
	  1);

      ADLCPPGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn2, ifNotSkipBlockIndex++);
    }


    ACO_CallBlock.addElement(ifNotSkip);
    
    return ACO_CallBlock;
  }


  /**
   * Generates the generic final code for each function of an ACO class.
   * For instance for function "bankAccount *open_account(long amount, long &acctNum)
   * throw (negativeAmountExc, bankFullExc)" of class bank:
   <PRE>
   |  if (wasThrownException) {
   |    if (wasThrown_negativeAmountExc)
   |       throw *(negativeAmountExc *)adl_thrownException;
   |    if (wasThrown_bankFullExc)
   |       throw *(bankFullExc *)adl_thrownException;
   |  }
   |
   |  return __ADL_return;
   </PRE>
   * @return The handle on the generated statements
   */
  Vector makeACO_Final() {

    if (debug) System.out.println("ACOauxiliary: makeACO_Final ...");

    ACO_Final = new Vector();

    //  if (wasThrownException)

    ASTIfNode finalIf = new
        ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);
    ADLCPPGenericVisitor.parentChild
      ( finalIf,
	ADLCPPGenericVisitor.makeIdExpr("wasThrownException"), 0);

    /**
      * Exception propagation for Solaris platform
      *
    ASTCompoundStatement ifBody = new
        ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    for (int i=0; (!(fctThrowsList == null)&&(i< fctThrowsList.length)); i++) {
      ASTIfNode ifExc = new
        ASTIfNode(ADLCPPParserTreeConstants.JJTIFNODE);
      ADLCPPGenericVisitor.parentChild
	( ifExc,
	  ADLCPPGenericVisitor.makeIdExpr("wasThrown_Exc"+i), 0);
      // throw *(negativeAmountExc *)adl_thrownException;

      ASTThrowStmt throwExc = new
        ASTThrowStmt(ADLCPPParserTreeConstants.JJTTHROWSTMT);
      ADLCPPGenericVisitor.parentChild(ifExc, throwExc, 1);

      ASTUnaryExpr subNode = new
        ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
      subNode.setOp("*");
      ADLCPPGenericVisitor.parentChild(throwExc, subNode, 0);

      ASTCastExpr subNode_0 = new
        ASTCastExpr(ADLCPPParserTreeConstants.JJTCASTEXPR);
      ADLCPPGenericVisitor.parentChild(subNode, subNode_0, 0);

      ASTTypeName subNode_0_0 = new
        ASTTypeName(ADLCPPParserTreeConstants.JJTTYPENAME);
      ADLCPPGenericVisitor.parentChild(subNode_0, subNode_0_0, 0);
      ADLCPPGenericVisitor.parentChild
	(subNode_0, 
	 ADLCPPGenericVisitor.makeIdExpr("adl_thrownException"),
	 1);

      ASTDeclarationSpecifiers subNode_0_0_0 = new
        ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
      ASTBuiltinTypeSpec subNode_0_0_0_0 = new
        ASTBuiltinTypeSpec(ADLCPPParserTreeConstants.JJTBUILTINTYPESPEC);
      subNode_0_0_0_0.setType(fctThrowsList[i].getName());
      ADLCPPGenericVisitor.parentChild(subNode_0_0_0, subNode_0_0_0_0, 0);
      ASTAbstractDeclarator subNode_0_0_1 = new
        ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
      ASTPtrOperator subNode_0_0_1_0 = new
        ASTPtrOperator(ADLCPPParserTreeConstants.JJTPTROPERATOR);
      subNode_0_0_1_0.setPtrkind("PointerTo");
      ASTAbstractDeclarator subNode_0_0_1_1 = new
        ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
      ADLCPPGenericVisitor.parentChild(subNode_0_0_1, subNode_0_0_1_0, 0);
      ADLCPPGenericVisitor.parentChild(subNode_0_0_1, subNode_0_0_1_1, 1);

      ADLCPPGenericVisitor.parentChild(subNode_0_0, subNode_0_0_0, 0);
      ADLCPPGenericVisitor.parentChild(subNode_0_0, subNode_0_0_1, 1);
      
      ADLCPPGenericVisitor.parentChild(ifBody, ifExc, i);
    }
    if (fctThrowsList == null) {
      ASTThrowStmt finalThrow = new
        ASTThrowStmt(ADLCPPParserTreeConstants.JJTTHROWSTMT);
      ADLCPPGenericVisitor.parentChild(finalIf, finalThrow, 1);
    }
    else {
      ADLCPPGenericVisitor.parentChild(finalIf, ifBody, 1);
    }
    *
    **/

    ASTThrowStmt finalThrow = new
      ASTThrowStmt(ADLCPPParserTreeConstants.JJTTHROWSTMT);
    ADLCPPGenericVisitor.parentChild(finalIf, finalThrow, 1);

    ACO_Final.addElement(finalIf);   

    //    [ return __ADL_return ; ]

    if ( ! fctReturnType.equals("void")) {
      ASTReturnStmt finalReturn = new
        ASTReturnStmt(ADLCPPParserTreeConstants.JJTRETURNSTMT);
      ADLCPPGenericVisitor.parentChild
	(finalReturn, ADLCPPGenericVisitor.makeIdExpr("__ADL_return"), 0);
      ACO_Final.addElement(finalReturn);
    }
    firstFct = false;
    return ACO_Final;
  }

}

// ***********************************************************************
// History Log:
// $Log: ACOauxiliary.java,v $
// Revision 1.15  1998/10/15 18:10:46  mallet
// Minor cosmetic changes to some messages.
//
// Revision 1.14  1998/06/30 18:22:26  bareau
// Bug fix: in constructors, the __ADL_return variable was not
// correctly typed.
//
// Revision 1.13  1998/06/30 13:14:53  mallet
// Added import of ORG.opengroup.adl.parsers.*.
// Removed function getFctTypeDefaultValue to use getDefaultValue in
// CPPTypeAux instead.
// Replaced test on second child of ASTFunctionDeclarator by a test on
// first child: what happened when there was no second child?
//
// Revision 1.12  1998/06/18 10:32:09  mallet
// Fixed incorrect use of type short name instead of full one (FQN).
// Made use of makeQualidifedType systematic.
//
// Revision 1.11  1998/06/03 16:23:18  mallet
// Changed initialization of doReport in ACO methods to avoid reporting
// the "*** Checking... ***" message in Short report mode: doReport is
// now initialized to finalJournalReporting && debugMode.
//
// Revision 1.10  1998/04/28 12:39:22  cinard
// Updated code to take into account replacement of CSymbolTableMgr,
// CSymbolFactory and CBuiltinType by CPPSymbolTableMgr, CPPSymbolFactory
// and CPPBuiltinType.
//
// Revision 1.9  1998/03/19 10:43:39  rami
// Code clean-up: maximum use of builders.
//
// Revision 1.8  1998/03/10 17:26:41  mallet
// Merged v1.1 with development trunk
//
// Revision 1.7.2.3  1998/03/06 12:45:22  rami
// Added method findExc(LType) retunring string value (used in ASTADLThrownExpr visit).
// Improved exceptions generation :
// 	Used index of the exception in the array returned by getExceptions() method instead of 	using exception names in order to avoid cases of pointed or referenced types.
// 	i.e: `throw (int *)` was transformed `wasThrown_int *`
//
// Revision 1.7.2.2  1998/03/05 13:36:33  rami
// Bug fix: changed exceptions from pointed to referenced in the catch clause od _ADL_return.
//
// Revision 1.7.2.1  1998/03/04 16:11:51  rami
// Modified some code according to the removal of scope override from fct name in class declaration.
//
// Revision 1.7  1998/03/02 13:30:25  rami
// Fixed tested method name in infoline report message:
// name is overrided by <adlClassName> and not by <adlClassName+"ACO">.
//
// Revision 1.6  1998/02/27 15:43:34  rami
// Updated copyright notice.
//
// Revision 1.5  1998/02/24 10:09:48  rami
// Fixed scope override of checked functions in report message.
//
// Revision 1.4  1998/02/12 16:03:39  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/02/12 11:29:12  rami
// Version OK for examples.
// Debug flag set to false.
//
// Revision 1.2  1998/02/04 14:00:12  rami
// Used the parentChild and make* methods from the generic visitor.
//
// Revision 1.1  1998/01/21 15:43:22  rami
// Auxiliary class for the construction of generic parts of code in ACOs.
// Not completely tested.
//
// ***********************************************************************
