// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/AssignmentBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/04/01 16:49:50 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        AssignmentBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 12, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ assignment.
  * @author Rami KHEMAKHEM
  */

public class AssignmentBuilder {

  /** Root of the generated tree */
  private ASTAssignmentExpr root;


  /**
    * Creates a C++ assignment expression
    * @param lhs The name of the variable to assign
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (String lhs, String op, Node rhs) {

    this(ADLCPPGenericVisitor.makeIdExpr(lhs), op, rhs);
  }


  /**
    * Creates a C++ assignment expression
    * @param lhs The variable to assign
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (ASTIdExpr lhs, String op, Node rhs) {

    root = new
      ASTAssignmentExpr(ADLCPPParserTreeConstants.JJTASSIGNMENTEXPR);
    root.setOp(op);
    ADLCPPGenericVisitor.parentChild(root, lhs, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a C++ assignment expression
    * @param lhs The variable to assign (case of an array element)
    * @param op The assignment operator
    * @param rhs The right-hand side expression
    */
  public AssignmentBuilder (SimpleNode lhs, String op, Node rhs) {

    root = new
      ASTAssignmentExpr(ADLCPPParserTreeConstants.JJTASSIGNMENTEXPR);
    root.setOp(op);
    ADLCPPGenericVisitor.parentChild(root, lhs, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTAssignmentExpr generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log
// ***********************************************************************
