// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/CPPDirectiveBuilder.java,v $
// $Revision: 1.10 $
// $Date: 1998/10/14 08:14:22 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        CPPDirectiveBuilder
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 4, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlcpp.*;
import ORG.opengroup.adl.symboltable.c.CModifier;
import ORG.opengroup.adl.symboltable.*;
import java.util.Vector;
import java.util.Enumeration;

/**
  * Creates a Java compilation unit AST that corresponds to a directive
  * declaration.
  * <BR>Uses CPPDatasetExprVisitor; is used by CPPTDDTransformVisitor.
  * <P>Example: definition of directive "mydir" in tdd class bankTest:
  * <BR>"mydir : test (bank b = B_COMBINE,int d = DEPOSITS,int w = WITHDRAWS+DEPOSITS)
  * test_bank (b,d,w);"
  * @author Cyrille Bareau
  * @see CPPDatasetExprVisitor
  * @see CPPTDDTransformVisitor
  */

public class CPPDirectiveBuilder extends CPPBuilder {

  /** Handle on the TDD single declarators used in domains in the directive
    * parameters */
  private Vector singleDecls;

  /** Names of the ACOs to generate. */
  private Vector acoNames = new Vector();
  

  public Vector getACONames() {
    return acoNames;
  }

  static private boolean debug = false;


  /**
    * Single constructor.
    * <BR>Initializes the unit name (that begins with "D_")
    * @param node the directive declaration node
    * @param unit source unit name
    * @param nb index of anonymous directives
    */
  public CPPDirectiveBuilder (ASTTDDTestDirective node, String unit, ASTIncludeFileList incList)
    throws ParseException {
    super(node, "D_", unit, incList);
    singleDecls = new Vector();
    build(node);
  }


  /**
    * Performs some initializations (call to inherited CPPBuilder.build) and
    * launches the methods that generate the code.
    * @see CPPBuilder#build(String)
    */
  private void build(ASTTDDTestDirective node) throws ParseException {

    if (debug)
      System.out.println("CPPDirectiveBuilder build");

    build(unitName + "TDD");

    int nbChildren = compilUnit.jjtGetNumChildren();

    ADLCPPGenericVisitor.parentChild(compilUnit, makeConstructor(node), nbChildren++);
    ADLCPPGenericVisitor.parentChild(compilUnit, makeRunInstance(node), nbChildren++);


    // ADLTest *ADL_theTest = new D_bankTest_test_bank_1();

    ASTDeclarationSpecifiers adlTest = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    ADLCPPGenericVisitor.parentChild(adlTest,
                                     ADLCPPGenericVisitor.makeQualifiedType("ADLTest"),
                                     0);

    ASTDeclaration theTest = new DeclarationBuilder
      ( adlTest,
        new InitDeclaratorBuilder
        ( new DeclaratorBuilder("PointerTo", "ADL_theTest").generatedAST(),
          new NewExprBuilder(className,
                             new ArgumentListBuilder().generatedAST())
          .generatedAST())
        .generatedAST())
      .generatedAST();

    ADLCPPGenericVisitor.parentChild(compilUnit, theTest, nbChildren++);
  }


  /**
    * Creates the constructor of the generated directive class.
    * Example for directive mydir:
<PRE>
| 1|  D_bankTest_mydir::D_bankTest_mydir() {
| 2|     S_bankTest_B_COMBINE *elt_p1 = new S_bankTest_B_COMBINE(this);
| 3|     S_bankTest_DEPOSITS *elt_p2 = new S_bankTest_DEPOSITS(this);
| 4|     ADLConcatenationSet *elt_p3 = new ADLConcatenationSet(this, null);
| 5|     S_bankTest_WITHDRAWS *elt_p3_p1 = new S_bankTest_WITHDRAWS(this);
| 6|     S_bankTest_DEPOSITS *elt_p3_p2 = new S_bankTest_DEPOSITS(this);
| 7|     elt_p3->addElement(elt_p3_p1);
| 8|     elt_p3->addElement(elt_p3_p2);
| 9|     addParameter(elt_p1);
|10|     addParameter(elt_p2);
|11|     addParameter(elt_p3);
|12|  }
</PRE>
    * The transformation of each directive parameter dataset expression is
    * performed by calling a CPPDatasetExprVisitor (in the example, lines
    * 2 to 8).
    * @see CPPDatasetExprVisitor
    * @param testDir the test directive
    * @return The root of the constructor (ASTConstructorDeclaration).
    */
  private ASTCtorDefinition makeConstructor(ASTTDDTestDirective testDir) throws ParseException {

    if (debug)
      System.out.println("CPPDirectiveBuilder makeConstructor");

    addConstructorDecl(true);

    ASTCtorDefinition createdNode = new
      ASTCtorDefinition(ADLCPPParserTreeConstants.JJTCTORDEFINITION);

    ASTCtorDeclarator ctorDecl = new
      ASTCtorDeclarator(ADLCPPParserTreeConstants.JJTCTORDECLARATOR);
    ADLCPPGenericVisitor.parentChild(createdNode, ctorDecl, 0);

    ADLCPPGenericVisitor.parentChild(ctorDecl,
                                     ADLCPPGenericVisitor.makeQualifiedId(className + "::" + className),
                                     0);


    String strStackInit = "elt";
    ASTTDDDatasetDomain domain;
    CPPDatasetExprVisitor exprVisitor =
      new CPPDatasetExprVisitor(strStackInit, "this", true, this);

    ASTCompoundStatement body = new
      ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCPPGenericVisitor.parentChild(createdNode, body, 1);
    int nbChildren = 0; LType expectedType;

    // visit all test directive parameters to gather the statements 
    // that create them
    for (int i = 0; i < testDir.jjtGetNumChildren()-1; i++) {
      domain = (ASTTDDDatasetDomain)testDir.jjtGetChild(i);
      if (domain.jjtGetNumChildren() > 1) {
        expectedType = ((ASTDeclarationSpecifiers)domain.jjtGetChild(0).jjtGetChild(0))
          .getType();
        singleDecls.addElement(domain.jjtGetChild(0));

        if (((ASTDeclarationSpecifiers)domain.jjtGetChild(0).jjtGetChild(0)).getModifier() != 0x0000)
          CPPTDDTransformVisitor.warningMessage("Type modifier(s) for tdd directive parameters will be ignored",
                                                domain.getLineNumber());
      }
      else { // void dataset
        expectedType = null;
        singleDecls.addElement(null);
      }

      exprVisitor.setExpectedType(expectedType);

      domain.jjtGetChild(domain.jjtGetNumChildren()-1).jjtAccept(exprVisitor, null);

    }

    // now attach all the statements that create the directive
    // parameters (components of the DatasetExprVisitor) to the current
    // block
    for (Enumeration e = ((Vector)exprVisitor.getDatasetComponents().peek()).elements();
           e.hasMoreElements(); ) {
        ADLCPPGenericVisitor.parentChild(body,
					 (SimpleNode)e.nextElement(),
					 nbChildren++);
    }

    // and also add the "addParameter" statements for the directive
    // parameters
    for (int i = 0; i < testDir.jjtGetNumChildren()-1; i++) {
      ADLCPPGenericVisitor.parentChild(body, 
                                       new StatementBuilder
                                       ( new PostfixExprBuilder
                                         ( "addParameter",
                                           new ArgumentListBuilder
                                           ( ADLCPPGenericVisitor.makeIdExpr(strStackInit + "_p" + (i+1)))
                                           .generatedAST())
                                         .generatedAST())
                                       .generatedAST(),
                                       nbChildren++);
    }
    return createdNode;
  }

   
  /**
    * Creates the "runInstance" method of the generated directive class.
    * Example for directive mydir:
<PRE>
|  void D_bankTest_mydir::runInstance(void **params) {
|     bank *b = (bank *)params[0];
|     infoline(adlstr.reset() << "Parameter bank *b = " << b);
|     int d = *((int *)params[1]);
|     infoline(adlstr.reset() << "Parameter int d = " << d);
|     int w = *((int *)params[2]);
|     infoline(adlstr.reset() << "Parameter int w = " << w);
|     &#60;directive body&362;
|  }
</PRE>
    * <B>Note:</B> the directive body is visited by an RemoveADLVisitor
    * to transform the ADL/ADL_new expressions it contains.
    * @see RemoveADLVisitor
    * @return The root of the constructor (ASTConstructorDeclaration).
    */
  private ASTFunctionDefinition makeRunInstance(ASTTDDTestDirective testDir)
    throws ParseException {

    if (debug)
      System.out.println("CPPDirectiveBuilder makeRunInstance");

    addFunctionDecl("void", false, "runInstance", false);

    // protected void runInstance(Object[] paramValues)

    ASTFunctionDefinition createdNode = new
      ASTFunctionDefinition(ADLCPPParserTreeConstants.JJTFUNCTIONDEFINITION);
    ADLCPPGenericVisitor.parentChild(createdNode, ADLCPPGenericVisitor.makeBuiltinType("void"), 0);

    ASTFunctionDirectDeclarator fctDecl = new
      ASTFunctionDirectDeclarator(ADLCPPParserTreeConstants.JJTFUNCTIONDIRECTDECLARATOR);
    ADLCPPGenericVisitor.parentChild(createdNode, fctDecl, 1);
    ADLCPPGenericVisitor.parentChild(fctDecl,
                                  ADLCPPGenericVisitor.makeQualifiedId(className + "::" + "runInstance"),
                                  0);
    ADLCPPGenericVisitor.parentChild
      (fctDecl,
       new ParameterDeclarationBuilder
       ( ADLCPPGenericVisitor.makeBuiltinType("void"),
         new DeclaratorBuilder("PointerTo", "PointerTo",
                               new DirectDeclaratorBuilder("params").generatedAST())
         .generatedAST())
       .generatedAST(),
       1);

    ASTCompoundStatement body = new
      ASTCompoundStatement(ADLCPPParserTreeConstants.JJTCOMPOUNDSTATEMENT);
    ADLCPPGenericVisitor.parentChild(createdNode, body, 2);

    int nbChildren = 0;
    ASTDeclaration single; ASTInitDeclarator initDecl;
    ASTDeclarationSpecifiers typ1; Node declTor = null, declTorBis;
    ASTAbstractDeclarator abstr, abstrBis; ASTCastExpr castExpr;
    ASTTypeName typeName; ASTArraySuffix suffix; ASTUnaryExpr unary;
    ASTString str; ASTConstant index; boolean isPrimitiveType = false;
    ASTBinaryArithExpr shift; ASTDirectDeclarator id = null;
    ASTDotAccessSuffix reset; String paramName = "";

    for (int indexSD = 0; indexSD < singleDecls.size(); indexSD++) {
      single = (ASTDeclaration)singleDecls.elementAt(indexSD);
      if (single != null) { // not void dataset parameter

        if ((single.jjtGetNumChildren() < 2) ||
            (single.jjtGetChild(1) instanceof ASTInitDeclarator) ||
            (single.jjtGetChild(1) instanceof ASTInitDeclaratorList))
          CPPTDDTransformVisitor.errorMessage("Incorrect directive parameter",
                                       ((SimpleNode)single.jjtGetChild(1)).getLineNumber());

        typ1 = (ASTDeclarationSpecifiers)single.jjtGetChild(0);
        if (single.jjtGetChild(1) instanceof ASTDirectDeclarator) { // non-pointer type
          if (single.jjtGetChild(1).jjtGetNumChildren() > 1)
            CPPTDDTransformVisitor.errorMessage("No suffix for directive parameter",
                                         ((SimpleNode)single.jjtGetChild(1)).getLineNumber());
          if (single.jjtGetChild(1).jjtGetChild(0) instanceof ASTQualifiedId) {
            isPrimitiveType = typ1.jjtGetChild(0) instanceof ASTBuiltinTypeSpec;
            declTor = (ASTDirectDeclarator)single.jjtGetChild(1);
            paramName = ((ASTQualifiedId)single.jjtGetChild(1).jjtGetChild(0)).getFullName();
            id = (ASTDirectDeclarator)declTor;
          }
          else CPPTDDTransformVisitor.errorMessage("No deletor or nested types in directive parameters",
                                            ((SimpleNode)single.jjtGetChild(1).jjtGetChild(0)).getLineNumber());
        }
        else {
          isPrimitiveType = false;
          declTor = (ASTDeclarator)single.jjtGetChild(1);
        }

        if (isPrimitiveType) {
          // <typ1> <id> = *((<typ1> *)params[<i>]);
          unary = new
            ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
          unary.setOp("*");

          initDecl = new InitDeclaratorBuilder(declTor, unary).generatedAST();

          ASTParenthExpr par = new
            ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);

          castExpr = new
            ASTCastExpr(ADLCPPParserTreeConstants.JJTCASTEXPR);
          ADLCPPGenericVisitor.parentChild(par, castExpr, 0);
          ADLCPPGenericVisitor.parentChild(unary, par, 0);

          typeName = new
            ASTTypeName(ADLCPPParserTreeConstants.JJTTYPENAME);
          ADLCPPGenericVisitor.parentChild(castExpr, typeName, 0);
          ADLCPPGenericVisitor.parentChild(typeName, typ1, 0);

          abstr = new
            ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
          ADLCPPGenericVisitor.parentChild(typeName, abstr, 1);

          ADLCPPGenericVisitor.parentChild(abstr,
                                           ADLCPPGenericVisitor.makePtrOperator("PointerTo"),
                                           0);

          // primary expression
          suffix = new
            ASTArraySuffix(ADLCPPParserTreeConstants.JJTARRAYSUFFIX);
          index = new
            ASTConstant(ADLCPPParserTreeConstants.JJTCONSTANT);
          index.setCstVal(String.valueOf(indexSD));
          ADLCPPGenericVisitor.parentChild(suffix, index, 0);

          ADLCPPGenericVisitor.parentChild(castExpr,
                                           new PostfixExprBuilder
                                           ( ADLCPPGenericVisitor.makeIdExpr("params"), suffix)
                                           .generatedAST(),
                                           1);

          ADLCPPGenericVisitor.parentChild(body,
                                           new DeclarationBuilder(typ1, initDecl)
                                           .generatedAST(),
                                           nbChildren++);
        }
        else {
          // <typ1> <id> = (<typ1>)params[<i>];

          castExpr = new
            ASTCastExpr(ADLCPPParserTreeConstants.JJTCASTEXPR);

          initDecl = new InitDeclaratorBuilder(declTor, castExpr)
            .generatedAST();

          typeName = new
            ASTTypeName(ADLCPPParserTreeConstants.JJTTYPENAME);
          ADLCPPGenericVisitor.parentChild(castExpr, typeName, 0);
          ADLCPPGenericVisitor.parentChild(typeName, typ1, 0);

          declTorBis = declTor;
          abstr = new
            ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
          ADLCPPGenericVisitor.parentChild(typeName, abstr, 1);

          while (declTorBis instanceof ASTDeclarator) {
            ADLCPPGenericVisitor.parentChild(abstr, declTorBis.jjtGetChild(0), 0);
            abstrBis = new
              ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
            ADLCPPGenericVisitor.parentChild(abstr, abstrBis, 1);
            abstr = abstrBis;
            paramName += ((ASTPtrOperator)declTorBis.jjtGetChild(0)).getPtrstring();
            declTorBis = declTorBis.jjtGetChild(1);
          }
          if (declTorBis.jjtGetNumChildren() > 1)
            CPPTDDTransformVisitor.errorMessage("No suffix for directive parameter",
                                         ((SimpleNode)declTorBis).getLineNumber());
          if (declTorBis.jjtGetChild(0) instanceof ASTQualifiedId) {
            paramName += ((ASTQualifiedId)declTorBis.jjtGetChild(0)).getFullName();
          }
          else CPPTDDTransformVisitor.errorMessage("No nested type in parameter",
                                            ((SimpleNode)declTorBis.jjtGetChild(0)).getLineNumber());
          id = (ASTDirectDeclarator)declTorBis;

          // primary expression
          suffix = new
            ASTArraySuffix(ADLCPPParserTreeConstants.JJTARRAYSUFFIX);
          index = new
            ASTConstant(ADLCPPParserTreeConstants.JJTCONSTANT);
          index.setCstVal(String.valueOf(indexSD));
          ADLCPPGenericVisitor.parentChild(suffix, index, 0);

          ADLCPPGenericVisitor.parentChild(castExpr,
                                        new PostfixExprBuilder
                                        ( ADLCPPGenericVisitor.makeIdExpr("params"), suffix)
                                        .generatedAST(),
                                        1);

          ADLCPPGenericVisitor.parentChild(body,
                                           new DeclarationBuilder(typ1, initDecl)
                                           .generatedAST(),
                                           nbChildren++);
        }


        // infoline(adlstr.reset() << "Parameter int w = " << w);
        shift = new
          ASTBinaryArithExpr(ADLCPPParserTreeConstants.JJTBINARYARITHEXPR);
        shift.addOp("<<"); shift.addOp("<<");

        reset = new
          ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);
        ADLCPPGenericVisitor.parentChild(reset,
                                         ADLCPPGenericVisitor.makeIdExpr("reset"),
                                         0);
        ADLCPPGenericVisitor.parentChild(reset,
                                         new ArgumentListBuilder().generatedAST(),
                                         1);

        ADLCPPGenericVisitor.parentChild(shift,
                                         new PostfixExprBuilder
                                         ( ADLCPPGenericVisitor.makeIdExpr("adlstr"),
                                           reset).generatedAST(),
                                         0);

        str = new
          ASTString(ADLCPPParserTreeConstants.JJTSTRING);
        str.setName("Parameter " + typ1.getType().getName() + " " + paramName + " = ");
        ADLCPPGenericVisitor.parentChild(shift, str, 1);
        ADLCPPGenericVisitor.parentChild(shift, id, 2);
        ADLCPPGenericVisitor.parentChild(body,
                                         new StatementBuilder
                                         ( new PrimaryExpressionBuilder
                                           ( ADLCPPGenericVisitor.makeIdExpr("infoline"),
                                             new ArgumentListBuilder(shift).generatedAST())
                                           .generatedAST())
                                         .generatedAST(),
                                         nbChildren++);
      }
    }

    // user code:

    // visit the user code to change ADL/ADL_new expressions

    RemoveADLVisitor v = new RemoveADLVisitor();
    Node userStmt = (Node)testDir.jjtGetChild(testDir.jjtGetNumChildren()-1)
                                 .jjtAccept(v, null);
    Vector inclACO = v.getACONames();
    for (Enumeration e = inclACO.elements(); e.hasMoreElements(); ) {
      String acoName = (String)e.nextElement();
      if ( ! acoNames.contains(acoName) ) {
	acoNames.addElement(acoName);
      }
      addInclude(acoName + ".cpp");
    }

    if (userStmt instanceof ASTCompoundStatement) {
      for (int i = 0; i < userStmt.jjtGetNumChildren(); i++) {
        ADLCPPGenericVisitor.parentChild(body, userStmt.jjtGetChild(i), nbChildren++);
      }
    }
    else {
      ADLCPPGenericVisitor.parentChild(body, userStmt, nbChildren);
    }

    return createdNode;
  }

}

// ***********************************************************************
// History Log:
// $Log: CPPDirectiveBuilder.java,v $
// Revision 1.10  1998/10/14 08:14:22  mallet
// Bugs fixed: pr/155, pr/156: removed useless index of anonymous
// directive as argument to constructor.
//
// Revision 1.9  1998/06/30 13:12:23  mallet
// Replaced import of PropertyAccessException in
// ORG.opengroup.adl.parsers by an import on demand (.*).
//
// Revision 1.8  1998/06/12 15:44:50  mallet
// Updated code due to changes in CPPDatasetExprVisitor: only one instance
// of this class is created for all parameters of a directive, the
// list of components (statements) generated for the parameters is also
// recovered once for all parameters, and these components are "added" at
// the end of the constructor for the directive.
// Changed constructor to add a new argument: the list of include files
// from the TDD file (as in CPPBuilder).
//
// Revision 1.7  1998/06/08 15:06:05  mallet
// Added a new vector "acoNames" and accessor method to store the list of
// ACO used in a test directive (call to ADL(...) or ADL_new ...).
// Added code to get the list of used ACO from RemoveADLVisitor.
//
// Revision 1.6  1998/03/11 14:54:12  bareau
// Added new warnings: type modifiers in tdd files (dataset definitions, factory/directive
// parameters) are ignored. New static method warningMessage in CPPTDDTransformVisitor.
// Improved support for casted expressions (there were useless double cast expressions
// in generated code).
//
// Revision 1.5  1998/03/10 17:26:44  mallet
// Merged v1.1 with development trunk
//
// Revision 1.4.2.5  1998/03/09 15:09:56  bareau
// Bug fix: pointers to pointers to ... were incorrectly reproduced in the
// generated code.
//
// Revision 1.4.2.4  1998/03/09 10:28:36  bareau
// Cleaned up the code for error handling (wrote a static method
// "errorMessage" in CPPTDDTransformVisitor).
//
// Revision 1.4.2.3  1998/03/04 17:39:43  bareau
// Added new checks.
// Cleaned up the code.
//
// Revision 1.4.2.2  1998/03/04 11:05:58  bareau
// The dataset and directive builders now send to the dataset expr visitor a
// string that represents the "expected" type of the dataset expression, so that
// this visitor generate an upcast if the actual type is different.
//
// Revision 1.4.2.1  1998/03/03 10:59:55  bareau
// Set the debug flag to false.
// Bug fix in CPPDatasetExprVisitor: ASTTDDDatasetSingleton nodes were
// sent to the unparser.
//
// Revision 1.4  1998/02/27 13:07:29  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/26 17:49:59  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// Revision 1.2  1998/02/12 16:03:42  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/12 13:48:21  bareau
// Classes used in the TDD translation. Added because of code freeze: this is a
// pre-pre-version (it does not even compile yet).
//
// ***********************************************************************
