// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/CPPTDDTransformVisitor.java,v $
// $Revision: 1.13 $
// $Date: 1998/10/14 08:17:27 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.cpp
// NAME:        CPPTDDTransformVisitor
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 5, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlcpp.*;
import ORG.opengroup.adl.symboltable.c.*;
import java.util.Vector;
import java.util.Enumeration;

/**
  * Transformation visitor.  Must change a TDD AST into Java ASTs.
  * <P> For an overview of the algorithms used by this transformation,
  * see the ADL2.0 Translation System Design Specification, Chapter 5
  * (Code Generation), paragraph 5.2.2.
  * @see ADLCPPGenericVisitor
  * @see ADLCPPParserVisitor
  * @author Cyrille Bareau
 */

public class CPPTDDTransformVisitor extends ADLCPPGenericVisitor {

  /** Public handle for vector of generated Java ASTs that
    * correspond to datasets */
  public Vector datasets;

  /** Public handle for vector of generated Java ASTs that
    * correspond to factories */
  public Vector factories;

  /** Public handle for vector of generated Java ASTs that
    * correspond to directives */
  public Vector directives;

  /** Public handle for the generated Java AST that
    * corresponds to the interface for the TDD fields */
  public ASTTranslationUnit constantsClass;

  /** Handle for the generated Java AST that corresponds
    * to the source tddclass (contains only test functions) */
  private ASTTranslationUnit TDDClass;

  /** Handle for the "include" declarations of source tdd class */
  private ASTIncludeFileList includeList;

  /** Handle for the "include" declarations of generated CST class */
  private ASTIncludeFileList includeListCST;

  /** Handle for the "include" declarations of generated TDD class */
  private ASTIncludeFileList includeListTDD;

  /** Handle for the declaration of the TDD local fields
    * (will be used to construct constantsClass)
    * @see #constantsClass */
  private Vector constants;

  /** Name of the TDD class (i.e. name of the source file prefix) */
  private static String unitName; // tddClassName

  /** Names of the ACOs to generate. */
  private Vector acoNames = null;

  /** Names of the TDD classes to generate. */
  private Vector tddClassNames = null;

  /** Names of the TDD constant classes to generate. */
  private Vector cstClassNames = null;

  /** Names of the TDD datasets to generates. */
  private Vector tddDatasetNames = null;

  /** Names of the TDD factories to generates. */
  private Vector tddFactoryNames = null;

  /** Names of the TDD test directives to generates. */
  private Vector tddTestDirectiveNames = null;


  static final private boolean debug = false;


  /** 
    * Entry point. Initializes needed handles and launches the visit.
    * @param topNode the root node of the tree to visit
    * @return the root node of the transformed tree
    */
  public Node translate(Node topNode) throws ParseException {

    init();
    return (Node)topNode.jjtAccept(this, null);
  }


  /**
    * Initialization of handles.
    */
  private void init() {

    datasets = new Vector();
    directives = new Vector();
    factories = new Vector();
    constants = new Vector();
    constantsClass = null;

    /** Names of the ACOs to generate. */
    acoNames = new Vector();

    /** Names of the TDD classes to generate. */
    tddClassNames = new Vector();

    /** Names of the TDD constant classes to generate. */
    cstClassNames = new Vector();

    /** Names of the TDD datasets to generate. */
    tddDatasetNames = new Vector();

    /** Names of the TDD factories to generate. */
    tddFactoryNames = new Vector();

    /** Names of the TDD test directives to generate. */
    tddTestDirectiveNames = new Vector();

  }


  /** Names of the ACOs to generate. */
  public Vector getACONames() { return acoNames; }

  /** Names of the TDD classes to generate. */
  public Vector getTDDClassNames() { return tddClassNames; }

  /** Names of the TDD constant classes to generate. */
  public Vector getTDDConstantNames() { return cstClassNames; }
  
  /** Names of the TDD datasets to generates. */
  public Vector getTDDDatasetNames() { return tddDatasetNames; }

  /** Names of the TDD factories to generates. */
  public Vector getTDDFactoryNames() { return tddFactoryNames; }

  /** Names of the TDD test directives to generates. */
  public Vector getTDDTestDirectiveNames() { return tddTestDirectiveNames; }



  /**
    * Visit pattern for a node of type ASTTranslationUnit.
    * <P><B>Algorithm</B><UL>
    * <LI>Stores the ASTIncludeFileDeclaration children nodes in includeDecls;
    * <LI>Visits the ASTTDD_ClassDeclaration child node.
    * </UL>
    * @param node the ASTADL_CompilationUnit node to visit
    * @param data unused
    * @return the transformed ASTADL_CompilationUnit node
    */
  public Object visit(ASTTranslationUnit node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTranslationUnit");

    // save former stm context
    Object savedStmContext = SimpleNode.stm.getContext();

    // set stm context for this ast
    try {
      Object stmContext = node.getProperty("stmContext");
      SimpleNode.stm.setContext(stmContext);
    }
    catch ( PropertyAccessException e ) { }

    unitName = node.getName();

    Node child;
    TDDClass = new
      ASTTranslationUnit(ADLCPPParserTreeConstants.JJTTRANSLATIONUNIT);

    includeList = (ASTIncludeFileList)node.jjtGetChild(0);
    try {
      includeListCST = (ASTIncludeFileList)includeList.clone();
    } catch (CloneNotSupportedException e) {
      includeListCST = includeList;
    }
    includeListTDD = new
      ASTIncludeFileList(ADLCPPParserTreeConstants.JJTINCLUDEFILELIST);
    addInclude(includeListTDD, "ADLTest.hh");
    addInclude(includeListTDD, "ADLString.hh");

    // visit class declaration node, i.e., the last one
    // doing so we skip the TDDUseDeclaration node
    node.jjtGetChild(node.jjtGetNumChildren() - 1).jjtAccept(this, data);

    TDDClass.setName(unitName + "TDD");
    TDDClass.setNodeExtension("cpp");

    // restore former stm context
    SimpleNode.stm.setContext(savedStmContext);

    return TDDClass;
  }

  /**
    * Visit pattern for a node of type ASTTDDClassDecl.
    * <P><B>Algorithm</B><OL>
    * <LI>Creates an ASTClassDeclaration node for the class
    * "&#60;tddclass name&#62;TDD". This class has an implementation child
    * for interface "&#60;tddclass name&#62;_CST".
    * <LI>The input node children that correspond to method declarations
    * are added to the created node while the others are just visited
    * (they don't return anything but create the public handles datasets,
    * factories and directives).
    * <LI>Calls makeConstantsInterface to create the interface
    * "&#60;tddclass name&#62;_CST".
    * <LI>Returns the created node.
    * </OL>
    * @param node the ASTTDD_ClassDeclaration node to visit
    * @param data unused
    * @return the created node (ASTClassDeclaration)
    */
  public Object visit(ASTTDDClassDecl node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_ClassDeclaration");

    addInclude(includeListTDD, unitName + "_CST.cpp");

    // update tdd classes names list
    tddClassNames.addElement(unitName + "TDD");

    // update constant classes names list
    cstClassNames.addElement(unitName + "_CST");


    parentChild(TDDClass, includeListTDD, 0);

    ASTDeclaration declaration = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    parentChild(TDDClass, declaration, 1);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    parentChild(declaration, declSpec, 0);

    ASTClassSpecifier classSpec = new
      ASTClassSpecifier(ADLCPPParserTreeConstants.JJTCLASSSPECIFIER);
    parentChild(declSpec, classSpec, 0);
    classSpec.setClassKind("class");
    classSpec.setName(unitName + "TDD");

    ASTBaseClause baseClause = new
      ASTBaseClause(ADLCPPParserTreeConstants.JJTBASECLAUSE);
    parentChild(classSpec, baseClause, 0);

    ASTBaseSpecifier baseSpec1 = new
      ASTBaseSpecifier(ADLCPPParserTreeConstants.JJTBASESPECIFIER);
    baseSpec1.setAccessSpec(CModifier.PUBLIC);
    baseSpec1.setName("ADLTest");
    parentChild(baseClause, baseSpec1, 0);

    ASTBaseSpecifier baseSpec2 = new
      ASTBaseSpecifier(ADLCPPParserTreeConstants.JJTBASESPECIFIER);
    baseSpec2.setAccessSpec(CModifier.PUBLIC);
    baseSpec2.setName(unitName + "_CST");
    parentChild(baseClause, baseSpec2, 1);

    ASTClassMemberList classMemberList = new
      ASTClassMemberList(ADLCPPParserTreeConstants.JJTCLASSMEMBERLIST);
    parentChild(classSpec, classMemberList, 1);

    ASTAccessSpecifier member = new
      ASTAccessSpecifier(ADLCPPParserTreeConstants.JJTACCESSSPECIFIER);
    parentChild(classMemberList, member, 0);
    member.setAccessSpecifierName("public");

    makeConstantsClass();

    node.childrenAccept(this, data);

    return null;
  }


  /**
    * Visit pattern for a node of type ASTTDDDatasetDecl.
    * <BR>Creates a CPPDatasetBuilder with as parameters the current node
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current dataset definition, and adds this field in the "datasets" handle.
    * @see CPPDatasetBuilder
    * @param node the ASTTDDDatasetDecl node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDDDatasetDecl node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDDDatasetDeclaration");

    CPPDatasetBuilder jdb =
      new CPPDatasetBuilder(node, unitName, includeList);

    // update tdd dataset name list
    tddDatasetNames.addElement(jdb.getClassName());

    datasets.addElement(jdb.compilUnit);
    return null;
  }


  /**
    * Visit pattern for a node of type ASTTDDFactoryDef.
    * <BR>Creates a CPPFactoryBuilder with as parameters the current node
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current factory definition, and adds this field in the "factories" handle.
    * @see CPPFactoryBuilder
    * @param node the ASTTDDFactoryDef node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDDFactoryDef node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_FactoryDeclaration");

    CPPFactoryBuilder jfb =
      new CPPFactoryBuilder(node, unitName, includeList);

    // update tdd factory name list
    tddFactoryNames.addElement(jfb.getClassName());    

    factories.addElement(jfb.compilUnit);

    return null;
  }


  /**
    * Visit pattern for a node of type ASTTDDTestDirective.
    * <BR>Creates a CPPDirectiveBuilder with as parameters the current node,
    * and the tdd class name. Gets the "compilUnit" of this builder, that
    * corresponds as the root of the AST for the class that implements the
    * current directive definition, and adds this field in the "directives" handle.
    * @see CPPDirectiveBuilder
    * @param node the ASTTDDTestDirective node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDDTestDirective node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_TestDirective");

    CPPDirectiveBuilder jdb = new CPPDirectiveBuilder(node, unitName,
						      includeList);

    // update tdd test directive names list
    tddTestDirectiveNames.addElement(jdb.getClassName());

    directives.addElement(jdb.compilUnit);

    // update list of ACO names
    Enumeration e = jdb.getACONames().elements();
    while ( e.hasMoreElements() ) {
      String acoName = (String)e.nextElement();
      if ( ! acoNames.contains(acoName) ) {
	acoNames.addElement(acoName);
      }
    }

    return null;
  }
  

  /**
    * Visit pattern for a node of type ASTTDDFieldDeclaration.
    * <BR>Creates Java field declaration that corresponds to the TDD field
    * declaration, and adds it to the "constants" handle.
    * @see #makeConstantsClass
    * @param node the ASTTDDFieldDeclaration node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTTDDFieldDeclaration node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_FieldDeclaration");

    ASTTDDFieldDeclaration cloned;
    try {
      cloned = (ASTTDDFieldDeclaration)node.clone();
    }
    catch(CloneNotSupportedException e) { cloned = null; }

    ASTClassMemberList memberList =
      (ASTClassMemberList)constantsClass.jjtGetChild(1).jjtGetChild(0).jjtGetChild(0).jjtGetChild(0);

    ASTDeclaration decl = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);

    ASTDeclarationSpecifiers declSpec = (ASTDeclarationSpecifiers)node.jjtGetChild(0);
    parentChild(decl, declSpec, 0);
    if (declSpec.getModifier() != 0x0000)
      CPPTDDTransformVisitor.warningMessage("Type modifier(s) for tdd fields will be ignored",
                                            declSpec.getLineNumber());

    declSpec.setModifier(CModifier.STATIC);

    if (node.jjtGetNumChildren() == 3) {
      parentChild(decl, node.jjtGetChild(1), 1);
    }
    else {
      ASTInitDeclaratorList initList = new
        ASTInitDeclaratorList(ADLCPPParserTreeConstants.JJTINITDECLARATORLIST);
      parentChild(decl, initList, 1);
      for (int i = 1; i < node.jjtGetNumChildren(); i += 2) {
        parentChild(initList, node.jjtGetChild(i), (i-1)/2);
      }
    }

    parentChild(memberList, decl, memberList.jjtGetNumChildren());


    ASTDeclaration def; Node declarator, declbis;
    int nbChildren = constantsClass.jjtGetNumChildren();
    ASTDeclarationSpecifiers defSpec = null;

    for (int i = 1; i < cloned.jjtGetNumChildren(); i += 2) {
      def = new
        ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
      try {
        defSpec = (ASTDeclarationSpecifiers)
          ((ASTDeclarationSpecifiers)cloned.jjtGetChild(0)).clone();
      } catch (CloneNotSupportedException e) {}
      parentChild(def, defSpec, 0);

      declarator = cloned.jjtGetChild(i);
      declbis = declarator;
      while(declbis instanceof ASTDeclarator) {
        declbis = declbis.jjtGetChild(1);
      }
      if (declbis.jjtGetNumChildren() > 1)
        errorMessage("No suffix for declaration of constant",
                     ((SimpleNode)declbis).getLineNumber());
      if (declbis.jjtGetChild(0) instanceof ASTQualifiedId) {
        ((ASTQualifiedId)declbis.jjtGetChild(0))
          .setName(unitName + "_CST::" +
                   ((ASTQualifiedId)declbis.jjtGetChild(0)).getName());
      }
      else errorMessage("tdd class constants cannot have nested declarators",
                        ((SimpleNode)declbis).getLineNumber());

      // visit Initializer children to fix up IdExpr names
      SimpleNode initializer = (SimpleNode)node.jjtGetChild(i+1);
      initializer.jjtAccept(this, data);

      ASTInitDeclarator initDecl = new
        ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);
      parentChild(def, initDecl, 1);
      parentChild(initDecl, declarator, 0);
      parentChild(initDecl, initializer, 1);
      parentChild(constantsClass, def, nbChildren++);
    }
    return null;
  }
  
  
  /**
    * Visit pattern for a node of type IdExpr.
    * <P><B>Algorithm</B><UL>
    * <li>Gets defining tdd class name
    * <LI><B>IF</B> the name is not a dataset name, then it is a
    *    constant:<ul>
    * <li>Sets name to
    *    <samp>&lt;defining tdd class name>_CST.&lt;constant name></samp>
    * </UL></UL>
    * @see CPPBuilder#makeQualifiedName(String, String, String)
    * @param node the IdExpr node to visit
    * @param data unused
    * @return nothing
    */
  public Object visit(ASTIdExpr node, Object data) throws ParseException {
    if (debug)
      System.out.println("visiting IdExpr : " + node.getIdName());

    boolean isDS = node.isDataset;
    String TDDClassName = null;
    String datasetName;
    try {
      TDDClassName = (String)node.getProperty("enclosingConstructName");
    }
    catch (PropertyAccessException e) { }
    
    if (! isDS) { // field name
      if (TDDClassName != null && !TDDClassName.equals("")) {
	// TDD field name
	datasetName = TDDClassName + "_CST::" + node.getIdName();
	
	addInclude(includeListCST, TDDClassName + "_CST.cpp");
	
	node.setIdName(datasetName);
      }
    }

    return null;
  }


  /**
    * Visit pattern for a node of type ASTFunctionDefinition.
    * <BR>Returns the node unchanged except for last child (ASTCompoundStatement)
    * which is visited by an RemoveADLVisitor to transform its casual
    * ADL/ADL_new expressions.
    * @see CPPRemoveADLVisitor
    * @param node the ASTFunctionDefinition node to visit
    * @param data unused
    * @return the transformed node
    */
  public Object visit(ASTFunctionDefinition node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTFunctionDefinition");

    int bodyIndex = node.jjtGetNumChildren()-1;
    if (! (node.jjtGetChild(bodyIndex) instanceof ASTCompoundStatement))
      errorMessage("Test functions must have a body",
                   ((SimpleNode)node.jjtGetChild(bodyIndex)).getLineNumber());

    if (debug)
      System.out.println("visiting ASTFunctionDefinition");

    RemoveADLVisitor v = new RemoveADLVisitor();
    node.jjtGetChild(bodyIndex).jjtAccept(v, null);
    Vector inclACO = v.getACONames();
    for (Enumeration e = inclACO.elements(); e.hasMoreElements(); ) {
      addInclude(includeListTDD, (String)e.nextElement() + ".cpp");
    }

    if (debug)
      System.out.println("visiting ASTFunctionDefinition");

    ASTClassMemberList memberList =
      (ASTClassMemberList)TDDClass.jjtGetChild(1).jjtGetChild(0).jjtGetChild(0).jjtGetChild(1);

    if (debug)
      System.out.println("visiting ASTFunctionDefinition");

    ASTFunctionDefinition fct = new
      ASTFunctionDefinition(ADLCPPParserTreeConstants.JJTFUNCTIONDEFINITION);
    for (int i = 0; i < bodyIndex; i++) {
      try {
        parentChild(fct,
                    (Node)((SimpleNode)node.jjtGetChild(i)).clone(),
                    i);
      }
      catch(CloneNotSupportedException e) {}
    }
    if (!(fct.jjtGetChild(0) instanceof ASTDeclarationSpecifiers))
      errorMessage("Test function are not constructors",
                   ((SimpleNode)fct.jjtGetChild(0)).getLineNumber());

    Node fctdecl = node.jjtGetChild(1);
    while (fctdecl instanceof ASTFunctionDeclarator)
      fctdecl = fctdecl.jjtGetChild(1);
    parentChild(fctdecl,
                makeQualifiedId(unitName + "TDD::" + ((ASTQualifiedId)fctdecl.jjtGetChild(0)).getName()),
                0);

    parentChild(memberList, fct, memberList.jjtGetNumChildren());

    parentChild(TDDClass, node, TDDClass.jjtGetNumChildren());


    Enumeration e = v.getACONames().elements();
    while ( e.hasMoreElements() )
    {
       String acoName = (String)e.nextElement();
       if ( ! acoNames.contains(acoName) )
       {
          acoNames.addElement(acoName);
       }
    }

    return null;
  }


  /**
    * Creates the AST for interface "&#60;tddclass name&#62;_CST", that
    * gathers the tdd field declarations stored in the "constants" handle.
    * This interface will be implemented by all created classes.
    * @see #visit(ORG.opengroup.parsers.adljava.ASTTDD_FieldDeclaration, Object)
    */
  private void makeConstantsClass() {

    if (debug)
      System.out.println("makeConstantsInterface");

    constantsClass = new
      ASTTranslationUnit(ADLCPPParserTreeConstants.JJTTRANSLATIONUNIT);
    constantsClass.setName(unitName + "_CST");
    constantsClass.setNodeExtension("cpp");

    parentChild(constantsClass, includeListCST, 0);

    ASTDeclaration declaration = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    parentChild(constantsClass, declaration, 1);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    parentChild(declaration, declSpec, 0);

    ASTClassSpecifier classSpec = new
      ASTClassSpecifier(ADLCPPParserTreeConstants.JJTCLASSSPECIFIER);
    parentChild(declSpec, classSpec, 0);
    classSpec.setClassKind("class");
    classSpec.setName(unitName + "_CST");

    ASTClassMemberList cstMemberList = new
      ASTClassMemberList(ADLCPPParserTreeConstants.JJTCLASSMEMBERLIST);
    parentChild(classSpec, cstMemberList, 0);

    ASTAccessSpecifier publicAccess = new
      ASTAccessSpecifier(ADLCPPParserTreeConstants.JJTACCESSSPECIFIER);
    publicAccess.setAccessSpecifierName("public");
    parentChild(cstMemberList, publicAccess, 0);
  }


  public void addInclude(ASTIncludeFileList incList, String inclFile) {

    if (debug)
      System.out.println("addInclude: " + inclFile);

    ASTIncludeFileDeclaration incl = new
      ASTIncludeFileDeclaration(ADLCPPParserTreeConstants.JJTINCLUDEFILEDECLARATION);
    // waiting for Wassel to commit in development trunk
//     incl.setFileName(/*"\"" +*/ inclFile /*+ "\""*/);
    incl.setFileName("\"" + inclFile + "\"");
    parentChild(incList, incl, incList.jjtGetNumChildren());
  }


  /**
    * Print an error message.
    * @param msg the error message
    * @param lineNumber the line number of the error in the source file
    */
  static public void errorMessage(String msg, int lineNumber) throws ParseException {

    if (lineNumber == 0) {
      System.err.println(unitName + ": " + msg);
    }
    else {
      System.err.println(unitName + ".tdd:" + lineNumber + ": " + msg);
    }
    throw new ParseException();
  }


  /**
    * Print a warning message.
    * @param msg the warning message
    * @param lineNumber the line number of the error in the source file
    */
  static public void warningMessage(String msg, int lineNumber) {

    if (lineNumber == 0) {
      System.err.println(unitName + " * WARNING: " + msg);
    }
    else {
      System.err.println(unitName + ".tdd:" + lineNumber + " * WARNING: " + msg);
    }
  }

}

// ***********************************************************************
// History Log:
// $Log: CPPTDDTransformVisitor.java,v $
// Revision 1.13  1998/10/14 08:17:27  mallet
// Bugs fixed: pr/155, pr/156: removed useless index of anonymous test
// directives.
//
// Revision 1.12  1998/06/30 13:12:24  mallet
// Replaced import of PropertyAccessException in
// ORG.opengroup.adl.parsers by an import on demand (.*).
//
// Revision 1.11  1998/06/12 16:07:50  mallet
// Added a new field includeList to hold the list of include files in the
// TDD file: other lists are cloned from this one.
// Changed creation of Dataset and Factory builder to pass the
// includeList field as argument (according to changes in
// CPPDatasetBuilder, CPPFactoryBuilder and CPPBuilder).
// Added visit of initializer for TDD_FieldDeclaration to fix up name of
// IdExpr nodes in case of a "use"d TDD field.
// Added visit method for IdExpr to fix name in case of a "use"d TDD field.
//
// Revision 1.10  1998/06/05 14:32:43  mallet
// Changed location of method makeQualifiedName from NameAux to JBuilder.
// Added code to get the list of used ACOs from the JDirectiveBuilder:
// this fixes a bug where those ACOs were not listed in the dependencies
// of the TDD class.
//
// Revision 1.9  1998/05/27 12:07:03  cinard
// Visitor now sets the symbol table manager context before visiting the ast.
//
// Revision 1.8  1998/04/01 16:49:51  mallet
// Replaced ADLNode by SimpleNode.
//
// Revision 1.7  1998/03/31 15:00:37  bareau
// Add "" to include file strings.
//
// Revision 1.6  1998/03/11 14:54:12  bareau
// Added new warnings: type modifiers in tdd files (dataset definitions, factory/directive
// parameters) are ignored. New static method warningMessage in CPPTDDTransformVisitor.
// Improved support for casted expressions (there were useless double cast expressions
// in generated code).
//
// Revision 1.5  1998/03/10 17:26:44  mallet
// Merged v1.1 with development trunk
//
// Revision 1.4.2.4  1998/03/09 10:28:37  bareau
// Cleaned up the code for error handling (wrote a static method
// "errorMessage" in CPPTDDTransformVisitor).
//
// Revision 1.4.2.3  1998/03/04 17:39:44  bareau
// Added new checks.
// Cleaned up the code.
//
// Revision 1.4.2.2  1998/03/04 10:52:52  bareau
// The entry point for the tdd visitor is now a method "translate"
// that takes as parameter the root node of the source tree, performs
// necessary initializations and launches the visit.
//
// Revision 1.4.2.1  1998/03/03 10:59:56  bareau
// Set the debug flag to false.
// Bug fix in CPPDatasetExprVisitor: ASTTDDDatasetSingleton nodes were
// sent to the unparser.
//
// Revision 1.4  1998/02/27 13:07:31  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/26 17:50:00  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// Revision 1.2  1998/02/12 16:03:42  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/12 13:48:21  bareau
// Classes used in the TDD translation. Added because of code freeze: this is a
// pre-pre-version (it does not even compile yet).
//
// ***********************************************************************
