// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/CPPTypeAux.java,v $
// $Revision: 1.7 $
// $Date: 1998/06/30 13:11:07 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        CPPTypeAux
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Feb 6, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.PropertyAccessException;
import ORG.opengroup.adl.parsers.adlcpp.*;
import ORG.opengroup.adl.symboltable.*;
import ORG.opengroup.adl.symboltable.c.*;

/**
 * Auxiliary class that gathers static methods to deal with types.
 * @author Cyrille Bareau
 */

public final class CPPTypeAux {

  /**
    * Returns "true" if the type parameter is a primitive type
    * (boolean or numeric).
    */
  public static boolean isPrimitiveType(ASTDeclarationSpecifiers type) {

    return (type.jjtGetChild(0) instanceof ASTBuiltinTypeSpec);
  }


  /**
    * Returns "true" if the function parameter has no return type.
    */
  public static boolean isVoid(ASTDeclarationSpecifiers declSpec, Node declTor) {

    if (declTor.jjtGetChild(0) instanceof ASTPtrOperator) { // return pointer
      return false;
    }
    else return isVoid(declSpec);
  }


  /**
    * Returns "true" if the type parameter is the void type.
    */
  public static boolean isVoid(ASTDeclarationSpecifiers type) {

    if (type.jjtGetChild(0) instanceof ASTBuiltinTypeSpec)
      return ((ASTBuiltinTypeSpec)type.jjtGetChild(0)).getTypeName().equals("void");
    else return false;
  }


  /**
    * Returns the default value for the parameter type
    * (0 for builtin types, null for objects, etc.)
    */
  public static Node getDefaultValue(LType type, boolean staticInitializer) {

    if (type.isA(SymbolId.ARRAY)) {
      if (staticInitializer) {
	ASTInitializer initArray = new
	  ASTInitializer(ADLCPPParserTreeConstants.JJTINITIALIZER);
	ADLCPPGenericVisitor.parentChild(initArray,
					 ADLCPPGenericVisitor.makeConstant("0"),
					 0);
	return initArray;
      } else
      throw new Error("Cannot handle array in such expression!");
    }
    else if (type.isA(SymbolId.REFERENCE | SymbolId.CLASS)) {
      // for "foo &" or "foo", return "foo()" and emit warning message
      System.out.println("Warning: " +
			 "be sure you have a default ctor for class " +
			 type.getName());
      return new PrimaryExpressionBuilder(type.getName(),
					  new ArgumentListBuilder()
					  .generatedAST())
	.generatedAST();
    }
    else if (type.isA(SymbolId.STRUCT | SymbolId.UNION))
      // for "struct foo", return "foo()", idem for union
      return new PrimaryExpressionBuilder(type.getName(),
					  new ArgumentListBuilder()
					  .generatedAST())
	.generatedAST();
    else if (type.isA(SymbolId.BUILTIN_TYPE | SymbolId.ENUM))
      return ADLCPPGenericVisitor.makeConstant("0");
    else if (type.isA(SymbolId.POINTER |
		      SymbolId.POINTER_TO_FUNCTION |
		      SymbolId.POINTER_TO_FUNCTION_MEMBER |
		      SymbolId.POINTER_TO_FIELD_MEMBER))
      return ADLCPPGenericVisitor.makeIdExpr("NULL");
    else
      throw new Error("Cannot handle " +
		      SymbolId.stringize(type.getSymbolId()) +
		      " in such expression!");
  }
}

// ***********************************************************************
// History Log:
// $Log: CPPTypeAux.java,v $
// Revision 1.7  1998/06/30 13:11:07  mallet
// Fixed bug in method getDefaultValue(): now handles more cases to
// determine the appropriate initialization value. A new argument has
// been added to indicate whether the returned value appears in a static
// initializer or in an expression. This makes a difference for array for
// instance since they cannot be initialized to {0} when this value is
// used in a conditional expression.
//
// Revision 1.6  1998/04/28 12:39:26  cinard
// Updated code to take into account replacement of CSymbolTableMgr,
// CSymbolFactory and CBuiltinType by CPPSymbolTableMgr, CPPSymbolFactory
// and CPPBuiltinType.
//
// Revision 1.5  1998/03/16 10:14:46  rami
// Added getDefaultValue(LType) returning node for default value of a type.
//
// Revision 1.4  1998/02/27 13:07:31  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/26 17:50:00  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// Revision 1.2  1998/02/12 16:03:43  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/12 13:48:22  bareau
// Classes used in the TDD translation. Added because of code freeze: this is a
// pre-pre-version (it does not even compile yet).
//
// ***********************************************************************
