// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/DeclarationBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:32 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        DeclarationBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 11, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ declaration
  * (Declaration).
  * @author Rami KHEMAKHEM
  */

public class DeclarationBuilder {

  /** Root of the generated tree */
  private ASTDeclaration root;

  /**
    * Creates a declaration with 2 children.
    * @param lhs First child is declaration specifiers.
    * @param rhs Second child can be Init declarator,
    * declarator or direct declarator.
    */
  public DeclarationBuilder (ASTDeclarationSpecifiers lhs, Node rhs) {

    root = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    ADLCPPGenericVisitor.parentChild(root, lhs, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a non-initialized declaration with 2 children.
    * @param lhs The child of declaration specifiers.
    * @param rhs The second child of the declaration.
    */
  public DeclarationBuilder (Node lhs, Node rhs) {

    root = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    ASTDeclarationSpecifiers declSpec = 
      new ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild(declSpec, lhs, 0);
    ADLCPPGenericVisitor.parentChild(root, declSpec, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }


  /**
    * Creates a non-initialized declaration with 2 children.
    * @param type The builtin type, child of declaration specifiers.
    * @param rhs The second child of the declaration.
    */
  public DeclarationBuilder (ASTBuiltinTypeSpec type, Node rhs) {

    root = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    ASTDeclarationSpecifiers declSpec = 
      new ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild(declSpec, type, 0);
    ADLCPPGenericVisitor.parentChild(root, declSpec, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }

  /**
    * Creates a non-initialized declaration with 2 children.
    * @param type The qualified type, child of declaration specifiers.
    * @param rhs The second child of the declaration.
    */
  public DeclarationBuilder (ASTQualifiedType type, Node rhs) {

    root = new
      ASTDeclaration(ADLCPPParserTreeConstants.JJTDECLARATION);
    ASTDeclarationSpecifiers declSpec = 
      new ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild(declSpec, type, 0);
    ADLCPPGenericVisitor.parentChild(root, declSpec, 0);
    ADLCPPGenericVisitor.parentChild(root, rhs, 1);
  }

  /**
    * @return the root of the generated tree
    */
  public ASTDeclaration generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DeclarationBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:32  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:44  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:02  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:14  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
