// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/DeclaratorBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:07:32 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        DeclaratorBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 26, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ declarator
  * (Declarator).
  * @author Rami KHEMAKHEM
  */

public class DeclaratorBuilder {

  /** Root of the generated tree */
  private ASTDeclarator root;

  /** Second child of declarator */
  private ASTDirectDeclarator directDecl;

  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, ASTDirectDeclarator decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ptr, 0);
    ADLCPPGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr1 The 1st pointer of the declarator.
    * @param ptr1 The snd pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr1, ASTPtrOperator ptr2, ASTDirectDeclarator decl) {

    this(ptr1, new DeclaratorBuilder(ptr2, decl).generatedAST());
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (String ptr, ASTDirectDeclarator decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makePtrOperator(ptr), 0);
    ADLCPPGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (String ptr1, String ptr2, ASTDirectDeclarator decl) {

    this(ptr1, new DeclaratorBuilder(ptr2, decl).generatedAST());
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, ASTDeclarator decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ptr, 0);
    ADLCPPGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The direct declarator.
    */
  public DeclaratorBuilder (String ptr, ASTDeclarator decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makePtrOperator(ptr), 0);
    ADLCPPGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param decl The qualified id, child of direct declarator.
    */
  public DeclaratorBuilder (ASTPtrOperator ptr, ASTQualifiedId decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);
    directDecl = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(directDecl, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, ptr, 0);
    ADLCPPGenericVisitor.parentChild(root, decl, 1);
  }


  /**
    * Creates a declarator.
    * @param ptr The pointer of the declarator.
    * @param id The Qualified id, child of direct declarator.
    */
  public DeclaratorBuilder (String ptr, String decl) {

    root = new
      ASTDeclarator(ADLCPPParserTreeConstants.JJTDECLARATOR);
    directDecl = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(directDecl, ADLCPPGenericVisitor.makeQualifiedId(decl), 0);
    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makePtrOperator(ptr), 0);
    ADLCPPGenericVisitor.parentChild(root, directDecl, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DeclaratorBuilder.java,v $
// Revision 1.5  1998/02/27 13:07:32  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:03:44  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/02/12 10:25:51  bareau
// Removed target tdd in the Makefile for the code freeze.
// Minor patches in DeclaratorBuilder and ParameterDeclarationBuilder.
//
// Revision 1.2  1998/02/04 14:02:03  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:15  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
