// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/DirectDeclaratorBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:32 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        DirectDeclaratorBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 26, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ directDeclarator
  * (DirectDeclarator).
  * @author Rami KHEMAKHEM
  */

public class DirectDeclaratorBuilder {

  /** Root of the generated tree */
  private ASTDirectDeclarator root;


  /**
    * Creates a direct declarator.
    * @param first First child of the direct declarator.
    * @param second Second child of the direct declarator.
    */
  public DirectDeclaratorBuilder (Node first, Node second) {

    root = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, first, 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }


  /**
    * Creates a direct declarator.
    * @param id The qualified id, child of the direct declarator.
    * @param declSuff The declaration suffixes.
    */
  public DirectDeclaratorBuilder (String id, Node declSuff) {

    root = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeQualifiedId(id), 0);
    ADLCPPGenericVisitor.parentChild(root, declSuff, 1);
  }


  /**
    * Creates a direct declarator.
    * @param id The qualified id, child of the direct declarator.
    * @param declSuff The declaration suffixes.
    */
  public DirectDeclaratorBuilder (ASTQualifiedId id, Node declSuff) {

    root = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, id, 0);
    ADLCPPGenericVisitor.parentChild(root, declSuff, 1);
  }


  /**
    * Creates a direct declarator.
    * @param id String to build the qualified id, child of the direct declarator.
    */
  public DirectDeclaratorBuilder (String id) {

    root = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeQualifiedId(id), 0);
  }


  /**
    * Creates a direct declarator.
    * @param id The qualified id, child of the direct declarator.
    */
  public DirectDeclaratorBuilder (ASTQualifiedId id) {

    root = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, id, 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDirectDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DirectDeclaratorBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:32  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:45  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:03  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:15  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
