// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Sourc$
// $Revision: 1.1 $
// $Date: 1998/03/12 16:15:03 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        DotAccessSuffixBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Mar 10, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ dot access suffix:
  *    "."  IdExpr  [ ArgumentList ]
  * @author Rami KHEMAKHEM
  */

public class DotAccessSuffixBuilder {

  /** Root of the generated tree */
  private ASTDotAccessSuffix root;


  /**
    * Creates a dot access suffix composed of 
    * "." and an expression
    * @param expr The expression
    */
  public DotAccessSuffixBuilder (Node expr) {

    root = new
      ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
  }


  /**
    * Creates a reference access suffix composed of 
    * "." and an Id Expression
    * @param expr An id expression
    */
  public DotAccessSuffixBuilder (ASTIdExpr expr) {

    root = new
      ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
  }

  /**
    * Creates a reference access suffix composed of 
    * "." and an Id Expression
    * @param expr The name of the id expression 
    */
  public DotAccessSuffixBuilder (String expr) {

    root = new
      ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(expr), 0);
  }

  /**
    * Creates a reference access suffix composed of 
    * ".", an IdExpr and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public DotAccessSuffixBuilder (Node expr, Node arg) {

    root = new
      ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
    ADLCPPGenericVisitor.parentChild(root, arg, 1);
  }

  /**
    * Creates a reference access suffix composed of 
    * ".", an IdExpr and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public DotAccessSuffixBuilder (String expr, Node arg) {

    root = new
      ASTDotAccessSuffix(ADLCPPParserTreeConstants.JJTDOTACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(expr), 0);
    ADLCPPGenericVisitor.parentChild(root, arg, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTDotAccessSuffix generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: DotAccessSuffixBuilder.java,v $
// Revision 1.1  1998/03/12 16:15:03  rami
// Added to build a reference access suffix node.
//
// ***********************************************************************
