// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/FactoryVisitor.java,v $
// $Revision: 1.3 $
// $Date: 1998/06/18 10:32:11 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.cpp
// NAME:        FactoryVisitor.java
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 17, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
 * Transformation visitor.  Must change a "return expr;",
 * where expr is of primitive type, into "return new &#60;JavaType&#62;(expr);"
 * where JavaType is the Java wrapper object type that corresponds to the
 * primitive type (Integer for int, etc.)
 * <BR>
 * Used in factories: the runtime mechanism for factories accept
 * only object elements.
 * @see CPPFactoryBuilder
 * @author Cyrille Bareau
 */

public class FactoryVisitor extends ADLCPPGenericVisitor {

  static final private boolean debug = false;

  /**
    * Visit pattern for a node of type ASTReturnStatement.
    * @param node the ASTReturnStatement to visit.
    * @param data a String that contains the Java wrapper type.
    * @return the transformed node.
    */
  public Object visit(ASTReturnStmt node, Object data) {

    if (debug)
      System.out.println("visiting FactoryVisitor.ASTReturnStatement");

    if (node.jjtGetNumChildren() == 0) {
      return node;
    }

    // return *((<type> *) <expr>);
    ASTUnaryExpr unary = new
      ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
    unary.setOp("*");

    ASTParenthExpr parenth = new
      ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);
    parentChild(unary, parenth, 0);

    ASTCastExpr castExpr = new
      ASTCastExpr(ADLCPPParserTreeConstants.JJTCASTEXPR);
    parentChild(parenth, castExpr, 0);
    parentChild(castExpr, node.jjtGetChild(0), 1);

    ASTTypeName typeName = new
      ASTTypeName(ADLCPPParserTreeConstants.JJTTYPENAME);
    parentChild(castExpr, typeName, 0);

    ASTDeclarationSpecifiers decl = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    parentChild(decl,
                makeQualifiedType(((Typed)node.jjtGetChild(0)).getType().getFullName()),
                0);
    parentChild(typeName, decl, 0);

    ASTAbstractDeclarator abstr = new
      ASTAbstractDeclarator(ADLCPPParserTreeConstants.JJTABSTRACTDECLARATOR);
    parentChild(typeName, abstr, 1);

    parentChild(abstr,
                makePtrOperator("PointerTo"),
                0);

    ASTReturnStmt createdNode = new
      ASTReturnStmt(ADLCPPParserTreeConstants.JJTRETURNSTMT);
    parentChild(createdNode, unary, 0);
    return createdNode;
  }

}

// ***********************************************************************
// History Log:
// $Log: FactoryVisitor.java,v $
// Revision 1.3  1998/06/18 10:32:11  mallet
// Fixed incorrect use of type short name instead of full one (FQN).
// Made use of makeQualidifedType systematic.
//
// Revision 1.2  1998/02/27 13:07:33  bareau
// Updated copyright notice.
//
// Revision 1.1  1998/02/26 17:50:01  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// ***********************************************************************
