// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/InitDeclaratorBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:33 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        InitDeclaratorBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Jan 05, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ init declarator
  * (InitDeclarator).
  * @author Rami KHEMAKHEM
  */

public class InitDeclaratorBuilder {

  /** Root of the generated tree */
  private ASTInitDeclarator root;


  /**
    * Creates an initialized declarator.
    * @param  decl The first child is declarator or direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(Node decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The first child is declarator or direct declarator.
    * @param init The initializer -) an Id expression.
    */
  public InitDeclaratorBuilder(Node decl, String init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(init), 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The declarator (with pointer operator).
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(ASTDeclarator decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The declarator (with pointer operator).
    * @param init The initializer -) an Id expression.
    */
  public InitDeclaratorBuilder(ASTDeclarator decl, String init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(init), 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(ASTDirectDeclarator decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The direct declarator.
    * @param init The initializer -) an id expression.
    */
  public InitDeclaratorBuilder(ASTDirectDeclarator decl, String init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);

    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(init), 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The name of the direct declarator.
    * @param init The initializer.
    */
  public InitDeclaratorBuilder(String decl, Node init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(directDecl, ADLCPPGenericVisitor.makeQualifiedId(decl), 0);
    ADLCPPGenericVisitor.parentChild(root, directDecl, 0);
    ADLCPPGenericVisitor.parentChild(root, init, 1);

  }

  /**
    * Creates an initialized declarator.
    * @param  decl The name of the direct declarator.
    * @param init The initializer -) an id expression.
    */
  public InitDeclaratorBuilder(String decl, String init) {

    root = new
      ASTInitDeclarator(ADLCPPParserTreeConstants.JJTINITDECLARATOR);
    ASTDirectDeclarator directDecl = new
      ASTDirectDeclarator(ADLCPPParserTreeConstants.JJTDIRECTDECLARATOR);

    ADLCPPGenericVisitor.parentChild(directDecl, ADLCPPGenericVisitor.makeQualifiedId(decl), 0);
    ADLCPPGenericVisitor.parentChild(root, directDecl, 0);
    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(init), 1);

  }

  /**
    * @return the root of the generated tree
    */
  public ASTInitDeclarator generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: InitDeclaratorBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:33  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:45  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:03  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:15  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
