// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/NameAux.java,v $
// $Revision: 1.4 $
// $Date: 1998/03/10 17:26:45 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        NameAux
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Feb 3, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
 * Auxiliary class that gathers static methods to deal with file names,
 * notably with imported features ("use" clause).
 * @author Cyrille Bareau
 */

public class NameAux {

  /** The name of the processed tdd class */
  private static String compilUnitName;


  /**
    * Initializes the compilUnitName field.
    */
  public static void setFileName(String name) {

    compilUnitName = name;
  }


  /**
    * Returns the compilUnitName field.
    */
  static public String getSourceName() {

    return compilUnitName;
  }


  /**
    * Builds the qualified name of the file that implements the feature
    * (dataset or factory) described by the parameters.
    * <BR>Examples:<UL>
    * <LI>Case of an imported dataset:
    * makeQualifiedName("S_", "pack.tdd1.DS3") returns "pack.S_tdd1_DS3"
    * <LI>Case of a local dataset (tddclass "MyTdd"):
    * makeQualifiedName("S_", "DS1") returns "S_MyTdd_DS1"
    * </UL>
    * @param prefix "F_" for a factory, "S_" for a dataset.
    * @param name the feature name, qualified if imported.
    * @return the built file name
    */
  public static String makeQualifiedName(String prefix, String name) {

      return prefix + compilUnitName + "_" + name;
  }

}

// ***********************************************************************
// History Log:
// $Log: NameAux.java,v $
// Revision 1.4  1998/03/10 17:26:45  mallet
// Merged v1.1 with development trunk
//
// Revision 1.3.2.1  1998/03/09 10:28:38  bareau
// Cleaned up the code for error handling (wrote a static method
// "errorMessage" in CPPTDDTransformVisitor).
//
// Revision 1.3  1998/02/27 13:07:34  bareau
// Updated copyright notice.
//
// Revision 1.2  1998/02/12 16:03:47  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/12 13:48:22  bareau
// Classes used in the TDD translation. Added because of code freeze: this is a
// pre-pre-version (it does not even compile yet).
//
// ***********************************************************************
