// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/NewExprBuilder.java,v $
// $Revision: 1.2 $
// $Date: 1998/02/27 13:07:34 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        NewExprBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Feb 17, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ "new" allocation expression
  * @author Cyrille Bareau
  */

public class NewExprBuilder {

  /** Root of the generated tree */
  private ASTNewExpr root;

  /**
    * Creates a "new" expression with 2 children.
    * @param name the name of the class whom an instance is being created
    * @param args the arguments of the constructor
    */
  public NewExprBuilder (ASTTypeName name, ASTArgumentList args) {

    root = new
      ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
    ADLCPPGenericVisitor.parentChild(root, name, 0);
    ADLCPPGenericVisitor.parentChild(root, args, 1);
  }


  /**
    * Creates a "new" expression with 2 children.
    * @param name the name of the class whom an instance is being created
    * @param args the arguments of the constructor
    */
  public NewExprBuilder (ASTDeclarationSpecifiers name, ASTArgumentList args) {

    root = new
      ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
    ADLCPPGenericVisitor.parentChild(root, name, 0);
    ADLCPPGenericVisitor.parentChild(root, args, 1);
  }


  /**
    * Creates a "new" expression with 2 children.
    * @param name the name of the class whom an instance is being created
    * @param args the arguments of the constructor
    */
  public NewExprBuilder (ASTNewTypeId name, ASTArgumentList args) {

    root = new
      ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
    ADLCPPGenericVisitor.parentChild(root, name, 0);
    ADLCPPGenericVisitor.parentChild(root, args, 1);
  }


  /**
    * Creates a "new" expression with 2 children.
    * @param name the name of the class whom an instance is being created
    * @param args the arguments of the constructor
    */
  public NewExprBuilder (String name, ASTArgumentList args) {

    root = new
      ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);

    ASTDeclarationSpecifiers decl = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    ADLCPPGenericVisitor.parentChild(decl,
                                     ADLCPPGenericVisitor.makeQualifiedType(name),
                                     0);
    ADLCPPGenericVisitor.parentChild(root, decl, 0);
    ADLCPPGenericVisitor.parentChild(root, args, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTNewExpr generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: NewExprBuilder.java,v $
// Revision 1.2  1998/02/27 13:07:34  bareau
// Updated copyright notice.
//
// Revision 1.1  1998/02/26 17:50:01  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// ***********************************************************************
