// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/NotBuilder.java,v $
// $Revision: 1.1 $
// $Date: 1998/03/13 16:23:41 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        NotBuilder
//
// HISTORY:
//      Rami Khemakhem, The Open Group RI - Mar 13, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ "!" expression
  * (UnaryExpression).
  * @author Rami Khemakhem
  */

public class NotBuilder {

  /** Root of the generated tree */
  private ASTUnaryExpr root;


  /**
    * Creates a <I>not</I> of an expression.
    * @param node The input expression
    * @param parenth Flag to precise if the input expression
    * must be parenthetized (default yes)
    */
  public NotBuilder (Node node, boolean parenth) {

    root = new
      ASTUnaryExpr(ADLCPPParserTreeConstants.JJTUNARYEXPR);
    root.setOp("!");

    if (parenth) {
      ASTParenthExpr parenthnode = new
        ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);

      ADLCPPGenericVisitor.parentChild(root, parenthnode, 0);
      ADLCPPGenericVisitor.parentChild(parenthnode, node, 0);
    }
    else {
      ADLCPPGenericVisitor.parentChild(root, node, 0);
    }
  }


  /**
    * Creates a <I>not</I> of an expression (with parentheses).
    * @param node The input expression
    */
  public NotBuilder (Node node) {

    this(node, true);
  }
      

  /**
    * @return the root of the generated tree
    */
  public ASTUnaryExpr generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: NotBuilder.java,v $
// Revision 1.1  1998/03/13 16:23:41  rami
// Added builder for unary expression with "!" operator.
// Added NotBuilder.java and DotAccessSuffixBuilder.java in sources list.
//
// ***********************************************************************




