// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Sourc$
// $Revision: 1.6 $
// $Date: 1998/04/01 16:49:52 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.cpp
// NAME:        ParameterDeclarationBuilder
//
// HISTORY:
//      Rami Khemakhem, The Open Group RI - Dec 24, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ parameter declaration
  * @author Rami Khemakhem
  */

public class ParameterDeclarationBuilder {

  /** Root of the generated tree */
  private ASTParameterDeclaration root;


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTBuiltinTypeSpec type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild(root, declSpec, 0);
    ADLCPPGenericVisitor.parentChild(declSpec, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTQualifiedType type, ASTDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ASTDeclarationSpecifiers declSpec = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);

    ADLCPPGenericVisitor.parentChild(root, declSpec, 0);
    ADLCPPGenericVisitor.parentChild(declSpec, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter direct declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDirectDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter abstract declarator (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTAbstractDeclarator param) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
  }



  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
    ADLCPPGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter direct declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTDirectDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
    ADLCPPGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * Creates a parameter declaration
    * @param type The parameter type (AST)
    * @param param The parameter abstract declarator (AST)
    * @param assign The parameter assigned expression (AST)
    */
  public ParameterDeclarationBuilder (ASTDeclarationSpecifiers type, ASTAbstractDeclarator param, SimpleNode assign) {

    root = new
      ASTParameterDeclaration(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATION);

    ADLCPPGenericVisitor.parentChild(root, type, 0);
    ADLCPPGenericVisitor.parentChild(root, param, 1);
    ADLCPPGenericVisitor.parentChild(root, assign, 2);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTParameterDeclaration generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ParameterDeclarationBuilder.java,v $
// Revision 1.6  1998/04/01 16:49:52  mallet
// Replaced ADLNode by SimpleNode.
//
// Revision 1.5  1998/02/27 13:07:35  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:03:48  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/02/12 10:25:52  bareau
// Removed target tdd in the Makefile for the code freeze.
// Minor patches in DeclaratorBuilder and ParameterDeclarationBuilder.
//
// Revision 1.2  1998/02/04 14:02:04  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:16  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
