// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/ParameterDeclarationListBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:35 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.cpp
// NAME:        ParameterDeclarationListBuilder
//
// HISTORY:
//      Rami Khemakhem, The Open Group RI - Dec 24, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ list of parameters
  * @author Rami Khemakhem
  */

public class ParameterDeclarationListBuilder {

  /** Root of the generated tree */
  private ASTParameterDeclarationList root;

  /** index of parameters */
  private int index;


  /**
    * Creates an empty list of parameters
    */
  public ParameterDeclarationListBuilder () {

    root = new
      ASTParameterDeclarationList(ADLCPPParserTreeConstants.JJTPARAMETERDECLARATIONLIST);
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial parameter
    */
  public ParameterDeclarationListBuilder (ASTParameterDeclaration param) {

    this();
    ADLCPPGenericVisitor.parentChild(root, param, 0);
    index = 1;
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial parameter builder
    */
  public ParameterDeclarationListBuilder (ParameterDeclarationBuilder param) {

    this();
    ADLCPPGenericVisitor.parentChild(root, param.generatedAST(), 0);
    index = 1;
  }


  /**
    * Adds a parameter to the parameter declration list 
    * @param arg The new parameter declaration
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public ParameterDeclarationListBuilder addParameter (ASTParameterDeclaration param) {

    ADLCPPGenericVisitor.parentChild(root, param, index++);
    return this;
  }


  /**
    * Adds a parameter to the parameter declaration list
    * @param arg The new parameter declaration builder
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public ParameterDeclarationListBuilder addParameter (ParameterDeclarationBuilder param) {

    ADLCPPGenericVisitor.parentChild(root, param.generatedAST(), index++);
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTParameterDeclarationList generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ParameterDeclarationListBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:35  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:48  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:04  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:16  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
