// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/PostfixExprBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:35 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        PostfixExprBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 12, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ postfix expression
  * @author Rami KHEMAKHEM
  */

public class PostfixExprBuilder {

  /** Root of the generated tree */
  private ASTPostfixExpr root;


  /**
    * Creates a postfix expression composed of 
    * an expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (Node first, Node second) {

    root = new
      ASTPostfixExpr(ADLCPPParserTreeConstants.JJTPOSTFIXEXPR);

    ADLCPPGenericVisitor.parentChild(root, first, 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * an id expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (ASTIdExpr first, Node second) {

    root = new
      ASTPostfixExpr(ADLCPPParserTreeConstants.JJTPOSTFIXEXPR);

    ADLCPPGenericVisitor.parentChild(root, first, 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * a id expression build form a string
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first A string to build the first child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PostfixExprBuilder (String first, Node second) {

    root = new
      ASTPostfixExpr(ADLCPPParserTreeConstants.JJTPOSTFIXEXPR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(first), 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTPostfixExpr generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: PostfixExprBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:35  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:49  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:05  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:17  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
