// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/PrimaryExpressionBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:07:36 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        PrimaryExpressionBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Feb 4, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ postfix expression
  * @author Cyrille Bareau
  */

public class PrimaryExpressionBuilder {

  /** Root of the generated tree */
  private ASTPrimaryExpr root;


  /**
    * Creates a postfix expression composed of 
    * an expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PrimaryExpressionBuilder (Node first, Node second) {

    root = new
      ASTPrimaryExpr(ADLCPPParserTreeConstants.JJTPRIMARYEXPR);

    ADLCPPGenericVisitor.parentChild(root, first, 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * an id expression : Child of PrimaryExpression (not a node)
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first First child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PrimaryExpressionBuilder (ASTIdExpr first, Node second) {

    root = new
      ASTPrimaryExpr(ADLCPPParserTreeConstants.JJTPRIMARYEXPR);

    ADLCPPGenericVisitor.parentChild(root, first, 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }

  /**
    * Creates a postfix expression composed of 
    * a id expression build form a string
    * and a Reference Access Builder : "->" IdExpr [ArgumentList]
    * @param first A string to build the first child of the postfix expression
    * @param second Second child of the postfix expression
    */
  public PrimaryExpressionBuilder (String first, Node second) {

    root = new
      ASTPrimaryExpr(ADLCPPParserTreeConstants.JJTPRIMARYEXPR);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(first), 0);
    ADLCPPGenericVisitor.parentChild(root, second, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTPrimaryExpr generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: PrimaryExpressionBuilder.java,v $
// Revision 1.5  1998/02/27 13:07:36  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:03:49  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/02/12 13:48:22  bareau
// Classes used in the TDD translation. Added because of code freeze: this is a
// pre-pre-version (it does not even compile yet).
//
// ***********************************************************************
