// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/RefAccessSuffixBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:36 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        RefAccessSuffixBuilder
//
// HISTORY:
//      Rami KHEMAKHEM, The Open Group RI - Dec 12, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.adlcpp.*;

/**
  * Creates a tree for a C++ reference access suffix:
  *    "->"  IdExpr  [ ArgumentList ]
  * @author Rami KHEMAKHEM
  */

public class RefAccessSuffixBuilder {

  /** Root of the generated tree */
  private ASTRefAccessSuffix root;


  /**
    * Creates a reference access suffix composed of 
    * "->" and an expression
    * @param expr The expression
    */
  public RefAccessSuffixBuilder (Node expr) {

    root = new
      ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
  }


  /**
    * Creates a reference access suffix composed of 
    * "->" and an Id Expression
    * @param expr An id expression
    */
  public RefAccessSuffixBuilder (ASTIdExpr expr) {

    root = new
      ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
  }

  /**
    * Creates a reference access suffix composed of 
    * "->" and an Id Expression
    * @param expr The name of the id expression 
    */
  public RefAccessSuffixBuilder (String expr) {

    root = new
      ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(expr), 0);
  }

  /**
    * Creates a reference access suffix composed of 
    * "->", an IdExpr and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public RefAccessSuffixBuilder (Node expr, Node arg) {

    root = new
      ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, expr, 0);
    ADLCPPGenericVisitor.parentChild(root, arg, 1);
  }

  /**
    * Creates a reference access suffix composed of 
    * "->", an IdExpr and an ArgumentList
    * @param expr Id expression
    * @param arg Argument list within braces
    */
  public RefAccessSuffixBuilder (String expr, Node arg) {

    root = new
      ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);

    ADLCPPGenericVisitor.parentChild(root, ADLCPPGenericVisitor.makeIdExpr(expr), 0);
    ADLCPPGenericVisitor.parentChild(root, arg, 1);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTRefAccessSuffix generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: RefAccessSuffixBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:36  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:50  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/02/04 14:02:05  rami
// Update the use of parentChild moved to the generic visitor.
//
// Revision 1.1  1998/01/21 15:30:17  rami
// <node>Builder.java generates an AST<node> node for transformation.
// First release for tests.
//
// ***********************************************************************
