// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/cpp/RemoveADLVisitor.java,v $
// $Revision: 1.4 $
// $Date: 1998/06/30 13:12:24 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.cpp
// NAME:        RemoveADLVisitor.java
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Feb 12, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.cpp;

import ORG.opengroup.adl.parsers.*;
import ORG.opengroup.adl.parsers.adlcpp.*;
import ORG.opengroup.adl.symboltable.*;
import java.util.Vector;

/**
 * Transformation visitor.  Must change an ADL or ADLnew expression.
 * <BR>
 * Used in directives and test methods.
 * @see CPPDirectiveBuilder
 * @author Cyrille Bareau
 */

public class RemoveADLVisitor extends CPPGeneric {

  /** Names of the ACOs to generate. */
  private Vector acoNames = new Vector();

  static private boolean debug = false;
  

  public Vector getACONames() {
    return acoNames;
  }


   /**
    * Visit pattern for a node of type ASTPrimaryExpression.
    * <P><B>Algorithm</B><UL>
    * <LI><B>IF</B> the node's first child if a "ADL_new" expression, create
    * an ASTPrimaryExpression that is the return of the visit to this child,
    * and add as suffixes to this node the suffixes of the input node.
    * <BR>"ADL_new &#60;name&#62; &#60;suffix&#62;<SUP>*</SUP>  --&#62;
    * "new &#60;name&#62;ACO(null, this)&#60;suffix&#62;<SUP>*</SUP>"
    * <LI><B>ELSE</B> visit the children nodes.
    * </UL>
    * @param node the ASTPrimaryExpression to visit.
    * @param data unused.
    * @return the transformed node.
    */
  public Object visit(ASTPrimaryExpr node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTPrimaryExpr");

    Node child = node.jjtGetChild(0);
    if (child instanceof ASTTDDADLnewExpr) {
      ASTParenthExpr createdNode = new
        ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);
      ASTPostfixExpr postfix = new
        ASTPostfixExpr(ADLCPPParserTreeConstants.JJTPOSTFIXEXPR);
      parentChild(createdNode, postfix, 0);

      String type = "new_" + ((ASTIdExpr)child.jjtGetChild(0)).getIdName();

      Node visitedNew = (Node)child.jjtAccept(this, data);
      parentChild(postfix, visitedNew, 0);

      if (debug)
        System.out.println("After ASTTDDADLnewExpr visit");

      ASTRefAccessSuffix ref = new
        ASTRefAccessSuffix(ADLCPPParserTreeConstants.JJTREFACCESSSUFFIX);
      parentChild(ref, makeIdExpr(type), 0);
      parentChild(ref, node.jjtGetChild(1), 1);
      parentChild(postfix, ref, 1);

      return createdNode;
    }
    else {
      for (int i = 0; i < node.jjtGetNumChildren(); i++) {
        parentChild(node,
                    (Node)node.jjtGetChild(i).jjtAccept(this, data),
                    i);
      }
      return node;
    }
  }


  /**
    * Visit pattern for a node of type ASTTDD_ADLnewExpression.
    * <BR>Transforms the code "ADL_new &#60;name&#62;" into
    * "new &#60;name&#62;ACO(null, this).&#60;name&#62;"
    * @param node the ASTTDD_ADLnewExpression to visit.
    * @param data unused.
    * @return an ASTPrimaryExpression.
    */
  public Object visit(ASTTDDADLnewExpr node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_ADLnewExpression");

    ASTParenthExpr createdNode = new
      ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);

    ASTNewExpr newExpr = new ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
    parentChild(createdNode, newExpr, 0);

    ASTDeclarationSpecifiers decl = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    parentChild(newExpr, decl, 0);

    String acoName = ((ASTIdExpr)node.jjtGetChild(0)).getIdName() + "ACO";
    parentChild(decl, makeQualifiedType(acoName), 0);

    if (debug)
      System.out.println("acoName: " + acoName);

    // update aco names
    if ( ! acoNames.contains(acoName) ) {
      acoNames.addElement(acoName);
    }

    ASTArgumentList args = new ArgumentListBuilder
      (makeIdExpr("NULL"))
      .addArgument(makeIdExpr("this"))
      .generatedAST();

    parentChild(newExpr, args, 1);

    return createdNode;
  }


  /**
    * Visit pattern for a node of type ASTTDD_ADLExpression.
    * <P><B>Algorithm</B><UL>
    * <LI><B>IF</B> the argument of the ADL expression is a class name
    * ("ADL(&#60;className&#62;)": call to a checked static method), returns an
    * ASTPrimaryExpression that is the root for code
    * "new &#60;className&#62;ACO(null, this)"
    * <LI><B>ELSE</B> visits the expression node, gets its type, and returns an
    * ASTPrimaryExpression that is the root for code
    * "new &#60;expressionType&#62;ACO(&#60;visited expression&#62;, this)"
    * </UL>
    * @param node the ASTTDD_ADLExpression to visit.
    * @param data unused.
    * @return the transformed node.
    */
  public Object visit(ASTTDDADLExpr node, Object data) throws ParseException {

    if (debug)
      System.out.println("visiting ASTTDD_ADLExpression. TypeName: ");

    ASTParenthExpr createdNode = new
      ASTParenthExpr(ADLCPPParserTreeConstants.JJTPARENTHEXPR);

    ASTNewExpr newExpr = new
      ASTNewExpr(ADLCPPParserTreeConstants.JJTNEWEXPR);
    parentChild(createdNode, newExpr, 0);

    ASTDeclarationSpecifiers decl = new
      ASTDeclarationSpecifiers(ADLCPPParserTreeConstants.JJTDECLARATIONSPECIFIERS);
    parentChild(newExpr, decl, 0);

    LType type = ((Typed)node.jjtGetChild(0)).getType();
    if (!(type instanceof LPointer))
      CPPTDDTransformVisitor.errorMessage("Non-pointer Expression in ADL() expression",
                                          node.getLineNumber());

    LType pointedType = ((LPointer)type).getPointedToType();
    if (pointedType instanceof LPointer)
      CPPTDDTransformVisitor.errorMessage("Non-pointer Expression in ADL() expression",
                                          node.getLineNumber());

    String acoName = pointedType.getName() + "ACO";

    if (debug)
      System.out.println("acoName : " + acoName);

    parentChild(decl, makeQualifiedType(acoName), 0);

    // update aco names
    if ( ! acoNames.contains(acoName) ) {
      acoNames.addElement(acoName);
    }

    ASTArgumentList args = new ArgumentListBuilder
      ( (Node)node.jjtGetChild(0).jjtAccept(this, data))
      .addArgument(makeIdExpr("this"))
      .generatedAST();

    parentChild(newExpr, args, 1);

    /*
      $$$$ call to static methods
    boolean isClass;
    try {
      isClass = ((Boolean)((SimpleNode)node.jjtGetChild(0))
                 .getProperty("isClass")).booleanValue();
    }
    catch(PropertyAccessException e) { isClass = false; }

    if (isClass) {
      // ADL(AClass) : call to a static method of a class
      // -> new AClassACO(null, this)
      parentChild(aco,
                  new ArgumentsBuilder(makeName("null"))
                  .addArgument(makeName("this"))
                  .generatedAST(),
                  1);
    }
    else {     // new <type>ACO(<expr>, this)
      parentChild(aco,
                  new ArgumentsBuilder((Node)node.jjtGetChild(0).jjtAccept(this, data))
                  .addArgument(makeName("this"))
                  .generatedAST(),
                  1);
    }
    */
    return createdNode;
  }

}

// ***********************************************************************
// History Log:
// $Log: RemoveADLVisitor.java,v $
// Revision 1.4  1998/06/30 13:12:24  mallet
// Replaced import of PropertyAccessException in
// ORG.opengroup.adl.parsers by an import on demand (.*).
//
// Revision 1.3  1998/03/10 17:26:45  mallet
// Merged v1.1 with development trunk
//
// Revision 1.2.2.2  1998/03/09 10:28:38  bareau
// Cleaned up the code for error handling (wrote a static method
// "errorMessage" in CPPTDDTransformVisitor).
//
// Revision 1.2.2.1  1998/03/03 10:59:56  bareau
// Set the debug flag to false.
// Bug fix in CPPDatasetExprVisitor: ASTTDDDatasetSingleton nodes were
// sent to the unparser.
//
// Revision 1.2  1998/02/27 13:07:36  bareau
// Updated copyright notice.
//
// Revision 1.1  1998/02/26 17:50:01  bareau
// ADLT2 TDD translator, first release. Version OK for the bank example.
//
// ***********************************************************************
