// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1997-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions to use, copy, modify and distribute this software are
// governed by the terms and conditions set forth in the file COPYRIGHT,
// located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/ACOauxiliary.java,v $
// $Revision: 1.16 $
// $Date: 1998/10/15 18:06:38 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        ACOauxiliary.java
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Dec 1, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.PropertyAccessException;
import ORG.opengroup.adl.parsers.adljava.*;
import ORG.opengroup.adl.symboltable.j.JModifier;
import java.util.Vector;

/**
 * Auxiliary class for the construction of generic parts of code in ACOs.
 * @see JADLTransformVisitor
 * @author Cyrille Bareau
 */

public class ACOauxiliary {

  /* handles */

  /** Handle for the name of the adl class */
  private String adlClassName;

  /** Handle for the name of the implementation class tested */
  private String implClassName;

  /** Handle for the name of the inherited adl class (may be null) */
  private ASTName extendedClass;

  /** Handle for the current method return type */
  private ASTResultType methodReturnType;

  /** Name of the current method */
  private String methodName;

  /** Handle for the formal parameters of the current method */
  private ASTFormalParameters methodFormalParam;

  /** Handle for the list of thrown exceptions of the current method */
  private String[] methodThrowsList;

  /** The current method calls uses the "super.semantics" feature */
  private boolean superSemantics;

  /** Name of the extended adl class from which the current constructor overrides */
  private String inheritedConstructor;

  /** The current tested method is a constructor */
  private boolean isConstructor;

  /** Handle for the "normal" expression */
  private SimpleNode normalSpec;

  /** Handle for the "abnormal" expression */
  private SimpleNode abnormalSpec;


  /** Statements for the initialization of the current method */
  private Vector ACO_Initialization;

  /** Statements for the call to the real object method,
    * catch of exceptions, etc */
  private Vector ACO_CallBlock;

  /** Statements for the finalization of the current method
    * (e.g. re-throw of exceptions) */
  private Vector ACO_Final;

  /** Statements for the initialization of the assertion code */
  private Vector ACO_AssertionInit;

  /** The ADL transform visitor that uses the current ACO class */
  private JADLTransformVisitor associatedVisitor;

  /** "Trick" flag to generate once for all some statements
    * that are exactly similar for each tested method */
  private boolean firstMethod;

  /** Flag set true when the current tested method is static */
  private boolean staticMethod;

  /** Name for exception thrown by the real object method call */
  private String excName;

  private ASTStatement setblockStmt, stmt1, stmt2, stmt3, stmt4;


  /**
    * Single constructor
    * @param visitor The ADL transform visitor that uses the current ACO class
    */
  ACOauxiliary(JADLTransformVisitor visitor) {

    associatedVisitor = visitor;
    extendedClass =
      JGenericVisitor.makeName("ORG.opengroup.adl.runtime.ADLObjectACO"); // by default
    firstMethod = true;
  }


  /** Returns the boolean literal "true" */
  ASTBooleanLiteral trueLit() {

    ASTBooleanLiteral lit = new
      ASTBooleanLiteral(ADLJavaParserTreeConstants.JJTBOOLEANLITERAL);
    lit.setValue(true);
    return lit;
  }


  /** Returns the boolean literal "false" */
  private ASTBooleanLiteral falseLit() {

    ASTBooleanLiteral lit = new
      ASTBooleanLiteral(ADLJavaParserTreeConstants.JJTBOOLEANLITERAL);
    lit.setValue(false);
    return lit;
  }


  /**
    * Initializes the ACO class fields.
    */
  void initializeMethod(boolean isStatic) {

    ACO_Initialization = new Vector();
    ACO_CallBlock = new Vector();
    ACO_Final = new Vector();
    ACO_AssertionInit = new Vector();
    superSemantics = false;
    isConstructor = false;
    methodReturnType = null;
    methodName = null;
    methodFormalParam = null;
    methodThrowsList = null;
    staticMethod = isStatic;
    normalSpec = null;
    abnormalSpec = null;
  }


  /**
    * Sets extendedClass
    * @see #extendedClass
    */
  void setExtendedClass(ASTName name) {
    extendedClass = name;
    extendedClass.setName(extendedClass.getName() + "ACO");
  }


  /**
    * Gets extendedClass
    * @see #extendedClass
    */
  ASTName getExtendedClass() {
    return extendedClass;
  }


  /**
    * Sets normalSpec
    * @see #normalSpec
    */
  void setNormalSpec(SimpleNode s) {
    normalSpec = s;
  }


  /**
    * Sets abnormalSpec
    * @see #abnormalSpec
    */
  void setAbnormalSpec(SimpleNode s) {
    abnormalSpec = s;
  }


  /**
    * Sets the isConstructor flag, and in the case of a constructor,
    * sets the methodReturnType field with the same name as the
    * class name.
    * @see #isConstructor
    */
  void setIsConstructor(boolean s) {
    isConstructor = s;
    if (isConstructor) {
      methodReturnType = new
        ASTResultType(ADLJavaParserTreeConstants.JJTRESULTTYPE);
      ASTType typ = new ASTType(ADLJavaParserTreeConstants.JJTTYPE);
      typ.setName(implClassName);
      JGenericVisitor.parentChild(methodReturnType, typ, 0);
    }
  }


  /**
    * Sets inheritedConstructor
    */
  void setInheritedConstructor(String s) {
    inheritedConstructor = s;
  }


  /**
    * Sets methodReturnType
    * @see #methodReturnType
    */
  void setMethodReturnType(ASTResultType type) {
    methodReturnType = type;
  }


  /**
    * Sets methodName
    * @see #methodName
    */
  void setMethodName(String name) {
    methodName = name;
  }


  /**
    * Gets methodName
    * @see #methodName
    */
  String getMethodName() {
    return methodName;
  }


  /**
    * Sets methodFormalParam
    * @see #methodFormalParam
    */
  void setMethodFormalParam(ASTFormalParameters params) {
    methodFormalParam = params;
  }


  /**
    * Gets abnormalSpec
    * @see #abnormalSpec
    */
  void setMethodThrowsList(ASTNameList throwslist) {
    try {
      methodThrowsList = (String[])throwslist.getProperty("namelist");
    }
    catch(PropertyAccessException e) {
      methodThrowsList = new String[1];
      methodThrowsList[0] = "<no throws list>";
    }
  }


  /**
    * Sets adlClassName
    * @see #adlClassName
    */
  void setAdlClassName(String name) {
    adlClassName = name;
  }
  

  /**
    * Gets adlClassName
    * @see #adlClassName
    */
  String getAdlClassName() {
    return adlClassName;
  }


  /**
    * Sets implementation class name
    * @see #implClassName
    */
  void setImplementationClassName(String name) {
    implClassName = name;
  }
  

  /**
    * Gets implementation class name
    * @see #implClassName
    */
  String getImplementationClassName() {
    return implClassName;
  }


  /**
    * Generates the generic constructor of the ACO class.
    * For instance for class "StockBroker":
<PRE>
|  protected  StockBrokerACO(StockBroker me, ADLTest tc) {
|     super(me, tc);
|  }
</PRE>
    * @return The ConstructorDeclaration AST
    */
  ASTConstructorDeclaration makeACO_Constructor() {

    ASTConstructorDeclaration createdNode = new
      ASTConstructorDeclaration(ADLJavaParserTreeConstants.JJTCONSTRUCTORDECLARATION);
    try {
      int modif = JModifier.PROTECTED;
      createdNode.setProperty("modifier", new Integer(modif));
    }
    catch(PropertyAccessException e) {}
    createdNode.setName(adlClassName + "ACO");

    ASTFormalParameters fps = new FormalParametersBuilder
      (new FormalParameterBuilder(implClassName, "me").generatedAST())
      .addParameter(new FormalParameterBuilder("ADLTest", "tc").generatedAST())
      .generatedAST();
    JGenericVisitor.parentChild(createdNode, fps, 0);

    ASTExplicitConstructorInvocation sup = new
      ASTExplicitConstructorInvocation
        (ADLJavaParserTreeConstants.JJTEXPLICITCONSTRUCTORINVOCATION);
    sup.setName("super");
    JGenericVisitor.parentChild(createdNode, sup, 1);

    ASTArguments args = new
      ArgumentsBuilder(JGenericVisitor.makeName("me"))
      .addArgument(JGenericVisitor.makeName("tc")).generatedAST();
    JGenericVisitor.parentChild(sup, args, 0);

    return createdNode;
  }


  /**
    * Generates the generic initialization code for each
    * method of an ACO class.
<PRE>
|  boolean doReport = finalJournalReporting && debugMode;
|  boolean abnormal = false;
|  boolean normal = true;
|  boolean tddRes;
|  boolean wasThrownException = false;
|  Throwable adl_thrownException = null;
</PRE>
    * and in the case of a constructor or a static method:
<PRE>
|  skipAssertions |= testContext.skipAssertions;
</PRE>
    * else:
<PRE>
|  skipAssertions |= testContext.skipAssertions || realObject == null;
|  StockBroker StockBrokerObject = (StockBroker)realObject;
</PRE>
    * @param supers flag when the tested method contains the clause
    * "super.semantics;".
    * @return The handle on the generated statements (BlockStatement ASTs)
    */
  Vector makeACO_Initialization(boolean supers) {

    superSemantics = supers;

    // statements that are common to every methods

    // boolean doReport = finalJournalReporting && debugMode;
    ASTConditionalAndExpression doReportInit = new
      ASTConditionalAndExpression(ADLJavaParserTreeConstants.JJTCONDITIONALANDEXPRESSION);
    JGenericVisitor.parentChild(doReportInit,
				JGenericVisitor.makeName("finalJournalReporting"),
				0);
    JGenericVisitor.parentChild(doReportInit,
				JGenericVisitor.makeName("debugMode"),
				1);
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "boolean", "doReport", doReportInit))
                                   .generatedAST());
    // boolean abnormal = false;
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "boolean", "abnormal", falseLit()))
                                   .generatedAST());
    // boolean normal = true;
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "boolean", "normal", trueLit()))
                                   .generatedAST());
    // boolean tddRes;
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "boolean", "tddRes"))
                                   .generatedAST());
    // boolean wasThrownException = false;
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "boolean", "wasThrownException", falseLit()))
                                   .generatedAST());
    // Throwable adl_thrownException = null;
    ACO_Initialization.addElement( new BlockStatementBuilder
                                   ( new LocalVariableDeclarationBuilder
                                     ( "Throwable", "adl_thrownException",
                                       new ASTNullLiteral(ADLJavaParserTreeConstants.JJTNULLLITERAL)))
                                   .generatedAST());


    if (staticMethod || isConstructor) {
      // skipAssertions |= testContext.skipAssertions;
      ACO_Initialization.addElement( new StatementBuilder
                                     ( new AssignmentBuilder
                                       ( "skipAssertions", "|=",
                                         JGenericVisitor.makeName("testContext.skipAssertions")))
                                     .generatedAST());
    }
    else {
      // skipAssertions |= testContext.skipAssertions || (realObject == null);
      ASTConditionalOrExpression exp = new
        ASTConditionalOrExpression(ADLJavaParserTreeConstants.JJTCONDITIONALOREXPRESSION);
      JGenericVisitor.parentChild(exp,
                                  JGenericVisitor.makeName("testContext.skipAssertions"),
                                  0);

      ASTEqualityExpression exp2 = new
        ASTEqualityExpression(ADLJavaParserTreeConstants.JJTEQUALITYEXPRESSION);
      String[] ops = { "==" };
      exp2.setOps(ops);
      JGenericVisitor.parentChild(exp, exp2, 1);

      JGenericVisitor.parentChild(exp2, JGenericVisitor.makeName("realObject"), 0);
      ASTNullLiteral exp2_2 = new
        ASTNullLiteral(ADLJavaParserTreeConstants.JJTNULLLITERAL);
      JGenericVisitor.parentChild(exp2, exp2_2, 1);

      ACO_Initialization.addElement( new StatementBuilder
                                     ( new AssignmentBuilder
                                       ( "skipAssertions", "|=", exp))
                                     .generatedAST());

      // <implClassName> <adlClassName>Object = (<implClassName>)realObject;
      ASTCastExpression castExpr = new
        ASTCastExpression(ADLJavaParserTreeConstants.JJTCASTEXPRESSION);
      JGenericVisitor.parentChild(castExpr,
                                  JGenericVisitor.makeName(implClassName), 0);
      JGenericVisitor.parentChild(castExpr,
                                  JGenericVisitor.makeName("realObject"), 1);

      ACO_Initialization.addElement( new BlockStatementBuilder
                                     ( new LocalVariableDeclarationBuilder
                                       ( implClassName, adlClassName + "Object", castExpr)) // $$$$ ???
                                     .generatedAST());
    }

    return ACO_Initialization;
  }


  /**
    * Generates the generic code for the call to the real object method.
    * For instance for method "bankAccount open_account(long amount)
    * throws negativeAmountExc, bankFullExc" of class bank:
<PRE>
|  if (doReport) {
|     testContext.setblock();
|     testContext.infoline("*** Checking method bank.open_account(long) ***");
|  }
|  bankAccount __ADL_returned = null;
|  if (!skipAssertions) {
|     // finalJournalReporting = false; // if super.semantics
|     try {
|        __ADL_returned = bankObject.open_account(amount);
|     }
|     catch (Throwable open_accountExc) {
|        adl_thrownException = open_accountExc;
|        wasThrownException = true;
|        if (!(open_accountExc instanceof negativeAmountExc ||
|              open_accountExc instanceof bankFullExc)) {
|           testContext.infoline("Unexpected exception caught (" +
|                                open_accountExc.getClass().getName() +
|                                ") when calling bank.open_account(long)");
|           testContext.result(ADL_UNRESOLVED);
|           skipAssertions = true;
|           testContext.skipAssertions = true;
|           wasThrownException = false;
|        }
|     }
|     // finalJournalReporting = doReport; // if super.semantics
|     abnormal = (adl_thrownException instanceof negativeAmountExc
|                 || adl_thrownException instanceof bankFullExc);
|     normal = !abnormal;
|  }
</PRE>
    * @return The handle on the generated statements
    */
  Vector makeACO_CallBlock() {

    /* if (doReport) {
         testContext.setblock();
         testContext.infoline("*** Checking " + <implClassName> +
                              "." + <methodName> + " ***");
       }
    */
    ASTIfStatement ifDoReport = new
      ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
    JGenericVisitor.parentChild(ifDoReport, JGenericVisitor.makeName("doReport"), 0);

    if (firstMethod) {
      setblockStmt = new StatementBuilder
        ( new PrimaryExpressionBuilder
          ( JGenericVisitor.makeName("testContext.setblock"),
            new ArgumentsBuilder().generatedAST()))
        .generatedAST();
    }

    String checkStr;
    if (isConstructor)
      checkStr = "\"*** Checking constructor ";
    else
      checkStr = "\"*** Checking method ";
    checkStr += implClassName + "." +  methodName + "(";

    int nbFP = methodFormalParam.jjtGetNumChildren();
    for (int i = 0; i < nbFP-1; i++) {
      checkStr += ((SimpleNode)methodFormalParam.jjtGetChild(i).jjtGetChild(0)).getName() + ", ";
    }
    if (nbFP > 0) {
      checkStr += ((SimpleNode)methodFormalParam.jjtGetChild(nbFP-1).jjtGetChild(0)).getName();
    }
    checkStr += ") ***\"";
    ASTStringLiteral str = new
      ASTStringLiteral(ADLJavaParserTreeConstants.JJTSTRINGLITERAL);
    str.setValue(checkStr);
    ASTPrimaryExpression infolineStmt = new PrimaryExpressionBuilder
      ( JGenericVisitor.makeName("testContext.infoline"),
        new ArgumentsBuilder(str).generatedAST())
      .generatedAST();

    ASTBlock thenBlk = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);

    JGenericVisitor.parentChild(thenBlk, setblockStmt, 0);

    JGenericVisitor.parentChild(thenBlk,
                                new StatementBuilder(infolineStmt).generatedAST(),
                                1);
    JGenericVisitor.parentChild(ifDoReport, thenBlk, 1);
    
    ACO_CallBlock.addElement(ifDoReport);


    // <methodReturnType> __ADL_returned = <default value>;
    if (! (TypeAux.isVoid(methodReturnType))) {
      ACO_CallBlock.addElement( new BlockStatementBuilder
                                ( new LocalVariableDeclarationBuilder
                                  ( TypeAux.resultType2Type(methodReturnType),
                                    "__ADL_returned",
                                    TypeAux.getDefaultValue(methodReturnType)))
                                .generatedAST());
    }

    // if (! skipAssertions)
    ASTIfStatement ifNotSkip = new
      ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
    JGenericVisitor.parentChild(ifNotSkip,
                                new NotBuilder
                                ( JGenericVisitor.makeName("skipAssertions"), false).generatedAST(),
                                0);

    ASTBlock ifNotSkipBlock = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);
    JGenericVisitor.parentChild(ifNotSkip, ifNotSkipBlock, 1);
    int ifNotSkipBlockIndex = 0;

    // finalJournalReporting = false; // if super.semantics

    if (superSemantics) {
      JGenericVisitor.parentChild(ifNotSkipBlock,
                                  new StatementBuilder
                                  ( new AssignmentBuilder
                                    ( "finalJournalReporting", "=", falseLit()))
                                  .generatedAST(),
                                  ifNotSkipBlockIndex++);
    }

    // try { [ __ADL_returned = ] <adlClassName>Object.<methodName>(<methodFormalParam.name>); }
    // or, if "super.semantics;":
    // try { [ __ADL_returned = ] super.<methodName>(<methodFormalParam.name>); }
    // or, if static method:
    // try { [ __ADL_returned = ] <implClassName>.<methodName>(<methodFormalParam.name>); }
    // or, if constructor:
    // try { __ADL_returned = new <implClassName>(<methodFormalParam.name>); }

    ASTTryStatement tryStmt = new
      ASTTryStatement(ADLJavaParserTreeConstants.JJTTRYSTATEMENT);
    JGenericVisitor.parentChild(ifNotSkipBlock, tryStmt, ifNotSkipBlockIndex++);

    ASTBlock tryBlock = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);
    JGenericVisitor.parentChild(tryStmt, tryBlock, 0);

    ArgumentsBuilder callArgs = new ArgumentsBuilder();
    for (int i = 0; i < methodFormalParam.jjtGetNumChildren(); i++) {
      callArgs.addArgument(methodFormalParam.jjtGetChild(i).jjtGetChild(1));
    }

    if (isConstructor) {
      if (superSemantics) {
        // call the ACO method that corresponds to the inherited constructor
        // <inheritedConstructor>(<args>);
        JGenericVisitor.parentChild(tryBlock,
                                    new StatementBuilder
                                    ( new PrimaryExpressionBuilder
                                      ( JGenericVisitor.makeName(inheritedConstructor),
                                        callArgs.generatedAST())
                                      .generatedAST())
                                    .generatedAST(),
                                    0);
      }
      ASTAllocationExpression newObj = new
        ASTAllocationExpression(ADLJavaParserTreeConstants.JJTALLOCATIONEXPRESSION);
      JGenericVisitor.parentChild(newObj,
				  JGenericVisitor.makeName(implClassName),
				  0);
      JGenericVisitor.parentChild(newObj, callArgs.generatedAST(), 1);
      JGenericVisitor.parentChild(tryBlock,
                                  new StatementBuilder
                                  ( new AssignmentBuilder
                                    ( "__ADL_returned", "=", newObj))
                                  .generatedAST(),
                                  superSemantics ? 1 : 0);
    }
    else {
      ASTPrimaryExpression callMethod;
      String caller = null;
      if (superSemantics) {
        caller = "super." + methodName;
      }
      else if (staticMethod) {
        caller = implClassName + "." + methodName;
      }
      else {
        caller = adlClassName + "Object." + methodName;
      }
      callMethod = new PrimaryExpressionBuilder
        ( JGenericVisitor.makeName(caller),
          callArgs.generatedAST() )
        .generatedAST();

      if (TypeAux.isVoid(methodReturnType)){
        JGenericVisitor.parentChild(tryBlock,
                                    new StatementBuilder(callMethod).generatedAST(),
                                    0);
      }
      else { // methodReturnType != void
        JGenericVisitor.parentChild(tryBlock,
                                    new StatementBuilder
                                    ( new AssignmentBuilder
                                      ( "__ADL_returned",
                                        "=", callMethod))
                                    .generatedAST(),
                                    0);
      }
    }

    // catch (Throwable <methodName>Exc)

    excName = methodName + "Exc";
    JGenericVisitor.parentChild(tryStmt,
                                new FormalParameterBuilder("Throwable", excName)
                                .generatedAST(),
                                1);

    // catch block

    ASTBlock catchBlock = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);
    JGenericVisitor.parentChild(tryStmt, catchBlock, 2);

    // adl_thrownException = <methodName>Exc;
    JGenericVisitor.parentChild(catchBlock,
                                new StatementBuilder
                                ( new AssignmentBuilder
                                  ( "adl_thrownException", "=",
                                    JGenericVisitor.makeName(excName)))
                                .generatedAST(),
                                0);

    // wasThrownException = true;
    JGenericVisitor.parentChild(catchBlock,
                                new StatementBuilder
                                ( new AssignmentBuilder
                                  ( "wasThrownException", "=", trueLit()))
                                .generatedAST(),
                                1);

    // if (!( (<methodName>Exc instanceof <methodThrowsList_0>) || ...
    // or "if (true)" if the method's throws list is empty

    ASTIfStatement ifThrown = new
      ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
    JGenericVisitor.parentChild(catchBlock, ifThrown, 2);

    if (methodThrowsList == null) {
      JGenericVisitor.parentChild(ifThrown, trueLit(), 0);
    }
    else {
      if (methodThrowsList.length == 1) {
        ASTInstanceOfExpression inst = new
          ASTInstanceOfExpression(ADLJavaParserTreeConstants.JJTINSTANCEOFEXPRESSION);
        JGenericVisitor.parentChild
          (inst, JGenericVisitor.makeName(excName), 0);
        JGenericVisitor.parentChild
          (inst, JGenericVisitor.makeType(methodThrowsList[0]), 1);
        JGenericVisitor.parentChild(ifThrown, new NotBuilder(inst).generatedAST(), 0);
      }
      else { // methodThrowsList.length > 1  => ConditionarOrExpression
        ASTConditionalOrExpression cond = new
          ASTConditionalOrExpression(ADLJavaParserTreeConstants.JJTCONDITIONALOREXPRESSION);
        ASTInstanceOfExpression inst;
        for (int i = 0; i < methodThrowsList.length; i++) {
          inst = new
            ASTInstanceOfExpression(ADLJavaParserTreeConstants.JJTINSTANCEOFEXPRESSION);
          JGenericVisitor.parentChild(cond, inst, i);
          JGenericVisitor.parentChild
            (inst, JGenericVisitor.makeName(excName), 0);
          JGenericVisitor.parentChild
            (inst, JGenericVisitor.makeType(methodThrowsList[i]), 1);
        }
        JGenericVisitor.parentChild(ifThrown, new NotBuilder(cond).generatedAST(), 0);
      }
    }

    // "then" block

    thenBlk = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);
    JGenericVisitor.parentChild(ifThrown, thenBlk, 1);


    //    testContext.infoline("Unexpected exception caught (" + 
    //                         <methodName>Exc.getClass().getName() +
    //                         ") when calling <implClassName>.<methodName>(<methodFormalParam>)");

    ASTAdditiveExpression addStr = new
      ASTAdditiveExpression(ADLJavaParserTreeConstants.JJTADDITIVEEXPRESSION);
    String[] ops = { "+", "+" };
    addStr.setOps(ops);
    ASTStringLiteral str1 = new
      ASTStringLiteral(ADLJavaParserTreeConstants.JJTSTRINGLITERAL);
    str1.setValue("\"Unexpected exception caught (\"");
    JGenericVisitor.parentChild(addStr, str1, 0);

    ASTDotSuffix getN = new ASTDotSuffix(ADLJavaParserTreeConstants.JJTDOTSUFFIX);
    getN.setName("getName");

    ASTPrimaryExpression str2 = new PrimaryExpressionBuilder
      ( JGenericVisitor.makeName(excName + ".getClass"),
        new ArgumentsBuilder().generatedAST())
      .addSuffix(getN)
      .addSuffix(new ArgumentsBuilder().generatedAST())
      .generatedAST();
    JGenericVisitor.parentChild(addStr, str2, 1);

    ASTStringLiteral str3 = new
      ASTStringLiteral(ADLJavaParserTreeConstants.JJTSTRINGLITERAL);
    String str3Value = "\") when calling " + implClassName + "." + methodName + "(";
    for (int i = 0; i < methodFormalParam.jjtGetNumChildren()-1; i++) {
      str3Value += ((ASTType)methodFormalParam.jjtGetChild(i).jjtGetChild(0)).getName() + ", ";
    }
    if (methodFormalParam.jjtGetNumChildren() > 0) {
      str3Value += ((ASTType)methodFormalParam.jjtGetChild
                    (methodFormalParam.jjtGetNumChildren()-1).jjtGetChild(0)).getName();
    }
    str3Value += ")\"";
    str3.setValue(str3Value);
    JGenericVisitor.parentChild(addStr, str3, 2);

    JGenericVisitor.parentChild(thenBlk,
                                new StatementBuilder
                                ( new PrimaryExpressionBuilder
                                  ( JGenericVisitor.makeName("testContext.infoline"),
                                    new ArgumentsBuilder(addStr).generatedAST()))
                                .generatedAST(),
                                0);


    // testContext.result(ADL_UNRESOLVED);
    if (firstMethod) {
      stmt1 = new StatementBuilder
        ( new PrimaryExpressionBuilder
          ( JGenericVisitor.makeName("testContext.result"),
            new ArgumentsBuilder
            ( JGenericVisitor.makeName("ADL_UNRESOLVED")).generatedAST())
          .generatedAST())
        .generatedAST();

      // skipAssertions = true;
      stmt2 = new StatementBuilder
        ( new AssignmentBuilder
          ( "skipAssertions", "=", trueLit()))
        .generatedAST();

      // testContext.skipAssertions = true;
      stmt3 = new StatementBuilder
        ( new AssignmentBuilder
          ( JGenericVisitor.makeName("testContext.skipAssertions"),
            "=", trueLit()))
        .generatedAST();

      // wasThrownException = false;
      stmt4 = new StatementBuilder
        ( new AssignmentBuilder
          ( "wasThrownException", "=", falseLit()))
        .generatedAST();
    }

    JGenericVisitor.parentChild(thenBlk, stmt1, 1);
    JGenericVisitor.parentChild(thenBlk, stmt2, 2);
    JGenericVisitor.parentChild(thenBlk, stmt3, 3);
    JGenericVisitor.parentChild(thenBlk, stmt4, 4);


    // finalJournalReporting = doReport; // if super.semantics

    if (superSemantics) {
      JGenericVisitor.parentChild(ifNotSkipBlock,
                                  new StatementBuilder
                                  ( new AssignmentBuilder
                                    ( "finalJournalReporting", "=",
                                      JGenericVisitor.makeName("doReport")))
                                  .generatedAST(),
                                  ifNotSkipBlockIndex++);
    }

    // abnormal = <abnormalSpec>
    // normal = <normalSpec>

    boolean bothSpecified = false;

    // visit (ab)normalSpec if already defined

    if (abnormalSpec != null) {
      abnormalSpec = (SimpleNode)abnormalSpec.jjtAccept(associatedVisitor, null);
    }
    if (normalSpec != null) {
      normalSpec = (SimpleNode)normalSpec.jjtAccept(associatedVisitor, null);
    }

    if (normalSpec == null) {
      if (abnormalSpec == null) {

        // default: abnormalSpec = thrown(Throwable)
        ASTName subnode = JGenericVisitor.makeName("Throwable");
        ASTNameList nlist = new
          ASTNameList(ADLJavaParserTreeConstants.JJTNAMELIST);
        try {
          String[] th = { "Throwable" };
          nlist.setProperty("namelist", th);
        }
        catch(PropertyAccessException e) {}
        JGenericVisitor.parentChild(nlist, subnode, 0);
        ASTADL_ThrownExpression abn = new 
          ASTADL_ThrownExpression(ADLJavaParserTreeConstants.JJTADL_THROWNEXPRESSION);
        JGenericVisitor.parentChild(abn, nlist, 0);
        abnormalSpec = (SimpleNode)abn.jjtAccept(associatedVisitor, null);
      }

      // default: normalSpec = !abnormal
      normalSpec = (SimpleNode) new NotBuilder
        (JGenericVisitor.makeName("abnormal"), false).generatedAST();
    }
    else { // normalSpec != null
      if (abnormalSpec == null) {
        // abnormalSpec = !normalSpec
        abnormalSpec = (SimpleNode) new NotBuilder
          (JGenericVisitor.makeName("normal"), false).generatedAST();
      }
      else // normal and abnormal both user-defined
        bothSpecified = true;
    }

    JGenericVisitor.parentChild(ifNotSkipBlock,
                                new StatementBuilder
                                ( new AssignmentBuilder
                                  ("abnormal", "=", abnormalSpec))
                                .generatedAST(),
                                ifNotSkipBlockIndex++);

    JGenericVisitor.parentChild(ifNotSkipBlock,
                                new StatementBuilder
                                ( new AssignmentBuilder
                                  ("normal", "=", normalSpec))
                                .generatedAST(),
                                ifNotSkipBlockIndex++);


    // if (normal && abnormal) testContext.result(ADL_AMBIGUOUS);

    if (bothSpecified) {
      // if normal and/or abnormal have not been user-defined, they have been given
      // default opposite values, hence the condition (normal && abnormalSpec) is
      // false and the code is not to be generated.

      ASTIfStatement ifNormAbn = new
        ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
      ASTConditionalAndExpression nANDabn = 
        new ASTConditionalAndExpression(ADLJavaParserTreeConstants.JJTCONDITIONALANDEXPRESSION);
      JGenericVisitor.parentChild(nANDabn,
                                  JGenericVisitor.makeName("normal"),
                                  0);
      JGenericVisitor.parentChild(nANDabn,
                                  JGenericVisitor.makeName("abnormal"),
                                  1);
      JGenericVisitor.parentChild(ifNormAbn, nANDabn, 0);
      JGenericVisitor.parentChild(ifNormAbn,
                                  new StatementBuilder
                                  ( new PrimaryExpressionBuilder
                                    ( JGenericVisitor.makeName("testContext.result"),
                                      new ArgumentsBuilder
                                      ( JGenericVisitor.makeName("ADL_AMBIGUOUS")).generatedAST())
                                    .generatedAST())
                                  .generatedAST(),
                                  1);
      JGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn, ifNotSkipBlockIndex++);


      // if (!(normal || abnormalSpec)) testContext.result(ADL_UNDEFINED);

      // if normal and/or abnormal have not been user-defined, they have been given
      // default opposite values, hence the condition !(normal || abnormalSpec) is
      // false and the code is not to be generated.

      ASTIfStatement ifNormAbn2 = new
        ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
      ASTConditionalOrExpression nORabn = 
        new ASTConditionalOrExpression(ADLJavaParserTreeConstants.JJTCONDITIONALOREXPRESSION);
      JGenericVisitor.parentChild(nORabn,
                                  JGenericVisitor.makeName("normal"),
                                  0);
      JGenericVisitor.parentChild(nORabn,
                                  JGenericVisitor.makeName("abnormal"),
                                  1);
      JGenericVisitor.parentChild(ifNormAbn2,
                                  new NotBuilder(nORabn).generatedAST(),
                                  0);
      JGenericVisitor.parentChild(ifNormAbn2,
                                  new StatementBuilder
                                  ( new PrimaryExpressionBuilder
                                    ( JGenericVisitor.makeName("testContext.result"),
                                      new ArgumentsBuilder
                                      ( JGenericVisitor.makeName("ADL_UNDEFINED")).generatedAST())
                                    .generatedAST())
                                  .generatedAST(),
                                  1);
      JGenericVisitor.parentChild(ifNotSkipBlock, ifNormAbn2, ifNotSkipBlockIndex++);
    }

    ACO_CallBlock.addElement(ifNotSkip);
    
    return ACO_CallBlock;
  }


  /**
    * Generates the generic final code for each method of an ACO class.
    * For instance for method "bankAccount open_account(long amount)
    * throws negativeAmountExc, bankFullExc" of class bank:
<PRE>
|  if (wasThrownException) {
|     if (adl_thrownException instanceof negativeAmountExc)
|       throw (negativeAmountExc)adl_thrownException;
|     if (adl_thrownException instanceof bankFullExc)
|        throw (bankFullExc)adl_thrownException;
|  }
|  return __ADL_returned;
</PRE>
    * @return The handle on the generated statements
    */
  Vector makeACO_Final() {

    ACO_Final = new Vector();

    //    if (wasThrownException) {
    //      if (adl_thrownException instanceof <methodThrowsList_0>)
    //        throw (<methodThrowsList_0>)adl_thrownException;
    //      if ...
    //    }

    // if methodThrowsList is null, nothing to generate: wasThrownException
    // was set to false in the call block

    if (methodThrowsList != null) {
      ASTIfStatement finalIf = new
        ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
      JGenericVisitor.parentChild(finalIf,
                                  JGenericVisitor.makeName("wasThrownException"), 0);

      ASTBlock finalBlk = new ASTBlock(ADLJavaParserTreeConstants.JJTBLOCK);
      JGenericVisitor.parentChild(finalIf, finalBlk, 1);

      ASTIfStatement ifInst; ASTThrowStatement throwStmt;
      ASTInstanceOfExpression instExpr; ASTCastExpression castExpr;

      for (int i = 0; i < methodThrowsList.length; i++) {
        ifInst = new
          ASTIfStatement(ADLJavaParserTreeConstants.JJTIFSTATEMENT);
        JGenericVisitor.parentChild(finalBlk, ifInst, i);

        instExpr = new
          ASTInstanceOfExpression(ADLJavaParserTreeConstants.JJTINSTANCEOFEXPRESSION);
        JGenericVisitor.parentChild(instExpr,
                                    JGenericVisitor.makeName("adl_thrownException"), 0);
        JGenericVisitor.parentChild(instExpr,
                                    JGenericVisitor.makeType(methodThrowsList[i]), 1);
        JGenericVisitor.parentChild(ifInst, instExpr, 0);

        castExpr = new
          ASTCastExpression(ADLJavaParserTreeConstants.JJTCASTEXPRESSION);
        JGenericVisitor.parentChild(castExpr,
                                    JGenericVisitor.makeType(methodThrowsList[i]), 0);
        JGenericVisitor.parentChild(castExpr,
                                    JGenericVisitor.makeName("adl_thrownException"), 1);

        throwStmt = new
          ASTThrowStatement(ADLJavaParserTreeConstants.JJTTHROWSTATEMENT);
        JGenericVisitor.parentChild(ifInst, throwStmt, 1);
        JGenericVisitor.parentChild(throwStmt, castExpr, 0);
      }
      ACO_Final.addElement(finalIf);
    }

    //    [ return __ADL_returned; ]

    if (!(TypeAux.isVoid(methodReturnType))) {
      ASTReturnStatement finalReturn = new
        ASTReturnStatement(ADLJavaParserTreeConstants.JJTRETURNSTATEMENT);
      JGenericVisitor.parentChild(finalReturn,
                                  JGenericVisitor.makeName("__ADL_returned"), 0);
      ACO_Final.addElement(finalReturn);
    }

    firstMethod = false;
    return ACO_Final;
  }

}

// ***********************************************************************
// History Log:
// $Log: ACOauxiliary.java,v $
// Revision 1.16  1998/10/15 18:06:38  mallet
// Minor cosmetic changes to some messages.
// Changed visibility of trueLit() method from private to package: now
// used in JADLTransformVisitor.
//
// Revision 1.15  1998/08/07 10:18:12  mallet
// Bug pr/150 fixed: use implementation class name instead of method name
// for a call to the tested constructor.
// Some cosmetic changes to the messages issued in verbose mode.
//
// Revision 1.14  1998/06/23 12:26:09  mallet
// Added support for FQ name of adlclass (like org.omg.CORBA.Object): the
// ACO class uses the short name for its name but the FQN for all
// variables and references to the implementation class.
//
// Revision 1.13  1998/06/23 08:09:27  bareau
// Bug fix: the node methodReturnType, of type ASTResultType, has a child ASTType.
//
// Revision 1.12  1998/06/18 10:30:36  mallet
// Fixed incorrect use of type short name instead of full one (FQN).
//
// Revision 1.11  1998/06/03 16:23:17  mallet
// Changed initialization of doReport in ACO methods to avoid reporting
// the "*** Checking... ***" message in Short report mode: doReport is
// now initialized to finalJournalReporting && debugMode.
//
// Revision 1.10  1998/04/15 13:57:40  bareau
// Update according to the new NL mechanism: there are no NLD_* nodes.
//
// Revision 1.9  1998/03/31 12:39:25  bareau
// Use of "super.semantics" in a constructor: call first the method of the
// inherited ACO that corresponds to the overridden constructor (these
// methods are not overridden by default).
//
// Revision 1.8  1998/02/27 13:07:54  bareau
// Updated copyright notice.
//
// Revision 1.7  1998/02/17 10:57:21  mallet
// Merged v1.0 with development trunk
//
// Revision 1.6.2.1  1998/02/16 08:40:46  bareau
// Bug fix: the case of methods with no parameters was incorrect.
//
// Revision 1.6  1998/02/12 16:03:52  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.5  1998/02/12 13:42:58  bareau
// Bug fix: the number of formal parameters may be 0.
//
// Revision 1.4  1998/01/26 15:12:44  bareau
// Minor bug fixes.
//
// Revision 1.3  1998/01/21 16:23:51  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:44  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:17:57  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:21  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
