// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/ArgumentsBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:07:54 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        ArgumentsBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates an AST tree for a Java list of arguments.
  * @author Cyrille Bareau
  */

public class ArgumentsBuilder {

  /** Root of the generated tree */
  private ASTArguments root;

  /** list of arguments */
  private ASTArgumentList argList;

  /** number of arguments */
  private int index;


  /**
    * Creates an empty list of arguments.
    */
  public ArgumentsBuilder () {

    root = new
      ASTArguments(ADLJavaParserTreeConstants.JJTARGUMENTS);
  }


  /**
    * Creates a list of 1 argument.
    * @param arg The initial argument
    */
  public ArgumentsBuilder (Node arg) {

    root = new
      ASTArguments(ADLJavaParserTreeConstants.JJTARGUMENTS);
    argList = new
      ASTArgumentList(ADLJavaParserTreeConstants.JJTARGUMENTLIST);
    JGenericVisitor.parentChild(root, argList, 0);
    JGenericVisitor.parentChild(argList, arg, 0);
    index = 1;
  }


  /**
    * Adds an argument to the arguments list.
    * @param arg The new argument
    * @return this (so that it is possible to chain-adding
    * new arguments)
    */
  public ArgumentsBuilder addArgument (Node arg) {

    if (argList == null) {
      argList = new
        ASTArgumentList(ADLJavaParserTreeConstants.JJTARGUMENTLIST);
      JGenericVisitor.parentChild(root, argList, 0);
    }
    JGenericVisitor.parentChild(argList, arg, index);
    index += 1;
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTArguments generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: ArgumentsBuilder.java,v $
// Revision 1.5  1998/02/27 13:07:54  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:03:52  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/21 16:23:52  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:45  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:17:58  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:22  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
