// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/BlockStatementBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:55 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        BlockStatementBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java block statement
  * @author Cyrille Bareau
  */

public class BlockStatementBuilder {

  /** Root of the generated tree */
  private ASTBlockStatement root;


  /**
    * Creates a Java block statement that is a local variable declaration
    * @param vardecl The local variable declaration (AST)
    */
  public BlockStatementBuilder (ASTLocalVariableDeclaration vardecl) {

    root = new
      ASTBlockStatement(ADLJavaParserTreeConstants.JJTBLOCKSTATEMENT);
    JGenericVisitor.parentChild(root, vardecl, 0);
  }


  /**
    * Creates a Java block statement that is a local variable declaration
    * @param vardeclB The local variable declaration builder
    */
  public BlockStatementBuilder (LocalVariableDeclarationBuilder vardeclB) {

    root = new
      ASTBlockStatement(ADLJavaParserTreeConstants.JJTBLOCKSTATEMENT);
    JGenericVisitor.parentChild(root, vardeclB.generatedAST(), 0);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTBlockStatement generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: BlockStatementBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:55  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:53  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/08 16:32:45  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:17:59  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:22  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
