// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/FactoryVisitor.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:07:56 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        FactoryVisitor.java
//
// HISTORY:
//      Cyrille Bareau, TOG-RI - Dec 3, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
 * Transformation visitor.  Must change a "return expr;",
 * where expr is of primitive type, into "return new &#60;JavaType&#62;(expr);"
 * where JavaType is the Java wrapper object type that corresponds to the
 * primitive type (Integer for int, etc.)
 * <BR>
 * Used in factories: the runtime mechanism for factories accept
 * only object elements.
 * @see JFactoryBuilder
 * @author Cyrille Bareau
 */

public class FactoryVisitor extends JGenericVisitor {

  static final private boolean debug = false;

  /**
    * Visit pattern for a node of type ASTReturnStatement.
    * @param node the ASTReturnStatement to visit.
    * @param data a String that contains the Java wrapper type.
    * @return the transformed node.
    */
  public Object visit(ASTReturnStatement node, Object data) {

    if (debug)
      System.out.println("visiting FactoryVisitor.ASTReturnStatement");

    if (node.jjtGetNumChildren() == 0) {
      return node;
    }

    Node expr = node.jjtGetChild(0);
    //      if (((SimpleNode)expr).getType().isPrimitiveType()) {
    // useless: FactoryVisitor is called only for factories that have
    // primitive types, and the check is already done in semantic checks
    ASTAllocationExpression alloc = new
      ASTAllocationExpression(ADLJavaParserTreeConstants.JJTALLOCATIONEXPRESSION);
    parentChild(alloc, makeName((String)data), 0);
    parentChild(alloc, new ArgumentsBuilder(expr).generatedAST(), 1);

    ASTReturnStatement createdNode = new
      ASTReturnStatement(ADLJavaParserTreeConstants.JJTRETURNSTATEMENT);
    parentChild(createdNode, alloc, 0);
    return createdNode;
  }

}

// ***********************************************************************
// History Log:
// $Log: FactoryVisitor.java,v $
// Revision 1.5  1998/02/27 13:07:56  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:03:54  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/21 16:23:53  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:46  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:17:59  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:23  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
