// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/FormalParameterBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:56 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        FormalParameterBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java formal parameter
  * @author Cyrille Bareau
  */

public class FormalParameterBuilder {

  /** Root of the generated tree */
  private ASTFormalParameter root;


  /**
    * Creates a formal parameter
    * @param type The type of the parameter (AST)
    * @param param The name of the parameter (AST)
    */
  public FormalParameterBuilder (ASTType type, ASTName param) {

    root = new
      ASTFormalParameter(ADLJavaParserTreeConstants.JJTFORMALPARAMETER);

    JGenericVisitor.parentChild(root, type, 0);
    JGenericVisitor.parentChild(root, param, 1);
  }


  /**
    * Creates a formal parameter
    * @param type The type name of the parameter
    * @param param The name of the parameter (AST)
    */
  public FormalParameterBuilder (String type, ASTName param) {

    this(JGenericVisitor.makeType(type), param);
  }


  /**
    * Creates a formal parameter
    * @param type The type nof the parameter (AST)
    * @param param The name of the parameter (String)
    */
  public FormalParameterBuilder (ASTType type, String param) {

    this(type, JGenericVisitor.makeName(param));
  }


  /**
    * Creates a formal parameter
    * @param type The type name of the parameter
    * @param param The name of the parameter
    */
  public FormalParameterBuilder (String type, String param) {

    this(JGenericVisitor.makeType(type),
         JGenericVisitor.makeName(param));
  }


  /**
    * @return the root of the generated tree
    */
  public ASTFormalParameter generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: FormalParameterBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:56  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:55  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/08 16:32:46  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:00  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:23  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
