// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/FormalParametersBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:07:57 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        FormalParametersBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java list of formal parameters
  * @author Cyrille Bareau
  */

public class FormalParametersBuilder {

  /** Root of the generated tree */
  private ASTFormalParameters root;

  /** index of formal parameters */
  private int index;


  /**
    * Creates an empty list of formal parameters
    */
  public FormalParametersBuilder () {

    root = new
      ASTFormalParameters(ADLJavaParserTreeConstants.JJTFORMALPARAMETERS);
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial formal parameter
    */
  public FormalParametersBuilder (ASTFormalParameter param) {

    this();
    JGenericVisitor.parentChild(root, param, 0);
    index = 1;
  }


  /**
    * Creates a list of 1 parameter
    * @param param The initial formal parameter builder
    */
  public FormalParametersBuilder (FormalParameterBuilder param) {

    this();
    JGenericVisitor.parentChild(root, param.generatedAST(), 0);
    index = 1;
  }


  /**
    * Adds a formal parameter to the list of formal parameters
    * @param arg The new formal parameter
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public FormalParametersBuilder addParameter (ASTFormalParameter param) {

    JGenericVisitor.parentChild(root, param, index++);
    return this;
  }


  /**
    * Adds a formal parameter to the list of formal parameters
    * @param arg The new formal parameter builder
    * @return this (so that it is possible to chain-adding
    * new parameters)
    */
  public FormalParametersBuilder addParameter (FormalParameterBuilder param) {

    JGenericVisitor.parentChild(root, param.generatedAST(), index++);
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTFormalParameters generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: FormalParametersBuilder.java,v $
// Revision 1.4  1998/02/27 13:07:57  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:03:55  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/08 16:32:47  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:00  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:24  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
