// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/LocalVariableDeclarationBuilder.java,v $
// $Revision: 1.4 $
// $Date: 1998/02/27 13:08:01 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        LocalVariableDeclarationBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java local variable declaration
  * (LocalVariableDeclaration).
  * @author Cyrille Bareau
  */

public class LocalVariableDeclarationBuilder {

  /** Root of the generated tree */
  private ASTLocalVariableDeclaration root;


  /**
    * Creates a non-initialized local variable declaration.
    * @param type The type of the variable.
    * @param lhs The name of the variable (ASTName)
    */
  public LocalVariableDeclarationBuilder (ASTType type, ASTName lhs) {

    root = new
      ASTLocalVariableDeclaration(ADLJavaParserTreeConstants.JJTLOCALVARIABLEDECLARATION);
    ASTVariableDeclarator varDecl = 
      new ASTVariableDeclarator(ADLJavaParserTreeConstants.JJTVARIABLEDECLARATOR);

    JGenericVisitor.parentChild(root, type, 0);
    JGenericVisitor.parentChild(root, varDecl, 1);
    JGenericVisitor.parentChild(varDecl, lhs, 0);
  }


  /**
    * Creates a non-initialized local variable declaration.
    * @param type The type of the variable.
    * @param lhs The name of the variable (String)
    */
  public LocalVariableDeclarationBuilder (ASTType type, String lhs) {

    this(type, JGenericVisitor.makeName(lhs));
  }


  /**
    * Creates a non-initialized local variable declaration.
    * @param type The type name of the variable.
    * @param lhs The name of the variable.
    */
  public LocalVariableDeclarationBuilder (String type, String lhs) {

    this(JGenericVisitor.makeType(type), lhs);
  }


  /**
    * Creates an initialized local variable declaration.
    * @param type The type of the variable.
    * @param lhs The name of the variable (ASTName)
    * @param rhs The initializion expression.
    */
  public LocalVariableDeclarationBuilder (ASTType type, ASTName lhs, Node rhs) {

    root = new
      ASTLocalVariableDeclaration(ADLJavaParserTreeConstants.JJTLOCALVARIABLEDECLARATION);
    ASTVariableDeclarator varDecl = 
      new ASTVariableDeclarator(ADLJavaParserTreeConstants.JJTVARIABLEDECLARATOR);

    JGenericVisitor.parentChild(root, type, 0);
    JGenericVisitor.parentChild(root, varDecl, 1);
    JGenericVisitor.parentChild(varDecl, lhs, 0);
    JGenericVisitor.parentChild(varDecl, rhs, 1);
  }


  /**
    * Creates an initialized local variable declaration.
    * @param type The type of the variable.
    * @param lhs The name of the variable (String).
    * @param rhs The initializion expression.
    */
  public LocalVariableDeclarationBuilder (ASTType type, String lhs, Node rhs) {

    this(type, JGenericVisitor.makeName(lhs), rhs);
  }


  /**
    * Creates an initialized local variable declaration.
    * @param type The type of the variable.
    * @param lhs The name of the variable.
    * @param rhs The initializion expression.
    */
  public LocalVariableDeclarationBuilder (String type, ASTName lhs, Node rhs) {

    this(JGenericVisitor.makeType(type), lhs, rhs);
  }


  /**
    * Creates an initialized local variable declaration.
    * @param type The type name of the variable.
    * @param lhs The name of the variable.
    * @param rhs The initializion expression.
    */
  public LocalVariableDeclarationBuilder (String type, String lhs, Node rhs) {

    this(JGenericVisitor.makeType(type),
         JGenericVisitor.makeName(lhs),
         rhs);
  }


  /**
    * @return the root of the generated tree
    */
  public ASTLocalVariableDeclaration generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: LocalVariableDeclarationBuilder.java,v $
// Revision 1.4  1998/02/27 13:08:01  bareau
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:04:00  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/08 16:32:50  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:03  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:24  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
