// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/NotBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:08:01 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        NotBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Dec 10, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java "!" expression
  * (UnaryNotPlusMinus).
  * @author Cyrille Bareau
  */

public class NotBuilder {

  /** Root of the generated tree */
  private ASTUnaryNotPlusMinus root;


  /**
    * Creates a <I>not</I> of an expression.
    * @param node The input expression
    * @param parenth Flag to precise if the input expression
    * must be parenthetized (default yes)
    */
  public NotBuilder (Node node, boolean parenth) {

    root = new
      ASTUnaryNotPlusMinus(ADLJavaParserTreeConstants.JJTUNARYNOTPLUSMINUS);
    root.setOp("!");

    if (parenth) {
      ASTParenthExpression parenthnode = new
        ASTParenthExpression(ADLJavaParserTreeConstants.JJTPARENTHEXPRESSION);

      JGenericVisitor.parentChild(root, parenthnode, 0);
      JGenericVisitor.parentChild(parenthnode, node, 0);
    }
    else {
      JGenericVisitor.parentChild(root, node, 0);
    }
  }


  /**
    * Creates a <I>not</I> of an expression (with parentheses).
    * @param node The input expression
    */
  public NotBuilder (Node node) {

    this(node, true);
  }
      

  /**
    * @return the root of the generated tree
    */
  public ASTUnaryNotPlusMinus generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: NotBuilder.java,v $
// Revision 1.5  1998/02/27 13:08:01  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:04:01  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/21 16:23:56  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:51  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:04  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// ***********************************************************************
