// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/PrimaryExpressionBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:08:02 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        PrimaryExpressionBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Nov 28, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java primary expression
  * (&#60;prefix&#62; &#60;suffix&#62;<SUP>+</SUP>)
  * @author Cyrille Bareau
  */

public class PrimaryExpressionBuilder {

  /** Root of the generated tree */
  private ASTPrimaryExpression root;

  /** index of suffixes */
  private int index;


  /**
    * Creates an <U>incomplete</U> primary expression composed of a
    * prefix only.
    * It is <U>necessary</U> to add a suffix afterwards.
    * @see #addSuffix
    * @param prefix The prefix of the primary expression
    */
  public PrimaryExpressionBuilder (Node prefix) {

    root = new
      ASTPrimaryExpression(ADLJavaParserTreeConstants.JJTPRIMARYEXPRESSION);

    JGenericVisitor.parentChild(root, prefix, 0);
    index = 1;
  }


  /**
    * Creates a primary expression composed of 1 prefix and 1 suffix
    * @param prefix The prefix of the primary expression
    * @param suffix The suffix of the primary expression
    */
  public PrimaryExpressionBuilder (Node prefix, Node suffix) {

    root = new
      ASTPrimaryExpression(ADLJavaParserTreeConstants.JJTPRIMARYEXPRESSION);

    JGenericVisitor.parentChild(root, prefix, 0);
    JGenericVisitor.parentChild(root, suffix, 1);
    index = 2;
  }


  /**
    * Adds a suffix to the primary expression
    * @param suffix The new suffix
    * @return this (so that it is possible to chain-adding
    * new suffixes)
    */
  public PrimaryExpressionBuilder addSuffix (Node suffix) {

    JGenericVisitor.parentChild(root, suffix, index++);
    return this;
  }


  /**
    * @return the root of the generated tree
    */
  public ASTPrimaryExpression generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: PrimaryExpressionBuilder.java,v $
// Revision 1.5  1998/02/27 13:08:02  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:04:02  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/21 16:23:57  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:51  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:04  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// Revision 1.1  1997/12/03 17:46:24  bareau
// Auxiliary classes that build an AST subtree of a given AST type: <ASTtype>Builder.java
// ACOauxiliary is used for the ADL transformation: produces "generic" code.
//
// ***********************************************************************
