// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/StatementBuilder.java,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 13:08:02 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:  .translators.j
// NAME:        StatementBuilder
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Dec 10, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.adljava.*;

/**
  * Creates a tree for a Java statement (assignment or primary expression).
  * @author Cyrille Bareau
  */

public class StatementBuilder {

  /** Root of the generated tree */
  private ASTStatement root;


  /**
    * Creates a Java block statement that is a primary expression
    * (method call for instance).
    * @param expr The primary expression (AST)
    */
  public StatementBuilder (ASTPrimaryExpression expr) {

    root = new
      ASTStatement(ADLJavaParserTreeConstants.JJTSTATEMENT);

    JGenericVisitor.parentChild(root, expr, 0);
  }


  /**
    * Creates a Java block statement that is a primary expression builder
    * (method call for instance).
    * @param exprB The primary expression (AST)
    */
  public StatementBuilder (PrimaryExpressionBuilder exprB) {

    root = new
      ASTStatement(ADLJavaParserTreeConstants.JJTSTATEMENT);

    JGenericVisitor.parentChild(root, exprB.generatedAST(), 0);
  }


  /**
    * Creates a Java block statement that is an assignment.
    * @param assig The assignment (AST)
    */
  public StatementBuilder (ASTAssignment assig) {

    root = new
      ASTStatement(ADLJavaParserTreeConstants.JJTSTATEMENT);

    JGenericVisitor.parentChild(root, assig, 0);
  }


  /**
    * Creates a Java block statement that is an assignment builder.
    * @param assig The assignment builder
    */
  public StatementBuilder (AssignmentBuilder assigB) {

    this(assigB.generatedAST());
  }


  /**
    * @return the root of the generated tree
    */
  public ASTStatement generatedAST() {
    return root;
  }
}

// ***********************************************************************
// History Log:
// $Log: StatementBuilder.java,v $
// Revision 1.5  1998/02/27 13:08:02  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/12 16:04:02  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.3  1998/01/21 16:23:57  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.2  1998/01/08 16:32:51  bareau
// Version OK for bank/stock examples.
// Need more comments.
//
// Revision 1.1  1997/12/19 15:18:04  bareau
// Reorganisation of the translator package, with subpackages
// (only the "j" subdirectory for now).
// Added Makefiles.
//
// ***********************************************************************
