// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/translators/j/TypeAux.java,v $
// $Revision: 1.6 $
// $Date: 1998/06/18 10:29:44 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   translators.j
// NAME:        TypeAux
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Dec 23, 1997: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.translators.j;

import ORG.opengroup.adl.parsers.PropertyAccessException;
import ORG.opengroup.adl.parsers.adljava.*;
import ORG.opengroup.adl.symboltable.LType;
import ORG.opengroup.adl.symboltable.LArray;

/**
 * Auxiliary class that gathers static methods to deal with types.
 * @author Cyrille Bareau
 */

public final class TypeAux {

  /**
    * Returns "true" if the type parameter is a primitive type
    * (boolean or numeric).
    */
  public static boolean isPrimitiveType(TypedNode type) {

    return isPrimitiveType(type.getTypeName());
  }


  /**
    * Returns "true" if the type parameter is a primitive type
    * (boolean or numeric).
    */
  public static boolean isPrimitiveType(ASTType type) {

    return isPrimitiveType(type.getName());
  }


  /**
    * Returns "true" if the type parameter is a primitive type
    * (boolean or numeric).
    */
  public static boolean isPrimitiveType(ASTResultType type) {

    return isPrimitiveType(type.getName());
  }


  /**
    * Returns "true" if the type parameter is a primitive type
    * (boolean or numeric).
    */
  public static boolean isPrimitiveType(String type) {

    return 
      type.equals("boolean") || type.equals("char") || type.equals("byte") ||
      type.equals("short") || type.equals("int") || type.equals("long") ||
      type.equals("float") || type.equals("double");
  }


  /**
    * Returns "true" if the type parameter is the void type.
    */
  public static boolean isVoid(TypedNode type) {

    return isVoid(type.getTypeName());
  }


  /**
    * Returns "true" if the type parameter is the void type.
    */
  public static boolean isVoid(ASTType type) {

    return isVoid(type.getName());
  }


  /**
    * Returns "true" if the type parameter is the void type.
    */
  public static boolean isVoid(ASTResultType type) {

    return isVoid(type.getName());
  }


  /**
    * Returns "true" if the type parameter is the void type.
    */
  public static boolean isVoid(String type) {

    return type.equals("void");
  }


  /**
    * Returns the name of the Java wrapper class for the primitive type
    * (Integer for int, etc.)
    */
  public static String getJavaTypeName(TypedNode type) {

    return getJavaTypeName(type.getTypeName());
  }


  /**
    * Returns the name of the Java wrapper class for the primitive type
    * (Integer for int, etc.)
    */
  public static String getJavaTypeName(ASTType type) {

    return getJavaTypeName(type.getName());
  }


  /**
    * Returns the name of the Java wrapper class for the primitive type
    * (Integer for int, etc.)
    */
  public static String getJavaTypeName(ASTResultType type) {

    return getJavaTypeName(type.getName());
  }


  /**
    * Returns the name of the Java wrapper class for the primitive type
    * (Integer for int, etc.)
    */
  public static String getJavaTypeName(String type) {

    if (type.equals("boolean")) return "Boolean";
    if (type.equals("int")) return "Integer";
    if (type.equals("byte")) return "Byte";
    if (type.equals("char")) return "Character";
    if (type.equals("short")) return "Short";
    if (type.equals("long")) return "Long";
    if (type.equals("float")) return "Float";
    if (type.equals("double")) return "Double";
    return null;
  }


  /**
    * Returns the default value for the parameter type
    * (false for boolean, 0 for int, null for objects, etc.)
    */
  public static Node getDefaultValue(TypedNode type) {

    return getDefaultValue(type.getTypeName());
  }


  /**
    * Returns the default value for the parameter type
    * (false for boolean, 0 for int, null for objects, etc.)
    */
  public static Node getDefaultValue(ASTType type) {

    return getDefaultValue(type.getName());
  }


  /**
    * Returns the default value for the parameter type
    * (false for boolean, 0 for int, null for objects, etc.)
    */
  public static Node getDefaultValue(ASTResultType type) {

    return getDefaultValue(type.getName());
  }


  /**
    * Returns the default value for the parameter type
    * (false for boolean, 0 for int, null for objects, etc.)
    */
  public static Node getDefaultValue(String type) {

    Node defaultValue;
    if (type.equals("boolean")) {
      defaultValue = new
        ASTBooleanLiteral(ADLJavaParserTreeConstants.JJTBOOLEANLITERAL);
      ((SimpleNode)defaultValue).setValue(false);
    }
    else if (type.equals("char") || type.equals("byte") ||
             type.equals("short") || type.equals("int") || type.equals("long")) {
      defaultValue = new
        ASTIntegerLiteral(ADLJavaParserTreeConstants.JJTINTEGERLITERAL);
      ((SimpleNode)defaultValue).setValue(0);
    }
    else if (type.equals("float") || type.equals("double")) {
      defaultValue = new
        ASTFloatLiteral(ADLJavaParserTreeConstants.JJTFLOATLITERAL);
      ((SimpleNode)defaultValue).setValue(0.0);
    }
    else {
      defaultValue = new
        ASTNullLiteral(ADLJavaParserTreeConstants.JJTNULLLITERAL);
    }
    return defaultValue;
  }


  /**
    * Transforms an ASTResultType into an ASTType (or null if the
    * parameter type is void).
    */
  public static ASTType resultType2Type(ASTResultType node) {

    if (isVoid(node))
      return null;
    else {
      ASTType ret = (ASTType)node.jjtGetChild(0);
      try {
	ret = (ASTType)ret.clone();
      } catch (CloneNotSupportedException e) {}
      return ret;
    }
  }


  /**
    * Builds an ASTType from a LType.
    */
  public static ASTType LType2ASTType(LType ltype) {

    ASTType typ = new ASTType(ADLJavaParserTreeConstants.JJTTYPE);
    typ.setName(ltype.getFullName());
    if (ltype instanceof LArray) {
      try {
        typ.setProperty("arrayLevel", new Integer(((LArray)ltype).getArrayLevel()));
      }
      catch(PropertyAccessException e) {}
    }
    return typ;
  }
}

// ***********************************************************************
// History Log:
// $Log: TypeAux.java,v $
// Revision 1.6  1998/06/18 10:29:44  mallet
// Fixed method resultType2Type which was incorrectly copying the
// properties of an ASTResultType into an ASTType: now cloning the
// ASTType child of ASTResultType.
// Fixed LType2ASTType to use full name of LType instead of short name.
//
// Revision 1.5  1998/02/27 13:08:03  bareau
// Updated copyright notice.
//
// Revision 1.4  1998/02/17 10:57:22  mallet
// Merged v1.0 with development trunk
//
// Revision 1.3.2.3  1998/02/16 21:06:27  bareau
// "Number" used only for casting, not for creation.
//
// Revision 1.3.2.2  1998/02/16 19:45:22  bareau
// Bug fix: missing return in getJavaTypeName.
//
// Revision 1.3.2.1  1998/02/16 16:41:10  bareau
// getJavaTypeName now always returns "Number" instead of "Long", "Integer", etc.
// (java.lang.Number is the ancestor of all Java numeric holders).
//
// Revision 1.3  1998/02/12 16:04:02  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/21 16:23:57  bareau
// Javadoc comments + a few bug fixes.
//
// Revision 1.1  1998/01/08 16:31:11  bareau
// Auxiliary class to deal with types.
//
// ***********************************************************************
