// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/unparsers/DTD_ACO.java,v $
// $Revision: 1.7 $
// $Date: 1998/09/16 07:51:05 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   Unparser
// NAME:        DTD_ACO.java
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - Apr 22, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.unparsers;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Hashtable;
import com.objectspace.jgl.*;
import ORG.opengroup.adl.symboltable.LClass;

public class DTD_ACO extends DTD {

  private String nl_func_tmpl;

  private String methodName;

  private String methodSignature;

  private String inheritedClassName;

  private Vector params;

  private boolean formattedErrorsFlag;

  private boolean formattedAssertionsFlag;

  private String formatAssertion;

  private Stack assertionStack;

  private int formatIndex;


  public DTD_ACO(String name, Hashtable userEntities) {

    super(userEntities);

    className = name;
    classTmpl = makeDList(openFile("aco.tmpl"));
    classTmpl.replace("&mangled-name;", SGMLNormalize(name) + "ACO");
    nl_func_tmpl = openFile("aco_func.tmpl");
  }


  public void removeEntities() {

    super.removeEntities();
    classTmpl.remove("&aco-funcs;", 1);
  }


  public void emitClassHeader(String fullName, String modif,
                              String extendsName, String implementsName) {

    StringBuffer classSignature = new StringBuffer("<emphasis>");
    if (! (modif == null || modif.equals(""))) {
      classSignature.append(modif);
    }
    classSignature.append("class</emphasis><type> ")
      .append(className)
      .append(" </type>");
    if (! (extendsName == null || extendsName.equals(""))) {
      classSignature.append("<emphasis>extends</emphasis> ")
        .append(extendsName);
    }
    if (! (implementsName == null || implementsName.equals(""))) {
      classSignature.append("<emphasis>implements</emphasis> ")
        .append(implementsName);
    }

    classTmpl.replace("&signature;", classSignature.toString());
    classTmpl.replace("&fqn;", fullName);
  }

  public void emitClassHeader(String fullName, String modif,
                              String[] extendsNames) {

    StringBuffer classSignature = new StringBuffer("<emphasis>");
    if (! (modif == null || modif.equals(""))) {
      classSignature.append(modif);
    }
    classSignature.append("class</emphasis><type> ")
      .append(className)
      .append(" </type>");
    if (! (extendsNames == null || extendsNames.length == 0)) {
      classSignature.append("<emphasis>extends</emphasis> ")
        .append(extendsNames[0]);
      for (int i = 1; i < extendsNames.length; i++) {
        classSignature.append(", ").append(extendsNames[i]);
      }
    }

    classTmpl.replace("&signature;", classSignature.toString());
    classTmpl.replace("&fqn;", fullName);
  }


  public void emitClassInherited(String inhClassName) {

    inheritedClassName = inhClassName;
    classTmpl.replace("&inherits;",
                      makeExternalLink(inhClassName, "ACO"));// +
  }

  public void emitClassInherited(Vector inhClassNames) {

    String rpl = "<itemizedlist>";
    for (Enumeration enum = inhClassNames.elements();
         enum.hasMoreElements(); ) {
      rpl += "\n<listitem><para>" +
             makeExternalLink(((LClass)enum.nextElement()).getName(), "ACO") +
             "</para></listitem>";
    }
    classTmpl.replace("&inherits;", rpl + "</itemizedlist>");
  }


  public void beginMethodTemplate(String methName) {

    currentCompTmpl = makeDList(nl_func_tmpl);
    currentCompTmpl.replace("&mangled-class-name;", SGMLNormalize(className));
    currentCompTmpl.replace("&name;", methName);
    params = new Vector();
    methodName = methName;
    formattedErrorsFlag = false;
    formattedAssertionsFlag = false;
    assertionStack = new Stack();
    codeIndent = "";
  }


  public void endMethodTemplate() {

    currentCompTmpl.remove("&formatted-errors;");
    currentCompTmpl.remove("&formatted-error*;");
    currentCompTmpl.remove("&formatted-assertions;");
    currentCompTmpl.remove("&formatted-assertion*;");
    currentCompTmpl.remove("&assertions-code;");
    classTmpl.splice(classTmpl.find("&aco-funcs;"), currentCompTmpl);
  }


  public void emitParam(String paramType, String paramName, String paramDesc) {

    if (! paramType.equals(""))
      params.addElement(paramType);

    super.emitParam(paramType, paramName, paramDesc);
  }


  public void endPrototype() {

    methodSignature = "(";
    int nbp = params.size();
    String param;

    for (int i = 0; i < nbp; i++) {
      param = (String)params.elementAt(i);
      methodSignature += param;
      if (i < nbp-1)
        methodSignature += ", ";
    }
    methodSignature += ")";

    currentCompTmpl.replace("&signature;", methodName + methodSignature);
    currentCompTmpl.replace("&mangled-signature;",
                            SGMLNormalizeSignature(methodName + methodSignature));
  }


  public void emitSuperSemantics(String inhClassName, boolean constructor) {

    if (inhClassName == null) {
      String name = constructor ? inheritedClassName : methodName;
      currentCompTmpl.replace("&supersemantics;",
                              makeExternalMethodLink(inheritedClassName,
                                                     name + methodSignature));
    }
    else {
      String name = constructor ? inhClassName : methodName;
      currentCompTmpl.replace("&supersemantics;",
                              makeExternalMethodLink(inhClassName,
                                                     name + methodSignature));
    }
  }


  public void emitAssertion(String ass, String label,
                            String image, boolean excBhv) {

    if (excBhv) {
      if (! formattedErrorsFlag) {
        DListIterator iter = currentCompTmpl.find("&formatted-errors;");
        currentCompTmpl.insert(iter, "<itemizedlist>");
        currentCompTmpl.insert(iter, "&formatted-error*;");
        currentCompTmpl.insert(iter, "</itemizedlist>");
        formattedErrorsFlag = true;
      }
    }
    else firstAssertion();

    String assertion;
    if (label == null || label.equals("")) {
      assertion = "<listitem><para>" + ass + ".</para></listitem>\n";
    }
    else {
      assertion = "<listitem><para><emphasis>[" + label + "]</emphasis> " +
                  ass + ".</para></listitem>\n";
    }

    currentCompTmpl.insert(currentCompTmpl.find(excBhv ? "&formatted-error*;" : formatAssertion),
                          assertion);

    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + image + "\n");
  }


  public void emitCondition(String ass, String image, boolean withElse) {

    firstAssertion();
    DListIterator iter;
    DList ifTmpl;
    iter = currentCompTmpl.find(formatAssertion);
    assertionStack.push(formatAssertion);

    if (withElse) {
      formatAssertion = "&formatted-then-assertion*-"
	+ (formatIndex++) + ";";
      String formatElse = "&formatted-else-assertion*-"
	+ formatIndex + ";";
      assertionStack.push(formatElse);

      ifTmpl = makeDList(openFile("if_else.tmpl"));
      ifTmpl.replace("&condition;", ass);
      ifTmpl.replace("&then-assertion;", formatAssertion);
      ifTmpl.replace("&else-assertion;", formatElse);
      
    }
    else {
      formatAssertion = "&formatted-then-assertion*-"
	+ (formatIndex++) + ";";

      ifTmpl = makeDList(openFile("if.tmpl"));
      ifTmpl.replace("&condition;", ass);
      ifTmpl.replace("&then-assertion;", formatAssertion);
    }

    // insert the if template in the current template
    currentCompTmpl.splice(iter, ifTmpl);

    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "if (" +
                          image + ") {\n");
    codeIndent += tab;
  }

  public void emitEndCondition() {

    currentCompTmpl.remove(formatAssertion, 1);
    formatAssertion = (String)assertionStack.pop();

    codeIndent = codeIndent.substring(tablength);
    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "}\n");
  }

  public void emitElse() {

    currentCompTmpl.remove(formatAssertion, 1);
    formatAssertion = (String)assertionStack.pop();

    codeIndent = codeIndent.substring(tablength);
    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "} else {\n");
    codeIndent += tab;
  }


  public void emitTry(int nbCatch) {

    firstAssertion();
    DListIterator iter = currentCompTmpl.find(formatAssertion);
    assertionStack.push(formatAssertion);

    formatAssertion = "&formatted-try-assertion*-"
      + (formatIndex++) + ";";

    DList tryTmpl = makeDList(openFile("try.tmpl"));
    tryTmpl.replace("&try-assertion;", formatAssertion);

    DListIterator catchPos = tryTmpl.find("&caught-assertions;");
    for (int i = 0; i < nbCatch; i++) {
      assertionStack.push(new Integer(formatIndex + (nbCatch-i-1)));
      tryTmpl.insert(catchPos, "&formatted-catch-assertiongroup-" +
		     (formatIndex+i) + ";");
    }

    // don't forget to remove the caught-assertions entity
    tryTmpl.remove(catchPos.index());

    // insert the try template in the current template
    currentCompTmpl.splice(iter, tryTmpl);

    formatIndex += nbCatch;

    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "try {\n");
    codeIndent += tab;
  }

  public void emitEndTry() {

    currentCompTmpl.remove(formatAssertion, 1);
    formatAssertion = (String)assertionStack.pop();

    codeIndent = codeIndent.substring(tablength);
    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "}\n");
  }

  public void emitCatch(String excType, String excName) {

    currentCompTmpl.remove(formatAssertion, 1);
    int ind = ((Integer)assertionStack.pop()).intValue();

    DListIterator iter = 
      currentCompTmpl.find("&formatted-catch-assertiongroup-"
			   + ind + ";");
    formatAssertion = "&formatted-catch-assertion*-" + ind + ";";

    DList catchTmpl = makeDList(openFile("catch.tmpl"));
    catchTmpl.replace("&exception-name;", excName);
    catchTmpl.replace("&exception-type;", excType);
    catchTmpl.replace("&assertion;", formatAssertion);

    // insert the catch template in the current template
    currentCompTmpl.splice(iter, catchTmpl);

    // don't forget to remove the formatted-catch-assertiongroup-? entity
    currentCompTmpl.remove(iter.index());

    codeIndent = codeIndent.substring(tablength);
    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          "} catch(" + excType + " " + excName + ") {\n");
    codeIndent += tab;
  }


  public void emitQuantifier(String label, String quantifier, String image) {

    firstAssertion();
    DListIterator iter = currentCompTmpl.find(formatAssertion);
    assertionStack.push(formatAssertion);

    formatAssertion = "&formatted-quantified-assertion*-"
      + (formatIndex++) + ";";
    String formattedLabel = (label.equals("") ? "" :
			     ("[" + label + "] "));

    DList quantifiedTmpl;

    if (quantifier.equals("forall"))
      quantifiedTmpl = makeDList(openFile("forall.tmpl"));
    else
      quantifiedTmpl = makeDList(openFile("exists.tmpl"));

    quantifiedTmpl.replace("&label;", formattedLabel);
    quantifiedTmpl.replace("&assertion;", formatAssertion);

    // insert the quantified assertion template in the current template
    currentCompTmpl.splice(iter, quantifiedTmpl);

    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                           codeIndent
			   + (label.equals("") ? "" : (label + " : "))
                           + quantifier + "(" + image + ") {\n");
    codeIndent += tab;
  }

  public void emitDomain(String name, String domain, int domIndex) {

    DListIterator iter = currentCompTmpl.find("&domain;");

    DList domainTmpl = makeDList(openFile("domain.tmpl"));
    domainTmpl.replace("&name;", name);
    domainTmpl.replace("&domain-def;", domain);

    // insert the domain template in the current template
    currentCompTmpl.splice(iter, domainTmpl);
  }

  public void emitEndQuantified() {

    // don't forget to remove the domain entity
    currentCompTmpl.remove("&domain;", 1);

    currentCompTmpl.remove(formatAssertion, 1);
    formatAssertion = (String)assertionStack.pop();

    codeIndent = codeIndent.substring(tablength);
    currentCompTmpl.insert(currentCompTmpl.find("&assertions-code;"),
                          codeIndent + "}\n");
  }


  public void emitNormal(String expr, boolean normal) {

    replace(currentCompTmpl, (normal ? "&normal;" : "&abnormal;"), expr);
  }


  private void firstAssertion() {

    if (! formattedAssertionsFlag) {
      formatAssertion = "&formatted-assertion*;";
      DListIterator iter = currentCompTmpl.find("&formatted-assertions;");
      currentCompTmpl.insert(iter, "<itemizedlist>");
      currentCompTmpl.insert(iter, formatAssertion);
      currentCompTmpl.insert(iter, "</itemizedlist>");
      formattedAssertionsFlag = true;
    }
  }

}

// ***********************************************************************
// History Log:
// $Log: DTD_ACO.java,v $
// Revision 1.7  1998/09/16 07:51:05  mallet
// Fixed bug: replaced hard-coded translations for if-then-else,
// try-catch, forall and exists by appropriate template files.
//
// Revision 1.6  1998/06/30 13:31:34  mallet
// Fixed small bug in generated semantics for quantified assertions.
//
// Revision 1.5  1998/06/12 13:22:14  bareau
// Added support for NL unparsing of quantified assertions.
//
// Revision 1.4  1998/06/10 15:52:56  bareau
// Bug fix: when the behavior of a constructor overrides the inherited
// one, the created link was incorrect.
//
// Revision 1.3  1998/06/05 14:08:39  bareau
// Added support for multiple inheritance in C++.
// Began to javadoc.
//
// Revision 1.2  1998/05/26 15:27:17  bareau
// Added links to inherited classes and inherited behavior (super.semantics).
// Suppressed incorrect tag lineannotation for code.
//
// Revision 1.1  1998/05/15 13:49:29  bareau
// Added classes to deal with SGML docbook template.
// NLDGenericUnparser is the ancestor for NLD unparsers (this is the
// class that deals with DTD classes).
//
//
// ***********************************************************************
