// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/unparsers/DTD_TDD.java,v $
// $Revision: 1.2 $
// $Date: 1998/05/26 15:28:27 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   Unparser
// NAME:        DTD_TDD.java
//
// HISTORY:
//      Cyrille Bareau, The Open Group RI - May 11, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.unparsers;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Stack;
import java.util.Hashtable;
import com.objectspace.jgl.*;

public class DTD_TDD extends DTD {

  private String tddClassName;

  private String dataset_tmpl;

  private String factory_tmpl;

  private String testdirective_tmpl;

  private String testfunc_tmpl;

  private boolean firstConstant = true;

  private boolean firstUse = true;

  private DListIterator test_data;

  private DListIterator test_funcs;

  private DListIterator acos;

  private Hashtable acoTable;

  private Hashtable funcsTable;


  public DTD_TDD(String name, Hashtable userEntities) {

    super(userEntities);

    classTmpl = makeDList(openFile("tdd.tmpl"));
    tddClassName = name;
    className = SGMLNormalize(tddClassName + "TDD");
    classTmpl.replace("&tdd-name;", name);
    classTmpl.replace("&mangled-tdd-name;", className);
  }


  public String getName() {
    return tddClassName;
  }


  public void removeEntities() {

    super.removeEntities();
    //    classTmpl.remove("&acos;");
    classTmpl.remove("&constant*;");
    classTmpl.remove("&used-class*;");
    if (dataset_tmpl != null)
      classTmpl.remove("&datasets;");
    if (factory_tmpl != null)
      classTmpl.remove("&factories;");
    if (testfunc_tmpl != null)
      classTmpl.remove("&test-funcs;");
    if (testdirective_tmpl == null) {
      classTmpl.replace("&test-directives;",
                        "<refentry><refnamediv><refname></refname>" +
                        "<refpurpose></refpurpose></refnamediv><refsect1>" +
                        "<title>Test Directives</title><para>" +
                        "No test directive</para></refsect1></refentry>");
    }
    else classTmpl.remove("&test-directives;");
  }


  public void emitUseClass(String usedClass) {

    DListIterator use;
    if (firstUse) {
      firstUse = false;
      use = classTmpl.find("&used-classes;");
      classTmpl.insert(use, "\n<itemizedlist>");
      classTmpl.insert(use, "&used-class*;");
      classTmpl.insert(use, "\n</itemizedlist>");
      classTmpl.remove(use.index());
    }
    use = classTmpl.find("&used-class*;");
    classTmpl.insert(use, "\n<listitem><para>" +
                     makeExternalLink(usedClass, "TDD") +
                     "</para></listitem>\n");
  }


  public void emitConstant(String type, String name,
                           String expr, String desc) {

    DListIterator cst;
    if (firstConstant) {
      firstConstant = false;
      cst = classTmpl.find("&constants;");
      classTmpl.insert(cst, "\n<blockquote><variablelist>\n");
      classTmpl.insert(cst, "&constant*;");
      classTmpl.insert(cst, "\n</variablelist></blockquote>");
      classTmpl.remove(cst.index());
    }
    cst = classTmpl.find("&constant*;");
    classTmpl.insert(cst, "<varlistentry><term>" + type + " <parameter>" +
                           name + "</parameter> = " + expr + "</term>\n<listitem><para>" +
                           desc + "</para></listitem></varlistentry>\n");
                           
  }


  public void emitDataset(String dsName, String dsType,
                          String dsExpr, String dsDesc) {

    if (dataset_tmpl == null) {
      dataset_tmpl = openFile("dataset.tmpl");
    }
    currentCompTmpl = makeDList(dataset_tmpl);
    currentCompTmpl.replace("&tdd-name;", className);
    currentCompTmpl.replace("&name;", dsName);
    currentCompTmpl.replace("&type;", dsType);
    currentCompTmpl.replace("&expr;", dsExpr);
    if (! dsDesc.equals(""))
      emitComponentDescription(dsDesc);

    emitSGMLNormalizedName(dsName);

    classTmpl.splice(classTmpl.find("&datasets;"), currentCompTmpl);
  }


  public void beginFactoryTemplate(String factName) {

    if (factory_tmpl == null) {
      factory_tmpl = openFile("factory.tmpl");
    }
    currentCompTmpl = makeDList(factory_tmpl);
    currentCompTmpl.replace("&tdd-name;", className);
    currentCompTmpl.replace("&name;", factName);

    emitSGMLNormalizedName(factName);
  }

  public void endFactoryTemplate() {

    classTmpl.splice(classTmpl.find("&factories;"), currentCompTmpl);
  }


  public void beginTestFuncTemplate(String funcName, String signature) {

    if (testfunc_tmpl == null) {
      testfunc_tmpl = openFile("test_func.tmpl");
    }
    currentCompTmpl = makeDList(testfunc_tmpl);
    currentCompTmpl.replace("&tdd-name;", className);
    currentCompTmpl.replace("&name;", funcName);

    currentCompTmpl.replace("&mangled-signature;",
                            SGMLNormalizeSignature(funcName + signature));
    currentCompTmpl.replace("&signature;",
                            funcName + signature);

    acoTable = new Hashtable();
  }

  public void endTestFuncTemplate() {

    currentCompTmpl.remove("&aco;", 1);
    if (acoTable.size() != 0)
      currentCompTmpl.remove("&acos;");
    classTmpl.splice(classTmpl.find("&test-funcs;"), currentCompTmpl);
  }


  public void beginDirectiveTemplate(String dirName, String purpose) {

    if (testdirective_tmpl == null) {
      testdirective_tmpl = openFile("test_directive.tmpl");
    }
    currentCompTmpl = makeDList(testdirective_tmpl);
    currentCompTmpl.replace("&tdd-name;", className);
    currentCompTmpl.replace("&name;", dirName);
    if (! purpose.equals(""))
      emitComponentPurpose(purpose);

    emitSGMLNormalizedName(dirName);

    test_data = null;
    acos = null;
    test_funcs = null;
    acoTable = new Hashtable();
    funcsTable = new Hashtable();
  }

  public void endDirectiveTemplate() {

    currentCompTmpl.remove("&d-set*;", 1);
    currentCompTmpl.remove("&aco;", 1);
    currentCompTmpl.remove("&test-func;", 1);
    if (test_funcs != null)
      currentCompTmpl.remove("&used-test-funcs;");
    if (acoTable.size() != 0)
      currentCompTmpl.remove("&acos;");
    classTmpl.splice(classTmpl.find("&test-directives;"), currentCompTmpl);
  }


  public void emitCode(String code) {

    currentCompTmpl.replace("&code;", code);
  }


  public void emitDomain(String dsExpr) {

    if (test_data == null) {
      test_data = currentCompTmpl.find("&test-data;");
      currentCompTmpl.insert(test_data, "<itemizedlist>");
      currentCompTmpl.insert(test_data, "&d-set*;");
      currentCompTmpl.insert(test_data, "</itemizedlist>\n");
      currentCompTmpl.remove(test_data.index());
    }
    DListIterator iter = currentCompTmpl.find("&d-set*;");
    currentCompTmpl.insert(iter, "<listitem><para>" + dsExpr +
                                 "</para></listitem>\n");
  }


  public void emitACO(String acoName) {

    if (acos == null) {
      acos = currentCompTmpl.find("&acos;");
      currentCompTmpl.insert(acos, "<itemizedlist>");
      currentCompTmpl.insert(acos, "&aco;");
      currentCompTmpl.insert(acos, "</itemizedlist>\n");
    }
    if (! acoTable.contains(acoName)) {
      acoTable.put(acoName, acoName);
      DListIterator iter = currentCompTmpl.find("&aco;");
      currentCompTmpl.insert(iter, "<listitem><para>" +
                             makeExternalLink(acoName, "ACO") +
                             "</para></listitem>\n");
    }
  }


  public void emitTestFunction(String funcName, String signature) {

    if (test_funcs == null) {
      test_funcs = currentCompTmpl.find("&used-test-funcs;");
      currentCompTmpl.insert(test_funcs, "<itemizedlist>");
      currentCompTmpl.insert(test_funcs, "&test-func;");
      currentCompTmpl.insert(test_funcs, "</itemizedlist>\n");
    }
    if (! funcsTable.contains(funcName)) {
      funcsTable.put(funcName, funcName);
      DListIterator iter = currentCompTmpl.find("&test-func;");
      currentCompTmpl.insert(iter, "<listitem><para>" +
                             makeLocalLinkSignature(funcName + signature) +
                             "</para></listitem>\n");
    }
  }

}

// ***********************************************************************
// History Log:
// $Log: DTD_TDD.java,v $
// Revision 1.2  1998/05/26 15:28:27  bareau
// Added references to "used" tdd classes.
//
// Revision 1.1  1998/05/15 13:49:30  bareau
// Added classes to deal with SGML docbook template.
// NLDGenericUnparser is the ancestor for NLD unparsers (this is the
// class that deals with DTD classes).
//
//
// ***********************************************************************
