// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/unparsers/ScenarioGenerator.java,v $
// $Revision: 1.4 $
// $Date: 1998/03/10 17:26:48 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL Compiler
// NAME:        ScenarioGenerator.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group RI - Jan 28, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.unparsers;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;

import ORG.opengroup.adl.unparsers.GenericUnparserVisitor;
import ORG.opengroup.adl.compiler.Adlt;

/** 
 * Scenario generator supplies methods enbling to generate tet scenarii.
 */
public class ScenarioGenerator extends GenericUnparserVisitor
{
  /** The version number of this makefile generator */
  static private String versionId = "$Revision: 1.4 $";

  /**
   * Default constructor: call superclass constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   */
  public ScenarioGenerator(String toolname) {
    super(toolname, versionId);
  }

  /**
   * Basic constructor specifying generator parameters: call superclass
   * constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   * @param linelimit the desired maximum length of lines in output file
   * @param regindent the regular indentation level
   * @param subindent the sub-expression indentation level
   */
  public ScenarioGenerator(String toolname, int linelimit, int regindent,
		   int subindent) {
    super(toolname, versionId, linelimit, regindent, subindent);
  }

  /**
   *  Set tag used to distinguish sections in the generated scenarii file.
   */
  public void setTag(String tag)
  {
     _tag = tag;
     beginMark = "#FB# " + _tag + " ##"; 
     endMark   = "#FE# " + _tag + " ##";
  }
  
  /** 
   * Generate scenario.
   *
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   * @param included flag indicating wether the the generated scenario will 
   *                 be included in a test suite level one
   */
  public void generateScenario(Vector tddTestDirectiveNames,boolean include)
  {

     String scenarioName = outputDirectory 
                         + File.separator 
                         + "tet_scen";

     extractSections(scenarioName);

     // Open scenario file.
     try 
     {

        if (Adlt.verboseMode()) 
        {
           System.out.println("+++ generating " + scenarioName);
        }

        OutputStreamWriter outStream = 
           new OutputStreamWriter
               (new FileOutputStream(scenarioName));
        
        out = new PrintWriter(new BufferedWriter(outStream));
     
     }
     catch (IOException exc) 
     {
        System.err.println("Error: " + exc.getMessage());
        return;
     }

     // now reset currentColumn and currentIndent
     currentColumn = 0;
     setIndent(0);

     // emit head of file
     if ( isNewFile() )
     {
        // emit general comment
        emit(makefileHeaderString);
        newline();
        newline();

        // emit TDD_CLASSPATH = <adl.incpath>
        emit("## This file can be modified, except for the part in between");
        newline();
        emit("## tags FB and FE, and for the \"all\" line.");
        newline();
        newline();
        emit("all");
        newline();
     }
     else
     {
        // emit unmodified head of formerly generated file
        // till line containing "all" target where the new include section
        // is inserted
        for (int i = 0; i < head.length; i++)
        {
           emit(head[i]);
           newline();
           if ( head[i].equals("all") && ! hasMyIncludeEntry() )
           {
              emitIncludeSection(tddTestDirectiveNames);
           }
        }
     }
     
     if ( isNewFile() || hasMyIncludeEntry() )
     {
        emitIncludeSection(tddTestDirectiveNames);
     }


     // emit unmodified middle of formerly generated file
     for (int i = 0; i < middle.length; i++)
     {
        emit(middle[i]);
        newline();
     }


     if ( ! include )
     {
        emit(beginMark);
        newline();
        newline();
        Enumeration f = tddTestDirectiveNames.elements();
        while ( f.hasMoreElements() )
        { 
           String testDirective = (String)f.nextElement();
           emit(testDirective);
           newline();
           emit("\t/" + testDirective);
           newline();
           newline();
        }
        emit(endMark);
        newline();
     }


     // emit unmodified tail of formerly generated file
     for (int i = 0; i < tail.length; i++)
     {
        emit(tail[i]);
        newline();
     }

     // close stream and reset handle
     out.close();
     out = null;
  }

  // -- Private fields --

  /**
   * Is generated file new ?
   */
  private boolean isNewFile()         { return head.length == 0;  }

  /**
   * Does generated file already has a section for target "All" ?
   */
  private boolean hasIncludeSection() { return head.length > 0;   }

  /** 
   * Does generated file contains a tagged section for target "All" ?
   */
  private boolean hasMyIncludeEntry() { return middle.length > 0; }

  /**
   * Does generated file contains a tagged section for development target.
   */
  private boolean hasMyTargetEntry()  { return tail.length > 0;   }

  /**
   * Emit include section (target is "All").
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   */
  private void emitIncludeSection(Vector tddTestDirectiveNames)
  {
        // insert new for all targets here
        emit(beginMark);
        newline();
        
        Enumeration e = tddTestDirectiveNames.elements();
        while ( e.hasMoreElements() )
        { 
           emit("\t/" + e.nextElement());
           newline();
        }
        
        emit(endMark);
        newline();
  }

  /**
   * Extracts sections from formerly genrated files. Note this file
   * may not exist.
   */
  private void extractSections(String scenarioName)
  {
     try
     {
        // Save part of file that has not to be regenerated
        File file = new File(scenarioName);
        if ( file.isFile() && file.canRead() )
        {
           BufferedReader br = new BufferedReader(new FileReader(file));
           FileSectionExtractor fse = new FileSectionExtractor(br);
           head   = fse.extractFirstSection(beginMark);
           middle = fse.extractMiddleSection(endMark,beginMark);
           tail   = fse.extractLastSection(endMark);
           br.close();
        }
     }
     catch (IOException exc) 
     {
        System.err.println("Error: " + exc.getMessage());
        return;
     }
  }

  // Tag used to distinguish sections in the generated scenarii file.

  /** Unmodified head of generated file. */
  private String[] head   = {}; 

  /** Unmodified middle of generated file. */
  private String[] middle = {}; 

  /** Unmodified tail of generated file. */
  private String[] tail   = {};

  /** Tag used to distinguish sections in generated file. */
  private String   _tag = "";

  /** Beginning mark for generated sections. */
  private String   beginMark = "#FB# " + _tag + " ##"; 

  /** End mark for generated sections. */
  private String   endMark   = "#FE# " + _tag + " ##";

}

// ***********************************************************************
// History Log:
// $Log: ScenarioGenerator.java,v $
// Revision 1.4  1998/03/10 17:26:48  mallet
// Merged v1.1 with development trunk
//
// Revision 1.3.2.1  1998/03/04 18:48:32  cinard
// Now a scenario can contains sections for development targets coming from
// several tdd files.
//
// Revision 1.3  1998/02/27 13:02:53  cinard
// Updated copyright notice.
//
// Revision 1.2  1998/02/12 16:04:05  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/10 15:21:46  cinard
// Made scenario generator generic.
//
// ***********************************************************************
