// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/unparsers/cpp/CPPMakefileGenerator.java,v $
// $Revision: 1.12 $
// $Date: 1998/06/12 13:32:22 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL/CPP Compiler
// NAME:        CPPMakefileGenerator.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group RI - Fev 5, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.unparsers.cpp;

import ORG.opengroup.adl.unparsers.GenericUnparserVisitor;
import ORG.opengroup.adl.unparsers.StringTool;
import ORG.opengroup.adl.unparsers.LineCutter;
import ORG.opengroup.adl.unparsers.FileSectionExtractor;

import ORG.opengroup.adl.compiler.Adlt;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;


/** 
 * Makefile generator supplies methods enbling to generate Makefiles.
 */
public class CPPMakefileGenerator extends GenericUnparserVisitor
{
  /** The version number of this makefile generator */
  static private String versionId = "$Revision: 1.12 $";

  /** The name of the generic Makefile */
  static private String genericMakefileName = "Makefile";

  /**
   * Default constructor: call superclass constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   */
  public CPPMakefileGenerator(String toolname) {
    super(toolname, versionId);
  }

  /**
   * Basic constructor specifying generator parameters: call superclass
   * constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   * @param linelimit the desired maximum length of lines in output file
   * @param regindent the regular indentation level
   * @param subindent the sub-expression indentation level
   */
  public CPPMakefileGenerator(String toolname, int linelimit, int regindent,
		   int subindent) {
    super(toolname, versionId, linelimit, regindent, subindent);
  }

  /** 
   * Generate Makefiles.
   *
   * @param tddClassName name of the TDD class for which a .class file has to
   *                     be generated.
   * @param acoNames names of the ACOs used in the TDD test directives.
   * @param tddConstantNames names of the TDD constant classes for which 
   *                         a .class file has to be generated.
   * @param tddDatasetNames names of the TDD datasets for which a .class 
   *                        file has to be generated
   * @param tddFactoryNames names of the TDD factories for which a .class 
   *                        file has to be generated
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   * @param devTargets      development targets have been requested
   */
  public void generateMakefile(String tddClassName,
                               Vector acoNames,
                               Vector tddConstantNames,
                               Vector tddDatasetNames,
                               Vector tddFactoryNames,
                               Vector tddTestDirectiveNames,
			       boolean devTargets)
  {
    // generate Makefile
    generateGenericMakefile((String)tddClassName,
			acoNames,
			tddConstantNames,
			tddDatasetNames,
			tddFactoryNames,
			tddTestDirectiveNames);

     // generate target Makefile
     // get rid of tdd suffix
     String makefilePrefix = tddClassName.substring
       (0,tddClassName.length() - "TDD".length());
     String makefileName   = makefilePrefix + ".mk";
     String fileName = outputDirectory + File.separator + makefileName;
     
     if (Adlt.verboseMode()) {
       System.out.println("+++ generating " + fileName);
     }
     
     try {
        FileWriter outWriter = new FileWriter(fileName);
        
        out = new PrintWriter(new BufferedWriter(outWriter));
     
     } catch (IOException exc) {
        System.err.println("Error: " + exc.getMessage());
        return;
     }

     // now reset currentColumn and currentIndent
     currentColumn = 0;
     setIndent(0);

     // emit general comment
     emit(makefileHeaderString);
     newline();
     newline();

     // emit TDD_INC = (map -I, <adl.incpath>)
     emit("TDD_INC =");

     String inc = System.getProperty("adl.incpath","");
     StringTokenizer st = new StringTokenizer(inc, File.pathSeparator);
     while ( st.hasMoreTokens() )
     {
       String directory = st.nextToken();
       emit(" -I");

       if (!(new File(directory)).isAbsolute()) {
	 // relative directory, since we generate in a sub-directory
	 // we must add ../ to maitain a valid directory name
	 emit(".." + File.separator + directory);
       } else {
	 // absolute directory, nothing specific to do
	 emit(directory);
       }
     }

     newline();
     newline();

     // emit targets
     Enumeration targets = tddTestDirectiveNames.elements();
     long targetNumber   = 1; // target number
     while ( targets.hasMoreElements() )
     {
        emit( "TARGET"+targetNumber + " = " + targets.nextElement());
        newline();
        targetNumber++;
     }
     newline();
     newline();
     
     emit("# User *must* provide the value of the following variable");
     newline();
     emit("#IMPL_OBJS = ...");
     newline();
     newline();
     // Warning message to users
     System.out.println("!!! Don't forget to edit " +
			fileName +
			" to define the IMPL_OBJS variable !!!");

     LineCutter lc = new LineCutter(lineLimit," \\", "\t");

     // count test directives
     long testDirectiveCount = tddTestDirectiveNames.size();

     // emit rules
     for (int i = 1; i <= testDirectiveCount; i++)
     {
        String _TARGET = "$(TARGET" + i + ")";
        emit(_TARGET + " : " + _TARGET + "$(O)");
        newline();
        emit("\t$(CCC) $(CCFLAGS) -o $@ $(IMPL_OBJS) " +
	     _TARGET + "$(O) $(LDLIBS)");
        newline();
     }
     
     // emit rules for cleaning
     for (int i = 1; i <= testDirectiveCount; i++)
     {
        String _TARGET = "$(TARGET" + i + ")";
       
        emit(_TARGET + "_clean:");
        newline();
        // no .o to remove
     }
     newline();

     if (devTargets) {
       // emit ADLT variable definition
       emit("ADLT = $(ADL2HOME)/bin/adlt -v -cpp -dev");
       if (!inc.equals("")) emit(" -incpath " + inc);
       newline();
       newline();

       // ACO dependencies
       String acoDependencies = StringTool.join(acoNames.elements(),
						".cpp", " ");

       // emit clean generated rules
       String compiledFiles = 
	 tddClassName + ".cpp" + " " +
	 makefileName           + " " +
	 genericMakefileName        + " " +
	 StringTool.join(tddTestDirectiveNames.elements(),".cpp"," ") + " " +
	 acoDependencies + " " +
	 StringTool.join(tddConstantNames.elements(),     ".cpp"," ") + " " +
	 StringTool.join(tddDatasetNames.elements() ,     ".cpp"," ") + " " +
	 StringTool.join(tddFactoryNames.elements() ,     ".cpp"," ");

       emit("devclean:");
       newline();
       emit(lc.cutLine("\t$(RM) -f " + compiledFiles));
       newline();

       // emit additional directive dependencies
       for (int i = 0; i < testDirectiveCount; i++) {
	 emit(lc.cutLine((String)tddTestDirectiveNames.elementAt(i) +
			 ".cpp : " + acoDependencies + 
			 " " + tddClassName + ".cpp"));
	 newline();
       }
     
       // emit target for compiling ACOs
       Enumeration f = acoNames.elements();
       while ( f.hasMoreElements() )
	 {
	   String acoName = (String)f.nextElement();
	   String adlName = acoName.substring(0,acoName.length()-"ACO".length());
	   // adl file is in .., hence the dependency and rule
	   emit(acoName + ".cpp : ../" + adlName + ".adl");
	   newline();
	   emit("\t(cd .. && $(ADLT) " + adlName + ".adl)");
	   newline();
	   newline();
	 }
       
       // emit target for compiling TDDs
       String tddTestFile = tddClassName.substring
	 (0,tddClassName.length() - "TDD".length());
       // tdd file is in .., hence the dependency and rule
       emit(tddClassName + ".cpp : ../" + tddTestFile + ".tdd");
       newline();
       emit("\t(cd .. && $(ADLT) " + tddTestFile + ".tdd)");
       newline();
       newline();
     }

     // close stream and reset handle
     out.close();
     out = null;
  }

  /** Generate generic Makefile.
   *
   * @param tddClassName name of the TDD class for which a .class file has to
   *                     be generated.
   * @param acoNames names of the ACOs used in the TDD test directives.
   * @param tddConstantNames names of the TDD constant classes for which 
   *                         a .class file has to be generated.
   * @param tddDatasetNames names of the TDD datasets for which a .class 
   *                        file has to be generated
   * @param tddFactoryNames names of the TDD factories for which a .class 
   *                        file has to be generated
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   */
  public void generateGenericMakefile(String tddClassName,
                                  Vector acoNames,
                                  Vector tddConstantNames,
                                  Vector tddDatasetNames,
                                  Vector tddFactoryNames,
                                  Vector tddTestDirectiveNames)
  {
     // Makefile names
     String makefilePrefix = tddClassName.substring
                             (0,tddClassName.length() - "TDD".length());
     String makefileName   = makefilePrefix + ".mk";


     String[] head = {}; // unmodified head of generated file
     String[] tail = {}; // unmodified tail of generated file
     String   beginMark = "#FB# " + tddClassName + " ##"; 
     String   endMark   = "#FE# " + tddClassName + " ##";

     try {

        // Dev makefile name absolute name
        String fileName = outputDirectory 
                        + File.separator 
                        + genericMakefileName;

        // Save part of file that has not to be regenerated
        File file = new File(fileName);
        if ( file.isFile() && file.canRead() )
        {
           BufferedReader br = new BufferedReader(new FileReader(file));
           FileSectionExtractor fse = new FileSectionExtractor(br);
           head = fse.extractFirstSection(beginMark);
           tail = fse.extractLastSection(endMark);
           br.close();
        }

        if (Adlt.verboseMode()) {
           System.out.println("+++ generating " + fileName);
        }

        FileWriter outWriter = new FileWriter(fileName);
        
        out = new PrintWriter(new BufferedWriter(outWriter));
     
     } catch (IOException exc) {
        System.err.println("Error: " + exc.getMessage());
        return;
     }

     // now reset currentColumn and currentIndent
     currentColumn = 0;
     setIndent(0);

     // emit head of file
     if ( head.length == 0 )
     {
        // no former header, emit general comment
        emit(makefileHeaderString);
        newline();
        emit("## This file can be modified, except for the part in between");
        newline();
        emit("## tags FB and FE.");
        newline();
        newline();
     }
     else
     {
        // emit unmodified head of formerly generated file
        for (int i = 0; i < head.length; i++)
        {
           emit(head[i]);
           newline();
        }
     }

     emit(beginMark);
     newline();
     newline();

     Enumeration e = tddTestDirectiveNames.elements();
     while ( e.hasMoreElements() )
     { 
        emit(e.nextElement() + ": FORCE");
        newline();
        emit("\t$(MAKE) -f $(ADL2HOME)/lib/generic_cpp.mk $(ADLCONFIG)"
             + " -f " + makefileName
             + " TARGET=$@ $(ADLTARGET)");
        newline();
     }
     newline();

     emit("FORCE:");
     newline();
     newline();

     emit(endMark);
     newline();

     // emit unmodified tail of formerly generated file
     for (int i = 0; i < tail.length; i++)
     {
        emit(tail[i]);
        newline();
     }


     // close stream and reset handle
     out.close();
     out = null;
  }

}

// ***********************************************************************
// History Log:
// $Log: CPPMakefileGenerator.java,v $
// Revision 1.12  1998/06/12 13:32:22  mallet
// Fixed a bug that was introduced in previous version while removing the
// dependency with "FORCE" for each test directive: without this
// dependency, since the name of the target to execute and clean is also the
// name of the executable for the directive (at least on UNIX systems),
// tcc -e|-c does nothing ("directive" is up to date)!
//
// Revision 1.11  1998/06/04 14:12:52  mallet
// Renamed generateDevMakefile as generateGenericMakefile, removed
// devTargets argument.
// Replaced OutputStreamWriter over FileOutputStream by a single FileWriter.
// Moved development targets from Makefile to <tdd class>.mk.
// Fixed dependencies and rules for .tdd->.cpp and .adl->.cpp.
// Added dependencies between directive .o and ACO classes and tdd
// generated class (removed directive dependencies variables DEP?).
// Added -dev, -v and -incpath options to adlt command in ADLT variable.
//
// Revision 1.10  1998/05/14 13:44:33  mallet
// Reverted to defining the target dependencies as the list of ACO
// sources: initially we used ACO objects, then we removed them.
//
// Revision 1.9  1998/05/04 15:49:58  mallet
// Removed unneeded -I. at the end of generated TDD_INC: C (and by
// extension C++) specifies that this is a compiler default.
//
// Revision 1.8  1998/03/10 17:26:48  mallet
// Merged v1.1 with development trunk
//
// Revision 1.7.2.1  1998/03/06 16:43:29  mallet
// Added warning message when generating .mk to recall user that he must
// provide the definition for the IMPL_OBJS variable.
//
// Revision 1.7  1998/02/27 13:02:54  cinard
// Updated copyright notice.
//
// Revision 1.6  1998/02/26 10:54:35  bareau
// Bug fix: suppressed a blank between $(TARGET) and $(O).
// Variables DEP are now empty (no generated .o).
//
// Revision 1.5  1998/02/26 10:28:44  bareau
// Bug fix: replaced "TARGETi = <dirName>.class" by "TARGETi = <dirName>".
//
// Revision 1.4  1998/02/25 08:20:49  bareau
// Syntactic bug fixed in CPPMakefileGenerator.
// Changed a newline in CPPUnparserVisitor visit method for ASTDeclaration.
//
// Revision 1.3  1998/02/25 08:07:18  mallet
// Changed interface of generateMakefile to be compliant with Java
// Makefile generator: added boolean flag devTargets to indicate whether
// to generate the development targets or not.
// Changed generation of TDD_INC from adl.incpath because of the
// generation in the adltest subdirectory: prepend '../' to relative
// directories. Added '-I.' in TDD_INC.
// Added generation of development targets.
//
// Revision 1.2  1998/02/12 16:04:06  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/10 14:25:21  cinard
// Class enabling to generate tet makefiles.
//
// ***********************************************************************
