// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/ORG/opengroup/adl/unparsers/j/JMakefileGenerator.java,v $
// $Revision: 1.9 $
// $Date: 1998/06/04 13:42:19 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   ADL/Java Compiler
// NAME:        JMakefileGenerator.java
//
// HISTORY:
//      Cyril Inard d'Argence, The Open Group RI - Jan 28, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

package ORG.opengroup.adl.unparsers.j;

import ORG.opengroup.adl.unparsers.GenericUnparserVisitor;
import ORG.opengroup.adl.unparsers.StringTool;
import ORG.opengroup.adl.unparsers.LineCutter;
import ORG.opengroup.adl.unparsers.FileSectionExtractor;

import java.io.*;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;

import ORG.opengroup.adl.compiler.Adlt;

/** 
 * Makefile generator supplies methods enbling to generate Makefiles.
 */
public class JMakefileGenerator extends GenericUnparserVisitor
{
  /** The version number of this makefile generator */
  static private String versionId = "$Revision: 1.9 $";

  /** The name of the generic Makefile */
  static private String genericMakefileName = "Makefile";

  /**
   * Default constructor: call superclass constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   */
  public JMakefileGenerator(String toolname) {
    super(toolname, versionId);
  }

  /**
   * Basic constructor specifying generator parameters: call superclass
   * constructor.
   *
   * @param toolname the name of the tool using this unparser visitor
   * @param linelimit the desired maximum length of lines in output file
   * @param regindent the regular indentation level
   * @param subindent the sub-expression indentation level
   */
  public JMakefileGenerator(String toolname, int linelimit, int regindent,
		   int subindent) {
    super(toolname, versionId, linelimit, regindent, subindent);
  }

  /** 
   * Generate Makefiles.
   *
   * @param tddClassName name of the TDD class for which a .class file has to
   *                     be generated.
   * @param acoNames names of the ACOs used in the TDD test directives.
   * @param tddConstantNames names of the TDD constant classes for which 
   *                         a .class file has to be generated.
   * @param tddDatasetNames names of the TDD datasets for which a .class 
   *                        file has to be generated
   * @param tddFactoryNames names of the TDD factories for which a .class 
   *                        file has to be generated
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   * @param devTargets      development targets have been requested
   */
  public void generateMakefile(String tddClassName,
                               Vector acoNames,
                               Vector tddConstantNames,
                               Vector tddDatasetNames,
                               Vector tddFactoryNames,
                               Vector tddTestDirectiveNames,
			       boolean devTargets)
  {
    // generate Makefile
    generateGenericMakefile((String)tddClassName,
			    acoNames,
			    tddConstantNames,
			    tddDatasetNames,
			    tddFactoryNames,
			    tddTestDirectiveNames);

     // generate target <tddFile>.mk
     // get rid of tdd suffix
     String makefilePrefix = tddClassName.substring
       (0, tddClassName.length() - "TDD".length());
     String makefileName   = makefilePrefix + ".mk";
     String fileName = outputDirectory + File.separator + makefileName;
     
     if (Adlt.verboseMode()) {
       System.out.println("+++ generating " + fileName);
     }

     try {
        FileWriter outWriter = new FileWriter(fileName);
        
        out = new PrintWriter(new BufferedWriter(outWriter));
     
     } catch (IOException exc) {
        System.err.println("Error: " + exc.getMessage());
        return;
     }

     // now reset currentColumn and currentIndent
     currentColumn = 0;
     setIndent(0);

     // emit general comment
     emit(makefileHeaderString);
     newline();
     newline();

     // emit TDD_CLASSPATH = <adl.incpath>
     emit("TDD_CLASSPATH = ");

     boolean firstDirectory = true;
     StringTokenizer incpath = null;
     incpath = new StringTokenizer(System.getProperty("adl.incpath",""),
				   File.pathSeparator);
     while (incpath != null && incpath.hasMoreTokens()) {
       String directory = incpath.nextToken();

       if (!firstDirectory) {
	 emit("$(PATHSEP)");
       } else {
	 firstDirectory = false;
       }

       if (!(new File(directory)).isAbsolute()) {
	 // relative directory, since we generate in a sub-directory
	 // we must add ../ to maitain a valid directory name
	 emit(".." + File.separator + directory);
       } else {
	 // absolute directory, nothing specific to do
	 emit(directory);
       }
     }
 
     newline();
     newline();

     // emit targets
     Enumeration targets = tddTestDirectiveNames.elements();
     long targetNumber   = 1; // target number
     while ( targets.hasMoreElements() )
     {
        emit( "TARGET"+targetNumber + " = " + targets.nextElement() + ".class");
        newline();
        targetNumber++;
     }
     newline();

     // compute dependancies     
     String dependencies = 
            tddClassName + ".class" + " "
          + StringTool.join(acoNames.elements(),        ".class"," ")+ " "
          + StringTool.join(tddConstantNames.elements(),".class"," ")+ " "
          + StringTool.join(tddDatasetNames.elements() ,".class"," ")+ " "
          + StringTool.join(tddFactoryNames.elements() ,".class"," ");

     // count test directives
     long testDirectiveCount = tddTestDirectiveNames.size();

     // emit dependencies makefile variables
     LineCutter lc = new LineCutter(lineLimit," \\", "\t");
     for (int i = 1; i <= testDirectiveCount; i++)
     {
        emit(lc.cutLine("DEP" + i + " =  " + dependencies));
        newline();
     }
     
     // emit rules for cleaning
     for (int i = 1; i <= testDirectiveCount; i++)
     {
        emit("$(TARGET" + i + ")_clean:");
        newline();
        emit("\t@-$(RM) $(DEP" + i + ")");
        newline();
     }
     newline();

     if (devTargets) {
       // emit ADLT variable definition
       emit("ADLT = $(ADL2HOME)/bin/adlt -v -dev");
       String inc = System.getProperty("adl.incpath");
       if (inc != null) emit(" -incpath " + inc);
       newline();
       newline();

       // emit clean generated rules
       String compiledFiles = 
	 tddClassName + ".java" + " " +
	 makefileName           + " " +
	 genericMakefileName    + " " +
	 StringTool.join(tddTestDirectiveNames.elements(),".java"," ") + " " +
	 StringTool.join(acoNames.elements(),             ".java"," ") + " " +
	 StringTool.join(tddConstantNames.elements(),     ".java"," ") + " " +
	 StringTool.join(tddDatasetNames.elements() ,     ".java"," ") + " " +
	 StringTool.join(tddFactoryNames.elements() ,     ".java"," ");

       emit("devclean:");
       newline();
       emit(lc.cutLine("\t$(RM) -f " + compiledFiles));
       newline();

       // emit additional directive dependencies
       for (int i = 0; i < testDirectiveCount; i++) {
	 emit(lc.cutLine((String)tddTestDirectiveNames.elementAt(i) +
			 ".java : " +
			 StringTool.join(acoNames.elements(), ".java"," ") +
			 " " + tddClassName + ".java"));
	 newline();
       }
     
       // emit target for compiling ACOs
       Enumeration f = acoNames.elements();
       while ( f.hasMoreElements() )
	 {
	   String acoName = (String)f.nextElement();
	   String adlName = acoName.substring(0,acoName.length()-"ACO".length());
	   // adl file is in .., hence the dependency and rule
	   emit(acoName + ".java : ../" + adlName + ".adl");
	   newline();
	   emit("\t(cd .. && $(ADLT) " + adlName + ".adl)");
	   newline();
	   newline();
	 }
       
       // emit target for compiling TDDs
       String tddTestFile = tddClassName.substring
	 (0,tddClassName.length() - "TDD".length());
       // tdd file is in .., hence the dependency and rule
       emit(tddClassName + ".java : ../" + tddTestFile + ".tdd");
       newline();
       emit("\t(cd .. && $(ADLT) " + tddTestFile + ".tdd)");
       newline();
       newline();
     }

     // close stream and reset handle
     out.close();
     out = null;
  }

  /** Generate generic Makefile.
   *
   * @param tddClassName name of the TDD class for which a .class file has to
   *                     be generated.
   * @param acoNames names of the ACOs used in the TDD test directives.
   * @param tddConstantNames names of the TDD constant classes for which 
   *                         a .class file has to be generated.
   * @param tddDatasetNames names of the TDD datasets for which a .class 
   *                        file has to be generated
   * @param tddFactoryNames names of the TDD factories for which a .class 
   *                        file has to be generated
   * @param tddTestDirectiveNames names of the TDD test directives for which 
   *                              a .class file has to be generated.
   */
  private void generateGenericMakefile(String tddClassName,
                                  Vector acoNames,
                                  Vector tddConstantNames,
                                  Vector tddDatasetNames,
                                  Vector tddFactoryNames,
                                  Vector tddTestDirectiveNames)
  {
     // Makefile names
     String makefilePrefix = tddClassName.substring
                             (0,tddClassName.length() - "TDD".length());
     String makefileName   = makefilePrefix + ".mk";

     String[] head = {}; // unmodified head of generated file
     String[] tail = {}; // unmodified tail of generated file
     String   beginMark = "#FB# " + tddClassName + " ##"; 
     String   endMark   = "#FE# " + tddClassName + " ##";

     try {

        // Dev makefile name absolute name
        String fileName = outputDirectory 
                        + File.separator 
                        + genericMakefileName;

        // Save part of file that has not to be regenerated
        File file = new File(fileName);
        if ( file.isFile() && file.canRead() )
        {
           BufferedReader br = new BufferedReader(new FileReader(file));
           FileSectionExtractor fse = new FileSectionExtractor(br);
           head = fse.extractFirstSection(beginMark);
           tail = fse.extractLastSection(endMark);
           br.close();
        }
        
        if (Adlt.verboseMode()) {
           System.out.println("+++ generating " + fileName);
        }

        FileWriter outWriter = new FileWriter(fileName);
        
        out = new PrintWriter(new BufferedWriter(outWriter));
     
     } catch (IOException exc) {
        System.err.println("Error: " + exc.getMessage());
        return;
     }

     // now reset currentColumn and currentIndent
     currentColumn = 0;
     setIndent(0);

     // emit head of file
     if ( head.length == 0 )
     {
        // no former header, emit general comment
        emit(makefileHeaderString);
        newline();
        emit("## This file can be modified, except for the part in between");
        newline();
        emit("## tags FB and FE.");
        newline();
        newline();
     }
     else
     {
        // emit unmodified head of formerly generated file
        for (int i = 0; i < head.length; i++)
        {
           emit(head[i]);
           newline();
        }
     }

     emit(beginMark);
     newline();
     newline();

     Enumeration e = tddTestDirectiveNames.elements();
     while ( e.hasMoreElements() )
     { 
        emit(e.nextElement() + ":");
        newline();
        emit("\t$(MAKE) -f $(ADL2HOME)/lib/generic_java.mk $(ADLCONFIG)"
             + " -f " + makefileName
             + " TARGET=$@ $(ADLTARGET)");
        newline();
     }
     newline();

     emit(endMark);
     newline();

     // emit unmodified tail of formerly generated file
     for (int i = 0; i < tail.length; i++)
     {
        emit(tail[i]);
        newline();
     }

     // close stream and reset handle
     out.close();
     out = null;
  }

}

// ***********************************************************************
// History Log:
// $Log: JMakefileGenerator.java,v $
// Revision 1.9  1998/06/04 13:42:19  mallet
// Renamed generateDevMakefile as generateGenericMakefile, removed
// devTargets argument.
// Replaced OutputStreamWriter over FileOutputStream by a single FileWriter.
// Moved development targets from Makefile to <tdd class>.mk.
// Fixed dependencies and rules for .tdd->.java and .adl->.java.
// Added dependencies between directive .class and ACO classes and tdd
// generated class.
// Added -dev, -v and -incpath options to adlt command in ADLT variable.
//
// Revision 1.8  1998/02/27 13:02:56  cinard
// Updated copyright notice.
//
// Revision 1.7  1998/02/25 08:03:27  mallet
// Changed generation of TDD_CLASSPATH from adl.incpath because of the
// generation in the adltest subdirectory. Also replaced hard-coded path
// separator by $(PATHSEP): this is a first try to make generated
// Makefile platform independent.
// Fixed bug: the clean target should only be generated when the
// development targets are requested.
// Added generation of the definition of ADLT in the generated Makefile.
//
// Revision 1.6  1998/02/17 10:57:23  mallet
// Merged v1.0 with development trunk
//
// Revision 1.5.2.1  1998/02/13 16:11:32  mallet
// Bug fixed: confusion between generating the Makefile and adding in
// development targets. Now there remains only a single public method
// generateMakefile with an additional boolean argument devTargets. This
// method in turn calls the private method generateDevMakefiles with that
// additional argument propagated. NEEDS SOME CLEAN UP!!!
//
// Revision 1.5  1998/02/12 16:04:09  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.4  1998/02/10 14:20:23  cinard
// Now use StringTool and LineCutter from unparser package.
//
// Revision 1.3  1998/02/02 15:43:00  cinard
// Fixed a bug in target and dependencies numbers.
//
// Revision 1.2  1998/02/02 13:06:18  cinard
// Removed options for Adlt invocation in generated Makefile (default options
// are assumed).
//
// Revision 1.1  1998/02/02 12:53:20  cinard
// Class enabling to generate test makefile and development makefile.
//
// ***********************************************************************
