// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// Copyright (c) 1994-1996 Sun Microsystems Inc.
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ************************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/examples/java/bank_example/bank.java,v $
// $Revision: 1.3 $
// $Date: 1998/02/27 10:49:26 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	bank example
// NAME:	bank.java
// 	
// HISTORY:
// 	Matt Evans, Sun Microsystems Inc. - Sep 27, 1996: Created
//      Cyrille Bareau, The Open Group RI - Jan 1998: Completed
// 		
// ************************************************************************

package bank_example;

/**	
 *  class bank
 *
 *  @version 	        $Id: bank.java,v 1.3 1998/02/27 10:49:26 bareau Exp $
 *  @date created       Oct 8, 1996
 *  @author	        Matt Evans
 *
 */

public class bank {

  private final int maxAccountsDefault = 16;
  private int maxAccounts;	
  private bankAccount accountList[];
  /*private*/ public int numAccounts = 0;

  public bank(){
    maxAccounts = maxAccountsDefault;
    accountList = new bankAccount[maxAccounts];
  }

  public bank(int bankSize) {
    maxAccounts = bankSize;
    accountList = new bankAccount[maxAccounts];
  }

  public int get_max_accounts() {
    return maxAccounts;
  }

  public int get_active_accounts() {
    return numAccounts;
  }

  public bankAccount get_account (long acctNumber)
    throws invalidAccountNumberExc, accountNotFoundExc {

    int index = (int) acctNumber - 1;

    if (index < 0 || index >= maxAccounts) {
      throw (new invalidAccountNumberExc());
    }
    if (accountList[index] == null) {
      throw (new accountNotFoundExc());
    }
    return accountList[index];
  }

  public bankAccount open_account(long amount)
    throws negativeAmountExc, bankFullExc   {

    if (amount < 0) {
      throw (new negativeAmountExc());
    }
    if (numAccounts >= maxAccounts) {
      throw (new bankFullExc());
    }

    for (int x = 0; x < maxAccounts; x++) {
      if (accountList[x] == null) {
	accountList[x] = new bankAccount(x + 1, amount);
	numAccounts++;
	return accountList[x];
      }
    }

    // this is an error
    throw (new Error("Error: bank account list corrupted"));
  }

  public void close_account (long acctNumber)
    throws invalidAccountNumberExc, accountNotFoundExc,
           accountNotZeroExc, invalidAccountExc {

    int index = (int) acctNumber - 1;

    if (index < 0 || index >= maxAccounts) {
      throw (new invalidAccountNumberExc());
    }
    if (accountList[index] == null) {
      throw (new accountNotFoundExc());
    }
    bankAccount ba = accountList[index];
    close_account (ba);
  }

  public void close_account (bankAccount ba)
    throws invalidAccountExc, accountNotFoundExc, accountNotZeroExc {

    if (ba == null) {
      throw (new invalidAccountExc());
    }

    int index = (int) ba.get_account_number() - 1;

    if (index < 0 || index >= maxAccounts || accountList[index] == null) {
      throw (new invalidAccountExc());
    }

    bankAccount my_acct =  accountList[index];

    if (index != (int) my_acct.get_account_number() - 1 &&
	ba.get_balance() != my_acct.get_balance()) {
      throw (new invalidAccountExc());
    }

    // assume the input account is the same as the account
    // in the list.

    if (my_acct.get_balance() != 0) {
      throw (new accountNotZeroExc());
    }

    // remove it from the list

    accountList[index] = null;
    numAccounts--;
  }

  public void close_all_accounts()
    throws invalidAccountExc, accountNotFoundExc, accountNotZeroExc  {

    if (numAccounts > 0) {
      for (int x = 0; x < maxAccounts; x++) {
	if (accountList[x] != null) {
	   close_account(accountList[x]);
	}
      }
    }
  }
}
