//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /vobstore/cvsstore/adl/adl2/src/runtime/cpp/ADLString.cpp,v $
// $Revision: 1.7 $
// $Date: 1998/03/10 17:27:27 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	Java-like string implementation
// NAME:	ADLString.cpp
// 	
// HISTORY:
// 	Lionel Mallet, The Open Group RI - Oct 28, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLString.hh"

ADLString::~ADLString() {
#ifdef WIN32
    rdbuf()->freeze(0);
#else
#ifdef __GNUC__
  if (rdbuf()->frozen()) {
#else
  if (rdbuf()->isfrozen()) {
#endif
    rdbuf()->freeze(0); // to be sure memory will be released 
  }
#endif
}

// conversion to char *
ADLString::operator char*() {
  return str();
}

// redefine str to be sure that the string is null terminated
char *ADLString::str() { 
  // append an end of string in case it's missing
  if (!nullTerminated) {
    operator<<('\0');
  }

  // get the current string
  char *ret = ostrstream::str();

  // now seek back before end of string and unfreeze so that 
  // we can append more stuff in the ADLString after that
  seekp(-1, ios::cur);
  rdbuf()->freeze(0);

  // sinve we've moved back before eos, the string is now no longer
  // null terminated
  nullTerminated = false;

  // return the current string
  return ret;
}

// method to reinitialize the string buffer
ADLString& ADLString::reset() {
  // indicate that the string is not null terminated
  nullTerminated = false;

  // seek back to the beginning of the ADLString and unfreeze
  seekp(0, ios::beg);
  rdbuf()->freeze(0);

  // return the ADLString itself so that a call to reset() can be
  // followed by successive appends with <<
  return *this;
}

// redefine operator to reuse inherited one but return an ADLString
// in case of an append to a null terminated string, it is reset first,
// hence the initial content is discarded
ADLString& ADLString::operator<<(char *s) {
  if (nullTerminated) reset(); ostream::operator<<(s); return *this; }
ADLString& ADLString::operator<<(const char *s) {
  if (nullTerminated) reset(); ostream::operator<<(s); return *this; }

ADLString& ADLString::operator<<(char c) {
  if (nullTerminated) reset();
  ostream::operator<<(c);
  if (c == '\0') nullTerminated = true;
  return *this;
}

ADLString& ADLString::operator<<(int i) {
  if (nullTerminated) reset(); ostream::operator<<(i); return *this; }
ADLString& ADLString::operator<<(unsigned int i) {
  if (nullTerminated) reset(); ostream::operator<<(i); return *this; }
ADLString& ADLString::operator<<(long l) {
  if (nullTerminated) reset(); ostream::operator<<(l); return *this; }
ADLString& ADLString::operator<<(unsigned long l) {
  if (nullTerminated) reset(); ostream::operator<<(l); return *this; }
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
ADLString& ADLString::operator<<(long long l) {
  if (nullTerminated) reset(); ostream::operator<<(l); return *this; }
ADLString& ADLString::operator<<(unsigned long long l) {
  if (nullTerminated) reset(); ostream::operator<<(l); return *this; }
#endif
ADLString& ADLString::operator<<(const void *v) {
  if (nullTerminated) reset(); ostream::operator<<(v); return *this; }
ADLString& ADLString::operator<<(double d) {
  if (nullTerminated) reset(); ostream::operator<<(d); return *this; }
#ifdef __linux__
ADLString& ADLString::operator<<(long double d) {
  if (nullTerminated) reset(); ostream::operator<<(d); return *this; }
#endif

ADLString& ADLString::operator<<(ostream& (*func)(ostream&)) {
#ifdef __GNUC__
  nullTerminated = (func == ends);
#else
  nullTerminated == true;
#endif
  return (ADLString &)(*func)(*this);
}

ADLString& ADLString::operator<<(ios& (*func)(ios&)) {
  (*func)(*this); return *this; }
ADLString& ADLString::operator<<(streambuf* b) {
  if (nullTerminated) reset(); ostream::operator<<(b); return *this; }

ADLString adlstr;

// ***********************************************************************
// History Log:
// $Log: ADLString.cpp,v $
// Revision 1.7  1998/03/10 17:27:27  mallet
// Merged v1.1 with development trunk
//
// Revision 1.4.2.1  1998/03/04 10:56:08  mallet
// Added Win32isms.
//
// Revision 1.6  1998/03/03 22:36:33  shane
// Fixed a typo that effected UNIX and not NT systems.
//
// Revision 1.5  1998/03/03 22:31:28  shane
// Added handling for WIN32-isms.
// Now compiles cleanly on Win32 - don't know if it runs.
//
// Revision 1.4  1998/02/27 10:01:06  mallet
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:04:29  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 17:03:04  mallet
// Made this class more portable by accessing directly the rdbuf().
// Had a problem with Sun's C++ library to access the ends function:
// nullTerminated is set to true whatever the function is.
// Fixed a bug in operator<<(char c): nullTerminated is set to true if c
// if '\0'.
//
// Revision 1.1  1997/11/14 10:13:31  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
